package com.cosylab.application;

import com.cosylab.util.CommonException;

/**
 * An exception describing the problem with plugin installation.
 * These checked exceptions are thrown when the construction or
 * initialization of the plugin fail; when there are cyclic references
 * in the plugin installation; or when the plugin does not conform
 * to the plugin design contract.
 */
public class PlugInException extends CommonException {
	private Class plugType = null;
	
	/**
	 * Creates a new instance of this exception.
	 * 
	 * @param source the object throwing the exception
	 * @param message message to be printed by this exception
	 * @param plugType class object representing the plug the installation of
	 *        which failed (can be <code>null</code>
	 * @param t exception that caused this exception to be thrown
	 *        (can be <code>null</code>)
	 */
	public PlugInException(Object source, String message, Class plugType, Throwable t)
	{
		super(source, message, t);
		this.plugType = plugType;
	}

	/**
	 * Returns a string rendering of this exception.
	 * 
	 * @return String overloaded <code>toString</code> return value.
	 */	
	public String toString()
	{
		String plugTypeName = "N/A";
		if (plugType != null) plugTypeName = plugType.getName();
		String tMessage = "";
		return getMessage() + "\nPlug type: " + plugTypeName + "\n" + "Caused by:\n" + tMessage;
	}
}

