package com.cosylab.application;

import java.util.EventListener;

/**
 * A listener interface for receiving notifications from 
 * <code>PlugInManager</code> regarding the changes in the state
 * or containment of its plugins.
 * 
 * @see PlugInManager
 */
public interface PlugInListener extends EventListener {
	
	/**
	 * Event notification for successfully installed plugin.
	 * This event is fired after all installation procedure, defined
	 * by the plugin manager, has completed.
	 * 
	 * @param event event object
	 */
	public void plugInInstalled(PlugInEvent event);
	/**
	 * Event notification for successfully removed plugin.
	 * This event is fired after all remove procedure, defined
	 * by the plugin manager, has completed.
	 * 
	 * @param event event object
	 */
	public void plugInRemoved(PlugInEvent event);
}

