/*
 * Copyright (c) 2003 by Cosylab d.o.o.
 *
 * The full license specifying the redistribution, modification, usage and other
 * rights and obligations is included with the distribution of this project in
 * the file license.html. If the license is not included you may find a copy at
 * http://www.cosylab.com/legal/abeans_license.htm or may write to Cosylab, d.o.o.
 *
 * THIS SOFTWARE IS PROVIDED AS-IS WITHOUT WARRANTY OF ANY KIND, NOT EVEN THE
 * IMPLIED WARRANTY OF MERCHANTABILITY. THE AUTHOR OF THIS SOFTWARE, ASSUMES
 * _NO_ RESPONSIBILITY FOR ANY CONSEQUENCE RESULTING FROM THE USE, MODIFICATION,
 * OR REDISTRIBUTION OF THIS SOFTWARE.
 */

package com.cosylab.gui;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.beans.PropertyVetoException;
import java.util.Map;

import javax.swing.AbstractAction;
import javax.swing.JFrame;

import com.cosylab.events.SetEvent;
import com.cosylab.events.SetListener;
import com.cosylab.gui.adapters.Converter;
import com.cosylab.gui.components.DialKnob;
import com.cosylab.gui.components.customizer.AbstractCustomizerPanel;
import com.cosylab.gui.components.range2.TrimValuePolicy;
import com.cosylab.gui.components.util.ColorHelper;
import com.cosylab.gui.components.util.CosyUIElements;
import com.cosylab.gui.components.util.PopupManager;
import com.cosylab.gui.displayers.ConvertibleDisplayer;
import com.cosylab.gui.displayers.DataConsumer;
import com.cosylab.gui.displayers.DataSource;
import com.cosylab.gui.displayers.DataSourceSupport;
import com.cosylab.gui.displayers.DataState;
import com.cosylab.gui.displayers.DisplayerUtilities;
import com.cosylab.gui.displayers.DoubleConsumer;
import com.cosylab.gui.displayers.DoubleConsumerMulticaster;
import com.cosylab.gui.displayers.DoubleDisplayer;
import com.cosylab.gui.displayers.NonblockingNumberConsumer;
import com.cosylab.util.CommonException;


/**
 * DialKnob implemented as <code>Displayer</code>.
 *
 * @author <a href="mailto:jernej.kamenik@cosylab.com">Jernej Kamenik</a>
 * @version $id$
 */
public class DialKnobDisplayer extends DialKnob implements DoubleDisplayer,
	DataSource, ConvertibleDisplayer
{
	private static final long serialVersionUID = 1L;

	private DataSourceSupport support = new DataSourceSupport(new Class[]{
			    NonblockingNumberConsumer.class
		    });

	//private InitializationValuePolicy ivp = new InitializationValuePolicy();
	private DataState dataState = new DataState(DataState.UNDEFINED);
	private int suspendCount = 0;
	private DataSource dataSource;
	private boolean flag;
	private Converter converter;
	private AbstractCustomizerPanel customizer;
	private InfoDialog infoDialog;
	private PopupManager popupManager;

	//protected DisplayerSupport delegate = new DisplayerSupport();
	//private SetterSupport setter = null;
	//private int requests = 0;
	//private SetMode setMode = SetMode.SYNCHRONOUS;

	/**
						 *
						 */
	public DialKnobDisplayer()
	{
		super();
		initialize();
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.displayers.DataConsumer#setCharacteristics(java.util.Map)
	 */
	public void setCharacteristics(Map characteristics)
	{
		if (characteristics == null) {
			throw new NullPointerException("characteristics");
		}

		DisplayerUtilities.setCharacteristics(characteristics, this);
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.displayers.DataSource#getConsumers()
	 */
	public DataConsumer[] getConsumers()
	{
		return support.getConsumers();
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.displayers.DataConsumer#getDataConsumer(java.lang.Class)
	 */
	public DataConsumer getDataConsumer(Class type)
	{
		if (type == DoubleConsumer.class) {
			return this;
		}

		return DoubleConsumerMulticaster.createDataConsumer(type, this);
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.displayers.CommonDisplayer#getDataState()
	 */
	public DataState getDataState()
	{
		return dataState;
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.displayers.DataConsumer#getDefaultDataConsumer()
	 */
	public DataConsumer getDefaultDataConsumer()
	{
		return this;
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.displayers.DataConsumer#getSupportedCharacteristics()
	 */
	public String[] getSupportedCharacteristics()
	{
		return DisplayerUtilities.COMMON_NUMERIC_DISPLAYER_CHARACTERISTICS;
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.displayers.DataSource#getSupportedConsumersTypes()
	 */
	public Class[] getAcceptableConsumerTypes()
	{
		return support.getAcceptableConsumerTypes();
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.displayers.CommonDisplayer#isSuspended()
	 */
	public boolean isSuspended()
	{
		return isEnabled();
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.displayers.DataSource#addConsumer(com.cosylab.gui.displayers.DataConsumer)
	 */
	public void addConsumer(DataConsumer consumer) throws PropertyVetoException
	{
		if (consumer == null) {
			throw new NullPointerException("consumer");
		}

		NonblockingNumberConsumer c = (NonblockingNumberConsumer)consumer
			.getDataConsumer(NonblockingNumberConsumer.class);

		if (c == null) {
			throw new PropertyVetoException("Consumer '" + consumer
			    + "' must support NonblockingNumberConsumer.", null);
		}

		support.addConsumer(c);
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.displayers.CommonDisplayer#cleanup()
	 */
	public void cleanup()
	{
		internalCleanup();
		updateDataState(new DataState(DataState.NOT_INITIALIZED));
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.displayers.Setter#addSetRequestListener(com.cosylab.gui.displayers.SetRequestListener)
	 */

	//public void addSetRequestListener(SetRequestListener l) {
	//	setter.addSetRequestListener(l);
	//}
	//	/* (non-Javadoc)
	//	 * @see com.cosylab.gui.displayers.Setter#removeSetRequestListener(com.cosylab.gui.displayers.SetRequestListener)
	//	 */
	//	public void removeSetRequestListener(SetRequestListener l) {
	//		setter.removeSetRequestListener(l);
	//	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.displayers.Setter#getSetRequestListeners()
	 */

	//	public SetRequestListener[] getSetRequestListeners() {
	//		return setter.getSetRequestListeners();
	//	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.displayers.Setter#getTracksRequests()
	 */

	//	public boolean getTracksRequests() {
	//		return setter.getTracksRequests();
	//	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.displayers.Setter#setTracksRequests(boolean)
	 */

	//	public void setTracksRequests(boolean value) {
	//		setter.setTracksRequests(value);
	//	}
	//
	//	/* (non-Javadoc)
	//	 * @see com.cosylab.gui.displayers.Setter#setTargetValueAsObject(java.lang.Object)
	//	 */
	//	public void setTargetValueAsObject(Object targetValue) {
	//		if (targetValue instanceof Double) setValue(((Double)targetValue).doubleValue());
	//	}
	//
	//	/* (non-Javadoc)
	//	 * @see com.cosylab.gui.displayers.Setter#getTargetValueAsObject()
	//	 */
	//	public Object getTargetValueAsObject() {
	//		return new Double(getValue());
	//	}
	//
	//	/* (non-Javadoc)
	//	 * @see com.cosylab.gui.displayers.Setter#reply(boolean, java.lang.Object)
	//	 */
	//	public boolean reply(boolean success, Object newTargetValue) {
	//		requests--;
	//		return false;
	//	}
	//
	//	/* (non-Javadoc)
	//	 * @see com.cosylab.gui.displayers.Setter#reply(com.cosylab.gui.displayers.SetRequestEvent)
	//	 */
	//	public boolean reply(SetRequestEvent event) {
	//		requests--;
	//		return false;
	//	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.displayers.Displayer#getAdapter()
	 */

	//public Adapter getAdapter() {
	//	return delegate.getAdapter();
	//}
	//	/* (non-Javadoc)
	//	 * @see com.cosylab.gui.displayers.Displayer#setAdapter(com.cosylab.gui.adapters.Adapter)
	//	 */
	//	public void setAdapter(Adapter ad) throws PropertyVetoException {
	//		delegate.setAdapter(ad);
	//		requests = 0;
	//
	//	}
	//
	//	/* (non-Javadoc)
	//	 * @see com.cosylab.gui.displayers.Displayer#setSettable(boolean)
	//	 */
	//	public void setSettable(boolean settable) {
	//		setEnabled(settable);
	//		requests = 0;
	//	}
	//
	//	/* (non-Javadoc)
	//	 * @see com.cosylab.gui.displayers.Displayer#isSettable()
	//	 */
	//	public boolean isSettable() {
	//		return isEnabled();
	//	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.displayers.Displayer#getDataManager()
	 */

	//public DisplayerDataManager getDataManager() {
	//	return delegate;
	//}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.displayers.CommonDisplayer#getDisplayerManager()
	 */

	//	public CommonDisplayerManager getDisplayerManager() {
	//		return delegate;
	//	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.core.CosyComponent#getCosyPanelParent()
	 */

	//public CosyApplicationPanel getCosyPanelParent() {
	//	return delegate.getCosyPanelParent();
	//}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.core.CosyComponent#destroy()
	 */

	//	public void destroy() {
	//	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.core.CosyComponent#setCosyPanelParent(com.cosylab.gui.core.CosyApplicationPanel)
	 */

	//	public void setCosyPanelParent(CosyApplicationPanel panel) {
	//		delegate.setCosyPanelParent(panel);
	//	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.core.CosyComponent#destroy()
	 */
	public void destroy()
	{
		cleanup();
		support.removeAllConsumers();
		support = null;
		dataState = null;
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.displayers.DataSource#removeConsumer(com.cosylab.gui.displayers.DataConsumer)
	 */
	public void removeConsumer(DataConsumer consumer)
	{
		support.removeConsumer(consumer);
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.displayers.CommonDisplayer#resume()
	 */
	public void resume()
	{
		if (suspendCount > 0) {
			suspendCount--;
		}

		if (suspendCount == 0) {
			setEnabled(true);
		}
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.displayers.CommonDisplayer#suspend()
	 */
	public void suspend()
	{
		suspendCount++;
		setEnabled(false);
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.displayers.DataConsumer#updateDataState(com.cosylab.gui.displayers.DataState)
	 */
	public void updateDataState(DataState state)
	{
		DataState old = dataState;
		dataState = state;
		firePropertyChange(DATA_STATE, old, dataState);
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.displayers.DoubleConsumer#updateValue(long, double)
	 */
	public void updateValue(long timestamp, double value)
		throws CommonException
	{
		setValue(value);
		/*
		 * Bug fix for RT 5053
		 */
	
		if (flag) {
			synchronize();
			flag = false;
		} 
	}

	private void initialize()
	{
		setBorder(new CosyUIElements.PanelFlushBorder());
		setBackground(ColorHelper.getCosyControl());
		addSetListener(new SetListener() {
				public void setPerformed(SetEvent e)
				{
					try {
						double value = e.getDoubleValue();

						DataConsumer[] d = getConsumers();

						for (int i = 0; i < d.length; i++) {
							NonblockingNumberConsumer dd = (NonblockingNumberConsumer)d[i];

							if (dd != null) {
								dd.updateNonblocking(new Double(value));
							}
						}
					} catch (Exception ex) {
						ex.printStackTrace();
					}
				}
			});
		setPopupEnabled(true);
		internalCleanup();
	}

	private void internalCleanup()
	{
		setTitle("Dialknob");
		setTitleVisible(true);
		setUnits(null);
		setUnitsVisible(true);
		getRangedValue().setValue(0, 100, 0);
		getUserRangedValue().setValue(0, 100, 0);
		setValuePolicy(new TrimValuePolicy());
		setEditable(true);
		setFormat("%3.2f");
		setEnhanced(true);
		setResizable(true);
		setTiltingEnabled(true);
		flag = true;
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.displayers.DataConsumer#getSupportedConsumerTypes()
	 */
	public Class[] getSupportedConsumerTypes()
	{
		return DoubleConsumerMulticaster.PREFERED_CONSUMER_TYPES;
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.displayers.Displayer#setDataSource(com.cosylab.gui.displayers.DataSource)
	 */
	public void setDataSource(DataSource dataSource)
		throws PropertyVetoException
	{
		DisplayerUtilities.prepareNewDataSource(dataSource,this);
		
		DataSource old= this.dataSource;
		this.dataSource = dataSource;
		
		firePropertyChange(DATA_SOURCE,old,dataSource);
	}

	
	/**
	 * Returns the converter.
	 * @return Returns the converter.
	 */
	public Converter getConverter() {
		return converter;
	}

	
	/**
	 * Sets new converter.
	 * @param converter The converter to set.
	 * @throws PropertyVetoException if set fails
	 */
	public void setConverter(Converter converter) throws PropertyVetoException {
		if (this.converter != null && this.converter.equals(converter) ||
				(this.converter == null && converter == null)) return;
		DisplayerUtilities.prepareNewConverter(converter,this);
		
		Converter old= this.converter;
		this.converter = converter;
		
		firePropertyChange(CONVERTER_PROPERTY,old,this.converter);
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.displayers.DataSource#removeAllConsumers()
	 */
	public void removeAllConsumers()
	{
		support.removeAllConsumers();
	}
	
	/* (non-Javadoc)
	 * @see com.cosylab.gui.displayers.Displayer#getDataSource()
	 */
	public DataSource getDataSource() {
		return dataSource;
	}
	

	/**
	 * Loads the default
	 * <code>com.cosylab.gui.components.customizer.Customizer</code> for this
	 * displayer and returns it.
	 *
	 * @return the Customizer intance for this displayer
	 */
	public AbstractCustomizerPanel getCustomizer()
	{
		if (customizer == null) {
			customizer = AbstractCustomizerPanel.findCustomizer(this);
		}

		return customizer;
	}
	
	public InfoDialog getInfoDialog() {
		if (infoDialog == null) {
			infoDialog = new InfoDialog(this);
		}
		return infoDialog;
	}
	
	public PopupManager getPopupManager()
	{
		if (popupManager == null) {
			popupManager = super.getPopupManager();
			popupManager.addAction(new AbstractAction("Info...") {
	
					public void actionPerformed(ActionEvent e)
					{
						getInfoDialog().setVisible(true);
					}
				});
		}
		
		return popupManager;
	}
	

	/**
	 * @param arg0
	 */
	public static void main(String[] arg0) {
		JFrame f = new JFrame();
		f.getContentPane().setLayout(new BorderLayout());
		f.getContentPane().add(new DialKnobDisplayer());
		f.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		f.show();
		
	}
}

/* __oOo__ */
