package com.cosylab.gui;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;

import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.SwingConstants;

import com.cosylab.gui.components.util.IconHelper;
import com.cosylab.gui.components.util.PaintHelper;

/**
 * 
 * @author Jaka Bobnar, Cosylab
 *
 */
public class IconEditor extends JPanel {
	
	private static final long serialVersionUID = 7625485410292542663L;

	public static final String ICON = "icon";
	
	private JLabel iconLabel;
	private JLabel titleLabel;
	private JTextField pathField;
	private String title;
	private Icon selectedIcon;
	private static final int HEIGHT = 32;
	private static final int WIDTH = 32;
	private static final Icon NULL_ICON = new Icon() {
		public int getIconHeight() {
			return HEIGHT;
		}
		public int getIconWidth() {
			return WIDTH;
		}
		public void paintIcon(Component c, Graphics g, int x, int y) {
			PaintHelper.paintDisabled(g, x, y, WIDTH, HEIGHT);			
		}
	};
	
	public IconEditor() {
		this("Icon");
	}
	
	public IconEditor(String title) {
		super();
		this.title = title;
		initialize();
	}
	
	private void initialize() {
		setLayout(new GridBagLayout());
		titleLabel = new JLabel(title);
		this.add(titleLabel, new GridBagConstraints(0,0,2,1,1,0,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(11,11,1,11),0,0));
		this.add(getPathField(), new GridBagConstraints(0,1,1,1,1,0,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(1,11,11,0),0,0));
		this.add(getIconLabel(), new GridBagConstraints(1,1,1,1,0,0,GridBagConstraints.EAST, GridBagConstraints.HORIZONTAL, new Insets(1,4,11,11),0,0));
	}
	
	private JTextField getPathField() {
		if (pathField == null) {
			pathField = new JTextField();
			pathField.setMaximumSize(new Dimension(100,21));
			pathField.setPreferredSize(new Dimension(100,21));
			pathField.setMinimumSize(new Dimension(100,21));
			pathField.addKeyListener(new KeyAdapter() {
				@Override
				public void keyReleased(KeyEvent e) {
					internalSetIcon(IconHelper.createIcon(pathField.getText()), false);
				}
			});
			pathField.setToolTipText("Type image file path from classpath.");
		}

		return pathField;
	}
	
	private JLabel getIconLabel() {
		if (iconLabel == null) {
			iconLabel = new JLabel();
			iconLabel.setHorizontalAlignment(SwingConstants.RIGHT);
			iconLabel.setMinimumSize(new Dimension(WIDTH + 5, HEIGHT + 5));
			iconLabel.setMaximumSize(new Dimension(WIDTH + 5, HEIGHT + 5));
			iconLabel.setPreferredSize(new Dimension(WIDTH + 5, HEIGHT + 5));
			iconLabel.setIcon(NULL_ICON);
		}
		return iconLabel;
	}
	
	public void setIcon(Icon icon) {
		internalSetIcon(icon, true);
	}
	
	public void setIcon(String path) {
		String old = getIconPath();
		if ((old != null && old.equals(path)) || (old == null && path == null)) return;
		internalSetIcon(IconHelper.createIcon(path), true);
		getPathField().setText(path);
	}
	
	private void internalSetIcon(Icon icon, boolean setPathField) {
		Icon oldValue = getIconLabel().getIcon();
		if ((oldValue != null && oldValue.equals(icon)) || 
			(oldValue == null && icon == null)) return;
		selectedIcon = icon;
		if (icon instanceof ImageIcon && icon.getIconHeight() > HEIGHT && icon.getIconWidth() > WIDTH) {
			getIconLabel().setIcon(IconCustomizer.getScaledIcon((ImageIcon)icon, HEIGHT,WIDTH));
		} else if (icon != null){
			getIconLabel().setIcon(icon);
		} else {
			getIconLabel().setIcon(NULL_ICON);
		}
		
		if (icon != null && setPathField)
			getPathField().setText(icon.toString());
		
		firePropertyChange(ICON, oldValue, icon);
	}
	
	public Icon getIcon() {
		return selectedIcon;
	}

	public String getTitle() {
		return titleLabel.getText();
	}
	
	public String getIconPath() {
		return getPathField().getText();
	}

	public void setTitle(String title) {
		this.title = title;
		titleLabel.setText(title);
	}
}
