package com.cosylab.gui;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;

import com.cosylab.gui.displayers.ConvertibleDisplayer;
import com.cosylab.gui.displayers.DataStateProvider;
import com.cosylab.gui.displayers.Displayer;
import com.cosylab.gui.infopanel.ConverterPanel;
import com.cosylab.gui.infopanel.DebugPanel;
import com.cosylab.gui.infopanel.PropertiesPanel;

/**
 * 
 * @author Jaka Bobnar, Cosylab
 *
 */
public class InfoDialog extends JDialog {
	
	private static final long serialVersionUID = 513076582058739594L;
	
	private JPanel contentPanel;
	private JTabbedPane tabPane;
	private ConverterPanel converterPanel;
	private PropertiesPanel propertiesPanel;
	private DebugPanel debugPanel;
	private Displayer displayer;
	private JButton closeButton;
	
	private PropertyChangeListener propertyListener;
	
	public InfoDialog() {
		this(null);
	}
	
	public InfoDialog(Displayer displayer) {
		initialize();
		setDisplayer(displayer);
		addPanel(getDebugPanel(), "Debug", 0, true);
		addPanel(getPropertiesPanel(), "Bean properties", 1, false);
		addPanel(getConverterPanel(),"Converter", 2, false);
		
	}
	
	private void initialize() {
		this.setSize(300,300);
		this.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
		this.setContentPane(getContentPanel());
	}
	
	private JPanel getContentPanel() {
		if (contentPanel == null) {
			contentPanel = new JPanel(new BorderLayout());
			contentPanel.add(getTabPane(), BorderLayout.CENTER);
			JPanel buttonPanel = new JPanel(new GridBagLayout());
			buttonPanel.add(getCloseButton(), new GridBagConstraints(0,0,1,1,1,1,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(1,1,1,1),0,0));
			contentPanel.add(buttonPanel, BorderLayout.SOUTH);
			
		}
		return contentPanel;
	}
	
	private JButton getCloseButton() {
		if (closeButton == null) {
			closeButton = new JButton("Close");
			closeButton.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					dispose();					
				}
			});
		}
		return closeButton;
	}
	
	private JTabbedPane getTabPane() {
		if (tabPane == null) {
			tabPane = new JTabbedPane();
		}
		return tabPane;
	}
	
	protected ConverterPanel getConverterPanel() {
		if (converterPanel == null) {
			converterPanel = new ConverterPanel();
		}
		return converterPanel;
	}
	
	protected PropertiesPanel getPropertiesPanel() {
		if (propertiesPanel == null) {
			propertiesPanel = new PropertiesPanel();
		}

		return propertiesPanel;
	}
	
	protected DebugPanel getDebugPanel() {
		if (debugPanel == null) {
			debugPanel = new DebugPanel();
		}
		return debugPanel;
	}
	
	protected void addPanel(Component panel, String name, int position, boolean select) {
		for (int i = 0; i < getTabPane().getTabCount(); i++) {
			if (getTabPane().getTitleAt(i).equals(name)) {
				getTabPane().removeTabAt(i);
				break;
			}
		}
		getTabPane().add(panel, name, position);
		if (select) {
			getTabPane().setSelectedComponent(panel);
		}
	}
	
	protected void addPanel(Component panel, String name) {
		for (int i = 0; i < getTabPane().getTabCount(); i++) {
			if (getTabPane().getTitleAt(i).equals(name)) {
				getTabPane().removeTabAt(i);
				break;
			}
		}
		getTabPane().add(panel, name);
	}
	
	public void setDisplayer(Displayer displayer) {
		if (this.displayer != null && !this.displayer.equals(displayer)) {
			this.displayer.removePropertyChangeListener(getPropertyListener());
			if (displayer != null) {
				displayer.addPropertyChangeListener(getPropertyListener());
			}
				
		} else if (this.displayer == null && displayer != null) {
			displayer.addPropertyChangeListener(getPropertyListener());
		}
		this.displayer = displayer;
		if (this.displayer != null) {
			this.setTitle(displayer.getName());
			getConverterPanel().setDataSource(this.displayer.getDataSource());
			if (displayer instanceof ConvertibleDisplayer)
				getConverterPanel().setConverter(((ConvertibleDisplayer)this.displayer).getConverter());
			getPropertiesPanel().setBean(displayer);
			getDebugPanel().setDataState(this.displayer.getDataState());
		} else {
			this.setTitle("Info");
			getConverterPanel().setConverter(null);
			getConverterPanel().setDataSource(null);
			getPropertiesPanel().setBean(new Object());
			getDebugPanel().setDataState(null);
		}
	}
	
	private PropertyChangeListener getPropertyListener() {
		if (propertyListener == null) {
			propertyListener = new PropertyChangeListener() {
				public void propertyChange(PropertyChangeEvent evt) {
					if (Displayer.DATA_SOURCE.equals(evt.getPropertyName())) {
						getConverterPanel().setDataSource(getDisplayer().getDataSource());
					} else if (ConvertibleDisplayer.CONVERTER_PROPERTY.equals(evt.getPropertyName())) {
						if (getDisplayer() instanceof ConvertibleDisplayer)
							getConverterPanel().setConverter(((ConvertibleDisplayer)getDisplayer()).getConverter());
					} else if (DataStateProvider.DATA_STATE.equals(evt.getPropertyName())) {
						getDebugPanel().setDataState(getDisplayer().getDataState());
					}
				}
			};
		}
		return propertyListener;
	}
	
	public Displayer getDisplayer() {
		return this.displayer;
	}
	
	public static void main(String[] args) {
		new InfoDialog().setVisible(true);
	}
	
}
