/*
 * Copyright (c) 2004 by Cosylab d.o.o.
 *
 * The full license specifying the redistribution, modification, usage and other
 * rights and obligations is included with the distribution of this project in
 * the file license.html. If the license is not included you may find a copy at
 * http://www.cosylab.com/legal/abeans_license.htm or may write to Cosylab, d.o.o.
 *
 * THIS SOFTWARE IS PROVIDED AS-IS WITHOUT WARRANTY OF ANY KIND, NOT EVEN THE
 * IMPLIED WARRANTY OF MERCHANTABILITY. THE AUTHOR OF THIS SOFTWARE, ASSUMES
 * _NO_ RESPONSIBILITY FOR ANY CONSEQUENCE RESULTING FROM THE USE, MODIFICATION,
 * OR REDISTRIBUTION OF THIS SOFTWARE.
 */

package com.cosylab.gui;

import java.awt.BorderLayout;
import java.awt.Button;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.PropertyVetoException;
import java.util.Map;

import javax.swing.AbstractAction;
import javax.swing.JButton;
import javax.swing.JFrame;

import com.cosylab.events.SetEvent;
import com.cosylab.events.SetListener;
import com.cosylab.gui.adapters.Converter;
import com.cosylab.gui.components.Slider;
import com.cosylab.gui.components.util.PopupManager;
import com.cosylab.gui.displayers.ConvertibleDisplayer;
import com.cosylab.gui.displayers.DataConsumer;
import com.cosylab.gui.displayers.DataSource;
import com.cosylab.gui.displayers.DataSourceSupport;
import com.cosylab.gui.displayers.DataState;
import com.cosylab.gui.displayers.DisplayerUtilities;
import com.cosylab.gui.displayers.DoubleConsumer;
import com.cosylab.gui.displayers.DoubleConsumerMulticaster;
import com.cosylab.gui.displayers.DoubleDisplayer;
import com.cosylab.gui.displayers.NonblockingNumberConsumer;
import com.cosylab.util.CommonException;


/**
 * Displays value on editable slider.
 *
 * @author <a href="mailto:ales.pucelj@cosylab.com">Ales Pucelj</a>
 * @version $id$
 */
public class SliderDisplayer extends Slider implements DoubleDisplayer,
	DataSource, ConvertibleDisplayer
{
	private DataSourceSupport support = new DataSourceSupport(new Class[]{
			    NonblockingNumberConsumer.class
		    });
	private DataState dataState = new DataState(DataState.UNDEFINED);
	private boolean newUpdate = false;
	private DataSource dataSource;
	private double lastValue = 0;
	private InfoDialog infoDialog;
	private PopupManager popupManager;

	//TODO
	private int suspendCount = 0;

	/**
	 * Constructor for SliderDisplayer.
	 */
	public SliderDisplayer()
	{
		super();
		initializeDisplayer();
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.displayers.DataSource#getSupportedConsumersTypes()
	 */
	public Class[] getAcceptableConsumerTypes()
	{
		return support.getAcceptableConsumerTypes();
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.displayers.DataConsumer#setCharacteristics(java.util.Map)
	 */
	public void setCharacteristics(Map characteristics)
	{
		if (characteristics == null) {
			throw new NullPointerException("characteristics");
		}

		newUpdate = true;

		DisplayerUtilities.setCharacteristics(characteristics, this);
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.displayers.DataSource#getConsumers()
	 */
	public DataConsumer[] getConsumers()
	{
		return support.getConsumers();
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.displayers.DataConsumer#getDataConsumer(java.lang.Class)
	 */
	public DataConsumer getDataConsumer(Class type)
	{
		if (type == DoubleConsumer.class) {
			return this;
		}

		return DoubleConsumerMulticaster.createDataConsumer(type, this);
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.displayers.CommonDisplayer#getDataState()
	 */
	public DataState getDataState()
	{
		return dataState;
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.displayers.DataConsumer#getDefaultDataConsumer()
	 */
	public DataConsumer getDefaultDataConsumer()
	{
		return this;
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.displayers.DataConsumer#getSupportedCharacteristics()
	 */
	public String[] getSupportedCharacteristics()
	{
		return DisplayerUtilities.COMMON_NUMERIC_DISPLAYER_CHARACTERISTICS;
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.displayers.DataConsumer#getSupportedConsumerTypes()
	 */
	public Class[] getSupportedConsumerTypes()
	{
		return DoubleConsumerMulticaster.PREFERED_CONSUMER_TYPES;
	}

	/**
	 * Accepts only consumers, which support
	 * <code>NonblockingNumberConsumer</code> which is used for receiving
	 * updates from user.
	 *
	 * @see com.cosylab.gui.displayers.DataSource#addConsumer(com.cosylab.gui.displayers.DataConsumer)
	 */
	public void addConsumer(DataConsumer consumer) throws PropertyVetoException
	{
		if (consumer == null) {
			throw new NullPointerException("consumer");
		}

		NonblockingNumberConsumer c = (NonblockingNumberConsumer)consumer
			.getDataConsumer(NonblockingNumberConsumer.class);

		if (c == null) {
			throw new PropertyVetoException("Consumer '" + consumer
			    + "' must support NonblockingNumberConsumer.", null);
		}

		support.addConsumer(c);
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.displayers.CommonDisplayer#cleanup()
	 */
	public void cleanup()
	{
		internalCleanup();
		updateDataState(new DataState(DataState.NOT_INITIALIZED));
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.core.CosyComponent#destroy()
	 */
	public void destroy()
	{
		cleanup();
		support.removeAllConsumers();
		dataState = null;
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.displayers.DataSource#removeConsumer(com.cosylab.gui.displayers.DataConsumer)
	 */
	public void removeConsumer(DataConsumer consumer)
	{
		support.removeConsumer(consumer);
	}

	/**
	 * Tells if slider is in suspend mode
	 *
	 * @return true if suspended
	 */
	public boolean isSuspended()
	{
		return (suspendCount > 0);
	}

	/**
	 * Resumes slider from suspend state , and sets last value(when was
	 * suspendet)
	 */
	public void resume()
	{
		if (suspendCount > 0) {
			suspendCount--;
		}

		if (suspendCount == 0) {
			setEnabled(true);
			setReadback(lastValue);

			if (newUpdate) {
				synchronize();
				newUpdate = false;
			}
		}
	}

	/**
	 * Suspend slider
	 */
	public void suspend()
	{
		setEnabled(false);
		suspendCount++;
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.displayers.DataConsumer#updateDataState(com.cosylab.gui.displayers.DataState)
	 */
	public void updateDataState(DataState state)
	{
		DataState old = dataState;
		dataState = state;
		firePropertyChange(DATA_STATE, old, dataState);
	}

	protected short update = 0;
	private Converter converter;
	
	/* (non-Javadoc)
	 * @see com.cosylab.gui.displayers.DoubleConsumer#updateValue(long, double)
	 */
	public void updateValue(long timestamp, double value)
		throws CommonException
	{
	    lastValue = value;
	    
	    
	    if (!isSuspended()) {
			setReadback(value);

			if (newUpdate) {
				synchronize();
				newUpdate = false;
			}
			
			if (update < 2) {
			    synchronize();
			    update++;
			}
					
		}
		
	}

	private void initializeDisplayer()
	{
		addSetListener(new SetListener() {
				public void setPerformed(SetEvent e)
				{
					try {
						double value = e.getDoubleValue();
						
						DataConsumer[] d = getConsumers();

						for (int i = 0; i < d.length; i++) {
							NonblockingNumberConsumer dd = (NonblockingNumberConsumer)d[i];

							if (dd != null) {
								dd.updateNonblocking(new Double(value));
							}
						}
					} catch (Exception ex) {
						ex.printStackTrace();
					}
				}
			});
		setPopupEnabled(true);
		internalCleanup();
	}

	private void internalCleanup()
	{
		setMinimum(0);
		setMaximum(100);
		setValue(0);
		setTitle("Slider");
		setUnits(null);
		setUnitsVisible(true);
		setFormat("%3.2f");
		setEditable(true);
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.displayers.Displayer#getDataSource()
	 */
	public DataSource getDataSource()
	{
	    return dataSource;
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.displayers.Displayer#setDataSource(com.cosylab.gui.displayers.DataSource)
	 */
	public void setDataSource(DataSource dataSource)
		throws PropertyVetoException
	{
		DisplayerUtilities.prepareNewDataSource(dataSource,this);
		
		DataSource old= this.dataSource;
		this.dataSource = dataSource;
		
		firePropertyChange(DATA_SOURCE,old,dataSource);
	}

	
	/**
	 * Returns the converter.
	 * @return Returns the converter.
	 */
	public Converter getConverter() {
		return converter;
	}

	
	/**
	 * Sets new converter.
	 * @param converter The converter to set.
	 * @throws PropertyVetoException if set fails
	 */
	public void setConverter(Converter converter) throws PropertyVetoException {
		if (this.converter != null && this.converter.equals(converter) ||
				(this.converter == null && converter == null)) return;
		DisplayerUtilities.prepareNewConverter(converter,this);
		
		Converter old= this.converter;
		this.converter = converter;
		
		firePropertyChange(CONVERTER_PROPERTY,old,this.converter);
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.displayers.DataSource#removeAllConsumers()
	 */
	public void removeAllConsumers()
	{
		support.removeAllConsumers();
	}
	
	public InfoDialog getInfoDialog() {
		if (infoDialog == null) {
			infoDialog = new InfoDialog(this);
		}
		return infoDialog;
	}
	
	public PopupManager getPopupManager()
	{
		if (popupManager == null) {
			popupManager = super.getPopupManager();
			popupManager.addAction(new AbstractAction("Info...") {
	
					public void actionPerformed(ActionEvent e)
					{
						getInfoDialog().setVisible(true);
					}
				});
		}
		
		return popupManager;
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @param args
	 */
	public static void main(String[] args)
	{
		JFrame f = new JFrame();
		f.getContentPane().setLayout(new BorderLayout());
		f.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

		final SliderDisplayer s = new SliderDisplayer();
		f.getContentPane().add(s, BorderLayout.SOUTH);

		JButton b = new JButton("S/R");
		f.getContentPane().add(b, BorderLayout.EAST);
		b.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e)
				{
					if (s.isSuspended()) {
						s.resume();
					} else {
						s.suspend();
					}
				}
			});
		f.show();
	}

}

/* __oOo__ */
