package com.cosylab.gui.adapters;

import java.awt.Component;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.beans.PropertyEditor;
import java.beans.PropertyEditorSupport;
import java.beans.PropertyVetoException;

import com.cosylab.gui.displayers.ConvertibleDisplayer;

/**
 * 
 * @author Jaka Bobnar, Cosylab
 *
 */
public class ConverterCustomizerPropertyEditor extends PropertyEditorSupport
		implements PropertyEditor {
	private static final long serialVersionUID = 1L;

	ConverterCustomizer customizer = null;

	public ConverterCustomizerPropertyEditor() {
		super();
	}

	/*
	 * (non-Javadoc)
	 * @see java.beans.PropertyEditorSupport#getCustomEditor()
	 */
	public Component getCustomEditor() {
		if (customizer == null) {
			customizer = new ConverterCustomizer();
			customizer.addPropertyChangeListener(ConvertibleDisplayer.CONVERTER_PROPERTY,
					new PropertyChangeListener() {
						public void propertyChange(PropertyChangeEvent e) {
							setValueSilently(customizer.getConverter());
						}
					});
		}
		return customizer;
	}

	private void setValueSilently(Converter value) {
		super.setValue(value);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.beans.PropertyEditorSupport#setValue(java.lang.Object)
	 */
	public void setValue(Object value) {
		super.setValue(value);
		if (customizer != null) {
			customizer.setConverter((Converter) value);
		}
	}
	
	/*
	 * (non-Javadoc)
	 * @see java.beans.PropertyEditorSupport#setAsText(java.lang.String)
	 */
	public void setAsText(String text) {
		String[] values = text.split(";");
		Converter[] converters = null;
		if (values != null && values.length > 0) {
			converters = new Converter[values.length];
			for (int i = 0; i < values.length; i++) {
				converters[i] = ConverterUtilities.getConverterFromString(values[i]);
			}
		} else {
			converters = new Converter[]{ConverterUtilities.getConverterFromString(text)};
		}
		try {
			setValue(new ConverterChain(converters));
		} catch (PropertyVetoException e) {
			e.printStackTrace();
		}
    }

	/*
	 * (non-Javadoc)
	 * @see java.beans.PropertyEditorSupport#getJavaInitializationString()
	 */
	public String getJavaInitializationString() {
		if (getValue() != null && getValue() instanceof Converter) {
			return ConverterUtilities.getInitializationString((Converter)getValue());
		} else {
			return "";
		}
	}
	
	/*
	 * (non-Javadoc)
	 * @see java.beans.PropertyEditorSupport#supportsCustomEditor()
	 */
	public boolean supportsCustomEditor() {
		return true;
	}
}
