/*
 * Copyright (c) 2003 by Cosylab d.o.o.
 *
 * The full license specifying the redistribution, modification, usage and other
 * rights and obligations is included with the distribution of this project in
 * the file license.html. If the license is not included you may find a copy at
 * http://www.cosylab.com/legal/abeans_license.htm or may write to Cosylab, d.o.o.
 *
 * THIS SOFTWARE IS PROVIDED AS-IS WITHOUT WARRANTY OF ANY KIND, NOT EVEN THE
 * IMPLIED WARRANTY OF MERCHANTABILITY. THE AUTHOR OF THIS SOFTWARE, ASSUMES
 * _NO_ RESPONSIBILITY FOR ANY CONSEQUENCE RESULTING FROM THE USE, MODIFICATION,
 * OR REDISTRIBUTION OF THIS SOFTWARE.
 */

package com.cosylab.gui.adapters;

import java.util.Map;

import com.cosylab.gui.displayers.DataConsumer;
import com.cosylab.gui.displayers.DataSourceSupport;
import com.cosylab.gui.displayers.DataState;


/**
 * <code>DataConsumerDispatcher</code> is support class for data consumer
 * dispatcher implementators.
 *
 * @author <a href="mailto:igor.kriznar@cosylab.com">Igor Kriznar</a>
 * @version $Id: DataConsumerDispatcher.java,v 1.4 2007-06-14 17:21:01 ikriznar Exp $
 *
 * @since Feb 14, 2004.
 */
public abstract class DataConsumerDispatcher extends DataSourceSupport
	implements DataConsumer
{
	protected String name = "DataConsumerDisplatcher";

	/**
	 * DOCUMENT ME!
	 *
	 * @param supportedTypes
	 *
	 * @throws IllegalArgumentException DOCUMENT ME!
	 */
	public DataConsumerDispatcher(Class[] supportedTypes)
	{
		super(supportedTypes);
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.displayers.DataConsumer#getDefaultDataConsumer()
	 */
	public DataConsumer getDefaultDataConsumer()
	{
		return this;
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.displayers.DataConsumer#getName()
	 */
	public String getName()
	{
		return name;
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.displayers.DataConsumer#getSupportedCharacteristics()
	 */
	public String[] getSupportedCharacteristics()
	{
		return extractSupportedCharacteristics();
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.displayers.DataConsumer#getSupportedConsumerTypes()
	 */
	public Class[] getSupportedConsumerTypes()
	{
		return getAcceptableConsumerTypes();
	}

	public DataConsumer getDataConsumer(Class type) {
		if (type==null) {
			throw new NullPointerException("type");
		}
		if (type.isAssignableFrom(getClass())) {
			return this;
		}
		return null;
	}

	public void updateDataState(DataState state) {
		DataConsumer[] delegates = getConsumers();
	
		for (int i = 0; i < delegates.length; i++) {
			try {
				delegates[i].updateDataState(state);
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
	}

	public void setCharacteristics(Map characteristics) {
		DataConsumer[] delegates = getConsumers();
	
		for (int i = 0; i < delegates.length; i++) {
			try {
				delegates[i].setCharacteristics(characteristics);
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
	}
	
}

/* __oOo__ */
