package com.cosylab.gui.adapters;

import java.util.HashMap;
import java.util.Map;

import com.cosylab.gui.displayers.CommonDisplayer;


public class ExponentialConverter extends SimpleConverterSupport {

	private static final long serialVersionUID = -1789030566494308338L;

	public static final String SHORT_NAME = "exponential";
	
	private double base = Math.E, logBase = 1;
	
	public ExponentialConverter() {
		setBase(Math.E);
	}
	
	public ExponentialConverter(double base){
		setBase(base);
	}
	
	@Override
	protected double inverseTransform(double value) {
		if (value <= 0) return 0;
		return Math.log(value)/getLogBase();
	}

	@Override
	protected double transform(double value) {
		return Math.pow(getBase(), value);
	}
	public String getName(){
		return SHORT_NAME;
	}
	
	public String toString(){
		return SHORT_NAME + ": " + getBase();
	}
	
	public double getBase() {
		return base;
	}

	public void setBase(double base) {
		if (base <= 0) return;
		this.base = base;
		setLogBase(Math.log(base));
	}

	protected double getLogBase() {
		return logBase;
	}

	protected void setLogBase(double logBase) {
		this.logBase = logBase;
	}
	
	public void setCharacteristics(Map characteristics)
	{
		// We store unmodified characteristic for possible later reinitialization of 
		// consumers, eg. if multiplication factor changes
		cacheLastCharacteristics(characteristics);

		HashMap map = new HashMap(characteristics);

		double min;
		double max;
		
		// Modifying the graph_min and graph_max characteristics.
		if (map.containsKey(CommonDisplayer.C_GRAPH_MIN)
		    && map.containsKey(CommonDisplayer.C_GRAPH_MAX)) {
			min = ((Number)map.get(CommonDisplayer.C_GRAPH_MIN)).doubleValue();
			max = ((Number)map.get(CommonDisplayer.C_GRAPH_MAX)).doubleValue();
				
			map.put(CommonDisplayer.C_GRAPH_MIN,
			    new Double(transform(min)));
			map.put(CommonDisplayer.C_GRAPH_MAX,
			    new Double(transform(max)));
		}
		
		// Modifying the minimum and maximum characteristics.
		if (map.containsKey(CommonDisplayer.C_MINIMUM)
		    && map.containsKey(CommonDisplayer.C_MAXIMUM)) {
			min = ((Number)map.get(CommonDisplayer.C_MINIMUM)).doubleValue();
			max = ((Number)map.get(CommonDisplayer.C_MAXIMUM)).doubleValue();
						
			map.put(CommonDisplayer.C_MINIMUM,
			    new Double(transform(min)));
			map.put(CommonDisplayer.C_MAXIMUM,
			    new Double(transform(max)));
		}

		// Modifying the units characteristics
		map.put(CommonDisplayer.C_UNITS,
		    "log_"+getBase()+"("+ map.get(CommonDisplayer.C_UNITS)+")");

		super.setCharacteristics(map);
	}
	
	@Override
	public boolean equals(Object obj) {
		if (!(obj instanceof ExponentialConverter)) return false;
		ExponentialConverter c = (ExponentialConverter)obj;
		return c.getBase() == getBase();
	}
}
