package com.cosylab.gui.adapters;

import java.util.HashMap;
import java.util.Map;

import com.cosylab.gui.displayers.CommonDisplayer;
import com.cosylab.gui.displayers.DoubleConsumer;

public class PotentialConverter extends SimpleConverterSupport implements
		DoubleConsumer {

	private static final long serialVersionUID = 3449182777513744784L;

	public static final String SHORT_NAME = "Potential";
	
	private double exponent = 1;
	
	public PotentialConverter() {
		this(1);
	}
	
	public PotentialConverter(double exponent){
		setExponent(exponent);
	}
	
	@Override
	protected double inverseTransform(double value) {
		return Math.pow(value, 1.0 / getExponent());
	}

	public double getExponent() {
		return exponent;
	}

	@Override
	protected double transform(double value) {
		return Math.pow(value, getExponent());
	}
	
	public void  setExponent(double exponent){
		if (exponent <= 0) this.exponent = 1;
		else this.exponent = exponent;
	}
	
	public String getName(){
		return SHORT_NAME;
	}
	
	public String toString(){
		return SHORT_NAME + ": " + getExponent();
	}
	
	public void setCharacteristics(Map characteristics)
	{
		// We store unmodified characteristic for possible later reinitialization of 
		// consumers, eg. if multiplication factor changes
		cacheLastCharacteristics(characteristics);

		HashMap map = new HashMap(characteristics);

		double min;
		double max;
		
		// Modifying the graph_min and graph_max characteristics.
		if (map.containsKey(CommonDisplayer.C_GRAPH_MIN)
		    && map.containsKey(CommonDisplayer.C_GRAPH_MAX)) {
			min = ((Number)map.get(CommonDisplayer.C_GRAPH_MIN)).doubleValue();
			max = ((Number)map.get(CommonDisplayer.C_GRAPH_MAX)).doubleValue();
				
			map.put(CommonDisplayer.C_GRAPH_MIN,
			    new Double(transform(min)));
			map.put(CommonDisplayer.C_GRAPH_MAX,
			    new Double(transform(max)));
		}
		
		// Modifying the minimum and maximum characteristics.
		if (map.containsKey(CommonDisplayer.C_MINIMUM)
		    && map.containsKey(CommonDisplayer.C_MAXIMUM)) {
			min = ((Number)map.get(CommonDisplayer.C_MINIMUM)).doubleValue();
			max = ((Number)map.get(CommonDisplayer.C_MAXIMUM)).doubleValue();
						
			map.put(CommonDisplayer.C_MINIMUM,
			    new Double(transform(min)));
			map.put(CommonDisplayer.C_MAXIMUM,
			    new Double(transform(max)));
		}

//		// Modifying the units characteristics
		map.put(CommonDisplayer.C_UNITS,
		    map.get(CommonDisplayer.C_UNITS) +"^1/"+getExponent());

		super.setCharacteristics(map);
	}
	
	@Override
	public boolean equals(Object obj) {
		if (!(obj instanceof PotentialConverter)) return false;
		PotentialConverter c = (PotentialConverter)obj;
		return c.getExponent() == exponent;
	}
}
