/*
 * Copyright (c) 2004 by Cosylab d.o.o.
 *
 * The full license specifying the redistribution, modification, usage and other
 * rights and obligations is included with the distribution of this project in
 * the file license.html. If the license is not included you may find a copy at
 * http://www.cosylab.com/legal/abeans_license.htm or may write to Cosylab, d.o.o.
 *
 * THIS SOFTWARE IS PROVIDED AS-IS WITHOUT WARRANTY OF ANY KIND, NOT EVEN THE
 * IMPLIED WARRANTY OF MERCHANTABILITY. THE AUTHOR OF THIS SOFTWARE, ASSUMES
 * _NO_ RESPONSIBILITY FOR ANY CONSEQUENCE RESULTING FROM THE USE, MODIFICATION,
 * OR REDISTRIBUTION OF THIS SOFTWARE.
 */

package com.cosylab.gui.adapters;

import com.cosylab.gui.displayers.DataConsumer;
import com.cosylab.gui.displayers.DoubleSeqConsumer;
import com.cosylab.util.CommonException;


/**
 * <code>SimpleConverterSupport</code> is implementation of
 * <code>Converter</code> interface intended for simple implementation, which
 * use transfromation function on double data flow.
 * 
 * <p>
 * <b>Note!</b> This implementation works only with data sources and consumers,
 * whcih support <code>double[]</code> as value type.
 * </p>
 *
 * @author <a href="mailto:igor.kriznar@cosylab.com">Igor Kriznar</a>
 * @version $Id: SimpleArrayConverterSupport.java,v 1.2 2007/01/31 10:11:59 jbobnar Exp $
 *
 * @see com.cosylab.gui.adapters.Converter
 * @since Feb 14, 2004.
 */
public abstract class SimpleArrayConverterSupport extends AbstractConverter
	implements Converter, DoubleSeqConsumer
{

	/**
	 * Creates new converter.
	 */
	public SimpleArrayConverterSupport()
	{
		// we dalagate to super suppport class only the consumer type/types we want to suppport
		super(new Class[]{ DoubleSeqConsumer.class });

		// the name of this converter when it acts as a consumer
		name = "SimpleConverter";
	}

	/**
	 * This is called by peer data source. Call is delegated to contained
	 * consumers.
	 *
	 * @see com.cosylab.gui.displayers.DoubleConsumer#updateValue(long, double)
	 */
	public void updateValue(long timestamp, double[] value)
		throws CommonException
	{

		DataConsumer[] delegates = getConsumers();

		// transform value
		final double[] newValue = transform(value);

		for (int i = 0; i < delegates.length; i++) {
			try {
				((DoubleSeqConsumer)delegates[i]).updateValue(timestamp, newValue);
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
	}

	/**
	 * User must implement this method and from prvided parameter calculate
	 * with own transfromatin function new value and return it.
	 *
	 * @param value the value to be transfirmed
	 *
	 * @return the transformed value
	 */
	abstract protected double[] transform(double[] value);

}

/* __oOo__ */
