/*
 * Copyright (c) 2006 by Cosylab d.o.o.
 *
 * The full license specifying the redistribution, modification, usage and other
 * rights and obligations is included with the distribution of this project in
 * the file license.html. If the license is not included you may find a copy at
 * http://www.cosylab.com/legal/abeans_license.htm or may write to Cosylab, d.o.o.
 *
 * THIS SOFTWARE IS PROVIDED AS-IS WITHOUT WARRANTY OF ANY KIND, NOT EVEN THE
 * IMPLIED WARRANTY OF MERCHANTABILITY. THE AUTHOR OF THIS SOFTWARE, ASSUMES
 * _NO_ RESPONSIBILITY FOR ANY CONSEQUENCE RESULTING FROM THE USE, MODIFICATION,
 * OR REDISTRIBUTION OF THIS SOFTWARE.
 */
/**
 * 
 */
package com.cosylab.gui.components;

import java.awt.Component;
import java.awt.Graphics;
import java.awt.Rectangle;
import java.beans.IntrospectionException;
import java.beans.PropertyChangeListener;
import java.beans.PropertyEditor;
import java.beans.PropertyEditorSupport;

import com.cosylab.util.BeanInfoSupport;


/**
 * @author Igor Kriznar (igor.kriznarATcosylab.com)
 *
 */
public class AbstractDisplayerPanelBeanInfo extends BeanInfoSupport {
	
	public class OrientationPropertyEditor extends PropertyEditorSupport {
		private static final String DYNAMIC_LAYOUT = "Automatic Layout";
		private static final String VERTICAL_LAYOUT = "Vertical Layout";
		private static final String HORIZONTAL_LAYOUT = "Horizontal Layout";

		private int value=0;

		public boolean isPaintable() {
			return false;
		}

		public boolean supportsCustomEditor() {
			return false;
		}

		public Component getCustomEditor() {
			return null;
		}

		public Object getValue() {
			return new Integer(value);
		}

		public void setValue(Object value) {
			this.value = ((Number)value).intValue();
			firePropertyChange();
		}

		public String getAsText() {
			if (value==AbstractDisplayerPanel.DYNAMIC_LAYOUT) return DYNAMIC_LAYOUT;
			if (value==AbstractDisplayerPanel.HORIZONTAL_LAYOUT) return HORIZONTAL_LAYOUT;
			if (value==AbstractDisplayerPanel.VERTICAL_LAYOUT) return VERTICAL_LAYOUT;
			return null;
		}

		public String getJavaInitializationString() {
			return Integer.toString(value);
		}

		public String[] getTags() {
			return new String[]{HORIZONTAL_LAYOUT,VERTICAL_LAYOUT,DYNAMIC_LAYOUT};
		}

		public void setAsText(String text) throws IllegalArgumentException {
			if (text.equals(DYNAMIC_LAYOUT)) value = AbstractDisplayerPanel.DYNAMIC_LAYOUT;
			if (text.equals(HORIZONTAL_LAYOUT)) value = AbstractDisplayerPanel.HORIZONTAL_LAYOUT;
			if (text.equals(VERTICAL_LAYOUT)) value = AbstractDisplayerPanel.VERTICAL_LAYOUT;			
			firePropertyChange();
		}

	}	

	
	/**
	 * Creates new instance of AbstractDisplayerPanelBeanInfo.
	 * @throws IntrospectionException 
	 */
	public AbstractDisplayerPanelBeanInfo() throws IntrospectionException {
		setBeanDescriptor(AbstractDisplayerPanelCustomizer.class);
		loadDefaultPropertyDescriptors();
		setPropertyEditor("layoutOrientation",OrientationPropertyEditor.class);
		removePropertyDescriptor("state");
	}

}
