/*
 * Copyright (c) 2003 by Cosylab d.o.o.
 *
 * The full license specifying the redistribution, modification, usage and other
 * rights and obligations is included with the distribution of this project in
 * the file license.html. If the license is not included you may find a copy at
 * http://www.cosylab.com/legal/abeans_license.htm or may write to Cosylab, d.o.o.
 *
 * THIS SOFTWARE IS PROVIDED AS-IS WITHOUT WARRANTY OF ANY KIND, NOT EVEN THE
 * IMPLIED WARRANTY OF MERCHANTABILITY. THE AUTHOR OF THIS SOFTWARE, ASSUMES
 * _NO_ RESPONSIBILITY FOR ANY CONSEQUENCE RESULTING FROM THE USE, MODIFICATION,
 * OR REDISTRIBUTION OF THIS SOFTWARE.
 */

package com.cosylab.gui.components;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;

import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;

import javax.swing.JDialog;


/**
 * This is a container for DateChooser and SimpleTimeSelector components,
 * combining the functionality of both. The components consists of six
 * user-editable fields for year, month, day, hour, minute and seconds. All
 * fields are validating allowing user to select only valid date and time
 * values. These are the obtained using getDate and setDate methods. Creation
 * date: (2/7/02 12:04:48 PM)
 *
 * @author <a href="mailto:jernej.kamenik@cosylab.com">Jernej Kamenik</a>
 * @version $id$
 */
public class DateTimeSelector extends javax.swing.JPanel
{
	private SimpleDateSelector dateChooser;
	private SimpleTimeSelector timeChooser;
	private Calendar calendar = new GregorianCalendar();

	/**
	 * Constructs a new DateTimeSelector.
	 */
	public DateTimeSelector()
	{
		super(new GridBagLayout());
		createComponents();
	}

	/**
	 * Internal helper routine creates all the components. Creation date:
	 * (2/7/02 12:12:14 PM)
	 */
	protected void createComponents()
	{
		GridBagConstraints c = new GridBagConstraints();
		c.weightx = 1.0;
		c.weighty = 0.0;
		c.fill = GridBagConstraints.HORIZONTAL;
		c.gridx = 0;
		c.insets = new Insets(4, 4, 4, 4);

		c.gridy = 0;

		dateChooser = new SimpleDateSelector();
		add(dateChooser, c);

		timeChooser = new SimpleTimeSelector();
		c.gridy = 1;
		add(timeChooser, c);
	}

	/**
	 * Returns the date displayed by this component. The result is
	 * java.util.Date object. Only the following fields are set: SECOND,
	 * MINUTE, HOUR_OF_DAY, DAY_OF_MONTH, MONTH, YEAR. For more information on
	 * these fields see the java.util.Calendar. Creation date: (2/7/02
	 * 12:19:59 PM)
	 *
	 * @return java.util.Date
	 */
	public Date getDate()
	{
		calendar.set(Calendar.SECOND, timeChooser.getSeconds());
		calendar.set(Calendar.MINUTE, timeChooser.getMinute());
		calendar.set(Calendar.HOUR_OF_DAY, timeChooser.getHour());
		calendar.set(Calendar.YEAR, dateChooser.getYear());
		calendar.set(Calendar.MONTH, dateChooser.getMonth());
		calendar.set(Calendar.DAY_OF_MONTH, dateChooser.getDay());

		return calendar.getTime();
	}

	/**
	 * Run test applet.
	 *
	 * @param args command line arguments
	 */
	public static void main(String[] args)
	{
		JDialog d = new JDialog();
		DateTimeSelector dtc = new DateTimeSelector();

		GridBagConstraints c = new GridBagConstraints();

		c.fill = GridBagConstraints.BOTH;

		d.getContentPane().setLayout(new GridBagLayout());
		d.getContentPane().add(dtc, c);

		d.setModal(true);
		d.setVisible(true);
		System.out.println(dtc.getDate().toString());
	}

	/**
	 * Sets the date to display in this selector. Value is set through the
	 * java.util.Date object. Only the following fields are considerer:
	 * SECOND, MINUTE, HOUR_OF_DAY, DAY_OF_MONTH, MONTH, YEAR. For more
	 * information on these fields see the java.util.Calendar class. Creation
	 * date: (2/7/02 12:19:27 PM)
	 *
	 * @param date java.util.Date
	 */
	public void setDate(Date date)
	{
		dateChooser.setDate(date);
		timeChooser.setTime(date);
	}

	/**
	 * Sets the enabled state of this component and the components contained in
	 * it. Creation date: (2/7/02 12:14:21 PM)
	 *
	 * @param how boolean
	 */
	public void setEnabled(boolean how)
	{
		super.setEnabled(how);
		dateChooser.setEnabled(how);
		timeChooser.setEnabled(how);
	}
}

/* __oOo__ */
