package com.cosylab.gui.components;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.Customizer;
import javax.swing.ButtonGroup;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.event.CaretEvent;
import javax.swing.event.CaretListener;

import com.cosylab.gui.components.customizer.AbstractCustomizerPanel;

public class DialKnobCustomizer extends AbstractCustomizerPanel {
	private static final long serialVersionUID = 1L;
	
	/** DOCUMENT ME! */
	public static final String VISUAL_BASIC = "Visual";

	/** DOCUMENT ME! */
	public static final String VALUE_DISPLAY = "Value Display";
	
	public static final String AUTO_SYNCHRONIZATION = "Auto synchronization";

	/** DOCUMENT ME! */
	public static final String VALUE_POLICY = "Value Display/Value Range Policy";

	/** DOCUMENT ME! */
	public static final String VALUE_DISPLAY_ADVANCED = "Value Display/Advanced";

	/** DOCUMENT ME! */
	public static String[] ASPECTS = {
		VISUAL_BASIC, VALUE_DISPLAY, VALUE_DISPLAY_ADVANCED, VALUE_POLICY
	};

	/** DOCUMENT ME! */
	public static final String TITLE_VISIBLE = "titleVisible";

	/** DOCUMENT ME! */
	public static final String TITLE = "title";

	/** DOCUMENT ME! */
	public static final String UNITS = "units";

	/** DOCUMENT ME! */
	public static final String UNITS_VISIBLE = "unitsVisible";

	/** DOCUMENT ME! */
	public static final String FORMAT = "format";

	/** DOCUMENT ME! */
	public static final String GRAPH_MAX = "graphMax";

	/** DOCUMENT ME! */
	public static final String GRAPH_MIN = "graphMin";

	/** DOCUMENT ME! */
	public static final String MAXIMUM = "maximum";

	/** DOCUMENT ME! */
	public static final String MINIMUM = "minimum";
	
	/** DOCUMENT ME! */
	public static final String EDITABLE = "editable";

	/** DOCUMENT ME! */
	public static final String RESIZABLE = "resizable";

	/** DOCUMENT ME! */
	public static final String ENHANCED = "enhanced";

	/** DOCUMENT ME! */
	public static final String TILTING_ENABLED = "tiltingEnabled";

	/** DOCUMENT ME! */
	public static String[] VALUE_DISPLAY_PROPERTIES = {
		MINIMUM, MAXIMUM, UNITS, UNITS_VISIBLE, FORMAT
	};

	/** DOCUMENT ME! */
	public static String[] VISUAL_BASIC_PROPERTIES = {
		TITLE, TITLE_VISIBLE, RESIZABLE, ENHANCED, TILTING_ENABLED
	};

	/** DOCUMENT ME! */
	public static String[] ADVANCED_PROPERTIES = { EDITABLE };	
	
	/**
	 * 
	 * <code>SyncPanel</code> enables configuring auto synchronization of the DialKnob.
	 *
	 * @author <a href="mailto:jaka.bobnar@cosylab.com">Jaka Bobnar</a>
	 * @version $Id: DialKnobCustomizer.java,v 1.13 2007-07-26 12:42:30 jbobnar Exp $
	 * 
	 * @since VERSION
	 */
	private class SyncPanel extends JPanel implements Customizer {
		private static final long serialVersionUID = 1L;
		private DialKnob displayer;
		private boolean initialized = false;
		private JPanel displayPanel;
		private NumberField autoSyncDelay;
		private JLabel millis;
		private JLabel syncDelay;
		private JRadioButton syncManual;
		private JRadioButton syncAuto;
		
		public SyncPanel(){
			if (displayer != null){
				initialize();
			}
		}
		
		private void initialize()
		{
			if (initialized) {
				return;
			}

			setLayout(new BorderLayout());

			add(getDisplayPanel(), BorderLayout.CENTER);
			updateEditor();
			initialized = true;

			setSize(300, 450);
		}
		
		private void updateEditor() {
			syncAuto.setSelected(displayer.isAutoSynchronize());
			syncManual.setSelected(!displayer.isAutoSynchronize());
			autoSyncDelay.setLongValue(displayer.getAutoSynchronizeDelay());
			autoSyncDelay.setEnabled(displayer.isAutoSynchronize());
		}
		
		private JPanel getDisplayPanel() {
			if (displayPanel == null) {
				displayPanel = new JPanel();
			}
			displayPanel.setBorder(new GroupBoxBorder("Synchronization configuration :"));
			displayPanel.setLayout(new GridBagLayout());
						
			syncManual = new JRadioButton("Manual synchronization");
			syncManual.addActionListener(new ActionListener() {

				public void actionPerformed(ActionEvent e) {
					autoSyncDelay.setEnabled(false);
					applySettings();
				}
				
				
			});
			syncAuto = new JRadioButton("Automatic synchronization");
			syncAuto.addActionListener(new ActionListener() {

				public void actionPerformed(ActionEvent e) {
					autoSyncDelay.setEnabled(true);	
					applySettings();
				}
				
				
			});
			ButtonGroup syncModeButtonGroup = new ButtonGroup();
			syncModeButtonGroup.add(syncManual);
			syncModeButtonGroup.add(syncAuto);
						
			displayPanel.add(syncManual, new GridBagConstraints(0,0,1,1,1,1,
					GridBagConstraints.WEST, GridBagConstraints.NONE,
					new Insets(0,0,0,0), 1,1));
			displayPanel.add(syncAuto, new GridBagConstraints(0,1,1,1,1,1,
					GridBagConstraints.WEST, GridBagConstraints.NONE,
					new Insets(0,0,0,0), 1,1));
			
			syncDelay = new JLabel("Automatic synchronization delay:");
			displayPanel.add(syncDelay, new GridBagConstraints(0,2,1,1,1,1,
					GridBagConstraints.WEST, GridBagConstraints.NONE,
					new Insets(0,21,0,0), 1,1));
			
			JPanel fieldPanel = new JPanel();
			fieldPanel.setLayout(new GridBagLayout());
			autoSyncDelay = new NumberField();
			autoSyncDelay.setNumberType(long.class);
			autoSyncDelay.addCaretListener(new CaretListener(){

				public void caretUpdate(CaretEvent e) {
					applySettings();
				}
				
			});
			autoSyncDelay.setPreferredSize(new Dimension(70, 20));
			fieldPanel.add(autoSyncDelay, new GridBagConstraints(0,0,1,1,1,1,
					GridBagConstraints.WEST, GridBagConstraints.NONE,
					new Insets(0,0,0,0), 1,1));
			
			millis = new JLabel("milliseconds");
			fieldPanel.add(millis, new GridBagConstraints(1,0,1,1,1,1,
					GridBagConstraints.WEST, GridBagConstraints.NONE,
					new Insets(0,10,0,0), 1,1));
			
			displayPanel.add(fieldPanel, new GridBagConstraints(0,3,1,1,1,1,
					GridBagConstraints.WEST, GridBagConstraints.NONE,
					new Insets(5,21,0,0), 1,1));
			return displayPanel;
		}
		
		/*
		 *  (non-Javadoc)
		 * @see com.cosylab.gui.components.customizer.Editor#getEditorComponent(java.lang.Object, java.lang.String)
		 */
		public JComponent getEditorComponent(Object displayer, String aspect) throws IllegalArgumentException {
			if (!(displayer instanceof DialKnob)) {
				throw new IllegalArgumentException("Component '" + displayer
				    + "' is not instance of DialKnob.");
			}

			displayer = (DialKnob)displayer;

			initialize();

			return this;
		}

		/*
		 *  (non-Javadoc)
		 * @see com.cosylab.gui.components.customizer.Editor#applySettings()
		 */
		public void applySettings() {
			boolean auto = syncAuto.isSelected();
			displayer.setAutoSynchronize(auto);
			if (syncAuto.isSelected()) {
				displayer.setAutoSynchronizeDelay(autoSyncDelay.getLongValue());
				firePropertyChange("autoSynchronizeDelay", -1, autoSyncDelay.getLongValue());
			}
			firePropertyChange("autoSynchronize", !auto, auto);
			
		}

		/*
		 *  (non-Javadoc)
		 * @see com.cosylab.gui.components.customizer.Editor#canEdit(java.lang.Object, java.lang.String)
		 */
		public boolean canEdit(Object displayer, String aspect) {
			return displayer instanceof DialKnob;
		}

		/*
		 *  (non-Javadoc)
		 * @see com.cosylab.gui.components.customizer.Editor#revertSettings()
		 */
		public void revertSettings() {
			
		}

		/*
		 *  (non-Javadoc)
		 * @see com.cosylab.gui.components.customizer.Editor#stopEditing()
		 */
		public void stopEditing() {
			displayer = null;
		}

		public void setObject(Object arg0) {
			displayer = (DialKnob) arg0;
			initialize();
		}
		
	}
	
	public DialKnobCustomizer() {
		addCustomizerTable(VISUAL_BASIC, VISUAL_BASIC_PROPERTIES);
		addCustomizerTable(VALUE_DISPLAY, VALUE_DISPLAY_PROPERTIES);
		addCustomizerTable(VALUE_DISPLAY_ADVANCED, ADVANCED_PROPERTIES);
		addCustomizer(VALUE_POLICY, new ValuePolicyCustomizer());
		addCustomizer(AUTO_SYNCHRONIZATION, new SyncPanel());
		setSize(423, 184);
	}

}
