/*
 * Copyright (c) 2003 by Cosylab d.o.o.
 *
 * The full license specifying the redistribution, modification, usage and other
 * rights and obligations is included with the distribution of this project in
 * the file license.html. If the license is not included you may find a copy at
 * http://www.cosylab.com/legal/abeans_license.htm or may write to Cosylab, d.o.o.
 *
 * THIS SOFTWARE IS PROVIDED AS-IS WITHOUT WARRANTY OF ANY KIND, NOT EVEN THE
 * IMPLIED WARRANTY OF MERCHANTABILITY. THE AUTHOR OF THIS SOFTWARE, ASSUMES
 * _NO_ RESPONSIBILITY FOR ANY CONSEQUENCE RESULTING FROM THE USE, MODIFICATION,
 * OR REDISTRIBUTION OF THIS SOFTWARE.
 */

package com.cosylab.gui.components;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;

import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.TransferHandler;

import com.cosylab.application.state.State;
import com.cosylab.gui.components.customizer.AbstractCustomizerPanel;
import com.cosylab.gui.components.util.CosyTransferHandler;
import com.cosylab.gui.components.util.PopupManager;


/**
 * Gauger displayer.
 *
 * @version 1.0
 */
public class LabelledGauger extends AbstractDisplayerPanel
{
	private static final long serialVersionUID = 1L;
	private Gauger gauger = null;
	private AbstractCustomizerPanel customizer;
	private boolean popupEnabled;
	/**
	 * Creates new gauger displayer. It defaults to initial values as specified
	 * in the gauger component.
	 */
	public LabelledGauger(String title)
	{
		super();
		setTitle(title);
		setTitleVisible(true);
		setLayoutOrientation(VERTICAL_LAYOUT);
		setResizable(true);
		setEnhanced(true);
		setValue(5);
		setFormat("%1.1f");
		setUnits("");
	}

	/**
	 * Creates new gauger displayer. It defaults to initial values as specified
	 * in the gauger component.
	 */
	public LabelledGauger()
	{
		this(null);
	}

	/**
	 * Run test applet.
	 *
	 * @param args command line parameters
	 */
	public static void main(String[] args)
	{
		Thread t = new Thread(){
			public void run(){
				JFrame f= new JFrame();
				f.setSize(400,400);
				f.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
				
				LabelledGauger g= new LabelledGauger("Labelled Gauger");
//				g.setMaximum(100);
//				g.setMinimum(0);
//				g.setValue(50);
				g.setResizable(true);
				g.setUnits("V");
				g.setFormat("%1.2f");
				f.getContentPane().add(g);
				
				f.setVisible(true);
//				while (true){
//					g.setValue(g.getMinimum() + Math.random() * (g.getMaximum() - g.getMinimum()));
//					
//					try {
//						sleep(500);
//					} catch (InterruptedException e) {
//						break;
//					}
//				}
			}
		};
		t.run();
	}

	/**
	 * @see java.awt.Component#setBackground(Color)
	 */
	public void setBackground(Color c)
	{
//				gauger.getPreferences().setBackgroundColor(c);
//				gauger.setBackground(c);
		super.setBackground(c);
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.displayers.Displayer#isEditable()
	 */
	public boolean isEditable()
	{
		return false;
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.displayers.DoubleDisplayer#setFormat(java.lang.String)
	 */
	public void setFormat(String value)
	{
		String oldValue = getFormat();
		getGauger().setFormat(value);
		firePropertyChange("format", oldValue, value);
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.displayers.DoubleDisplayer#getFormat()
	 */
	public String getFormat()
	{
		return getGauger().getFormat();
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @param b DOCUMENT ME!
	 */
	public void setLogarithmicScale(boolean b)
	{
		if (b) {
			setLogarithmicScale();
		} else {
			setLinearScale();
		}
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @return
	 */
	public boolean isLogarithmicScale()
	{
		return getGauger().isLogarithmicScale();
	}

	/**
	 * @see DoubleDisplayer#setGraphMax(double)
	 */
	public void setMaximum(double value)
	{	
		double oldValue = getMaximum();
		getGauger().setMaximum(value);
		firePropertyChange("maximum", oldValue, value);
	}

	/**
	 * @see DoubleDisplayer#getGraphMax()
	 */
	public double getMaximum()
	{
		return getGauger().getMaximum();
	}

	/**
	 * @see DoubleDisplayer#setGraphMin(double)
	 */
	public void setMinimum(double value)
	{
		double oldValue = getMinimum();
		getGauger().setMinimum(value);
		firePropertyChange("minimum", oldValue, value);
	}

	/**
	 * @see DoubleDisplayer#getGraphMin()
	 */
	public double getMinimum()
	{
		return getGauger().getMinimum();
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @param policy
	 */
	public void setScaleMode(int policy)
	{
		int oldValue = getScaleMode();
		getGauger().setScaleMode(policy);
		firePropertyChange("scaleMode", oldValue, policy);
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @return
	 */
	public int getScaleMode()
	{
		return getGauger().getScaleMode();
	}

	/* (non-Javadoc)
	 * @see com.cosylab.application.state.StateOriginator#setState(com.cosylab.application.state.State)
	 */
	public void setState(State state)
	{
		super.setState(state);
		getGauger().setState(state.getState("gauger"));
	}

	/* (non-Javadoc)
	 * @see com.cosylab.application.state.StateOriginator#getState()
	 */
	public State getState()
	{
		State state = super.getState(); 
		state.putState("gauger",getGauger().getState());
		return state;
	}

	/**
	 * Sets the units visible property to the gauger.
	 * 
	 * @param b
	 */
	public void setUnitsVisible(boolean b) {
		boolean oldB = isUnitsVisible();
		
		if (oldB == b) return;
		
		getGauger().setUnitsVisible(b);
		
		firePropertyChange("unitsVisible",oldB,b);
	}
	
	/**
	 * Gets the units visible property of the gauger.
	 * 
	 * @return true if units are displayed.
	 */
	public boolean isUnitsVisible() {
		return getGauger().isUnitsVisible();
	}
	
	public Gauger getGauger() {
		if (gauger==null) {
			gauger = new Gauger(){
				private static final long serialVersionUID = 1L;
				
				public AbstractCustomizerPanel getCustomizer()
				{
					if (customizer == null) {
						customizer = AbstractCustomizerPanel.findCustomizer(LabelledGauger.this);
					}

					return customizer;
				}
			};
			Dimension size = new Dimension(250, 150);
			gauger.setPreferredSize(size);
			gauger.setSize(size);
			gauger.setMinimumSize(new Dimension(120,85));
			gauger.addMouseListener(getPopupManager().getMouseHook());

		}
		return gauger;
	}
	
	@Override
	protected JComponent getValueComponent() {
		return getGauger();
	}
		
	public double getValue() {
		return getGauger().getValue();
	}
	public void setValue(double d) {
		double oldValue = getValue();
		getGauger().setValue(d);
		firePropertyChange("value", oldValue, d);
	}
	
	@Override
	protected void layoutDisplayer() {
		super.layoutDisplayer();
		getGauger().doLayout();
	}
	
	public String getUnits() {
		return getGauger().getUnits();
	}
	
	public void setUnits(String u) {
		String oldValue = getUnits();
		getGauger().setUnits(u);
		firePropertyChange("units", oldValue, u);
	}
	
	public AbstractCustomizerPanel getCustomizer()
	{
		if (customizer == null) {
			customizer = AbstractCustomizerPanel.findCustomizer(this);
		}

		return customizer;
	}
	
	public boolean isPopupEnabled()
	{
		return popupEnabled;
	}
	
	/**
	 * Enables or disables the popup capabilities of the component.
	 *
	 * @param b the component should bring up popup dialog on mouse click.
	 */
	public void setPopupEnabled(boolean b)
	{
		if (popupEnabled == b) return;
		else {
			if (b) {
				addMouseListener(getPopupManager().getMouseHook());
			} else {
				removeMouseListener(getPopupManager().getMouseHook());
			}
		}

		popupEnabled = b;
		firePropertyChange("popupEnabled", !b, b);
	}
	
	public PopupManager getPopupManager()
	{
		return getGauger().getPopupManager();
	}

	public Font getValueLabelFont() {
	    return gauger.getValueLabelFont();
    }
	
	public void setValueLabelFont(Font font) {
		Font oldValue = getValueLabelFont();
		gauger.setValueLabelFont(font);
		firePropertyChange("valueLabelFont", oldValue, font);
	}
	
	public void setLogarithmicScale() {
		boolean oldValue = isLogarithmicScale();
		gauger.setLogarithmicScale();
		firePropertyChange("logarithmicScale", oldValue, true);
	}
	
	public void setLinearScale() {
		boolean oldValue = isLinearScale();
		gauger.setLinearScale();
		firePropertyChange("linearScale", oldValue, true);
	}
	
	public boolean isLinearScale() {
		return gauger.isLinearScale();
	}
	
	public void setValue(double newMin, double newMax, double newValue) {
		double oldValue = getValue();
		gauger.setValue(newMin, newMax, newValue);
		firePropertyChange("value", oldValue, newValue);
	}
	
    /* (non-Javadoc)
     * @see javax.swing.JComponent#setTransferHandler(javax.swing.TransferHandler)
     */
    @Override
    public void setTransferHandler(TransferHandler newHandler) {
    	super.setTransferHandler(newHandler);
    	CosyTransferHandler.registerTransferHandler(this,newHandler,new JComponent[]{getValueComponent()});
    }

}

/* __oOo__ */
