package com.cosylab.gui.components;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.Customizer;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import javax.swing.ButtonGroup;
import javax.swing.JCheckBox;
import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.JRadioButton;

public class LabelledGaugerCustomizer extends AbstractDisplayerPanelCustomizer {
	private static final long serialVersionUID = 1L;
	
	public static final String VALUE_DISPLAY = "Value Display";
	public static final String GRAPH_MIN = "minimum";
	public static final String GRAPH_MAX = "maximum";
	public static final String UNITS_VISIBLE = "unitsVisible";
	public static final String FORMAT = "format";
	public static final String UNITS = "units";
	public static final String VISUAL_SPECIFIC = "Value Display/Scale";
	public static final String[] VISUAL_BASIC_PROPERTIES = { TITLE, TITLE_VISIBLE, RESIZABLE };
	public static String[] VALUE_DISPLAY_PROPERTIES = {
		GRAPH_MIN, GRAPH_MAX, UNITS, UNITS_VISIBLE, FORMAT
	};
	
	private class WPanel extends JPanel implements Customizer
	{
		private static final long serialVersionUID = 1L;
		private ButtonGroup scaleGroup;
		private LabelledGauger disp;

		// GUI elements
		private JCheckBox logarithmic;
		private JRadioButton fixedButton;
		private JRadioButton slideButton;
		private JRadioButton stretchButton;
		private boolean initialized = false;
		private ActionListener al = null;
		public WPanel() {
			al = new ActionListener(){

				public void actionPerformed(ActionEvent e) {
					applySettings();
				}
				
			};
			initialize();
		}

		/**
		 * Starts new editing session withthis editor.
		 *
		 * @see com.cosylab.gui.components.customizer.Editor#getEditor(java.lang.Object,
		 *      java.lang.String)
		 */
		public JComponent getEditorComponent(Object displayer, String aspect)
			throws IllegalArgumentException
		{
			if (!(displayer instanceof LabelledGauger)) {
				throw new IllegalArgumentException("Component '" + displayer
				    + "' is not instance of GaugerDisplayer.");
			}

			disp = (LabelledGauger)displayer;

			initialize();
			update();

			return this;
		}

		/* (non-Javadoc)
		 * @see com.cosylab.gui.components.customizer.Editor#applySetting()
		 */
		public void applySettings()
		{
			applyDefaultProperties();
			update();
		}

		/* (non-Javadoc)
		 * @see com.cosylab.gui.components.customizer.Editor#canEdit(java.lang.Object, java.lang.String)
		 */
		public boolean canEdit(Object displayer, String aspect)
		{
			return displayer instanceof LabelledGauger;
		}

		/* (non-Javadoc)
		 * @see com.cosylab.gui.components.customizer.Editor#revertSettings()
		 */
		public void revertSettings()
		{
			update();
		}

		/* (non-Javadoc)
		 * @see com.cosylab.gui.components.customizer.Editor#stopEditing()
		 */
		public void stopEditing()
		{
			disp = null;
			update();
		}

		private JRadioButton getFixedButton()
		{
			if (fixedButton == null) {
				fixedButton = new JRadioButton("Fixed");
				getScaleGroup().add(fixedButton);
				fixedButton.addActionListener(al);
			}

			return fixedButton;
		}

		private JCheckBox getLogarithmic()
		{
			if (logarithmic == null) {
				logarithmic = new JCheckBox("Logarithmic");
				logarithmic.setPreferredSize(new Dimension(150, 21));
				logarithmic.setMinimumSize(new Dimension(75, 21));
				logarithmic.setToolTipText("Use logarithmic scale.");
				logarithmic.addActionListener(al);
			}

			return logarithmic;
		}

		private ButtonGroup getScaleGroup()
		{
			if (scaleGroup == null) {
				scaleGroup = new ButtonGroup();
			}

			return scaleGroup;
		}

		private JRadioButton getSlideButton()
		{
			if (slideButton == null) {
				slideButton = new JRadioButton("Slide");
				getScaleGroup().add(slideButton);
				slideButton.addActionListener(al);
			}

			return slideButton;
		}

		private JRadioButton getStretchButton()
		{
			if (stretchButton == null) {
				stretchButton = new JRadioButton("Stretch");
				getScaleGroup().add(stretchButton);
				stretchButton.addActionListener(al);
			}

			return stretchButton;
		}

		/*
		 * Sets editor values to displayer.
		 */
		private void applyDefaultProperties()
		{
			if (disp == null) {
				return;
			}
			boolean log = getLogarithmic().isSelected();
			disp.setLogarithmicScale(log);

			int policy = Gauger.FIXED_SCALE;

			if (getFixedButton().isSelected()) {
				policy = Gauger.FIXED_SCALE;
			}

			if (getSlideButton().isSelected()) {
				policy = Gauger.CONSTANT_SPAN;
			}

			if (getStretchButton().isSelected()) {
				policy = Gauger.STRETCH_SCALE;
			}

			disp.setScaleMode(policy);
			LabelledGaugerCustomizer.this.firePropertyChange("logarithmicScale", !log, log);
			LabelledGaugerCustomizer.this.firePropertyChange("scaleMode", null, policy);
		}

		private void initialize()
		{
			if (initialized) {
				return;
			}

			initialized = true;

			setLayout(new GridBagLayout());
			setSize(300, 450);
			setBorder(new GroupBoxBorder("Scale preferences"));

			Insets insets = new Insets(2, 2, 2, 2);

			add(getLogarithmic(),
			    new GridBagConstraints(0, 1, 3, 1, 1.0, 1.0,
			        GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL,
			        insets, 0, 0));

			add(getFixedButton(),
			    new GridBagConstraints(0, 0, 1, 1, 1.0, 1.0,
			        GridBagConstraints.WEST, GridBagConstraints.NONE, insets,
			        0, 0));

			add(getStretchButton(),
			    new GridBagConstraints(1, 0, 1, 1, 1.0, 1.0,
			        GridBagConstraints.WEST, GridBagConstraints.NONE, insets,
			        0, 0));

			add(getSlideButton(),
			    new GridBagConstraints(2, 0, 1, 1, 1.0, 1.0,
			        GridBagConstraints.WEST, GridBagConstraints.NONE, insets,
			        0, 0));
		}

		/*
		 * Updates editor with values from diplayer.
		 */
		private void update()
		{
			if (disp != null) {
				getLogarithmic().setSelected(disp.isLogarithmicScale());

				switch (disp.getScaleMode()) {
				case Gauger.FIXED_SCALE:
					getFixedButton().setSelected(true);

					break;

				case Gauger.CONSTANT_SPAN:
					getSlideButton().setSelected(true);

					break;

				default:
				case Gauger.STRETCH_SCALE:
					getStretchButton().setSelected(true);

					break;
				}
			}
		}

		public void setObject(Object bean) {
			disp = (LabelledGauger) bean;
			disp.addPropertyChangeListener("scaleMode", new PropertyChangeListener(){
				public void propertyChange(PropertyChangeEvent evt) {
					update();
				}
			});
			disp.addPropertyChangeListener("logarithmicScale", new PropertyChangeListener(){
				public void propertyChange(PropertyChangeEvent evt) {
					update();
				}
			});
			update();
			
		}
	}
	
	
	public LabelledGaugerCustomizer() {
		addCustomizer(VISUAL_SPECIFIC, new WPanel());
		addCustomizerTable(VALUE_DISPLAY, VALUE_DISPLAY_PROPERTIES);
		addCustomizerTable(VISUAL_BASIC, VISUAL_BASIC_PROPERTIES);
		setSize(357, 352);
	}
}
