package com.cosylab.gui.components;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.Customizer;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.event.CaretEvent;
import javax.swing.event.CaretListener;

import com.cosylab.gui.components.customizer.Editor;

public class LabelledNumberFieldCustomizer extends
		AbstractNumericDisplayerPanelCustomizer {
	private static final long serialVersionUID = 1L;

	/** Aspect for bounds. */
	public static final String VALUE_DISPLAY_BOUNDS = "Value Display/Bounds";
	
	
	public static class WPanel extends JPanel implements Customizer
	{
		private static final long serialVersionUID = 1L;
		private LabelledNumberField displayer = null;
		NumberField min;
		NumberField max;

		/**
		 * Creates a new WPanel object.
		 */
		public WPanel()
		{
			super();

			setLayout(new GridBagLayout());

			min = new NumberField();
			min.addCaretListener(new CaretListener(){
				public void caretUpdate(CaretEvent arg0) {
					applySettings();
				}
				
			});
			min.setNumberType(Double.class);
			min.setPreferredSize(new Dimension(100, 21));
			min.setToolTipText("Minimal allowed value");
			if (displayer != null) min.setValue(displayer.getMinimumValue());
			add(new JLabel("Minimum"),
			    new GridBagConstraints(0, 2, 1, 1, 0.0, 0.0,
			        GridBagConstraints.WEST, GridBagConstraints.NONE,
			        new Insets(4, 11, 4, 4), 0, 0));
			add(min,
			    new GridBagConstraints(1, 2, 1, 1, 1.0, 1.0,
			        GridBagConstraints.WEST, GridBagConstraints.NONE,
			        new Insets(4, 4, 4, 4), 0, 0));

			max = new NumberField();
			max.addCaretListener(new CaretListener(){
				public void caretUpdate(CaretEvent arg0) {
					applySettings();
				}
				
			});
			max.setNumberType(Double.class);
			max.setPreferredSize(new Dimension(100, 21));
			max.setToolTipText("Maximal allowed value");
			if (displayer != null) max.setValue(displayer.getMaximumValue());
			ActionListener l = new ActionListener(){

				public void actionPerformed(ActionEvent arg0) {
					applySettings();
				}
			};
			max.addActionListener(l);
			min.addActionListener(l);
			
			add(new JLabel("Maximum"),
			    new GridBagConstraints(0, 3, 1, 1, 0.0, 0.0,
			        GridBagConstraints.WEST, GridBagConstraints.NONE,
			        new Insets(4, 11, 4, 4), 0, 0));
			add(max,
			    new GridBagConstraints(1, 3, 1, 1, 1.0, 1.0,
			        GridBagConstraints.WEST, GridBagConstraints.NONE,
			        new Insets(4, 4, 4, 4), 0, 0));
		}

		public void setObject(Object arg0) {
			initialize((LabelledNumberField)arg0);
		}

		private void initialize(LabelledNumberField newDisplayer)
		{
			if (newDisplayer == null) {
				return;
			}
			
			displayer = newDisplayer;
			displayer.addPropertyChangeListener("maximumValue", new PropertyChangeListener(){
				public void propertyChange(PropertyChangeEvent evt) {
					max.setValue(displayer.getMaximumValue());
				}
			});
			displayer.addPropertyChangeListener("minimumValue", new PropertyChangeListener(){
				public void propertyChange(PropertyChangeEvent evt) {
					min.setValue(displayer.getMinimumValue());
				}
			});
			max.setNumberType(displayer.getNumberType());
			max.setValue(displayer.getMaximumValue());
			min.setNumberType(displayer.getNumberType());
			min.setValue(displayer.getMinimumValue());
		}

		/**
		 * @see Editor#applySettings()
		 */
		public void applySettings()
		{
			if (displayer == null) {
				return;
			}

			displayer.setMaximumValue(max.getValue());
			displayer.setMinimumValue(min.getValue());
			initialize(displayer);
			firePropertyChange("maximumValue", null, max.getValue());
			firePropertyChange("minimumValue", null, min.getValue());
		}

	}

	public LabelledNumberFieldCustomizer() {
		addCustomizer(VALUE_DISPLAY_BOUNDS, new WPanel());
		setSize(445, 185);
	}
}
