/*
 * Copyright (c) 2003 by Cosylab d.o.o.
 *
 * The full license specifying the redistribution, modification, usage and other
 * rights and obligations is included with the distribution of this project in
 * the file license.html. If the license is not included you may find a copy at
 * http://www.cosylab.com/legal/abeans_license.htm or may write to Cosylab, d.o.o.
 *
 * THIS SOFTWARE IS PROVIDED AS-IS WITHOUT WARRANTY OF ANY KIND, NOT EVEN THE
 * IMPLIED WARRANTY OF MERCHANTABILITY. THE AUTHOR OF THIS SOFTWARE, ASSUMES
 * _NO_ RESPONSIBILITY FOR ANY CONSEQUENCE RESULTING FROM THE USE, MODIFICATION,
 * OR REDISTRIBUTION OF THIS SOFTWARE.
 */

package com.cosylab.gui.components;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.event.ActionEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.swing.AbstractAction;
import javax.swing.JComponent;
import javax.swing.JFrame;

import com.cosylab.application.state.State;
import com.cosylab.events.SetListener;
import com.cosylab.gui.components.customizer.AbstractCustomizerPanel;
import com.cosylab.gui.components.util.PopupManageable;
import com.cosylab.gui.components.util.PopupManager;
import com.cosylab.gui.components.util.ScreenCapturer;
import com.cosylab.gui.components.wheelswitch.AbstractWheelswitchFormatter;
import com.cosylab.gui.components.wheelswitch.WheelswitchFormatter;


/**
 * <code>LabelledWheelswitch</code> is extended <code>Wheelswitch</code> with
 * title label and unit label in same style.
 *
 * @author <a href="mailto:igor.kriznar@cosylab.com">Igor Kriznar</a>
 * @author <a href="mailto:jernej.kamenik@cosylab.com">Jernej Kamenik</a>
 * @version $id$
 */
public class LabelledWheelswitch extends AbstractNumericDisplayerPanel implements PopupManageable
{
	private static final long serialVersionUID = 1L;
	/** Value property name. */
	public static final String VALUE = Wheelswitch.VALUE;
	private Wheelswitch number;
	private PopupManager popupManager;
	private AbstractCustomizerPanel customizer;
	/**
	 * Creates a new LabelledWheelswitch object.
	 */
	public LabelledWheelswitch()
	{
		this(0.0, "3.2f", null, null);
	}

	/**
	 * Creates a new LabelledWheelswitch object.
	 *
	 * @param value initial double value
	 */
	public LabelledWheelswitch(double value)
	{
		this(value, "3.2f" , null, null);
	}

	/**
	 * Creates a new LabelledWheelswitch object.
	 *
	 * @param format initial number (C-style) format
	 */
	public LabelledWheelswitch(String format)
	{
		this(0.0, format, null, null);
	}

	/**
	 * Creates a new LabelledWheelswitch object.
	 *
	 * @param value initial double value
	 * @param format initial number (C-style) format
	 * @param title initial title
	 * @param units initial units
	 */
	public LabelledWheelswitch(double value, String format, String title,
	    String units)
	{
		super();

		setValue(value);
		setFormat(format);
		setUnits(units);
		setTitle(title);

		setUnitsVisible(true);
		setTitleVisible(true);
		setLayoutOrientation(HORIZONTAL_LAYOUT);
		setResizable(true);
		setEnhanced(true);
		addMouseListener(getPopupManager().getMouseHook());
		
		
	}
		public PopupManager getPopupManager()
		{
			if (popupManager == null) {
				popupManager = new PopupManager(this, true);
				popupManager.addAction(new AbstractAction("Preferences..."){
					private static final long serialVersionUID = 1L;
					public void actionPerformed(ActionEvent e){
						getCustomizer().showDialog();
					}
				});
						popupManager.addAction(new AbstractAction("Capture screen..."){
							private static final long serialVersionUID = 1L;

							public void actionPerformed(ActionEvent e){
							   	ScreenCapturer sc = new ScreenCapturer(LabelledWheelswitch.this);
								sc.showScreenDialog();
							   }
						});
					}

					return popupManager;
		}
		
		public AbstractCustomizerPanel getCustomizer()
		{
			if (customizer == null) {
				customizer = AbstractCustomizerPanel.findCustomizer(this);
			}

			return customizer;
		}
			
	
		


	/**
	 * Runs simple test applet.
	 *
	 * @param args String[]
	 */
	public static void main(String[] args)
	{
		LabelledWheelswitch ws = new LabelledWheelswitch(2.449, "%2.1f",
			    "Current", "A");

		JFrame frame = new JFrame("Wheelwsitch Testing Applet");
		frame.getContentPane().setLayout(new BorderLayout(10, 10));
		frame.getContentPane().add(ws);
		frame.setSize(300, 100);
		frame.addWindowListener(new WindowAdapter() {
				public void windowClosing(WindowEvent e)
				{
					System.exit(0);
				}
			});
		frame.setVisible(true);
	}

	/**
	 * Sets the editability of the wheelswitch.
	 *
	 * @param newEditable
	 */
	public void setEditable(boolean newEditable)
	{
		if (newEditable == isEditable()) return;
		getNumber().setEditable(newEditable);
		//has to fire event EVERY time!
		firePropertyChange("editable",!newEditable,newEditable);
	}

	/**
	 * Returns whether the wheelswitch can be edited  by the user.
	 *
	 * @return boolean
	 */
	public boolean isEditable()
	{
		return getNumber().isEditable();
	}

	/**
	 * Sets the maximum allowed value.
	 *
	 * @param newValue
	 *
	 * @see WheelswitchFormatter#setMaximum(double)
	 */
	public void setMaximum(double newValue)
	{
		double oldValue = getMaximum();
		getNumber().setGraphMax(newValue);
		super.setMaximumValue(new Double(getNumber().getGraphMax()));
		firePropertyChange("maximum", oldValue, newValue);
	}

	/**
	 * Gets the maximum alowed value.
	 *
	 * @return double
	 *
	 * @see WheelswitchFormatter#getMaximum()
	 */
	public double getMaximum()
	{
		return getNumber().getGraphMax();
	}

	/**
	 * Sets the minimum allowed value.
	 *
	 * @param newValue
	 *
	 * @see WheelswitchFormatter#setMinimum(double)
	 */
	public void setMinimum(double newValue)
	{
		double oldValue = getMinimum();
		getNumber().setGraphMin(newValue);
		super.setMinimumValue(new Double(getNumber().getGraphMin()));
		firePropertyChange("minimum", oldValue, newValue);
	}

	/**
	 * Gets the minimum alowed value.
	 *
	 * @return double
	 *
	 * @see WheelswitchFormatter#getMaximum()
	 */
	public double getMinimum()
	{
		return getNumber().getGraphMin();
	}

	/**
	 * Sets the maximum and minimum allowed values.
	 *
	 * @param max
	 * @param min
	 *
	 * @see com.cosylab.gui.components.Wheelswitch#setMaxMin(double, double)
	 */
	public void setMaxMin(double max, double min)
	{
		setMaximum(max);
		setMinimum(min);
	}

	/**
	 * Overriden to implement additional features loading.
	 *
	 * @param state to be loaded.
	 */
	public void setState(State state)
	{
		super.setState(state);

		setTiltingEnabled(state.getBoolean("tiltingEnabled", isTiltingEnabled()));
		setEditable(state.getBoolean("editable", isEditable()));
		setMaximum(state.getDouble("maximum", getMaximum()));
		setMinimum(state.getDouble("minimum", getMinimum()));
	}

	/**
	 * Overriden to implement additional features saving.
	 *
	 * @return state of the component.
	 */
	public State getState()
	{
		State state = super.getState();

		state.putDouble("maximum", getMaximum());
		state.putDouble("minimum", getMinimum());
		state.putBoolean("editable", isEditable());
		state.putBoolean("tiltingEnabled", isTiltingEnabled());

		return state;
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @param b DOCUMENT ME!
	 */
	public void setTiltingEnabled(boolean b)
	{	
		if (isTiltingEnabled() == b) return; 
		getNumber().setTiltingEnabled(b);
		firePropertyChange("tiltingEnabled", !b, b);
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @return DOCUMENT ME!
	 */
	public boolean isTiltingEnabled()
	{
		return getNumber().isTiltingEnabled();
	}

	/**
	 * Sets the value and displays it in the wheelswitch. The method may  also
	 * change the current digit selection if neccessary in order to point to
	 * the same decimal digit of the displayed value.
	 *
	 * @param newValue
	 *
	 * @see WheelswitchFormatter#setValue(double)
	 */
	public void setValue(double newValue)
	{	
		double oldValue = getValue();
		getNumber().setValue(newValue);
		firePropertyChange("value", oldValue, newValue);
	}
	
	/**
	 * Sets the animated property.
	 * 
	 * @param animated
	 */
	public void setAnimated(boolean animated) {
		if (isAnimated() == animated) return;
		getNumber().setAnimated(animated);
		firePropertyChange("animated", !animated, animated);
	}
	
	/**
	 * Returns weather the labelled wheelswitch is animated.
	 * 
	 * @return animated property
	 */
	public boolean isAnimated() {
		return getNumber().isAnimated();
	}

	/**
	 * Gets the value displayed by the <code>Wheelswitch</code> and stored by
	 * the <code>formatter</code>.
	 *
	 * @return double
	 *
	 * @see WheelswitchFormatter#getValue()
	 */
	public double getValue()
	{
		return getNumber().getValue();
	}

	/**
	 * Adds set listener to the <code>NumberField</code>.
	 *
	 * @see NumberField#addSetListener(SetListener)
	 */
	public void addSetListener(SetListener l)
	{
		getNumber().addSetListener(l);
	}

	/**
	 * Removes set listener from the <code>NumberField</code>.
	 *
	 * @see NumberField#removeSetListener(SetListener)
	 */
	public void removeSetListener(SetListener l)
	{
		getNumber().removeSetListener(l);
	}

	protected Wheelswitch getNumber()
	{
		if (number == null) {
			number = new Wheelswitch();
			
			//			number.addPropertyChangeListener(new PropertyChangeListener() {
			//					/**
			//					 * @see java.beans.PropertyChangeListener#propertyChange(PropertyChangeEvent)
			//					 */
			//					public void propertyChange(PropertyChangeEvent evt)
			//					{
			//						firePropertyChange(evt.getPropertyName(),
			//						    evt.getOldValue(), evt.getNewValue());
			//					}
			//				});
			number.addMouseListener(getPopupManager().getMouseHook());
			number.setUnitSeparate(true);
		}

		return number;
	}

	/**
	 * Returns value displaying GUI component: the <code>Wheelswitch</code>.
	 *
	 * @see AbstractDisplayerPanel#getValueComponent()
	 */
	protected JComponent getValueComponent()
	{
		return getNumber();
	}

	/**
	 * Overriden to implement additional functionality.
	 *
	 * @see com.cosylab.gui.components.AbstractDisplayerPanel#setEnabled(boolean)
	 */
	protected void internalSetEnabled()
	{
		super.internalSetEnabled();
		getNumber().setEnabled(isEnabled());
	}

	/**
	 * Overriden to implement additional functionality.
	 *
	 * @see com.cosylab.gui.components.AbstractDisplayerPanel#setEnhanced(boolean)
	 */
	protected void internalSetEnhanced()
	{
		super.internalSetEnhanced();
		getNumber().setEnhanced(isEnhanced());
	}

	/**
	 * Sets the C-style format of the value display.
	 *
	 * @see WheelswitchFormatter#setFormat(String)
	 */
	protected void internalSetFormat()
	{
		super.internalSetFormat();

		String tFormat = getFormat();

		if (tFormat == null) {
			tFormat = NumberField.createDefaultFormat(Double.class);
		}

		getNumber().setFormat(WheelswitchFormatter.transformFormat(tFormat));
	}

	/**
	 * Overriden to implement additional functionality.
	 *
	 * @see com.cosylab.gui.components.AbstractDisplayerPanel#setResizable(boolean)
	 */
	protected void internalSetResizable()
	{
		super.internalSetResizable();
		getNumber().revalidate();
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.components.AbstractNumericDisplayerPanel#setUnitsVisible(boolean)
	 */
	public void setUnitsVisible(boolean b) {
		if (b) getNumber().setUnit(getUnits()); 
		else getNumber().setUnit(null);
		super.setUnitsVisible(b);
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.components.AbstractNumericDisplayerPanel#setUnits(java.lang.String)
	 */
	public void setUnits(String value) {
		if (isUnitsVisible()) {
			getNumber().setUnit(value);
		}
		super.setUnits(value);
	}
	/* (non-Javadoc)
	 * @see com.cosylab.gui.components.AbstractNumericDisplayerPanel#layoutValueAndTitleAndUnits()
	 */
	protected void layoutValueAndTitleAndUnits() {
		super.layoutValueAndTitle();
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.components.AbstractNumericDisplayerPanel#layoutValueAndTitleAndUnitsAndBounds()
	 */
	protected void layoutValueAndTitleAndUnitsAndBounds() {
		super.layoutValueAndTitleAndBounds();
	}
	
	/* (non-Javadoc)
	 * @see com.cosylab.gui.components.AbstractNumericDisplayerPanel#layoutValueAndUnits()
	 */
	protected void layoutValueAndUnits() {
		super.layoutValue();
	}
	
	/* (non-Javadoc)
	 * @see com.cosylab.gui.components.AbstractNumericDisplayerPanel#layoutValueAndUnitsAndBounds()
	 */
	protected void layoutValueAndUnitsAndBounds() {
		super.layoutValueAndBounds();
	}
	
	/* (non-Javadoc)
	 * @see javax.swing.JComponent#setBackground(java.awt.Color)
	 */
	public void setBackground(Color bg) {
		getNumber().setBackground(bg);
		super.setBackground(bg);
	}
	/**
	 * @see Wheelswitch#setFormatter(AbstractWheelswitchFormatter)
	 */
	public void setFormatter(AbstractWheelswitchFormatter formatter) {
		AbstractWheelswitchFormatter oldValue = getFormatter();
		getNumber().setFormatter(formatter);
		firePropertyChange("formatter", oldValue, formatter);
	}
	/**
	 * @see Wheelswitch#getFormatter()
	 */
	public AbstractWheelswitchFormatter getFormatter() {
		return getNumber().getFormatter();
	}
	
	/* (non-Javadoc)
     * @see com.cosylab.gui.components.AbstractNumericDisplayerPanel#setTitleMaximumFontSize(int)
     */
    public void setTitleMaximumFontSize(int max) {
        getNumber().setUnitsMaximumFontSize(max);
        super.setTitleMaximumFontSize(max);
    }
    
    /* (non-Javadoc)
     * @see com.cosylab.gui.components.AbstractNumericDisplayerPanel#setTitleMinimumFontSize(int)
     */
    public void setTitleMinimumFontSize(int min) {
        getNumber().setUnitsMinimumFontSize(min);        
        super.setTitleMinimumFontSize(min);
    }
}

/* __oOo__ */
