package com.cosylab.gui.components;

import java.beans.Customizer;

import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.ButtonModel;
import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.JRadioButton;

import com.cosylab.gui.components.range2.RescalingValuePolicy;
import com.cosylab.gui.components.range2.ShiftValuePolicy;
import com.cosylab.gui.components.range2.TrimValuePolicy;

public class PiperCustomizer extends AbstractDisplayerPanelCustomizer {
	private static final long serialVersionUID = 1L;
	public static final String VALUE_DISPLAY = "Value Display";
	public static final String VALUE_POLICY = "Value Range Policy";
	private static final String UNITS = "units";
	private static final String UNITS_VISIBLE = "unitsVisible";
	private static final String FORMAT = "format";
	private static final String GRAPH_MIN = "minimum";
	private static final String GRAPH_MAX = "maximum";
	private static final String ENHANCED = "enhanced";
	private static final String STRETCH = "stretch";
	private static final String TILTING_ENABLED = "tiltingEnabled";
	private static final String RESIZABLE = "resizable";
	public static final String TITLE = "title";
	public static final String[] VISUAL_BASIC_PROPERTIES = {TITLE, RESIZABLE, ENHANCED,  TILTING_ENABLED, STRETCH };
	public static String[] VALUE_DISPLAY_PROPERTIES = {
		GRAPH_MIN, GRAPH_MAX, UNITS,UNITS_VISIBLE, FORMAT
	};

	/** Supported aspects. */
	public static String[] ASPECTS = { VISUAL_BASIC, VALUE_DISPLAY, VALUE_POLICY };
	
	/*
	 * Auxilary Editor implementation for customizing DisplayerPanel layout.
	 */
	private class WPanel extends JPanel implements Customizer
	{
		private static final long serialVersionUID = 1L;
		private Piper displayer;
		private JRadioButton trimButton;
		private JRadioButton rescalingButton;
		private JRadioButton shiftButton;
		private ButtonGroup bg;

		/**
		 * Creates a new WPanel object.
		 */
		public WPanel()
		{
			bg = new ButtonGroup();

			trimButton = new JRadioButton("Trim Value Range");
			rescalingButton = new JRadioButton("Rescale Value Range");
			shiftButton = new JRadioButton("Shift Value Range");

			bg.add(trimButton);
			bg.add(rescalingButton);
			bg.add(shiftButton);

			setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
			add(trimButton);
			add(rescalingButton);
			add(shiftButton);
		}

		/**
		 * DOCUMENT ME!
		 *
		 * @param object DOCUMENT ME!
		 * @param aspect DOCUMENT ME!
		 *
		 * @return DOCUMENT ME!
		 *
		 * @throws IllegalArgumentException DOCUMENT ME!
		 */
		public JComponent getEditorComponent(Object object, String aspect)
			throws IllegalArgumentException
		{
			if (aspect.equals(VALUE_POLICY)) {
				displayer = (Piper)object;
				initialize();

				return this;
			}

			return null;
		}

		private void initialize()
		{
			if (displayer == null) {
				return;
			}

			if (displayer.getValuePolicy() instanceof RescalingValuePolicy) {
				rescalingButton.setSelected(true);
			}
			if (displayer.getValuePolicy() instanceof TrimValuePolicy) {
				trimButton.setSelected(true);
			}
			if (displayer.getValuePolicy() instanceof ShiftValuePolicy) {
				shiftButton.setSelected(true);
			}
		}

		/**
		 * DOCUMENT ME!
		 *
		 * @param object DOCUMENT ME!
		 * @param aspect DOCUMENT ME!
		 *
		 * @return DOCUMENT ME!
		 */
		public boolean canEdit(Object object, String aspect)
		{
			return aspect.equals(VALUE_POLICY);
		}

		/**
		 * DOCUMENT ME!
		 */
		public void applySettings()
		{
			ButtonModel bm = bg.getSelection();

			if (bm == null) {
				return;
			}

			if (bm == shiftButton.getModel()) {
				displayer.setValuePolicy(new ShiftValuePolicy());
			} else if (bm == rescalingButton.getModel()) {
				displayer.setValuePolicy(new RescalingValuePolicy());
			} else if (bm == trimButton.getModel()) {
				displayer.setValuePolicy(new TrimValuePolicy());
			} else {
				throw new AssertionError();
			}

			initialize();
		}

		/**
		 * DOCUMENT ME!
		 */
		public void revertSettings()
		{
		}

		/**
		 * DOCUMENT ME!
		 */
		public void stopEditing()
		{
		}

		public void setObject(Object bean) {
			displayer = (Piper) bean;
		}
	}

	public PiperCustomizer() {
		addCustomizerTable(VISUAL_BASIC, VISUAL_BASIC_PROPERTIES);
		addCustomizerTable(VALUE_DISPLAY, VALUE_DISPLAY_PROPERTIES);
		addCustomizer(VALUE_POLICY, new WPanel());
		setSize(331, 185);
	}
}
