/*
 * Copyright (c) 2003 by Cosylab d.o.o.
 *
 * The full license specifying the redistribution, modification, usage and other
 * rights and obligations is included with the distribution of this project in
 * the file license.html. If the license is not included you may find a copy at
 * http://www.cosylab.com/legal/abeans_license.htm or may write to Cosylab, d.o.o.
 *
 * THIS SOFTWARE IS PROVIDED AS-IS WITHOUT WARRANTY OF ANY KIND, NOT EVEN THE
 * IMPLIED WARRANTY OF MERCHANTABILITY. THE AUTHOR OF THIS SOFTWARE, ASSUMES
 * _NO_ RESPONSIBILITY FOR ANY CONSEQUENCE RESULTING FROM THE USE, MODIFICATION,
 * OR REDISTRIBUTION OF THIS SOFTWARE.
 */

package com.cosylab.gui.components;

import java.util.EventObject;


/**
 * Event for describing task progress change.
 *
 * @author <a href="mailto:ales.pucelj@cosylab.com">Ales Pucelj</a>
 * @version $id$
 */
public class ProgressEvent extends EventObject
{
	private String status;
	private int current;
	private int total;

	/**
	 * Creates new instance of progress change event.
	 *
	 * @param source ProgressMonitor that has triggered this event.
	 * @param status Text description of progress.
	 * @param current New progress value.
	 * @param total Progress value when completed.
	 */
	public ProgressEvent(Object source, String status, int current, int total)
	{
		super(source);
		this.status = status;
		this.current = current;
		this.total = total;
	}

	/**
	 * Returns current value of progress.
	 *
	 * @return current value of progress.
	 */
	public int getCurrent()
	{
		return current;
	}

	/**
	 * Returns status description.
	 *
	 * @return Textual description of current status.
	 */
	public String getStatus()
	{
		return status;
	}

	/**
	 * Returns total ammount.
	 *
	 * @return total ammount for progress.
	 */
	public int getTotal()
	{
		return total;
	}
}

/* __oOo__ */
