/*
 * Copyright (c) 2003 by Cosylab d.o.o.
 *
 * The full license specifying the redistribution, modification, usage and other
 * rights and obligations is included with the distribution of this project in
 * the file license.html. If the license is not included you may find a copy at
 * http://www.cosylab.com/legal/abeans_license.htm or may write to Cosylab, d.o.o.
 *
 * THIS SOFTWARE IS PROVIDED AS-IS WITHOUT WARRANTY OF ANY KIND, NOT EVEN THE
 * IMPLIED WARRANTY OF MERCHANTABILITY. THE AUTHOR OF THIS SOFTWARE, ASSUMES
 * _NO_ RESPONSIBILITY FOR ANY CONSEQUENCE RESULTING FROM THE USE, MODIFICATION,
 * OR REDISTRIBUTION OF THIS SOFTWARE.
 */

package com.cosylab.gui.components;

/**
 * Interface defining progress monitoring for long running tasks. This
 * interface defines listeners to be notified on progress of long running tasks.<br>
 * Progress is defined as an integer value without any predefined scale.
 * Implementor will provide maximum ammount (<code>getTotal()</code>) and
 * periodically update current value. When <code>getCurrent()</code> equals
 * <code>getTotal()</code> the task is expected to end.
 *
 * @author <a href="mailto:ales.pucelj@cosylab.com">Ales Pucelj</a>
 * @version $Revision: 1.1 $
 */
public interface ProgressMonitor
{
	/**
	 * Add progress listener to this object. Listener will be notified about
	 * task start and end and also itermediate progress or interruptions.
	 *
	 * @param listener to add.
	 */
	void addProgressListener(ProgressListener listener);

	/**
	 * Returns current progress, value will be between 0 and
	 * <code>getStatus()</code>.
	 *
	 * @return current value of progress.
	 */
	int getCurrent();

	/**
	 * Returns total progress, that is value when this task will be complete.
	 *
	 * @return Total progress.
	 */
	int getTotal();

	/**
	 * Returns description of current task. This text may change at any time to
	 * indicate different phases.
	 *
	 * @return Textual description.
	 */
	String getStatus();

	/**
	 * Removes listener.
	 *
	 * @param listener to remove.
	 */
	void removeProgressListener(ProgressListener listener);
}

/* __oOo__ */
