/*
 * Copyright (c) 2004 by Cosylab d.o.o.
 *
 * The full license specifying the redistribution, modification, usage and other
 * rights and obligations is included with the distribution of this project in
 * the file license.html. If the license is not included you may find a copy at
 * http://www.cosylab.com/legal/abeans_license.htm or may write to Cosylab, d.o.o.
 *
 * THIS SOFTWARE IS PROVIDED AS-IS WITHOUT WARRANTY OF ANY KIND, NOT EVEN THE
 * IMPLIED WARRANTY OF MERCHANTABILITY. THE AUTHOR OF THIS SOFTWARE, ASSUMES
 * _NO_ RESPONSIBILITY FOR ANY CONSEQUENCE RESULTING FROM THE USE, MODIFICATION,
 * OR REDISTRIBUTION OF THIS SOFTWARE.
 */

package com.cosylab.gui.components;

import com.cosylab.application.state.State;
import com.cosylab.application.state.StateFactory;
import com.cosylab.application.state.StateOriginator;

import com.cosylab.events.SetEvent;
import com.cosylab.events.SetListener;

import com.cosylab.gui.components.customizer.AbstractCustomizerPanel;
import com.cosylab.gui.components.range2.IntegerValuePolicy;
import com.cosylab.gui.components.range2.LinearRange;
import com.cosylab.gui.components.range2.RangedValueListener;
import com.cosylab.gui.components.range2.RangedValueController;
import com.cosylab.gui.components.range2.RangedValueEvent;
import com.cosylab.gui.components.range2.TickParameters;
import com.cosylab.gui.components.slider.CosySliderUI;
import com.cosylab.gui.components.slider.InfoBar;
import com.cosylab.gui.components.slider.NavigationBar;
import com.cosylab.gui.components.util.ColorHelper;
import com.cosylab.gui.components.util.CosyTransferHandler;
import com.cosylab.gui.components.util.CosyUIElements;
import com.cosylab.gui.components.util.PopupManageable;
import com.cosylab.gui.components.util.PopupManager;
import com.cosylab.util.PrintfFormat;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import java.util.ArrayList;

import javax.swing.AbstractAction;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JSlider;
import javax.swing.Timer;
import javax.swing.UIDefaults;
import javax.swing.UIManager;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;


/**
 * Slider is a GUI component for changing double value. Slider defines limits
 * within which the value may be set and fire update events when this value is
 * changed. For precise control, textual entry and button controls are also
 * available.
 *
 * @author <a href="mailto:ales.pucelj@cosylab.com">Ales Pucelj </a>
 * @version $id$
 */
public class Slider extends JPanel implements TickParameters, PopupManageable,
	StateOriginator
{
	
	public enum Button {
		FAST_INCREMENT, FAST_DECREMENT, BIT_INCREMENT, BIT_DECREMENT, SLOW_INCREMENT, SLOW_DECREMENT 
	}

	private static final long serialVersionUID = 1L;

	private boolean popupEnabled;

	/** Set only when demanded
	 * @deprecated use SliderSetMode instead.*/
	public static final int SET_MANUAL = 0;

	/** Set when dragging stops
	 * @deprecated use SliderSetMode instead.*/
	public static final int SET_ON_RELEASE = 1;

	/** Set when value changes
	 * @deprecated use SliderSetMode instead.*/
	public static final int SET_ON_CHANGE = 2;

	private AbstractCustomizerPanel customizer;
	private PopupManager popupManager;
	private boolean settable=true;
	private final ArrayList<SetListener> listeners = new ArrayList<SetListener>();
	protected JSlider slider = null;
	private CosySliderUI sliderUI = null;
	private RangedValueController rangedValue;
	private InfoBar infoBar = null;
	private boolean showIntegerOnly = false;
	
	// suspended count
	private int suspended = 0;

	private boolean supressChanges=false;
	private boolean supressSetEvent=false;
	private boolean supressPropChEvent=false;

	NavigationBar navigationBar = null;
	private SliderSetMode mode = SliderSetMode.SET_ON_CHANGE;

	//	 If isBitSteepRelative=false then we change absulut value else relative!!!
	private boolean isBitStepRelative = true;
	private double defaultBitStep = 1;

	//	BitStep value , to tell slider how big the step should be
	private double bitStep = defaultBitStep;
	private double defaultSmallStep = 10;

	//	SmallStep value , to tell slider how big the step should be
	private double smallStep = defaultSmallStep;

	//	 If isSmallSteepRelative=false then we change absulut value else
	// relative!!!
	private boolean isSmallStepRelative = true;
	private double defaultLargeStep = 25;

	//	LargeStep value , to tell slider how big the step should be
	private double largeStep = defaultLargeStep;

	//	 If isLargeSteepRelative=false then we change absulut value else
	// relative!!!
	private boolean isLargeStepRelative = true;
	
	//For autosynchronization see RT #22324
	/* Flag indicating whether the slider is automatically synchronized each time the
	 *  readback changes. Synchronization occurs after the autoSynchronizeDelay
	 *  miliseconds pass since the last user intervention. */
	private boolean autoSynchronize = false;
	
	/* The time in milliseconds that have to pass since last user intervention for
	 *  the automatical synchronization to occur for the first time. */
	private long autoSynchronizeDelay = 2000;
		
	private boolean allowAutoSync = true;
	
	private boolean externalSet = false;

	private Timer autoSyncTimer;
	
	private PrintfFormat formatter=new PrintfFormat("%f"); 
		
	private class ValueListener implements RangedValueListener
	{
		/**
		 * @see com.cosylab.gui.components.range.RangedValueListener#valueChanged(RangedValueEvent)
		 */
		public void valueChanged(RangedValueEvent event)
		{
			if (event.isMinOrMaxChanged()) {
				
				getSlider().setValue((int)getRangedValue().getRelativeValue());
				
				getSliderUI().setTrailerPosition(getSliderUI()
				    .getTrailerPosition());
				getNavigationBar().setMinimum(getRangedValue().getMinimum());
				getNavigationBar().setMaximum(getRangedValue().getMaximum());
				repaint();

			}
		}
	}

	/**
	 * Resets sliders steps to default values
	 */
	public void resetSteps()
	{
		setBitStep(defaultBitStep);
		setBitStepRelative(true);
		setSmallStep(defaultSmallStep);
		setSmallStepRelative(true);
		setLargeStep(defaultLargeStep);
		setLargeStepRelative(true);
	}

	/**
	 * Listens for changes to draggable slider button.
	 *
	 * @author <a href="mailto:ales.pucelj@cosylab.com">Ales Pucelj </a>
	 * @version $id$
	 */
	private class SliderChange implements ChangeListener
	{
		/**
		 * Notifies of value change.
		 *
		 * @param event ChangeEvent
		 *
		 * @see javax.swing.event.ChangeListener#stateChanged(ChangeEvent)
		 */
		public void stateChanged(ChangeEvent event)
		{	
			if (mode == SliderSetMode.SET_ON_CHANGE || mode == SliderSetMode.SET_MANUAL) {
				performInternalSliderSet();
			} 
			if (!externalSet) {
				getNavigationBar().setValue(getRangedValue().toAbsolute(getSlider().getValue()));
			}
		}
	}

	/**
	 * Creates a new Slider object.
	 */
	public Slider()
	{
		super();
		initialize();
	}

	private GridBagConstraints newGC(int y)
	{
		GridBagConstraints gc = new GridBagConstraints();

		gc.gridx = 0;
		gc.gridy = y;
		gc.weightx = 1.0;
		gc.weighty = 0.0;
		gc.fill = GridBagConstraints.HORIZONTAL;
		gc.insets = new Insets(1, 2, 2, 2);

		return gc;
	}

	/**
	 * Adds set listener.
	 *
	 * @param listener SetListener
	 */
	public void addSetListener(SetListener listener)
	{
		synchronized (listeners) {
			listeners.add(listener);
		}
	}

	/**
	 * Removes set listeners.
	 *
	 * @param listener SetListener
	 */
	public void removeSetListener(SetListener listener)
	{
		synchronized (listeners) {
			listeners.remove(listener);
		}
	}

	/**
	 * Fires event that value has changed.
	 *
	 * @param newValue double
	 */
	protected void fireSetPerformed(double newValue)
	{
		final SetEvent e = new SetEvent(this, newValue);

		synchronized (listeners) {
			final int n = listeners.size();

			for (int i = 0; i < n; i++) {
				((SetListener)listeners.get(i)).setPerformed(e);
			}
		}
	}

	private void initialize()
	{
		setLayout(new GridBagLayout());		
		setBorder(CosyUIElements.getPlainBorder(true));
		setBackground(ColorHelper.getCosyControl());

		add(getInfoBar(), newGC(0));

		add(getNavigationBar(), newGC(1));

		GridBagConstraints gc = new GridBagConstraints();

		gc.gridx = 0;
		gc.gridy = 2;
		gc.weightx = 1.0;
		gc.weighty = 0.0;
		gc.fill = GridBagConstraints.HORIZONTAL;
		gc.insets = new Insets(0, 2, 0, 2);

		getSliderUI().setTolerance(0.0000001);
		add(getSlider(), gc);
		
		setValue(0);
		setForeground(Color.BLACK);
	}

	protected InfoBar getInfoBar()
	{
		if (infoBar == null) {
			infoBar = new InfoBar(this);
		}

		return infoBar;
	}

	protected NavigationBar getNavigationBar()
	{
		if (navigationBar == null) {
			navigationBar = new NavigationBar(this);
			navigationBar.addSetListener(new SetListener() {
			
				public void setPerformed(SetEvent e) {
					performInternalSet(e.getDoubleValue());
				}
			
			});
		}

		return navigationBar;
	}

	protected JSlider getSlider()
	{
		if (slider == null) {
			slider = new JSlider();
			slider.setBackground(ColorHelper.getCosyControl());
			slider.setPaintLabels(true);
			slider.setPaintTicks(true);
			slider.setMinorTickSpacing(10000);
			slider.setMajorTickSpacing(10000);

			slider.setMinimum(0);
			slider.setMaximum(10000);

			// *** BUGFIX ***
			// This is bugfix for netbeans 3.6RC1 L&F problem
			// It appears there is a problem with how the NB L&F is implemented.
			// Done to resolve RT#5535
			// Setting these global variables may affect other sliders,
			// depending
			// on Metal or all L&F settings
			// ***
			UIManager.put("Slider.trackWidth", new Integer(7));
			UIManager.put("Slider.majorTickLength", new Integer(6));
			UIManager.put("Slider.horizontalThumbIcon",
			    new UIDefaults.ProxyLazyValue(
			        "javax.swing.plaf.metal.MetalIconFactory",
			        "getHorizontalSliderThumbIcon"));
			UIManager.put("Slider.verticalThumbIcon",
			    new UIDefaults.ProxyLazyValue(
			        "javax.swing.plaf.metal.MetalIconFactory",
			        "getVerticalSliderThumbIcon"));

			// *** END BUGFIX ***
			slider.setUI(getSliderUI());
			slider.addChangeListener(new SliderChange());
			slider.addMouseListener(new MouseAdapter() {

				@Override
				public void mousePressed(MouseEvent e) {
					allowAutoSync = false;
				}
				@Override
				public void mouseReleased(MouseEvent e) {
					if (mode == SliderSetMode.SET_ON_RELEASE) {
						performInternalSliderSet();
					} else if (showIntegerOnly) {
						getSlider().setValue((int)(getRangedValue().getRelativeValue()));
					}
					allowAutoSync = true;

				}
			});
			slider.setPreferredSize(new Dimension(150, 50));
		}
		return slider;
	}
	
	private int old = Integer.MIN_VALUE;
//	private int oldOld = Integer.MAX_VALUE;
	
	private void performInternalSliderSet() {
		synchronized(this) {
			if (!supressChanges) {

				int newV = getSlider().getValue();
	
				//the slider mouse listener works strange
				//if you only click (not even drag) this method is 
				//eventually called, causing a chain of events that value
				//on slider was changed, which is not necessarily true
				//System.out.println(newV+" "+old+" "+oldOld);
				//apparently not anymore - if this is turned on Set_On_release
				//doesn't work properly
//				if (old != newV && newV != oldOld && old != oldOld) {
				if (old != newV) {
					//System.out.println(newV);
					performInternalSetRelative(newV);
					//getRangedValue().setRelativeValue(newV);
				}
				old = newV;
				
			}
		}
	}
	
	private void performInternalSet(double value) {
		
		value= getRangedValue().validate(value);
		
		if (value==getValue()) {
			return;
		}
		
		getNavigationBar().setValue(value);
		
		if (mode==SliderSetMode.SET_MANUAL) {
			synchronized (this) {
				if (!supressChanges) {
					supressChanges = true;

					getRangedValue().setValue(value);
					// distribute change
					navigationBar.setValue(getValue());
					getSlider().setValue((int)getRangedValue().getRelativeValue());

					supressChanges= false;
				}
			}
			return;
		}
		
		synchronized (this) {
			if (!supressChanges) {
				supressChanges = true;

				getAutoSyncTimer().stop();

				double old= getValue();
				getRangedValue().setValue(value);
				
				// distribute change
				navigationBar.setValue(getValue());
				getSlider().setValue((int)getRangedValue().getRelativeValue());

				if (!isEditable()) {
					setReadback(getValue());
				}
				
				if (!supressSetEvent && !isSuspended()) {
					fireSetPerformed(getValue());
				}
				if (!supressPropChEvent && !isSuspended()) {
					firePropertyChange("value", old, getValue());
				}
				
				if (autoSynchronize) {
					getAutoSyncTimer().restart();
				}
				
				supressChanges= false;
			}
		}
		
	}

	private void performInternalSetRelative(double rVal) {
		
		performInternalSet(getRangedValue().toAbsolute(rVal));
		
	}

	protected CosySliderUI getSliderUI()
	{
		if (sliderUI == null) {
			sliderUI = new CosySliderUI(getRangedValue());
		}
		
		return sliderUI;
	}

	/**
	 * Sets new minimum value.
	 *
	 * @param minimum New minimum value.
	 */
	public void setMinimum(double minimum)
	{	
		double oldValue = getMinimum();
		double trailerValue = getSliderUI().getTrailerPosition();
		getRangedValue().setMinimum(minimum);
		/*
		 * If minimum has changed, navigationBar's minimum and slider's scale 
		 * and position has to be modified.
		 */
		navigationBar.setMinimum(minimum);
		modifySliderPosition(trailerValue);
		firePropertyChange("minimum", oldValue, minimum);
	}

	/**
	 * Sets new maximum value.
	 *
	 * @param maximum New maximum value.
	 */
	public void setMaximum(double maximum)
	{	
		double oldValue = getMaximum();
		double trailerValue = getSliderUI().getTrailerPosition();
		getRangedValue().setMaximum(maximum);
		/*
		 * If maximum has changed, navigationBar's maximum and slider's scale 
		 * and position has to be modified.
		 */
		navigationBar.setMaximum(maximum);
		modifySliderPosition(trailerValue);
		firePropertyChange("maximum", oldValue, maximum);
	}

	/**
	 * Sets the new value this slider will display.
	 *
	 * @param value New value.
	 */
	public void setValue(double value)
	{	
		getAutoSyncTimer().stop();

		double oldValue = getValue();
		
		synchronized (this) {
			supressChanges=true;
			getRangedValue().setValue(value);
			// distribute change
			navigationBar.setValue(getValue());
			externalSet = true;
			getSlider().setValue((int)getRangedValue().getRelativeValue());

			if (!isEditable()) {
				setReadback(getValue());
			}

			supressChanges=false;
		}
		
		if (autoSynchronize) {
			getAutoSyncTimer().restart();
		}

		firePropertyChange("value", oldValue, value);
	}
	
	private Timer getAutoSyncTimer() {
		if (autoSyncTimer == null) {
			autoSyncTimer = new Timer((int)autoSynchronizeDelay, new ActionListener(){
				public void actionPerformed(ActionEvent e) {
					if (allowAutoSync)
						synchronize();
					autoSyncTimer.stop();
				}
			});

			
		}

		return autoSyncTimer;
	}

	/**
	 * Returns currently set minimum value.
	 *
	 * @return double minimum value.
	 */
	public double getMinimum()
	{
		return getRangedValue().getMinimum();
	}

	/**
	 * Returns currently set maxium value.
	 *
	 * @return double maximum value.
	 */
	public double getMaximum()
	{
		return getRangedValue().getMaximum();
	}

	/**
	 * Returns current value the slider is displaying.
	 *
	 * @return double Current value.
	 */
	public double getValue()
	{
		/*
		 * Fix for out of bounds behaviour of this component (RT #26250).
		 * Replaced:
		 * return getRangedValue().getValue();
		 */
		return getRangedValue().getRawValue();
	}

	/**
	 * Sets string representing units displayed next to value entry field.
	 * @param units New value for units, may be null.
	 */
	public void setUnits(String units)
	{	
		String oldValue = getUnits();
		getNavigationBar().setUnits(units);
		firePropertyChange("units", oldValue, units);
	}

	/**
	 * Returns string representing units that are displayed next to value entry
	 * field.
	 *
	 * @return String units string.
	 */
	public String getUnits()
	{
		return getNavigationBar().getUnits();
	}

	/**
	 * Sets wther units are displayed next to value entry field.
	 *
	 * @param b wehter units are displayed.
	 */
	public void setUnitsVisible(boolean b)
	{
		if (isUnitsVisible() == b) return;
		getNavigationBar().setUnitsVisible(b);
		firePropertyChange("unitsVisible", !b, b);
	}

	/**
	 * Returns true if units are displayed next to value entry field.
	 *
	 * @return boolean unitsVisible.
	 */
	public boolean isUnitsVisible()
	{
		return getNavigationBar().isUnitsVisible();
	}

	private double lastReadbackValue = Double.NaN;
	/**
	 * Sets the readback indicator position. This determines the position of
	 * trailer shadow that is displayed to indicate last readback. <br>
	 * This method has no effect if slider is not settable. Othwerwise the
	 * readback will have the same value as slider value and parameter will be
	 * ignored.
	 *
	 * @param value double New readback position.
	 */
	public void setReadback(double value)
	{
		double oldValue = getReadback();
		getSliderUI().setTrailerPosition(value);
		if (value < getMinimum() || value > getMaximum()) {
			getNavigationBar().setValue(value);
		}
		if (autoSynchronize) {
			if (lastReadbackValue != value) {
				getAutoSyncTimer().restart();
			} else {
				if (!getAutoSyncTimer().isRunning()) {
					getAutoSyncTimer().start();
				}
			}
		
		}
		
		
		lastReadbackValue = value;
		firePropertyChange("readBack", oldValue, value);
		if (!isEditable()) {
			synchronize();
		}
	}

	/**
	 * Returns current readback position. This is the position of the trailer
	 * shadow thad indicates last readback. <br>
	 * This value will return last set value if the slider is settable.
	 * Othwerwise it returns the slider value itself.
	 *
	 * @return double Last set value or slider value if not settable.
	 */
	public double getReadback()
	{
		return getSliderUI().getTrailerPosition();
	}

	/**
	 * Increments by a single bit in long representation of double value.
	 */
	public void incrementBit()
	{	
		if (isBitStepRelative()) {
			performInternalFractionSet(getBitStep() / 100);
		} else {
			performInternalSet(getRangedValue().validate(getValue() + getBitStep()));
		}
	}

	/**
	 * Sets format for editable field.
	 *
	 * @param s new C-Style format.
	 */
	public void setFormat(String s)
	{	
		String oldValue = getFormat();
		getNavigationBar().setFormat(s);
		if (s.contains("%d")) {
			showIntegerOnly = true;
			getRangedValue().addPolicy(new IntegerValuePolicy());
			//round to tick
			setValue(getValue());
		} else {
			getRangedValue().removePolicyByType(IntegerValuePolicy.class);
			showIntegerOnly = false;
		}
		getSliderUI().setFormat(s);
		formatter=new PrintfFormat(s);
		firePropertyChange("format", oldValue, s);
		//repaint ticks
		repaint();
	}
	
	/**
	 * Returns format of the editable field.
	 *
	 * @return C-Style format.
	 */
	public String getFormat()
	{
		return getNavigationBar().getFormat();
	}

	/**
	 * Returns title.
	 *
	 * @return title.
	 */
	public String getTitle()
	{
		return getInfoBar().getTitle();
	}

	/**
	 * Sets title.
	 *
	 * @param title
	 */
	public void setTitle(String title)
	{
		String oldValue = getTitle();
		this.title = title;
		getInfoBar().setTitle(title);
		firePropertyChange("title", oldValue, title);
	}
	
	private String title = " ";
	public void setTitleVisible(boolean titleVisible){
		if (getTitleVisible() == titleVisible) return;
		if (!titleVisible) {
			title = getInfoBar().getTitle();
			getInfoBar().setTitle(" ");
		}
		else {
			getInfoBar().setTitle(this.title);
		}
		firePropertyChange("titleVisible", !titleVisible, titleVisible);
	}
	
	public boolean getTitleVisible(){
		return !getInfoBar().getTitle().equals("");
	}
	
	// SIMBOLIC METHODS
	public int getTitleMinimumFontSize(){
		return 12;
	}
	public void setTitleMinimumFontSize(int i){
		
	}
	public int getTitleMaximumFontSize(){
		return 12;
	}
	public void setTitleMaximumFontSize(int i){
	}
	public void setResizable(boolean b){
	}
	public boolean getResizable(){
		return true;
	}

	/**
	 * Small increment (by 10 percent).
	 */
	public void incrementSmall()
	{
		if (isSmallStepRelative()) {
			performInternalFractionSet(getSmallStep() / 100);
		} else {
			performInternalSet(getRangedValue().validate(getValue() + getSmallStep()));
		}
	}

	/**
	 * Large increment (by 25 percent).
	 */
	public void incrementLarge()
	{
		if (isLargeStepRelative() == true) {
			performInternalFractionSet(getLargeStep() / 100);
		} else {
			performInternalSet(getRangedValue().validate(getValue() + getLargeStep()));
		}
	}

	/**
	 * Decrements by a single bit in long representation of the double.
	 */
	public void decrementBit()
	{
		if (isBitStepRelative()) {
			performInternalFractionSet(-getBitStep() / 100);
		} else {
			performInternalSet(getRangedValue().validate(getValue() - getBitStep()));
		}
	}

	/**
	 * SetBitStep sets the value of BitStep
	 *
	 * @param value bitStep
	 */
	public void setBitStep(double value)
	{
		double oldValue = getBitStep();
		bitStep = value;
		firePropertyChange("bitStep", oldValue, value);
	}

	/**
	 * Returns the value of bitStep
	 *
	 * @return bitStep
	 */
	public double getBitStep()
	{
		return bitStep;
	}

	/**
	 * Returns the value of smallStep
	 *
	 * @return smallStep
	 */
	public double getSmallStep()
	{
		return smallStep;
	}

	/**
	 * SetSmallStep sets the value of smallStep
	 *
	 * @param value smallStep
	 */
	public void setSmallStep(double value)
	{	
		double oldValue = getSmallStep();
		smallStep = value;
		firePropertyChange("smallStep", oldValue, value);
	}

	/**
	 * SetLargeStep sets the value of largeStep
	 *
	 * @param value largeStep
	 */
	public void setLargeStep(double value)
	{	
		double oldValue = getLargeStep();
		largeStep = value;
		firePropertyChange("largeStep", oldValue, value);
	}

	/**
	 * Returns the value of largeStep
	 *
	 * @return largeStep
	 */
	public double getLargeStep()
	{
		return largeStep;
	}

	/**
	 * Returns true if bitStep is relative , else false
	 *
	 * @return
	 */
	public boolean isBitStepRelative()
	{
		return isBitStepRelative;
	}

	/**
	 * Sets bitStep to be relative if true(example : bitStep=20 , this means
	 * that bitStep will change for 20% of sliders current range(min,max))
	 *
	 * @param value
	 */
	public void setBitStepRelative(boolean value)
	{	
		if (isBitStepRelative() == value) return;
		isBitStepRelative = value;
		firePropertyChange("bitStepRelative", !value, value);
	}

	/**
	 * Returns true if smallStep is relative , else false
	 *
	 * @return
	 */
	public boolean isSmallStepRelative()
	{
		return isSmallStepRelative;
	}

	/**
	 * Sets smallStep to be relative if true(example : smallStep=20 , this
	 * means that smallStep will change for 20% of sliders current
	 * range(min,max))
	 *
	 * @param value
	 */
	public void setSmallStepRelative(boolean value)
	{	
		if (isSmallStepRelative() == value) return;
		isSmallStepRelative = value;
		firePropertyChange("smallStepRelative", !value, value);
	}

	/**
	 * Returns true if largeStep is relative , else fasle
	 *
	 * @return
	 */
	public boolean isLargeStepRelative()
	{
		return isLargeStepRelative;
	}

	/**
	 * Sets largeStep to be relative if true(example : largeStep=20 , this
	 * means that largeStep will change for 20% of sliders current
	 * range(min,max))
	 *
	 * @param value
	 */
	public void setLargeStepRelative(boolean value)
	{	
		if (isLargeStepRelative() == value) return;
		isLargeStepRelative = value;
		firePropertyChange("largeStepRelative", !value, value);
	}

	/**
	 * Decrements by 10 percent.
	 */
	public void decrementSmall()
	{
		if (isSmallStepRelative()) {
			performInternalFractionSet(-getSmallStep() / 100);
		} else {
			performInternalSet(getRangedValue().validate(getValue() - getSmallStep()));
		}
	}

	/**
	 * Decrements by 25 percent.
	 */
	public void decrementLarge()
	{
		if (isLargeStepRelative()) {
			performInternalFractionSet(-getLargeStep() / 100);
		} else {
			performInternalSet(getRangedValue().validate(getValue() - getLargeStep()));
		}
	}

	private void performInternalFractionSet(double fraction)
	{
		performInternalSetRelative(getRangedValue().getRelativeValue()+fraction*getRangedValue().getScalingFactor());
					
	}

	/**
	 * Return set mode.
	 *
	 * @return int
	 * @deprecated use {@link Slider#getSetMode()}
	 */
	public int getMode()
	{
		return mode.ordinal();
	}

	/**
	 * Returns ranged value representing this slider's value.
	 *
	 * @return RangedValue.
	 */
	private RangedValueController getRangedValue()
	{
		if (rangedValue == null) {
			rangedValue = new RangedValueController();
			rangedValue.setRange(new LinearRange());
			rangedValue.setScalingFactor(10000);
			rangedValue.addRangedValueListener(new ValueListener());
		}

		return rangedValue;
	}

	/**
	 * Sets the set mode.
	 *
	 * @param newMode int
	 * @deprecated use {@link Slider#setSetMode(com.cosylab.gui.components.Slider.SetMode)}
	 */
	public void setMode(int newMode)
	{	
		setSetMode(SliderSetMode.valueOf(newMode));
	}
	
	 public void setSetMode(SliderSetMode mode) {
		 SliderSetMode oldValue = getSetMode();
		 this.mode = mode;
		 getInfoBar().setSetButtonVisible(mode == SliderSetMode.SET_MANUAL);
		 firePropertyChange("setMode", oldValue, this.mode);
	 }
	 
	 public SliderSetMode getSetMode() {
		 return mode;
	 }

	/**
	 * Run test applet.
	 *
	 * @param args command line parameters
	 */
	public static void main(String[] args)
	{
		
		JFrame dialog = new JFrame();
		dialog.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		
		final Slider slider = new Slider();
		slider.setTitle("TEST");
		slider.setEnabled(false);
		slider.setSetMode(SliderSetMode.SET_ON_RELEASE);
		slider.setMinimum(0);
		slider.setMaximum(150);
		slider.setFormat("%f");
		slider.addPropertyChangeListener("value", new PropertyChangeListener(){
			public void propertyChange(PropertyChangeEvent evt) {
				System.out.println(slider.getValue());				
			}
		});
		
		final NumberField nf = new NumberField();
		nf.setFormat("%f");
		
		nf.addPropertyChangeListener("value", new PropertyChangeListener(){
			public void propertyChange(PropertyChangeEvent evt) {
				slider.setValue(nf.getDoubleValue());
			}
		});
		
		JPanel p= new JPanel(new BorderLayout());
		p.add(slider, BorderLayout.CENTER);
		p.add(nf, BorderLayout.SOUTH);
		
		dialog.setContentPane(p);
		dialog.setSize(500,200);
		dialog.setVisible(true);
	}
	
	

	/**
	 * Returns size of tick text.
	 *
	 * @see com.cosylab.gui.components.range.TickCallback#measureTick(double,
	 *      String)
	 */
	public int measureTick(double position, String text)
	{
		if (text == null) {
			return 0;
		}

		return getGraphics().getFontMetrics().stringWidth(text);
	}

	/**
	 * Sets whether this slider is settable.
	 *
	 * @param settable true/false
	 */
	public void setEditable(boolean settable)
	{
		this.settable = settable;
		getNavigationBar().setEnabled(settable&&isEnabled());
		getInfoBar().setEnabled(settable&&isEnabled());
		slider.setEnabled(settable&&isEnabled());
		getInfoBar().setEnabled(settable&&isEnabled());
	}

	/**
	 * Returns whether this slider is settable.
	 *
	 * @return true/false
	 */
	public boolean isEditable()
	{
		return settable;
	}

	/**
	 * Returns popup manager for adding popup actions.
	 *
	 * @see com.cosylab.gui.components.util.PopupManageable#getPopupManager()
	 */
	public PopupManager getPopupManager()
	{
		if (popupManager == null) {
			popupManager = new PopupManager(this, true);
			popupManager.addAction(new AbstractAction("Synchronize") {
				private static final long serialVersionUID = 1L;

					public void actionPerformed(ActionEvent e)
					{
						synchronize();
					}
				});
			popupManager.addAction(null);
			popupManager.addAction(new AbstractAction("Preferences...") {
				private static final long serialVersionUID = 1L;
					public void actionPerformed(ActionEvent e)
					{
						getCustomizer().showDialog();
					}
				});
		}

		return popupManager;
	}

	/**
	 * Returns the Customizer instance for this component.
	 *
	 * @return the Customizer instance for this component.
	 */
	public AbstractCustomizerPanel getCustomizer()
	{
		if (customizer == null) {
			customizer = AbstractCustomizerPanel.findCustomizer(this);
		}
		
		return customizer;
	}

	/**
	 * Sets slider value to readback.
	 */
	public void synchronize()
	{
		synchronized (this) {
			supressPropChEvent=supressSetEvent=true;
			performInternalSet(getReadback());
			supressPropChEvent=supressSetEvent=false;
		}
	}
	
	public void setManual() {
		if (mode == SliderSetMode.SET_MANUAL) {
			double oldValue = getValue();
			double value= getRangedValue().validate(getNavigationBar().getValue());
			
			synchronized (this) {
				if (!supressChanges) {
					supressChanges = true;
					
					getAutoSyncTimer().stop();

					getRangedValue().setValue(value);
										
					if (!isSuspended()) {
						fireSetPerformed(getValue());
						firePropertyChange("value", oldValue, getValue());
					}
					
					if (autoSynchronize) {
						getAutoSyncTimer().restart();
					}
					
					supressChanges= false;
				}
			}
		}
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see com.cosylab.application.state.StateOriginator#getState()
	 */
	public State getState()
	{
		State s = StateFactory.createState();
		s.putDouble("Min", getMinimum());
		s.putDouble("Max", getMaximum());
		s.putString("Format", getFormat());
		s.putString("Units", getUnits());

		return s;
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see com.cosylab.application.state.StateOriginator#setState(com.cosylab.application.state.State)
	 */
	public void setState(State state)
	{
		setMinimum(state.getDouble("Min", getMinimum()));
		setMaximum(state.getDouble("Max", getMaximum()));
		setFormat(state.getString("Format", getFormat()));
		setUnits(state.getString("Units", getUnits()));
	}

	/**
	 * Returns whether the user can trigger the component's popup menu through
	 * mouse click.
	 *
	 * @return true if popup is enabled
	 */
	public boolean isPopupEnabled()
	{
		return popupEnabled;
	}

	/**
	 * Enables or disables the popup capabilities of the component.
	 *
	 * @param b the component should bring up popup dialog on mouse click.
	 */
	public void setPopupEnabled(boolean b)
	{
		if (popupEnabled != b) {
			if (b) {
				getSlider().addMouseListener(getPopupManager().getMouseHook());
				addMouseListener(getPopupManager().getMouseHook());
			} else {
				getSlider().removeMouseListener(getPopupManager().getMouseHook());
				removeMouseListener(getPopupManager().getMouseHook());
			}
		}

		popupEnabled = b;
		firePropertyChange("popupEnabled", !b, b);
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see javax.swing.JComponent#getToolTipText()
	 */
	public String getToolTipText()
	{
		StringBuffer sb = new StringBuffer();
		sb.append("setpoint= ");
		sb.append(getNavigationBar().format(getValue()));
		sb.append(" readback= ");
		sb.append(getNavigationBar().format(getReadback()));

		return sb.toString();
	}

	/**
	 * Suspends displayer, minimal GUI action is taken, no data value events
	 * fired and slider is disabled. Call <code>resume()</code> as many times
	 * as suspend was called.
	 */
	public void suspend()
	{
		suspended++;
		setEnabled(false);
	}

	/**
	 * Resumes suspended slider. Call <code>resume()</code> as many times as
	 * suspend was called.
	 *
	 * @see Slider#suspend();
	 */
	public void resume()
	{
		if (suspended > 0) {
			suspended--;
		} else {
			setEnabled(true);
		}
	}

	/**
	 * Returns <code>true</code> if slider is suspended
	 *
	 * @return <code>true</code> if suspended
	 *
	 * @see Slider#suspend()
	 */
	public boolean isSuspended()
	{
		return suspended > 0;
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see javax.swing.JComponent#setEnabled(boolean)
	 */
	public void setEnabled(boolean enabled)
	{
		super.setEnabled(enabled);
		getSlider().setEnabled(enabled&&settable);
		getNavigationBar().setEnabled(enabled&&settable);
		getInfoBar().setEnabled(enabled && settable);
	}
	
	/**
	 * Returns the state of autoSynchronization feature (enabled / disabled).
	 * 
	 * @return
	 */
	public boolean isAutoSynchronize() {
		return autoSynchronize;
	}

	/**
	 * Turns the autoSynchronization on/off.
	 * 
	 * @param autoSynchronize
	 */
	public void setAutoSynchronize(boolean autoSynchronize) {
		if(isAutoSynchronize() == autoSynchronize) return;
		this.autoSynchronize = autoSynchronize;
		if (autoSynchronize) {
			getAutoSyncTimer().restart();
		} else {
			getAutoSyncTimer().stop();
		}
		firePropertyChange("autoSynchronize", !autoSynchronize, autoSynchronize);
	}

	/**
	 * Returns the time in milliseconds that specify the delay of the autoSynchronization.
	 * 
	 * @return
	 */
	public long getAutoSynchronizeDelay() {
		return autoSynchronizeDelay;
	}

	/**
	 * Sets the autoSynchronization in miliseconds.
	 * 
	 * @param autoSynchronizeDelay
	 */
	public void setAutoSynchronizeDelay(long autoSynchronizeDelay) {
		long oldValue = getAutoSynchronizeDelay();
		this.autoSynchronizeDelay = autoSynchronizeDelay;
		
		getAutoSyncTimer().setDelay((int) autoSynchronizeDelay);
		getAutoSyncTimer().setInitialDelay((int) autoSynchronizeDelay);
		if (autoSynchronize) {
			getAutoSyncTimer().restart();
		} else {
			getAutoSyncTimer().stop();
		}
		firePropertyChange("autoSynchronizeDelay", oldValue, this.autoSynchronizeDelay);
		
	}
	
	/**
	 * Modifies slider position. Recalculates position on JSlider's integer 
	 * scale and repaints this component.
	 *
	 */
	private void modifySliderPosition(double trailerValue) {
		getSlider().setValue((int)getRangedValue().getRelativeValue());
		repaint();
	}
	
	public void setDragEnabled(boolean enabled) {
		if (isDragEnabled() == enabled) return;
		if (getTransferHandler() instanceof CosyTransferHandler) {
			if (((CosyTransferHandler)getTransferHandler()).isExportEnabled() == enabled) return;
			
			((CosyTransferHandler)getTransferHandler()).setExportEnabled(enabled, this);
		}
		firePropertyChange("dragEnabled", !enabled, enabled);
	}
	
	public boolean isDragEnabled() {
		if (getTransferHandler() instanceof CosyTransferHandler) {
			return ((CosyTransferHandler)getTransferHandler()).isExportEnabled();
		} else {
			return getTransferHandler() != null;
		}
	}
	
	public void setDropEnabled(boolean enabled) {
		if (isDropEnabled() == enabled) return;
		if (getTransferHandler() instanceof CosyTransferHandler) {
			if (((CosyTransferHandler)getTransferHandler()).isReceiveEnabled() == enabled) return;
			
			((CosyTransferHandler)getTransferHandler()).setReceiveEnabled(enabled, this);
		}
		firePropertyChange("dropEnabled", !enabled, enabled);
	}
	
	public boolean isDropEnabled() {
		if (getTransferHandler() instanceof CosyTransferHandler) {
			return ((CosyTransferHandler)getTransferHandler()).isReceiveEnabled();
		} else {
			return getTransferHandler() != null;
		}
	}
	
	/*
	 * (non-Javadoc)
	 * @see javax.swing.JComponent#setBackground(java.awt.Color)
	 */
	@Override
	public void setBackground(Color bg) {
		super.setBackground(bg);
		getNavigationBar().setBackground(bg);
		getSlider().setBackground(bg);
		getInfoBar().setBackground(bg);
	}
	
	/*
	 * (non-Javadoc)
	 * @see javax.swing.JComponent#setForeground(java.awt.Color)
	 */
	@Override
	public void setForeground(Color fg) {
		super.setForeground(fg);
		getNavigationBar().setForeground(fg);
		getSlider().setForeground(fg);
		getInfoBar().setForeground(fg);
	}

	public String formatNumber(double value) {
		return this.formatter.sprintf(value);
	}
	
	/**
	 * Sets a tooltip to desired button. 
	 * 
	 * @param b 		<code>Button</code> to which a tooltip we want to set.
	 * @param tooltip	<code>String</code> containing a tooltip for a <code>Button</code>.
	 */
	
	public void setButtonTooltip(Button b, String tooltip) {
			
		if (b==Button.FAST_INCREMENT || 
				b==Button.FAST_DECREMENT || 
				b==Button.SLOW_INCREMENT || 
				b==Button.SLOW_DECREMENT || 
				b==Button.BIT_INCREMENT || 
				b==Button.BIT_DECREMENT) {
			getNavigationBar().setButtonTooltip(b, tooltip);
		}
	}
	
	/**
	 * Returns a tooltip of a desired button. 
	 * 
	 * @param b 	<code>Button</code> of which a tooltip we want to get.
	 * @return		<code>String</code> containing a tooltip for a <code>Button</code> if a tooltip is set.
	 */
	
	public String getButtonTooltip(Button b) {
				
		if (b==Button.FAST_INCREMENT || 
				b==Button.FAST_DECREMENT || 
				b==Button.SLOW_INCREMENT || 
				b==Button.SLOW_DECREMENT || 
				b==Button.BIT_INCREMENT || 
				b==Button.BIT_DECREMENT ) {
		return getNavigationBar().getButtonTooltip(b);
		}
		return null;
	}
	
	/**
	 * Sets the tolerance for current value set via slider. 
	 * 
	 * @param precision
	 */
	
	public void setValuePrecision(double precision) {
		getSliderUI().setTolerance(precision);
	}
	
	/**
	 * Returns the tolerance for current value set via slider. 
	 * 
	 * @return precision	<code>Double</code> precision value
	 */
	
	public double getValuePrecision() {
		return getSliderUI().getTolerance();
	}
	
	public void setTitleFont(Font font) {
		Font oldFont = getTitleFont();
		getInfoBar().setFont(font);
		firePropertyChange("titleFont", oldFont, font);
	}
	
	public Font getTitleFont() {
		return getInfoBar().getFont();
	}
	
	public void setUnitsFont(Font font) {
		Font oldFont = getUnitsFont();
		getNavigationBar().setUnitsFont(font);
		firePropertyChange("unitsFont", oldFont, font);
	}
	
	public Font getUnitsFont() {
		return getNavigationBar().getFont();
	}
	
}

/* __oOo__ */
