package com.cosylab.gui.components;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.Customizer;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.ButtonModel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;

import com.cosylab.gui.components.range2.RangedValuePolicy;
import com.cosylab.gui.components.range2.RescalingValuePolicy;
import com.cosylab.gui.components.range2.ShiftValuePolicy;
import com.cosylab.gui.components.range2.TrimValuePolicy;

public class ValuePolicyCustomizer extends JPanel implements Customizer
{
	private static final long serialVersionUID = 1L;
	private ButtonGroup bg;
	private DialKnob displayer;
	private JRadioButton rescalingButton;
	private JRadioButton shiftButton;
	private JRadioButton trimButton;

	private RangedValuePolicy policy;
	/**
	 * Creates a new WPanel object.
	 */
	public ValuePolicyCustomizer()
	{
		bg = new ButtonGroup();

		trimButton = new JRadioButton("Trim Value Range");
		rescalingButton = new JRadioButton("Rescale Value Range");
		shiftButton = new JRadioButton("Shift Value Range");

		bg.add(trimButton);
		bg.add(rescalingButton);
		bg.add(shiftButton);

		setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
		add(trimButton);
		add(rescalingButton);
		add(shiftButton);

		ActionListener l= new ActionListener() {
		
			public void actionPerformed(ActionEvent e) {
				applySettings();
			}
		};
		
		trimButton.addActionListener(l);
		rescalingButton.addActionListener(l);
		shiftButton.addActionListener(l);
		
	}
	
	/* (non-Javadoc)
	 * @see java.beans.Customizer#setObject(java.lang.Object)
	 */
	public void setObject(Object bean) {
		displayer= (DialKnob)bean;
		displayer.addPropertyChangeListener("valuePolicy",new PropertyChangeListener() {
			public void propertyChange(PropertyChangeEvent evt) {
				if (!"valuePolicy".equals(evt.getPropertyName())) {
					return;
				}
				setValuePolicy(displayer.getValuePolicy());
			}
		
		});
		setValuePolicy(displayer.getValuePolicy());
	}


	/**
	 * DOCUMENT ME!
	 *
	 * @throws AssertionError DOCUMENT ME!
	 */
	public void applySettings()
	{
		ButtonModel bm = bg.getSelection();

		if (bm == null) {
			return;
		}
		RangedValuePolicy policy = null;
		if (bm == shiftButton.getModel()) {
			policy = new ShiftValuePolicy();
		} else if (bm == rescalingButton.getModel()) {
			policy = new RescalingValuePolicy();
		} else if (bm == trimButton.getModel()) {
			policy = new TrimValuePolicy();
		} else {
			throw new AssertionError();
		}
		
		displayer.setValuePolicy(policy);
		
		firePropertyChange("valuePolicy", null, policy);

	}
	
	public void setValuePolicy(RangedValuePolicy policy) {
		if (displayer != null) {
			if (policy instanceof RescalingValuePolicy && !rescalingButton.isSelected()) {
				rescalingButton.setSelected(true);
			} else if (policy instanceof TrimValuePolicy && !trimButton.isSelected()) {
				trimButton.setSelected(true);
			} else if (policy instanceof ShiftValuePolicy && !shiftButton.isSelected()) {
				shiftButton.setSelected(true);
			}
		}
		this.policy = policy;
		firePropertyChange("valuePolicy", null, policy);		
	}
	
	public RangedValuePolicy getValuePolicy() {
		return policy;
	}
}