package com.cosylab.gui.components.customizer;

import java.beans.BeanInfo;
import java.beans.Customizer;
import java.beans.IntrospectionException;
import java.beans.Introspector;
import java.beans.PropertyDescriptor;
import java.util.ArrayList;
import java.util.HashMap;

/**
 * Provides methods to simplifly creation of Customizers.
 * 
 * @author Alen Vrecko
 * 
 */
public class CustomizerFactory {

	/**
	 * Returns a simple customizer for the provided properties.
	 * 
	 * @param properties
	 * @param type
	 * @return
	 */
	public static Customizer createCustomizer(String[] properties) {
		return new PropertiesTableCustomizer(properties);
	}

	/**
	 * Returns a simple customizer for the provided properties.
	 * 
	 * @param properties
	 * @param type
	 * @return
	 */
	public static Customizer createCustomizer(PropertyDescriptor[] properties) {
		return new PropertiesTableCustomizer(properties);
	}

	/**
	 * Method to convert a set of string descriptior properties
	 * 
	 * @param properties
	 * @param type
	 * @return
	 */
	public static PropertyDescriptor[] propertyStringsToDescriptors(String[] properties,
			Class type) {
		if (properties == null || properties.length == 0) {

			return null;
		}

		HashMap<String, PropertyDescriptor> nameToDescriptor = new HashMap<String, PropertyDescriptor>();

		// populate the map with names <-> propertydscriptors
		try {
			BeanInfo beanInfo = Introspector.getBeanInfo(type);

			PropertyDescriptor[] propertyDescriptors = beanInfo.getPropertyDescriptors();

			for (PropertyDescriptor descriptor : propertyDescriptors) {
				nameToDescriptor.put(descriptor.getName(), descriptor);
			}
		} catch (IntrospectionException e) {

			return null;
		}

		// create array of PropertyDescriptors

		ArrayList<PropertyDescriptor> descriptors = new ArrayList<PropertyDescriptor>();

		PropertyDescriptor temp = null;

		for (String property : properties) {
			temp = nameToDescriptor.get(property);
			if (temp != null) {
				descriptors.add(temp);
			}
		}

		return descriptors.toArray(new PropertyDescriptor[] {});
	}
}
