/*
 * Copyright (c) 2003 by Cosylab d.o.o.
 *
 * The full license specifying the redistribution, modification, usage and other
 * rights and obligations is included with the distribution of this project in
 * the file license.html. If the license is not included you may find a copy at
 * http://www.cosylab.com/legal/abeans_license.htm or may write to Cosylab, d.o.o.
 *
 * THIS SOFTWARE IS PROVIDED AS-IS WITHOUT WARRANTY OF ANY KIND, NOT EVEN THE
 * IMPLIED WARRANTY OF MERCHANTABILITY. THE AUTHOR OF THIS SOFTWARE, ASSUMES
 * _NO_ RESPONSIBILITY FOR ANY CONSEQUENCE RESULTING FROM THE USE, MODIFICATION,
 * OR REDISTRIBUTION OF THIS SOFTWARE.
 */

package com.cosylab.gui.components.customizer;

import javax.swing.JComponent;


/**
 * <p>
 * <code>Editor</code> controls editor panel, which prowides GUI editor for
 * some aspect of displayer.
 * </p>
 * 
 * <p>
 * Implementator of this interface must provide defult constructor for
 * implementation class.
 * </p>
 *
 * @author <a href="mailto:igor.kriznar@cosylab.com">Igor Kriznar</a>
 * @author <a href="mailto:jernej.kamenik@cosylab.com">Jernej Kamenik</a>
 * @version $id$
 */
public interface Editor
{
	/** List of names of editable aspects with this editor.*/
	public static String[] ASPECTS = {  };

	/**
	 * Starts editing <b>session</b> and returns conponent, which will provide
	 * GUI for editing some <code>aspect</code>, given with  parameter,  of
	 * the given displayer. IF displayer is not design to edit given component
	 * in  given aspect, <code>IllegalArgumentException</code> is thrown.
	 *
	 * @param aspect the aspect in which displayer will be edited
	 *
	 * @return editor editor component
	 */
	JComponent getEditorComponent(Object object, String aspect)
		throws IllegalArgumentException;

	/**
	 * Applies current settings to the edited displayer.  This does not stop
	 * current editing session.
	 */
	void applySettings();

	/**
	 * Test if this editor can edit given displayer with given aspect.
	 *
	 * @param aspect the aspect in which displayer should be edited
	 *
	 * @return true if this editor can edit given displayer
	 */
	boolean canEdit(Object object, String aspect);

	/**
	 * Reverts editor values to currently used settings. This does not stop
	 * current editing session.
	 */
	void revertSettings();

	/**
	 * Stop current editing session and release edited displayer.
	 */
	void stopEditing();
}

/* __oOo__ */
