/*
 * Copyright (c) 2003 by Cosylab d.o.o.
 *
 * The full license specifying the redistribution, modification, usage and other
 * rights and obligations is included with the distribution of this project in
 * the file license.html. If the license is not included you may find a copy at
 * http://www.cosylab.com/legal/abeans_license.htm or may write to Cosylab, d.o.o.
 *
 * THIS SOFTWARE IS PROVIDED AS-IS WITHOUT WARRANTY OF ANY KIND, NOT EVEN THE
 * IMPLIED WARRANTY OF MERCHANTABILITY. THE AUTHOR OF THIS SOFTWARE, ASSUMES
 * _NO_ RESPONSIBILITY FOR ANY CONSEQUENCE RESULTING FROM THE USE, MODIFICATION,
 * OR REDISTRIBUTION OF THIS SOFTWARE.
 */

package com.cosylab.gui.components.gauger;

import java.awt.Graphics;
import java.awt.Rectangle;
import java.awt.geom.Point2D;


/**
 * Defines vertical linear transform.
 *
 * @author <a href="mailto:ales.pucelj@cosylab.com">Ales Pucelj</a>
 * @version $id$
 */
public class LinearVerticalTransform extends ScaleTransform
{
	private static final int MAXIMUM_WIDTH = 80;
	private static final int MINIMUM_WIDTH = 25;
	private static final double RELATIVE_X_OFFSET = 0.1;
	private static final double RELATIVE_Y_OFFSET = 0.05;
	private int width = 0;
	private int height = 0;
	private int xOffset = 0;
	private int yOffset = 0;
	private int tickOffset = 0;

	/**
	 * @see ScaleTransform#setParameters(int, int, int, int, int)
	 */
	public void setParameters(int w, int h, int marginX, int marginY,
	    int tickOffset)
	{
		this.tickOffset = tickOffset;

		width = w - (int)(2 * w * RELATIVE_X_OFFSET) - 5;
		height = h - (int)(2 * h * RELATIVE_Y_OFFSET) - 5;
		width = Math.min(width, MAXIMUM_WIDTH);
		width = Math.max(width, MINIMUM_WIDTH);

		xOffset = (int)(0.5 * (w - width));
		yOffset = (int)(0.5 * (h - height));

		Rectangle outline = new Rectangle(xOffset, yOffset, width, height);
		addSegment(new RectangleSegment(outline));

		setLabelPosition((int)(w / 2), (int)(0.5 * (h - height - yOffset)));
	}

	/**
	 * @see ScaleTransform#mapUVtoXY(PointDouble, Point)
	 */
	public void mapUVtoXY(Point2D scaleSpace, Point2D cartesianSpace)
	{
		double x = (xOffset + width * scaleSpace.getY());
		double y = (yOffset + scaleWidth(0.0) * (scaleSpace.getX())
			+ tickOffset);
		cartesianSpace.setLocation(x, y);
	}

	/**
	 * @see ScaleTransform#maxXYtoUV(Point, PointDouble)
	 */
	public void mapXYtoUV(Point2D cartesianSpace, Point2D scaleScape)
	{
	}

	/**
	 * @see ScaleTransform#scaleWidth(double)
	 */
	public double scaleWidth(double v)
	{
		return Math.abs(height - 2 * tickOffset);
	}

	/**
	 * @see ScaleTransform#scaleHeight(double)
	 */
	public double scaleHeight(double u)
	{
		return width;
	}

	/**
	 * @see com.cosylab.gui.components.gauger.ScaleTransform#measureTick(Graphics,
	 *      double, String)
	 */
	public int measureTick(Graphics g, double x, String text)
	{
		return g.getFontMetrics().getHeight();
	}
}

/* __oOo__ */
