/*
 * Copyright (c) 2003 by Cosylab d.o.o.
 *
 * The full license specifying the redistribution, modification, usage and other
 * rights and obligations is included with the distribution of this project in
 * the file license.html. If the license is not included you may find a copy at
 * http://www.cosylab.com/legal/abeans_license.htm or may write to Cosylab, d.o.o.
 *
 * THIS SOFTWARE IS PROVIDED AS-IS WITHOUT WARRANTY OF ANY KIND, NOT EVEN THE
 * IMPLIED WARRANTY OF MERCHANTABILITY. THE AUTHOR OF THIS SOFTWARE, ASSUMES
 * _NO_ RESPONSIBILITY FOR ANY CONSEQUENCE RESULTING FROM THE USE, MODIFICATION,
 * OR REDISTRIBUTION OF THIS SOFTWARE.
 */

package com.cosylab.gui.components.gauger;

import java.awt.geom.Point2D;


/**
 * Defines polar transform for angles larger than 180 degrees, but less than
 * 360. Minimum will be at lower left and maximum at lower right. 
 *
 * @author <a href="mailto:ales.pucelj@cosylab.com">Ales Pucelj</a>
 * @version $id$
 */
public class PolarFullTransform extends PolarTransform
{
	private static final int MAXIMUM_HEIGHT = 60;
	private static final int MINIMUM_HEIGHT = 35;
	private static final double RELATIVE_OFFSET = 0.9;
	private static final double RELATIVE_SCALE_SIZE_COMPARED_TO_SIZE = 0.15;

	/*
	 * @see ScaleTransform#setParameters(int, int, int, int, int)
	 */
	public void setParameters(int w, int h, int marginX, int marginY,
	    int tickOffset)
	{
		super.setParameters(w, h, marginX, marginY, tickOffset);

		double size = (Math.min(w, h * 1.171573411) * RELATIVE_OFFSET);

		R = size / 2.0;
		centre = new Point2D.Double(0.5 * w, 0.5 * h + (R * 0.17));

		h = (int)(h * RELATIVE_SCALE_SIZE_COMPARED_TO_SIZE);
		h = Math.min(h, MAXIMUM_HEIGHT);
		h = Math.max(h, MINIMUM_HEIGHT);
		height = h;

		r = R - h;

		alpha = 225.0;
		span = 270.0;

		setLabelPosition((int)centre.getX(), (int)centre.getY());

		//		this.tickOffset = PolarPoint.degToRad(tickOffset) / r;
		this.tickOffset = Math.toDegrees(tickOffset) / r;

		ArcSegment segment = new ArcSegment(centre, r, R, alpha, span);
		addSegment(segment);
	}
}

/* __oOo__ */
