/*
 * Copyright (c) 2003 by Cosylab d.o.o.
 *
 * The full license specifying the redistribution, modification, usage and other
 * rights and obligations is included with the distribution of this project in
 * the file license.html. If the license is not included you may find a copy at
 * http://www.cosylab.com/legal/abeans_license.htm or may write to Cosylab, d.o.o.
 *
 * THIS SOFTWARE IS PROVIDED AS-IS WITHOUT WARRANTY OF ANY KIND, NOT EVEN THE
 * IMPLIED WARRANTY OF MERCHANTABILITY. THE AUTHOR OF THIS SOFTWARE, ASSUMES
 * _NO_ RESPONSIBILITY FOR ANY CONSEQUENCE RESULTING FROM THE USE, MODIFICATION,
 * OR REDISTRIBUTION OF THIS SOFTWARE.
 */

package com.cosylab.gui.components.gauger;

import java.awt.Graphics;
import java.awt.geom.Point2D;


/**
 * Base class for polar transforms. All polar transforms will be rendered using
 * scale in the shape of arc.
 *
 * @author <a href="mailto:ales.pucelj@cosylab.com">Ales Pucelj</a>
 * @version $id$
 */
public abstract class PolarTransform extends ScaleTransform
{
	protected Point2D centre;
	protected double R;
	protected double r;
	protected double alpha;
	protected double span;
	protected int height;
	protected double tickOffset;

	/*
	 * @see ScaleTransform#setParameters(int, int, int, int, int)
	 */
	public void setParameters(int w, int h, int marginX, int marginY,
	    int tickOffset)
	{
		setPolar(true);
	}

	/*
	 * @see ScaleTransform#mapUVtoXY(PointDouble, Point)
	 */
	public void mapUVtoXY(Point2D scaleSpace, Point2D cartesianSpace)
	{
		double phi = alpha
			- (tickOffset + scaleSpace.getX() * (span - 2.0 * tickOffset));

		PolarPoint.polarToCartesian(cartesianSpace,
		    r + scaleSpace.getY() * height, phi);

		cartesianSpace.setLocation(cartesianSpace.getX() + centre.getX(),
		    cartesianSpace.getY() + centre.getY());
	}

	/*
	 * @see ScaleTransform#maxXYtoUV(Point, PointDouble)
	 */
	public void mapXYtoUV(Point2D cartesianSpace, Point2D scaleScape)
	{
		// not implemented
	}

	/*
	 * @see ScaleTransform#scaleWidth(double)
	 */
	public double scaleWidth(double v)
	{
		return Math.abs(((r + v * height) * Math.PI * (span - 2 * tickOffset) / 180.0));
	}

	/*
	 * @see ScaleTransform#scaleHeight(double)
	 */
	public double scaleHeight(double u)
	{
		return height;
	}

	/**
	 * @see com.cosylab.gui.components.gauger.ScaleTransform#measureTick(Graphics,
	 *      double, String)
	 */
	public int measureTick(Graphics g, double x, String text)
	{
		return g.getFontMetrics().stringWidth(text);
	}
}

/* __oOo__ */
