/*
 * Copyright (c) 2003 by Cosylab d.o.o.
 *
 * The full license specifying the redistribution, modification, usage and other
 * rights and obligations is included with the distribution of this project in
 * the file license.html. If the license is not included you may find a copy at
 * http://www.cosylab.com/legal/abeans_license.htm or may write to Cosylab, d.o.o.
 *
 * THIS SOFTWARE IS PROVIDED AS-IS WITHOUT WARRANTY OF ANY KIND, NOT EVEN THE
 * IMPLIED WARRANTY OF MERCHANTABILITY. THE AUTHOR OF THIS SOFTWARE, ASSUMES
 * _NO_ RESPONSIBILITY FOR ANY CONSEQUENCE RESULTING FROM THE USE, MODIFICATION,
 * OR REDISTRIBUTION OF THIS SOFTWARE.
 */

package com.cosylab.gui.components.introspection;

import com.cosylab.introspection.BeanIntrospector;

import java.beans.IntrospectionException;

import java.lang.reflect.InvocationTargetException;

import java.util.ArrayList;


/**
 * <code>BeanPropertiesTableModel</code> is table model which displayes
 * properties of a bean and allows property manipulation based on
 * introspection. This implementationis build around
 * <code>PropertiesTableModel</code> with help of
 * <code>BeanIntrospector</code>.
 * 
 * <p>
 * When value in table is changed, new value is only stored in model. To set
 * all property values to bean use <code>applyProperties</code> method.
 * </p>
 *
 * @author <a href="mailto:igor.kriznar@cosylab.com">Igor Kriznar</a>
 * @version $Id: BeanPropertiesTableModel.java,v 1.1 2004/06/28 02:03:28 ikriznar Exp $
 *
 * @see com.cosylab.gui.components.introspection.PropertiesTableModel
 * @see com.cosylab.introspection.BeanIntrospector
 * @since May 19, 2004.
 */
public class BeanPropertiesTableModel extends PropertiesTableModel
{
	private Object bean;

	/**
	 * Default constructor creates empty model.
	 */
	public BeanPropertiesTableModel()
	{
		super();
	}

	/**
	 * Creates new model with specifies bean and property names, which model
	 * will display in table and manipulate
	 *
	 * @param bean
	 * @param names
	 */
	public BeanPropertiesTableModel(Object bean, String[] names)
	{
		super();

		this.bean = bean;

		ArrayList typesList = new ArrayList();
		ArrayList valuesList = new ArrayList();

		for (int i = 0; i < names.length; i++) {
			try {
				Object value = BeanIntrospector.getPropertyValue(bean, names[i]);
				Class type = BeanIntrospector.getPropertyType(bean.getClass(),
					    names[i]);
				valuesList.add(value);
				typesList.add(type);
			} catch (Exception e) {
				e.printStackTrace();
			}
		}

		Class[] types = new Class[typesList.size()];
		typesList.toArray(types);

		Object[] values = valuesList.toArray();

		setPropertyNames(names);
		setPropertyTypes(types);
		setPropertyValues(values);
	}

	/**
	 * Sets all in model stored property values to the bean.
	 *
	 * @throws IllegalArgumentException if introspection manipulation failed
	 * @throws IllegalAccessException if introspection manipulation failed
	 * @throws InvocationTargetException if introspection manipulation failed
	 * @throws NoSuchMethodException if introspection manipulation failed
	 * @throws IntrospectionException if introspection manipulation failed
	 * @throws NullPointerException if bean was not set
	 */
	public void applyProperties()
		throws IllegalArgumentException, IllegalAccessException, 
			InvocationTargetException, NoSuchMethodException, 
			IntrospectionException
	{
		if (bean == null) {
			throw new NullPointerException("No bean object specified in model.");
		}

		String[] names = getPropertyNames();
		Object[] values = getPropertyValues();

		for (int j = 0; j < names.length; j++) {
			BeanIntrospector.setPropertyValue(bean, names[j], values[j]);

			values[j] = BeanIntrospector.getPropertyValue(bean, names[j]);
		}

		setPropertyValues(values);
		fireTableDataChanged();
	}

	/**
	 * Returns bean for which properties apply.
	 *
	 * @return bean for which properties apply
	 */
	public Object getBean()
	{
		return bean;
	}

	/**
	 * Sets the bean for which properties apply.
	 *
	 * @param bean new bean object
	 */
	public void setBean(Object bean)
	{
		this.bean = bean;
	}
}

/* __oOo__ */
