/*
 * @@COPYRIGHT@@
 */
package com.cosylab.gui.components.introspection;

import java.lang.reflect.Method;


/**
 * A class that holds the necessary information for invoking 
 * a method on a class (if the method is static) or an object.
 * 
 * @author <a href="mailto:miha.kadunc@cosylab.com">Miha Kadunc</a>
 * @version $id$
 */
public class MethodInvocationSettings {
	private Method method=null;
	private Object[] parameters=null;
	private Object object=null;
	private boolean staticMethod=false;
	
	/**
	 * @param method Method
	 */
	public MethodInvocationSettings(Method method){
		super();
		setMethod(method);
	}
	private boolean checkObject(){
		if (isStaticMethod()) return true;
		if (method.getDeclaringClass().isAssignableFrom(object.getClass())) return true;
		return false;
	}
	private boolean checkParameters(){
		if (method==null) return false;
		Class[] paramTypes=method.getParameterTypes();
		if (paramTypes.length!=parameters.length) return false;
		for (int i=0; i<parameters.length;i++){
			if (!paramTypes[i].isAssignableFrom(parameters[i].getClass())) return(false);
		}
		return true;
	}
	/**
	 * Gets the method
	 * @return Returns a Method
	 */
	public Method getMethod() {
		return method;
	}
	/**
	 * Gets the object
	 * @return Returns a Object
	 */
	public Object getObject() {
		return object;
	}
	/**
	 * Gets the parameters
	 * @return Returns a String[]
	 */
	public Object[] getParameters() {
		return parameters;
	}
	/**
	 * Gets the staticMethod
	 * @return Returns a boolean
	 */
	public boolean isStaticMethod() {
		return staticMethod;
	}
	/**
	 * Sets the method
	 * @param method The method to set
	 */
	
	public void setMethod(Method method){
		this.method=method;
		staticMethod=java.lang.reflect.Modifier.isStatic(method.getModifiers());
	}
	/**
	 * Sets the object
	 * @param object The object to set
	 */
	public void setObject(Object object) {
		this.object = object;
	}
	/**
	 * Sets the parameters
	 * @param parameters The parameters to set
	 */
	public void setParameters(Object[] parameters) {
		this.parameters = parameters;
	}
	/**
	 * @return String
	 */
	public String toString(){
		String className=method.getClass().getName();
		StringBuffer b=new StringBuffer(className.substring(className.lastIndexOf(".")));
		b.append(".");
		b.append(method.getName());
		b.append("(");
		b.append(")");
		if (staticMethod) b.append("<S>");
		return b.toString();
	}
}
