/*
 * Copyright (c) 2006 by Cosylab d.o.o.
 *
 * The full license specifying the redistribution, modification, usage and other
 * rights and obligations is included with the distribution of this project in
 * the file license.html. If the license is not included you may find a copy at
 * http://www.cosylab.com/legal/abeans_license.htm or may write to Cosylab, d.o.o.
 *
 * THIS SOFTWARE IS PROVIDED AS-IS WITHOUT WARRANTY OF ANY KIND, NOT EVEN THE
 * IMPLIED WARRANTY OF MERCHANTABILITY. THE AUTHOR OF THIS SOFTWARE, ASSUMES
 * _NO_ RESPONSIBILITY FOR ANY CONSEQUENCE RESULTING FROM THE USE, MODIFICATION,
 * OR REDISTRIBUTION OF THIS SOFTWARE.
 */

package com.cosylab.gui.components.introspection;

import com.cosylab.introspection.BeanIntrospector;
import com.cosylab.introspection.ClassIntrospector;

import java.awt.BorderLayout;

import java.beans.IntrospectionException;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.beans.PropertyDescriptor;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;


/**
 * This table parforms introspection on Java bean and displays editable table
 * in which user can change bean properties.
 *
 * @author <a href="mailto:miha.kadunc@cosylab.com">Miha Kadunc</a>
 * @version $id$
 */
public class PropertiesTable extends DefaultPropertiesTable
{
	private String[] propertyNames;
	private PropertyDescriptor[] propertyDescriptors;
	private Object fieldBean = null;

	/**
	 * ParametersTable constructor comment.
	 */
	public PropertiesTable()
	{
		super();
		initialize();
	}

	/**
	 * ParametersTable constructor comment.
	 *
	 * @param properties DOCUMENT ME!
	 */
	public PropertiesTable(String[] properties)
	{
		super();
		initialize();
		propertyNames = properties;
	}
	
	/**
	 * ParametersTable constructor comment.
	 *
	 * @param properties DOCUMENT ME!
	 */
	public PropertiesTable(PropertyDescriptor[] properties)
	{
		super();
		initialize();
		propertyDescriptors= properties;
		propertyNames = BeanIntrospector.getPropertyNames(properties);
	}
	
	

	/**
	 * Gets the bean property value.
	 *
	 * @return The bean property value.
	 *
	 * @see #setMethod
	 */
	public Object getBean()
	{
		return fieldBean;
	}

	/**
	 * Initialize the class.
	 *
	 * @throws RuntimeException DOCUMENT ME!
	 */
	private void initialize()
	{
		final PropertiesTableModel parametersModel = (PropertiesTableModel)getModel();
		parametersModel.addTableModelListener(new TableModelListener() {
				public void tableChanged(TableModelEvent e)
				{
					if (e.getColumn() == 1) {
						try {
							String name = parametersModel.getPropertyNames()[e
								.getFirstRow()];
							Object val = parametersModel.getPropertyValues()[e
								.getFirstRow()];

							/*firePropertyChange is necessary for NetBeans gui builder*/
							firePropertyChange(name, null, val);
							BeanIntrospector.setPropertyValue(fieldBean, name,
							    val);
						} catch (Exception exc) {
							throw new RuntimeException("Could not automatically set the property on the bean",
							    exc);
						}
					}
				}
			});
	}

	/**
	 * main entrypoint - starts the part when it is run as an application
	 *
	 * @param args java.lang.String[]
	 */
	public static void main(java.lang.String[] args)
	{
		try {
			JFrame frame = new JFrame();
			JPanel panel = new JPanel();
			JScrollPane sp = new JScrollPane();
			PropertiesTable aParametersTable;
			aParametersTable = new PropertiesTable();

			sp.add(aParametersTable);
			sp.setViewportView(aParametersTable);
			panel.setLayout(new BorderLayout());
			panel.add(sp, BorderLayout.CENTER);

			JButton but = new JButton();
			panel.add(but, BorderLayout.NORTH);
			frame.setContentPane(panel);
			frame.setSize(200, 100);
			frame.addWindowListener(new java.awt.event.WindowAdapter() {
					public void windowClosing(java.awt.event.WindowEvent e)
					{
						System.exit(0);
					}
					;
				});
			frame.setVisible(true);
			aParametersTable.setBean(but);

			java.awt.Insets insets = frame.getInsets();
			frame.setSize(frame.getWidth() + insets.left + insets.right,
			    frame.getHeight() + insets.top + insets.bottom);
			frame.setVisible(true);
		} catch (Throwable exception) {
			System.err.println(
			    "Exception occurred in main() of com.cosylab.introspection.ParametersTable");
			exception.printStackTrace(System.out);
		}
	}

	/**
	 * Sets the method property (java.lang.reflect.Method) value.
	 *
	 * @param bean The new value for the property.
	 *
	 * @throws IntrospectionException DOCUMENT ME!
	 * @throws IllegalAccessException DOCUMENT ME!
	 * @throws InvocationTargetException DOCUMENT ME!
	 *
	 * @see #getMethod
	 */
	public void setBean(Object bean)
		throws IntrospectionException, IllegalAccessException, 
			InvocationTargetException
	{

		if (propertyNames != null) {
			if (propertyDescriptors==null) {
				propertyDescriptors = new PropertyDescriptor[propertyNames.length];
	
				for (int i = 0; i < propertyNames.length; i++) {
					propertyDescriptors[i] = BeanIntrospector.getPropertyDescriptor(bean
						    .getClass(), propertyNames[i]);
					if (propertyDescriptors[i]==null) {
						propertyDescriptors =null;
						throw new IntrospectionException("Bean "+bean.getClass().getName()+" does not contain property "+propertyNames[i]);
					}
				}
			}
		} else {
			PropertyDescriptor[] preProps = BeanIntrospector
				.getPropertyDescriptors(bean.getClass());
			propertyDescriptors = BeanIntrospector.getReadable(BeanIntrospector
				    .getWritable(preProps));
			propertyNames = BeanIntrospector.getPropertyNames(propertyDescriptors);
		}

		final PropertiesTableModel m = (PropertiesTableModel)getModel();
		m.setPropertyNames(propertyNames);
		m.setPropertyTypes(BeanIntrospector.getPropertyTypes(
		        propertyDescriptors));
		m.setPropertyValues(BeanIntrospector.getPropertyValues(
		        propertyDescriptors, bean));

		Method pcm = ClassIntrospector.getMethod(bean.getClass(),
			    "addPropertyChangeListener",
			    new Class[]{ PropertyChangeListener.class });

		if (pcm != null) {
			pcm.invoke(bean,
			    new Object[]{
				    new PropertyChangeListener() {
						public void propertyChange(PropertyChangeEvent evt)
						{
							if (m.containsProperty(evt.getPropertyName())) {
								m.setValueForProperty(evt.getPropertyName(),
								        evt.getNewValue());
							}
						}
					}
			    });
		}

		Object oldValue = fieldBean;
		fieldBean = bean;
		firePropertyChange("bean", oldValue, fieldBean);
		revalidate();
	}

	/**
	 * Returns the propertyDescriptors.
	 *
	 * @return Returns the propertyDescriptors.
	 */
	public PropertyDescriptor[] getPropertyDescriptors()
	{
		return propertyDescriptors;
	}

	/**
	 * Returns the propertyNames.
	 *
	 * @return Returns the propertyNames.
	 */
	public String[] getPropertyNames()
	{
		return propertyNames;
	}
}

/* __oOo__ */
