/*
 * @@COPYRIGHT@@
 */
package com.cosylab.gui.components.ledder;
import java.awt.Color;

/**
 * Interface representing the led model. This interface provides minimum
 * requirements neccessary to uniquely define the model of leds. In this
 * definition the model is immutable, although this is not a requirement. 
 * Implementations should provide methods for modifying the contents during
 * runtime and fire appropriate events.
 * 
 * @author <a href="mailto:ales.pucelj@cosylab.com">Ales Pucelj</a>
 * @version $id$
 */
public interface BitDescriptor {

	/**
	 * Returns description for the led at specified index or null if
	 * such led does not exist.
	 * 
	 * @param index int
	 * @return String
	 */
	String getDescription(int index);

	/**
	 * Returns color for the led at specified index or null if
	 * such led does not exist. This is the color that will be displayed when 
	 * the led indicates ON state.
	 * @param index int
	 * @return Color
	 */
	Color getColorWhenOn(int index);

	/**
	 * Returns color for the led at specified index or null if such led does not
	 * exist. This is the color that will be displayed when the led indicates 
	 * OFF state.
	 * 
	 * @param index int
	 * @return Color
	 */
	Color getColorWhenOff(int index);
}
