/*
 * @@COPYRIGHT@@
 */
package com.cosylab.gui.components.ledder;
import java.awt.Color;
import java.util.HashMap;

import javax.swing.Icon;

/**
 * Creates LED icon for given color.
 *  
 * @author <a href="mailto:ales.pucelj@cosylab.com">Ales Pucelj</a>
 * @version $id$
 */
public class LedIconFactory {

	private static HashMap cache;
	
	
	/**
	 * Returns led icon from cache. New instance will be created if it does not
	 * exist already.
	 * 
	 * @param color Color of led.
	 * @return Instance of led icon.
	 */
	private static final Icon getCached(Color color) {
		if (cache == null) {
			cache = new HashMap();
		}
		Icon ic = (Icon)cache.get(color);
		if (ic == null) {
			ic = new CustomLedIcon(color);
			cache.put(color, ic);
		}
		return ic;
	}

	/**
	 * Creates the icon from <code>java.awt.Color</code>. See the class overview
	 * for details.
	 * 
	 * @param color color of the icon.
	 * @return Icon always an instance of an icon 
	 */
	public static Icon createIcon(Color color) {
		return getCached(color);
	}

}