/*
 * Copyright (c) 2004 by Cosylab d.o.o.
 *
 * The full license specifying the redistribution, modification, usage and other
 * rights and obligations is included with the distribution of this project in
 * the file license.html. If the license is not included you may find a copy at
 * http://www.cosylab.com/legal/abeans_license.htm or may write to Cosylab, d.o.o.
 *
 * THIS SOFTWARE IS PROVIDED AS-IS WITHOUT WARRANTY OF ANY KIND, NOT EVEN THE
 * IMPLIED WARRANTY OF MERCHANTABILITY. THE AUTHOR OF THIS SOFTWARE, ASSUMES
 * _NO_ RESPONSIBILITY FOR ANY CONSEQUENCE RESULTING FROM THE USE, MODIFICATION,
 * OR REDISTRIBUTION OF THIS SOFTWARE.
 */

package com.cosylab.gui.components.numberfield;

import com.cosylab.logging.DebugLogger;

import com.cosylab.util.PrintfFormat;
import com.cosylab.util.StringUtilities;

import java.util.logging.Level;
import java.util.logging.Logger;


/**
 * DOCUMENT ME!
 *
 * @author $Author: ikriznar $
 * @version $Revision: 1.5 $
 */
public class DefaultNumberDescriptor implements NumberDescriptor
{
	/**
	 * @see NumberDescriptor
	 */
	public final static String NAME = "Default Visualization";

	/**
	 * @see NumberDescriptor
	 */
	public final static String DESCRIPTION = "Prints value to string according to ANSI format string.";
	private final Logger logger = DebugLogger.getLogger("DND", Level.OFF);
	private String format;
	private Class type = Double.class;
	private PrintfFormat formatter;

	/* (non-Javadoc)
	 * @see com.cosylab.gui.components.numberfield.NumberDescriptor#parseNumber(java.lang.String)
	 */
	public Number parseNumber(String number)
	{
		try {
			if (type == Long.class) {
				if ("-".equals(number.trim())) {
					return new Long(0);
				}

				logger.fine("Long '" + number + "'");

				double d = StringUtilities.parseDouble(number);

				if (Double.isNaN(d)) {
					return null;
				}

				return new Long((long)d);
			}

			double newValue = StringUtilities.parseDouble(number);

			if (Double.isInfinite(Math.abs(newValue))) {
				logger.fine("Infinite '" + number + "'");

				return null;
			}

			if (Double.isNaN(newValue)) {
				logger.fine("NaN '" + number + "'");

				return null;
			}

			logger.fine("Dobule '" + number + "' -> " + newValue);

			return new Double(newValue);
		} catch (Exception e) {
			// ignored
			logger.fine("Exception " + e);
		}

		logger.fine("Plain failed '" + number + "'");

		return null;
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.components.numberfield.NumberDescriptor#toString(java.lang.Number)
	 */
	public String printString(Number number)
	{
		String s = formatter.sprintf(number).replace(',', '.');
		logger.fine("'" + number + "' -> '" + s + "'");

		return s;
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.components.numberfield.NumberDescriptor#toEditString(java.lang.Number)
	 */
	public String printEditString(Number number)
	{
		String s = printString(number).trim();

		if (s.charAt(0) == '+') {
			s = s.substring(1);
		}

		logger.fine("'" + number + "' -> '" + s + "'");
		return s;
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.components.numberfield.NumberDescriptor#getFormat()
	 */
	public String getFormat()
	{
		return format;
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.components.numberfield.NumberDescriptor#setFormat(java.lang.String)
	 */
	public void setFormat(String format)
	{
		logger.info("newFormat '" + format + "'");
		this.format = format;
		formatter = new PrintfFormat(format);
	}

	/**
	 * Supported types are <code>Double.class</code> and
	 * <code>Long.class</code>.
	 *
	 * @see com.cosylab.gui.components.numberfield.NumberDescriptor#setNumberType(java.lang.Class)
	 */
	public void setNumberType(Class newClass)
	{
		logger.info("newClass '" + newClass + "'");

		if (newClass != Double.class && newClass != Long.class) {
			throw new IllegalArgumentException("Type '" + newClass
			    + "' not supported.");
		}

		type = newClass;
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.components.numberfield.NumberDescriptor#getNumberType()
	 */
	public Class getNumberType()
	{
		return type;
	}
}

/* __oOo__ */
