/*
 * Copyright (c) 2004 by Cosylab d.o.o.
 *
 * The full license specifying the redistribution, modification, usage and other
 * rights and obligations is included with the distribution of this project in
 * the file license.html. If the license is not included you may find a copy at
 * http://www.cosylab.com/legal/abeans_license.htm or may write to Cosylab, d.o.o.
 *
 * THIS SOFTWARE IS PROVIDED AS-IS WITHOUT WARRANTY OF ANY KIND, NOT EVEN THE
 * IMPLIED WARRANTY OF MERCHANTABILITY. THE AUTHOR OF THIS SOFTWARE, ASSUMES
 * _NO_ RESPONSIBILITY FOR ANY CONSEQUENCE RESULTING FROM THE USE, MODIFICATION,
 * OR REDISTRIBUTION OF THIS SOFTWARE.
 */

package com.cosylab.gui.components.numberfield;

import com.cosylab.util.StringUtilities;


/**
 * This is the implementation of the <code>AbstractNumberDocument</code> class
 * implementing the floating point parser.
 *
 * @author <a href="mailto:ales.pucelj@cosylab.com">Ales Pucelj</a>
 * @version $id$
 */
public class DoubleDocument extends AbstractNumberDocument
{
	/**
	 * Constructor for DoubleDocument.
	 */
	public DoubleDocument()
	{
		super();
	}

	/**
	 * Parses the string as a floating point value.
	 *
	 * @param s String
	 *
	 * @return Number
	 *
	 * @see AbstractNumberDocument#parseNumber(String)
	 */
	protected Number parseNumber(String s)
	{
        if (s==null || s.length()<1) {
            return null;
        }
		double newValue;

		newValue = StringUtilities.parseDouble(s);

		if (Double.isInfinite(Math.abs(newValue))) {
			return null;
		}

		if (Double.isNaN(newValue)) {
			return null;
		}

		return new Double(newValue);
	}
}

/* __oOo__ */
