/*
 * Copyright (c) 2004 by Cosylab d.o.o.
 *
 * The full license specifying the redistribution, modification, usage and other
 * rights and obligations is included with the distribution of this project in
 * the file license.html. If the license is not included you may find a copy at
 * http://www.cosylab.com/legal/abeans_license.htm or may write to Cosylab, d.o.o.
 *
 * THIS SOFTWARE IS PROVIDED AS-IS WITHOUT WARRANTY OF ANY KIND, NOT EVEN THE
 * IMPLIED WARRANTY OF MERCHANTABILITY. THE AUTHOR OF THIS SOFTWARE, ASSUMES
 * _NO_ RESPONSIBILITY FOR ANY CONSEQUENCE RESULTING FROM THE USE, MODIFICATION,
 * OR REDISTRIBUTION OF THIS SOFTWARE.
 */

package com.cosylab.gui.components.numberfield;

/**
 * <p>
 * Implementation of this interface provides conversion of a
 * <code>Number</code> to <code>String</code> and back. This interface acts as
 * kind of interpretation of a <code>Number</code> with bijective
 * transformation to string.
 * </p>
 * 
 * <p>
 * This interfaces provides more flexible visualization of number as string
 * than is available with standard ANSI C kind of formatting. For example:
 * display and edit double value in hh:mm:ss format (interpret it as degrees).
 * </p>
 * 
 * <p>
 * It is advised for the descriptor implementation to define two final static String fields: <code>NAME</code> and <code>DESCRIPTION</code>. First should be short descriptive name and second should be short description. These strings are used in GUI choosers for NumberField visualization. 

 * </p>
 *
 * @author <a href="mailto:igor.kriznar@cosylab.com">Igor Kriznar</a>
 */
public interface NumberDescriptor
{
	/**
	 * Parses string and creates <code>Number</code>. If string is not
	 * presentation of a number and can not be parsed, then null must be
	 * returned.
	 *
	 * @param number a string representing a number
	 *
	 * @return instance of <code>Number</code> or <code>null</code> if string
	 *         can not be parsed
	 */
	public Number parseNumber(String number);

	/**
	 * Converts provided <code>Number</code> to a string, which is used for
	 * displaying the value.
	 *
	 * @param number a number to be transformed to string
	 *
	 * @return a <code>String</code> representing a number.
	 */
	public String printString(Number number);

	/**
	 * Converts provided <code>Number</code> to a string, which is displayed
	 * when NumberField enters in to edit mode and wants to edit value. Edit
	 * string is usually stripped from unnecessary characters and prepared for
	 * easyer user editing.
	 *
	 * @param number a number to be transformed to string
	 *
	 * @return a <code>String</code> representation a number prepared for
	 *         editing.
	 */
	public String printEditString(Number number);

	/**
	 * Returns the format
	 *
	 * @return Format
	 */
	public String getFormat();

	/**
	 * Set the Number format
	 *
	 * @param format
	 */
	public void setFormat(String format);

	/**
	 * Sets number type to descriptor, ame as used on number field.
	 *
	 * @param newClass number type
	 */
	public void setNumberType(Class newClass);

	/**
	 * Returns last set number type.
	 *
	 * @return last set number type
	 */
	public Class getNumberType();

}

/* __oOo__ */
