/*
 * Copyright (c) 2004 by Cosylab d.o.o.
 *
 * The full license specifying the redistribution, modification, usage and other
 * rights and obligations is included with the distribution of this project in
 * the file license.html. If the license is not included you may find a copy at
 * http://www.cosylab.com/legal/abeans_license.htm or may write to Cosylab, d.o.o.
 *
 * THIS SOFTWARE IS PROVIDED AS-IS WITHOUT WARRANTY OF ANY KIND, NOT EVEN THE
 * IMPLIED WARRANTY OF MERCHANTABILITY. THE AUTHOR OF THIS SOFTWARE, ASSUMES
 * _NO_ RESPONSIBILITY FOR ANY CONSEQUENCE RESULTING FROM THE USE, MODIFICATION,
 * OR REDISTRIBUTION OF THIS SOFTWARE.
 */

package com.cosylab.gui.components.numberfield;

/**
 * Visualize double value in radians by converting it to degrees and displays
 * it in format XX\u00B0XX'XX\".
 *
 * @author <a href="mailto:igor.kriznar@cosylab.com">Igor Kriznar</a>
 */
public class Rad2DegNumberDescriptor extends AngleNumberDescriptor
{
	/**
	 * @see NumberDescriptor
	 */
	public final static String NAME = "Radians as Degrees Visualization";

	/**
	 * @see NumberDescriptor
	 */
	public final static String DESCRIPTION = "Converts double value from radians to degrees and diplays in degree format: XX\u00B0XX'XX\".";
	/**
		 *
		 */
	public Rad2DegNumberDescriptor()
	{
		super();
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @param full
	 */
	public Rad2DegNumberDescriptor(boolean full)
	{
		super(full);
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @param degreeSeparator
	 * @param minuteSeparator
	 * @param secondSeparator
	 */
	public Rad2DegNumberDescriptor(char degreeSeparator, char minuteSeparator,
	    char secondSeparator)
	{
		super(degreeSeparator, minuteSeparator, secondSeparator);
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @param degreeSeparator
	 * @param minuteSeparator
	 * @param secondSeparator
	 * @param full
	 */
	public Rad2DegNumberDescriptor(char degreeSeparator, char minuteSeparator,
	    char secondSeparator, boolean full)
	{
		super(degreeSeparator, minuteSeparator, secondSeparator, full);
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.components.numberfield.AngleNumberDescriptor#printString(java.lang.Number)
	 */
	public String printString(Number number)
	{
		return super.printString(new Double(Math.toDegrees(number.doubleValue())));
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.components.numberfield.AngleNumberDescriptor#parseNumber(java.lang.String)
	 */
	public Number parseNumber(String number)
	{
		Number n = super.parseNumber(number);

		if (n == null) {
			return n;
		}

		return new Double(Math.toRadians(n.doubleValue()));
	}
}

/* __oOo__ */
