/*
 * Copyright (c) 2004 by Cosylab d.o.o.
 *
 * The full license specifying the redistribution, modification, usage and other
 * rights and obligations is included with the distribution of this project in
 * the file license.html. If the license is not included you may find a copy at
 * http://www.cosylab.com/legal/abeans_license.htm or may write to Cosylab, d.o.o.
 *
 * THIS SOFTWARE IS PROVIDED AS-IS WITHOUT WARRANTY OF ANY KIND, NOT EVEN THE
 * IMPLIED WARRANTY OF MERCHANTABILITY. THE AUTHOR OF THIS SOFTWARE, ASSUMES
 * _NO_ RESPONSIBILITY FOR ANY CONSEQUENCE RESULTING FROM THE USE, MODIFICATION,
 * OR REDISTRIBUTION OF THIS SOFTWARE.
 */

package com.cosylab.gui.components.numberfield;

/**
 * Visualize double value in radians by converting it to hours and displays it
 * in format XX:XX:XX.
 *
 * @author <a href="mailto:igor.kriznar@cosylab.com">Igor Kriznar</a>
 */
public class Rad2HMSNumberDescriptor extends HMSNumberDescriptor
{
	/**
	 * Converts angle in radians to angle in hours. 24 hours is 2PI radians.
	 *
	 * @param rad angle in radians
	 *
	 * @return angle in hours
	 */
	public final static double toHours(double rad)
	{
		return rad * 12.0 / Math.PI;
	}

	/**
	 * Converts angle in hours to angle in radians. 24 hours is 2PI radians.
	 *
	 * @param hour angle in hours
	 *
	 * @return angle in radians
	 */
	public static double toRadians(double hour)
	{
		return hour / 12.0 * Math.PI;
	}

	/**
	 * @see NumberDescriptor
	 */
	public final static String NAME = "Radians as hours Visualization";

	/**
	 * @see NumberDescriptor
	 */
	public final static String DESCRIPTION = "Converts double value from radians to hours and diplays in degree format: XX:XX:XX.";

	/**
				 *
				 */
	public Rad2HMSNumberDescriptor()
	{
		super();
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.components.numberfield.AngleNumberDescriptor#printString(java.lang.Number)
	 */
	public String printString(Number number)
	{
		return super.printString(new Double(toHours(number.doubleValue())));
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.components.numberfield.AngleNumberDescriptor#parseNumber(java.lang.String)
	 */
	public Number parseNumber(String number)
	{
		Number n = super.parseNumber(number);

		if (n == null) {
			return n;
		}

		return new Double(toRadians(n.doubleValue()));
	}
}

/* __oOo__ */
