/*
 * Copyright (c) 2003 by Cosylab d.o.o.
 *
 * The full license specifying the redistribution, modification, usage and other
 * rights and obligations is included with the distribution of this project in
 * the file license.html. If the license is not included you may find a copy at
 * http://www.cosylab.com/legal/abeans_license.htm or may write to Cosylab, d.o.o.
 *
 * THIS SOFTWARE IS PROVIDED AS-IS WITHOUT WARRANTY OF ANY KIND, NOT EVEN THE
 * IMPLIED WARRANTY OF MERCHANTABILITY. THE AUTHOR OF THIS SOFTWARE, ASSUMES
 * _NO_ RESPONSIBILITY FOR ANY CONSEQUENCE RESULTING FROM THE USE, MODIFICATION,
 * OR REDISTRIBUTION OF THIS SOFTWARE.
 */

package com.cosylab.gui.components.range2;

/**
 * Implementation of linear range.
 *
 * @author <a href="mailto:ales.pucelj@cosylab.com">Ales Pucelj</a>
 * @version $id$
 */
public class LinearRange implements Range
{

	/**
	 * Returns the parameters.
	 *
	 * @see com.cosylab.gui.components.range.Range#checkValue(double)
	 */
	public double validate(double value)
	{
		return value;
	}

	/**
	 * Converts relative value in this range to absolute. If span of range is
	 * infinite, return value is 0. Conversion is implemented for all values
	 * even outside the 0.0 - 1.0 range.
	 *
	 * @param relative value in range 0.0 to 1.0 inclusively.
	 *
	 * @return absolute value.
	 *
	 * @see com.cosylab.gui.components.range.Range#toAbsolute(double)
	 */
	public double toAbsolute(double relative, RangedValue ranged)
	{
		final double min = ranged.getMinimum();
		final double max = ranged.getMaximum();

		if (Double.isInfinite(max) || (Double.isInfinite(min))) {
			return 0.0;
		}

		return min + relative * (max - min);
	}

	/**
	 * Converts absolute value into relative (defined in 0.0 to 1.0 range). If
	 * span of the range is infinite, return value is 0.5. Conversion is
	 * defined for all values of even outside the range.
	 *
	 * @param absolute value.
	 *
	 * @return relative value.
	 *
	 * @see com.cosylab.gui.components.range.Range#toRelative(double)
	 */
	public double toRelative(double absolute, RangedValue ranged)
	{
		double range = ranged.getMaximum() - ranged.getMinimum();

		if (range == 0.0) {
			return 0.5;
		}

		if (Double.isInfinite(range)) {
			return 0.5;
		}

		return (absolute - ranged.getMinimum()) / range;
	}

	public Tick[] calculateTicks(RangedValue val, int length) {
		return new LinearTickCollector(this,val,3,length,false).toTicks();
	}
	
	public Tick[] calculateTicks(RangedValue val, int length, TickParameters measurer) {
		return new LinearTickCollector(this,val,3,length,false).toTicks(measurer);
	}
}

/* __oOo__ */
