package com.cosylab.gui.components.range2;

/**
 * This policy enforces the proper order of values. For a triplet of numbers
 * specified in <code>RangedValueHolder</code> it will rearange them (if
 * neccessary) so that minimum will be less or equal to value and value will
 * be less or equal to maximum.
 * 
 * @author <a href="mailto:ales.pucelj@cosylab.com">Ales Pucelj</a>
 * @version $id$
 */
public class ProperOrderPolicy extends RangedValuePolicy {

	/**
	 * Swaps the parameters as neccessary to establis proper ordering of values.
	 * 
	 * @see com.cosylab.gui.components.range.RangedValuePolicy#validate(RangedValueHolder)
	 */
	public void validate(RangedValue params) {
		double min = params.getMinimum();
		double max = params.getMaximum();
		double val = params.getValue();
		
		double temp;

		if (min > val) {
			temp = min;
			min = val;
			val = temp;
		}
		if (min > max) {
			temp = min;
			min = max;
			max = temp;
		}
		if (val > max) {
			temp = val;
			val = max;
			max = temp;
		}
		
		params.setValue(min, max, val);
		
		validatePeerPolicy(params);
	}
}
