package com.cosylab.gui.components.range2;

import java.io.Serializable;

/**
 * This class stores values representing RangedValue. No checks are made for
 * validity of the values. Only purpose of this class is to provide data
 * transfer between <code>RangedValue</code> and <code>RangedValuePolicy</code>.
 * As such, this class will normaly be used only internaly by 
 * <code>RangedValue</code>.
 * 
 * @author <a href="mailto:ales.pucelj@cosylab.com">Ales Pucelj</a>
 * @version $id$
 */
public final class RangedValue implements Serializable {

	private double min;
	
	private double max;
	
	private double val;
	
	public RangedValue() {
		setValue(0, 100, 1);
	}

	/**
	 * @param minimum double
	 * @param maximum double
	 * @param value double
	 */
	public RangedValue(double minimum, double maximum, double value) {
		setValue(minimum, maximum, value);
	}
	
	/**
	 * Returns the minimum.
	 * 
	 * @return double
	 */
	public final double getMinimum() {
		return min;
	}
	
	/**
	 * @return double
	 */
	public final double getMaximum() {
		return max;
	}
	
	/**
	 * Returns the value.
	 * 
	 * @return double
	 */
	public final double getValue() {
		return val;
	}
	
	/**
	 * @param minimum double
	 */
	public final void setMinimum(final double minimum) {
		min = minimum;
	}
	
	/**
	 * @param maximum double
	 */
	public final void setMaximum(final double maximum) {
		max = maximum;
	}
	
	/**
	 * @param value double
	 */
	public final void setValue(final double value) {
		val = value;
	}

	public final void setValue(double minimum, double maximum) {
		setMinimum(minimum);
		setMaximum(maximum);
	}
	
	public final void setValue(double minimum, double maximum, double value) {
		setMinimum(minimum);
		setValue(value);
		setMaximum(maximum);
	}
	
	public final boolean isValid() {
		return ((min <= val) && (val <= max));
	}
	
	public String toString() {
		return getClass().getName()+" { min = "+min+", max = "+max+", val = "+val+" }";
	}
}
