/*
 * Copyright (c) 2003 by Cosylab d.o.o.
 *
 * The full license specifying the redistribution, modification, usage and other
 * rights and obligations is included with the distribution of this project in
 * the file license.html. If the license is not included you may find a copy at
 * http://www.cosylab.com/legal/abeans_license.htm or may write to Cosylab, d.o.o.
 *
 * THIS SOFTWARE IS PROVIDED AS-IS WITHOUT WARRANTY OF ANY KIND, NOT EVEN THE
 * IMPLIED WARRANTY OF MERCHANTABILITY. THE AUTHOR OF THIS SOFTWARE, ASSUMES
 * _NO_ RESPONSIBILITY FOR ANY CONSEQUENCE RESULTING FROM THE USE, MODIFICATION,
 * OR REDISTRIBUTION OF THIS SOFTWARE.
 */

package com.cosylab.gui.components.range2;

/**
 * Ensures that value will never be outside the bounds. Bounds will never be
 * adjusted, only the value will be trimmed to minimum and maximum. If values
 * are changed, value will be adjusted accordingly.
 *
 * @author <a href="mailto:ales.pucelj@cosylab.com">Ales Pucelj</a>
 * @version $id$
 */
public final class TrimValuePolicy extends RangedValuePolicy
{
	
	/**
	 * Trims value to bounds.
	 */
	public static void trim(RangedValue params)
	{
		double value = params.getValue();
		value = Math.max(value, params.getMinimum());
		value = Math.min(value, params.getMaximum());
		params.setValue(value);
	}

	/**
	 * Trims value to bounds.
	 *
	 * @see com.cosylab.gui.components.range.RangedValuePolicy#validate(RangedValueHolder)
	 */
	public void validate(RangedValue params)
	{
		trim(params);
		validatePeerPolicy(params);
	}
}

/* __oOo__ */
