/*
 * Copyright (c) 2003 by Cosylab d.o.o.
 *
 * The full license specifying the redistribution, modification, usage and other
 * rights and obligations is included with the distribution of this project in
 * the file license.html. If the license is not included you may find a copy at
 * http://www.cosylab.com/legal/abeans_license.htm or may write to Cosylab, d.o.o.
 *
 * THIS SOFTWARE IS PROVIDED AS-IS WITHOUT WARRANTY OF ANY KIND, NOT EVEN THE
 * IMPLIED WARRANTY OF MERCHANTABILITY. THE AUTHOR OF THIS SOFTWARE, ASSUMES
 * _NO_ RESPONSIBILITY FOR ANY CONSEQUENCE RESULTING FROM THE USE, MODIFICATION,
 * OR REDISTRIBUTION OF THIS SOFTWARE.
 */

package com.cosylab.gui.components.slider;

import java.awt.AlphaComposite;
import java.awt.Color;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.image.BufferedImage;

import javax.swing.Icon;
import javax.swing.JComponent;
import javax.swing.JSlider;
import javax.swing.plaf.metal.MetalSliderUI;

import com.cosylab.gui.components.range2.RangedValueController;
import com.cosylab.gui.components.range2.Tick;
import com.cosylab.gui.components.range2.TickParameters;
import com.cosylab.util.PrintfFormat;


/**
 * This is customized Swing l&f for JSlider. It is customized to allow 
 * operation
 *
 * @author <a href="mailto:ales.pucelj@cosylab.com">Ales Pucelj</a>
 * @version $id$
 */
public class CosySliderUI extends MetalSliderUI
{
	private boolean checkValue;
	protected Icon vertTrailerIcon = null;
	protected BufferedImage horizTrailerBuffer = null;
	private AlphaComposite alphaComposite;
	protected Rectangle trailerRect = new Rectangle(0, 0, 16, 16);
	protected Rectangle trailerCache = new Rectangle();
	protected double trailerPosition = 0;
	private boolean showIntegerOnly = false;
	private String format = null;
	private double tolerance = 0.0001;
	
	protected static final Color[] GREEN_LED = {
		new Color(0, 160, 0), new Color(0, 220, 0), new Color(0, 250, 0)
	};
	protected static final Color[] RED_LED = {
		new Color(160, 0, 0), new Color(220, 0, 0), new Color(250, 0, 0)
	};
	protected static final Color[] YELLOW_LED = {
		new Color(150, 150, 0), new Color(200, 200, 0), new Color(250, 250, 0)
	};
	protected static final Color[] GREY_LED = {
		new Color(64, 64, 64), new Color(128, 128, 128),
		new Color(160, 160, 160)
	};

	private class TickMeasurer implements TickParameters
	{
		private FontMetrics fm;
		private PrintfFormat formatter;

		/**
		 * Creates a new TickMeasurer object.
		 *
		 * @param g DOCUMENT ME!
		 */
		public TickMeasurer(Graphics g, String format)
		{
			fm = g.getFontMetrics();
			if (format != null)
				formatter = new PrintfFormat(format);
		}

		/**
		 * DOCUMENT ME!
		 *
		 * @param position DOCUMENT ME!
		 * @param text DOCUMENT ME!
		 *
		 * @return DOCUMENT ME!
		 */
		public int measureTick(double position, String text)
		{
			//			System.out.println(fm.stringWidth(text));
			if(text==null) {
				return 0;
			}
			return fm.stringWidth(text);
		}

		public String formatNumber(double value) {
			if (formatter != null)
				return formatter.sprintf(value);
			else
				return null;
		}
	}

	private RangedValueController rangedValue;

	/**
	 * Creates a new CosySliderUI object.
	 *
	 * @param range DOCUMENT ME!
	 */
	public CosySliderUI(RangedValueController range)
	{
		super();
		rangedValue = range;
		alphaComposite = AlphaComposite.getInstance(AlphaComposite.DST_OVER,
			    (float)0.35);
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @param g Graphics
	 */
	private void renderHorizontalTrailer(Graphics g)
	{
		int[] xPoints = { 1, 14, 15, 15, 8, 0, 0 };
		int[] yPoints = { 0, 0, 1, 9, 16, 9, 1 };
				
		g.setColor((checkValue ? Color.black : YELLOW_LED[0]));
		g.fillPolygon(xPoints, yPoints, xPoints.length);
	}

	/**
	 * Returns the image used to display the trailer. This method uses lazy
	 * initialization and loads the image using the IconHelper class.
	 *
	 * @return BufferedImage
	 */
	protected BufferedImage getTrailer()
	{
		if(horizTrailerBuffer == null) {
			//Icon horizTrailerIcon = null;

			//try {
			//	horizTrailerIcon = IconHelper.createIcon(
			//		    "/icons/specific/SliderTrailer16.gif");
			//	trailerRect.width = horizTrailerIcon.getIconWidth();
			//	trailerRect.height = horizTrailerIcon.getIconHeight();

				trailerCache.width = trailerRect.width;
				trailerCache.height = trailerRect.height;
			//} catch(Exception e) {
			//}

			horizTrailerBuffer = new BufferedImage(16, 16,
				    BufferedImage.TYPE_INT_ARGB);

			Graphics2D gbi = horizTrailerBuffer.createGraphics();
			gbi.setComposite(alphaComposite);

			renderHorizontalTrailer(gbi);
		}

		return horizTrailerBuffer;
	}

	protected void calculateTrailerLocation()
	{
		int t = (int)(rangedValue.toRelative(getTrailerPosition()));

		if(slider.getOrientation() == JSlider.HORIZONTAL) {
			int valuePosition = xPositionForValue(t);

			trailerRect.x = valuePosition - (trailerRect.width / 2);
			trailerRect.y = thumbRect.y;
		} else {
			int valuePosition = yPositionForValue(t);

			trailerRect.x = thumbRect.x;
			trailerRect.y = valuePosition - (trailerRect.height / 2);
		}
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @return double
	 */
	public double getTrailerPosition()
	{
		return trailerPosition;
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @param value double
	 */
	public void setTrailerPosition(double value)
	{
		trailerCache.x = trailerRect.x;
		trailerCache.y = trailerRect.y;
				
		trailerPosition = value;
		calculateTrailerLocation();

		//System.out.println(">>>> Ranged Value: " + rangedValue.getValue());
		//System.out.println(">>>> Trailer Position: " + trailerPosition);
		
		boolean oldCheck = checkValue;

		checkValue = (trailerPosition<=rangedValue.getMaximum() && trailerPosition>=rangedValue.getMinimum());

		if (checkValue!=oldCheck) horizTrailerBuffer = null;

		slider.repaint(trailerCache);
		slider.repaint(trailerRect);
		slider.repaint(new Rectangle(0, 0, slider.getWidth(), slider.getHeight()));
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @param g Graphics
	 */
	public void paintTrailer(Graphics g)
	{
		g.translate(trailerRect.x, thumbRect.y);

		if(slider.getOrientation() == JSlider.HORIZONTAL) {
			((Graphics2D)g).drawImage(getTrailer(), null, 0, 0);
		} else {
		}

		g.translate(-trailerRect.x, -thumbRect.y);

		if(thumbRect.x == trailerRect.x) {
			paintThumb(g);
		}
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @return Color[]
	 */
	protected Color[] getIndicatorColor()
	{
		Color[] retVal;
		if (slider.isEnabled()) {
			if (!checkValue) {
				retVal = RED_LED;
			} else {
				//retVal = (Math.abs(thumbRect.x - trailerRect.x) < 2) ? GREEN_LED : RED_LED;
				retVal = (Math.abs(rangedValue.getValue() - trailerPosition) < tolerance) ? GREEN_LED : RED_LED;
			}
		} else {
			retVal = GREY_LED;
		}
		return retVal;
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @param g Graphics
	 */
	public void paintThumb(Graphics g)
	{
		super.paintThumb(g);

		Rectangle knobBounds = thumbRect;
		g.translate(knobBounds.x, knobBounds.y);

		if(slider.getOrientation() == JSlider.HORIZONTAL) {
			Color[] colors = getIndicatorColor();

			g.setColor(colors[0]);
			g.drawLine(1, 3, 13, 3);
			g.drawLine(13, 2, 13, 1);

			g.setColor(colors[1]);
			g.drawLine(1, 1, 13, 1);
			g.drawLine(1, 1, 1, 3);
			g.setColor(colors[2]);
			g.drawLine(2, 2, 12, 2);
			g.setColor(Color.black);
			g.drawLine(1, 4, 13, 4);
		} else {
		}

		g.translate(-knobBounds.x, -knobBounds.y);
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @param g Graphics
	 * @param c JComponent
	 *
	 * @see ComponentUI#paint(Graphics, JComponent)
	 */
	public void paint(Graphics g, JComponent c)
	{
		super.paint(g, c);
		calculateTrailerLocation();

		Rectangle clip = g.getClipBounds();

		if(clip.intersects(trailerRect)) {
			paintTrailer(g);
		}

		if(trailerRect.intersects(thumbRect)) {
			paintThumb(g);
		}
	}

	/**
	 * @see javax.swing.plaf.basic.BasicSliderUI#paintTicks(Graphics)
	 */
	public void paintTicks(Graphics g)
	{
		final Rectangle tr = tickRect;
		final Rectangle lr = labelRect;
		int w = tr.width - 1;

		g.setColor(slider.getBackground());
		g.fillRect(tr.x, tr.y, tr.width, tr.height);

		final Tick[] ticks = rangedValue.calculateTicks(w,new TickMeasurer(g, format));

		if(ticks == null) {
			return;
		}

		g.translate(tr.x, tr.y-2);

		final FontMetrics fm = g.getFontMetrics();
		String text;
		int left;
		int x;
		double tickValue;
		for(int i = 0; i < ticks.length; i++) {
			x = (int)(ticks[i].proportional * w);
			if (showIntegerOnly) {
				tickValue = ticks[i].absolute;
				if(((int)(tickValue*1000+0.5))/1000. !=  (int)(tickValue+0.5)) 
					continue;
				else if (ticks[i].text != null && ticks[i].text.length() > 3){
					//ignore everything behind decimal separator
					ticks[i].text = ticks[i].text.substring(0, ticks[i].text.length()-2);
				}
			}
			
			if(ticks[i].major) {
				paintMajorTickForHorizSlider(g, tr, x);

				text = ticks[i].text;
				if(text==null) {
					left = x;
				} else {
				    left = x - (fm.stringWidth(text) / 2);
				}

//				g.setColor(Color.BLACK);
				g.setColor(slider.getForeground());
				if(text!=null) {
					g.drawString(text, left, lr.y - 8);
				}
			} else {
				paintMinorTickForHorizSlider(g, tr, x);
			}
		}

		g.translate(-tr.x, -tr.y+2);
	}

	/**
	 * @see javax.swing.plaf.basic.BasicSliderUI#paintLabels(Graphics)
	 */
	public void paintLabels(Graphics g)
	{
		paintTicks(g);
	}

	/**
	 * @see javax.swing.plaf.basic.BasicSliderUI#scrollDueToClickInTrack(int)
	 */
	protected void scrollDueToClickInTrack(int dir)
	{
		// This has been disabled to prevent accidental value changes.
		// super.scrollDueToClickInTrack(dir);
	}
	
	public void setFormat(String format) {
		this.format = format;
		if (format.contains("%d")) {
			showIntegerOnly = true;
		} else {
			showIntegerOnly = false;
		}
	}
	
	public void setTolerance(double tol) {
		this.tolerance=tol;
	}
	
	public double getTolerance() {
		return this.tolerance;
	}
	
}

/* __oOo__ */
