/*
 * Copyright (c) 2003 by Cosylab d.o.o.
 *
 * The full license specifying the redistribution, modification, usage and other
 * rights and obligations is included with the distribution of this project in
 * the file license.html. If the license is not included you may find a copy at
 * http://www.cosylab.com/legal/abeans_license.htm or may write to Cosylab, d.o.o.
 *
 * THIS SOFTWARE IS PROVIDED AS-IS WITHOUT WARRANTY OF ANY KIND, NOT EVEN THE
 * IMPLIED WARRANTY OF MERCHANTABILITY. THE AUTHOR OF THIS SOFTWARE, ASSUMES
 * _NO_ RESPONSIBILITY FOR ANY CONSEQUENCE RESULTING FROM THE USE, MODIFICATION,
 * OR REDISTRIBUTION OF THIS SOFTWARE.
 */

package com.cosylab.gui.components.slider;

import com.cosylab.gui.components.ResizableTextLabel;
import com.cosylab.gui.components.Slider;
import com.cosylab.gui.components.util.ColorHelper;

import java.awt.Color;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.SwingConstants;


/**
 * Part of slider UI, displays title and any other general information. This
 * class is intended to be extended.
 *
 * @author <a href="mailto:ales.pucelj@cosylab.com">Ales Pucelj</a>
 * @version $id$
 */
public class InfoBar extends JPanel
{
	private static final long serialVersionUID = 1L;
	private ResizableTextLabel deviceName;
	private JButton sync;
	private JButton set;
	private Slider slider;

	/**
	 * Creates a new InfoBar object.
	 *
	 * @param slider DOCUMENT ME!
	 */
	public InfoBar(Slider slider)
	{
		super();
		this.slider = slider;
		initialize();
	}

	private GridBagConstraints newGC(int x, boolean fill)
	{
		GridBagConstraints gc = new GridBagConstraints();
		gc.gridx = x;
		gc.gridy = 0;
		gc.weightx = (fill ? 1.0 : 0.0);
		gc.weighty = 0;
		gc.fill = (fill ? GridBagConstraints.HORIZONTAL : GridBagConstraints.NONE);

		return gc;
	}

	private void initialize()
	{
		setLayout(new GridBagLayout());
		setOpaque(false);

		
		add(getTitleComponent(), newGC(1, true));

		

		GridBagConstraints gc = newGC(1, false);
		gc.anchor = GridBagConstraints.EAST;
		gc.insets = new Insets(1, 1, 1, 1);
		gc.weightx = 0.0;
		gc.weighty = 0.0;

		add(getSyncButton(), gc);
		
		gc.anchor = GridBagConstraints.WEST;
		add(getSetButton(),gc);
		getSetButton().setVisible(false);
	}
	
	private ResizableTextLabel getTitleComponent() {
		if (deviceName == null) {
			deviceName = new ResizableTextLabel("");
			deviceName.setOpaque(false);
			deviceName.setResizable(false);
			deviceName.setHorizontalAlignment(SwingConstants.CENTER);
		}

		return deviceName;
	}
	
	private JButton getSyncButton() {
		if (sync == null) {
			sync = new JButton("Sync");
			sync.setMargin(new Insets(0, 1, 0, 1));
			sync.setFont(new Font(sync.getFont().getName(), Font.PLAIN, 9));
			sync.setOpaque(true);
			sync.setBackground(ColorHelper.getCosyControl());
			sync.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent e)
					{
						slider.synchronize();
					}
				});
			
		}

		return sync;
	}
	
	private JButton getSetButton() {
		if (set == null) {
			set = new JButton("Set");
			set.setMargin(new Insets(0, 1, 0, 1));
			set.setFont(new Font(set.getFont().getName(), Font.PLAIN, 9));
			set.setOpaque(true);
			set.setBackground(ColorHelper.getCosyControl());
			set.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent e)
					{
						slider.setManual();
					}
				});
			
		}

		return set;
	}
	
	public void setSetButtonVisible(boolean visible) {
		getSetButton().setVisible(visible);
	}
	
	@Override
	public void setBackground(Color bg) {
		super.setBackground(bg);
		getSyncButton().setBackground(bg);
		getTitleComponent().setBackground(bg);
	}
	
	@Override
	public void setForeground(Color fg) {
		super.setForeground(fg);
		getSyncButton().setForeground(fg);
		getTitleComponent().setForeground(fg);
	}

	/**
	 * Sets the title for slider.
	 *
	 * @param title of slider.
	 */
	public void setTitle(String title)
	{
		getTitleComponent().setText(title);
	}

	/**
	 * Returns title of slider.
	 *
	 * @return current title of slider.
	 */
	public String getTitle()
	{
		return getTitleComponent().getText();
	}
	
	public void setFont(Font font) {
		getTitleComponent().setFont(font);
	}
	
	public Font getFont() {
		return getTitleComponent().getFont();
	}
	
	@Override
	public void setEnabled(boolean enabled) {
		super.setEnabled(enabled);
		getSyncButton().setEnabled(enabled);
		getSetButton().setEnabled(enabled);
	}
}

/* __oOo__ */
