/*
 * Copyright (c) 2004 by Cosylab d.o.o.
 *
 * The full license specifying the redistribution, modification, usage and other
 * rights and obligations is included with the distribution of this project in
 * the file license.html. If the license is not included you may find a copy at
 * http://www.cosylab.com/legal/abeans_license.htm or may write to Cosylab, d.o.o.
 *
 * THIS SOFTWARE IS PROVIDED AS-IS WITHOUT WARRANTY OF ANY KIND, NOT EVEN THE
 * IMPLIED WARRANTY OF MERCHANTABILITY. THE AUTHOR OF THIS SOFTWARE, ASSUMES
 * _NO_ RESPONSIBILITY FOR ANY CONSEQUENCE RESULTING FROM THE USE, MODIFICATION,
 * OR REDISTRIBUTION OF THIS SOFTWARE.
 */

package com.cosylab.gui.components.table;

import com.cosylab.gui.components.table.cells.TableCell;


/**
 * This is table row implementation, which converts table row to array of
 * <code>TableCell</code> instances. Specific implementation may inherit from
 * this class and override <code>getTableCells</code> method.
 *
 * @author <a href="mailto:igor.kriznar@cosylab.com">Igor Kriznar</a>
 */
public class DefaultObjectTableEngine implements ObjectTableEngine
{
	/**
				 *
				 */
	public DefaultObjectTableEngine()
	{
		super();
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.components.table.TableRowFactory#getTableRow(java.lang.Object)
	 */
	public TableRow getTableRow(Object key)
	{
		TableCell[] tc = getTableCells(key);

		TableRowArray rta = new TableRowArray();

		for (int i = 0; i < tc.length; i++) {
			rta.addValue(tc[i]);
		}

		return rta;
		
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.components.table.TableRowFactory#releaseTableRow(java.lang.Object, com.cosylab.gui.components.table.TableRow)
	 */
	public void releaseTableRow(Object key, TableRow row)
	{
		// not used
	}

	/**
	 * Override this method to convert key object to own array of
	 * <code>TableCell</code> instances. Default implementation creates for
	 * each object own <code>TableCell</code> and returns it in an array.
	 *
	 * @param key key object
	 *
	 * @return array of table cells
	 */
	protected TableCell[] getTableCells(Object key)
	{
		if (key instanceof Object[]) {
			Object[] o = (Object[])key;

			TableCell[] tc = new TableCell[o.length];

			for (int i = 0; i < o.length; i++) {
				tc[i] = new TableCell(null, o[i], o[i], null);
			}
		}

		return new TableCell[]{ new TableCell(null, key, key, null) };
	}
	
	/**
	 * Dummy implementation: casts row element to TableCell and sets value 
	 * @see com.cosylab.gui.components.table.ObjectTableEngine#setValue(Object, Object, TableRow, int)
	 */
	public void setValue(Object value, Object key, TableRow row, int col) {
		TableCell r= (TableCell)row.getValueAt(col);
		r.setValue(value);
	}
	/* (non-Javadoc)
	 * @see com.cosylab.gui.components.table.ObjectTableEngine#getColumnName(int)
	 */
	public String getColumnName(int i) {
		return null;
	}
}

/* __oOo__ */
