/*
 * Copyright (c) 2003 by Cosylab d.o.o.
 *
 * The full license specifying the redistribution, modification, usage and other
 * rights and obligations is included with the distribution of this project in
 * the file license.html. If the license is not included you may find a copy at
 * http://www.cosylab.com/legal/abeans_license.htm or may write to Cosylab, d.o.o.
 *
 * THIS SOFTWARE IS PROVIDED AS-IS WITHOUT WARRANTY OF ANY KIND, NOT EVEN THE
 * IMPLIED WARRANTY OF MERCHANTABILITY. THE AUTHOR OF THIS SOFTWARE, ASSUMES
 * _NO_ RESPONSIBILITY FOR ANY CONSEQUENCE RESULTING FROM THE USE, MODIFICATION,
 * OR REDISTRIBUTION OF THIS SOFTWARE.
 */

package com.cosylab.gui.components.table;

import java.util.Comparator;

/**
 * <code>MutableTableModelRows</code> is implementation of
 * <code>TableModelRows</code> which acts as mutable
 * <code>TabmeRowModel</code> in the same time. User can set this model to
 * <code>JTable</code> and then add or remove rows directly from this model.
 *
 * @author <a href="mailto:igor.kriznar@cosylab.com">Igor Kriznar</a>
 * @version $Id: MutableTableModelRows.java,v 1.2 2007-02-28 09:54:59 tkusterle Exp $
 *
 * @since May 4, 2004.
 */
public class MutableTableModelRows extends TableModelRows
{
	private TableRowModelSupport support;
	private Comparator comparator;

	/**
	 * Creates new instance.
	 */
	public MutableTableModelRows()
	{
		super(new TableRowModelSupport());
		support = (TableRowModelSupport)getRowModel();
	}

	/**
	 * Inserts row at the specified index.
	 *
	 * @param index the index where to insert
	 * @param row the row to be added
	 */
	public synchronized void addRow(int index, TableRow row)
	{
		support.addRow(index, row);
	}

	/**
	 * Adds row to the end of the row list.
	 *
	 * @param row the row to be added
	 */
	public synchronized void addRow(TableRow row)
	{
		support.addRow(row);
	}

	/**
	 * Returns row at given index,
	 *
	 * @param index the index of  the row
	 *
	 * @return the row at the specified index
	 */
	public synchronized TableRow getRow(int index)
	{
		return support.getRow(index);
	}

	/**
	 * Removes all rows from this model.
	 */
	public synchronized void removeAllRows()
	{
		support.removeAllRows();
	}

	/**
	 * Removes row from this model.
	 *
	 * @param index rowindex to be roemoved
	 *
	 * @return the removed row
	 */
	public synchronized TableRow removeRow(int index)
	{
		return support.removeRow(index);
	}

	/**
	 * Removes row from this model.
	 *
	 * @param row the row to be removed
	 */
	public synchronized void removeRow(TableRow row)
	{
		support.removeRow(row);
	}

	/**
	 * Sets the number of columns this model presents.
	 *
	 * @param count the number of columns
	 */
	public void setColumnCount(int count)
	{
		support.setColumnCount(count);
	}

	/**
	 * Sets column names.
	 *
	 * @param names the column names
	 */
	public void setColumnNames(String[] names)
	{
		support.setColumnNames(names);
	}

	/**
	 * @return the comparator
	 */
	public Comparator getComparator() {
		return comparator;
	}

	/**
	 * @param comparator the comparator to set
	 */
	public void setComparator(Comparator comparator) {
		this.comparator = comparator;
		support.setComparator(comparator);
	}
}

/* __oOo__ */
