/*
 * Copyright (c) 2004 by Cosylab d.o.o.
 *
 * The full license specifying the redistribution, modification, usage and other
 * rights and obligations is included with the distribution of this project in
 * the file license.html. If the license is not included you may find a copy at
 * http://www.cosylab.com/legal/abeans_license.htm or may write to Cosylab, d.o.o.
 *
 * THIS SOFTWARE IS PROVIDED AS-IS WITHOUT WARRANTY OF ANY KIND, NOT EVEN THE
 * IMPLIED WARRANTY OF MERCHANTABILITY. THE AUTHOR OF THIS SOFTWARE, ASSUMES
 * _NO_ RESPONSIBILITY FOR ANY CONSEQUENCE RESULTING FROM THE USE, MODIFICATION,
 * OR REDISTRIBUTION OF THIS SOFTWARE.
 */

package com.cosylab.gui.components.table;

/**
 * <code>ObjectTableEngine</code> is intermediator betwean arbitrary objects
 * and <code>ObjectTableModel</code> structure. It converts objects to
 * <code>TableRow</code>s and executes operations on rows.
 *
 * @author <a href="mailto:igor.kriznar@cosylab.com">Igor Kriznar</a>
 */
public interface ObjectTableEngine
{
	/**
	 * Creates new row for key object.
	 *
	 * @param key key for which row is created
	 *
	 * @return new or recycled table row
	 */
	public TableRow getTableRow(Object key);

	/**
	 * Signals to factory that row with associated key is no longer in use.
	 *
	 * @param key kay object for row
	 * @param row row to be released
	 */
	public void releaseTableRow(Object key, TableRow row);

	/**
	 * Sets new value which resulted from user interaction with table to the
	 * context underlayng kay object.
	 *
	 * @param value new value as user has set with table editors
	 * @param key the key object
	 * @param row row associated with kay object
	 * @param columnIndex DOCUMENT ME!
	 */
	public void setValue(Object value, Object key, TableRow row, int columnIndex);

	/**
	 * Returns name of column. Same order  of columns must be used
	 * when construction rows.
	 *
	 * @return name of column, or <code>null</code>
	 */
	public String getColumnName(int index);
}

/* __oOo__ */
