/*
 * Copyright (c) 2003 by Cosylab d.o.o.
 *
 * The full license specifying the redistribution, modification, usage and other
 * rights and obligations is included with the distribution of this project in
 * the file license.html. If the license is not included you may find a copy at
 * http://www.cosylab.com/legal/abeans_license.htm or may write to Cosylab, d.o.o.
 *
 * THIS SOFTWARE IS PROVIDED AS-IS WITHOUT WARRANTY OF ANY KIND, NOT EVEN THE
 * IMPLIED WARRANTY OF MERCHANTABILITY. THE AUTHOR OF THIS SOFTWARE, ASSUMES
 * _NO_ RESPONSIBILITY FOR ANY CONSEQUENCE RESULTING FROM THE USE, MODIFICATION,
 * OR REDISTRIBUTION OF THIS SOFTWARE.
 */

package com.cosylab.gui.components.table;

import java.util.Vector;

import javax.swing.JTable;
import javax.swing.ListSelectionModel;
import javax.swing.table.TableColumnModel;
import javax.swing.table.TableModel;


/**
 * <code>QueueTable</code> uses <code>QueueTableModelInterceptor</code> update
 * table  in ferformance and thread safe way. User can use this table exactely
 * the same way as <code>JTable</code>, all the event intercetion is done
 * automagically inside this implementation.
 *
 * @author <a href="mailto:igor.kriznar@cosylab.com">Igor Kriznar</a>
 * @version $Id: QueueTable.java,v 1.3 2005/11/18 09:29:58 ikriznar Exp $
 *
 * @since May 3, 2004.
 */
public class QueueTable extends JTable
{
	private TableModel model;
	private QueueTableModelInterceptor interceptor;

	/* @see JTable#JTable()
	 */
	public QueueTable()
	{
		super();
	}

	/* @see JTable#JTable(int, int)
	 */
	public QueueTable(int numRows, int numColumns)
	{
		super(numRows, numColumns);
	}

	/* @see JTable#JTable(javax.swing.table.TableModel)
	 */
	public QueueTable(TableModel dm)
	{
		super(dm);
	}

	/* @see JTable#JTable(java.lang.Object[][], java.lang.Object[])
	 */
	public QueueTable(Object[][] rowData, Object[] columnNames)
	{
		super(rowData, columnNames);
	}

	/* @see JTable#JTable(java.util.Vector, java.util.Vector)
	 */
	public QueueTable(Vector rowData, Vector columnNames)
	{
		super(rowData, columnNames);
	}

	/* @see JTable#JTable(javax.swing.table.TableModel, javax.swing.table.TableColumnModel)
	 */
	public QueueTable(TableModel dm, TableColumnModel cm)
	{
		super(dm, cm);
	}

	/* @see JTable#JTable(javax.swing.table.TableModel, javax.swing.table.TableColumnModel, javax.swing.ListSelectionModel)
	 */
	public QueueTable(TableModel dm, TableColumnModel cm, ListSelectionModel sm)
	{
		super(dm, cm, sm);
	}

	/* (non-Javadoc)
	 * @see javax.swing.JTable#setModel(javax.swing.table.TableModel)
	 */
	public void setModel(TableModel dataModel)
	{
		model = dataModel;

		if (model == null) {
			super.setModel(null);
		} else {
			//super.setModel(model);
			QueueTableModelInterceptor i=new QueueTableModelInterceptor(model);
			if (interceptor!=null) {
				i.setRelaxationTime(interceptor.getRelaxationTime());
				interceptor.destroy();
			}
			super.setModel(interceptor=i);
		}
	}
	
	/* (non-Javadoc)
	 * @see javax.swing.JTable#getModel()
	 */
	public TableModel getModel() {
		return model;
	}

	/**
	 * Returns minimum time betwean two table updates interceptor will wait.
	 * @return minimum time betwean two table updates 
	 */
	public int getRelaxationTime() {
		return interceptor.getRelaxationTime();
	}

	/**
	 * Sets minimum time betwean two table updates interceptor will wait.
	 * @param relaxationTime
	 */
	public void setRelaxationTime(int relaxationTime) {
		interceptor.setRelaxationTime(relaxationTime);
	}
}

/* __oOo__ */
