/*
 * Copyright (c) 2004 by Cosylab d.o.o.
 *
 * The full license specifying the redistribution, modification, usage and other
 * rights and obligations is included with the distribution of this project in
 * the file license.html. If the license is not included you may find a copy at
 * http://www.cosylab.com/legal/abeans_license.htm or may write to Cosylab, d.o.o.
 *
 * THIS SOFTWARE IS PROVIDED AS-IS WITHOUT WARRANTY OF ANY KIND, NOT EVEN THE
 * IMPLIED WARRANTY OF MERCHANTABILITY. THE AUTHOR OF THIS SOFTWARE, ASSUMES
 * _NO_ RESPONSIBILITY FOR ANY CONSEQUENCE RESULTING FROM THE USE, MODIFICATION,
 * OR REDISTRIBUTION OF THIS SOFTWARE.
 */

package com.cosylab.gui.components.table;

import javax.swing.event.TableModelEvent;
import javax.swing.table.AbstractTableModel;


/**
 * <code>TableModelRows</code> is implementation of <code>TableModel</code>,
 * which accepts data in form of <code>TableRowModel</code>, which cvan be set
 * only as construction parameter.
 *
 * @author <a href="mailto:igor.kriznar@cosylab.com">Igor Kriznar</a>
 * @version $Id: TableModelRows.java,v 1.3 2004/11/15 18:06:05 ikriznar Exp $
 *
 * @since May 3, 2004.
 */
public class TableModelRows extends AbstractTableModel
	implements TableRowModelListener
{
	private TableRowModel rowModel;

	/**
	 * Creates a new TableModelRows object.
	 *
	 * @param model the row mode, which is used as data source
	 */
	public TableModelRows(TableRowModel model)
	{
		this();
		rowModel = model;
		model.addTableRowModelListener(this);
	}

	protected TableModelRows()
	{
		super();
	}

	/*
	 * @see javax.swing.table.TableModel#getColumnCount()
	 */
	public int getColumnCount()
	{
		return rowModel.getColumnCount();
	}

	/* (non-Javadoc)
	 * @see javax.swing.table.TableModel#getRowCount()
	 */
	public int getRowCount()
	{
		return rowModel.getRowCount();
	}

	/* (non-Javadoc)
	 * @see javax.swing.table.TableModel#getValueAt(int, int)
	 */
	public Object getValueAt(int rowIndex, int columnIndex)
	{
		TableRow r = rowModel.getRowAt(rowIndex);

		if (r != null) {
			return r.getValueAt(columnIndex);
		}

		return null;
	}

	/**
	 * Returns row model wrapped inside thia table model.
	 *
	 * @return the row model
	 */
	public TableRowModel getRowModel()
	{
		return rowModel;
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.components.table.TableRowModelListener#tableChanged(com.cosylab.gui.components.table.TableRowModelEvent)
	 */
	public void tableChanged(TableRowModelEvent event)
	{
		int first = event.getRowIndex();
		int last = first;
		
		if (first==TableRowModelEvent.HEADER_ROW) {
			fireTableChanged(new TableModelEvent(this, TableModelEvent.HEADER_ROW));
			return;
		}

		for (int i = 1; i < event.getRows().length; i++) {
			int j = rowModel.getRowIndex(event.getRows()[i]);

			if (j < first) {
				first = j;
			} else if (j > last) {
				last = j;
			}
		}

		if (first == -1) {
			first = 0;
			last = getRowCount();
		}

		fireTableChanged(new TableModelEvent(this, first, last,
		        event.getValue(), event.getType()));
	}

	/* (non-Javadoc)
	 * @see javax.swing.table.AbstractTableModel#getColumnName(int)
	 */
	public String getColumnName(int column)
	{
		return rowModel.getColumnName(column);
	}

	/* (non-Javadoc)
	 * @see javax.swing.table.AbstractTableModel#isCellEditable(int, int)
	 */
	public boolean isCellEditable(int rowIndex, int columnIndex)
	{
		TableRow r = rowModel.getRowAt(rowIndex);

		if (r == null) {
			return false;
		}

		return r.isValueSettable(columnIndex);
	}
}

/* __oOo__ */
