/*
 * Copyright (c) 2004 by Cosylab d.o.o.
 *
 * The full license specifying the redistribution, modification, usage and other
 * rights and obligations is included with the distribution of this project in
 * the file license.html. If the license is not included you may find a copy at
 * http://www.cosylab.com/legal/abeans_license.htm or may write to Cosylab, d.o.o.
 *
 * THIS SOFTWARE IS PROVIDED AS-IS WITHOUT WARRANTY OF ANY KIND, NOT EVEN THE
 * IMPLIED WARRANTY OF MERCHANTABILITY. THE AUTHOR OF THIS SOFTWARE, ASSUMES
 * _NO_ RESPONSIBILITY FOR ANY CONSEQUENCE RESULTING FROM THE USE, MODIFICATION,
 * OR REDISTRIBUTION OF THIS SOFTWARE.
 */

package com.cosylab.gui.components.table;

/**
 * <code>TableRow</code> presents a row in a table.
 *
 * @author <a href="mailto:igor.kriznar@cosylab.com">Igor Kriznar</a>
 * @version $Id: TableRow.java,v 1.4 2005/02/03 15:37:02 akosmrlj Exp $
 *
 * @since May 3, 2004.
 */
public interface TableRow
{
	/**
	 * Returns value at column index.
	 *
	 * @param column the column index
	 *
	 * @return value at column index
	 */
	public Object getValueAt(int column);

	/**
	 * Returns the number values int this row.
	 *
	 * @return the number values int this row
	 */
	public int getValueCount();

	/**
	 * This method is called when row is added to model. When row is remove
	 * from the model, this method is called with parameter <code>null</code>.
	 * 
	 * <p>
	 * Row can call on parent model method
	 * <code>fireRowUpdate(TableRow,int)</code> when want to notify model
	 * listeners (specially the JTable) about data changes.
	 * </p>
	 *
	 * @param model the model to which row was added, or <code>null</code>,
	 *        when row is removed form model
	 *
	 * @see TableRowModel#fireRowUpdate(TableRow, int)
	 */
	public void setParentModel(TableRowModel model);

	/**
	 * The parent model which contains this row.
	 *
	 * @return the parent model which contains this row.
	 */
	public TableRowModel getParentModel();

	/**
	 * Return <code>true</code> if value at specified column index is editable.
	 *
	 * @param index the column index
	 *
	 * @return <code>true</code> if value at the specified column index is
	 *         editable
	 */
	public boolean isValueSettable(int index);

	/**
	 * Compares this table row with the specified table row for order.  Returns
	 * a negative integer, zero, or a positive integer as this row is less
	 * than, equal to, or greater than the specified row.
	 *
	 * @param row a row, to which it is compared
	 * @param valueIndex the index of value, which is compared in row
	 *
	 * @return negative integer, zero, or a positive integer as this row is
	 *         less than, equal to, or greater than the specified row.
	 */
	public int compareTo(TableRow row, int valueIndex);
}

/* __oOo__ */
