/*
 * Copyright (c) 2004 by Cosylab d.o.o.
 *
 * The full license specifying the redistribution, modification, usage and other
 * rights and obligations is included with the distribution of this project in
 * the file license.html. If the license is not included you may find a copy at
 * http://www.cosylab.com/legal/abeans_license.htm or may write to Cosylab, d.o.o.
 *
 * THIS SOFTWARE IS PROVIDED AS-IS WITHOUT WARRANTY OF ANY KIND, NOT EVEN THE
 * IMPLIED WARRANTY OF MERCHANTABILITY. THE AUTHOR OF THIS SOFTWARE, ASSUMES
 * _NO_ RESPONSIBILITY FOR ANY CONSEQUENCE RESULTING FROM THE USE, MODIFICATION,
 * OR REDISTRIBUTION OF THIS SOFTWARE.
 */

package com.cosylab.gui.components.table;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;


/**
 * <code>TableRowArray</code> is support  implementation of
 * <code>TableRow</code> build around <code>ArrayList</code>.
 *
 * @author <a href="mailto:igor.kriznar@cosylab.com">Igor Kriznar</a>
 * @version $Id: TableRowArray.java,v 1.4 2005/02/03 15:37:02 akosmrlj Exp $
 *
 * @since May 5, 2004.
 */
public class TableRowArray implements TableRow
{
	protected List values;
	protected TableRowModel parentModel;

	/**
	 * Creates new instance.
	 */
	public TableRowArray()
	{
		super();
		values = new ArrayList();
	}

	/**
	 * Creates new instance.
	 *
	 * @param l values as array
	 */
	public TableRowArray(Object[] l)
	{
		super();
		values = Arrays.asList(l);
	}

	/**
	 * Creates new instance.
	 *
	 * @param l values as List
	 */
	public TableRowArray(List l)
	{
		super();
		values = l;
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.components.table.TableRow#getValueAt(int)
	 */
	public Object getValueAt(int column)
	{
		return values.get(column);
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.components.table.TableRow#getValueCount()
	 */
	public int getValueCount()
	{
		return values.size();
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.components.table.TableRow#setParentModel(com.cosylab.gui.components.table.TableRowModel)
	 */
	public void setParentModel(TableRowModel model)
	{
		parentModel = model;
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.components.table.TableRow#getParentModel()
	 */
	public TableRowModel getParentModel()
	{
		return parentModel;
	}

	/**
	 * Adds new value to the end of this row.
	 *
	 * @param value the value added to the end of this row
	 */
	public void addValue(Object value)
	{
		values.add(value);
	}

	/**
	 * Inserts new value to this row at specified position.
	 *
	 * @param index where to insert value
	 * @param value new inserted value
	 */
	public void addValue(int index, Object value)
	{
		values.add(index, value);
	}

	/**
	 * Replaces value at specified index.
	 *
	 * @param index where value is set
	 * @param newValue new value
	 *
	 * @return old value at index
	 */
	public Object setValue(int index, Object newValue)
	{
		return values.set(index, newValue);
	}

	/*
	 * Fires update event for entire row.
	 */
	protected void fireUpdate()
	{
		fireUpdate(TableRowModelEvent.ALL_VALUES);
	}

	/*
	 * Fires update event for value in row.
	 */
	protected void fireUpdate(int index)
	{
		if (parentModel != null) {
			parentModel.fireRowUpdate(this, index);
		}
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.components.table.TableRow#isEditable(int)
	 */
	public boolean isValueSettable(int index)
	{
		return false;
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.components.table.TableRow#compateTo(com.cosylab.gui.components.table.TableRow, int)
	 */
	public int compareTo(TableRow row, int valueIndex)
	{
		Object o1 = getValueAt(valueIndex);
		Object o2 = row.getValueAt(valueIndex);

		if (o1 instanceof Comparable) {
			return ((Comparable)o1).compareTo(o2);
		}

		return 0;
	}
}

/* __oOo__ */
