/*
 * Copyright (c) 2004 by Cosylab d.o.o.
 *
 * The full license specifying the redistribution, modification, usage and other
 * rights and obligations is included with the distribution of this project in
 * the file license.html. If the license is not included you may find a copy at
 * http://www.cosylab.com/legal/abeans_license.htm or may write to Cosylab, d.o.o.
 *
 * THIS SOFTWARE IS PROVIDED AS-IS WITHOUT WARRANTY OF ANY KIND, NOT EVEN THE
 * IMPLIED WARRANTY OF MERCHANTABILITY. THE AUTHOR OF THIS SOFTWARE, ASSUMES
 * _NO_ RESPONSIBILITY FOR ANY CONSEQUENCE RESULTING FROM THE USE, MODIFICATION,
 * OR REDISTRIBUTION OF THIS SOFTWARE.
 */

package com.cosylab.gui.components.table;

/**
 * <code>TableRowModel</code> is table model which natural organization of data
 * is grouped by rows. This model can not be used directly in
 * <code>JTable</code>, it must be used in collaboration with
 * <code>TableModelRows</code> or similar implementation of
 * <code>TableModel</code>.
 *
 * @author <a href="mailto:igor.kriznar@cosylab.com">Igor Kriznar</a>
 * @version $Id: TableRowModel.java,v 1.2 2005/02/03 12:39:16 ikriznar Exp $
 *
 * @since May 3, 2004.
 */
public interface TableRowModel
{
	/**
	 * Number of columns inside this model.
	 *
	 * @return number of columns inside this model
	 */
	public int getColumnCount();

	/**
	 * Returns number of rows contained in this model.
	 *
	 * @return number of  rows
	 */
	public int getRowCount();

	/**
	 * Returns a row object at row index.
	 *
	 * @param rowIndex the row index
	 *
	 * @return a row at the row index
	 */
	public TableRow getRowAt(int rowIndex);

	/**
	 * Returns index of a row.
	 *
	 * @param row a row form this model
	 *
	 * @return the index ot the row
	 */
	public int getRowIndex(TableRow row);

	/**
	 * Add listener, which receives notification of changes in the model.
	 *
	 * @param l a <code>TableRowModelListener</code> listener
	 */
	public void addTableRowModelListener(TableRowModelListener l);

	/**
	 * Removes listener.
	 *
	 * @param l a <code>TableRowModelListener</code> listener
	 */
	public void removeTableRowModelListener(TableRowModelListener l);

	/**
	 * Returns name of column.
	 *
	 * @param index the index of  the colummn
	 *
	 * @return the name of the column under provided index
	 *
	 * @see javax.swing.table.TableModel#getColumnName(int)
	 */
	public String getColumnName(int index);

	/**
	 * This method is called from table row, when wants to notify its parent
	 * model about value change.
	 *
	 * @param row the row which value has changes
	 * @param valueIndex the index of the value, if <code>-1</code> all values
	 *        has been changed
	 */
	public void fireRowUpdate(TableRow row, int valueIndex);

	/**
	 * Sort rows in ascending or descengin order according to the natural order
	 * of cells in specified column. Natural order is determined by cells,
	 * which implements <code>Comparable</code>.
	 *
	 * @param column the column index, according to which rows are sorted
	 * @param descending sorted rows are ordered ascending if
	 *        <code>false</code>, or descending if <code>true</code>.
	 */
	public void sortRows(int column, boolean descending);
}

/* __oOo__ */
