/*
 * Copyright (c) 2004 by Cosylab d.o.o.
 *
 * The full license specifying the redistribution, modification, usage and other
 * rights and obligations is included with the distribution of this project in
 * the file license.html. If the license is not included you may find a copy at
 * http://www.cosylab.com/legal/abeans_license.htm or may write to Cosylab, d.o.o.
 *
 * THIS SOFTWARE IS PROVIDED AS-IS WITHOUT WARRANTY OF ANY KIND, NOT EVEN THE
 * IMPLIED WARRANTY OF MERCHANTABILITY. THE AUTHOR OF THIS SOFTWARE, ASSUMES
 * _NO_ RESPONSIBILITY FOR ANY CONSEQUENCE RESULTING FROM THE USE, MODIFICATION,
 * OR REDISTRIBUTION OF THIS SOFTWARE.
 */

package com.cosylab.gui.components.table.cells;

/**
 * LongENumCell holds enumeration valies. Each long value from min to max
 * represents one enumeration state, which can have own name and value.
 *
 * @author <a href="mailto:igor.kriznar@cosylab.com">Igor Kriznar</a>
 */
public class LongEnumCell extends LongCell
{
	/** DOCUMENT ME! */
	public static final String STATE_DESCRIPTION = "stateDescription";

	/** DOCUMENT ME! */
	public static final String STATE_VALUE = "stateValue";

	/**
			 *
			 */
	public LongEnumCell()
	{
		super();

		// TODO Auto-generated constructor stub
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @param identifier
	 * @param dataSource
	 * @param value
	 * @param commands
	 */
	public LongEnumCell(String identifier, Object dataSource, long value,
	    Command[] commands)
	{
		super(identifier, dataSource, value, commands);
	}

	/**
	 * Creates a new LongEnumCell object.
	 *
	 * @param identifier DOCUMENT ME!
	 * @param dataSource DOCUMENT ME!
	 * @param value DOCUMENT ME!
	 * @param commands DOCUMENT ME!
	 * @param descriptions DOCUMENT ME!
	 * @param values DOCUMENT ME!
	 */
	public LongEnumCell(String identifier, Object dataSource, long value,
	    Command[] commands, String[] descriptions, Object[] values)
	{
		super(identifier, dataSource, value, commands);

		for (int i = 0; i < values.length; i++) {
			putCharacteristic(STATE_VALUE + i, values[i]);
		}

		for (int i = 0; i < descriptions.length; i++) {
			putCharacteristic(STATE_DESCRIPTION + i, descriptions[i]);
		}
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @param i DOCUMENT ME!
	 *
	 * @return DOCUMENT ME!
	 */
	public String getStateDescription(long i)
	{
		return (String)getCharacteristic(STATE_DESCRIPTION + i);
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @param i DOCUMENT ME!
	 *
	 * @return DOCUMENT ME!
	 */
	public Object getStateValue(long i)
	{
		return getCharacteristic(STATE_DESCRIPTION + i);
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @param l DOCUMENT ME!
	 *
	 * @return DOCUMENT ME!
	 */
	public String getStateLabel(long l)
	{
		String s = getStateDescription(l);

		if (s == null) {
			StringBuffer sb = new StringBuffer(8);
			sb.append('<');
			sb.append(l);
			sb.append('>');
			s = sb.toString();
		}

		return s;
	}

	/**
	 * Trys to lookup state table and find enum index with this label. If value
	 * is not found, -1 is returned.
	 *
	 * @param label enum state description label
	 *
	 * @return state index or -1
	 */
	public long toLongValue(String label)
	{
		label = label.trim();

		if (label == null || label.length() < 3) {
			return -1;
		}

		if (label.charAt(0) == '<' && label.charAt(label.length() - 1) == '>') {
			// this is generated label
			return Long.parseLong(label.substring(1, label.length() - 1));
		}

		// this is label from descriptions
		int min = getMinimum(new Long(0)).intValue();
		int max = getMaximum(new Long(15)).intValue();

		for (int i = min; i <= max; i++) {
			if (label.equals(getCharacteristic(STATE_DESCRIPTION + i))) {
				return i;
			}
		}

		return -1;
	}
}

/* __oOo__ */
