/*
 * Copyright (c) 2004 by Cosylab d.o.o.
 *
 * The full license specifying the redistribution, modification, usage and other
 * rights and obligations is included with the distribution of this project in
 * the file license.html. If the license is not included you may find a copy at
 * http://www.cosylab.com/legal/abeans_license.htm or may write to Cosylab, d.o.o.
 *
 * THIS SOFTWARE IS PROVIDED AS-IS WITHOUT WARRANTY OF ANY KIND, NOT EVEN THE
 * IMPLIED WARRANTY OF MERCHANTABILITY. THE AUTHOR OF THIS SOFTWARE, ASSUMES
 * _NO_ RESPONSIBILITY FOR ANY CONSEQUENCE RESULTING FROM THE USE, MODIFICATION,
 * OR REDISTRIBUTION OF THIS SOFTWARE.
 */

package com.cosylab.gui.components.table.cells;

/**
 * Numeric cell holds numeric value: double or long.
 *
 * @author <a href="mailto:igor.kriznar@cosylab.com">Igor Kriznar</a>
 */
public abstract class NumericCell extends TableCell
{
	/** Characteristic name for string describing ANSI type of format. */
	public static final String FORMAT = "format";

	/** Characteristic name for string describing physical units of value. */
	public static final String UNITS = "units";

	/** Characteristic name for string describing minimal allowed value. */
	public static final String MINIMUM = "minimum";

	/** Characteristic name for string describing maximal allowed value. */
	public static final String MAXIMUM = "maximum";

	/** Characteristic name for value defining if units should be visible. */
	public static final String UNITS_VISIBLE = "unitsVisible";
	
	/**
			 *
			 */
	public NumericCell()
	{
		super();
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @param identifier
	 * @param dataSource
	 * @param value
	 * @param commands
	 */
	public NumericCell(String identifier, Object dataSource, Object value,
	    Command[] commands)
	{
		super(identifier, dataSource, value, commands);
	}

	/**
	 * Returns value of FORMAT characteristic.
	 *
	 * @return value of FORMAT characteristic
	 */
	public String getFormat()
	{
		return (String)getCharacteristic(FORMAT);
	}

	/**
	 * Returns value of UNITS characteristic.
	 *
	 * @return value of UNITS characteristic
	 */
	public String getUnits()
	{
		return (String)getCharacteristic(UNITS);
	}
	
	/**
	 * Returns a flag if units are visible.
	 *  
	 * @return
	 */
	public boolean isUnitsVisible() {
		Boolean b = (Boolean) getCharacteristic(UNITS_VISIBLE);
		if (b == null) return false;
		else return b;
	}

	/**
	 * Returns value of MINIMUM characteristic.
	 *
	 * @return value of MINIMUM characteristic
	 */
	public Number getMinimum()
	{
		return (Number)getCharacteristic(MINIMUM);
	}

	/**
	 * Returns value of MINIMUM characteristic.
	 *
	 * @param defaultValue DOCUMENT ME!
	 *
	 * @return value of MINIMUM characteristic
	 */
	public Number getMinimum(Number defaultValue)
	{
		return (Number)getCharacteristic(MINIMUM, defaultValue);
	}

	/**
	 * Returns value of MAXIMUM characteristic.
	 *
	 * @return value of MAXIMUM characteristic
	 */
	public Number getMaximum()
	{
		return (Number)getCharacteristic(MAXIMUM);
	}

	/**
	 * Returns value of MAXIMUM characteristic.
	 *
	 * @param defaultValue DOCUMENT ME!
	 *
	 * @return value of MAXIMUM characteristic
	 */
	public Number getMaximum(Number defaultValue)
	{
		return (Number)getCharacteristic(MAXIMUM, defaultValue);
	}
}

/* __oOo__ */
