/*
 * Copyright (c) 2004 by Cosylab d.o.o.
 *
 * The full license specifying the redistribution, modification, usage and other
 * rights and obligations is included with the distribution of this project in
 * the file license.html. If the license is not included you may find a copy at
 * http://www.cosylab.com/legal/abeans_license.htm or may write to Cosylab, d.o.o.
 *
 * THIS SOFTWARE IS PROVIDED AS-IS WITHOUT WARRANTY OF ANY KIND, NOT EVEN THE
 * IMPLIED WARRANTY OF MERCHANTABILITY. THE AUTHOR OF THIS SOFTWARE, ASSUMES
 * _NO_ RESPONSIBILITY FOR ANY CONSEQUENCE RESULTING FROM THE USE, MODIFICATION,
 * OR REDISTRIBUTION OF THIS SOFTWARE.
 */

package com.cosylab.gui.components.table.cells;

import java.awt.Color;


/**
 * PAtternCell describes pattern of bits.
 *
 * @author <a href="mailto:igor.kriznar@cosylab.com">Igor Kriznar</a>
 * @version $id$
 */
public class PatternCell extends LongCell
{
	/**
		 *
		 */
	public PatternCell()
	{
		super();
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @param identifier
	 * @param dataSource
	 * @param value
	 * @param commands
	 */
	public PatternCell(String identifier, Object dataSource, long value,
	    Command[] commands)
	{
		super(identifier, dataSource, value, commands);
	}

	/**
	 * Characteristic name for array with colors which are used when bit in
	 * value is 1.
	 */
	public static final String COLORS_WHEN_ON = "colorsWhenOn";

	/**
	 * Characteristic name for array with colors which are used when bit in
	 * value is 0.
	 */
	public static final String COLORS_WHEN_OFF = "colorsWhenOff";

	/**
	 * Characteristic  name for array with description strings for each pattern
	 * bit.
	 */
	public static final String DESCRIPTIONS = "descriptions";

	/**
	 * Returns value of COLORS_WHEN_ON characteristic.
	 *
	 * @return value of COLORS_WHEN_ON characteristic
	 */
	public Color[] getColorsWhenOn()
	{
		return (Color[])getCharacteristic(COLORS_WHEN_ON);
	}

	/**
	 * Returns value of COLORS_WHEN_OFF characteristic.
	 *
	 * @return value of COLORS_WHEN_OFF characteristic
	 */
	public Color[] getColorsWhenOff()
	{
		return (Color[])getCharacteristic(COLORS_WHEN_OFF);
	}

	/**
	 * Returns value of DESCRIPTIONS characteristic.
	 *
	 * @return value of DESCRIPTIONS characteristic
	 */
	public String[] getDescriptions()
	{
		return (String[])getCharacteristic(DESCRIPTIONS);
	}
}

/* __oOo__ */
