/*
 * @@COPYRIGHT@@
 */
package com.cosylab.gui.components.table.renderers;

import java.awt.Component;

import javax.swing.JCheckBox;
import javax.swing.JTable;
import javax.swing.table.TableCellRenderer;

import com.cosylab.gui.components.table.cells.BooleanCell;

/**
 * This is the default renderer for the ObjectTable cells implementing 
 * BooleanCell. Is is rendered as a checkbox that can be in two states.
 * Since it is a renderer, the event handling for changing the states must
 * be provided at ObjectTable level.
 * 
 * @author <a href="mailto:ales.pucelj@cosylab.com">Ales Pucelj</a>  
 * @version $id$
 */
public class BooleanCellRenderer
	extends JCheckBox
	implements TableCellRenderer {

	/**
	 * Default constructor for BooleanTableCellRenderer.
	 */
	public BooleanCellRenderer() {
		super();
		setOpaque(true);
	}

	/**
	 * Sets this components parameters.
	 * 
	 * @param table JTable
	 * @param booleanTableCell Object
	 * @param isSelected boolean
	 * @param hasFocus boolean
	 * @param row int
	 * @param column int
	 * @return Component
	 * @see javax.swing.table.TableCellRenderer.getTableCellRendererComponent
	 */
	public Component getTableCellRendererComponent(
		JTable table,
		Object booleanTableCell,
		boolean isSelected,
		boolean hasFocus,
		int row,
		int column) {

		BooleanCell cellProperty = (BooleanCell) booleanTableCell;
		setForeground(table.getForeground());
		setBackground(table.getBackground());

		setSelected(cellProperty.getBooleanValue());
		return this;
	}
}
