/*
 * Copyright (c) 2003 by Cosylab d.o.o.
 *
 * The full license specifying the redistribution, modification, usage and other
 * rights and obligations is included with the distribution of this project in
 * the file license.html. If the license is not included you may find a copy at
 * http://www.cosylab.com/legal/abeans_license.htm or may write to Cosylab, d.o.o.
 *
 * THIS SOFTWARE IS PROVIDED AS-IS WITHOUT WARRANTY OF ANY KIND, NOT EVEN THE
 * IMPLIED WARRANTY OF MERCHANTABILITY. THE AUTHOR OF THIS SOFTWARE, ASSUMES
 * _NO_ RESPONSIBILITY FOR ANY CONSEQUENCE RESULTING FROM THE USE, MODIFICATION,
 * OR REDISTRIBUTION OF THIS SOFTWARE.
 */

package com.cosylab.gui.components.table.renderers;

import java.awt.Color;
import java.awt.Component;
import java.awt.Graphics;

import javax.swing.JTable;
import javax.swing.border.BevelBorder;
import javax.swing.border.Border;
import javax.swing.border.CompoundBorder;
import javax.swing.border.EmptyBorder;
import javax.swing.table.DefaultTableCellRenderer;


/**
 * Renders cells containing <code>java.awt.Color</code> objects. Cell will be
 * rendered as opaque colored cells of specified color with beveled border.
 *
 * @author <a href="mailto:ales.pucelj@cosylab.com">Ales Pucelj</a>
 * @version $id$
 */
public class ColorRenderer extends DefaultTableCellRenderer
{
	private static final Border border = new CompoundBorder(new EmptyBorder(2, 2, 2, 2),
		    new BevelBorder(BevelBorder.RAISED));

	/**
	 * Constructor for ColorRenderer.
	 */
	public ColorRenderer()
	{
		super();
		setOpaque(true);
	}

	/**
	 * Sets up the component.
	 * 
	 * @see javax.swing.table.TableCellRenderer#getTableCellRendererComponent(JTable,
	 *      Object, boolean, boolean, int, int)
	 */
	public Component getTableCellRendererComponent(JTable table, Object value,
	    boolean isSelected, boolean hasFocus, int row, int column)
	{
		Component cell = super.getTableCellRendererComponent(table, value,
			    isSelected, hasFocus, row, column);

		setBorder(border);

		if (value instanceof Color) {
			setBackground((Color)value);
		}

		setText("");

		return cell;
	}

	/**
	 * Paints the component.
	 * 
	 * @see javax.swing.JComponent#printComponent(java.awt.Graphics)
	 */
	protected void paintComponent(Graphics g)
	{
		super.paintComponent(g);
		g.setColor(Color.WHITE);
		g.drawRect(0, 0, getWidth() - 1, getHeight() - 1);
		g.drawRect(0, 0, getWidth() - 2, getHeight() - 2);
		g.drawRect(1, 1, getWidth() - 3, getHeight() - 3);
	}
}

/* __oOo__ */
