/*
 * Copyright (c) 2004 by Cosylab d.o.o.
 *
 * The full license specifying the redistribution, modification, usage and other
 * rights and obligations is included with the distribution of this project in
 * the file license.html. If the license is not included you may find a copy at
 * http://www.cosylab.com/legal/abeans_license.htm or may write to Cosylab, d.o.o.
 *
 * THIS SOFTWARE IS PROVIDED AS-IS WITHOUT WARRANTY OF ANY KIND, NOT EVEN THE
 * IMPLIED WARRANTY OF MERCHANTABILITY. THE AUTHOR OF THIS SOFTWARE, ASSUMES
 * _NO_ RESPONSIBILITY FOR ANY CONSEQUENCE RESULTING FROM THE USE, MODIFICATION,
 * OR REDISTRIBUTION OF THIS SOFTWARE.
 */

/*
 * Created on 26-Oct-2003
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package com.cosylab.gui.components.table.renderers;

import com.cosylab.gui.components.table.cells.TableCell;
import com.cosylab.gui.components.util.ColorHelper;
import com.cosylab.gui.components.util.PaintHelper;

import java.awt.Color;
import java.awt.Component;
import java.awt.Graphics;

import java.util.Date;

import javax.swing.JTable;


/**
 * Default renderer used for all <code>PropertyCell</code> objects. Uses color
 * coding to indicate state of cell, but does not render value.
 *
 * @author <a href="mailto:igor.kriznar@cosylab.com">Igor Kriznar</a>
 * @version $id$
 */
public class DefaultTableCellRenderer
	extends javax.swing.table.DefaultTableCellRenderer
{
	/**
	 * Default constructor.
	 */
	public DefaultTableCellRenderer()
	{
		this(false,true);
	}
	/**
	 * Default constructor.
	 */
	public DefaultTableCellRenderer(boolean decorateBorder, boolean decorateBackground)
	{
		super();
		setOpaque(true);
		this.decorateBorder=decorateBorder;
		this.decorateBackground=decorateBackground;
	}

	protected boolean timeout;
	protected boolean error;
	protected boolean alarm;
	protected boolean active;
	protected String severity;
	protected boolean decorateBorder=false;
	protected boolean decorateBackground=true;

	/**
	 * Returns cell renderer component. This implementation is specific to
	 * <code>PropertyCell</code> objects. Color coding will be used to
	 * indicate states. If cell indicates timeout, background will be rendered
	 * gray, if value is out of bounds as defined by <code>BoundedCell</code>,
	 * background will be colored red.
	 *
	 * @param table JTable
	 * @param cell Object
	 * @param isSelected boolean
	 * @param hasFocus boolean
	 * @param row int
	 * @param column int
	 *
	 * @return Component
	 *
	 * @see javax.swing.table.TableCellRenderer#getTableCellRendererComponent(JTable,
	 *      Object, boolean, boolean, int, int)
	 */
	public Component getTableCellRendererComponent(JTable table, Object cell,
	    boolean isSelected, boolean hasFocus, int row, int column)
	{
		Component comp = super.getTableCellRendererComponent(table, cell,
			    isSelected, hasFocus, row, column);

		// Workaround for ugly default behaviour of DefaultTableCellRenderer
		if (isSelected) {
			comp.setBackground(table.getSelectionBackground());
		} else {
			comp.setBackground(table.getBackground());
		}

		if (cell instanceof TableCell) {
			TableCell tc = (TableCell)cell;

			if (tc.isOutOfBounds()) {
				comp.setBackground(ColorHelper.getWarning());
			}

			timeout = tc.isTimeout();
			error = tc.isError();
			active = !tc.isSuspended();
			alarm = tc.isAlarm();
			severity = (String)tc.getCharacteristic(TableCell.SEVERITY);

			String t = tc.getTooltip();
			setToolTipText(t);
			
			if (decorateBackground) {
				if (error || timeout) {
					setBackground(ColorHelper.getAlarmInvalidBackground());
				}

				if (alarm) {
					if (severity == null) {
						setBackground(Color.WHITE);
					} else if (severity.equals("MAJOR_ALARM")) {
						setBackground(ColorHelper.getAlarmMajorBackground());
					} else if (severity.equals("MINOR_ALARM")) {
						setBackground(ColorHelper.getAlarmMinorBackground());
					} else if (severity.equals("INVALID_ALARM")) {
						setBackground(ColorHelper.getAlarmInvalidBackground());
					}
				}

			}
		}

		return comp;
	}

	/* (non-Javadoc)
	 * @see javax.swing.JComponent#paintComponent(java.awt.Graphics)
	 */
	protected void paintComponent(Graphics g)
	{
		super.paintComponent(g);

		paintDecorations(g);
	}

	protected void paintSuperComponent(Graphics g)
	{
		super.paintComponent(g);
	}

	protected void paintDecorations(Graphics g)
	{
		if (!active) {
			PaintHelper.paintDisabled(g, 0, 0, getWidth(), getHeight());
		}
		
		if (!decorateBorder) {
			return;
		}

		int size = getHeight();
		int y = 0;
		int x = (getWidth() - size) / 2;

		if (error && !timeout) {
			PaintHelper.paintAlarm(g, x, y, size);
		}

		if (!error && timeout) {
			PaintHelper.paintTimeout(g, x, y, size, new Date(0, 0, 0, 3, 0));
		}

		if (error && timeout) {
			x = (getWidth() - 2 * size) / 2;
			PaintHelper.paintAlarm(g, x, y, size);
			x += size;
			PaintHelper.paintTimeout(g, x, y, size, new Date(0, 0, 0, 3, 0));
		}

		if (alarm) {
			if (severity == null) {
				PaintHelper.paintRectangle(g, 0, 0, getWidth(), getHeight(),
				    ColorHelper.getAlarm(), 2);
			} else if (severity.equals("MAJOR_ALARM")) {
				PaintHelper.paintRectangle(g, 0, 0, getWidth(), getHeight(),
				    ColorHelper.getAlarmMajor(), 2);
			} else if (severity.equals("MINOR_ALARM")) {
				PaintHelper.paintRectangle(g, 0, 0, getWidth(), getHeight(),
				    ColorHelper.getAlarmMinor(), 2);
			} else if (severity.equals("INVALID_ALARM")) {
				PaintHelper.paintRectangle(g, 0, 0, getWidth(), getHeight(),
				    ColorHelper.getAlarmInvalid(), 2);
			}
		}
	}
	public boolean isDecorateBackground() {
		return decorateBackground;
	}
	public void setDecorateBackground(boolean decorateBackground) {
		this.decorateBackground = decorateBackground;
	}
	public boolean isDecorateBorder() {
		return decorateBorder;
	}
	public void setDecorateBorder(boolean decorateBorder) {
		this.decorateBorder = decorateBorder;
	}
}

/* __oOo__ */
