/*
 * Copyright (c) 2004 by Cosylab d.o.o.
 *
 * The full license specifying the redistribution, modification, usage and other
 * rights and obligations is included with the distribution of this project in
 * the file license.html. If the license is not included you may find a copy at
 * http://www.cosylab.com/legal/abeans_license.htm or may write to Cosylab, d.o.o.
 *
 * THIS SOFTWARE IS PROVIDED AS-IS WITHOUT WARRANTY OF ANY KIND, NOT EVEN THE
 * IMPLIED WARRANTY OF MERCHANTABILITY. THE AUTHOR OF THIS SOFTWARE, ASSUMES
 * _NO_ RESPONSIBILITY FOR ANY CONSEQUENCE RESULTING FROM THE USE, MODIFICATION,
 * OR REDISTRIBUTION OF THIS SOFTWARE.
 */

package com.cosylab.gui.components.table.renderers;

import java.awt.Color;
import java.awt.Component;
import java.awt.Font;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.lang.ref.WeakReference;
import java.util.EventObject;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;
import javax.swing.JSeparator;
import javax.swing.JTable;
import javax.swing.SwingConstants;
import javax.swing.event.CellEditorListener;
import javax.swing.event.ChangeEvent;
import javax.swing.table.TableCellEditor;

import com.cosylab.gui.components.ObjectTable;
import com.cosylab.gui.components.table.cells.Command;
import com.cosylab.gui.components.table.cells.DeviceCell;
import com.cosylab.logging.DebugLogger;
import com.cosylab.util.ObjectList;


/**
 * Renderer for <code>com.cosylab.gui.components.table.cells.DeviceCell</code>
 * cells. This implementation displays device name.
 *
 * @author <a href="mailto:ales.pucelj@cosylab.com">Ales Pucelj</a>
 * @version $id$
 */
public class DeviceCellRenderer extends DefaultTableCellRenderer
	implements TableCellEditor
{
	private static final Color SELECTED = Color.GREEN;
	private Color unselected;
	private CommandPopUp popup;
	private Logger logger = DebugLogger.getLogger("DevCellRen", Level.OFF);

	class CommandPopUp extends JPopupMenu implements ActionListener
	{
		//private DeviceCell cell;
		private WeakReference ref;
		private boolean prepared = false;

		DeviceCell getCell()
		{
			return (DeviceCell)ref.get();
		}

		void setCell(DeviceCell cell)
		{
			if (ref!=null && ref.get() == cell) {
				return;
			}

			ref= new WeakReference(cell);
			removeAll();
			prepared = false;
		}

		/**
		 * Prepares menu.
		 */
		public void prepare()
		{
			if (prepared) {
				return;
			}

			prepared = true;
			
			DeviceCell cell= getCell();
			if (cell==null) {
				return;
			}

			JMenuItem item = new JMenuItem(cell.getIdentifier());
			item.setFont(item.getFont().deriveFont(Font.BOLD).deriveFont((float)(item.getFont()
			        .getSize() * 1.2)));
			item.setEnabled(false);
			add(item);
			add(new JSeparator(SwingConstants.HORIZONTAL));

			Command[] c = cell.getCommands();

			if (c == null) {
				return;
			}

			for (int i = 0; i < c.length; i++) {
				JMenuItem jmi= new JMenuItem(c[i].getName());
				jmi.addActionListener(this);
				jmi.setActionCommand(c[i].getName());
				add(jmi);
			}
		}
		
		/* (non-Javadoc)
		 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
		 */
		public void actionPerformed(ActionEvent e) {
			
			DeviceCell cell=getCell();
			
			if (cell==null) {
				return;
			}
			Command[] c = cell.getCommands();

			if (c == null) {
				return;
			}
			
			for (int i = 0; i < c.length; i++) {
				if (c[i].getName().equals(e.getActionCommand())) {
					try {
						c[i].execute(cell);
					} catch (Exception ex) {
						ex.printStackTrace();
					}
					return;
				}
			}
		}
	}

	public DeviceCellRenderer()
	{
		this(false,true);
	}
	/**
	 * DeviceCellRenderer constructor comment.
	 */
	public DeviceCellRenderer(boolean decorateBorder, boolean decorateBackground)
	{
		super(decorateBorder,decorateBackground);
		unselected = getBackground();

		//setOpaque(true);
		popup = new CommandPopUp();
		addMouseListener(new MouseAdapter() {
				//private Point lastPoint;

				/* (non-Javadoc)
				 * @see java.awt.event.MouseAdapter#mouseClicked(java.awt.event.MouseEvent)
				 */
				public void mouseClicked(MouseEvent e) {
					logger.finer("CLICK");
					popup.prepare();
					popup.show(DeviceCellRenderer.this, e.getX(), e.getY());
				}
				/**
				 * @see java.awt.event.MouseListener#mousePressed(java.awt.event.MouseEvent)
				 */
				public void mousePressed(MouseEvent e)
				{
//					lastPoint = e.getPoint();
					logger.finer("PRESS");
					popup.prepare();
					popup.show(DeviceCellRenderer.this, e.getX(), e.getY());

					/*if (SwingUtilities.isRightMouseButton(e)) {
					    logger.finer("RIGTH");
					    lastPoint = e.getPoint();
					} else {
					    logger.finer("CLICK");
					    lastPoint = null;
					}*/
				}

				/**
				 * @see java.awt.event.MouseListener#mouseReleased(java.awt.event.MouseEvent)
				 */
				public void mouseReleased(MouseEvent e)
				{
					logger.finer("RELEASE");
					popup.prepare();
					popup.show(DeviceCellRenderer.this, e.getX(), e.getY());
/*
					//if (SwingUtilities.isRightMouseButton(e)) {
					if ((e.getSource() == DeviceCellRenderer.this)
					    && (lastPoint != null)) {
						logger.finer("POPUP");
						popup.prepare();
						popup.show(DeviceCellRenderer.this, e.getX(), e.getY());
					}

					lastPoint = null;
*/
					//}
				}
			});
	}

	/**
	 * Sets up the renderer.
	 *
	 * @param table table.
	 * @param propertyCell cell
	 * @param isSelected is cell selected.
	 * @param hasFocus is cell focused
	 * @param row index
	 * @param column index
	 *
	 * @return renderer with all properties set up.
	 */
	public Component getTableCellRendererComponent(JTable table,
	    Object propertyCell, boolean isSelected, boolean hasFocus, int row,
	    int column)
	{
		super.getTableCellRendererComponent(table, propertyCell, isSelected,
		    hasFocus, row, column);

		if (propertyCell instanceof DeviceCell) {
			DeviceCell cellProperty = (DeviceCell)propertyCell;
			popup.setCell(cellProperty);

			setFont(getFont().deriveFont(Font.BOLD));
			setText(cellProperty.getIdentifier());
			if (cellProperty.isHighlightCell()) {
				setBackground(SELECTED);
			} else {
				setBackground(unselected);
			}
			
			String t= cellProperty.getTooltip();
			if (t==null) {
				t=getText();
			}
			setToolTipText(t);
		}

		logger.fine("RENDERER");

		return this;
	}

	/* (non-Javadoc)
	 * @see javax.swing.CellEditor#stopCellEditing()
	 */
	public boolean stopCellEditing()
	{
		fireCancel(new ChangeEvent(this));

		return false;
	}

	/*private void fireStop(ChangeEvent ev)
	{
		CellEditorListener[] l = (CellEditorListener[])listeners.toArray();

		for (int i = 0; i < l.length; i++) {
			try {
				l[i].editingStopped(ev);
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
	}*/

	private void fireCancel(ChangeEvent ev)
	{
		CellEditorListener[] l = (CellEditorListener[])listeners.toArray();

		for (int i = 0; i < l.length; i++) {
			try {
				l[i].editingCanceled(ev);
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
	}

	private ObjectList listeners = new ObjectList(CellEditorListener.class);

	/* (non-Javadoc)
	 * @see javax.swing.table.TableCellEditor#getTableCellEditorComponent(javax.swing.JTable, java.lang.Object, boolean, int, int)
	 */
	public Component getTableCellEditorComponent(JTable table, Object value,
	    boolean isSelected, int row, int column)
	{
		super.getTableCellRendererComponent(table, value, isSelected, true,
		    row, column);

		if (value instanceof DeviceCell) {
			DeviceCell cellProperty = (DeviceCell)value;
			popup.setCell(cellProperty);

			setFont(getFont().deriveFont(Font.BOLD));
			setText(cellProperty.getIdentifier());
			setBackground(SELECTED);

			String t=cellProperty.getTooltip();
			if (t==null) {
			if (cellProperty.getCommands() != null
			    && cellProperty.getCommands().length > 0) {
				t="Click to send command";
			} else {
				t=getText();
			}}
			setToolTipText(t);
		}

		logger.fine("EDITOR");

		return this;
	}

	/* (non-Javadoc)
	 * @see javax.swing.CellEditor#addCellEditorListener(javax.swing.event.CellEditorListener)
	 */
	public void addCellEditorListener(CellEditorListener l)
	{
		listeners.add(l);
	}

	/* (non-Javadoc)
	 * @see javax.swing.CellEditor#cancelCellEditing()
	 */
	public void cancelCellEditing()
	{
		fireCancel(new ChangeEvent(this));
	}

	/* (non-Javadoc)
	 * @see javax.swing.CellEditor#getCellEditorValue()
	 */
	public Object getCellEditorValue()
	{
		return null;
	}

	/* (non-Javadoc)
	 * @see javax.swing.CellEditor#isCellEditable(java.util.EventObject)
	 */
	public boolean isCellEditable(EventObject anEvent)
	{
		if (anEvent instanceof MouseEvent) {
			Point p = ((MouseEvent)anEvent).getPoint();
			ObjectTable table = (ObjectTable)anEvent.getSource();
			int idx = table.rowAtPoint(p);
			return (idx == table.getSelectedRow());
		}
		return true;
	}

	/* (non-Javadoc)
	 * @see javax.swing.CellEditor#removeCellEditorListener(javax.swing.event.CellEditorListener)
	 */
	public void removeCellEditorListener(CellEditorListener l)
	{
		listeners.remove(l);
	}

	/* (non-Javadoc)
	 * @see javax.swing.CellEditor#shouldSelectCell(java.util.EventObject)
	 */
	public boolean shouldSelectCell(EventObject anEvent)
	{
		return true;
	}
	
}

/* __oOo__ */
