/*
 * Copyright (c) 2004 by Cosylab d.o.o.
 *
 * The full license specifying the redistribution, modification, usage and other
 * rights and obligations is included with the distribution of this project in
 * the file license.html. If the license is not included you may find a copy at
 * http://www.cosylab.com/legal/abeans_license.htm or may write to Cosylab, d.o.o.
 *
 * THIS SOFTWARE IS PROVIDED AS-IS WITHOUT WARRANTY OF ANY KIND, NOT EVEN THE
 * IMPLIED WARRANTY OF MERCHANTABILITY. THE AUTHOR OF THIS SOFTWARE, ASSUMES
 * _NO_ RESPONSIBILITY FOR ANY CONSEQUENCE RESULTING FROM THE USE, MODIFICATION,
 * OR REDISTRIBUTION OF THIS SOFTWARE.
 */

package com.cosylab.gui.components.table.renderers;

import com.cosylab.gui.components.numberfield.DoubleDocument;
import com.cosylab.gui.components.table.cells.DoubleCell;

import com.cosylab.util.FormatCache;
import com.cosylab.util.PrintfFormat;
import com.cosylab.util.StringUtilities;

import java.awt.Component;

import java.text.DecimalFormat;

import java.util.EventObject;

import javax.swing.JTable;


/**
 * Edits DoubleCell value.
 *
 * @author <a href="mailto:igor.kriznar@cosylab.com">Igor Kriznar</a>
 * @version $id$
 */
public class DoubleCellEditor extends JTextFieldCellEditor
{
	private DoubleDocument document;
	private boolean active;
	private DecimalFormat f = new DecimalFormat("#.####################");

	/**
	 * Creates a new DoubleCellEditor object.
	 */
	public DoubleCellEditor()
	{
		setDocument(document = new DoubleDocument());
	}

	/**
	 * @see javax.swing.table.TableCellEditor#getTableCellEditorComponent(javax.swing.JTable,
	 *      java.lang.Object, boolean, int, int)
	 */
	public Component getTableCellEditorComponent(JTable table, Object cell,
	    boolean isSelected, int row, int column)
	{
		if (cell instanceof DoubleCell) {
			active = !((DoubleCell)cell).isSuspended();

			document.setMaxValue(((DoubleCell)cell).getMaximum());
			document.setMinValue(((DoubleCell)cell).getMinimum());

			double d = ((DoubleCell)cell).getDoubleValue();

			if (d < 1e20 && d > 1e-20) {
				if (active) {
					setText(f.format(d));
				} else {
					setText("");
				}
			} else {
				PrintfFormat formatter = FormatCache.getFormatter(((DoubleCell)cell)
					    .getFormat());

				if (formatter == null) {
					formatter = FormatCache.getDefaultFormatter();
				}

				if (active) {
					setText(formatter.sprintf(d));
				} else {
					setText("");
				}
			}

			selectAll();
		}

		return this;
	}

	/* (non-Javadoc)
	 * @see javax.swing.DefaultCellEditor#getCellEditorValue()
	 */
	public Object getCellEditorValue()
	{
		return document.getValue();
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.components.table.renderers.JTextFieldCellEditor#isValueValid()
	 */
	protected boolean isValueValid()
	{
		return StringUtilities.isTrueNumber(getText());
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.components.table.renderers.JTextFieldCellEditor#isCellEditable(java.util.EventObject)
	 */
	public boolean isCellEditable(EventObject anEvent)
	{
		//	    if (!active) {
		//	        return false;
		//		}
		return super.isCellEditable(anEvent);
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @param args DOCUMENT ME!
	 */
	public static void main(String[] args)
	{
		PrintfFormat formatter = FormatCache.getFormatter("%.f");

		System.out.println(formatter.sprintf(1e-20));

		DecimalFormat f = new DecimalFormat("#.####################");

		System.out.println(f.format(1e20));
	}
}

/* __oOo__ */
