/*
 * Copyright (c) 2004 by Cosylab d.o.o.
 *
 * The full license specifying the redistribution, modification, usage and other
 * rights and obligations is included with the distribution of this project in
 * the file license.html. If the license is not included you may find a copy at
 * http://www.cosylab.com/legal/abeans_license.htm or may write to Cosylab, d.o.o.
 *
 * THIS SOFTWARE IS PROVIDED AS-IS WITHOUT WARRANTY OF ANY KIND, NOT EVEN THE
 * IMPLIED WARRANTY OF MERCHANTABILITY. THE AUTHOR OF THIS SOFTWARE, ASSUMES
 * _NO_ RESPONSIBILITY FOR ANY CONSEQUENCE RESULTING FROM THE USE, MODIFICATION,
 * OR REDISTRIBUTION OF THIS SOFTWARE.
 */

package com.cosylab.gui.components.table.renderers;

import com.cosylab.gui.components.table.cells.DoubleCell;

import com.cosylab.util.FormatCache;
import com.cosylab.util.PrintfFormat;

import java.awt.Component;

import javax.swing.JTable;


/**
 * This is the default renderer for ObjectTable cells implementing DoubleCell
 * interface.
 *
 * @author <a href="mailto:ales.pucelj@cosylab.com">Ales Pucelj</a>
 * @version $id$
 */
public class DoubleCellRenderer extends DefaultTableCellRenderer
{
	
	/**
	 * 
	 */
	public DoubleCellRenderer() {
		this(false,true);
	}
	/**
			 *
			 */
	public DoubleCellRenderer(boolean decorateBorder, boolean decorateBackground)
	{
		super(decorateBorder,decorateBackground);
		setOpaque(true);
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @param table JTable
	 * @param cell Object
	 * @param isSelected boolean
	 * @param hasFocus boolean
	 * @param row int
	 * @param column int
	 *
	 * @return Component
	 *
	 * @see javax.swing.table.TableCellRenderer#getTableCellRendererComponent(JTable,
	 *      Object, boolean, boolean, int, int)
	 */
	public Component getTableCellRendererComponent(JTable table, Object cell,
	    boolean isSelected, boolean hasFocus, int row, int column)
	{
		super.getTableCellRendererComponent(table, cell, isSelected, hasFocus,
		    row, column);

		if (cell instanceof DoubleCell) {
			if (((DoubleCell)cell).isSuspended()) {
				setText("");
			} else {
				PrintfFormat formatter = FormatCache.getFormatter(((DoubleCell)cell)
					    .getFormat());

				if (formatter == null) {
					formatter = FormatCache.getDefaultFormatter();
				}
				
				String value = formatter.sprintf(((DoubleCell)cell).getDoubleValue());
				if (((DoubleCell)cell).isUnitsVisible()) {
					value += " " + ((DoubleCell)cell).getUnits();
				}
				setText(value);
			}
		}

		return this;
	}
}

/* __oOo__ */
