/*
 * @@COPYRIGHT@@
 */
package com.cosylab.gui.components.table.renderers;

import java.awt.Component;

import javax.swing.JLabel;
import javax.swing.JTable;
import javax.swing.table.TableCellRenderer;

/**
 * This renderer is used to display cells whose contents are null. It accepts no
 * parameters an is rendered as a blank cell. Purpose of this class is to 
 * simplify drawing of cells when there could be null contents in the 
 * TableModel.
 * 
 * @author <a href="mailto:ales.pucelj@cosylab.com">Ales Pucelj</a>  
 * @version $id$
 */
public class EmptyCellRenderer extends JLabel implements TableCellRenderer {

	/**
	 * Default constructor for EmptyCellRenderer.
	 */
	public EmptyCellRenderer() {
		setOpaque(true);
	}
	/**
	 * description
	 * 
	 * @param table JTable
	 * @param cell Object
	 * @param isSelected boolean
	 * @param hasFocus boolean
	 * @param row int
	 * @param column int
	 * @return Component
	 * @see javax.swing.table.TableCellRenderer#getTableCellRendererComponent(JTable, Object, boolean, boolean, int, int)
	 */
	public Component getTableCellRendererComponent(
		JTable table,
		Object cell,
		boolean isSelected,
		boolean hasFocus,
		int row,
		int column) {

		if (isSelected) {
			setBackground(table.getSelectionBackground());
			setForeground(table.getSelectionForeground());
		} else {
			setBackground(table.getBackground());
			setForeground(table.getForeground());
		}


		return this;			
	}

}
