/*
 * Copyright (c) 2004 by Cosylab d.o.o.
 *
 * The full license specifying the redistribution, modification, usage and other
 * rights and obligations is included with the distribution of this project in
 * the file license.html. If the license is not included you may find a copy at
 * http://www.cosylab.com/legal/abeans_license.htm or may write to Cosylab, d.o.o.
 *
 * THIS SOFTWARE IS PROVIDED AS-IS WITHOUT WARRANTY OF ANY KIND, NOT EVEN THE
 * IMPLIED WARRANTY OF MERCHANTABILITY. THE AUTHOR OF THIS SOFTWARE, ASSUMES
 * _NO_ RESPONSIBILITY FOR ANY CONSEQUENCE RESULTING FROM THE USE, MODIFICATION,
 * OR REDISTRIBUTION OF THIS SOFTWARE.
 */

package com.cosylab.gui.components.table.renderers;

import com.cosylab.util.ObjectList;

import java.awt.Component;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;

import java.util.EventObject;

import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.event.CellEditorListener;
import javax.swing.event.ChangeEvent;
import javax.swing.table.TableCellEditor;


/**
 * Abstract editor, which allows edited value to be applied only when enter is
 * pressed.
 *
 * @author <a href="mailto:igor.kriznar@cosylab.com">Igor Kriznar</a>
 * @version $id$
 */
public abstract class JTextFieldCellEditor extends JTextField
	implements TableCellEditor
{
	private ObjectList listeners = new ObjectList(CellEditorListener.class);
	private boolean firstGain=true;
	/**
	 * Creates a new DoubleCellEditor object.
	 */
	public JTextFieldCellEditor()
	{
		/*addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e)
				{
					//System.out.println("ACTION "+e);
					if (isValueValid()) {
						fireStop(new ChangeEvent(JTextFieldCellEditor.this));
					} else {
						fireCancle(new ChangeEvent(JTextFieldCellEditor.this));
					}
				}
			});*/
		/*addKeyListener(new KeyListener() {
			public void keyPressed(KeyEvent e) {
				System.out.println("PRESSED "+e.getKeyChar());
			}

			public void keyReleased(KeyEvent e) {
				System.out.println("RELEASED "+e.getKeyChar());
			}

			public void keyTyped(KeyEvent e) {
				System.out.println("TYPED "+e.getKeyChar());
			}
		});*/
		addFocusListener(new FocusListener() {
			public void focusGained(FocusEvent e) {
				//System.out.println("FOCUS GAINED");
				if (!firstGain) fireCancle(new ChangeEvent(this));
				else firstGain=false;
			}

			public void focusLost(FocusEvent e) {
				firstGain=true;
				//System.out.println("FOCUS LOST");
				fireCancle(new ChangeEvent(this));
			}
		});
		/*addMouseListener(new MouseListener() {
			public void mouseClicked(MouseEvent e) {
				System.out.println("M CLICKED");
			}

			public void mouseEntered(MouseEvent e) {
				System.out.println("M ENTERED");
			}

			public void mouseExited(MouseEvent e) {
				System.out.println("M EXITED");
			}

			public void mousePressed(MouseEvent e) {
				System.out.println("M PRESSED");
			}

			public void mouseReleased(MouseEvent e) {
				System.out.println("M RELEASED");
			}
		});*/
		
	}

	/**
	 * @see javax.swing.table.TableCellEditor#getTableCellEditorComponent(javax.swing.JTable,
	 *      java.lang.Object, boolean, int, int)
	 */
	public Component getTableCellEditorComponent(JTable table, Object cell,
	    boolean isSelected, int row, int column)
	{
		return this;
	}

	/* (non-Javadoc)
	 * @see javax.swing.DefaultCellEditor#stopCellEditing()
	 */
	public boolean stopCellEditing()
	{
		firstGain=true;
		//Thread.dumpStack();
		fireStop(new ChangeEvent(this));

		return false;
	}

	/* (non-Javadoc)
	 * @see javax.swing.CellEditor#cancelCellEditing()
	 */
	public void cancelCellEditing()
	{
		firstGain=true;
		//Thread.dumpStack();
		fireCancle(new ChangeEvent(this));
	}

	/* (non-Javadoc)
	 * @see javax.swing.CellEditor#isCellEditable(java.util.EventObject)
	 */
	public boolean isCellEditable(EventObject anEvent)
	{
		return true;
	}

	/* (non-Javadoc)
	 * @see javax.swing.CellEditor#shouldSelectCell(java.util.EventObject)
	 */
	public boolean shouldSelectCell(EventObject anEvent)
	{
		return true;
	}

	/* (non-Javadoc)
	 * @see javax.swing.CellEditor#addCellEditorListener(javax.swing.event.CellEditorListener)
	 */
	public void addCellEditorListener(CellEditorListener l)
	{
		listeners.add(l);
	}

	private void fireCancle(ChangeEvent ev)
	{
		//System.out.println("CANCLE "+ev);
		CellEditorListener[] l = (CellEditorListener[])listeners.toArray();
	    
		for (int i = 0; i < l.length; i++) {
			try {
				l[i].editingCanceled(ev);
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
	}

	private void fireStop(ChangeEvent ev)
	{
		//System.out.println("STOP "+ev);
		CellEditorListener[] l = (CellEditorListener[])listeners.toArray();

		for (int i = 0; i < l.length; i++) {
			try {
				l[i].editingStopped(ev);
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
	}

	/* (non-Javadoc)
	 * @see javax.swing.CellEditor#removeCellEditorListener(javax.swing.event.CellEditorListener)
	 */
	public void removeCellEditorListener(CellEditorListener l)
	{
		listeners.remove(l);
	}

	/**
	 * Should return <code>true</code> only if edited value is valid and can be
	 * applied to table cell.  Default implementation returns
	 * <code>true</code> allways. Override this method to provide own
	 * implementation.
	 *
	 * @return <code>true</code> only if value can be applied
	 */
	protected boolean isValueValid()
	{
		return true;
	}
}

/* __oOo__ */
