/*
 * Copyright (c) 2004 by Cosylab d.o.o.
 *
 * The full license specifying the redistribution, modification, usage and other
 * rights and obligations is included with the distribution of this project in
 * the file license.html. If the license is not included you may find a copy at
 * http://www.cosylab.com/legal/abeans_license.htm or may write to Cosylab, d.o.o.
 *
 * THIS SOFTWARE IS PROVIDED AS-IS WITHOUT WARRANTY OF ANY KIND, NOT EVEN THE
 * IMPLIED WARRANTY OF MERCHANTABILITY. THE AUTHOR OF THIS SOFTWARE, ASSUMES
 * _NO_ RESPONSIBILITY FOR ANY CONSEQUENCE RESULTING FROM THE USE, MODIFICATION,
 * OR REDISTRIBUTION OF THIS SOFTWARE.
 */

package com.cosylab.gui.components.table.renderers;

import com.cosylab.gui.components.numberfield.LongDocument;
import com.cosylab.gui.components.table.cells.LongCell;

import com.cosylab.util.StringUtilities;

import java.awt.Component;

import javax.swing.JTable;


/**
 * Edits LongCell value.
 *
 * @author <a href="mailto:igor.kriznar@cosylab.com">Igor Kriznar</a>
 * @version $id$
 */
public class LongCellEditor extends JTextFieldCellEditor
{
	private LongDocument document;
	private boolean active;

	/**
	 * Creates a new DoubleCellEditor object.
	 */
	public LongCellEditor()
	{
		setDocument(document = new LongDocument());
	}

	/**
	 * @see javax.swing.table.TableCellEditor#getTableCellEditorComponent(javax.swing.JTable,
	 *      java.lang.Object, boolean, int, int)
	 */
	public Component getTableCellEditorComponent(JTable table, Object cell,
	    boolean isSelected, int row, int column)
	{
		if (cell instanceof LongCell) {
			active = !((LongCell)cell).isSuspended();

			document.setMaxValue(((LongCell)cell).getMaximum());
			document.setMinValue(((LongCell)cell).getMinimum());

			if (active) {
				setText(Long.toString(((LongCell)cell).getLongValue()));
			} else {
				setText("");
			}

			selectAll();
		}

		return super.getTableCellEditorComponent(table, cell, isSelected, row,
		    column);
	}

	/* (non-Javadoc)
	 * @see javax.swing.DefaultCellEditor#getCellEditorValue()
	 */
	public Object getCellEditorValue()
	{
		return document.getValue();
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.components.table.renderers.JTextFieldCellEditor#isValueValid()
	 */
	protected boolean isValueValid()
	{
		return StringUtilities.isTrueNumber(getText());
	}
}

/* __oOo__ */
