/*
 * Copyright (c) 2004 by Cosylab d.o.o.
 *
 * The full license specifying the redistribution, modification, usage and other
 * rights and obligations is included with the distribution of this project in
 * the file license.html. If the license is not included you may find a copy at
 * http://www.cosylab.com/legal/abeans_license.htm or may write to Cosylab, d.o.o.
 *
 * THIS SOFTWARE IS PROVIDED AS-IS WITHOUT WARRANTY OF ANY KIND, NOT EVEN THE
 * IMPLIED WARRANTY OF MERCHANTABILITY. THE AUTHOR OF THIS SOFTWARE, ASSUMES
 * _NO_ RESPONSIBILITY FOR ANY CONSEQUENCE RESULTING FROM THE USE, MODIFICATION,
 * OR REDISTRIBUTION OF THIS SOFTWARE.
 */

package com.cosylab.gui.components.table.renderers;

import com.cosylab.gui.components.table.cells.PatternCell;
import com.cosylab.gui.components.util.PaintHelper;
import com.cosylab.util.BitSetUtilities;

import java.awt.Color;
import java.awt.Component;
import java.awt.Graphics;
import java.awt.Point;
import java.awt.event.MouseEvent;

import javax.swing.JTable;
import javax.swing.ToolTipManager;


/**
 * Renderer for <code>com.cosylab.gui.components.table.cells.PatternCell</code>
 * cells. Divides available area into horizontal regions and displays values
 * as rectangular areas matching the color specified in <code>PatternCell
 * </code>.
 *
 * @author <a href="mailto:ales.pucelj@cosylab.com">Ales Pucelj</a>
 * @version $id$
 */
public class PatternCellRenderer extends DefaultTableCellRenderer
{
	protected TimeoutBorder border = new TimeoutBorder();
	private Point position = null;
	private int size;
	private int h;
	private int w;
	private long value;
	private Color[] colorsOn;
	private Color[] colorsOff;
	private String[] descriptions;
	public static final Color DEFAULT_ON = Color.GREEN;
	public static final Color DEFAULT_OFF = Color.GRAY;

	/**
	 * Creates a new PatternCellRenderer object.
	 */
	public PatternCellRenderer()
	{
		super(true, false);
		setToolTipText("");
		w = getWidth();
		ToolTipManager.sharedInstance().setInitialDelay(0);
		ToolTipManager.sharedInstance().setDismissDelay(1000);

		//setBorder(border);
	}

	/**
	 * Sets up the renderer.
	 *
	 * @param table table.
	 * @param patternCell cell.
	 * @param isSelected isSelected.
	 * @param hasFocus hasFocus.
	 * @param row index.
	 * @param column index.
	 *
	 * @return properly set up component.
	 */
	public Component getTableCellRendererComponent(JTable table,
	    Object patternCell, boolean isSelected, boolean hasFocus, int row,
	    int column)
	{
		super.getTableCellRendererComponent(table, patternCell, isSelected,
		    hasFocus, row, column);

		if (patternCell instanceof PatternCell) {
			PatternCell cell = (PatternCell)patternCell;
			descriptions = cell.getDescriptions();
			value = cell.getLongValue();
			
			if (descriptions == null) {
				size = BitSetUtilities.forLong(value).length();
				if (size == 0) size = 1;
			} else {
				size = descriptions.length;
			}
			
			colorsOn = cell.getColorsWhenOn();
			colorsOff = cell.getColorsWhenOff();
	
			//border.setEnabled(cell.isTimeout());
				
			setText("");
		}
		return this;
	}

	/**
	 * Returns tool tip text based on area the mouse is over.
	 *
	 * @param e event
	 *
	 * @return description of selected led.
	 */
	public String getToolTipText(MouseEvent e)
	{
		position = e.getPoint();

		for (int i = 0; i < size; i++) {
			if (position.getX() > (i * w) / size
			    && position.getX() < ((i + 1) * w) / size) {
				if (descriptions != null) {
					return descriptions[i];
				} else {
					return String.valueOf(i);
				}
			}
		}

		return null;
	}

	/**
	 * Paints the component.
	 *
	 * @param g graphics
	 */
	public void paintComponent(Graphics g)
	{
		paintSuperComponent(g);

		if (active) {
			w = getWidth() - 1;
			h = getHeight() - 1;

			for (int i = 0; i < size; i++) {
				if ((value & 1) == 1) {
					if (colorsOn != null && colorsOn.length > i) {
						g.setColor(colorsOn[i]);
					} else {
						g.setColor(DEFAULT_ON);
					}					
				} else {
					if (colorsOn != null && colorsOn.length > i) {
						g.setColor(colorsOff[i]);
					} else {
						g.setColor(DEFAULT_OFF);
					}
				}

				value >>= 1;
				int bin= w/size+1;
				g.fillRect((i * w) / size, 0, bin, h);
				g.setColor(Color.WHITE);
				g.drawRect((i * w) / size, 0, bin, h);
			}
		}

		paintDecorations(g);

		if (alarm) {
				PaintHelper.paintRectangle(g, 2, 2, getWidth()-5, getHeight()-5,
				    Color.WHITE, 1);
		}
}
}

/* __oOo__ */
