/*
 * Copyright (c) 2003 by Cosylab d.o.o.
 *
 * The full license specifying the redistribution, modification, usage and other
 * rights and obligations is included with the distribution of this project in
 * the file license.html. If the license is not included you may find a copy at
 * http://www.cosylab.com/legal/abeans_license.htm or may write to Cosylab, d.o.o.
 *
 * THIS SOFTWARE IS PROVIDED AS-IS WITHOUT WARRANTY OF ANY KIND, NOT EVEN THE
 * IMPLIED WARRANTY OF MERCHANTABILITY. THE AUTHOR OF THIS SOFTWARE, ASSUMES
 * _NO_ RESPONSIBILITY FOR ANY CONSEQUENCE RESULTING FROM THE USE, MODIFICATION,
 * OR REDISTRIBUTION OF THIS SOFTWARE.
 */

package com.cosylab.gui.components.table.renderers;

import java.awt.Color;

import javax.swing.border.LineBorder;


/**
 * Border displayed whenever timeout is detected for individual cell.
 *
 * @author <a href="mailto:ales.pucelj@cosylab.com">Ales Pucelj</a>
 * @version $id$
 */
public class TimeoutBorder extends LineBorder
{
	private Color disabledColor = Color.white;
	protected boolean enabled;
	private Color color;

	/**
	 * TimeoutBorder constructor comment.
	 */
	public TimeoutBorder()
	{
		super(new java.awt.Color(255, 90, 200), 2);
		color = lineColor;
	}

	/**
	 * Returns color of this border when cell is disabled.
	 *
	 * @return Color of this cell when disabled.
	 */
	public Color getDisabledColor()
	{
		return disabledColor;
	}

	/**
	 * Returns whether this border is enabled.
	 *
	 * @return true if enabled.
	 */
	public boolean isEnabled()
	{
		return enabled;
	}

	/**
	 * Sets the color when this border is disabled.
	 *
	 * @param newDisabledColor new disabled color.
	 */
	public void setDisabledColor(Color newDisabledColor)
	{
		disabledColor = newDisabledColor;
	}

	/**
	 * Sets whether this border is enabled.
	 *
	 * @param newEnabled boolean
	 */
	public void setEnabled(boolean newEnabled)
	{
		enabled = newEnabled;
		lineColor = enabled ? color : disabledColor;
	}
}

/* __oOo__ */
