/*
 * Created on Mar 22, 2005
 *
 * To change the template for this generated file go to
 * Window - Preferences - Java - Code Generation - Code and Comments
 */
package com.cosylab.gui.components.util;

import java.awt.Color;
import java.awt.TexturePaint;
import java.awt.geom.Rectangle2D;
import java.awt.image.BufferedImage;

/**
 * A paint which renders a 1px dither pattern of the specified color.
 * Density of the pattern is specified by the level constructor parameter.
 * All pixels that are not colored, are transparent.
 * 
 * @author <a href="mailto:miha.kadunc@cosylab.com">Miha Kadunc</a>
 */
public class DitherPaint extends TexturePaint {
    /**
     * 25% dither
     */
    public static final int DITHER_25=25;
    /**
     * 50% dither
     */
    public static final int DITHER_50=50;
    /**
     * 75% dither
     */
    public static final int DITHER_75=75;
    
    /**
     * Creates a new paint with the specified color and density level
     * @param color The opaque pixels' color
     * @param ditherLevel DITHER_25, DITHER_50 or DITHER_75 
     */
    public DitherPaint(Color color,int ditherLevel) {
        super(createImage(color,ditherLevel),new Rectangle2D.Double(0,0,2,2));
    }
    
    private static final BufferedImage createImage(Color color, int ditherLevel) {
        BufferedImage bufImg=new BufferedImage(2,2,BufferedImage.TYPE_INT_ARGB);
        switch (ditherLevel) {
        case DITHER_75:
            bufImg.setRGB(1,0,color.getRGB());
            break;
        case DITHER_50:
            bufImg.setRGB(1,1,color.getRGB());
            break;
        case DITHER_25:
            bufImg.setRGB(0,0,color.getRGB());
            break;
        default:
            break;
        }
        return bufImg;
    }
}
