package com.cosylab.gui.components.util;

import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.LayoutManager;

import javax.swing.JPanel;


/**
 * This is a specialisation of <code>JPanel</code> that displays an image in its backgorund.
 * The image is obtained from the <code>name</code> string.
 * The name specifies the location of the resource.
 *
 * @author <a href="mailto:igor.verstovsek@cosylab.com">Igor Verstovsek</a>
 * @version $id$
 */
public class ImagePanel extends JPanel {
    private String imageName;
    private Image image;

    /**
     * Default constructor. Sets the layout to null.
     */
    public ImagePanel() {
        super();
        setLayout(null);
        initialize();
    }

    /**
     * This constructor sets the layout to null.
     *
     * @param isDoubleBuffered
     */
    public ImagePanel(boolean isDoubleBuffered) {
        super(isDoubleBuffered);
        setLayout(null);
        initialize();
    }

    /**
     * @param layout
     */
    public ImagePanel(LayoutManager layout) {
        super(layout);
        initialize();
    }

    /**
     * @param layout
     * @param isDoubleBuffered
     */
    public ImagePanel(LayoutManager layout, boolean isDoubleBuffered) {
        super(layout, isDoubleBuffered);
        initialize();
    }

    /**
     * Overriden to display an image that is set as a string that points to a resource.
     *
     */
    protected void paintComponent(Graphics g) {
        super.paintComponent(g);

        if (image != null) {
            g.drawImage(image, 0, 0, this);
        }
    }

    /**
     * Returns the name of the rewource that points to an image.
     *
     * @return
     */
    public String getImageName() {
        return imageName;
    }

    /**
     * Sets the name that points to the location of the image.
     *
     * @param string
     */
    public void setImageName(String name) {
        this.imageName = name;
        image = IconHelper.createImage(name);

        //if (image == null) throw new RuntimeException("Resource \"" + name + "\" was not found.");
    }

    /**
     * Initializes the ImagePanel with the default image - KGB hand :-).
     */
    private void initialize() {
        Dimension size = new Dimension(83, 83);
        setPreferredSize(size);
        setSize(size);
        setImageName("icons/cosylab/kgb.gif");
    }
}
