/*
 * @@COPYRIGHT@@
 */
package com.cosylab.gui.components.util;

import java.awt.event.ActionEvent;

import javax.swing.AbstractAction;
import javax.swing.JSplitPane;


/**
 * An action that, when associated with a JSplitPane,
 * changes the orientation from horizontal to vertical
 * and vice versa.
 *
 * @author <a href="mailto:miha.kadunc@cosylab.com">Miha Kadunc</a>
 * @version $id$
 */
public class SplitPaneOrientationAction extends AbstractAction {
    private JSplitPane split;

    /**
     * Creates a new action, associated with the <code>pane</code> to
     * change its orientation.
     */
    public SplitPaneOrientationAction(JSplitPane pane) {
        super();
        split = pane;
        putValue(AbstractAction.SHORT_DESCRIPTION, "Change Orientation");
        putValue(AbstractAction.LONG_DESCRIPTION,
            "Change orientation of the split pane");
        putValue(AbstractAction.NAME, "Change Orientation");
        putValue(AbstractAction.SMALL_ICON,
            IconHelper.createIcon(
                "icons/window/ChangeOrientation16.gif"));
    }

    /**
     * Called when the action is performed. Switches the orientation.
     */
    public void actionPerformed(ActionEvent e) {
        if (split.getOrientation() == JSplitPane.HORIZONTAL_SPLIT) {
            split.setOrientation(JSplitPane.VERTICAL_SPLIT);
        } else {
            split.setOrientation(JSplitPane.HORIZONTAL_SPLIT);
        }
    }
}
