/*
 * @@COPYRIGHT@@
 */
package com.cosylab.gui.components.wheelswitch;

import java.awt.Color;
import java.awt.Dimension;

import javax.swing.JComponent;
import javax.swing.event.EventListenerList;

import com.cosylab.gui.components.ArrowButton;
import com.cosylab.gui.components.util.ColorHelper;

/**
 * Descedant of <code>javax.swing.JComponent</code> that contains two 
 * <code>ArrowButton</code>s acting as a two-way (up/down) control. 
 * 
 * @author <a href="mailto:jernej.kamenik@cosylab.com">Jernej Kamenik</a>
 * @version $id$
 */
public class UpDownButton extends JComponent {
	private ArrowButton down;

	private ArrowButton up;

	/**
	 * Constructor for UpDownButton.
	 */			
	public UpDownButton() {
		super();
		listenerList = new EventListenerList();	
		setLayout(new UpDownLayout());
		add(getUpButton());
		add(getDownButton());
	}		

  /**
   * This method was overridden to implement correct 
   * <code>UpDownButton<code> placement inside the 
   * <code>Wheelswitch</code> container.
   * 
   * @see JComponent#getPreferredSize()
   */
	public Dimension getPreferredSize() {
		return new Dimension(12,24);
	}
	
  /**
   * This method was overridden to implement correct 
   * <code>UpDownButton<code> placement inside the 
   * <code>Wheelswitch</code> container.
   * 
   * @see JComponent#getMinimumSize()
   */
	public Dimension getMinimumSize() {
		return new Dimension(12,24);
	}	
	/* (non-Javadoc)
	 * @see java.awt.Component#setEnabled(boolean)
	 */
	public void setEnabled(boolean arg0) {
		super.setEnabled(arg0);
		getDownButton().setEnabled(arg0);
		getUpButton().setEnabled(arg0);
	}

	/**
	 * @param i
	 */
	public void setOutOfBounds(int i) {
		if (i>0) {
			getUpButton().setEnabled(false);
			if (isEnabled())getDownButton().setEnabled(true);
			getUpButton().setForeground(Color.BLACK);
			getDownButton().setForeground(ColorHelper.getAlarmOutline());
		} else if (i<0) {
			if (isEnabled())getUpButton().setEnabled(true);
			getDownButton().setEnabled(false);
			getUpButton().setForeground(ColorHelper.getAlarmOutline());
			getDownButton().setForeground(Color.BLACK);
		} else {
			if (isEnabled()) {
				getUpButton().setEnabled(true);
				getDownButton().setEnabled(true);
			}
			getUpButton().setForeground(Color.BLACK);
			getDownButton().setForeground(Color.BLACK);
		}
	}
	
	/**
	 * Laziliy initializes the up arrow button and returns it.
	 * 
	 * @return up button
	 */
	public ArrowButton getUpButton() {
		if (up==null) {
			up = new ArrowButton(ArrowButton.UP);
			up.setFocusable(false);
			up.setBackground(ColorHelper.getCosyControl());
		}

		return up;
	}
	
	/**
	 * Laziliy initializes the down arrow button and returns it.
	 * 
	 * @return down button
	 */
	public ArrowButton getDownButton() {
		if (down==null) {
			down = new ArrowButton(ArrowButton.DOWN);
			down.setFocusable(false);
			down.setBackground(ColorHelper.getCosyControl());
		}
		
		return down;
	}
	
	@Override
	public void setBackground(Color bg) {
		super.setBackground(bg);
		getUpButton().setBackground(bg);
		getDownButton().setBackground(bg);
	}
	
	@Override
	public void setForeground(Color fg) {
		super.setForeground(fg);
		getUpButton().setForeground(fg);
		getDownButton().setForeground(fg);
	}

}

