/*
 * Copyright (c) 2003 by Cosylab d.o.o.
 *
 * The full license specifying the redistribution, modification, usage and other
 * rights and obligations is included with the distribution of this project in
 * the file license.html. If the license is not included you may find a copy at
 * http://www.cosylab.com/legal/abeans_license.htm or may write to Cosylab, d.o.o.
 *
 * THIS SOFTWARE IS PROVIDED AS-IS WITHOUT WARRANTY OF ANY KIND, NOT EVEN THE
 * IMPLIED WARRANTY OF MERCHANTABILITY. THE AUTHOR OF THIS SOFTWARE, ASSUMES
 * _NO_ RESPONSIBILITY FOR ANY CONSEQUENCE RESULTING FROM THE USE, MODIFICATION,
 * OR REDISTRIBUTION OF THIS SOFTWARE.
 */

package com.cosylab.gui.displayers;

import com.cosylab.util.CommonException;


/**
 * <p>
 * <code>NumericAsynchronousConsumer</code> is consumer, which receives update
 * request and returns some time later respons with detail information about
 * request. Update method itself returns immediatelly.
 * </p>
 * 
 * <p>
 * This asynchronous consumer is specialized for consumers, which will use
 * target value as <code>Number</code>.
 * </p>
 * 
 * <p>
 * This interface is intended to be used by <code>DataSource</code>. Displayer
 * which want to notify data source about value changes done by user need to
 * implement <code>DataSource</code> which supports this type of consumer.
 * </p>
 *
 * @author <a href="mailto:igor.kriznar@cosylab.com">Igor Kriznar</a>
 * @version $Id: AsynchronousObjectConsumer.java,v 1.1 2004/06/28 03:59:33 ikriznar Exp $
 *
 * @see com.cosylab.gui.displayers.UpdateRequest#getTargetValue()
 * @since Dec 3, 2003.
 */
public interface AsynchronousObjectConsumer extends DataConsumer
{
	/**
	 * Calls asynchronous update on this data consumer. Data consumer must
	 * return this call as soon as possible and inform
	 * <code>UpdateRequestListener</code> inside <code>UpdateRequest</code>
	 * about response.
	 *
	 * @param request the update request, holds
	 *        <code>UpdateRequestlistener</code> which asynchronously receives
	 *        reponse
	 */
	public void updateAcynchronously(UpdateRequest request)
		throws CommonException;
}

/* __oOo__ */
