/*
 * Copyright (c) 2003 by Cosylab d.o.o.
 *
 * The full license specifying the redistribution, modification, usage and other
 * rights and obligations is included with the distribution of this project in
 * the file license.html. If the license is not included you may find a copy at
 * http://www.cosylab.com/legal/abeans_license.htm or may write to Cosylab, d.o.o.
 *
 * THIS SOFTWARE IS PROVIDED AS-IS WITHOUT WARRANTY OF ANY KIND, NOT EVEN THE
 * IMPLIED WARRANTY OF MERCHANTABILITY. THE AUTHOR OF THIS SOFTWARE, ASSUMES
 * _NO_ RESPONSIBILITY FOR ANY CONSEQUENCE RESULTING FROM THE USE, MODIFICATION,
 * OR REDISTRIBUTION OF THIS SOFTWARE.
 */

package com.cosylab.gui.displayers;

import com.cosylab.gui.components.util.PopupManageable;


/**
 * <code>CommonDisplayer</code> defines interface common to multiple and single
 * displayers. Displayer interface controls GUI component, which displays
 * single or multiple data sources on screen.
 *
 * @author <a href="mailto:igor.kriznar@cosylab.com">Igor Kriznar</a>
 * @version $Id: CommonDisplayer.java,v 1.8 2006/12/12 16:20:21 ikriznar Exp $
 *
 * @since Nov 24, 2003.
 */
public interface CommonDisplayer extends PopupManageable, DataStateProvider
{
	
	/** Property name for min step */
	/**
	 * Characteristic name for prefered color. With this characteristic gives
	 * data source a hint to Displayer which color should be used to visualize
	 * the source and distingish it from other sources, if they exist.
	 */
	public static final String C_COLOR = "color";
	public static final String C_MIN_STEP="minStep";
	public static final String C_MINIMUM="minimum";
	public static final String C_MAXIMUM="maximum";
	public static final String C_FORMAT="format";
	public static final String C_UNITS="units";
	public static final String C_DISPLAY_NAME="displayName";
	public static final String C_BIT_DESCRIPTIONS="bitDescriptions";
	public static final String C_BIT_MASK="bitMask";
	public static final String C_CONDITION_WHEN_CLEARED="conditionWhenCleared";
	public static final String C_CONDITION_WHEN_SET="conditionWhenSet";
	public static final String C_GRAPH_MAX="graphMax";
	public static final String C_GRAPH_MIN="graphMin";
	public static final String C_DESCRIPTION="description";
	public static final String C_PROPERTY_TYPE="propertyType";
	public static final String C_POSITION="position";
	public static final String C_SEQUENCE_LENGTH = "sequenceLength";

	/**
	 * Performs general cleanup of displayer resources. All by displayer issued
	 * object are not valid any more after cleanup. Displayer positions itself
	 * in initial state and DataState is changed to NOT_INITIALIZED state.
	 */
	public void cleanup();

	/**
	 * Suspends all updates of displayer. Displayer might receive data change
	 * updates, but does not call redraw on GUI components, which is time and
	 * CPU consuming.
	 */
	public void suspend();

	/**
	 * Resumes updates of GUI component.
	 *
	 * @see #suspend()
	 */
	public void resume();

	/**
	 * Returns <code>true</code> if GUI updates are suspended.
	 *
	 * @return <code>true</code> if GUI updates are suspended
	 */
	public boolean isSuspended();
}

/* __oOo__ */
