/*
 * Copyright (c) 2003 by Cosylab d.o.o.
 *
 * The full license specifying the redistribution, modification, usage and other
 * rights and obligations is included with the distribution of this project in
 * the file license.html. If the license is not included you may find a copy at
 * http://www.cosylab.com/legal/abeans_license.htm or may write to Cosylab, d.o.o.
 *
 * THIS SOFTWARE IS PROVIDED AS-IS WITHOUT WARRANTY OF ANY KIND, NOT EVEN THE
 * IMPLIED WARRANTY OF MERCHANTABILITY. THE AUTHOR OF THIS SOFTWARE, ASSUMES
 * _NO_ RESPONSIBILITY FOR ANY CONSEQUENCE RESULTING FROM THE USE, MODIFICATION,
 * OR REDISTRIBUTION OF THIS SOFTWARE.
 */
package com.cosylab.gui.displayers;

import java.beans.PropertyVetoException;


/**
 * <code>DataSource</code>  is interface for generic data source. Data source
 * source dispatches dynamic value updates to registered data consumers.
 *
 * @author <a href="mailto:igor.kriznar@cosylab.com">Igor Kriznar</a>
 * @version $Id: DataSource.java,v 1.2 2006/09/07 12:08:30 ikriznar Exp $
 *
 * @see com.cosylab.gui.displayers2.DataConsumer
 * @since Nov 24, 2003.
 */
public interface DataSource {
    /**
     * Registers data consumer at this data source. This data source can denay
     * consumer by throwing <code>PropertyVetoException</code> if consumer
     * does not provide necessary <code>DataConsumer</code> type.
     *
     * @param consumer the data consumer to be regitered at this data source
     *
     * @see DataConsumer#getDataConsumer(Class)
     */
    public void addConsumer(DataConsumer consumer) throws PropertyVetoException;

    /**
     * Deregisters data consumer from this data source.
     *
     * @param consumer data consumer to be deregistered
     */
    public void removeConsumer(DataConsumer consumer);

    /**
     * Returns array of registered data consumer.
     *
     * @return array of registered data consumer
     */
    public DataConsumer[] getConsumers();

    /**
     * Returns the array of consumer types, which are acceptable by this data
     * source.
     *
     * @return the array of acceptable data source types
     */
    public Class<DataConsumer>[] getAcceptableConsumerTypes();

    /**
     * Removes all consumers from data source and releases all resources
     * and bindings allocated inside datasource to the consumers.
     *
     */
    public void removeAllConsumers();
}


/* __oOo__ */
