/*
 * Copyright (c) 2004 by Cosylab d.o.o.
 *
 * The full license specifying the redistribution, modification, usage and other
 * rights and obligations is included with the distribution of this project in
 * the file license.html. If the license is not included you may find a copy at
 * http://www.cosylab.com/legal/abeans_license.htm or may write to Cosylab, d.o.o.
 *
 * THIS SOFTWARE IS PROVIDED AS-IS WITHOUT WARRANTY OF ANY KIND, NOT EVEN THE
 * IMPLIED WARRANTY OF MERCHANTABILITY. THE AUTHOR OF THIS SOFTWARE, ASSUMES
 * _NO_ RESPONSIBILITY FOR ANY CONSEQUENCE RESULTING FROM THE USE, MODIFICATION,
 * OR REDISTRIBUTION OF THIS SOFTWARE.
 */

package com.cosylab.gui.displayers;

import java.beans.PropertyChangeListener;


/**
 * <code>DataStateProvider</code> interface marks those components, which can
 * provide <code>DataState</code> Java property to describe theier data
 * quality state.
 *
 * @author <a href="mailto:igor.kriznar@cosylab.com">Igor Kriznar</a>
 *
 * @see com.cosylab.gui.displayers.DataState
 */
public interface DataStateProvider
{
	/** Property name for data state. */
	public String DATA_STATE = "dataState";

	/**
	 * Adds property change listener, which receives notifications when data
	 * state changes.
	 *
	 * @param l new listener
	 */
	public void addPropertyChangeListener(PropertyChangeListener l);

	/**
	 * Adds property change listener, which receives notifications when data
	 * state changes.
	 *
	 * @param propertyName the property name for which events will be received
	 * @param l new listener
	 */
	public void addPropertyChangeListener(String propertyName,
	    PropertyChangeListener l);

	/**
	 * Returns data quality state. This property is changed by data consumer
	 * interface. In case of multiple displayer, this state indicates integral
	 * state for all contained data consumers.
	 *
	 * @return dynamic data quality state
	 */
	public DataState getDataState();

	/**
	 * Removes property change listener.
	 *
	 * @param l removed listener
	 */
	public void removePropertyChangeListener(PropertyChangeListener l);

	/**
	 * Removes property change listener.
	 *
	 * @param propertyName the property name for which listener is removed
	 * @param l removed listener
	 */
	public void removePropertyChangeListener(String propertyName,
	    PropertyChangeListener l);
}

/* __oOo__ */
