/*
 * Copyright (c) 2003 by Cosylab d.o.o.
 *
 * The full license specifying the redistribution, modification, usage and other
 * rights and obligations is included with the distribution of this project in
 * the file license.html. If the license is not included you may find a copy at
 * http://www.cosylab.com/legal/abeans_license.htm or may write to Cosylab, d.o.o.
 *
 * THIS SOFTWARE IS PROVIDED AS-IS WITHOUT WARRANTY OF ANY KIND, NOT EVEN THE
 * IMPLIED WARRANTY OF MERCHANTABILITY. THE AUTHOR OF THIS SOFTWARE, ASSUMES
 * _NO_ RESPONSIBILITY FOR ANY CONSEQUENCE RESULTING FROM THE USE, MODIFICATION,
 * OR REDISTRIBUTION OF THIS SOFTWARE.
 */

package com.cosylab.gui.displayers;

/**
 * A specialization of the <code>Displayer</code> interface for Java primitive
 * type <code>double</code>.
 * 
 */
public interface DoubleDisplayer extends Displayer, DoubleConsumer
{
	/**
	 * Sets the value displayed by the displayer. This method may be invoked
	 * only by the adapter delivering a new value from the modeling layer.
	 *
	 * @param value new value to be displayed
	 */
	void setValue(double value);

	/**
	 * Gets the value displayed by the displayer. This method can be invoked by
	 * any object and will probably most often be invoked by the GUI rendition
	 * code.
	 *
	 * @return double currently displayed value
	 */
	public double getValue();

	/**
	 * Returns the lower limit for the value used to render the display. This
	 * lower limit must be lower or equal to the actual limit that the value
	 * can take. If may only be lower (and not equal) in order to round the
	 * minimum to some easily depictable value.
	 *
	 * @return double the lower limit for the value rendition
	 */
	public double getMinimum();

	/**
	 * Returns the upper limit for the value used to render the display. This
	 * upper limit must be greater or equal to the actual upper limit that
	 * the value can take. It may only be greater (and not equal) in order to
	 * round the maximum to some easiliy depictable value.
	 *
	 * @return double the upper limit for the value rendition
	 */
	public double getMaximum();

	/**
	 * Returns the physical units of this value represented by a string.
	 *
	 * @return String the units of the value, to be rendered, if possible, near
	 *         the actual value
	 */
	public String getUnits();

	/**
	 * Returns the C-style format string that is used to format the
	 * <code>double</code> value into text, if the displayer must perform such
	 * rendition.
	 *
	 * @return String printf style format specification
	 */
	public String getFormat();

	/**
	 * Sets the minimum value used for displaying in range.
	 *
	 * @param double new displaying minimum
	 *
	 * @see #getMinimum
	 */
	public void setMinimum(double value);

	/**
	 * Sets the maximum value used for displaying in range. 
	 *
	 * @param double new displaying maximum
	 *
	 * @see #getGraphMax
	 */
	public void setMaximum(double value);

	/**
	 * Sets the physical units for this value. Manly called by the adapter or
	 * user during  design time.
	 *
	 * @param String units for the value
	 *
	 * @see #getUnits
	 */
	public void setUnits(String value);

	/**
	 * Sets the C printf style format specification used to render double value
	 * into a String. Mainly called by the adapter or user during design time.
	 *
	 * @param String C-style format specification
	 *
	 * @see #getFormat
	 */
	public void setFormat(String value);
}

/* __oOo__ */
