package com.cosylab.gui.displayers;

import javax.swing.Icon;

import com.cosylab.gui.ValueIconPair;

/**
 * <code>IconDisplayer</code> can display long values (similar to LongDisplayer) 
 * but instead of showing the number value it shows different icons for each of
 * the received values. 
 * 
 * @author Jaka Bobnar, Cosylab
 *
 */
public interface IconDisplayer extends Displayer, LongConsumer {

	/**
	 * Sets the value displayed by the displayer. This method may be invoked
	 * only by the adapter delivering a new value from the modeling layer.
	 *
	 * @param value new value to be displayed
	 */
	public void setValue(long value);

	/**
	 * Gets the value displayed by the displayer. This method can be invoked by
	 * any object and will probably most often be invoked by the GUI rendition
	 * code.
	 *
	 * @return double currently displayed value
	 */
	public long getValue();
	
	/**
	 * Sets the default icon for this displayer. If there is no icon assigned to a specific
	 * value, a default icon is shown when that value arrives.
	 * 
	 * @param icon
	 */
	public void setDefaultIcon(Icon icon);
	
	/**
	 * Returns the default icon.
	 * 
	 * @return
	 */
	public Icon getDefaultIcon();
	
	/**
	 * Returns an array of all icons that were associated with values.
	 * 
	 * @return
	 */
	public ValueIconPair[] getIcons();
	
	/**
	 * Associates icons with values. This method overrides all previously loaded icons.
	 * 
	 * @param icons
	 */
	public void setIcons(ValueIconPair[] icons);
	
	/**
	 * Associates icons with values. This methods adds icons to previously
	 * loded inventory and overrides only those values that were loaded before and now
	 * is a different values associated with them. 
	 * 
	 * @param icons
	 */
	public void addIcons(ValueIconPair[] icons);
	
	/**
	 * Associates icon with value. This methods adds icon to previously
	 * loded inventory. 
	 * 
	 * @param icons
	 */
	public void addIcon(ValueIconPair doublet); 
	
	/**
	 * Removes icons.
	 * 
	 * @param icons
	 */
	public void removeIcon(ValueIconPair icons);
	
	/**
	 * Removes icons.
	 * 
	 * @param icons
	 */
	public void removeIcons(ValueIconPair[] icons);
}
