/*
 * Copyright (c) 2003 by Cosylab d.o.o.
 *
 * The full license specifying the redistribution, modification, usage and other
 * rights and obligations is included with the distribution of this project in
 * the file license.html. If the license is not included you may find a copy at
 * http://www.cosylab.com/legal/abeans_license.htm or may write to Cosylab, d.o.o.
 *
 * THIS SOFTWARE IS PROVIDED AS-IS WITHOUT WARRANTY OF ANY KIND, NOT EVEN THE
 * IMPLIED WARRANTY OF MERCHANTABILITY. THE AUTHOR OF THIS SOFTWARE, ASSUMES
 * _NO_ RESPONSIBILITY FOR ANY CONSEQUENCE RESULTING FROM THE USE, MODIFICATION,
 * OR REDISTRIBUTION OF THIS SOFTWARE.
 */

package com.cosylab.gui.displayers;

/**
 * <code>MultipleDataConsumer</code> is data consumer, which is able to process
 * at the same time several dynamic data sources by allocation own simple data
 * consumer for each source.
 *
 * @author <a href="mailto:igor.kriznar@cosylab.com">Igor Kriznar</a>
 * @version $Id: MultipleDataConsumer.java,v 1.2 2006/09/07 12:08:30 ikriznar Exp $
 *
 * @since Nov 24, 2003.
 */
public interface MultipleDataConsumer
{
	/**
	 * <p>
	 * Returns data consumer of specified data consumer type and for specified
	 * string name. The organization of data is indicated with name (list,
	 * table, tree organization). Multiple consumer might interprete this by
	 * it's own preferences or abilities. The name of returned consuer must
	 * equals to provided name.
	 * </p>
	 * 
	 * <p>
	 * Example. Names: <tt>PS1/current, PS1/readback, PS2/current,
	 * PS2/readback</tt> migth table displayer interprete as two rows (PS1 and
	 * PS2) wtih two columns (current and readback). Tree displayer migth
	 * interprete this names as two branches with two subbranches.
	 * </p>
	 * 
	 * <p>
	 * Note that name is not fixed consumer property. Some displayers migth
	 * expect for name to be same as DISPLAY_NAME and is changed if display
	 * name is changed.
	 * </p>
	 *
	 * @param name organized data name
	 * @param type required data soncumer type
	 *
	 * @return newly allocated consumer for required name and type
	 */
	public <D extends DataConsumer> D getConsumer(String name, Class<D> type);

	/**
	 * Requests frm this multiple consumer to release all resources connectet
	 * to provided single consumer. Provided consumer must be created by this
	 * multiple consumer.
	 *
	 * @param consumer the consumer to be released
	 */
	public void releaseConsumer(DataConsumer consumer);

	/**
	 * Returns all data consumers created by this multiple data consumer.
	 *
	 * @return all data consumers created by this multiple data consumer
	 */
	public DataConsumer[] getConsumers();

	/**
	 * Returns array of supported data consumer types, which can be used as
	 * parameter and returnned  with <code>getConsumer(String,Class)</code>
	 * method.
	 *
	 * @return array of supported data consumer types
	 */
	public Class<DataConsumer>[] getSupportedConsumerTypes();
}

/* __oOo__ */
