/*
 * Copyright (c) 2003 by Cosylab d.o.o.
 *
 * The full license specifying the redistribution, modification, usage and other
 * rights and obligations is included with the distribution of this project in
 * the file license.html. If the license is not included you may find a copy at
 * http://www.cosylab.com/legal/abeans_license.htm or may write to Cosylab, d.o.o.
 *
 * THIS SOFTWARE IS PROVIDED AS-IS WITHOUT WARRANTY OF ANY KIND, NOT EVEN THE
 * IMPLIED WARRANTY OF MERCHANTABILITY. THE AUTHOR OF THIS SOFTWARE, ASSUMES
 * _NO_ RESPONSIBILITY FOR ANY CONSEQUENCE RESULTING FROM THE USE, MODIFICATION,
 * OR REDISTRIBUTION OF THIS SOFTWARE.
 */

package com.cosylab.gui.displayers;

import com.cosylab.util.CommonException;


/**
 * <code>NumberConsumer</code> is data consumer for receiveing updates of
 * <code>Number</code> dynamic values, which will block call until update is
 * completer and throws exception on fail.
 * 
 * <p>
 * This interface is intended to be used by <code>DataSource</code>. Displayer
 * which want to notify data source about value changes done by user need to
 * implement <code>DataSource</code> which supports this type of consumer.
 * </p>
 *
 * @author <a href="mailto:igor.kriznar@cosylab.com">Igor Kriznar</a>
 * @version $Id: SynchronousNumberConsumer.java,v 1.1 2004/06/28 03:59:33 ikriznar Exp $
 *
 * @since Nov 24, 2003.
 */
public interface SynchronousNumberConsumer extends DataConsumer
{
	/**
	 * Notifys this consumer about new dynamic value update.
	 *
	 * @param timestamp the time of dynamic value event
	 * @param value new value
	 */
	public void updateSynchronously(long timestamp, Number value)
		throws CommonException;
}

/* __oOo__ */
