/*
 * @@COPYRIGHT@@
 */
package com.cosylab.gui.property.editors;

import java.beans.PropertyChangeListener;



/**
 * description
 * 
 * @author <a href="mailto:ales.pucelj@cosylab.com">Ales Pucelj</a> 
 * @version $id$
 */
public interface PropertyEditor {
	public static final String PROPERTY_VALUE_NAME="propertyValue";
	/**
	 * Returns the value contained in this editor. To provide generic approach,
	 * value must be contained within Object, primitive types should be wrapped
	 * within appropriate classes (int - Integer). Return value of null should be
	 * used only to indicate, the value contained within this editor is invalid.
	 * 
	 * @return Object
	 */	
	public Object getPropertyValue();

	/**
	 * PropertyEditor should set the value specified and display it accordingly.
	 * Return value show whether the value could be set. Return value should be true
	 * only, it the editor was able to set and display value. If the value parameter
	 * does not match the editors expected type, is null or has invalid value,
	 * return value should be false.
	 * 
	 * @param value Value to set
	 * @return boolean Indicates whether set was successful.
	 */
	public boolean setPropertyValue(Object value);

	/**
	 * Returns the description for this editor. This description will be 
	 * used to describe the component in the GUI. Its actual implementation
	 * depends on the implementing class. <p>
	 * If editor is unable to display the description, it must return null. This will
	 * allow for automatic creation of description and properly wrap the methods.
	 * If editor can display description but its value has not yet been defined,
	 * this method should return empty string.
	 * 
	 * @return String
	 */	
	public String getDescription();

	public void setDescription(String description);

	/**
	 * Returns the actual editor component. Since editors can be composed of more
	 * than one visual components, the actual editor the user edits may not be
	 * the same as the wrapper class. This method must return the true editor. If
	 * there is no need for that, it should return <code>this</code>.
	 * 
	 * @return PropertyEditor
	 */
	//public PropertyEditor getEditor();
	/**
	 * Registers a PropertyChangeListener for this editor, which will be updated
	 * when a "propertyValue" property changes. 
	 * @author Miha
	 */
	public void addPropertyChangeListener(PropertyChangeListener l);
	
	/**
	 * Deregisters a PropertyChangeListener from this editor. 
	 * @author Miha
	 */
	public void removePropertyChangeListener(PropertyChangeListener l);
	
	
	
}
