/**
 * 
 */
package de.desy.acop.displayers;
  
import java.awt.Color;
import java.awt.datatransfer.DataFlavor;
import java.awt.event.ActionEvent;
import java.beans.PropertyVetoException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import javax.swing.AbstractAction;
import javax.swing.Action;

import com.cosylab.gui.adapters.Converter;
import com.cosylab.gui.components.util.CosyTransferHandler;
import com.cosylab.gui.components.util.PopupManager;
import com.cosylab.gui.displayers.DataConsumer;
import com.cosylab.gui.displayers.DataSource;
import com.cosylab.gui.displayers.DataState;
import com.cosylab.gui.displayers.DoubleSeqConsumer;
import com.cosylab.gui.displayers.MultipleDisplayer;
import com.cosylab.gui.components.util.ColorManager;
import com.cosylab.gui.util.MultipleDisplayerParametersFlavor;
import com.cosylab.gui.util.UserSettingsProtection;
import com.cosylab.util.CommonException;

import de.desy.acop.chart.Acop;
import de.desy.acop.displayers.tools.AcopChartConsumer;
import de.desy.acop.displayers.tools.AcopDisplayerParametersFlavor;
import de.desy.acop.displayers.tools.AcopDisplayerParameters;
import de.desy.acop.displayers.tools.AcopGraphParameters;
import de.desy.acop.displayers.tools.AcopGraphParametersFlavor;
import de.desy.acop.displayers.tools.AcopMultipleDisplayerInfoDialog;
import de.desy.acop.displayers.tools.ConnectionParametersFlavor;
import de.desy.acop.displayers.tools.MultipleAcopDisplayer;
import de.desy.acop.displayers.tools.AcopChartTransferHandler;
import de.desy.acop.transport.ConnectionParameters;
import de.desy.acop.transport.adapters.AcopTransportDataSource;
import de.desy.acop.transport.adapters.AdapterFactory;
import de.desy.acop.transport.adapters.AdapterFactoryService;
        
/**
 * @author ikriznar
 *  
 */
public class AcopChart extends Acop implements MultipleDisplayer, MultipleAcopDisplayer<AcopGraphParameters> {
	
	private static final long serialVersionUID = -2579109976414960938L;
	
	private int maxLinkIndex=0;
	private int suspended=0;
	private Map<String,AcopChartConsumer> consumers= new HashMap<String,AcopChartConsumer>();
	private HashMap<AcopGraphParameters, AcopChartConsumer> connections = new HashMap<AcopGraphParameters, AcopChartConsumer>();
	private DataState dataState = new DataState(DataState.NORMAL);
	private AcopMultipleDisplayerInfoDialog dialog;
	private PopupManager popupManager;
	private ColorManager colorManager = new ColorManager();
	
	public AcopChart() {
        super(); 
        DataFlavor[] flavor = new DataFlavor[]{AcopGraphParametersFlavor.FLAVOR,
        		MultipleDisplayerParametersFlavor.FLAVOR, AcopDisplayerParametersFlavor.FLAVOR,
        		ConnectionParametersFlavor.FLAVOR,
        		DataFlavor.stringFlavor};
		new AcopChartTransferHandler(this, true, true, flavor, true);
//		new AcopTransferHandler(this, true, true);
		UserSettingsProtection.setProtection(this,new String[]{"caption", "yMin", "yMax"},false);
	}
	/* (non-Javadoc)
	 * @see com.cosylab.gui.displayers.CommonDisplayer#cleanup()
	 */
	public void cleanup() {
		for(AcopChartConsumer c: consumers.values()) {
			c.destroy();
		}
		consumers.clear();
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.displayers.CommonDisplayer#suspend()
	 */
	public void suspend() {
		suspended++;
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.displayers.CommonDisplayer#resume()
	 */
	public synchronized void resume() {
		if (suspended>0) {
			suspended--;
		}
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.displayers.CommonDisplayer#isSuspended()
	 */
	public boolean isSuspended() {
		return suspended>0;
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.displayers.DataStateProvider#getDataState()
	 */
	public DataState getDataState() {
		return dataState;
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.displayers.MultipleDataConsumer#getConsumer(java.lang.String, java.lang.Class)
	 */
	public <D extends DataConsumer> D getConsumer(String name, Class<D> type) {
		
		if (type.isAssignableFrom(AcopChartConsumer.class)) {
			if (consumers.containsKey(name)) {
				return type.cast(consumers.get(name));
			}
			
			AcopChartConsumer c= new AcopChartConsumer(this, name);
			c.setColor(colorManager.pickColor());
			maxLinkIndex++;
			consumers.put(name,c);
			return type.cast(c);
		}
		return null;
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.displayers.MultipleDataConsumer#releaseConsumer(com.cosylab.gui.displayers.DataConsumer)
	 */
	public void releaseConsumer(DataConsumer consumer) {
		consumers.remove(consumer.getName()).destroy();
		updateYScaleOnRemove((AcopChartConsumer)consumer);
		updateXScaleOnRemove((AcopChartConsumer)consumer);
		colorManager.dropColor(((AcopChartConsumer)consumer).getColor());
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.displayers.MultipleDataConsumer#getConsumers()
	 */
	public DataConsumer[] getConsumers() {
		return consumers.values().toArray(new DataConsumer[consumers.size()]);
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.displayers.MultipleDataConsumer#getSupportedConsumerTypes()
	 */
	@SuppressWarnings("unchecked")
	public Class<DataConsumer>[] getSupportedConsumerTypes() {
		return new Class[]{DoubleSeqConsumer.class};
	}
	
	/*
	 * (non-Javadoc)
	 * @see de.desy.acop.displayers.AcopMultipleDisplayerConnector#addConnectionParameters(de.desy.acop.transport.ConnectionParameters)
	 */
	public void addDisplayerParameters(AcopGraphParameters parameters) throws CommonException, PropertyVetoException {
		if (parameters == null) return;
		//System.out.println("ADD: '"+parameters.toString()+"' "+parameters.hashCode());
		parameters = synchronizeGraphParameters(parameters);
		if (connections.get(parameters) != null) {
			return;
		}
		AcopGraphParameters[] old = getDisplayerParameters();
		AcopChartConsumer consumer = (AcopChartConsumer) getConsumer(parameters.toString(), DoubleSeqConsumer.class);
			
		consumer.setDisplayerParameters(parameters);
		AdapterFactory factory = AdapterFactoryService.getInstance().getAdapterFactory();
		AcopTransportDataSource ds = factory.createDataSource(parameters.getConnectionParameters());
		if (parameters.getConverter() != null){
			Converter c = parameters.getConverter();
			c.addConsumer(consumer);
			ds.addConsumer(c);
		}
		else {
			ds.addConsumer(consumer);
		}
		connections.put(parameters, consumer);
		firePropertyChange(MultipleAcopDisplayer.DISPLAYER_PARAMETERS_PROPERTY, old, getDisplayerParameters());
    }
 	
	/* This method is required for backward compatibility. */
	public void addConnectionParameters(ConnectionParameters parameters) throws CommonException, PropertyVetoException {
		
		addDisplayerParameters(new AcopGraphParameters(parameters));
    }
	
	private AcopGraphParameters synchronizeGraphParameters(AcopGraphParameters parameters) {
		
		Color color = null;
		if (parameters.getColor() != null)
			color = parameters.getColor();
		else
			color = colorManager.previewNextColor();
			
		if (parameters.getDrawStyle() != -1)
			setDrawStyle(parameters.getDrawStyle());
		int graphStyle = getDrawStyle();
				
		if (parameters.getFFT() != -1)
			setFFT(parameters.getFFT());
		int fft = getFFT();
		
		if (parameters.getMode() != -1)
			setDisplayMode(parameters.getMode());
		int mode = getDisplayMode();
		
		int width = parameters.getWidth();
		setDrawWidth(width);
		
		ConnectionParameters cp =parameters.getConnectionParameters();
		if (parameters.isTrend())
			cp = cp.deriveWithPropertySize(1);
		return new AcopGraphParameters(new AcopDisplayerParameters(
						cp,
						parameters.getArrayIndex(), parameters.getMinimum(),
						parameters.getMaximum(), parameters.getUnits(),parameters.getFormat()),
						color,fft,graphStyle,mode,width,parameters.isTrend(),parameters.getTrendLength(),
						parameters.getConverter());
	}
	

	/*
	 * (non-Javadoc)
	 * @see de.desy.acop.displayers.AcopMultipleDisplayerConnector#getConnectionParameters()
	 */
	public AcopGraphParameters[] getDisplayerParameters() {
		AcopGraphParameters[] params = new AcopGraphParameters[connections.keySet().size()];
	    return connections.keySet().toArray(params);
    }
	
	/* This method is required for backward compatibility. */
	public ConnectionParameters[] getConnectionParameters() {
		
		AcopGraphParameters[] agp = getDisplayerParameters();
		ConnectionParameters[] cp = new ConnectionParameters[agp.length];
		for (int i = 0; i < agp.length; i++) {
			cp[i] = agp[i].getConnectionParameters();
		}
		return cp;
	}
	
	/*
	 * (non-Javadoc)
	 * @see de.desy.acop.displayers.AcopMultipleDisplayerConnector#removeConnectionParameters(de.desy.acop.transport.ConnectionParameters)
	 */
	public DataSource removeDisplayerParameters(AcopGraphParameters parameters) {
		//System.out.println("REM: '"+parameters.toString()+"' "+parameters.hashCode());
		AcopGraphParameters[] old = getDisplayerParameters();
		AdapterFactory factory = AdapterFactoryService.getInstance().getAdapterFactory();
		AcopChartConsumer consumer = connections.remove(parameters);
		if (consumer == null) return null;
		DataSource ds = factory.releaseDataSource(parameters.getConnectionParameters());
		if (parameters.getConverter() != null) {
			ds.removeConsumer(parameters.getConverter());
			parameters.getConverter().removeConsumer(consumer);
		} else {
			ds.removeConsumer(consumer);
		}
		releaseConsumer(consumer);

		firePropertyChange(MultipleAcopDisplayer.DISPLAYER_PARAMETERS_PROPERTY, old, getDisplayerParameters());
	    return ds;
	    
    }
	
	/* This method is required for backward compatibility. */
	public DataSource removeConnectionParameters(ConnectionParameters parameters) {
		
		return removeDisplayerParameters(new AcopGraphParameters(parameters, getForeground(),getFFT(), getDrawStyle(), getDisplayMode(), getDrawWidth(), false, 100, null));
	}
	
	/*
	 * (non-Javadoc)
	 * @see de.desy.acop.displayers.AcopMultipleDisplayerConnector#setConnectionParameters(de.desy.acop.transport.ConnectionParameters[])
	 */
	public void setDisplayerParameters(AcopGraphParameters[] parameters) throws CommonException, PropertyVetoException {
		AcopGraphParameters[] old = getDisplayerParameters();
		ArrayList<AcopGraphParameters> toBeRemoved = new ArrayList<AcopGraphParameters>();
		Iterator<AcopGraphParameters> it = connections.keySet().iterator();
		while(it.hasNext()) {
			toBeRemoved.add(it.next());
		}
		
		ArrayList<AcopGraphParameters> toBeAdded = new ArrayList<AcopGraphParameters>();
	
		for (AcopGraphParameters p : parameters) {
			if (toBeRemoved.contains(p)) {
				toBeRemoved.remove(p);
			} else {
				toBeAdded.add(p);
			}
		}
		
		for(AcopGraphParameters p : toBeRemoved) {
			removeDisplayerParameters(p);
		}
		
		for(AcopGraphParameters p : toBeAdded) {
			addDisplayerParameters(p);
		}
		
		firePropertyChange(MultipleAcopDisplayer.DISPLAYER_PARAMETERS_PROPERTY, old, parameters);
    }
	
	/* This method is required for backward compatibility. */
	public void setConnectionParameters(ConnectionParameters[] parameters) throws CommonException, PropertyVetoException {
		  
		AcopGraphParameters[] agp = new AcopGraphParameters[parameters.length];
		for (int i = 0; i < parameters.length; i++) {
			agp[i] = new AcopGraphParameters(parameters[i]);
		}
		setDisplayerParameters(agp);
	}
		
	public void updateYScale(AcopChartConsumer consumer) {
		if (!consumers.values().contains(consumer)) return;
		double max;
		double min;
		if (getConsumers().length == 1) {
			max = consumer.getPreferredYMax();
			min = consumer.getPreferredYMin();
		} else {
			max = Math.max(getYMax(), consumer.getPreferredYMax());
			min = Math.min(getYMin(), consumer.getPreferredYMin());
		}
		setYMax(max);
		setYMin(min);
	}
	
	private void updateYScaleOnRemove(AcopChartConsumer consumer) {
		if (!(consumer.getPreferredYMax() < getYMax() || consumer.getPreferredYMin() > getYMin())) {
			return;
		}
		Iterator<AcopChartConsumer> it = consumers.values().iterator();
		double max;
		double min;
		if (it.hasNext()) {
			AcopChartConsumer c = it.next();
			max = c.getPreferredYMax();
			min = c.getPreferredYMin(); 
		} else {
			return;
		}
		while (it.hasNext()) {
			AcopChartConsumer c = it.next();
			max = Math.max(max, c.getPreferredYMax());
			min = Math.min(min, c.getPreferredYMin());
		}
		setYMax(max);
		setYMin(min);
		
	}
	
	public void updateXScale(AcopChartConsumer consumer) {
		if (!consumers.values().contains(consumer)) return;
		double max;
		if (getConsumers().length == 1) {
			max = consumer.getArraySize();
		} else {
			max = Math.max(getXMax(), consumer.getArraySize());
		}
		setXMax(max);
	}
	
	private void updateXScaleOnRemove(AcopChartConsumer consumer) {
		if (!(consumer.getArraySize() < getXMax())) {
			return;
		}
		Iterator<AcopChartConsumer> it = consumers.values().iterator();
		double max;
		if (it.hasNext()) {
			AcopChartConsumer c = it.next();
			max = c.getArraySize();
		} else {
			return;
		}
		while (it.hasNext()) {
			AcopChartConsumer c = it.next();
			max = Math.max(max, c.getArraySize());
		}
		setXMax(max);				
	}
	
	public void setDragEnabled(boolean enabled) {
		if (getTransferHandler() instanceof CosyTransferHandler) {
			if (((CosyTransferHandler)getTransferHandler()).isExportEnabled() == enabled) return;
			
			((CosyTransferHandler)getTransferHandler()).setExportEnabled(enabled, this);
		}
	}
	
	public boolean isDragEnabled() {
		if (getTransferHandler() instanceof CosyTransferHandler) {
			return ((CosyTransferHandler)getTransferHandler()).isExportEnabled();
		} else {
			return getTransferHandler() != null;
		}
	}
	
	public void setDropEnabled(boolean enabled) {
		if (getTransferHandler() instanceof CosyTransferHandler) {
			if (((CosyTransferHandler)getTransferHandler()).isReceiveEnabled() == enabled) return;
			
			((CosyTransferHandler)getTransferHandler()).setReceiveEnabled(enabled, this);
		}
	}
	
	public boolean isDropEnabled() {
		if (getTransferHandler() instanceof CosyTransferHandler) {
			return ((CosyTransferHandler)getTransferHandler()).isReceiveEnabled();
		} else {
			return getTransferHandler() != null;
		}
	}
	

	public AcopMultipleDisplayerInfoDialog getInfoDialog() {
		if (dialog == null) {
			dialog = new AcopMultipleDisplayerInfoDialog(this);
		}
		return dialog;
	}
	
	/*
	 * (non-Javadoc)
	 * @see de.desy.acop.chart.Acop#getPopupManager()
	 */
	@Override
	public PopupManager getPopupManager()
	{
		if (popupManager == null) {
			popupManager = super.getPopupManager();
			popupManager.addAction(new AbstractAction("Info...") {
				private static final long serialVersionUID = 1L;

					public void actionPerformed(ActionEvent e)
					{
						getInfoDialog().setVisible(true);
					}
				});
		}

		return popupManager;
	}
	
	/**
	 * Enables/disables Properties item in the popup menu.
	 * @param enable
	 */
	public void setPropertiesPopupEnabled(boolean enable) {
		Action[] actions = getPopupManager().getActions();
		for (Action a : actions) {
			if (a != null && "Preferences...".equals(a.getValue(Action.NAME))) {
				a.setEnabled(enable);
				return;
			}
		}
	}
	
	/**
	 * Returns true if Properties item is enabled in the popup menu.
	 * @return
	 */
	public boolean isPropertiesPopupEnabled() {
		Action[] actions = getPopupManager().getActions();
		for (Action a : actions) {
			if (a != null && "Preferences...".equals(a.getValue(Action.NAME))) {
				return a.isEnabled();
			}
		}
		return false;
	}
	
}
