/**
 * 
 */
package de.desy.acop.displayers;
  
import java.awt.Color;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.beans.PropertyVetoException;
import java.util.ArrayList;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.JPanel;

import com.cosylab.gui.adapters.Converter;
import com.cosylab.gui.components.customizer.AbstractCustomizerPanel;
import com.cosylab.gui.components.util.CosyTransferHandler;
import com.cosylab.gui.components.util.PopupManageable;
import com.cosylab.gui.components.util.PopupManager;
import com.cosylab.gui.components.util.RunnerHelper;
import com.cosylab.gui.displayers.DataConsumer;
import com.cosylab.gui.displayers.DataSource;
import com.cosylab.gui.displayers.DataState;
import com.cosylab.gui.displayers.DoubleSeqConsumer;
import com.cosylab.gui.displayers.MultipleDisplayer;
import com.cosylab.gui.components.util.ColorManager;
import com.cosylab.gui.util.UserSettingsProtection;
import com.cosylab.util.CommonException;

import de.desy.acop.chart.Acop;
import de.desy.acop.chart.AcopAdapter;
import de.desy.acop.chart.AcopCursorMarkerModeEnum;
import de.desy.acop.chart.AcopDisplayMode;
import de.desy.acop.chart.AcopEvent;
import de.desy.acop.chart.AcopGraphStyleEnum;
import de.desy.acop.displayers.chart.AcopChartConsumer;
import de.desy.acop.displayers.chart.AcopChartReorgTransferHandler;
import de.desy.acop.displayers.tools.AcopDisplayerParameters;
import de.desy.acop.displayers.tools.AcopGraphParameters;
import de.desy.acop.displayers.tools.MultipleAcopDisplayer;
import de.desy.acop.transport.ConnectionParameters;
import de.desy.acop.transport.adapters.AcopTransportDataSource;
import de.desy.acop.transport.adapters.AdapterFactory;
import de.desy.acop.transport.adapters.AdapterFactoryService;
        
/**
 * @author ikriznar
 *  
 */
public class AcopChartReorg extends JPanel implements MultipleDisplayer, MultipleAcopDisplayer<AcopGraphParameters>, PopupManageable {
	
	private static final long serialVersionUID = -2579109976414960938L;

	public static final String TITLE = "title";
	public static final String LINE_STYLE = "displayMode";
	public static final String CHUBBY_LINES = "chubbyLines";
	public static final String AXIS_TRACE_VISIBLE = "axisTraceVisible";
	public static final String X_AUTOSCALE = "xAutoScale";
	public static final String Y_AUTOSCALE = "yAutoScale";
	public static final String Y_LOG_SCALE = "yLogScale";
	public static final String X_RANGE_MAX = "xRangeMax";
	public static final String X_RANGE_MIN = "xRangeMin";
	public static final String Y_RANGE_MAX = "yRangeMax";
	public static final String Y_RANGE_MIN = "yRangeMin";
	public static final String CONSUMERS = "consumers";
	
	protected ColorManager colorManager = new ColorManager();
	
	private boolean axisTraceVisible;
	private boolean xAutoScale;
	private boolean yAutoScale;
	private boolean yLogScale;
	private boolean isZoomed;
	private boolean chubbyLines;
	private AcopDisplayMode displayMode;
	private boolean showGrid = false;
	private boolean bestScale = false;
	
	protected int maxLinkIndex=0;
	private int suspended=0;
	
	protected ArrayList<AcopChartConsumer> consumersList = new ArrayList<AcopChartConsumer>();
	private DataState dataState = new DataState(DataState.NORMAL);
	private PopupManager popupManager;
	AbstractCustomizerPanel customizer;
	private Acop acop;
	
	private class AcopZoomListener extends AcopAdapter {

		/* (non-Javadoc)
		 * @see de.desy.acop.chart.AcopAdapter#acopMouseZoom(de.desy.acop.chart.AcopEvent)
		 */
		@Override
		public void acopMouseZoom(AcopEvent zoom) {
			if (getAcop().isZoomed()) {
				isZoomed = true;
			}
			else {
				isZoomed = false;
			}
		}
		
	}
	
	private class AcopChangeListener implements PropertyChangeListener {
		public void propertyChange(PropertyChangeEvent evt) {
			String property = evt.getPropertyName();
			if (property.equals("caption")) {
				firePropertyChange(TITLE, evt.getOldValue(), evt.getNewValue());
			}
			else if (property.equals("yMin")) {
				firePropertyChange(Y_RANGE_MIN, evt.getOldValue(), evt.getNewValue());
			}
			else if (property.equals("yMax")) {
				firePropertyChange(Y_RANGE_MAX, evt.getOldValue(), evt.getNewValue());
			}
			else if (property.equals("xMin")) {
				firePropertyChange(X_RANGE_MIN, evt.getOldValue(), evt.getNewValue());
			}
			else if (property.equals("xMax")) {
				firePropertyChange(X_RANGE_MAX, evt.getOldValue(), evt.getNewValue());
			}	
		}
	}
	
	public AcopChartReorg() {
        super();
        setLayout(new GridBagLayout());
        add(getAcop(), new GridBagConstraints(0,0,1,1,1,1,GridBagConstraints.CENTER,GridBagConstraints.BOTH,new Insets(0,0,0,0),0,0));
        new AcopChartReorgTransferHandler(this);
//		new ChartTransferHandler(this, true, true);
		UserSettingsProtection.setProtection(this,new String[]{"caption", "yMin", "yMax"},false);
		getPopupManager();
	}
	
	/**
	 * Returns this components </code>Acop</code> chart.
	 * @return the </code>Acop</code>
	 */
	public Acop getAcop() {
		if (acop == null) {
			acop = new Acop();
			acop.setXBestScale(bestScale);
			acop.setYBestScale(bestScale);
			acop.setXGrid(showGrid);
			acop.setYGrid(showGrid);
			acop.addPropertyChangeListener(new AcopChangeListener());
			acop.addAcopListener(new AcopZoomListener());
			acop.setCursorMarkerColor(Color.white);
			setTitle("");
		}
		return acop;
	}
	
	/* (non-Javadoc)
	 * @see com.cosylab.gui.displayers.CommonDisplayer#cleanup()
	 */
	public void cleanup() {
		DataConsumer[] oldConsumers = getConsumers();
		for(AcopChartConsumer c: consumersList) {
			c.destroy();
		}
		consumersList.clear();
		firePropertyChange(CONSUMERS, oldConsumers, getConsumers());
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.displayers.CommonDisplayer#suspend()
	 */
	public void suspend() {
		suspended++;
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.displayers.CommonDisplayer#resume()
	 */
	public synchronized void resume() {
		if (suspended>0) {
			suspended--;
		}
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.displayers.CommonDisplayer#isSuspended()
	 */
	public boolean isSuspended() {
		return suspended>0;
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.displayers.DataStateProvider#getDataState()
	 */
	public DataState getDataState() {
		return dataState;
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.displayers.MultipleDataConsumer#getConsumer(java.lang.String, java.lang.Class)
	 */
	public <D extends DataConsumer> D getConsumer(String name, Class<D> type) {
		
		if (type.isAssignableFrom(AcopChartConsumer.class)) {
			for (AcopChartConsumer c : consumersList) {
				if (c.getName().equals(name)) return type.cast(c);
			}
			
			DataConsumer[] oldConsumers = getConsumers();
			AcopChartConsumer c= new AcopChartConsumer(this, name);
			c.setColor(colorManager.pickColor());
			c.setChubbyLines(isChubbyLines());
			c.setDisplayMode(getDisplayMode());
			maxLinkIndex++;
			consumersList.add(c);
			firePropertyChange(CONSUMERS, oldConsumers, getConsumers());
			return type.cast(c);
		}
		return null;
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.displayers.MultipleDataConsumer#releaseConsumer(com.cosylab.gui.displayers.DataConsumer)
	 */
	public void releaseConsumer(DataConsumer consumer) {
		int index = consumersList.indexOf(consumer);
		if (index >= 0 && index < consumersList.size()) {
			DataConsumer[] oldConsumers = getConsumers();
			consumersList.remove(index).destroy();
			firePropertyChange(CONSUMERS, oldConsumers, getConsumers());
		}
		updateYScaleOnRemove((AcopChartConsumer)consumer);
		updateXScaleOnRemove((AcopChartConsumer)consumer);
		colorManager.dropColor(((AcopChartConsumer)consumer).getColor());
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.displayers.MultipleDataConsumer#getConsumers()
	 */
	public DataConsumer[] getConsumers() {
		return consumersList.toArray(new DataConsumer[consumersList.size()]);
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.displayers.MultipleDataConsumer#getSupportedConsumerTypes()
	 */
	@SuppressWarnings("unchecked")
	public Class<DataConsumer>[] getSupportedConsumerTypes() {
		return new Class[]{DoubleSeqConsumer.class};
	}
	
	/*
	 * (non-Javadoc)
	 * @see de.desy.acop.displayers.AcopMultipleDisplayerConnector#addConnectionParameters(de.desy.acop.transport.ConnectionParameters)
	 */
	public void addDisplayerParameters(AcopGraphParameters parameters) throws CommonException, PropertyVetoException {
		if (parameters == null) return;
		//System.out.println("ADD: '"+parameters.toString()+"' "+parameters.hashCode());
		parameters = synchronizeGraphParameters(parameters);
		for (AcopChartConsumer c : consumersList) {
			if (c.getDisplayerParameters().equals(parameters)) return;
		}
		AcopGraphParameters[] old = getDisplayerParameters();
		AcopChartConsumer consumer = (AcopChartConsumer) getConsumer(parameters.toString(), DoubleSeqConsumer.class);
			
		consumer.setDisplayerParameters(parameters);
		AdapterFactory factory = AdapterFactoryService.getInstance().getAdapterFactory();
		AcopTransportDataSource ds = factory.createDataSource(parameters.getConnectionParameters());
		if (parameters.getConverter() != null){
			Converter c = parameters.getConverter();
			c.addConsumer(consumer);
			ds.addConsumer(c);
		}
		else {
			ds.addConsumer(consumer);
		}
		firePropertyChange(MultipleAcopDisplayer.DISPLAYER_PARAMETERS_PROPERTY, old, getDisplayerParameters());
    }
 	
	/* This method is required for backward compatibility. */
	public void addConnectionParameters(ConnectionParameters parameters) throws CommonException, PropertyVetoException {
		
		addDisplayerParameters(new AcopGraphParameters(parameters));
    }
	
	protected AcopGraphParameters synchronizeGraphParameters(AcopGraphParameters parameters) {
		
		Color color = null;
		if (parameters.getColor() != null)
			color = parameters.getColor();
		else
			color = colorManager.previewNextColor();
			
		if (parameters.getDrawStyle() != -1)
			getAcop().setDrawStyle(parameters.getDrawStyle());
		int graphStyle = getAcop().getDrawStyle();
				
		if (parameters.getFFT() != -1)
			getAcop().setFFT(parameters.getFFT());
		int fft = getAcop().getFFT();
		
		if (parameters.getMode() != -1)
			getAcop().setDisplayMode(parameters.getMode());
		int mode = getAcop().getDisplayMode();
		
		int width = parameters.getWidth();
		getAcop().setDrawWidth(width);
		
		ConnectionParameters cp =parameters.getConnectionParameters();
		if (parameters.isTrend())
			cp = cp.deriveWithPropertySize(1);
		return new AcopGraphParameters(new AcopDisplayerParameters(
						cp,
						parameters.getArrayIndex(), parameters.getMinimum(),
						parameters.getMaximum(), parameters.getUnits(),parameters.getFormat()),
						color,fft,graphStyle,mode,width,parameters.isTrend(),parameters.getTrendLength(),
						parameters.getConverter());
	}
	

	/*
	 * (non-Javadoc)
	 * @see de.desy.acop.displayers.AcopMultipleDisplayerConnector#getConnectionParameters()
	 */
	public AcopGraphParameters[] getDisplayerParameters() {
		ArrayList<AcopGraphParameters> acopGraphParameters = new ArrayList<AcopGraphParameters>();
		for (AcopChartConsumer c : consumersList) {
			if (c.getDisplayerParameters() != null) acopGraphParameters.add(c.getDisplayerParameters());
		}
		AcopGraphParameters[] params = acopGraphParameters.toArray(new AcopGraphParameters[acopGraphParameters.size()]);
	    return params;
    }
	
	/* This method is required for backward compatibility. */
	public ConnectionParameters[] getConnectionParameters() {
		
		AcopGraphParameters[] agp = getDisplayerParameters();
		ConnectionParameters[] cp = new ConnectionParameters[agp.length];
		for (int i = 0; i < agp.length; i++) {
			cp[i] = agp[i].getConnectionParameters();
		}
		return cp;
	}
	
	/*
	 * (non-Javadoc)
	 * @see de.desy.acop.displayers.AcopMultipleDisplayerConnector#removeConnectionParameters(de.desy.acop.transport.ConnectionParameters)
	 */
	public DataSource removeDisplayerParameters(AcopGraphParameters parameters) {
		//System.out.println("REM: '"+parameters.toString()+"' "+parameters.hashCode());
		AcopGraphParameters[] old = getDisplayerParameters();
		AdapterFactory factory = AdapterFactoryService.getInstance().getAdapterFactory();
		AcopChartConsumer consumer = null;
		if (parameters != null) {
			for (AcopChartConsumer c : consumersList) {
				if (parameters.equals(c.getDisplayerParameters())) {
					consumer = c;
					break;
				}
			}
		}
		if (consumer == null) return null;
		DataSource ds = factory.releaseDataSource(parameters.getConnectionParameters());
		if (parameters.getConverter() != null) {
			ds.removeConsumer(parameters.getConverter());
			parameters.getConverter().removeConsumer(consumer);
		} else {
			ds.removeConsumer(consumer);
		}
		releaseConsumer(consumer);

		firePropertyChange(MultipleAcopDisplayer.DISPLAYER_PARAMETERS_PROPERTY, old, getDisplayerParameters());
	    return ds;
	    
    }
	
	/* This method is required for backward compatibility. */
	public DataSource removeConnectionParameters(ConnectionParameters parameters) {
		return removeDisplayerParameters(new AcopGraphParameters(parameters, getForeground(),getAcop().getFFT(), getAcop().getDrawStyle(), getAcop().getDisplayMode(), getAcop().getDrawWidth(), false, 100, null));
	}
	
	/*
	 * (non-Javadoc)
	 * @see de.desy.acop.displayers.AcopMultipleDisplayerConnector#setConnectionParameters(de.desy.acop.transport.ConnectionParameters[])
	 */
	public void setDisplayerParameters(AcopGraphParameters[] parameters) throws CommonException, PropertyVetoException {
		AcopGraphParameters[] old = getDisplayerParameters();
		ArrayList<AcopGraphParameters> toBeRemoved = new ArrayList<AcopGraphParameters>();
		for (AcopChartConsumer c : consumersList) {
			if (c.getDisplayerParameters() != null) toBeRemoved.add(c.getDisplayerParameters());
		}
		
		ArrayList<AcopGraphParameters> toBeAdded = new ArrayList<AcopGraphParameters>();
	
		for (AcopGraphParameters p : parameters) {
			if (toBeRemoved.contains(p)) {
				toBeRemoved.remove(p);
			} else {
				toBeAdded.add(p);
			}
		}
		
		for(AcopGraphParameters p : toBeRemoved) {
			removeDisplayerParameters(p);
		}
		
		for(AcopGraphParameters p : toBeAdded) {
			addDisplayerParameters(p);
		}
		
		firePropertyChange(MultipleAcopDisplayer.DISPLAYER_PARAMETERS_PROPERTY, old, parameters);
    }
	
	/* This method is required for backward compatibility. */
	public void setConnectionParameters(ConnectionParameters[] parameters) throws CommonException, PropertyVetoException {
		  
		AcopGraphParameters[] agp = new AcopGraphParameters[parameters.length];
		for (int i = 0; i < parameters.length; i++) {
			agp[i] = new AcopGraphParameters(parameters[i]);
		}
		setDisplayerParameters(agp);
	}
		
	public void updateYScale(AcopChartConsumer consumer) {
		if (!consumersList.contains(consumer)) return;
		double max;
		double min;
		if (getConsumers().length == 1) {
			max = consumer.getPreferredYMax();
			min = consumer.getPreferredYMin();
		} else {
			max = Math.max(getYRangeMax(), consumer.getPreferredYMax());
			min = Math.min(getYRangeMin(), consumer.getPreferredYMin());
		}
		setYRangeMax(max);
		setYRangeMin(min);
	}
	
	protected void updateYScaleOnRemove(AcopChartConsumer consumer) {
		if (!(consumer.getPreferredYMax() < getYRangeMax() || consumer.getPreferredYMin() > getYRangeMin())) {
			return;
		}
		if (consumersList.size() == 0) return;
		double max = -Double.MAX_VALUE;
		double min = Double.MAX_VALUE;
		for (AcopChartConsumer c : consumersList) {
			max = Math.max(max, c.getPreferredYMax());
			min = Math.min(min, c.getPreferredYMin());
		}
		setYRangeMax(max);
		setYRangeMin(min);
		
	}
	
	public void updateXScale(AcopChartConsumer consumer) {
		if (!consumersList.contains(consumer)) return;
		double max;
		double min;
		if (getConsumers().length == 1) {
			max = consumer.getPreferredXMax();
			min = consumer.getPreferredXMin();
		} else {
			max = Math.max(getXRangeMax(), consumer.getPreferredXMax());
			min = Math.min(getXRangeMin(), consumer.getPreferredXMin());
		}
		setXRangeMax(max);
		setXRangeMin(min);
	}
	
	protected void updateXScaleOnRemove(AcopChartConsumer consumer) {
		if (!(consumer.getPreferredXMax() < getXRangeMax() || consumer.getPreferredXMin() > getXRangeMin())) {
			return;
		}
		
		if (consumersList.size() == 0) return;
		double max = -Double.MAX_VALUE;
		double min = Double.MAX_VALUE;
		for (AcopChartConsumer c : consumersList) {
			max = Math.max(max, c.getPreferredXMax());
			min = Math.min(min, c.getPreferredXMin());
		}
		setXRangeMax(max);
		setXRangeMin(min);
	}
	
	public void setDragEnabled(boolean enabled) {
		if (getTransferHandler() instanceof CosyTransferHandler) {
			if (((CosyTransferHandler)getTransferHandler()).isExportEnabled() == enabled) return;
			
			((CosyTransferHandler)getTransferHandler()).setExportEnabled(enabled, this);
		}
	}
	
	public boolean isDragEnabled() {
		if (getTransferHandler() instanceof CosyTransferHandler) {
			return ((CosyTransferHandler)getTransferHandler()).isExportEnabled();
		} else {
			return getTransferHandler() != null;
		}
	}
	
	public void setDropEnabled(boolean enabled) {
		if (getTransferHandler() instanceof CosyTransferHandler) {
			if (((CosyTransferHandler)getTransferHandler()).isReceiveEnabled() == enabled) return;
			
			((CosyTransferHandler)getTransferHandler()).setReceiveEnabled(enabled, this);
		}
	}
	
	public boolean isDropEnabled() {
		if (getTransferHandler() instanceof CosyTransferHandler) {
			return ((CosyTransferHandler)getTransferHandler()).isReceiveEnabled();
		} else {
			return getTransferHandler() != null;
		}
	}
	
	public PopupManager getPopupManager()
	{
		if (popupManager == null) {
			popupManager = getAcop().getPopupManager();
			for (Action action : popupManager.getActions()) {
				if ("Preferences...".equals(action.getValue(Action.NAME))) popupManager.removeAction(action);
			}
			popupManager.addAction(new AbstractAction("Preferences...") {
				
				private static final long serialVersionUID = 1L;

				public void actionPerformed(ActionEvent e) {
					getCustomizer().showDialog();
					
				}});
		}

		return popupManager;
	}
	
	/**
	 * Returns the Customizer instance for this component.
	 *
	 * @return the Customizer instance for this component.
	 */
	public AbstractCustomizerPanel getCustomizer()
	{
		if (customizer == null) {
			customizer = AbstractCustomizerPanel.findCustomizer(this);
			customizer.setObject(this);
		}

		return customizer;
	}
	
	/**
	 * Enables/disables Properties item in the popup menu.
	 * @param enable
	 */
	public void setPropertiesPopupEnabled(boolean enable) {
		Action[] actions = getPopupManager().getActions();
		for (Action a : actions) {
			if (a != null && "Preferences...".equals(a.getValue(Action.NAME))) {
				a.setEnabled(enable);
				return;
			}
		}
	}
	
	/**
	 * Returns true if Properties item is enabled in the popup menu.
	 * @return
	 */
	public boolean isPropertiesPopupEnabled() {
		Action[] actions = getPopupManager().getActions();
		for (Action a : actions) {
			if (a != null && "Preferences...".equals(a.getValue(Action.NAME))) {
				return a.isEnabled();
			}
		}
		return false;
	}
	
	public void autoScale() {
		autoScale(isXAutoScale(), isXAutoScale(), isYAutoScale(), isYAutoScale());
	}
	
	protected void autoScale(boolean xmx, boolean xmn, boolean ymx, boolean ymn) {
		if (isZoomed || !canAutoScale()) return;
		// workaround: Acop doesn't fire property change when range is changed by autoscale
		double xMin = getXRangeMin();
		double xMax = getXRangeMax();
		double yMin = getYRangeMin();
		double yMax = getYRangeMax();
		getAcop().autoScale(xmx, xmn, ymx, ymn);
		if (xMin != getXRangeMin()) firePropertyChange(X_RANGE_MIN, xMin, getXRangeMin());
		if (xMax != getXRangeMax()) firePropertyChange(X_RANGE_MAX, xMax, getXRangeMax());
		if (yMin != getYRangeMin()) firePropertyChange(Y_RANGE_MIN, yMin, getYRangeMin());
		if (yMax != getYRangeMax()) firePropertyChange(Y_RANGE_MAX, yMax, getYRangeMax());
	}
	
	protected boolean canAutoScale() {
		for (AcopChartConsumer c : consumersList) {
			if (c.isDataDrawn()) return true;
		}
		return false;
	}
	
	public void autoScaleXOnce() {
		autoScale(true, true, false, false);
	}
	
	public void autoScaleYOnce() {
		autoScale(false, false, true, true);
	}
	
	public void resetZoom() {
		getAcop().SetZoom(true, true, true, true, getAcop().getXMax(), getAcop().getXMin(), getAcop().getYMax(), getAcop().getYMin());

	}
	
	/**
	 * Gets axis trace visible option.
	 * @return the isAxisTraceVisible
	 */
	public boolean isAxisTraceVisible() {
		return axisTraceVisible;
	}
	/**
	 * Sets axis trace visible option.
	 * @param visible the visible to set
	 */
	public void setAxisTraceVisible(boolean visible) {
		if (visible == axisTraceVisible) return;
		axisTraceVisible = visible;
		if (visible) getAcop().setCursorMarkerMode((AcopCursorMarkerModeEnum.CrossHair).ordinal());
		else getAcop().setCursorMarkerMode((AcopCursorMarkerModeEnum.NoMarker).ordinal());
		firePropertyChange(AXIS_TRACE_VISIBLE, !axisTraceVisible, axisTraceVisible);

	}
	/**
	 * Gets chart title.
	 * @return the title
	 */
	public String getTitle() {
		return getAcop().getCaption();
	}
	/**
	 * Sets chart title.
	 * @param title the title to set
	 */
	public void setTitle(String title) {
		String oldTitle = getTitle();
		getAcop().setCaption(title);
		firePropertyChange(TITLE, oldTitle, getTitle());
	}
	/**
	 * Gets x auto scale option.
	 * @return the xAutoScale
	 */
	public boolean isXAutoScale() {
		return xAutoScale;
	}
	/**
	 * Sets x auto scale option.
	 * @param autoScale the xAutoScale to set
	 */
	public void setXAutoScale(boolean autoScale) {
		if (autoScale == xAutoScale) return;
		xAutoScale = autoScale;
		if (autoScale) 
			autoScale(isXAutoScale(), isXAutoScale(), isYAutoScale(), isYAutoScale());
		firePropertyChange(X_AUTOSCALE, !xAutoScale, xAutoScale);
	}
	/**
	 * Gets maximum value for x axis.
	 * @return the xRangeMax
	 */
	public double getXRangeMax() {
		return getAcop().getXMax();
	}
	/**
	 * Sets maximum value for x axis.
	 * @param rangeMax the xRangeMax to set
	 */
	public void setXRangeMax(double rangeMax) {
		if (isZoomed) return;
		double oldValue = getXRangeMax();
		getAcop().setXMax(rangeMax);
		firePropertyChange(X_RANGE_MAX, oldValue, getXRangeMax());
	}
	/**
	 * Gets minimum value for x axis.
	 * @return the xRangeMin
	 */
	public double getXRangeMin() {
		return getAcop().getXMin();
	}
	/**
	 * Sets minimum value for x axis.
	 * @param rangeMin the xRangeMin to set
	 */
	public void setXRangeMin(double rangeMin) {
		if (isZoomed) return;
		double oldValue = getXRangeMin();
		getAcop().setXMin(rangeMin);
		firePropertyChange(X_RANGE_MIN, oldValue, getXRangeMin());
	}
	/**
	 * Gets y auto scale option.
	 * @return the yAutoScale
	 */
	public boolean isYAutoScale() {
		return yAutoScale;
	}
	/**
	 * Sets y auto scale option.
	 * @param autoScale the yAutoScale to set
	 */
	public void setYAutoScale(boolean autoScale) {
		if (autoScale == yAutoScale) return;
		yAutoScale = autoScale;
		if (autoScale)
			autoScale(isXAutoScale(), isXAutoScale(), isYAutoScale(), isYAutoScale());
		firePropertyChange(Y_AUTOSCALE, !yAutoScale, yAutoScale);
	}
	/**
	 * Gets y log scale option.
	 * @return the yLogScale
	 */
	public boolean isYLogScale() {
		return yLogScale;
	}
	/**
	 * Sets y log scale option.
	 * @param logScale the yLogScale to set
	 */
	public void setYLogScale(boolean logScale) {
		if (logScale == yLogScale) return;
		yLogScale = logScale;
		int style = getAcop().getGraphStyle();
		if (style == AcopGraphStyleEnum.LinLin.ordinal() || style == AcopGraphStyleEnum.LinLog.ordinal()) {
			if (logScale) getAcop().setGraphStyle((AcopGraphStyleEnum.LinLog).ordinal());
			else getAcop().setGraphStyle((AcopGraphStyleEnum.LinLin).ordinal());
		}
		else if (style == AcopGraphStyleEnum.TimeLin.ordinal() || style == AcopGraphStyleEnum.TimeLog.ordinal()) {
			if (logScale) getAcop().setGraphStyle((AcopGraphStyleEnum.TimeLog).ordinal());
			else getAcop().setGraphStyle((AcopGraphStyleEnum.TimeLin).ordinal());
		}
		else {
			if (logScale) getAcop().setGraphStyle((AcopGraphStyleEnum.LogLog).ordinal());
			else getAcop().setGraphStyle((AcopGraphStyleEnum.LogLin).ordinal());
		}
		firePropertyChange(Y_LOG_SCALE, !yLogScale, yLogScale);
	}
	/**
	 * Gets maximum value for y axis.
	 * @return the yRangeMax
	 */
	public double getYRangeMax() {
		return getAcop().getYMax();
	}
	/**
	 * Sets maximum value for y axis.
	 * @param rangeMax the yRangeMax to set
	 */
	public void setYRangeMax(double rangeMax) {
		if (isZoomed) return;
		double oldValue = getYRangeMax();
		getAcop().setYMax(rangeMax);
		firePropertyChange(Y_RANGE_MAX, oldValue, getYRangeMax());
		
	}
	/**
	 * Gets minimum value for y axis.
	 * @return the yRangeMin
	 */
	public double getYRangeMin() {
		return getAcop().getYMin();
	}
	/**
	 * Sets minimum value for y axis.
	 * @param rangeMin the yRangeMin to set
	 */
	public void setYRangeMin(double rangeMin) {
		if (isZoomed) return;
		double oldValue = getYRangeMin();
		getAcop().setYMin(rangeMin);
		firePropertyChange(Y_RANGE_MIN, oldValue, getYRangeMin());
	}
	/**
	 * Gets chubby lines value.
	 * @return the chubbyLines
	 */
	public boolean isChubbyLines() {
		return chubbyLines;
	}
	/**
	 * Sets chubby lines option. If chubby lines are selected, widths of all plots
	 * are doubled.
	 * @param chubbyLines the chubbyLines to set
	 */
	public void setChubbyLines(boolean chubbyLines) {
		if (this.chubbyLines == chubbyLines) return;
		this.chubbyLines = chubbyLines;
		for (AcopChartConsumer c : consumersList) {
			c.setChubbyLines(chubbyLines);
		}
		firePropertyChange(CHUBBY_LINES, !chubbyLines, chubbyLines);
	}
	/**
	 * Gets display mode of this chart.
	 * @return the displayMode
	 */
	public AcopDisplayMode getDisplayMode() {
		return displayMode;
	}
	/**
	 * Sets display mode to this chart (plots of all consumers have the same display 
	 * mode). If line style is </code>null</code> individual display modes of 
	 * consumers are allowed. 
	 * @param displayMode the displayMode to set
	 */
	public void setDisplayMode(AcopDisplayMode displayMode) {
		if (displayMode == this.displayMode) return;
		AcopDisplayMode oldMode = this.displayMode;
		this.displayMode = displayMode;
		for (AcopChartConsumer c : consumersList) {
			c.setDisplayMode(displayMode);
		}
		firePropertyChange(LINE_STYLE, oldMode, displayMode);
	}
	
	/**
	 * Toggles the best scale. When best scale is on, chart should scale
	 * the axis according to best visibility of data.
	 * @param bestScale
	 */
	public void setBestScale(boolean bestScale) {
		if (this.bestScale == bestScale) return;
		this.bestScale = bestScale;
		getAcop().setXBestScale(bestScale);
		getAcop().setYBestScale(bestScale);
		firePropertyChange("bestScale", !bestScale, bestScale);
	}
	
	/**
	 * Returns true if best scale is switched on.
	 * @return
	 */
	public boolean isBestScale() {
		return bestScale;
	}
	
	/**
	 * Toggles the visibility of grid lines.
	 * @param show
	 */
	public void setShowGrid(boolean show) {
		if (this.showGrid == show) return;
		this.showGrid = show;
		getAcop().setXGrid(show);
		getAcop().setYGrid(show);
		firePropertyChange("showGrid", !showGrid, showGrid);
	}
	
	/**
	 * Returns true if grid lines are visible.
	 * @return
	 */
	public boolean isShowGrid() {
		return showGrid;
	}
	
	public static void main(String[] args) {
		RunnerHelper.runComponent(new AcopChartReorg(), 800, 600);
	}
}
