package de.desy.acop.displayers;

import java.beans.PropertyVetoException;
import java.util.Map;

import javax.swing.Action;
import javax.swing.JFrame;

import com.cosylab.gui.InfoDialog;
import com.cosylab.gui.LabelDisplayer;
import com.cosylab.gui.adapters.Converter;
import com.cosylab.gui.util.UserSettingsProtection;
import com.cosylab.util.CommonException;

import de.desy.acop.displayers.tools.AcopDisplayer;
import de.desy.acop.displayers.tools.AcopDisplayerTransferHandler;
import de.desy.acop.displayers.tools.AcopInfoDialog;
import de.desy.acop.transport.ConnectionParameters;
import de.desy.acop.transport.adapters.AcopTransportDataSource;
import de.desy.acop.transport.adapters.AdapterFactory;
import de.desy.acop.transport.adapters.AdapterFactoryService;
import de.desy.tine.definitions.TArrayType;

public class AcopLabel extends LabelDisplayer implements AcopDisplayer {
	private static final long serialVersionUID = 1L;
 

	public static void main(String[] s) {
		JFrame f= new JFrame();
		f.setContentPane(new AcopLabel());
		f.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		f.pack();
		f.setVisible(true);
	}
	
	private ConnectionParameters connectionParameters;

	private AcopInfoDialog dialog;

	public AcopLabel() {
		super();
		
		new AcopDisplayerTransferHandler(this);
		UserSettingsProtection.setProtection(this,new String[]{"title"},false);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see de.desy.acop.displayers.AcopDisplayerConnector#getConnectionParameters()
	 */
	public ConnectionParameters getConnectionParameters() {
		return connectionParameters;
	}

	public Converter getConverter() {
		return null;
	}
	
	/*
	 * (non-Javadoc)
	 * @see com.cosylab.gui.DialKnobDisplayer#setCharacteristics(java.util.Map)
	 */
	@Override
	public void setCharacteristics(Map characteristics) {
		super.setCharacteristics(characteristics);
		
		Object o  = characteristics.get("arrayType");
		if (o != null) {
			if (((TArrayType)o).isChannel()) {
				try {
					UserSettingsProtection.setUnprotected(this,"arrayIndex",0);
				} catch (Exception e) {
					e.printStackTrace();
				}
			}
		}
	}

	@Override
	public InfoDialog getInfoDialog() {
		if (dialog == null) {
			dialog = new AcopInfoDialog(this);
		}
		return dialog;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see de.desy.acop.displayers.AcopDisplayerConnector#setConnectionParameters(de.desy.acop.transport.ConnectionParameters)
	 */
	public void setConnectionParameters(ConnectionParameters param)
			throws CommonException, PropertyVetoException {
	
		if (param != null && connectionParameters != null) {
			if (param.equals(connectionParameters))
				return;
		}
		
		ConnectionParameters old = connectionParameters;
		this.connectionParameters = param;
		AdapterFactory factory = AdapterFactoryService.getInstance()
				.getAdapterFactory();
		if (getDataSource() != null)
			factory.releaseDataSource((AcopTransportDataSource) getDataSource());
		if (param == null) {
			setDataSource(null);
			setTitle("No Connection");
		} else {
			if (param.getPropertySize() == ConnectionParameters.AUTO_PROPERTY_SIZE) {
				param = param.deriveWithPropertySize(1);
			}
			AcopTransportDataSource ds = factory.createDataSource(param);
			setDataSource(ds);
		}

		firePropertyChange(CONNECTION_PARAMETERS_PROPERTY, old,
				connectionParameters);
	}

	public void setConverter(Converter converter) throws PropertyVetoException {
	}

	/* (non-Javadoc)
	 * @see de.desy.acop.displayers.tools.AcopDisplayer#setArrayIndex(int)
	 */
	public void setArrayIndex(int index) {
	}

	/* (non-Javadoc)
	 * @see de.desy.acop.displayers.tools.AcopDisplayer#getArrayIndex()
	 */
	public int getArrayIndex() {
		return 0;
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.displayers.DoubleSeqConsumer#updateValue(long, double[])
	 */
	public void updateValue(long timestamp, double[] value) throws CommonException {
		if (value==null) {
			updateValue(timestamp, "N/A");
			return;
		}
		StringBuilder s= new StringBuilder(16*value.length);
		s.append('[');
		if (value.length>0) {
			s.append(value[0]);
		}
		for (int i = 1; i < value.length; i++) {
			s.append(',');
			s.append(value[i]);
		}
		s.append(']');
		updateValue(timestamp, s.toString());
	}
	
	/**
	 * Enables/disables Properties item in the popup menu.
	 * @param enable
	 */
	public void setPropertiesPopupEnabled(boolean enable) {
		Action[] actions = getPopupManager().getActions();
		for (Action a : actions) {
			if (a != null && "Preferences...".equals(a.getValue(Action.NAME))) {
				a.setEnabled(enable);
				return;
			}
		}
	}
	
	/**
	 * Returns true if Properties item is enabled in the popup menu.
	 * @return
	 */
	public boolean isPropertiesPopupEnabled() {
		Action[] actions = getPopupManager().getActions();
		for (Action a : actions) {
			if (a != null && "Preferences...".equals(a.getValue(Action.NAME))) {
				return a.isEnabled();
			}
		}
		return false;
	}

}
