package de.desy.acop.displayers;

import java.beans.PropertyVetoException;
import java.util.Map;

import javax.swing.Action;

import com.cosylab.gui.InfoDialog;
import com.cosylab.gui.SliderDisplayer;
import com.cosylab.gui.displayers.DataConsumer;
import com.cosylab.gui.displayers.DisplayerUtilities;
import com.cosylab.gui.displayers.DoubleConsumer;
import com.cosylab.gui.displayers.DoubleConsumerMulticaster;
import com.cosylab.gui.displayers.DoubleSeqConsumer;
import com.cosylab.gui.util.UserSettingsProtection;
import com.cosylab.util.CommonException;

import de.desy.acop.displayers.tools.AcopDisplayerTransferHandler;
import de.desy.acop.displayers.tools.AcopInfoDialog;
import de.desy.acop.displayers.tools.AcopDisplayer;
import de.desy.acop.transport.ConnectionParameters;
import de.desy.acop.transport.adapters.AcopTransportDataSource;
import de.desy.acop.transport.adapters.AdapterFactory;
import de.desy.acop.transport.adapters.AdapterFactoryService;
import de.desy.tine.definitions.TArrayType;

public class AcopSlider extends SliderDisplayer implements DoubleSeqConsumer,  
	AcopDisplayer {
	 
	private static final long serialVersionUID = -2503329416853153653L;
	private ConnectionParameters connectionParameters;
	private int arrayIndex=0;
	private AcopInfoDialog dialog;

	public AcopSlider() {
		super();
		
		new AcopDisplayerTransferHandler(this);
		UserSettingsProtection.setProtection(this,DisplayerUtilities.COMMON_NUMERIC_DISPLAYER_PROPERTIES,false);
	}
	
	public void updateValue(long timestamp, double[] value) throws CommonException {
	    updateValue(timestamp, DisplayerUtilities.extract(arrayIndex,value));	
    }

	/* (non-Javadoc)
	 * @see com.cosylab.gui.displayers.DataConsumer#getDataConsumer(java.lang.Class)
	 */
	@Override
	public DataConsumer getDataConsumer(Class type)
	{
		if (type == DoubleConsumer.class || type == DoubleSeqConsumer.class) {
			return this;
		}
		
		return DoubleConsumerMulticaster.createDataConsumer(type, this);
	}
	
	/*
	 * (non-Javadoc)
	 * @see com.cosylab.gui.DialKnobDisplayer#setCharacteristics(java.util.Map)
	 */
	@Override
	public void setCharacteristics(Map characteristics) {
		super.setCharacteristics(characteristics);
		
		Object o  = characteristics.get("arrayType");
		if (o != null) {
			if (((TArrayType)o).isChannel()) {
				try {
					UserSettingsProtection.setUnprotected(this,"arrayIndex",0);
				} catch (Exception e) {
					e.printStackTrace();
				}
			}
		}
	}
	
	/*
	 * (non-Javadoc)
	 * @see de.desy.acop.displayers.AcopDisplayerConnector#getConnectionParameters()
	 */
	public ConnectionParameters getConnectionParameters() {
		return connectionParameters;
	}
	
	/*
	 * (non-Javadoc)
	 * @see de.desy.acop.displayers.AcopDisplayerConnector#setConnectionParameters(de.desy.acop.transport.ConnectionParameters)
	 */
	public void setConnectionParameters(ConnectionParameters param) throws CommonException, PropertyVetoException {

		if (param!=null && connectionParameters != null) {
			if(param.equals(connectionParameters)) return;
		}
			
		ConnectionParameters old = connectionParameters;
		this.connectionParameters = param;
		AdapterFactory factory = AdapterFactoryService.getInstance().getAdapterFactory();
		if (getDataSource() != null)
			factory.releaseDataSource((AcopTransportDataSource) getDataSource());
		if (param == null) {
			setDataSource(null);
			setTitle("No Connection");
		} else {
			if (param.getPropertySize() == ConnectionParameters.AUTO_PROPERTY_SIZE) {
				param = param.deriveWithPropertySize(1);
			}
			AcopTransportDataSource ds = factory.createDataSource(param);
			setDataSource(ds);
		}
		firePropertyChange(CONNECTION_PARAMETERS_PROPERTY,old, connectionParameters);
	}

	
	/**
	 * Returns the arrayIndex.
	 * @return Returns the arrayIndex.
	 */
	public int getArrayIndex() {
		return arrayIndex;
	}

	
	/**
	 * Sets new arrayIndex.
	 * @param arrayIndex The arrayIndex to set.
	 */
	public void setArrayIndex(int arrayIndex) {
		if (this.arrayIndex == arrayIndex) return;
		int oldValue = this.arrayIndex;
		this.arrayIndex = arrayIndex;
		firePropertyChange("arrayIndex", oldValue, this.arrayIndex);
	}

	
	@Override
	public InfoDialog getInfoDialog() {
		if (dialog == null) {
			dialog = new AcopInfoDialog(this);
		}
		return dialog;
	}
	
	/**
	 * Enables/disables Properties item in the popup menu.
	 * @param enable
	 */
	public void setPropertiesPopupEnabled(boolean enable) {
		Action[] actions = getPopupManager().getActions();
		for (Action a : actions) {
			if (a != null && "Preferences...".equals(a.getValue(Action.NAME))) {
				a.setEnabled(enable);
				return;
			}
		}
	}
	
	/**
	 * Returns true if Properties item is enabled in the popup menu.
	 * @return
	 */
	public boolean isPropertiesPopupEnabled() {
		Action[] actions = getPopupManager().getActions();
		for (Action a : actions) {
			if (a != null && "Preferences...".equals(a.getValue(Action.NAME))) {
				return a.isEnabled();
			}
		}
		return false;
	}
}

/* __oOo__ */