package de.desy.acop.displayers;

import java.awt.AlphaComposite;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Image;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.awt.event.HierarchyEvent;
import java.awt.event.HierarchyListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.beans.Beans;
import java.util.ArrayList;
import java.util.Date;

import javax.swing.BorderFactory;
import javax.swing.ButtonGroup;
import javax.swing.DefaultListCellRenderer;
import javax.swing.DefaultListModel;
import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.JTextArea;
import javax.swing.SwingConstants;
import javax.swing.SwingUtilities;
import javax.swing.Timer;
import javax.swing.border.EtchedBorder;

import com.cosylab.gui.components.util.IconHelper;
import com.cosylab.gui.components.util.RunnerHelper;
import com.cosylab.gui.plugins.VitragePlugIn;

import de.desy.tine.client.TLink;
import de.desy.tine.client.TLinkFactory;

/**
 * 
 * @author Jaka Bobnar, Cosylab
 *
 */
public class AcopSpider extends JLabel {
	
	private static final long serialVersionUID = -3394462753800819240L;
	
	private static final Icon SPIDER_ICON = IconHelper.createIcon("acop/spider.gif");
	private SpiderDialog infoDialog;
	private boolean isError = false;
	private boolean autoResize = true;
	
	private VitragePlugIn vitrage;
	private boolean paintOverlayDecorations = true;
	
	private Timer timer;
	private boolean paintColor = true;
	private Thread linkListenerThread;
	
	public AcopSpider() {
		super();
		initialize();
	}
	
	private void initialize() {
		setOpaque(true);
		setHorizontalAlignment(SwingConstants.CENTER);
		setVerticalAlignment(SwingConstants.CENTER);
		setIgnoreRepaint(true);
		setBackground(Color.WHITE);
		setBorder(BorderFactory.createEtchedBorder(EtchedBorder.RAISED));
		setIcon(SPIDER_ICON);
		addComponentListener(new ComponentAdapter() {
			@Override
			public void componentResized(ComponentEvent e) {
				if (isAutoResize())
					rescaleIconToFit();
			}
		});
		addMouseListener(new MouseAdapter(){
			@Override
			public void mouseClicked(MouseEvent e) {
				if (getRootPane() != null) {
					getInfoDialog().setLocationRelativeTo(getRootPane());
				}
				getInfoDialog().setVisible(true);
			}
		});
		
		addHierarchyListener(new HierarchyListener() {
			public void hierarchyChanged(HierarchyEvent e) {
				if (vitrage == null) {
					if (getRootPane() != null) {
						installVitragePlugin(getRootPane());
					} 
				}
				if (linkListenerThread == null) {
					linkListenerThread = new Thread(new Runnable(){
						public void run() {
							TLink[] links;
							boolean errorExists = false;
							while (true) {
								links = TLinkFactory.getInstance().getActiveLinks();
//								links = TLinkFactory.getInstance().getLinkTable();
								if (!isError() && links != null) {
									int linkCount = 0;
									for (TLink l : links) {
										if (l != null){
											if (l.getLinkStatus() != 0) {
												getInfoDialog().update();
												linkCount = -1;
												break;
											}
											linkCount++;
										}
									}
									if (linkCount != -1 && infoDialog != null && infoDialog.listModel != null && linkCount != infoDialog.listModel.getSize()) {
										getInfoDialog().update();
									}
								} else if (isError()) {
									errorExists = false;
									if (links != null) {
										for (TLink l : links) {
											if (l != null && l.getLinkStatus() != 0) {
												errorExists = true;
												break;
											}
										}
									}
									if (!errorExists) {
										getInfoDialog().update();
									}
								}								
								
								try {
									Thread.sleep(1000);
								} catch (InterruptedException e) {
									addError(e);
								}
							} 
						}
					}, "Acop Spider");
					linkListenerThread.start();
				}
			}
		});
		
		setMaximumSize(new Dimension(80,80));
		setMinimumSize(new Dimension(16,16));
		setPreferredSize(new Dimension(30,30));
	}
	
	/*
	 * (non-Javadoc)
	 * @see javax.swing.JLabel#setIcon(javax.swing.Icon)
	 */
	@Override
	public void setIcon(Icon icon) {
		int height = getHeight() > 0 ? (int)(getHeight()*0.7) : 16;
		int width = getWidth() > 0 ? (int)(getWidth()*0.7) : 16;
		if (icon instanceof ImageIcon) {
			if (width <= 0) width = 1;
			icon = new ImageIcon(((ImageIcon)icon).getImage().getScaledInstance(
					width, -1, Image.SCALE_FAST));
			if (icon.getIconHeight() > height) {
				if (height <= 0) height = 1;
				icon = new ImageIcon(((ImageIcon)icon).getImage().getScaledInstance(
						-1, height, Image.SCALE_FAST));
			}
		} 
		super.setIcon(icon);
	}
	
	/*
	 * (non-Javadoc)
	 * @see javax.swing.JComponent#paintComponent(java.awt.Graphics)
	 */
	@Override
	protected void paintComponent(Graphics g) {
		super.paintComponent(g);
		((Graphics2D)g).setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_ATOP,0.5f));
		if (isError)
			if (paintColor) {
				g.setColor(Color.RED.brighter());
			} else {
				g.setColor(Color.GRAY.brighter());
			}
		else 
			g.setColor(Color.GREEN.darker());
		g.fillRect(0,0, getWidth(), getHeight());

	}
	
	public void setError(boolean isError) {
		if (this.isError == isError) return;
		this.isError = isError;
		if (isError) {
			getTimer().start();
		} else {
			getTimer().stop();
			repaint();
		}
		 
		
	}
	
	public boolean isError() {
		return this.isError;
	}
	
	private SpiderDialog getInfoDialog() {
		if (infoDialog == null) {
			infoDialog = new SpiderDialog();
		}
		return infoDialog;
	}
	
	
	private class SpiderDialog extends JDialog {
		
		class ListRenderer extends DefaultListCellRenderer {
			
			private static final long serialVersionUID = -6412407684875722175L;

			@Override
			public Component getListCellRendererComponent(JList list, Object value, int index, boolean isSelected, boolean cellHasFocus) {
				
				String name = "";
				
				if (value instanceof TLink) {
					try {
						TLink link = (TLink) value;
						Date date = new Date(link.getLastTimeStamp());
						String device = link.getFullDeviceName()+"/"+link.getProperty();
						String error = link.getLastError();
						name = date + "       " + error + "       " + device;
					} catch (Exception e) {
						addError(e);
					}
					
				}
				
				return super.getListCellRendererComponent(list, name, index, isSelected,
						cellHasFocus);
			}
			
		}
		

		private static final long serialVersionUID = 2063792587625913979L;
		private JList linkList;
		private DefaultListModel listModel;
		private JTabbedPane tabPane; 
		private JPanel contentPanel;
		private JPanel buttonPanel;
		private JPanel linkPanel;
		private JPanel exceptionModePanel;
		private JTextArea exceptionArea;
		
		private JButton closeButton;
		private JButton clearButton;
		private JButton refreshButton;
		private JCheckBox debugCheck;
		private JLabel debugLevelLabel;
		private JRadioButton debugLevel1;
		private JRadioButton debugLevel2;
		private ArrayList<Throwable> errors = new ArrayList<Throwable>();
		
		public void setVisible(boolean b) {
			if (b) {
				update();
			}
			super.setVisible(b);
			if (Beans.isDesignTime()) {
				getTabPane().setSelectedIndex(1);
			}
		}
		
		public SpiderDialog() {
			super();
			initialize();
			
		}
		
		private void initialize() {
			setSize(500,500);
			setContentPane(getContentPanel());
			setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
			setTitle("Tine Status Viewer");
		}
		
		private JPanel getContentPanel() {
			if (contentPanel == null) {
				contentPanel = new JPanel(new BorderLayout());
				contentPanel.add(getTabPane(), BorderLayout.CENTER);
				contentPanel.add(getButtonPanel(), BorderLayout.SOUTH);
			}

			return contentPanel;
		}
		
		private JTabbedPane getTabPane() {
			if (tabPane == null) {
				tabPane = new JTabbedPane();
				tabPane.add("Links", getLinkPanel());
				tabPane.add("Exceptions", getExceptionModePanel());
			}
			return tabPane;
		}
		
		private JTextArea getExceptionArea() {
			if (exceptionArea == null) {
				exceptionArea = new JTextArea();
				exceptionArea.setWrapStyleWord(true);
				exceptionArea.setEditable(false);
			}
			return exceptionArea;
		}
		
		private JPanel getExceptionModePanel() {
			if (exceptionModePanel == null) {
				exceptionModePanel = new JPanel(new BorderLayout());
				JScrollPane areaScroll = new JScrollPane(getExceptionArea(), JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
				exceptionModePanel.add(areaScroll, BorderLayout.CENTER);
			}
			return exceptionModePanel;
		}
		
		private JPanel getLinkPanel() {
			if (linkPanel == null) {
				linkPanel = new JPanel(new BorderLayout());
				linkList = new JList();
				listModel = new DefaultListModel();
				linkList.setModel(listModel);
				linkList.setCellRenderer(new ListRenderer());
				JScrollPane listScroll = new JScrollPane(linkList);
				linkPanel.add(listScroll, BorderLayout.CENTER);
			}

			return linkPanel;

		}
		
		private JPanel getButtonPanel() {
			if (buttonPanel == null) {
				buttonPanel = new JPanel(new GridBagLayout());
				
				closeButton = new JButton("Close");
				closeButton.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent e) {
						dispose();						
					}
				});
				buttonPanel.add(closeButton, new GridBagConstraints(0,0,1,1,0,0,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(2,2,2,2),0,0));
				clearButton = new JButton("Clear");
				clearButton.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent e) {
						listModel.clear();
						synchronized (errors) {
							errors.clear();
						}
						getExceptionArea().setText("");
						setError(false);
					}
				});
				buttonPanel.add(clearButton, new GridBagConstraints(1,0,1,1,0,0,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(2,2,2,2),0,0));
				refreshButton = new JButton("Refresh");
				refreshButton.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent e) {
						update();						
					}
				});
				buttonPanel.add(refreshButton, new GridBagConstraints(2,0,1,1,0,0,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(2,2,2,2),0,0));
				
				debugCheck = new JCheckBox("Debug");
				debugCheck.setSelected(false);
				debugCheck.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent e) {
						if (debugCheck.isSelected())
							TLinkFactory.setOutputDebugLevel(getDebugLevel());
						else 
							TLinkFactory.setOutputDebugLevel(0);
					}
				});
				buttonPanel.add(debugCheck, new GridBagConstraints(3,0,1,1,0,0,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(2,2,2,2),0,0));
				
				debugLevelLabel = new JLabel("Debug level:");
				buttonPanel.add(debugLevelLabel, new GridBagConstraints(4,0,1,1,0,0,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(2,2,2,2),0,0));
				
				ButtonGroup debugGroup = new ButtonGroup();
				debugLevel1 = new JRadioButton("1");
				debugGroup.add(debugLevel1);
				debugLevel2 = new JRadioButton("2");
				debugGroup.add(debugLevel2);
				debugLevel1.setSelected(true);
				buttonPanel.add(debugLevel1, new GridBagConstraints(5,0,1,1,0,0,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(2,2,2,2),0,0));
				buttonPanel.add(debugLevel2, new GridBagConstraints(6,0,1,1,0,0,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(2,2,2,2),0,0));
			}

			return buttonPanel;
		}
		
		private int getDebugLevel() {
			if (debugLevel1.isSelected()) return 1;
			else if (debugLevel2.isSelected()) return 2;
			else return 0;
		}
		
		public void update() {
			SwingUtilities.invokeLater(new Runnable(){
				public void run() {
					listModel.clear();
					TLink[] links = TLinkFactory.getInstance().getActiveLinks();
					boolean errorExists = false;
					if (links != null) {
						for (TLink l : links) {
							if (l != null) {
								listModel.addElement(l);
								if (l.getLinkStatus() != 0) {
									errorExists = true;
								}
							}
						}
					}
					
					synchronized (errors) {
						getExceptionArea().setText("");
						StringBuilder sb;
						for (Throwable e : errors) {
							sb = new StringBuilder("Error: \n");
							sb.append(e.toString() + "\n");
							for (StackTraceElement st : e.getStackTrace()) {
								sb.append(st.toString() + "\n");
							}
							getExceptionArea().append(sb.toString() + " \n \n");
						}
						errorExists |= errors.size() > 0;
						setError(errorExists);
					}
				}
			});		
		}
		
		public void addError(Throwable e) {
			synchronized(errors) {
				errors.add(e);
			}
		}
	}
	
	
	
	public static void main(String[] args) {
		final AcopSpider bean = new AcopSpider();
		RunnerHelper.runComponent(bean, 300,300);
		new Thread(new Runnable() {
			public void run() {
				while (true) {
				bean.setError(!bean.isError());
				try {
					Thread.sleep(5000);
				} catch (InterruptedException e) {
					e.printStackTrace();
				}
				}
			}
		}).start();
		System.out.println(bean.getRootPane());
	}

	private void installVitragePlugin(JComponent c) {
		if (vitrage != null) return;
		vitrage = new VitragePlugIn(c);
		vitrage.setVitrageEnabled(isPaintOverlayDecorations());
	}
	
	public boolean isPaintOverlayDecorations() {
		return paintOverlayDecorations;
	}

	public void setPaintOverlayDecorations(boolean useVitrage) {
		boolean old = isPaintOverlayDecorations();
		if (old == useVitrage) return;
		if (vitrage != null)
			vitrage.setVitrageEnabled(useVitrage);
		firePropertyChange("paintOverlayDecorations", old, useVitrage);
	}

	/**
	 * Returns true if icon resizes automatically.
	 * @return
	 */
	public boolean isAutoResize() {
		return autoResize;
	}

	/**
	 * Sets whether icon should be resized automatically.
	 * @param autoResize
	 */
	public void setAutoResize(boolean autoResize) {
		this.autoResize = autoResize;
	}
	
	/**
	 * Rescales the image icon to fit the current displayer.
	 *
	 */
	public void rescaleIconToFit() {
		setIcon(SPIDER_ICON);
	}
	
	private Timer getTimer() {
		if (timer == null) {
			timer = new Timer(700, new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					paintColor = !paintColor;
					AcopSpider.this.repaint();
				}
				
			});
		}
		return timer;
	}
	
	public void addError(Throwable e) {
		getInfoDialog().addError(e);
		setError(true);
	}
}
