/**
 * 
 */
package de.desy.acop.displayers.chart;

import java.awt.Color;
import java.beans.Beans;
import java.beans.PropertyChangeSupport;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Map;

import com.cosylab.gui.displayers.CommonDisplayer;
import com.cosylab.gui.displayers.Displayer;
import com.cosylab.util.CommonException;

import de.desy.acop.displayers.AcopTrendChart;
import de.desy.acop.displayers.selector.SelectorUtilities;
import de.desy.acop.displayers.tools.AcopGraphHistoryParameters;
import de.desy.acop.displayers.tools.AcopGraphParameters;

/**
 * @author Tilen Kusterle, Cosylab
 *
 */
public class AcopTrendChartConsumer extends AcopChartConsumer {

	public static final String SEQUENCE_LENGTH = "sequenceLength";
	public static final String SEQUENCE_INDEX = "sequenceIndex";
	// TODO AcopConst.max_points is protected!
	public static final int MAX_POINTS = 524288;
	
	private PropertyChangeSupport propertyChangeSupport;
	
	private int sequenceLength = 0;
	private int sequenceIndex = 0;
	
	private int plotLength = 0;
	private double maxHistoryTimestamp = 0;
	private HistoryDataConverter historyDataConverter;
	
	public AcopTrendChartConsumer(AcopTrendChart acopTrendChart, String name) {
		super(acopTrendChart, name);
		super.setTrendChart(true);
		xData = new double[0];
		yData = new double[0];
		historyDataConverter = new HistoryDataConverter();
	}
	
	protected void updateChart() {
		if (acopChartReorg == null) return;
		
		trimData();
		
		synchronized(acopChartReorg) {
			if (displayHandle < 0) {
				applyParameters();
			    displayHandle = acopChartReorg.getAcop().draw(yData,xData,disable,xLabels,arraySize,maxNumber);
			    plotLength = xData.length;
			}
			else {
				if (plotLength == xData.length) {
					acopChartReorg.getAcop().refreshScreen(yData, displayHandle, arraySize, 0, xData);
				}
				else reinitializePlot();
			}
			getAcopChart2().autoScale();
		}
		
	}

	/* (non-Javadoc)
	 * @see de.desy.acop.displayers.tools.AcopChartReorgConsumer#setCharacteristics(java.util.Map)
	 */
	@Override
	public void setCharacteristics(Map characteristics) {

		if (acopChartReorg == null) {
			return;
		}

		Object o1 = null;
		Object o2 = null;

		o1 = characteristics.get(CommonDisplayer.C_GRAPH_MAX);

		if (o1 == null) {
			o1 = characteristics.get(CommonDisplayer.C_MAXIMUM);
		}

		o2 = characteristics.get(CommonDisplayer.C_GRAPH_MIN);

		if (o2 == null) {
			o2 = characteristics.get(CommonDisplayer.C_MINIMUM);
		}

		if (o1 != null || o2 != null) {
			double max = o1 != null ? ((Number)o1).doubleValue()
				: acopChartReorg.getYRangeMax();
			double min = o2 != null ? ((Number)o2).doubleValue()
				: acopChartReorg.getYRangeMin();
			
			setPreferredYMax(max);
			setPreferredYMin(min);
			acopChartReorg.updateYScale(this);
		}

		o1 = characteristics.get(CommonDisplayer.C_DISPLAY_NAME);
		
		String newName = (String) o1;
		if (newName != null && !newName.equals(acopChartReorg.getTitle())) {
			// TODO why was this here?
//		    acopChart2.getAcop().setName(newName);
		    acopChartReorg.setTitle(newName);
		}
		
		o1 = characteristics.get(Displayer.C_UNITS);
		if (o1 != null) {
			acopChartReorg.getAcop().setYAxisLabel((String)o1);
		}

		//avoid setting colors through characterstics to avoid different threads
		//setting the same property at the same time - consumer might end up with wrong color
		o1 = characteristics.get(Displayer.C_COLOR);

		if (o1 != null) {
			color = (Color) o1;
			acopChartReorg.getAcop().setForeground(color);
		}
			
		o1 = characteristics.get("xLabels");
		
		o1 = characteristics.get(CommonDisplayer.C_SEQUENCE_LENGTH);
		setSequenceLength((Integer) o1);
		
		if (!Beans.isDesignTime()) {
			acopChartReorg.updateXScale(this);
		}
		acopChartReorg.repaint();
	}

	/* (non-Javadoc)
	 * @see de.desy.acop.displayers.tools.AcopChartReorgConsumer#setDisplayerParameters(de.desy.acop.displayers.tools.AcopGraphParameters)
	 */
	@Override
	public void setDisplayerParameters(AcopGraphParameters displayerParameters) {
		this.displayerParameters = displayerParameters;
		this.color = displayerParameters.getColor() != null ? displayerParameters.getColor() : acopChartReorg.getAcop().getForeground();
		this.width = displayerParameters.getWidth();
		this.dispalyMode = displayerParameters.getMode() > -1 ? displayerParameters.getMode() : acopChartReorg.getAcop().getDisplayMode();
		setSequenceLength(SelectorUtilities.getSequenceLength(displayerParameters.getConnectionParameters()));
		setSequenceIndex(displayerParameters.getArrayIndex());
		historyDataConverter.useConverter(displayerParameters.getConverter());
	}

	/* (non-Javadoc)
	 * @see de.desy.acop.displayers.tools.AcopChartReorgConsumer#updateValue(long, double[])
	 */
	@Override
	public void updateValue(long timestamp, double[] value) throws CommonException {
		lastTimestamp=timestamp;
		setSequenceLength(value.length);
		((AcopTrendChart) acopChartReorg).setLastTimestamp(timestamp/1000.);
		appendPoint(timestamp/1000., value[getSequenceIndex()]);
		updateChart();
	}

	/* (non-Javadoc)
	 * @see de.desy.acop.displayers.tools.AcopChartReorgConsumer#isTrendChart()
	 */
	@Override
	public boolean isTrendChart() {
		return true;
	}

	/* (non-Javadoc)
	 * @see de.desy.acop.displayers.tools.AcopChartReorgConsumer#setTrendChart(boolean)
	 */
	@Override
	public void setTrendChart(boolean isTrendChart) {
		// TODO do nothing or some notification?
	}
	
	private void firePropertyChange(String propertyName, int oldValue, int newValue) {
		getPropertyChangeSupport().firePropertyChange(propertyName, oldValue, newValue);
	}

	private PropertyChangeSupport getPropertyChangeSupport() {
		if (propertyChangeSupport == null) {
			propertyChangeSupport = new PropertyChangeSupport(this);
		}
		return propertyChangeSupport;
	}

	private void trimData() {
		if (!((AcopTrendChart) acopChartReorg).isAutoScaleToLiveData()) {
			int index;
			double presentTime = ((AcopTrendChart) acopChartReorg).getLastTimestamp();
			double timeSpan = ((AcopTrendChart) acopChartReorg).getTimeSpan();
			double startTime = presentTime-timeSpan;
			for (index = 0; index < xData.length; index++) {
				if (xData[index] > startTime) {
					break;
				}
			}
			if (index != 0) {
				if (index > xData.length) index = xData.length;
				double[] xDataCopy =  Arrays.copyOf(xData, xData.length);
				double[] yDataCopy =  Arrays.copyOf(yData, yData.length);
				xData = new double[xData.length-index];
				yData = new double[yData.length-index];
				System.arraycopy(xDataCopy, index, xData, 0, xDataCopy.length-index);
				System.arraycopy(yDataCopy, index, yData, 0, yDataCopy.length-index);
			}
		}
		if (xData.length > MAX_POINTS) {
			int i;
			for (i = 0; i < xData.length; i++) {
				if (xData[i] > maxHistoryTimestamp) {
					i--;
					break;
				}
			}
			if (i < 0 || i > MAX_POINTS) i = 0;
			double[] xDataCopy =  Arrays.copyOf(xData, xData.length);
			double[] yDataCopy =  Arrays.copyOf(yData, yData.length);
			xData = new double[MAX_POINTS];
			yData = new double[MAX_POINTS];
			System.arraycopy(xDataCopy, 0, xData, 0, i);
			System.arraycopy(yDataCopy, 0, yData, 0, i);
			System.arraycopy(xDataCopy, xDataCopy.length-(MAX_POINTS-i), xData, i, MAX_POINTS-i);
			System.arraycopy(yDataCopy, yDataCopy.length-(MAX_POINTS-i), yData, i, MAX_POINTS-i);
			acopChartReorg.autoScaleXOnce();
		}
		
	}
	
	private void mergeData(double[][] historyData) {
		ArrayList<Double> xList = new ArrayList<Double>();
		ArrayList<Double> yList = new ArrayList<Double>();
		boolean skip = false;
		for (int i = 0; i < historyData[0].length; i++) {
			double t = historyData[0][i];
			for (int j = 0; j < xList.size(); j++) {
				if (xList.get(j) >= t) {
					if (xList.get(j) > t) {
						xList.add(j, t);
						yList.add(j, historyData[1][i]);
					}
					skip = true;
					break;
				}
			}
			if (skip) {
				skip = false;
				continue;
			}
			xList.add(t);
			yList.add(historyData[1][i]);
		}
		for (int i = 0; i < xData.length; i++) {
			double t = xData[i];
			for (int j = 0; j < xList.size(); j++) {
				if (xList.get(j) >= t) {
					if (xList.get(j) > t) {
						xList.add(j, t);
						yList.add(j, yData[i]);
					}
					skip = true;
					break;
				}
			}
			if (skip) {
				skip = false;
				continue;
			}
			xList.add(t);
			yList.add(yData[i]);
		}
		if (historyData[0].length > 0) maxHistoryTimestamp = historyData[0][historyData[0].length-1];
		xData = new double[xList.size()];
		yData = new double[yList.size()];
		for (int i = 0; i < xList.size(); i++) {
			xData[i] = xList.get(i);
			yData[i] = yList.get(i);
		}
	}
	
	private void appendPoint(double x, double y) {
		double[] xDataCopy =  Arrays.copyOf(xData, xData.length);
		double[] yDataCopy =  Arrays.copyOf(yData, yData.length);
		xData = new double[xData.length+1];
		yData = new double[yData.length+1];
		System.arraycopy(xDataCopy, 0, xData, 0, xDataCopy.length);
		System.arraycopy(yDataCopy, 0, yData, 0, yDataCopy.length);
		xData[xData.length-1] = x;
		yData[yData.length-1] = y;
	}

	/* (non-Javadoc)
	 * @see de.desy.acop.displayers.tools.AcopChartReorgConsumer#dumpData()
	 */
	@Override
	public double[][] dumpData() {
		double[][] clone = new double[2][xData.length];
		System.arraycopy(xData, 0, clone[0] ,0,xData.length);
		System.arraycopy(yData, 0, clone[1] ,0,xData.length);
		return clone;
	}

	/**
	 * Gets the sequence index.
	 * @return the sequenceIndex
	 */
	public int getSequenceIndex() {
		return sequenceIndex;
	}

	/**
	 * Sets the sequence index.
	 * @param sequenceIndex the sequenceIndex to set
	 */
	public void setSequenceIndex(int sequenceIndex) {
		if (this.sequenceIndex == sequenceIndex) return;
		int oldIndex = getSequenceIndex();
		if (sequenceIndex >= getSequenceLength() || sequenceIndex < 0) {
			sequenceIndex = 0;
		}
		this.sequenceIndex = sequenceIndex;
		firePropertyChange(SEQUENCE_INDEX, oldIndex, getSequenceIndex());
	}

	/**
	 * Gets the length of the sequence.
	 * @return the sequenceLength
	 */
	public int getSequenceLength() {
		return sequenceLength;
	}

	/**
	 * Sets the length of the sequence.
	 * @param sequenceLength the sequenceLength to set
	 */
	public void setSequenceLength(int sequenceLength) {
		if (this.sequenceLength == sequenceLength) return;
		int oldLength = getSequenceLength();
		this.sequenceLength = sequenceLength;
		if (getSequenceIndex() >= getSequenceLength()) {
			setSequenceIndex(0);
		}
		firePropertyChange(SEQUENCE_LENGTH, oldLength, getSequenceLength());
	}

	public HistoryParameters getHistoryParameters() {
		AcopGraphParameters parameters = getDisplayerParameters();
		if (parameters instanceof AcopGraphHistoryParameters) return ((AcopGraphHistoryParameters) parameters).getHistoryParameters();
		return null;
	}

	public void loadHistoryData(double[][] historyData) {
		double[] convertedHistoryData = historyDataConverter.convertHistoryData(historyData[1]);
		historyData[1] = convertedHistoryData;
		mergeData(historyData);
		updateChart();
	}
	
}
