package de.desy.acop.displayers.chart;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;

import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JPanel;

import de.desy.acop.chart.AcopDisplayMode;

public class ChartOptionsPanel extends JPanel {

	private static final long serialVersionUID = 1L;
	private JCheckBox textCheckBox;
	private JCheckBox chubbyLinesCheckBox;
	private JComboBox lineDotsCombo;
	private JCheckBox extraDigitCheckBox;
	private JCheckBox showGridCheckBox;
	private JCheckBox useBestScaleCheckBox;
	
	private boolean textSelected;
	private AcopDisplayMode lineStyle;
	private boolean chubbyLines;
	private boolean extraDigits;
	private boolean showGrid;
	private boolean useBestScale;
	private boolean compactView = false;
	
	private Dimension comboDim = new Dimension(110, 24);
	private Dimension dim = new Dimension(110,21);
	
	public ChartOptionsPanel() {
		super();
		initialize();
	}
	
	private void initialize() {
		setLayout(new GridBagLayout());

		textCheckBox = new JCheckBox("Text");
		textCheckBox.addItemListener(new ItemListener(){
			public void itemStateChanged(ItemEvent e) {
				setTextSelected(textCheckBox.isSelected());
			}
		});
		textCheckBox.setMinimumSize(dim);
		textCheckBox.setMaximumSize(dim);
		textCheckBox.setPreferredSize(dim);
		textCheckBox.setToolTipText("Enables Text Markers");
		
		extraDigitCheckBox = new JCheckBox("Extra Digits");
		extraDigitCheckBox.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				setExtraDigits(extraDigitCheckBox.isSelected());				
			}
		});
		extraDigitCheckBox.setMinimumSize(dim);
		extraDigitCheckBox.setMaximumSize(dim);
		extraDigitCheckBox.setPreferredSize(dim);
		extraDigitCheckBox.setToolTipText("Shows Extra Digits in Traces Table");
		
		chubbyLinesCheckBox = new JCheckBox("Chubby Lines");
		chubbyLinesCheckBox.addItemListener(new ItemListener(){
			public void itemStateChanged(ItemEvent e) {
				setChubbyLines(chubbyLinesCheckBox.isSelected());
			}
		});	
		chubbyLinesCheckBox.setMinimumSize(dim);
		chubbyLinesCheckBox.setMaximumSize(dim);
		chubbyLinesCheckBox.setPreferredSize(dim);
		chubbyLinesCheckBox.setToolTipText("Toggles Chubby/Normal lines");
		
		showGridCheckBox = new JCheckBox("Show Grid");
		showGridCheckBox.addItemListener(new ItemListener(){
			public void itemStateChanged(ItemEvent e) {
				setShowGrid(showGridCheckBox.isSelected());
			}
		});	
		showGridCheckBox.setMinimumSize(dim);
		showGridCheckBox.setMaximumSize(dim);
		showGridCheckBox.setPreferredSize(dim);
		showGridCheckBox.setToolTipText("Toggles grid lines visibility");
		
		useBestScaleCheckBox = new JCheckBox("Best Scale");
		useBestScaleCheckBox.addItemListener(new ItemListener(){
			public void itemStateChanged(ItemEvent e) {
				setUseBestScale(useBestScaleCheckBox.isSelected());
			}
		});	
		useBestScaleCheckBox.setMinimumSize(dim);
		useBestScaleCheckBox.setMaximumSize(dim);
		useBestScaleCheckBox.setPreferredSize(dim);
		useBestScaleCheckBox.setToolTipText("Toggles XY best scale enabled/disabled");
		
		lineDotsCombo = new JComboBox(AcopDisplayMode.values());
		lineDotsCombo.addItemListener(new ItemListener(){
			public void itemStateChanged(ItemEvent e) {
				if (e.getStateChange() == ItemEvent.SELECTED) {
					setLineDisplayMode((AcopDisplayMode)lineDotsCombo.getSelectedItem());
				}
			}
		});
		lineDotsCombo.setMinimumSize(comboDim);
		lineDotsCombo.setMaximumSize(comboDim);
		lineDotsCombo.setPreferredSize(comboDim);
		lineDotsCombo.setToolTipText("Sets Chart Style");
				
		this.add(extraDigitCheckBox, new GridBagConstraints(0,0,1,1,0,0,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(4,4,2,4),0,0));
		this.add(textCheckBox, new GridBagConstraints(0,1,1,1,0,0,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(2,4,2,4),0,0));
		this.add(chubbyLinesCheckBox, new GridBagConstraints(0,2,1,1,0,0,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(2,4,2,4),0,0));
		this.add(showGridCheckBox, new GridBagConstraints(0,3,1,1,0,0,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(2,4,2,4),0,0));
		this.add(useBestScaleCheckBox, new GridBagConstraints(0,4,1,1,0,0,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(2,4,2,4),0,0));
		this.add(lineDotsCombo, new GridBagConstraints(0,5,1,1,0,0,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(2,4,4,4),0,0));
	}
	
	public boolean isChubbyLines() {
		return chubbyLines;
	}
	
	public void setChubbyLines(boolean chubby) {
		if (this.chubbyLines == chubby) return;
		boolean oldChubbyLines = this.chubbyLines;
		this.chubbyLines = chubby;
		chubbyLinesCheckBox.setSelected(this.chubbyLines);
		firePropertyChange("chubbyLines", oldChubbyLines, this.chubbyLines);

	}
	
	public void setExtraDigits(boolean extra) {
		if (this.extraDigits == extra) return;
		boolean oldExtraDigits = this.extraDigits;
		this.extraDigits = extra;
		extraDigitCheckBox.setSelected(this.extraDigits);
		firePropertyChange("extraDigits", oldExtraDigits, this.extraDigits);

	}
	
	public boolean isExtraDigits() {
		return extraDigits;
	}
	
	public AcopDisplayMode getLineDisplayMode() {
		return lineStyle;
	}
	
	public void setLineDisplayMode(AcopDisplayMode style) {
		if (lineStyle == style) return;
		AcopDisplayMode oldStyle = this.lineStyle;
		this.lineStyle = style;
		lineDotsCombo.setSelectedItem(style);
		firePropertyChange("lineDisplayMode", oldStyle, style);
	}
	
	public void setTextSelected(boolean visible) {
		if (this.textSelected == visible) return;
		boolean oldTextSelected = this.textSelected;
		this.textSelected = visible;
		textCheckBox.setSelected(this.textSelected);
		firePropertyChange("textSelected", oldTextSelected, this.textSelected);
	}
	
	public boolean isTextSelected() {
		return textSelected;
	}
		
	public boolean isUseBestScale() {
		return useBestScale;
	}
	
	public void setUseBestScale(boolean selected) {
		if (this.useBestScale == selected) return;
		boolean oldUseBestScale = this.useBestScale;
		this.useBestScale = selected;
		useBestScaleCheckBox.setSelected(this.useBestScale);
		firePropertyChange("useBestScale", oldUseBestScale, this.useBestScale);
	}
	
	public boolean isShowGrid() {
		return showGrid;
	}
	
	public void setShowGrid(boolean selected) {
		if (this.showGrid == selected) return;
		boolean oldShowGrid = this.showGrid;
		this.showGrid = selected;
		showGridCheckBox.setSelected(this.showGrid);
		firePropertyChange("showGrid", oldShowGrid, this.showGrid);
	}
	
	@Override
	public void setEnabled(boolean enabled) {
		super.setEnabled(enabled);
		textCheckBox.setEnabled(enabled);
		chubbyLinesCheckBox.setEnabled(enabled);
		lineDotsCombo.setEnabled(enabled);
		extraDigitCheckBox.setEnabled(enabled);
		showGridCheckBox.setEnabled(enabled);
		useBestScaleCheckBox.setEnabled(enabled);
	}
	
	public void setCompactView(boolean compact) {
		if (this.compactView == compact) return;
		this.compactView = compact;
		extraDigitCheckBox.setVisible(!compact);
		textCheckBox.setVisible(!compact);
		firePropertyChange("compactView", !compact, compact);
	}
	
	public boolean isCompactView() {
		return compactView;
	}
	
}
