/**
 * 
 */
package de.desy.acop.displayers.chart;

import java.beans.PropertyVetoException;

import com.cosylab.gui.adapters.Converter;
import com.cosylab.gui.displayers.DataSourceSupport;
import com.cosylab.gui.displayers.DoubleSeqConsumer;
import com.cosylab.gui.displayers.MinimalDataConsumer;
import com.cosylab.util.CommonException;

/**
 * @author Tilen Kusterle, Cosylab
 *
 */
public class HistoryDataConverter {
	
	private HistoryDataSource historyDataSource;
	private TransformedDataConsumer transformedDataConsumer;
	private Converter converter;
	private double[] trensformedData;
	
	private class HistoryDataSource extends DataSourceSupport {
		
		private static final long serialVersionUID = 1L;
		private DoubleSeqConsumer doubleSeqConsumer;
		
		public HistoryDataSource() {
			super(new Class[]{DoubleSeqConsumer.class});
		}
		
		public void setConverter(Converter c) {
			if (converter != null) converter.removeAllConsumers();
			converter = c;
			if (converter == null || !(converter instanceof DoubleSeqConsumer)) {
				resetConverter();
				return;
			}
			try {
				doubleSeqConsumer = converter.getDataConsumer(DoubleSeqConsumer.class);
				converter.addConsumer(transformedDataConsumer);
				addConsumer(converter);
			} catch (PropertyVetoException e) {
				resetConverter();
				e.printStackTrace();
			}
		}
		
		public void resetConverter() {
			converter = null;
			try {
				doubleSeqConsumer = transformedDataConsumer;
				addConsumer(transformedDataConsumer);
			} catch (PropertyVetoException e) {
				e.printStackTrace();
			}
		}
		
		public void transformData(long timestamp, double[] data) {
			try {
				doubleSeqConsumer.updateValue(timestamp, data);
			} catch (CommonException e) {
				e.printStackTrace();
			}
		}
		
	}
	
	private class TransformedDataConsumer extends MinimalDataConsumer implements DoubleSeqConsumer {
		public void updateValue(long timestamp, double[] value) throws CommonException {
			trensformedData = value;
		}
	}
	
	public HistoryDataConverter() {
		historyDataSource = new HistoryDataSource();
		transformedDataConsumer = new TransformedDataConsumer();
		historyDataSource.resetConverter();
	}
	
	/**
	 * Clones and uses the cloned corverter for converting history data. The converter
	 * passed as argument must be an instance of </code>DoubleSeqConsumer</code>,
	 * otherwise no converter is used. 
	 * @param c the </code>Converter</code> to be cloned and used for converting history
	 * data
	 */
	public void useConverter(Converter c) {
		if (c == null) {
			historyDataSource.resetConverter();
			return;
		}
		try {
			historyDataSource.setConverter((Converter) c.clone());
		} catch (CloneNotSupportedException e) {
			historyDataSource.resetConverter();
			e.printStackTrace();
		}
	}
	
	/**
	 * Converts history data by using the </code>Converter</code> set by 
	 * </code>useConverter</code> method.
	 * @param inputData raw history data
	 * @return converted history data
	 */
	public double[] convertHistoryData(double[] inputData) {
		historyDataSource.transformData(System.currentTimeMillis(), inputData);
		return trensformedData;
	}
	
}
