package de.desy.acop.displayers.chart;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;

import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

import com.cosylab.gui.components.numberfield.DoubleDocument;
import com.cosylab.gui.components.util.RunnerHelper;

public class MinMaxPanel extends JPanel {
	
	private static final long serialVersionUID = 1L;

	private JButton rescaleButton;
	
	private JTextField maxField;
	private JTextField minField;
	private DoubleDocument maxDocument;
	private DoubleDocument minDocument;
	private JLabel maxLabel;
	private JLabel minLabel;
	private double min = 0;
	private double max = 0;
	private Dimension fieldDim = new Dimension(70,21); 
	private boolean rescaleEnabled = false;
	
	
	
	public MinMaxPanel() {
		super();
		initialize();
	}
	
	private void initialize() {
		setLayout(new GridBagLayout());	
		
		maxField = new JTextField();
		maxDocument = new DoubleDocument();
		maxField.setDocument(maxDocument);
		maxField.setText(String.valueOf(max));
		maxField.addKeyListener(new KeyAdapter() {
			public void keyPressed(KeyEvent evt) {
				if (evt.getKeyCode() >= KeyEvent.VK_0 && evt.getKeyCode() <= KeyEvent.VK_9 || evt.getKeyCode() == KeyEvent.VK_DELETE || evt.getKeyCode() == KeyEvent.VK_BACK_SPACE) {
					rescaleEnabled = true;
					rescaleButton.setEnabled(isEnabled());
				}
			}
		});
		
		maxField.setMinimumSize(fieldDim);
		maxField.setPreferredSize(fieldDim);
		maxField.setToolTipText("Maximum Value");
		
		maxLabel = new JLabel("Max");
		this.add(maxField, new GridBagConstraints(1,0,1,1,1,0,GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(0,2,2,0),0,0));
		this.add(maxLabel, new GridBagConstraints(0,0,1,1,0,0,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0,0,2,2),0,0));
		
		minField = new JTextField();
		minDocument = new DoubleDocument();
		minField.setDocument(minDocument);
		minField.setText(String.valueOf(min));
		minField.addKeyListener(new KeyAdapter() {
			public void keyPressed(KeyEvent evt) {
				if (evt.getKeyCode() >= KeyEvent.VK_0 && evt.getKeyCode() <= KeyEvent.VK_9 ||
						evt.getKeyCode() == KeyEvent.VK_DELETE || evt.getKeyCode() == KeyEvent.VK_BACK_SPACE) {
					rescaleEnabled = true;
					rescaleButton.setEnabled(isEnabled());
				}
			}
		});
		minField.setMinimumSize(fieldDim);
		minField.setPreferredSize(fieldDim);
		minField.setToolTipText("Minimum Value");
		
		minLabel = new JLabel("Min");
		this.add(minField, new GridBagConstraints(1,1,1,1,1,0,GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(2,2,2,0),0,0));
		this.add(minLabel, new GridBagConstraints(0,1,1,1,0,0,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0,2,2,2),0,0));
		
		rescaleButton = new JButton("Apply Scale");
		rescaleButton.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				rescaleEnabled = false;
				setMin(minDocument.getValue().doubleValue());
				setMax(maxDocument.getValue().doubleValue());
				rescaleButton.setEnabled(false);
			}
		});
		rescaleButton.setEnabled(false);
		rescaleButton.setToolTipText("Apply New Scale Settings");
		rescaleButton.setEnabled(false);
		
		this.add(rescaleButton, new GridBagConstraints(0,2,2,1,1,0,GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(2,0,0,0),0,0));
	}
	
	/**
	 * Returns the maximum value.
	 * @return
	 */
	public double getMax() {
		return max;
	}

	/**
	 * Sets maximum value.
	 * @param max
	 */
	public void setMax(double max) {
		setMax(max, true);
	}
	
	public void setMax(double max, boolean notify) {
		if (this.max == max) return;
		double oldValue = this.max;
		this.max = max;
		boolean enabled = maxField.isEnabled();
		maxField.setEnabled(true);
		maxField.setText(String.valueOf(max));
		maxField.setEnabled(enabled);
		maxField.setCaretPosition(0);
		if (notify) {
			firePropertyChange("max", oldValue, this.max);
		}
	}

	/**
	 * Returns the minimum value.
	 * @return
	 */
	public double getMin() {
		return min;
	}

	/**
	 * Sets the minimum.
	 * @param min
	 */
	public void setMin(double min) {
		setMin(min, true);
	}
	
	public void setMin(double min, boolean notify) {
		if (this.min == min) return;
		double oldValue = this.min;
		this.min = min;
		boolean enabled = minField.isEnabled();
		minField.setEnabled(true);
		minField.setText(String.valueOf(min));
		minField.setEnabled(enabled);
		minField.setCaretPosition(0);
		if (notify) {
			firePropertyChange("min", oldValue, this.min);
		}
	}
	
	public void setRescaleEnabled(boolean enabled) {
		if (enabled == rescaleEnabled) return;
		this.rescaleEnabled = enabled;
		rescaleButton.setEnabled(enabled & isEnabled());
		firePropertyChange("rescaleEnabled", !enabled, enabled);
	}
	
	public boolean isRescaleEnabled() {
		return rescaleEnabled;
	}

	@Override
	public void setEnabled(boolean enabled) {
		super.setEnabled(enabled);
		minField.setEnabled(enabled);
		maxField.setEnabled(enabled);
		minLabel.setEnabled(enabled);
		maxLabel.setEnabled(enabled);
		rescaleButton.setEnabled(enabled & rescaleEnabled);
	}
	
	public static void main(String[] args) {
		MinMaxPanel p = new MinMaxPanel();
		RunnerHelper.runComponent(p, 100,300);
		p.setMin(1.99232123243214142E15);
	}
}
