package de.desy.acop.displayers.chart;

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import javax.swing.Icon;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;

public class ScalePanel extends JPanel {
	
	private static final long serialVersionUID = 1L;
	private JButton defaultScaleButton;
	private JCheckBox autoScaleCheckBox;
	private Dimension dim = new Dimension(110,21);
	private JComboBox linLogCombo;
	private JLabel logCriticalLabel;
	private JButton paneUpButton;
	private JButton paneDownButton;
	private JButton zoomInButton;
	private JButton zoomOutButton;
	private MinMaxPanel minMaxPanel;

	public static final String CLEAR_CMD = "clear";
	public static final String DEFAULT_SCALE_CMD = "defaultScale";

	private static final String LOG_CRITICAL = "Log critical";
	
	private LinLogStyle selectedStyle = LinLogStyle.LIN;
	private boolean logCritical = false;
	private boolean autoScale = false;
	private boolean relativeScaleMode = false;
	private double defaultMin = 0;
	private double defaultMax = 0;
	private boolean minMaxEnabled = true;
	
	
	public ScalePanel() {
		super();
		initialize();
	}
	
	private void initialize() {
		setLayout(new GridBagLayout());
		
		JPanel shiftZoomPanel = new JPanel(new GridBagLayout());
		
		paneDownButton = new JButton(getUpDownIcon(0));
		paneDownButton.setToolTipText("Pane down");
		paneDownButton.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				if (isAutoScale()) setAutoScale(false);
				double dx = (getMinMaxPanel().getMax() - getMinMaxPanel().getMin())/10;
				getMinMaxPanel().setMax(getMax() + dx);
				getMinMaxPanel().setMin(getMin() + dx);
				getMinMaxPanel().setRescaleEnabled(false);
			}
		});
		shiftZoomPanel.add(paneDownButton, new GridBagConstraints(3, 0, 1, 1, 1, 1, 14, 0, new Insets(2, 2, 2, 2), 0, 0));
		
		
		paneUpButton = new JButton(getUpDownIcon(1));
		paneUpButton.setToolTipText("Pane up");
		paneUpButton.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				if (isAutoScale()) setAutoScale(false);
				double dx = (getMinMaxPanel().getMax() - getMinMaxPanel().getMin())/10;
				getMinMaxPanel().setMax(getMax() - dx);
				getMinMaxPanel().setMin(getMin() - dx);
				getMinMaxPanel().setRescaleEnabled(false);
			}
		});
		shiftZoomPanel.add(paneUpButton, new GridBagConstraints(2, 0, 1, 1, 1, 1, 14, 0, new Insets(2, 2, 2, 2), 0, 0));
		
		zoomInButton = new JButton(getUpDownIcon(2));
		zoomInButton.setToolTipText("Zoom in");
		zoomInButton.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				if (isAutoScale()) setAutoScale(false);
				double dx = (getMinMaxPanel().getMax() - getMinMaxPanel().getMin())/20;
				getMinMaxPanel().setMax(getMax() - dx);
				getMinMaxPanel().setMin(getMin() + dx);
				getMinMaxPanel().setRescaleEnabled(false);
			}
		});
		shiftZoomPanel.add(zoomInButton, new GridBagConstraints(0, 0, 1, 1, 1, 1, 14, 0, new Insets(2, 2, 2, 2), 0, 0));
		
		zoomOutButton = new JButton(getUpDownIcon(3));
		zoomOutButton.setToolTipText("Zoom out");
		zoomOutButton.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				if (isAutoScale()) setAutoScale(false);
				double dx = (getMinMaxPanel().getMax() - getMinMaxPanel().getMin())/20;
				getMinMaxPanel().setMax(getMax() + dx);
				getMinMaxPanel().setMin(getMin() - dx);
				getMinMaxPanel().setRescaleEnabled(false);
			}
		});
		shiftZoomPanel.add(zoomOutButton, new GridBagConstraints(1, 0, 1, 1, 1, 1, 14, 0, new Insets(2, 2, 2, 2), 0, 0));
		
		this.add(shiftZoomPanel, new GridBagConstraints(0,0,3,1,0,0,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE, new Insets(2,4,2,2),0,0));
			
		this.add(getMinMaxPanel(), new GridBagConstraints(0,1,1,2,1,0,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH, new Insets(2,4,2,2),0,0));
		
		JPanel rightPanel = new JPanel(new GridBagLayout());
		
		linLogCombo = new JComboBox(LinLogStyle.values());		
		linLogCombo.addItemListener(new ItemListener(){
			public void itemStateChanged(ItemEvent e) {
				if (e.getStateChange() == ItemEvent.SELECTED) {
					setStyle((LinLogStyle)linLogCombo.getSelectedItem());
				} 
			}
		});
		linLogCombo.setMinimumSize(new Dimension(80,21));
		linLogCombo.setPreferredSize(new Dimension(80,21));
		linLogCombo.setToolTipText("Scale Type of the Trace");
		
		rightPanel.add(linLogCombo, new GridBagConstraints(0,0,1,1,1,0,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(2,2,2,4),0,0));
		
		logCriticalLabel = new JLabel(" ");
		logCriticalLabel.setForeground(Color.RED);
		logCriticalLabel.setOpaque(true);
		rightPanel.add(logCriticalLabel, new GridBagConstraints(0,1,1,1,1,0,GridBagConstraints.SOUTHEAST, GridBagConstraints.NONE, new Insets(2,2,2,4),0,0));
//		logCriticalLabel.setVisible(false);

		this.add(rightPanel, new GridBagConstraints(1,1,1,2,1,0,GridBagConstraints.NORTHEAST, GridBagConstraints.NONE, new Insets(2,4,2,2),0,0));
		
		defaultScaleButton = new JButton("Default Scale");
		defaultScaleButton.setActionCommand(ScalePanel.DEFAULT_SCALE_CMD);
		defaultScaleButton.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				setAutoScale(false);
				setMax(getDefaultMax());
				setMin(getDefaultMin());
			}
		});
		defaultScaleButton.setMinimumSize(dim);
		defaultScaleButton.setToolTipText("Apply Default Scale");
		
		this.add(defaultScaleButton, new GridBagConstraints(0,3,1,1,1,0,GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(2,4,4,2),0,0));

		autoScaleCheckBox = new JCheckBox("Auto Scale");
		autoScaleCheckBox.setHorizontalTextPosition(JCheckBox.LEADING);
		autoScaleCheckBox.addItemListener(new ItemListener(){
			public void itemStateChanged(ItemEvent e) {
				setAutoScale(autoScaleCheckBox.isSelected());
			}
		});
		autoScaleCheckBox.setEnabled(false);
		autoScaleCheckBox.setToolTipText("Toggle Optimal Autoscale");
		this.add(autoScaleCheckBox, new GridBagConstraints(1,3,1,1,1,0,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(2,2,2,0),0,0));
		
		
	}
	
	private MinMaxPanel getMinMaxPanel() {
		if (minMaxPanel == null) {
			minMaxPanel = new MinMaxPanel();
			minMaxPanel.addPropertyChangeListener(new PropertyChangeListener(){
				public void propertyChange(PropertyChangeEvent evt) {
					String name = evt.getPropertyName();
					if ("max".equals(name) || "min".equals(name) && minMaxEnabled) {
						firePropertyChange(name, evt.getOldValue(), evt.getNewValue());
					}
				}
			});
		}
		return minMaxPanel;
	}
	
	/*
	 * (non-Javadoc)
	 * @see javax.swing.JComponent#setEnabled(boolean)
	 */
	@Override
	public void setEnabled(boolean e) {
		super.setEnabled(e);
		getMinMaxPanel().setEnabled(e & relativeScaleMode & minMaxEnabled);
		autoScaleCheckBox.setEnabled(e & relativeScaleMode);
		defaultScaleButton.setEnabled(e & relativeScaleMode);
		linLogCombo.setEnabled(e & !logCritical);
		paneDownButton.setEnabled(e & relativeScaleMode);
		paneUpButton.setEnabled(e & relativeScaleMode);
		zoomInButton.setEnabled(e & relativeScaleMode);
		zoomOutButton.setEnabled(e & relativeScaleMode);
	}	
	
	public boolean isAutoScale() {
		return autoScaleCheckBox.isSelected();
	}
	
	public void setAutoScale(boolean auto) {
		if (this.autoScale == auto) return;
		boolean oldAutoScale = this.autoScale;
		this.autoScale = auto;
		autoScaleCheckBox.setSelected(this.autoScale);
		firePropertyChange("autoScale", oldAutoScale, this.autoScale);
	}
	
	public void setRelativeScaleMode(boolean relative) {
		if (relativeScaleMode == relative) return;
		this.relativeScaleMode = relative;
		boolean enabled = isEnabled();
		autoScaleCheckBox.setEnabled(enabled & relative);
		minMaxPanel.setEnabled(enabled & relative & minMaxEnabled);
		zoomInButton.setEnabled(enabled & relative);
		zoomOutButton.setEnabled(enabled & relative);
		paneDownButton.setEnabled(enabled & relative);
		paneUpButton.setEnabled(enabled & relative);
		defaultScaleButton.setEnabled(enabled & relative);
		linLogCombo.setEnabled(enabled);
		firePropertyChange("relativeScaleMode", !relative, relative);
	}
	
	public void setMinMaxEnabled(boolean enabled) {
		if (this.minMaxEnabled == enabled) return;
		this.minMaxEnabled = enabled;
		minMaxPanel.setEnabled(isEnabled() & relativeScaleMode & minMaxEnabled);
		firePropertyChange("minMaxEnabled", !enabled, enabled);
	}
	
	public boolean getRelativeScaleMode() {
		return relativeScaleMode;
	}

	/**
	 * Returns true if log critical warning is shown.
	 * @return
	 */
	public boolean isLogCritical() {
		return logCritical;
	}

	/**
	 * Shows/hides label showing a warning that data is logarithmically critical.
	 * @param logCritical
	 */
	public void setLogCritical(boolean logCritical) {
		if (this.logCritical == logCritical) return;
		this.logCritical = logCritical;
		if (logCritical) {
			logCriticalLabel.setText(LOG_CRITICAL);
		} else {
			logCriticalLabel.setText(" ");
		}
		linLogCombo.setEnabled(isEnabled() & !logCritical);
		firePropertyChange("logCritical", !logCritical, logCritical);
	}

	/**
	 * Returns the maximum value.
	 * @return
	 */
	public double getMax() {
		return getMinMaxPanel().getMax();
	}

	/**
	 * Sets maximum value.
	 * @param max
	 */
	public void setMax(double max) {
		getMinMaxPanel().setMax(max);
	}
	
	public void setMax(double max, boolean notify) {
		getMinMaxPanel().setMax(max, notify);
	}

	/**
	 * Returns the minimum value.
	 * @return
	 */
	public double getMin() {
		return getMinMaxPanel().getMin();
	}

	/**
	 * Sets the minimum.
	 * @param min
	 */
	public void setMin(double min) {
		getMinMaxPanel().setMin(min);
	}
	
	public void setMin(double min, boolean notify) {
		getMinMaxPanel().setMin(min, notify);
	}

	/**
	 * Returns the default maximum value.
	 * @return the defaultMax
	 */
	public double getDefaultMax() {
		return defaultMax;
	}

	/**
	 * Sets the default maximum value.
	 * @param defaultMax the defaultMax to set
	 */
	public void setDefaultMax(double defaultMax) {
		if (this.defaultMax == defaultMax) return;
		double oldValue = getDefaultMax();
		this.defaultMax = defaultMax;
		firePropertyChange("defaultMax", oldValue, getDefaultMax());
	}

	/**
	 * Returns the default minimum value.
	 * @return the defaultMin
	 */
	public double getDefaultMin() {
		return defaultMin;
	}

	/**
	 * Sets the default minimum value.
	 * @param defaultMin the defaultMin to set
	 */
	public void setDefaultMin(double defaultMin) {
		if (this.defaultMin == defaultMin) return;
		double oldValue = getDefaultMin();
		this.defaultMin = defaultMin;
		firePropertyChange("defaultMin", oldValue, getDefaultMin());
	}

	/**
	 * Return the style selected in the combo.
	 * @return
	 */
	public LinLogStyle getStyle() {
		return selectedStyle;
	}

	/**
	 * Sets selected style.
	 * @param style
	 */
	public void setStyle(LinLogStyle style) {
		if (this.selectedStyle == style) return;
		LinLogStyle oldValue = this.selectedStyle;
		this.selectedStyle = style;
		linLogCombo.setSelectedItem(this.selectedStyle);
		firePropertyChange("style", oldValue, this.selectedStyle);
	}
	
	/**
	 * Generates icon with up or down arrow. 
	 * 
	 * @param i if i == 0, down arrow is returned, if i == 1, up arrow is returned, if i == 2, "+" icon is returned, else "-" icon is returned
	 * @return arrow icon
	 */
	private Icon getUpDownIcon(int i){
		switch(i){
		case 0:
			// down arrow
			return new Icon(){
				public int getIconHeight() {
					return 10;
				}
				public int getIconWidth() {
					return 10;
				}
				public void paintIcon(Component c, Graphics g, int x, int y) {
					if (!c.isEnabled())g.setColor(Color.LIGHT_GRAY);
					else g.setColor(c.getForeground());
					g.drawLine(x, y + 4, x + 4, y + 8);
					g.drawLine(x + 4, y + 8, x + 8, y + 4);
					g.drawLine(x + 4, y, x + 4, y + 8);
				}
			};
		
		case 1:
			// up arrow
			return new Icon(){
				public int getIconHeight() {
					return 10;
				}
				public int getIconWidth() {
					return 10;
				}
				public void paintIcon(Component c, Graphics g, int x, int y) {
					if (!c.isEnabled())g.setColor(Color.LIGHT_GRAY);
					else g.setColor(c.getForeground());
					g.drawLine(x, y + 4, x + 4, y);
					g.drawLine(x + 4, y, x + 8, y + 4);
					g.drawLine(x + 4, y, x + 4, y + 8);
				}
			};
		
		case 2:
			// plus icon
			return new Icon(){
				public int getIconHeight() {
					return 10;
				}
				public int getIconWidth() {
					return 10;
				}
				public void paintIcon(Component c, Graphics g, int x, int y) {
					if (!c.isEnabled())g.setColor(Color.LIGHT_GRAY);
					else g.setColor(c.getForeground());
					g.drawLine(x, y + 4, x + 8, y + 4);
					g.drawLine(x + 4, y, x + 4, y + 8);
				}
			};
			
		default:
			// minus icon
			return new Icon(){
				public int getIconHeight() {
					return 10;
				}
				public int getIconWidth() {
					return 10;
				}
				public void paintIcon(Component c, Graphics g, int x, int y) {
					if (!c.isEnabled())g.setColor(Color.LIGHT_GRAY);
					else g.setColor(c.getForeground());
					g.drawLine(x, y + 4, x + 8, y + 4);
				}
			};
		}
	}
}