/**
 * 
 */
package de.desy.acop.displayers.chart;

import java.util.ArrayList;

import de.desy.tine.histUtils.THistory;
import de.desy.tine.types.DBLDBL;

/**
 * @author Tilen Kusterle, Cosylab
 *
 */
public class THistoryConnector {
	
	private boolean simulate;
	private boolean checkForBadData;
	private int timeout = 3000;

	/**
	 * Reads history data from server.
	 * @param context
	 * @param server
	 * @param device
	 * @param property
	 * @param start in seconds
	 * @param stop in seconds
	 * @return
	 */
	protected synchronized double[][] getHistoryData(String context, String server, String device, String property, int index, double start, double stop) {		
		int size = 1000;
		DBLDBL[] data = new DBLDBL[size];
		if (simulate) {
			double span = stop - start;
			for (int i = 0; i < size; i++) {
				data[i] = new DBLDBL(Math.random()*3 + 60, start + span/size * i);
			}
		} else {
			THistory.getArchivedData(context, server, property, device, index, start, stop, data, timeout);
		}
	
		ArrayList<Double> x = new ArrayList<Double>();
		ArrayList<Double> y = new ArrayList<Double>();
		if (isCheckForBadData()) {
			for (int i = 0; i < data.length; i++) {
				if (data[i].d2val == 0 || data[i].d1val > 1E30) continue;
				x.add(data[i].d2val);
				y.add(data[i].d1val);
			}
		} else {
			for (int i = 0; i < data.length; i++) {
				if (data[i].d2val == 0) continue;
				x.add(data[i].d2val);
				y.add(data[i].d1val);
			}
		}
		
		double[][] result = new double[2][x.size()];
		for (int i = 0; i < result[0].length; i++) {
			result[0][i] = x.get(i);
			result[1][i] = y.get(i);
		}
		return result;		
	}
	
	public double[][] loadHistory(HistoryParameters hp, double start, double stop) {
		if (hp == null) throw new IllegalArgumentException("History parameters can not be null.");
		return getHistoryData(hp.getHistoryContext(), hp.getHistoryServer(), hp.getHistoryDevice(), hp.getHistoryProperty(), hp.getHistoryIndex(), start, stop);
	}

	/**
	 * @return the checkForBadData
	 */
	public boolean isCheckForBadData() {
		return checkForBadData;
	}

	/**
	 * @param checkForBadData the checkForBadData to set
	 */
	public void setCheckForBadData(boolean checkForBadData) {
		this.checkForBadData = checkForBadData;
	}

	/**
	 * @return the simulate
	 */
	public boolean isSimulate() {
		return simulate;
	}

	/**
	 * @param simulate the simulate to set
	 */
	public void setSimulate(boolean simulate) {
		this.simulate = simulate;
	}
	
	/**
	 * @return the timeout
	 */
	public int getTimeout() {
		return timeout;
	}

	/**
	 * @param timeout the timeout to set
	 */
	public void setTimeout(int timeout) {
		this.timeout = timeout;
	}
	
}
