/**
 * 
 */
package de.desy.acop.displayers.chart;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.DefaultComboBoxModel;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

import com.cosylab.gui.components.util.RunnerHelper;


/**
 * @author Tilen Kusterle, Cosylab
 *
 */
public class TrendSelector extends JPanel {

	public static void main(String[] args) {
		RunnerHelper.runComponent(new TrendSelector(), 440,400);
	}
	private static final long serialVersionUID = 1L;
	private static final String SUBSYSTEM = "HIST";
	
	private JLabel subsystemLabel;
	private JLabel contextLabel;
	private JLabel serverLabel;
	private JLabel propertyLabel;
	private JLabel deviceLabel;
	private JLabel indexLabel;
	
	private JLabel subsystemValueLabel;
	private JTextField contextField;
	private JTextField serverField;
	private JTextField propertyField;
	private JTextField deviceField;
	private JComboBox indexCombo;
	
	private JCheckBox enableHistoryBox;
	private boolean historyEnabled = false;
	
	private IndexModel indexModel;
	
	private class IndexModel extends DefaultComboBoxModel {
		
		private static final long serialVersionUID = 1L;

		public void setMaxIndex(int maxIndex) {
			removeAllElements();
			for (int i = 0; i <= maxIndex; i++) {
				addElement(new Integer(i));
			}
		}
	}
	
	public TrendSelector() {
		initialize();
	}
	
	private void initialize() {
		setLayout(new GridBagLayout());
		
		add(getEnableHistoryBox(), new GridBagConstraints(0,0,2,1,1,1,GridBagConstraints.WEST,GridBagConstraints.NONE,new Insets(1,1,1,1),0,0));
		add(getSubsystemLabel(), new GridBagConstraints(0,1,1,1,1,1,GridBagConstraints.EAST,GridBagConstraints.NONE,new Insets(1,1,1,1),0,0));
		add(getSubsystemValueLabel(), new GridBagConstraints(1,1,1,1,1,1,GridBagConstraints.WEST,GridBagConstraints.NONE,new Insets(1,1,1,1),0,0));
		add(getContextLabel(), new GridBagConstraints(0,2,1,1,1,1,GridBagConstraints.EAST,GridBagConstraints.NONE,new Insets(1,1,1,1),0,0));
		add(getContextField(), new GridBagConstraints(1,2,1,1,1,1,GridBagConstraints.WEST,GridBagConstraints.NONE,new Insets(1,1,1,1),0,0));
		add(getServerLabel(), new GridBagConstraints(0,3,1,1,1,1,GridBagConstraints.EAST,GridBagConstraints.NONE,new Insets(1,1,1,1),0,0));
		add(getServerField(), new GridBagConstraints(1,3,1,1,1,1,GridBagConstraints.WEST,GridBagConstraints.NONE,new Insets(1,1,1,1),0,0));
		add(getDeviceLabel(), new GridBagConstraints(0,4,1,1,1,1,GridBagConstraints.EAST,GridBagConstraints.NONE,new Insets(1,1,1,1),0,0));
		add(getDeviceField(), new GridBagConstraints(1,4,1,1,1,1,GridBagConstraints.WEST,GridBagConstraints.NONE,new Insets(1,1,1,1),0,0));
		add(getPropertyLabel(), new GridBagConstraints(0,5,1,1,1,1,GridBagConstraints.EAST,GridBagConstraints.NONE,new Insets(1,1,1,1),0,0));
		add(getPropertyField(), new GridBagConstraints(1,5,1,1,1,1,GridBagConstraints.WEST,GridBagConstraints.NONE,new Insets(1,1,1,1),0,0));
		add(getIndexLabel(), new GridBagConstraints(0,6,1,1,1,1,GridBagConstraints.EAST,GridBagConstraints.NONE,new Insets(1,1,1,1),0,0));
		add(getIndexCombo(), new GridBagConstraints(1,6,1,1,1,1,GridBagConstraints.WEST,GridBagConstraints.NONE,new Insets(1,1,1,1),0,0));
		
		enableHistoryChoosing(false);
		getEnableHistoryBox().setSelected(false);
		
	}
	
	private JCheckBox getEnableHistoryBox() {
		if (enableHistoryBox == null) {
			enableHistoryBox = new JCheckBox("History Enabled");
			enableHistoryBox.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					enableHistoryChoosing(enableHistoryBox.isSelected());
				}});
			enableHistoryBox.setSelected(true);
		}

		return enableHistoryBox;
	}
	
	private JLabel getSubsystemLabel() {
		if (subsystemLabel == null) {
			subsystemLabel = new JLabel("Subsystem");
			subsystemLabel.setPreferredSize(new Dimension(120,21));
			subsystemLabel.setMinimumSize(new Dimension(120,21));
			subsystemLabel.setMaximumSize(new Dimension(120,21));
		}

		return subsystemLabel;
	}
	
	private JLabel getSubsystemValueLabel() {
		if (subsystemValueLabel == null) {
			subsystemValueLabel = new JLabel(SUBSYSTEM);
			subsystemValueLabel.setPreferredSize(new Dimension(120,21));
			subsystemValueLabel.setMinimumSize(new Dimension(120,21));
			subsystemValueLabel.setMaximumSize(new Dimension(120,21));
		}

		return subsystemValueLabel;
	}
	
	private JLabel getContextLabel() {
		if (contextLabel == null) {
			contextLabel = new JLabel("Context");
			contextLabel.setPreferredSize(new Dimension(120,21));
			contextLabel.setMinimumSize(new Dimension(120,21));
			contextLabel.setMaximumSize(new Dimension(120,21));
		}

		return contextLabel;
	}
	
	private JTextField getContextField() {
		if (contextField == null) {
			contextField = new JTextField();
			contextField.setPreferredSize(new Dimension(120,21));
			contextField.setMinimumSize(new Dimension(120,21));
			contextField.setMaximumSize(new Dimension(120,21));
		}

		return contextField;
	}
	
	private JLabel getServerLabel() {
		if (serverLabel == null) {
			serverLabel = new JLabel("Server");
			serverLabel.setPreferredSize(new Dimension(120,21));
			serverLabel.setMinimumSize(new Dimension(120,21));
			serverLabel.setMaximumSize(new Dimension(120,21));
		}

		return serverLabel;
	}
	
	private JTextField getServerField() {
		if (serverField == null) {
			serverField = new JTextField();
			serverField.setPreferredSize(new Dimension(120,21));
			serverField.setMinimumSize(new Dimension(120,21));
			serverField.setMaximumSize(new Dimension(120,21));
		}

		return serverField;
	}
	
	private JLabel getPropertyLabel() {
		if (propertyLabel == null) {
			propertyLabel = new JLabel("Property");
			propertyLabel.setPreferredSize(new Dimension(120,21));
			propertyLabel.setMinimumSize(new Dimension(120,21));
			propertyLabel.setMaximumSize(new Dimension(120,21));
		}

		return propertyLabel;
	}
	
	private JTextField getPropertyField() {
		if (propertyField == null) {
			propertyField = new JTextField();
			propertyField.setPreferredSize(new Dimension(120,21));
			propertyField.setMinimumSize(new Dimension(120,21));
			propertyField.setMaximumSize(new Dimension(120,21));
		}

		return propertyField;
	}
	
	private JLabel getDeviceLabel() {
		if (deviceLabel == null) {
			deviceLabel = new JLabel("Device");
			deviceLabel.setPreferredSize(new Dimension(120,21));
			deviceLabel.setMinimumSize(new Dimension(120,21));
			deviceLabel.setMaximumSize(new Dimension(120,21));
		}

		return deviceLabel;
	}
	
	private JTextField getDeviceField() {
		if (deviceField == null) {
			deviceField = new JTextField();
			deviceField.setPreferredSize(new Dimension(120,21));
			deviceField.setMinimumSize(new Dimension(120,21));
			deviceField.setMaximumSize(new Dimension(120,21));
		}

		return deviceField;
	}
	
	private JLabel getIndexLabel() {
		if (indexLabel == null) {
			indexLabel = new JLabel("Index");
			indexLabel.setPreferredSize(new Dimension(120,21));
			indexLabel.setMinimumSize(new Dimension(120,21));
			indexLabel.setMaximumSize(new Dimension(120,21));
		}

		return indexLabel;
	}
	
	private JComboBox getIndexCombo() {
		if (indexCombo == null) {
			indexCombo = new JComboBox();
			indexCombo.setModel(getIndexModel());
			indexCombo.setPreferredSize(new Dimension(120,21));
			indexCombo.setMinimumSize(new Dimension(120,21));
			indexCombo.setMaximumSize(new Dimension(120,21));
		}

		return indexCombo;
	}
	
	private IndexModel getIndexModel() {
		if (indexModel == null) {
			indexModel = new IndexModel();
			indexModel.addElement(new Integer(0));
		}

		return indexModel;
	}
	
	private void enableHistoryChoosing(boolean enable) {
		getContextField().setEnabled(enable);
		getServerField().setEnabled(enable);
		getDeviceField().setEnabled(enable);
		getPropertyField().setEnabled(enable);
	}
	
	public HistoryParameters getHistoryParameters() {
		if (!enableHistoryBox.isSelected()) return null;
		return new HistoryParameters(
				getContextField().getText(),
				getServerField().getText(),
				getDeviceField().getText(),
				getPropertyField().getText(),
				(Integer) getIndexCombo().getSelectedItem());
	}
	
	public void setHistoryParameters(HistoryParameters parameters) {
		if (parameters == null) {
			setValues(null, null, null, null);
		}
		else setValues(
				parameters.getHistoryContext(),
				parameters.getHistoryServer(),
				parameters.getHistoryDevice(),
				parameters.getHistoryProperty());
	}
	
	public void setHistoryParameters(String context, String server, String device, String property) {
		setValues(context, server, device, property);
		getIndexModel().setMaxIndex(0);
		
	}
	
	private void setValues(String context, String server, String device, String property) {
		getContextField().setText(context == null ? "" : context);
		getServerField().setText(server == null ? "" : server);
		getDeviceField().setText(device == null ? "" : device);
		getPropertyField().setText(property == null ? "" : property);
	}
	
	public void setSelectedIndex(int index) {
		getIndexCombo().setSelectedItem(new Integer(index));
	}
	
	public void setIndexMax(int max) {
		getIndexModel().setMaxIndex(max);
	}
	
	public int getSelectedIndex() {
		return getIndexCombo().getSelectedIndex();
	}

	/**
	 * @return the historyEnabled
	 */
	public boolean isHistoryEnabled() {
		return historyEnabled;
	}

	/**
	 * @param historyEnabled the historyEnabled to set
	 */
	public void setHistoryEnabled(boolean historyEnabled) {
		this.historyEnabled = historyEnabled;
		getEnableHistoryBox().setSelected(historyEnabled);
		enableHistoryChoosing(historyEnabled);
	}
	
	
}
