package de.desy.acop.displayers.info;

import java.awt.BorderLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.BorderFactory;
import javax.swing.JCheckBox;
import javax.swing.JPanel;
import javax.swing.JTextArea;

import de.desy.acop.transport.ConnectionParameters;
import de.desy.tine.client.TLinkFactory;
import de.desy.tine.queryUtils.TQuery;

/**
 * 
 * @author Jaka Bobnar, Cosylab
 *
 */
public class AcopDebugPanel extends JPanel {
		
		private static final long serialVersionUID = -1597262900075362868L;
		private JTextArea textArea;
		private ConnectionParameters connectionParameters;
		private JCheckBox debugCheckBox;
		
		public AcopDebugPanel() {
			super();
			initialize();
		}
		
		private void initialize() {
			this.setLayout(new BorderLayout());
			textArea = new JTextArea();
			textArea.setBorder(BorderFactory.createTitledBorder("Fec Information"));
			this.add(textArea, BorderLayout.CENTER);
			JPanel southPanel = new JPanel(new GridBagLayout());
			debugCheckBox = new JCheckBox("Debug");
			debugCheckBox.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					TLinkFactory.getInstance().setDebugLevel(debugCheckBox.isSelected() ? 2 : 0);					
				}
			});
			southPanel.add(debugCheckBox, new GridBagConstraints(0,0,1,1,1,1,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0,0,0,0),0,0));
			this.add(southPanel, BorderLayout.NORTH);			
		}
		
		public void setConnectionParameters(ConnectionParameters param) {
			if (this.connectionParameters != null && this.connectionParameters.equals(param)) return;
			this.connectionParameters = param;
			if (connectionParameters != null){
				textArea.setText("Updating ... Please wait.");
				new Thread(new Runnable() {
					public void run() {
						String srv = connectionParameters.getDeviceGroup();
				        String con = connectionParameters.getDeviceContext();
				        StringBuffer s = new StringBuffer(TQuery.getModuleAddressInfo(srv, con) +"\n");
				        s.append(TQuery.getTineVersion(con, srv) + "\n");
				        s.append(TQuery.getAppVersion(con, srv) + "\n");
				        s.append(TQuery.getAppDate(con, srv) + "\n");
				        textArea.setText(s.substring(0));
					}
				}).start(); 
			} else {
				textArea.setText("");
			}
		}
		
		public ConnectionParameters getConnectionParameters() {
			return connectionParameters;
		}
		
		public void setDebug(int debug) {
			if (debug > 0) {
				debugCheckBox.setSelected(true);
			} else {
				debugCheckBox.setSelected(false);
			}
		}
		
		public int isDebug() {
			return debugCheckBox.isSelected() ? 2 : 0;
		}
	}