package de.desy.acop.displayers.info;

import java.awt.BorderLayout;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Iterator;
import java.util.Map;

import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;

import com.cosylab.gui.components.introspection.PropertiesTableModel;

public class CharacteristicsPanel extends JPanel {
	
	private static final long serialVersionUID = 5967883467749353832L;
	private Map<String, Object> characteristics;
	private JTable table;
	private PropertiesTableModel model;
	private JScrollPane tableScroll;
	
	public CharacteristicsPanel() {
		super();
		initialize();
	}
	
	private void initialize() {
		this.setLayout(new BorderLayout());
		model = new PropertiesTableModel() {
			private static final long serialVersionUID = -8633216620068900661L;

			/*
			 * (non-Javadoc)
			 * @see com.cosylab.gui.components.introspection.PropertiesTableModel#getValueAt(int, int)
			 */
			@Override
			public synchronized Object getValueAt(int rowIndex, int columnIndex) {
				Object obj = super.getValueAt(rowIndex, columnIndex);
				if (obj instanceof Object[]) {
					return Arrays.toString((Object[])obj);
				}
				return obj;
			}
			
			/*
			 * (non-Javadoc)
			 * @see com.cosylab.gui.components.introspection.PropertiesTableModel#getColumnName(int)
			 */
			@Override
			public String getColumnName(int columnIndex)
			{	
				String name = super.getColumnName(columnIndex);
				
				if (name.equals("property")) {
					return "Characteristic";
				} else if (name.equals("value")) {
					return "Value";
				} else {
					return name;
				}
			}
		};
		model.setEditable(false);
		table = new JTable();
		table.setModel(model);
		table.setEnabled(false);
		tableScroll = new JScrollPane(table);
		this.add(tableScroll, BorderLayout.CENTER);
	}
	
	public void setCharacteristics(Map<String, Object> ch) {
		this.characteristics = ch;
		if (characteristics == null) {
			model.setPropertyNames(new String[]{});
			model.setPropertyTypes(new Class[]{});
			model.setPropertyValues(new Object[]{});
		} else {
			Iterator<String> it = characteristics.keySet().iterator();
			String key;
			Object object;
			ArrayList<String> names = new ArrayList<String>();
			ArrayList<Class> types = new ArrayList<Class>();
			ArrayList<Object> values = new ArrayList<Object>();
			while (it.hasNext()) {
				key = it.next();
				object = characteristics.get(key);
				names.add(key);
				types.add(object.getClass());
				values.add(object);
			}
			int size = names.size();
			model.setPropertyNames(names.toArray(new String[size]));
			model.setPropertyTypes(types.toArray(new Class[size]));
			model.setPropertyValues(values.toArray(new Object[size]));
		}
		model.fireTableDataChanged();
	}
}