package de.desy.acop.displayers.selector;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;
import javax.swing.border.EmptyBorder;

import com.cosylab.gui.adapters.ConverterCustomizer;

import de.desy.acop.displayers.tools.AcopDisplayerParameters;
import de.desy.acop.transport.ConnectionParameters;

/**
 * With help of this dialog special AcopDisplayerParameters 
 * are created and customized from ConnectionParameters or existing AcopDisplayerParameters. 
 * 
 * @author tkusterle
 *
 */
public abstract class AcopDisplayerParametersDialog<T extends AcopDisplayerParameters> extends JDialog {
	
	private static final long serialVersionUID = 1L;
	
	private JPanel contentPanel;
	private JPanel labelPanel;
	private JLabel connectionLabel;
	private JPanel buttonPanel;
	private JButton addButton;
	private JButton cancelButton;
	
	private AcopDisplayerParameters parameters;
	private T result;
	private ConverterCustomizer converterCustomizer;
	private JTabbedPane customizerTab;
	
	
	public AcopDisplayerParametersDialog(Component comp) {		
		super(JOptionPane.getFrameForComponent(comp), "Adding New Parameters", true);
		initialize();
	}
	
	private void initialize() {
		this.setLayout(new GridBagLayout());
		this.add(getContentPanel(), new GridBagConstraints(0,0,1,1,1,1,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(11,11,11,11),1,1));
		this.setSize(330, 300);
		this.addWindowListener(new WindowCloser());
	}
	
	private JPanel getContentPanel() {
		if (contentPanel == null) {
			contentPanel = new JPanel();
			contentPanel.setLayout(new BorderLayout());
			contentPanel.add(getCustomizerTabPane(), BorderLayout.CENTER);
			contentPanel.add(getLabelPanel(), BorderLayout.NORTH);
			contentPanel.add(getButtonPanel(), BorderLayout.SOUTH);
			
		}
		return contentPanel;
	}
	
	private JTabbedPane getCustomizerTabPane() {
		if (customizerTab == null) {
			customizerTab = new JTabbedPane();
			customizerTab.add(getCustomizerPanel(), "Properties");
			customizerTab.add(getConverterPanel(), "Converter");
		}
		return customizerTab;
	}
	
	protected ConverterCustomizer getConverterPanel() {
		if (converterCustomizer == null) {
			converterCustomizer = new ConverterCustomizer();
		}
		return converterCustomizer;
	}
	
	private JPanel getLabelPanel() {
		if (labelPanel == null) {
			labelPanel = new JPanel();
			labelPanel.setBorder(new EmptyBorder(new Insets(0,0,5,0)));
			labelPanel.add(getConnectionLabel());
		}
		return labelPanel;
	}
	
	private JLabel getConnectionLabel() {
		if (connectionLabel == null) {
			connectionLabel = new JLabel("Connection");
			connectionLabel.setToolTipText("Connection");
		}
		return connectionLabel;
	}
	
	private void setConnectionLabel(String connection) {
		getConnectionLabel().setText(connection);
		getConnectionLabel().setToolTipText(connection);
	}
	
	private JPanel getButtonPanel() {
		if (buttonPanel == null) {
			buttonPanel = new JPanel();
			buttonPanel.add(getAddButton());
			buttonPanel.add(getCancelButton());
		}
		return buttonPanel;
	}
	
	protected abstract MultipleDisplayerAbstractSettingsPanel<T> getCustomizerPanel();
	
	protected abstract T generateParameters();
	
	private JButton getAddButton() {
		if (addButton == null) {
			addButton = new JButton("Add");
			addButton.addActionListener(new ActionListener() {
	        	public void actionPerformed(ActionEvent e) {
	        		result = generateParameters();
	    		    closeDialog();
	        	}
	        });
	        
        }
		return addButton;
	}
	
	private JButton getCancelButton() {
		if (cancelButton == null) {
			cancelButton = new JButton("Cancel");
			cancelButton.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					result = null;
					closeDialog();
				}
			});
		}
		return cancelButton;
	}
	
	private void closeDialog() {
		this.setVisible(false);
	}
	
	public T showDialog(Component component, AcopDisplayerParameters adp) {
		parameters= adp;
		getCustomizerTabPane().setSelectedIndex(0);
		getConverterPanel().setConverter(null);
		this.setConnectionLabel(adp.getConnectionParameters().getRemoteName());
		this.setLocationRelativeTo(component);
		this.setVisible(true);
		return result;
	}
	
	public T showDialog(Component component, ConnectionParameters cp) {
		parameters = new AcopDisplayerParameters(cp);
		getCustomizerTabPane().setSelectedIndex(0);
		this.getConverterPanel().setConverter(null);
		this.setConnectionLabel(parameters.getConnectionParameters().getRemoteName());
		this.setLocationRelativeTo(component);
		this.setVisible(true);
		return result;
	}
	
	private class WindowCloser extends WindowAdapter {

		/* (non-Javadoc)
		 * @see java.awt.event.WindowAdapter#windowClosing(java.awt.event.WindowEvent)
		 */
		@Override
		public void windowClosing(WindowEvent e) {
			parameters = null;
			super.windowClosed(e);
		}
		
	}
	protected AcopDisplayerParameters getDisplayerParameters() {
		return parameters;
	}



}
