package de.desy.acop.displayers.selector;

import java.awt.Component;

import de.desy.acop.chart.AcopDisplayMode;
import de.desy.acop.chart.AcopDrawStyleEnum;
import de.desy.acop.chart.AcopFFTEnum;
import de.desy.acop.displayers.tools.AcopDisplayerParameters;
import de.desy.acop.displayers.tools.AcopGraphParameters;
import de.desy.acop.transport.ConnectionParameters;

/**
 * 
 * @author Jaka Bobnar, Cosylab
 *
 */
public class AcopGraphParametersDialog extends AcopDisplayerParametersDialog<AcopGraphParameters>{

	private static final long serialVersionUID = 4031445693248397678L;
	private GraphCustomizerPanel graphCustomizerPanel;
	
	public AcopGraphParametersDialog(Component comp) {
		super(comp);
	}

	/*
	 * (non-Javadoc)
	 * @see de.desy.acop.displayers.selector.AcopDisplayerParametersDialog#generateParameters()
	 */
	@Override
	protected AcopGraphParameters generateParameters() {
	    return  new AcopGraphParameters(
		getDisplayerParameters(),
		getGraphCustomizerPanel().getColor(),
		getGraphCustomizerPanel().getGraphFFT(),
		getGraphCustomizerPanel().getGraphStyle(),
		getGraphCustomizerPanel().getGraphMode(),
		getGraphCustomizerPanel().getGraphWidth(),
		getGraphCustomizerPanel().isTrend(),
		getGraphCustomizerPanel().getTrendLength(),
		getConverterPanel().getConverter());
	}

	/*
	 * (non-Javadoc)
	 * @see de.desy.acop.displayers.selector.AcopDisplayerParametersDialog#getCustomizerPanel()
	 */
	@Override
	protected MultipleDisplayerAbstractSettingsPanel<AcopGraphParameters> getCustomizerPanel() {
		return getGraphCustomizerPanel();
	}
	
	private GraphCustomizerPanel getGraphCustomizerPanel()
	{
		if (graphCustomizerPanel == null) {
			graphCustomizerPanel = new GraphCustomizerPanel();
			graphCustomizerPanel.setParameters(null);
		}
		return graphCustomizerPanel;
	}
	
	
	/*
	 * (non-Javadoc)
	 * @see de.desy.acop.displayers.selector.AcopDisplayerParametersDialog#showChartDialog(java.awt.Component, de.desy.acop.displayers.tools.AcopDisplayerParameters)
	 */
	@Override
	public AcopGraphParameters showDialog(Component component, AcopDisplayerParameters adp) {
		this.getGraphCustomizerPanel().setColor(null);
		this.getGraphCustomizerPanel().setGraphFFT(AcopFFTEnum.NoFFT.ordinal());
		this.getGraphCustomizerPanel().setGraphWidth(1);
		this.getGraphCustomizerPanel().setDrawStyle(AcopDrawStyleEnum.PS_SOLID.ordinal());
		if (SelectorUtilities.isChannel(adp.getConnectionParameters())) {
			getGraphCustomizerPanel().setGraphMode(AcopDisplayMode.SimpleHistogram.ordinal());
		} else {
			getGraphCustomizerPanel().setGraphMode(AcopDisplayMode.PolyLine.ordinal());
		}
		return super.showDialog(component, adp);
	}
	
	/*
	 * (non-Javadoc)
	 * @see de.desy.acop.displayers.selector.AcopDisplayerParametersDialog#showChartDialog(java.awt.Component, de.desy.acop.transport.ConnectionParameters)
	 */
	@Override
	public AcopGraphParameters showDialog(Component component, ConnectionParameters cp) {
		this.getGraphCustomizerPanel().setColor(null);
		this.getGraphCustomizerPanel().setGraphFFT(AcopFFTEnum.NoFFT.ordinal());
		this.getGraphCustomizerPanel().setGraphWidth(1);
		this.getGraphCustomizerPanel().setDrawStyle(AcopDrawStyleEnum.PS_SOLID.ordinal());
		if (SelectorUtilities.isChannel(cp)) {
			getGraphCustomizerPanel().setGraphMode(AcopDisplayMode.SimpleHistogram.ordinal());
		} else {
			getGraphCustomizerPanel().setGraphMode(AcopDisplayMode.PolyLine.ordinal());
		}
		return super.showDialog(component, cp);
	}

}
