package de.desy.acop.displayers.selector;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.beans.Customizer;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSlider;
import javax.swing.JSpinner;
import javax.swing.SpinnerNumberModel;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import com.cosylab.gui.components.util.RunnerHelper;

import de.desy.acop.displayers.tools.AcopDisplayer;
import de.desy.acop.transport.ConnectionParameters;

public class ArrayIndexCustomizer extends JPanel implements Customizer{
	
	private static final long serialVersionUID = -401693602095711422L;
	private AcopDisplayer disp;
	private int arrayIndex = 0;
	private JSlider slider;
	private ConnectionParameters connectionParameters;
	private JSpinner spinner;
	private SpinnerNumberModel spinnerModel;
	private int lastValue = 0;
	
	public ArrayIndexCustomizer() {
		super();
		initialize();
	}
	
	private void initialize() {
		this.setLayout(new GridBagLayout());
		add(new JLabel("Array index"), new GridBagConstraints(0,0,1,1,1,0,GridBagConstraints.EAST,GridBagConstraints.NONE,new Insets(4,11,4,4),0,0));
		add(getSpinner(), new GridBagConstraints(1,0,1,1,1,0,GridBagConstraints.WEST,GridBagConstraints.NONE,new Insets(4,4,4,11),0,0));
		add(getSlider(), new GridBagConstraints(0,1,2,1,1,0,GridBagConstraints.CENTER,GridBagConstraints.HORIZONTAL,new Insets(4,11,4,11),0,0));

	}
	
	private JSlider getSlider() {
		if (slider == null) {
			slider = new JSlider();
	
			slider.setMinimum(0);
			slider.setValue(0);
			slider.setMaximum(100);
			slider.setMajorTickSpacing((int)Math.floor(100/4.0));
			slider.setMinorTickSpacing(1);
			slider.setPaintLabels(true);
			slider.setPaintTicks(true);
			slider.setPaintTrack(true);
			slider.setSnapToTicks(true);
			slider.addChangeListener(new ChangeListener() {
				
				public void stateChanged(ChangeEvent e) {
					//register only when value is changed, dont care about max/min
					if (slider.getValue() != lastValue) {
						lastValue = slider.getValue();
						setArrayIndex(lastValue);
						if (disp!=null) {
							disp.setArrayIndex(lastValue);
						}
						
					}
				}
			
			});
			
		}
		return slider;
	}
	
	private void refreshIndexList() {
		if (connectionParameters != null) {
			int length = connectionParameters.getPropertySize();
			if (length <= 0) {
				length = SelectorUtilities.getSequenceLength(connectionParameters);
			}
			if (length <= 1 || SelectorUtilities.isChannel(connectionParameters)) {
				slider.setEnabled(false);
				spinner.setEnabled(false);
				slider.setMaximum(1);
				spinnerModel.setMaximum(1);
			} else {
				slider.setEnabled(true);
				spinner.setEnabled(true);
				slider.setMaximum(length-1);
				spinnerModel.setMaximum(length-1);
				slider.setLabelTable(null);
				slider.setLabelTable(null);
				slider.setMajorTickSpacing((int)Math.floor(length/4.0));
			}
		} else {
			slider.setMaximum(1);
			spinnerModel.setMaximum(1);
			slider.setEnabled(false);
			spinner.setEnabled(false);
		}
	}
		
	/*
	 * (non-Javadoc)
	 * @see java.beans.Customizer#setObject(java.lang.Object)
	 */
	public void setObject(Object bean) {
		if (bean instanceof AcopDisplayer) {
			disp = (AcopDisplayer) bean;
			disp.addPropertyChangeListener("arrayIndex",
					new PropertyChangeListener(){
				public void propertyChange(PropertyChangeEvent evt) {
					setArrayIndex(disp.getArrayIndex());				
				}
			});
			disp.addPropertyChangeListener(AcopDisplayer.CONNECTION_PARAMETERS_PROPERTY,
					new PropertyChangeListener(){
				public void propertyChange(PropertyChangeEvent evt) {
					setConnectionParameters(disp.getConnectionParameters());
					setArrayIndex(disp.getArrayIndex());				
				}
			});
		} else {
			throw new IllegalArgumentException("Only AcopCommonDisplayer and" +
					" MultipleAcopDisplayer can use this Customizer.");
		}
		setConnectionParameters(disp.getConnectionParameters());
		setArrayIndex(disp.getArrayIndex());
	}
	
	public void setArrayIndex(int index) {
		if (index == arrayIndex) return;
		int oldValue = arrayIndex;
		this.arrayIndex = index;
		getSlider().setValue(arrayIndex);
		getSpinner().setValue(arrayIndex);
		firePropertyChange("arrayIndex", oldValue, arrayIndex);
	}
	
	public int getArrayIndex() {
		return arrayIndex;
	}
	
	public void setConnectionParameters(ConnectionParameters parameters) {
		if (this.connectionParameters != null && this.connectionParameters.equals(parameters)) return;
		ConnectionParameters oldValue = this.connectionParameters;
		this.connectionParameters = parameters;
		refreshIndexList();
		firePropertyChange(AcopDisplayer.CONNECTION_PARAMETERS_PROPERTY, oldValue, this.connectionParameters);
	}
	
	public ConnectionParameters getConnectionParameters() {
		return connectionParameters;
	}
	
	public static void main(String[] args) {
		RunnerHelper.runComponent(new ArrayIndexCustomizer(), 500, 420);
	}

	
	/**
	 * Returns the spinner.
	 * @return Returns the spinner.
	 */
	private JSpinner getSpinner() {
		if (spinner==null) {
			spinner= new JSpinner();
			spinner.setMinimumSize(new Dimension(50,21));
			spinner.setPreferredSize(new Dimension(50,21));
			spinner.setModel(spinnerModel= new SpinnerNumberModel());
			spinnerModel.setStepSize(1);
			spinnerModel.setMinimum(0);
			spinnerModel.setMaximum(100);
			spinnerModel.addChangeListener(new ChangeListener() {
			
				public void stateChanged(ChangeEvent e) {
					getSlider().setValue(spinnerModel.getNumber().intValue());
				}
			
			});
		}
		return spinner;
	}

}
