package de.desy.acop.displayers.selector;

import java.awt.Component;

import javax.swing.DefaultListCellRenderer;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.ListCellRenderer;

import com.cosylab.gui.adapters.Converter;

import de.desy.acop.chart.AcopDisplayMode;
import de.desy.acop.displayers.tools.AcopGraphParameters;

/**
 * ConnectionCustomizer for multiple displayers that handle AcopGraphParameters (eg. AcopChart).
 * 
 * @author Jaka Bobnar, Cosylab
 *
 */
public class ChartConnectionCustomizer extends MultipleConnectionCustomizer<AcopGraphParameters>{
	
	private static final long serialVersionUID = 8873830017332674294L;
	private GraphCustomizerPanel graphCustomizerPanel;
	private SelectorGUI selector;
	
	protected class AcopGraphCellRenderer extends DefaultListCellRenderer {
		
		private static final long serialVersionUID = 7937185460891053942L;

		public AcopGraphCellRenderer() {
			super();
			setOpaque(true);
		}
		
		/*
		 * (non-Javadoc)
		 * @see javax.swing.DefaultListCellRenderer#getListCellRendererComponent(javax.swing.JList, java.lang.Object, int, boolean, boolean)
		 */
		@Override
		public Component getListCellRendererComponent(JList list, Object value, int index, boolean isSelected, boolean cellHasFocus) {
			Component c = super.getListCellRendererComponent(list, value, index, isSelected,
					cellHasFocus);
			
			if (value instanceof AcopGraphParameters) {
				c.setForeground(((AcopGraphParameters)value).getColor());
				((JLabel)c).setText(String.valueOf(((AcopGraphParameters)value).getConnectionParameters()) + (((AcopGraphParameters)value).getConverter() == null ? "" : "; Converter"));
			}
			return c;
		}
	}
	
	/*
	 * (non-Javadoc)
	 * @see de.desy.acop.displayers.selector.MultipleConnectionCustomizer#getListRenderer()
	 */
	@Override
	protected ListCellRenderer getListRenderer() {
		return new AcopGraphCellRenderer();
	}
	
	/*
	 * (non-Javadoc)
	 * @see de.desy.acop.displayers.selector.MultipleConnectionCustomizer#getSelectorGUI()
	 */
	@Override
	protected SelectorGUI getSelectorGUI() {
		if (selector == null) {
			selector = super.getSelectorGUI();
			selector.addSelectorGUIListener(new SelectorGUIListener() {
				public void selectionChanged(SelectorGUIEvent e) {
					if (e.isSelectionComplete() && SelectorUtilities.isChannel(selector.getConnectionParameters())) {
						getGraphCustomizerPanel().setGraphMode(AcopDisplayMode.SimpleHistogram.ordinal());
					} else {
						getGraphCustomizerPanel().setGraphMode(AcopDisplayMode.PolyLine.ordinal());
					}
				}
			});
		}
		
	    return super.getSelectorGUI();
    }
	
	/*
	 * (non-Javadoc)
	 * @see de.desy.acop.displayers.selector.MultipleConnectionCustomizer#generateParameters()
	 */
	@Override
	protected AcopGraphParameters generateParameters() {
		Converter c=null;
		try {
			c= getConverterPanel().getConverter();
			if (c!=null) {
				c= (Converter)c.clone();
			}
		} catch (CloneNotSupportedException e) {
			e.printStackTrace();
		}
		return new AcopGraphParameters(
				getSelectorGUI().getConnectionParameters(), 
				getGraphCustomizerPanel().getColor(),
				getGraphCustomizerPanel().getGraphFFT(),
				getGraphCustomizerPanel().getGraphStyle(),
				getGraphCustomizerPanel().getGraphMode(),
				getGraphCustomizerPanel().getGraphWidth(),
				getGraphCustomizerPanel().isTrend(),
				getGraphCustomizerPanel().getTrendLength(),
				c);
		
	}

	/*
	 * (non-Javadoc)
	 * @see de.desy.acop.displayers.selector.MultipleConnectionCustomizer#getSettingsPanel()
	 */
	@Override
	protected MultipleDisplayerAbstractSettingsPanel<AcopGraphParameters> getSettingsPanel() {
		return getGraphCustomizerPanel();
	}
	
	private GraphCustomizerPanel getGraphCustomizerPanel() {
		if (graphCustomizerPanel == null) {
			graphCustomizerPanel = new GraphCustomizerPanel();
		}
		return graphCustomizerPanel;
	}

	/*
	 * (non-Javadoc)
	 * @see de.desy.acop.displayers.selector.MultipleConnectionCustomizer#getDisplayerParameters()
	 */
	@Override
	public AcopGraphParameters[] getDisplayerParameters() {
		AcopGraphParameters[] p = new AcopGraphParameters[displayerParameters.size()];
    	return (AcopGraphParameters[]) displayerParameters.toArray(p);
	}
	
	public ChartConnectionCustomizer() {
		super();
		setConverterTabEnabled(true);
	}
}
