package de.desy.acop.displayers.selector;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.Beans;
import java.beans.Customizer;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.beans.PropertyVetoException;

import javax.swing.JButton;
import javax.swing.JPanel;

import com.cosylab.gui.components.util.RunnerHelper;
import com.cosylab.util.CommonException;

import de.desy.acop.displayers.tools.ConnectionParametersReceiver;
import de.desy.acop.displayers.tools.MultipleAcopDisplayer;
import de.desy.acop.transport.ConnectionParameters;

/**
 * <code>ConnectionCustomizer</code> enables device and property selection
 * through the custumizer of the displayer. The Customizer supports Drag and Drop
 * of the selected property.
 * 
 * @author Jaka Bobnar, Cosylab
 *
 */
public class ConnectionCustomizer extends JPanel implements Customizer {
	
	private static final long serialVersionUID = -4511139029213172995L;

	private Object bean;
	
	private JPanel commandPanel;
	private SelectorGUI selectorGUI;
	
	private JButton connectButton;
	private boolean connectButtonVisible = true;
	
	private ConnectionParameters connectionParameters;
	
	public ConnectionCustomizer() {
		super();
		initialize();
	}
	
	private void initialize() {
		
		this.setLayout(new GridBagLayout());
		this.add(getCommandPanel(), new GridBagConstraints(0,0,1,1,1,1,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(11,11,11,11),0,0));
	}
	
	private JButton getConnectButton() {
		if (connectButton == null) {
			connectButton = new JButton("Connect");
			connectButton.addActionListener(new ActionListener() {

				public void actionPerformed(ActionEvent e) {
					try {
						if (bean instanceof ConnectionParametersReceiver) {
							((ConnectionParametersReceiver)bean).setConnectionParameters(getSelectorGUI().getConnectionParameters());
						}
					} catch (CommonException e1) {
						e1.printStackTrace();
					} catch (PropertyVetoException e1) {
						e1.printStackTrace();
					}
                }
				
			});
			connectButton.setVisible(connectButtonVisible);
			connectButton.setEnabled(false);
		}
		return connectButton;
	}
	
	private SelectorGUI getSelectorGUI() {
		if (selectorGUI == null) {
			selectorGUI = new SelectorGUI();
			selectorGUI.addSelectorGUIListener(new SelectorGUIListener() {
				public void selectionChanged(SelectorGUIEvent e) {
					setButtonsEnabled(e.isSelectionComplete());
				}
			});
			selectorGUI.addPropertyChangeListener(ConnectionParametersReceiver.CONNECTION_PARAMETERS_PROPERTY, new PropertyChangeListener() {
				public void propertyChange(PropertyChangeEvent evt) {
				    if (!ConnectionParametersReceiver.CONNECTION_PARAMETERS_PROPERTY.equals(evt.getPropertyName())) return;
				    setConnectionParameters((ConnectionParameters) evt.getNewValue());
				}
			});
			if (bean != null)
				selectorGUI.setObject(bean);
		}
		return selectorGUI;
	}
	
	private JPanel getCommandPanel() {
		if (commandPanel == null) {
			commandPanel = new JPanel(new GridBagLayout());
			commandPanel.add(getSelectorGUI(), new GridBagConstraints(0,0,1,1,1,1,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(0,0,0,0),0,0));
			
			if (!Beans.isDesignTime()) {
				commandPanel.add(getConnectButton(), new GridBagConstraints(0,1,2,1,1,0,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(0,0,0,0),0,0));
			}			
		}
		return commandPanel;
	}
	
	
	/*
	 * (non-Javadoc)
	 * @see java.beans.Customizer#setObject(java.lang.Object)
	 */
	public void setObject(Object bean) {
		if (bean instanceof MultipleAcopDisplayer) {
			this.bean = bean;
		} else if (bean instanceof ConnectionParametersReceiver) {
			this.bean = bean;
			((ConnectionParametersReceiver)this.bean).addPropertyChangeListener(ConnectionParametersReceiver.CONNECTION_PARAMETERS_PROPERTY,
					new PropertyChangeListener(){
				public void propertyChange(PropertyChangeEvent evt) {
					setConnectionParameters(((ConnectionParametersReceiver)ConnectionCustomizer.this.bean).getConnectionParameters());				
				}
			});
		} else {
			throw new IllegalArgumentException("Only ConnectionParametersReceiver and" +
					" MultipleAcopDisplayer can use this Customizer.");
		}
	
		getSelectorGUI().setObject(bean);
//		setConnectionParameters(getSelectorGUI().getConnectionParameters());
    }

	/**
     * @return the parameters
     */
    public ConnectionParameters getConnectionParameters() {
    	return connectionParameters;
    }

	/**
     * @param parameters the parameters to set
     */
    public void setConnectionParameters(ConnectionParameters parameters) {
    	if (parameters != null && parameters.equals(connectionParameters)) return;
    	ConnectionParameters old = connectionParameters;
    	connectionParameters = parameters;
    	if (Beans.isDesignTime() && bean != null && bean instanceof ConnectionParametersReceiver && SelectorUtilities.isConnectionParametersValid(connectionParameters)) {
			try {
				((ConnectionParametersReceiver)bean).setConnectionParameters((ConnectionParameters) parameters);
            } catch (CommonException e) {
                 e.printStackTrace();
            } catch (PropertyVetoException e) {
                 e.printStackTrace();
            }
		}
    	getSelectorGUI().setConnectionParameters(parameters);
    	if (SelectorUtilities.isConnectionParametersValid(connectionParameters)) 
    		setButtonsEnabled(true);
    	firePropertyChange(ConnectionParametersReceiver.CONNECTION_PARAMETERS_PROPERTY, old, parameters);
    }
    
	/**
	 * Returns the connectButtonVisible.
	 * @return Returns the connectButtonVisible.
	 */
	public boolean isConnectButtonVisible() {
		return connectButtonVisible;
	}

	
	/**
	 * Sets new connectButtonVisible.
	 * @param connectButtonVisible The connectButtonVisible to set.
	 */
	public void setConnectButtonVisible(boolean connectButtonVisible) {
		this.connectButtonVisible = connectButtonVisible;
		if (connectButton!=null) {
			connectButton.setVisible(connectButtonVisible);
		}
	}
	
	private void setButtonsEnabled(boolean enabled) {
		getConnectButton().setEnabled(enabled);
	}
	
	public void setOnlyConnectionParametersMode(boolean onlyCP) {
		getSelectorGUI().setOnlyConnectionParametersMode(onlyCP);
	}
	
	public boolean getOnlyConnectionParametersMode() {
		return getSelectorGUI().getOnlyConnectionParametersMode();
	}
		
	public static void main(String[] args) {
		try {
			
			ConnectionCustomizer c= new ConnectionCustomizer();
			RunnerHelper.runComponent(c,300,300);
			
//			c.addPropertyChangeListener(new PropertyChangeListener() {
//			
//				public void propertyChange(PropertyChangeEvent evt) {
//					System.out.println("E: " + evt);
//				}
//			
//			});
			
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	

}
/* __oOo__ */