package de.desy.acop.displayers.selector;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.beans.Beans;
import java.beans.Customizer;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.beans.PropertyVetoException;
import java.util.ArrayList;
import java.util.Vector;

import javax.swing.DefaultListCellRenderer;
import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.ListCellRenderer;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import com.cosylab.gui.adapters.Converter;
import com.cosylab.gui.adapters.ConverterCustomizer;
import com.cosylab.gui.components.util.IconHelper;
import com.cosylab.util.CommonException;

import de.desy.acop.displayers.tools.AcopDisplayer;
import de.desy.acop.displayers.tools.AcopDisplayerParameters;
import de.desy.acop.displayers.tools.MultipleAcopDisplayer;


/**
 * 
 * @author Jaka Bobnar, Cosylab
 *
 */
public abstract class MultipleConnectionCustomizer<T extends AcopDisplayerParameters> extends JPanel implements Customizer {


	private static final long serialVersionUID = 4596748128425127898L;
	
	private MultipleAcopDisplayer<T> disp;
	
	private SelectorGUI selectorGUI;
	private JPanel selectorPanel;
	private JList selectionList;
	private DefaultListModel listModel;
	private JScrollPane selectionScroll;
	private JLabel selectedLabel;
	private JButton removeButton;
	private JTabbedPane tabPane;
	private ConverterCustomizer converterCustomizer;
	
	private JButton addButton;
	private JButton applyButton;
	
	private T selectedParameters;
		
	protected Vector<T> displayerParameters = new Vector<T>();

	public MultipleConnectionCustomizer() {
		super();
		initialize();
		
	}
	
	private void initialize() {
		this.setLayout(new GridBagLayout());
		this.add(getSelectorPanel(), new GridBagConstraints(0,0,1,1,1,1,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(11,11,11,11),0,0));
		
	}
	
	private JPanel getSelectorPanel() {
		if (selectorPanel == null) {
			selectorPanel = new JPanel(new GridBagLayout());
			selectedLabel = new JLabel("Selected connections: ");
			selectedLabel.setPreferredSize(new Dimension(200,21));
			selectedLabel.setMinimumSize(new Dimension(200,21));
			selectedLabel.setMaximumSize(new Dimension(200,21));
			selectorPanel.add(selectedLabel, new GridBagConstraints(0,0,1,1,1,0,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0,0,0,0), 0,0));		
			selectorPanel.add(getSelectionScroll(), new GridBagConstraints(0,1,1,1,1,1,GridBagConstraints.WEST, GridBagConstraints.BOTH, new Insets(0,0,0,0), 0,0));
			selectorPanel.add(getRemoveButton(), new GridBagConstraints(0,2,1,1,1,0,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(10,0,0,0), 0,0));
			selectorPanel.add(getCustomizerTabbedPane(), new GridBagConstraints(1,0,1,2,0,1,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(0,11,0,0),0,0));
			if (Beans.isDesignTime()) {
				JPanel p = new JPanel(new GridBagLayout());
				p.add(getAddButton(), new GridBagConstraints(0,0,1,1,0,0,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(0,0,0,5),0,0));
				p.add(getApplyButton() , new GridBagConstraints(1,0,1,1,0,0,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(00,5,0,0),0,0));
				selectorPanel.add(p, new GridBagConstraints(1,2,1,1,0,0,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(10,11,0,0),0,0));
			} else {
				selectorPanel.add(getAddButton(), new GridBagConstraints(1,2,1,1,0,0,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(10,0,0,0),0,0));
			}
		
		}
		return selectorPanel;
	}
	
	private JButton getApplyButton() {
		if (applyButton == null) {
			applyButton = new JButton("Apply");
			applyButton.setEnabled(false);
			applyButton.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					if (selectedParameters != null) {
						if (disp != null) {
		        			disp.removeDisplayerParameters(selectedParameters);
		        		} else {
	        				removeDisplayerParameters(selectedParameters);
		        		}
						T param = generateAndAddDisplayerParameters();
						getSelectionList().setSelectedValue(param, true);
		    		    getSettingsPanel().setParameters(param);
					}
				}
			});
		}
		return applyButton;
	}
	
	private JButton getRemoveButton() {
		if (removeButton == null) {
	        removeButton = new JButton("Remove", IconHelper.createIcon("icons/navigation/Minus16.gif"));
	        removeButton.addActionListener(new ActionListener() {
	        	public void actionPerformed(ActionEvent e) {
	        		Object[] selected = getSelectionList().getSelectedValues();
	        		if (disp != null) {
	        			for (Object p: selected) {
	        				disp.removeDisplayerParameters((T)p);
	        			}
	        		} else {
	        			for (Object p: selected) {
	        				removeDisplayerParameters((T)p);
	        			}
	        		}
	        	
	        	}
	        });
	        removeButton.setEnabled(false);
	        
        }

        return removeButton;
	}
	
	private JButton getAddButton() {
		if (addButton == null) {
			addButton = new JButton("Add", IconHelper.createIcon("icons/navigation/Plus16.gif"));
			addButton.addActionListener(new ActionListener() {
	        	public void actionPerformed(ActionEvent e) {
	        		T param = generateAndAddDisplayerParameters();
	        		getSettingsPanel().setParameters(param);
	        	}
	        });
			addButton.setEnabled(false);
	        
        }

        return addButton;
	}
	
	private JScrollPane getSelectionScroll() {
		if (selectionScroll == null) {
	        selectionScroll = new JScrollPane(getSelectionList());
        }
        return selectionScroll;
    }
	
	protected ListCellRenderer getListRenderer() {
		return new DefaultListCellRenderer();
	}
	
	protected JList getSelectionList() {
		if (selectionList == null) {
	        selectionList = new JList();
	        selectionList.setPreferredSize(new Dimension(200,200));
	        selectionList.setModel(getModel());
	        selectionList.setCellRenderer(getListRenderer());
	        selectionList.getSelectionModel().addListSelectionListener(new ListSelectionListener() {
	        	public void valueChanged(ListSelectionEvent e) {
	        		if (getSelectionList().getSelectedIndex() >= 0) {
	        			getRemoveButton().setEnabled(true);
	        		} else {
	        			getRemoveButton().setEnabled(false);
	        		}
	        		
	        		int sel = selectionList.getSelectedIndex();
	        		if (sel >= 0) {
	        			T parameters = (T) getModel().getElementAt(sel);
	        			setSelectedParameters(parameters);
	        			getSelectorGUI().setConnectionParameters(parameters.getConnectionParameters());
	        			getSettingsPanel().setParameters(parameters);
	        		} else {
	        			setSelectedParameters(null);
	        		}
	        	}
	        });
	        selectionList.addKeyListener(new KeyAdapter() {
	        	@Override
	        	public void keyPressed(KeyEvent e) {
	        		if (e.getKeyCode() == KeyEvent.VK_DELETE) {
	        			Object[] selected = selectionList.getSelectedValues();
	        			if (disp != null) {
		        			for (Object p: selected) {
		        				disp.removeDisplayerParameters((T)p);
		        			}
		        		} else {
		        			for (Object p: selected) {
		        				removeDisplayerParameters((T)p);
		        			}
		        		}
	        		}
	        	}
	        });
        }
		return selectionList;
    }
	
	private DefaultListModel getModel() {
		if (listModel == null) {
	        listModel = new DefaultListModel();
        }

        return listModel;
	}
	
	JPanel converterP;
	protected JTabbedPane getCustomizerTabbedPane() {
		if (tabPane == null) {
			tabPane = new JTabbedPane();
			JPanel selectorP = new JPanel(new GridBagLayout());
			selectorP.add(getSelectorGUI(), new GridBagConstraints(0,0,1,1,1,1,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(11,11,11,11),0,0));
			JPanel customizerP = new JPanel(new GridBagLayout());
			customizerP.add(getSettingsPanel(), new GridBagConstraints(0,0,1,1,1,1,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(11,11,11,11),0,0));
			converterP = new JPanel(new GridBagLayout());
			converterP.add(getConverterPanel(), new GridBagConstraints(0,0,1,1,1,1,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(11,11,11,11),0,0));
			tabPane.addTab("Connection", selectorP);
			tabPane.addTab("Style", customizerP);
			tabPane.addTab("Converter", converterP);
		}
		
		return tabPane;
	}
	
	protected ConverterCustomizer getConverterPanel() {
		if (converterCustomizer == null) {
			converterCustomizer = new ConverterCustomizer();
			converterCustomizer.setPreferredSize(new Dimension(200,200));
		}

		return converterCustomizer;
	}

	protected abstract MultipleDisplayerAbstractSettingsPanel<T> getSettingsPanel();
		
	protected SelectorGUI getSelectorGUI() {
		if (selectorGUI == null) {
			selectorGUI = new SelectorGUI();
			selectorGUI.addSelectorGUIListener(new SelectorGUIListener() {
				public void selectionChanged(SelectorGUIEvent e) {
					getAddButton().setEnabled(SelectorUtilities.isConnectionParametersValid(selectorGUI.getConnectionParameters()));
				}
			});
			selectorGUI.addPropertyChangeListener(AcopDisplayer.CONNECTION_PARAMETERS_PROPERTY,new PropertyChangeListener() {
				public void propertyChange(PropertyChangeEvent e) {
					getAddButton().setEnabled(SelectorUtilities.isConnectionParametersValid(selectorGUI.getConnectionParameters()));
				}
			});
	    }
	    return selectorGUI;
    }
	
	protected abstract T generateParameters();
	
	private T generateAndAddDisplayerParameters() {
		T param = generateParameters();
		addDisplayerParameters(param);
		return param;
	}
	
	private void addDisplayerParameters(T agp) {
		if (agp == null || displayerParameters.contains(agp)) return;
		T[] old = getDisplayerParameters();
		getModel().addElement(agp);
		displayerParameters.add(agp);
				
	    if (disp != null) {
		    try {
                disp.addDisplayerParameters(agp);
            } catch (CommonException e1) {
            	getSelectorGUI().handleException(e1);
            } catch (PropertyVetoException e1) {
            	getSelectorGUI().handleException(e1);
            }
	    }
		
		firePropertyChange(MultipleAcopDisplayer.DISPLAYER_PARAMETERS_PROPERTY,old,getDisplayerParameters());
	}
	
	private void removeDisplayerParameters(T agp) {
		if (agp == null || !displayerParameters.contains(agp)) return;
		T[] old = getDisplayerParameters();
		getModel().removeElement(agp);
		displayerParameters.remove(agp);
		
		firePropertyChange(MultipleAcopDisplayer.DISPLAYER_PARAMETERS_PROPERTY,old,getDisplayerParameters());
	}

	
	public void setObject(Object bean) {
		disp = (MultipleAcopDisplayer<T>)bean;
		getSelectorGUI().setObject(bean);
		setDisplayerParameters(disp.getDisplayerParameters());
		disp.addPropertyChangeListener(MultipleAcopDisplayer.DISPLAYER_PARAMETERS_PROPERTY, new PropertyChangeListener(){

			public void propertyChange(PropertyChangeEvent evt) {
				if (!MultipleAcopDisplayer.DISPLAYER_PARAMETERS_PROPERTY.equals(evt.getPropertyName())) 
					return;
				setDisplayerParameters(disp.getDisplayerParameters());
			}
		});     
    }

	/**
	 * Returns an array of displayer parameters containing all displayer parameters.
	 * (use displayerParameters vector and cast elements to T)
     * @return displayerParameters
     */
    public abstract T[] getDisplayerParameters();

	/**
     * @param cps the connectionParameters to set
     */
    public void setDisplayerParameters(T[] parameters) {
    	if (parameters == null) return;
		T[] old = getDisplayerParameters();
		ArrayList<T> toBeRemoved = new ArrayList<T>();
		
		if (displayerParameters != null) {
			for(T cp : displayerParameters) {
				toBeRemoved.add(cp);
			}
		}
		
		ArrayList<T> toBeAdded = new ArrayList<T>();
	
		for (T p : parameters) {
			if (toBeRemoved.contains(p)) {
				toBeRemoved.remove(p);
			} else {
				toBeAdded.add(p);
			}
		}
		
		for (T p : toBeRemoved) {
			getModel().removeElement(p);
			displayerParameters.remove(p);
		}
		
		for (T p : toBeAdded) {
			if (!displayerParameters.contains(p)) {
				getModel().addElement(p);
				displayerParameters.add(p);
			}
		}
		
		firePropertyChange(MultipleAcopDisplayer.DISPLAYER_PARAMETERS_PROPERTY,old,getDisplayerParameters());
		
    }
    
    protected void setSelectedParameters(T selected) {
    	this.selectedParameters = selected;
    	if (this.selectedParameters != null || SelectorUtilities.isConnectionParametersValid(getSelectorGUI().getConnectionParameters())) {
    		getApplyButton().setEnabled(true);
    		getAddButton().setEnabled(true);
    		if (selectedParameters != null && selectedParameters.getConverter()!=null) {
    			Converter c = selectedParameters.getConverter();
				try {
					c= (Converter)c.clone();
				} catch (CloneNotSupportedException e) {
					e.printStackTrace();
				}
    			getConverterPanel().setConverter(c);
    		} else {
    			getConverterPanel().setConverter(null);
    		}
    	} else {
    		getApplyButton().setEnabled(false);
    		getAddButton().setEnabled(false);
    		getConverterPanel().setConverter(null);
    	} 		
    }
    
    public void setConverterTabEnabled(boolean enabled){
    	for (int i = 0; i < tabPane.getComponentCount(); i++) if (tabPane != null && tabPane.getComponentAt(i).equals(converterP)){
    		tabPane.setEnabledAt(i, enabled);
    		break;
    	}
    }
    
}


/* __oOo__ */