package de.desy.acop.displayers.selector;

import java.awt.Component;

import javax.swing.DefaultListCellRenderer;
import javax.swing.JList;
import javax.swing.ListCellRenderer;

import com.cosylab.gui.adapters.Converter;

import de.desy.acop.displayers.table.AcopTableParameters;
import de.desy.acop.displayers.table.TableColumnType;
import de.desy.acop.transport.ConnectionParameters;


/**
 * ConnectionCustomizer for multiple displayers that handle AcopTableParameters (eg. AcopTable).
 * 
 * @author Jaka Bobnar, Cosylab
 *
 */
public class TableConnectionCustomizer extends MultipleConnectionCustomizer<AcopTableParameters> {


	private static final long serialVersionUID = 4596748128425127898L;
	private TableCustomizerPanel tableCustomizerPanel;
	private SelectorGUI selector;
	
	protected class AcopTableCellRenderer extends DefaultListCellRenderer {
		
		private static final long serialVersionUID = 7937185460891053942L;

		/*
		 * (non-Javadoc)
		 * @see javax.swing.DefaultListCellRenderer#getListCellRendererComponent(javax.swing.JList, java.lang.Object, int, boolean, boolean)
		 */
		@Override
		public Component getListCellRendererComponent(JList list, Object value, int index, boolean isSelected, boolean cellHasFocus) {
			if (value instanceof AcopTableParameters) {
				value = ((AcopTableParameters)value).getConnectionParameters() + "; " + ((AcopTableParameters)value).getColumnType() + (((AcopTableParameters)value).getConverter() != null ? "; Converter" : "");
			}
			
			return super.getListCellRendererComponent(list, value, index, isSelected,
					cellHasFocus);
		}
	}
	
	public TableConnectionCustomizer() {
		super();
		setConverterTabEnabled(true);
	}
	
	/*
	 * (non-Javadoc)
	 * @see de.desy.acop.displayers.selector.MultipleConnectionCustomizer#getListRenderer()
	 */
	@Override
	protected ListCellRenderer getListRenderer() {
		return new AcopTableCellRenderer();
	}
	
	/*
	 * (non-Javadoc)
	 * @see de.desy.acop.displayers.selector.MultipleConnectionCustomizer#generateParameters()
	 */
	@Override
	protected AcopTableParameters generateParameters() {
		Converter c=null;
		try {
			c= getConverterPanel().getConverter();
			if (c!=null) {
				c= (Converter)c.clone();
			}
		} catch (CloneNotSupportedException e) {
			e.printStackTrace();
		}
		return new AcopTableParameters(
				getSelectorGUI().getConnectionParameters(),
				getTableCustomizerPanel().getType(),
				c,
				getTableCustomizerPanel().getName()); 
	}
	
	/*
	 * (non-Javadoc)
	 * @see de.desy.acop.displayers.selector.MultipleConnectionCustomizer#getSettingsPanel()
	 */
	@Override
	protected MultipleDisplayerAbstractSettingsPanel<AcopTableParameters> getSettingsPanel() {
		return getTableCustomizerPanel();
	}
	
	private TableCustomizerPanel getTableCustomizerPanel() {
		if (tableCustomizerPanel == null) {
			tableCustomizerPanel = new TableCustomizerPanel();
		}
		return tableCustomizerPanel;
	}
	
	/*
	 * (non-Javadoc)
	 * @see de.desy.acop.displayers.selector.MultipleConnectionCustomizer#getSelectorGUI()
	 */
	@Override
	protected SelectorGUI getSelectorGUI() {
		if (selector == null) {
			selector = super.getSelectorGUI();
			selector.addSelectorGUIListener(new SelectorGUIListener() {
				public void selectionChanged(SelectorGUIEvent e) {
					ConnectionParameters parameters = selector.getConnectionParameters();
					if (parameters.getDeviceName().contains("*") ||
							parameters.getDeviceProperty().contains("*")) {
						getTableCustomizerPanel().setType(TableColumnType.STRING);
					}
					getTableCustomizerPanel().setName(parameters.getRemoteName());
				}
			});
		}
		
	    return super.getSelectorGUI();
    }
	
	/*
	 * (non-Javadoc)
	 * @see de.desy.acop.displayers.selector.MultipleConnectionCustomizer#getDisplayerParameters()
	 */
	@Override
	public AcopTableParameters[] getDisplayerParameters() {
		AcopTableParameters[] p = new AcopTableParameters[displayerParameters.size()];
    	return (AcopTableParameters[]) displayerParameters.toArray(p);
	}  
}


/* __oOo__ */