package de.desy.acop.displayers.selector;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;

import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

import de.desy.acop.displayers.table.AcopTableParameters;
import de.desy.acop.displayers.table.TableColumnType;

/**
 * 
 * @author Jaka Bobnar, Cosylab
 *
 */
public class TableCustomizerPanel extends MultipleDisplayerAbstractSettingsPanel<AcopTableParameters>{

	private static final long serialVersionUID = -4547732237651600499L;
	private JComboBox tableCustomizerCombo;
	private JLabel tableCustomizerLabel;
	
	private JTextField tableColumnNameField;
	private JLabel tableColumnNameLabel;
	
	private JPanel propertiesPanel;
	public TableCustomizerPanel() {
		super();
		initialize();
	}
	
	private void initialize() {
		setLayout(new GridBagLayout());
		add(getPropertiesPanel(), new GridBagConstraints(0,0,1,1,1,1,GridBagConstraints.NORTH, GridBagConstraints.NONE, new Insets(0,0,0,0),0,0));
	}
	
	private JPanel getPropertiesPanel() {
		if (propertiesPanel == null) {
			propertiesPanel = new JPanel(new GridBagLayout());
			tableCustomizerLabel = new JLabel("Type of data");
			tableCustomizerLabel.setPreferredSize(new Dimension(100,21));
			tableCustomizerLabel.setMinimumSize(new Dimension(100,21));
			tableCustomizerLabel.setMaximumSize(new Dimension(100,21));
			propertiesPanel.add(tableCustomizerLabel, new GridBagConstraints(0,0,1,1,1,0,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(1,1,1,1),0,0));
			propertiesPanel.add(getTableCustomizerCombo(), new GridBagConstraints(1,0,2,1,1,0,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(1,1,1,1),0,0));
			
			tableColumnNameLabel = new JLabel("Name of column");
			tableColumnNameLabel.setPreferredSize(new Dimension(100,21));
			tableColumnNameLabel.setMinimumSize(new Dimension(100,21));
			tableColumnNameLabel.setMaximumSize(new Dimension(100,21));
			propertiesPanel.add(tableColumnNameLabel, new GridBagConstraints(0,1,1,1,1,0,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(1,1,1,1),0,0));
			propertiesPanel.add(getTableColumnNameField(), new GridBagConstraints(1,1,2,1,1,0,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(1,1,1,1),0,0));
		
		}
		return propertiesPanel;
	}
	
	private JTextField getTableColumnNameField() {
		if (tableColumnNameField == null) {
			tableColumnNameField = new JTextField();
			tableColumnNameField.setPreferredSize(new Dimension(150,21));
			tableColumnNameField.setMinimumSize(new Dimension(150,21));
			tableColumnNameField.setMaximumSize(new Dimension(150,21));
		}
		return tableColumnNameField;
	}
	
	private JComboBox getTableCustomizerCombo() {
		if (tableCustomizerCombo == null) {
			tableCustomizerCombo = new JComboBox(TableColumnType.values());
			tableCustomizerCombo.setPreferredSize(new Dimension(150,21));
			tableCustomizerCombo.setMinimumSize(new Dimension(150,21));
			tableCustomizerCombo.setMaximumSize(new Dimension(150,21));
			tableCustomizerCombo.setSelectedItem(null);
		}
		return tableCustomizerCombo;
	}
	
	/**
	 * Returns the selected type.
	 * @return
	 */
	public TableColumnType getType() {
		if (getTableCustomizerCombo().getSelectedItem() != null) {
			return (TableColumnType) getTableCustomizerCombo().getSelectedItem();
		} else {
			return TableColumnType.VALUE;
		}
	}
	
	/**
	 * Sets the selected type. 
	 * @param type
	 */
	public void setType(TableColumnType type) {
		getTableCustomizerCombo().setSelectedItem(type);
	}
	
	/**
	 * Sets the name for the column.
	 * @param name 
	 */
	public void setName(String name) {
		if (name == null) {
			name = "";
		}
		getTableColumnNameField().setText(name);
		getTableColumnNameField().setToolTipText(name);
	}
	
	/**
	 * Returns the name for the column.
	 */
	public String getName() {
		return getTableColumnNameField().getText();
	}
		
	/*
	 * (non-Javadoc)
	 * @see de.desy.acop.displayers.selector.MultipleDisplayerAbstractSettingsPanel#setParameters(de.desy.acop.displayers.tools.AcopDisplayerParameters)
	 */
	@Override
	public void setParameters(AcopTableParameters parameters) {
		if (parameters == null) {
			getTableCustomizerCombo().setSelectedItem(null);
			setName("");
		} else {
			getTableCustomizerCombo().setSelectedItem(parameters.getColumnType());
			setName(parameters.getName());
		}
	}

	
}
