package de.desy.acop.displayers.selector;

import java.awt.Component;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.beans.PropertyEditorSupport;

import com.cosylab.gui.adapters.ConverterUtilities;

import de.desy.acop.displayers.table.AcopTableParameters;
import de.desy.acop.displayers.tools.MultipleAcopDisplayer;
import de.desy.acop.transport.ConnectionParameters;

/**
 * 
 * @author Jaka Bobnar, Cosylab
 *
 */
public class TableSelectorPropertyEditor extends PropertyEditorSupport {

	private TableConnectionCustomizer editor;

	/**
     * Creates new instance of SelectorPropertyEditor.
     */
	public TableSelectorPropertyEditor() {
		super();
	}

	/*
	 * (non-Javadoc)
	 * @see java.beans.PropertyEditorSupport#getCustomEditor()
	 */
	@Override
	public Component getCustomEditor() {
		if (editor == null) {
			editor = new TableConnectionCustomizer();
			editor.setDisplayerParameters((AcopTableParameters[]) getValue());
			editor.addPropertyChangeListener(MultipleAcopDisplayer.DISPLAYER_PARAMETERS_PROPERTY,
			        new PropertyChangeListener() {
				        public void propertyChange(PropertyChangeEvent evt) {
					        setValueSilently(editor.getDisplayerParameters());
				        }
			        });
		}
		return editor;
	}

	/* 
	 * (non-Javadoc)
	 * @see java.beans.PropertyEditorSupport#getJavaInitializationString()
	 */
	@Override
	public String getJavaInitializationString() {
		AcopTableParameters[] params = (AcopTableParameters[]) getValue();
		StringBuffer buffer = new StringBuffer();
		buffer.append("new de.desy.acop.displayers.table.AcopTableParameters[]{");
		ConnectionParameters cp; 
		if (params != null) {
			for (AcopTableParameters p : params) {
				cp = p.getConnectionParameters();
				buffer.append("new de.desy.acop.displayers.table.AcopTableParameters(");
				buffer.append("new de.desy.acop.transport.ConnectionParameters(\""
				        + cp.getRemoteName()
				        + "\",de.desy.acop.transport.AccessMode."
				        + cp.getAccessMode() + ", " + cp.getAccessRate() + "," + cp.getPropertySize()+ "),");
				buffer.append("de.desy.acop.displayers.table.TableDisplayType."+p.getColumnType() + 
						(p.getConverter() == null ? ", null" : ", " + 
						ConverterUtilities.getInitializationString(p.getConverter())) + "),");
			}
		}
		return buffer.toString() + "}";
	}

	/*
	 * (non-Javadoc)
	 * @see java.beans.PropertyEditorSupport#supportsCustomEditor()
	 */
	@Override
	public boolean supportsCustomEditor() {
		return true;
	}
	

	/*
	 * (non-Javadoc)
	 * @see java.beans.PropertyEditorSupport#getAsText()
	 */
	@Override
	public String getAsText() {
		AcopTableParameters[] params = (AcopTableParameters[]) getValue();	
	    if (params == null || params.length == 0) return "";
	    StringBuffer buffer = new StringBuffer();
	    for (AcopTableParameters cp : params) {
	    	buffer.append(cp.toString() + ";");
	    }
	    return buffer.toString();
	}

	
	private void setValueSilently(Object value) {
		super.setValue(value);
	}
	/*
     * (non-Javadoc)
     * 
     * @see java.beans.PropertyEditorSupport#setValue(java.lang.Object)
     */
	@Override
	public void setValue(Object value) {
		super.setValue(value);
		if (editor != null) {
			editor.setDisplayerParameters((AcopTableParameters[]) value);
		}
	}
}

/* __oOo__ */