/**
 * 
 */
package de.desy.acop.displayers.selector;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import com.cosylab.gui.components.util.RunnerHelper;

import de.desy.acop.displayers.chart.HistoryParameters;
import de.desy.acop.displayers.chart.TrendSelector;
import de.desy.acop.displayers.tools.AcopDisplayer;
import de.desy.acop.displayers.tools.AcopGraphHistoryParameters;
import de.desy.acop.displayers.tools.AcopGraphParameters;
import de.desy.acop.transport.ConnectionParameters;

/**
 * @author Tilen Kusterle, Cosylab
 *
 */
public class TrendChartConnectionCustomizer extends ChartConnectionCustomizer {
	
	public static void main(String[] args) {
		RunnerHelper.runComponent(new TrendChartConnectionCustomizer(), 800,400);
	}
	
	private static final long serialVersionUID = 1L;
	
	private TrendSelector trendSelector;
	private JPanel trendPanel;
	private JButton suggestButton;
	
	public TrendChartConnectionCustomizer() {
		super();
		JPanel p = new JPanel(new GridBagLayout());
		p.add(getTrendPanel(), new GridBagConstraints(0,0,1,1,1,1,GridBagConstraints.NORTH, GridBagConstraints.HORIZONTAL, new Insets(0,0,0,0),0,0));
		getCustomizerTabbedPane().add(p, "History", 0);
		
		getSelectorGUI().addSelectorGUIListener(new SelectorGUIListener() {

			public void selectionChanged(SelectorGUIEvent e) {
				setToTrendSelector(getSelectorGUI().getConnectionParameters(), null);
			}});
		getSelectorGUI().addPropertyChangeListener(AcopDisplayer.CONNECTION_PARAMETERS_PROPERTY,new PropertyChangeListener() {
			public void propertyChange(PropertyChangeEvent e) {
				setToTrendSelector(getSelectorGUI().getConnectionParameters(), null);
			}
		});
		getSelectionList().getSelectionModel().addListSelectionListener(new ListSelectionListener() {

			public void valueChanged(ListSelectionEvent e) {
				int sel = getSelectionList().getSelectedIndex();
        		if (sel >= 0) {
        			AcopGraphHistoryParameters parameters = (AcopGraphHistoryParameters) getSelectionList().getModel().getElementAt(sel);
        			HistoryParameters hp = parameters.getHistoryParameters();
        			getTrendSelector().setHistoryParameters(hp);
        			if (hp != null) getTrendSelector().setHistoryEnabled(true);
        			else getTrendSelector().setHistoryEnabled(false);
        		} else {
        			getTrendSelector().setHistoryParameters(null);
        			getTrendSelector().setHistoryEnabled(false);
        		}
				
			}});
	}
	
	private JPanel getTrendPanel() {
		if (trendPanel == null) {
			trendPanel = new JPanel(new GridBagLayout());
			trendPanel.add(getTrendSelector(), new GridBagConstraints(0,0,1,1,1,1,GridBagConstraints.CENTER,GridBagConstraints.BOTH,new Insets(1,1,1,1),0,0));
			trendPanel.add(getSuggestButton(), new GridBagConstraints(0,1,1,1,1,0,GridBagConstraints.CENTER,GridBagConstraints.NONE,new Insets(1,1,1,1),0,0));
		}

		return trendPanel;
	}
	
	private JButton getSuggestButton() {
		if (suggestButton == null) {
			suggestButton = new JButton("Suggest Default");
			suggestButton.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					int sel = getSelectionList().getSelectedIndex();
	        		if (sel >= 0) {
	        			AcopGraphHistoryParameters parameters = (AcopGraphHistoryParameters) getSelectionList().getModel().getElementAt(sel);
	        			HistoryParameters hp = parameters.getHistoryParameters();
	        			getTrendSelector().setHistoryParameters(hp);
	        			setToTrendSelector(parameters.getConnectionParameters(), hp);
	        		} else {
	        			setToTrendSelector(getSelectorGUI().getConnectionParameters(), null);
	        		}
				}});
		}

		return suggestButton;
	}
	
	private TrendSelector getTrendSelector() {
    	if (trendSelector == null) {
			trendSelector = new TrendSelector();
		}
		return trendSelector;
    }

	/* (non-Javadoc)
	 * @see de.desy.acop.displayers.selector.ChartConnectionCustomizer#getSettingsPanel()
	 */
	@Override
	protected MultipleDisplayerAbstractSettingsPanel<AcopGraphParameters> getSettingsPanel() {
		GraphCustomizerPanel gcp = (GraphCustomizerPanel) super.getSettingsPanel();
		gcp.toggleTrendControls(false);
		return gcp;
	}

	/* (non-Javadoc)
	 * @see de.desy.acop.displayers.selector.ChartConnectionCustomizer#generateParameters()
	 */
	@Override
	protected AcopGraphParameters generateParameters() {
		AcopGraphParameters agp = super.generateParameters();
		agp.deriveWith(getTrendSelector().getSelectedIndex());
		return new AcopGraphHistoryParameters(agp, getTrendSelector().getHistoryParameters());
	}
	
	private void setToTrendSelector(ConnectionParameters cp, HistoryParameters hp) {
		if (hp == null) {
			getTrendSelector().setHistoryParameters(
					cp.getDeviceContext(),
					cp.getDeviceGroup(),
					cp.getDeviceName(),
					cp.getDeviceProperty());
		}
		else {
			getTrendSelector().setHistoryParameters(hp);
		}
		if (SelectorUtilities.isConnectionParametersValid(cp)) {
			getTrendSelector().setIndexMax(SelectorUtilities.getSequenceLength(cp)-1);
		}
		if (hp != null) getTrendSelector().setSelectedIndex(hp.getHistoryIndex());
	}
	
}
