package de.desy.acop.displayers.table;

import com.cosylab.gui.components.table.cells.PatternCell;
import com.cosylab.gui.displayers.DataConsumer;
import com.cosylab.gui.displayers.DisplayerUtilities;
import com.cosylab.gui.displayers.DoubleSeqConsumer;
import com.cosylab.util.CommonException;

import de.desy.acop.displayers.AcopTable;

/**
 * Table column for presenting numerical values in binary form.
 * 
 * @author Jaka Bobnar, Cosylab
 *
 */
public class AcopBitTableColumn extends AcopTableColumn implements DoubleSeqConsumer {

	public static final Class[] SUPPORTED_CONSUMER_TYPES = {
		DoubleSeqConsumer.class
		};
	public static final String[] SUPPORTED_CHARACTERISTICS = {};
		
	protected double[] rows;
	
	public AcopBitTableColumn() {
		super();
	}
	
	public AcopBitTableColumn(AcopTable table, String uniqueName, String shortName) {
		super(table, uniqueName, shortName);
		rows = new double[0];
		cells = new PatternCell[0];
	}
	
	/*
	 * (non-Javadoc)
	 * @see de.desy.acop.displayers.tools.table.AcopTableColumn#getColumnClass()
	 */
	@Override
	public Class<?> getColumnClass() {
		return PatternCell.class; //String.class;
	}
	
	/*
	 * (non-Javadoc)
	 * @see de.desy.acop.displayers.tools.AcopNumericTableColumn#getValue(int)
	 */
	public Object getValue(int rowIndex) {
		if (rows == null || rowIndex >= rows.length || rowIndex < 0) return null;
		if (cells[rowIndex] == null) {
			cells[rowIndex] = new PatternCell(displayerParameters.getName()+"[" + rowIndex+"]", this, (long)rows[rowIndex], null);
			cells[rowIndex].putAllCharacteristics(characteristics);
		} else {
			cells[rowIndex].setValue((long)rows[rowIndex]);
		}
		setCharacteristicsOnCell(cells[rowIndex]);
		return cells[rowIndex];
	}	
	
	public void updateValue(long timestamp, double[] value) throws CommonException {
		rows = value;
		if (cells.length != value.length) {
			cells = new PatternCell[value.length];
		}
		fireColumnChanged();
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.displayers.DataConsumer#getSupportedCharacteristics()
	 */
	public String[] getSupportedCharacteristics() {
		return DisplayerUtilities.combineCharacteristics(DisplayerUtilities.COMMON_NUMERIC_DISPLAYER_CHARACTERISTICS,
			    SUPPORTED_CHARACTERISTICS);
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.displayers.DataConsumer#getSupportedConsumerTypes()
	 */
	public Class<DataConsumer>[] getSupportedConsumerTypes() {
		return SUPPORTED_CONSUMER_TYPES;
	}

}
