package de.desy.acop.displayers.table;

import com.cosylab.gui.components.table.cells.StringCell;
import com.cosylab.gui.components.table.cells.TableCell;
import com.cosylab.gui.displayers.DataConsumer;
import com.cosylab.gui.displayers.DataState;
import com.cosylab.gui.displayers.DisplayerUtilities;
import com.cosylab.gui.displayers.StringSeqConsumer;

import de.desy.acop.displayers.AcopTable;

/**
 * TableColumn for presenting string values. 
 * 
 * @author Jaka Bobnar, Cosylab
 *
 */
public class AcopStringTableColumn extends AcopTableColumn implements StringSeqConsumer {
	
	public static final Class[] SUPPORTED_CONSUMER_TYPES = {
		StringSeqConsumer.class
		};
	public static final String[] SUPPORTED_CHARACTERISTICS = {};
	
	private String[] rows;
	private DataState state = new DataState(DataState.NORMAL);
	
	public AcopStringTableColumn() {
		super();
	}
	
	public AcopStringTableColumn(AcopTable table, String uniqueName, String shortName) {
		super(table, uniqueName, shortName);
		rows = new String[0];
		cells= new StringCell[0];
	}
	
	/*
	 * (non-Javadoc)
	 * @see de.desy.acop.displayers.tools.table.AcopTableColumn#getColumnClass()
	 */
	public Class<?> getColumnClass() {
		return StringCell.class;
	}
	
	/* (non-Javadoc)
	 * @see com.cosylab.gui.displayers.DataConsumer#getSupportedCharacteristics()
	 */
	public String[] getSupportedCharacteristics() {
		return DisplayerUtilities.combineCharacteristics(DisplayerUtilities.COMMON_NUMERIC_DISPLAYER_CHARACTERISTICS,
			    SUPPORTED_CHARACTERISTICS);
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.displayers.DataConsumer#getSupportedConsumerTypes()
	 */
	public Class<DataConsumer>[] getSupportedConsumerTypes() {
		return SUPPORTED_CONSUMER_TYPES;
	}
	
	/*
	 * (non-Javadoc)
	 * @see de.desy.acop.displayers.tools.table.AcopTableColumn#getValue(int)
	 */	
	public Object getValue(int rowIndex) {
		if (rows == null || rowIndex >= rows.length || rowIndex < 0) return null;
		if (cells[rowIndex] == null) {
			cells[rowIndex] = new StringCell(displayerParameters.getName()+"[" + rowIndex+"]", this, rows[rowIndex], null);
			cells[rowIndex].putAllCharacteristics(characteristics);
			cells[rowIndex].putCharacteristic(TableCell.SUSPENDED, false);
			cells[rowIndex].putCharacteristic(TableCell.ALARM, false);
			cells[rowIndex].putCharacteristic(TableCell.WARNING, false);
			cells[rowIndex].putCharacteristic(TableCell.TIMEOUT, false);
			cells[rowIndex].putCharacteristic(TableCell.ERROR, false);
		} else {
			cells[rowIndex].setValue(rows[rowIndex]);
		}
		setCharacteristicsOnCell(cells[rowIndex]);
		return cells[rowIndex];
	}

	/*
	 * (non-Javadoc)
	 * @see com.cosylab.gui.displayers.StringSeqConsumer#updateValue(long, java.lang.String[])
	 */
	public void updateValue(long timestamp, String[] names) {
		rows = names;	
		if (cells.length != names.length) {
			cells = new StringCell[names.length];
		}
		fireColumnChanged();
	}
	
	/*
	 * (non-Javadoc)
	 * @see de.desy.acop.displayers.tools.table.AcopTableColumn#getDataState()
	 */
	public DataState getDataState() {
		return state;
	}
}
