package de.desy.acop.displayers.table;

import java.util.Vector;

import javax.swing.event.TableModelEvent;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.TableModel;

/**
 * 
 * @author Jaka Bobnar, Cosylab
 *
 */
public class AcopTableModel extends AbstractTableModel implements TableModel, TableColumnListener {
	
	private static final long serialVersionUID = 3387896763789247131L;

	private Vector<AcopTableColumn> columns = new Vector<AcopTableColumn>();
	
	private int rowCount = 10;
	
	public AcopTableModel() {
		super();
	}

	/*
	 * (non-Javadoc)
	 * @see javax.swing.table.TableModel#getColumnClass(int)
	 */
	public Class<?> getColumnClass(int columnIndex) {
		return columns.get(columnIndex).getColumnClass();
	}

	/*
	 * (non-Javadoc)
	 * @see javax.swing.table.TableModel#getColumnCount()
	 */
	public int getColumnCount() {
		return columns.size();
	}

	/*
	 * (non-Javadoc)
	 * @see javax.swing.table.TableModel#getColumnName(int)
	 */
	public String getColumnName(int columnIndex) {
		return columns.get(columnIndex).getHeaderName();
	}

	/*
	 * (non-Javadoc)
	 * @see javax.swing.table.TableModel#getRowCount()
	 */
	public int getRowCount() {
		return rowCount;
	}
	
	public void setRowCount(int rowCount) {
		this.rowCount = rowCount;
		for (AcopTableColumn c : columns) {
			c.setRowCount(rowCount);
		}
		fireTableStructureChanged();
	}

	/*
	 * (non-Javadoc)
	 * @see javax.swing.table.TableModel#getValueAt(int, int)
	 */
	public Object getValueAt(int rowIndex, int columnIndex) {
		if (columnIndex >= getColumnCount()) 
			throw new IllegalArgumentException("Column index should not exceed column count: " + columnIndex + " >= " + getColumnCount() + ".");
		
		return columns.get(columnIndex).getValue(rowIndex);
	}

	/*
	 * (non-Javadoc)
	 * @see javax.swing.table.TableModel#isCellEditable(int, int)
	 */
	public boolean isCellEditable(int rowIndex, int columnIndex) {
		return false;
	}

	/*
	 * (non-Javadoc)
	 * @see javax.swing.table.TableModel#setValueAt(java.lang.Object, int, int)
	 */
	public void setValueAt(Object aValue, int rowIndex, int columnIndex) {
		//read only
	}
	
	public void addColumn(AcopTableColumn column) {
		columns.add(column);
		column.addTableColumnListener(this);
		column.setRowCount(rowCount);
		fireTableStructureChanged();
	}
	
	public void removeColumn(AcopTableColumn column) {
		columns.remove(column);
		column.removeTableColumnListener(this);
		fireTableStructureChanged();
	}

	/*
	 * (non-Javadoc)
	 * @see de.desy.acop.displayers.tools.table.TableColumnListener#columnDataChanged(de.desy.acop.displayers.tools.table.TableColumnEvent)
	 */
	public void columnDataChanged(TableColumnEvent e) {
		AcopTableColumn column = e.getColumn();
		int columnIndex = -1;
		for (int i = 0; i < columns.size(); i++) {
			if (column.equals(columns.get(i))) {
				columnIndex = i;
			}
		}
		
		if (columnIndex != -1){
			TableModelEvent event = new TableModelEvent(this, 0, rowCount, columnIndex);
			fireTableChanged(event);
		}
		
	}	
}
