package de.desy.acop.displayers.tools;

import java.awt.Component;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.beans.PropertyVetoException;
import java.io.IOException;

import javax.swing.JComponent;

import com.cosylab.gui.displayers.DisplayerParameters;
import com.cosylab.gui.util.AbstractDisplayerParametersTransferHandler;

import de.desy.acop.transport.ConnectionParameters;

public abstract class AbstractMDPTransferHandler extends AbstractDisplayerParametersTransferHandler{
	
	private static final long serialVersionUID = 4277524441131938702L;

	public AbstractMDPTransferHandler(JComponent comp, boolean reciveEnabled, boolean exportEnabled, DataFlavor[] f, boolean resetFlavors) {
		super(comp, reciveEnabled, exportEnabled, f, resetFlavors);
	}
	
	/* (non-Javadoc)
	 * @see com.cosylab.gui.components.util.CosyTransferHandler#getTransferData(java.awt.datatransfer.DataFlavor)
	 */
	@Override
	public Object getTransferData(DataFlavor flavor)
			throws UnsupportedFlavorException, IOException {
		if (flavor instanceof AcopDisplayerParametersFlavor) {
			return (AcopDisplayerParameters[]) exportMultipleDisplayerParameters();
		} else if (flavor instanceof ConnectionParametersFlavor) {
			AcopDisplayerParameters[] p = (AcopDisplayerParameters[]) exportMultipleDisplayerParameters();
			if (p != null) {
				ConnectionParameters[] param = new ConnectionParameters[p.length];
				for (int i = 0; i < param.length; i++) {
					param[i] = p[i].getConnectionParameters();
				}
				return param;
			} else {
				return null;
			}
		} else if (flavor.isFlavorTextType()) {
			AcopDisplayerParameters[] p = exportMultipleDisplayerParameters();
			StringBuilder s= new StringBuilder(32*p.length);

			if (0<p.length) {
				s.append(p[0].getConnectionParameters().getRemoteName());
			}
			for (int i = 1; i < p.length; i++) {
				s.append("\n");
				s.append(p[i].getConnectionParameters().getRemoteName());
			}
			
			return s.toString();
		} 
		return super.getTransferData(flavor);
		
	}
	
	protected abstract AcopDisplayerParameters[] exportMultipleDisplayerParameters();
	
	protected AcopDisplayerParameters convertParameters(AcopDisplayerParameters adp) {
		return adp;
	}
	
	protected AcopDisplayerParameters convertParameters(ConnectionParameters adp) {
		return new AcopDisplayerParameters(adp);	
	}

	
	@Override
	protected boolean importString(String string) throws PropertyVetoException {
		ConnectionParameters cp = new ConnectionParameters(string);
		return importDisplayerParameters(new AcopDisplayerParameters[]{new AcopDisplayerParameters(cp)});
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.util.DisplayerParametersTransferHandler#importDisplayerParameters(com.cosylab.gui.displayers.DisplayerParameters[])
	 */
	@Override
	protected boolean importDisplayerParameters(DisplayerParameters[] dp) throws PropertyVetoException {
		try {
			if (!(dp instanceof AcopDisplayerParameters[])) {
				throw new UnsupportedOperationException("Cannot import displayer parameters of type '" + dp.getClass() + "'!");
			}
			AcopDisplayerParameters[] adp = (AcopDisplayerParameters[]) dp;
			
			for (int i = 0; i < adp.length; i++) {
				setParametersToDisplayer(displayer,adp[i]);
			}
			return true;
		} catch (Exception e) {
			e.printStackTrace();
		}
		return false;
		
	}
	
	/* (non-Javadoc)
	 * @see com.cosylab.gui.util.DisplayerParametersTransferHandler#importData(javax.swing.JComponent, java.awt.datatransfer.Transferable)
	 */
	@Override
	public boolean importData(JComponent comp, Transferable t) {
		try {
			if (t.isDataFlavorSupported(AcopDisplayerParametersFlavor.FLAVOR)) {
				Object drop = t.getTransferData(AcopDisplayerParametersFlavor.FLAVOR);

				AcopDisplayerParameters[] p = null;
				if (drop instanceof AcopDisplayerParameters) {
					p = new AcopDisplayerParameters[]{(AcopDisplayerParameters)drop};
				} else if (drop instanceof AcopDisplayerParameters[]) {
					p= (AcopDisplayerParameters[])drop;
				}
				
				if (p != null) {
					for (int i = 0; i < p.length; i++) {
						AcopDisplayerParameters a = convertParameters(p[i]);
						if (a!=null) {
							importDisplayerParameters(new AcopDisplayerParameters[]{a});
						}
					}
					return true;
				}
	
				return true;
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
		try {
			if (t.isDataFlavorSupported(ConnectionParametersFlavor.FLAVOR)) {
				Object drop = t.getTransferData(ConnectionParametersFlavor.FLAVOR); 

				ConnectionParameters[] cp=null;
				
				if (drop instanceof ConnectionParameters) {
					cp= new ConnectionParameters[]{(ConnectionParameters)drop};
				} else if (drop instanceof ConnectionParameters[]) {
					cp= (ConnectionParameters[])drop;
				}
				if (cp != null) {
					for (int i = 0; i < cp.length; i++) {
						AcopDisplayerParameters a = convertParameters(cp[i]);
						if (a!=null) {
							importDisplayerParameters(new AcopDisplayerParameters[]{a});
						}
					}
					return true;
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
		}

		return super.importData(comp,t);
	}

	protected DisplayerParameters exportDisplayerParameters() {
		DisplayerParameters[] d = exportMultipleDisplayerParameters();
//		return d[0];
		int i = selectParameters((Component)displayer,d);
		if (i<0) {
			return null;
		}
		return d[i];
	}
	
	/*
	 * (non-Javadoc)
	 * @see com.cosylab.gui.util.DisplayerParametersTransferHandler#castParameters(java.lang.Object[])
	 */
	@Override
	protected DisplayerParameters[] castParameters(Object... parameters) {
		if (parameters instanceof AcopDisplayerParameters[]) {
			return (AcopDisplayerParameters[])parameters.getClass().cast(parameters);
		} else if (parameters.length == 1) {
			if (parameters[0] instanceof AcopDisplayerParameters[]) {
				return (AcopDisplayerParameters[]) parameters[0];
			} else if (parameters[0] instanceof AcopDisplayerParameters) {
				return new AcopDisplayerParameters[] {(AcopDisplayerParameters)parameters[0]};
			}
		}
		return super.castParameters(parameters);
	}
	
}
