/**
 * 
 */
package de.desy.acop.displayers.tools;

import java.awt.Color;
import java.beans.Beans;
import java.util.Map;

import com.cosylab.gui.displayers.CommonDisplayer;
import com.cosylab.gui.displayers.DataConsumer;
import com.cosylab.gui.displayers.DataState;
import com.cosylab.gui.displayers.Displayer;
import com.cosylab.gui.displayers.DisplayerUtilities;
import com.cosylab.gui.displayers.DoubleSeqConsumer;
import com.cosylab.util.CommonException;

import de.desy.acop.chart.AcopGraphStyleEnum;
import de.desy.acop.displayers.AcopChart;

/**
 * @author ikriznar
 *
 */
public class AcopChartConsumer implements DoubleSeqConsumer {

	public static final Class[] SUPPORTED_CONSUMER_TYPES = {
		DoubleSeqConsumer.class
		};
	public static final String[] SUPPORTED_CHARACTERISTICS = {
		CommonDisplayer.C_GRAPH_MAX, CommonDisplayer.C_GRAPH_MIN
		};
	
	protected String name;
	protected AcopChart acop;

	protected double[] xData;
	protected double[] yData;
	protected int displayHandle = -1;
	protected int textHandle = -1;
	protected long lastTimestamp;
	protected Color color= Color.RED;
	protected String[] xLabels;
	protected Object disable;
	protected int arraySize;
	protected int maxNumber;
    
    private boolean isTrendChart = false;
    
    private double preferredYMax;
    private double preferredYMin;
    
    private AcopGraphParameters displayerParameters;
    	
	/**
	 * 
	 */
	public AcopChartConsumer(AcopChart acop, String name) {
		super();
		this.name=name;
		this.acop=acop;
		preferredYMax = acop.getYMax();
		preferredYMin = acop.getYMin();
		
		//color= new Color((int)(Math.random()*255),(int)(Math.random()*255),(int)(Math.random()*255));
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.displayers.DataConsumer#getDataConsumer(java.lang.Class)
	 */
	@SuppressWarnings("unchecked")
    public DataConsumer getDataConsumer(Class type) {
		if (type.isAssignableFrom(getClass())) {
			return this;
		}
		return null;
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.displayers.DataConsumer#getDefaultDataConsumer()
	 */
	public DataConsumer getDefaultDataConsumer() {
		return this;
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.displayers.DataConsumer#updateDataState(com.cosylab.gui.displayers.DataState)
	 */
	public void updateDataState(DataState state) {
		// TODO Auto-generated method stub

	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.displayers.DataConsumer#setCharacteristics(java.util.Map)
	 */
	public void setCharacteristics(Map characteristics) {
		
		if (acop == null) {
			return;
		}

		Object o1 = null;
		Object o2 = null;

		o1 = characteristics.get(CommonDisplayer.C_GRAPH_MAX);

		if (o1 == null) {
			o1 = characteristics.get(CommonDisplayer.C_MAXIMUM);
		}

		o2 = characteristics.get(CommonDisplayer.C_GRAPH_MIN);

		if (o2 == null) {
			o2 = characteristics.get(CommonDisplayer.C_MINIMUM);
		}

		if (o1 != null || o2 != null) {
			double max = o1 != null ? ((Number)o1).doubleValue()
				: acop.getYMax();
			double min = o2 != null ? ((Number)o2).doubleValue()
				: acop.getYMin();
			
			setPreferredYMax(max);
			setPreferredYMin(min);
			acop.updateYScale(this);
		}

		o1 = characteristics.get(CommonDisplayer.C_DISPLAY_NAME);
		
		String newName = (String) o1;
		if (newName != null && !newName.equals(acop.getName())) {
		    acop.setName(newName);
		    acop.setCaption(newName);
		}
		
		o1 = characteristics.get(Displayer.C_UNITS);
		if (o1 != null) {
			acop.setYAxisLabel((String)o1);
		}

		//avoid setting colors through characterstics to avoid different threads
		//setting the same property at the same time - consumer might end up with wrong color
		o1 = characteristics.get(Displayer.C_COLOR);

		if (o1 != null) {
			color = (Color) o1;
			acop.setForeground(color);
		}
			
		o1 = characteristics.get("xLabels");
		if (o1 != null && !isTrendChart) {
			setXLabels((String[])o1);
		}
		
		o1 = characteristics.get(CommonDisplayer.C_SEQUENCE_LENGTH);
		if (arraySize == 0 || arraySize == -1 && o1 != null) {
			setArraySize((Integer)o1);
		}
		
		if (!Beans.isDesignTime()) {
			acop.updateXScale(this);
		}
		acop.repaint();

	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.displayers.DataConsumer#getName()
	 */
	public String getName() {
		return name;
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.displayers.DataConsumer#getSupportedCharacteristics()
	 */
	public String[] getSupportedCharacteristics() {
		return DisplayerUtilities.combineCharacteristics(DisplayerUtilities.COMMON_NUMERIC_DISPLAYER_CHARACTERISTICS,
			    SUPPORTED_CHARACTERISTICS);
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.displayers.DataConsumer#getSupportedConsumerTypes()
	 */
	public Class<DataConsumer>[] getSupportedConsumerTypes() {
		return SUPPORTED_CONSUMER_TYPES;
	}
	
	private void updateChart() {
		if (acop==null) {
			return;
		}
		
		synchronized (acop) {

	        //System.out.println("disp id : " + displayHandle);
	//	    acop.clearScreen(-1,0); <- this will wipe out all display references
	        //Acop aChart = acop.getAcopChart();
	//      aChart.clearScreen(displayHandle,0); <- this removes the given reference (but Acop will reuse the handle, so we don't want to use this inside the AcopDataConsumer)
	//	    acop.draw(y,x,null,null,141,100);
	        if (displayHandle < 0)
	        { // display the initial data set ...
	          applyParameters();
		      displayHandle = acop.draw(yData,xData,disable,xLabels,arraySize,maxNumber);
	//        System.out.println("assigned display handle : " + displayHandle + "  color : " + color.toString());
	        }
	        else
	        {
	          if (!isTrendChart)
	          { // not a trend chart : just refresh the display ...
	            acop.refreshScreen(yData, displayHandle, arraySize, 0, xData);
	//          System.out.println("refresh display handle : " + displayHandle + "  color : " + color.toString());
	          }
	          else
	          { // append 1 value to the end ...  (ACOP should be in LIN_TIME or LOG_TIME graph mode)
	            double[] yValue = new double[1];
	            yValue[0] = yData[0];
	            double[] xValue = new double[1];
	            xValue[0] = lastTimestamp/1000.;
	            
//	            System.out.println("Append: ["+xValue[0]+", "+yValue[0]+"]");
	            acop.appendScreen(yData, displayHandle, xValue);            
	            
//	            double[][] data= dumpData();
//	            System.out.print("X: ");
//	            System.out.println(Arrays.toString(data[0]));
//	            System.out.print("Y: ");
//	            System.out.println(Arrays.toString(data[1]));
	          }
	        }
	//	    a.GetDataSize(d,null,-1);
	//	    acopLabel.setText(a.getStatus() + " " + a.GetDataSize(-1) + " elements " + a.getTimeStamp().toString());
		    //acop.setForeground(Color.BLUE);
	//	    a.draw(z,x,null,null,141,100);
		    //acop.draw(z);
		}
		
	}

	/**
	 * Returns all data from chart. First index is 0 for X data array and 1 for Y data array.
	 * @return all dat a from chart in X and Y array
	 */
	public double[][] dumpData() {
		
		double[][] array= new double[2][arraySize];
		
		acop.getDrawnData(array[0], array[1], arraySize, 0, displayHandle);
		
		return array;
	}
	
	public void destroy() {
		acop.clearScreen(displayHandle);
		acop=null;
	}
	
	/* (non-Javadoc)
	 * @see com.cosylab.gui.displayers.DoubleSeqConsumer#updateValue(long, double[])
	 */
	public void updateValue(long timestamp, double[] value) throws CommonException {
		lastTimestamp=timestamp;
		yData=value;
		updateChart();
	}

	/**
	 * @return Returns the xdata.
	 */
	public double[] getXData() {
		return xData;
	}

	/**
	 * @param ydata The xdata to set.
	 */
	public void setXData(double[] xdata) {
		this.xData = xdata;
	}

	/**
	 * @return Returns the yData.
	 */
	public double[] getYData() {
		return yData;
	}

	/**
	 * @param data The yData to set.
	 */
	public void setYData(double[] data) {
		yData = data;
	}

	/**
	 * @return Returns the acop.
	 */
	public AcopChart getAcop() {
		return acop;
	}

	/**
	 * @return Returns the lastTimestamp.
	 */
	public long getLastTimestamp() {
		return lastTimestamp;
	}

	/**
	 * @return Returns the color.
	 */
	public Color getColor() {
		return color;
	}

	/**
	 * @param color The color to set.
	 */
	public void setColor(Color color) {
		this.color = color;
	}

	/**
	 * @return Returns the xLabels.
	 */
	public String[] getXLabels() {
		return xLabels;
	}

	/**
	 * @param labels The xLabels to set.
	 */
	public void setXLabels(String[] labels) {
		xLabels = labels;
	}

	/**
	 * @return Returns the arraySize.
	 */
	public int getArraySize() {
		return arraySize;
	}

	/**
	 * @param arraySize The arraySize to set.
	 */
	public void setArraySize(int arraySize) {
		this.arraySize = arraySize;
	}

	/**
	 * @return Returns the disable.
	 */
	public Object getDisable() {
		return disable;
	}

	/**
	 * @param disable The disable to set.
	 */
	public void setDisable(Object disable) {
		this.disable = disable;
	}

	/**
	 * @return Returns the maxNumber.
	 */
	public int getMaxNumber() {
		return maxNumber;
	}

	/**
	 * @param maxNumber The maxNumber to set.
	 */
	public void setMaxNumber(int maxNumber) {
		this.maxNumber = maxNumber;
	}

	/**
     * @return the preferredYMax
     */
    public double getPreferredYMax() {
    	return preferredYMax;
    }

	/**
     * @param preferredYMax the preferredYMax to set
     */
    public void setPreferredYMax(double preferredYMax) {
    	this.preferredYMax = preferredYMax;
    }

	/**
     * @return the preferredYMin
     */
    public double getPreferredYMin() {
    	return preferredYMin;
    }

	/**
     * @param preferredYMin the preferredYMin to set
     */
    public void setPreferredYMin(double preferredYMin) {
    	this.preferredYMin = preferredYMin;
    }
    
    private void applyParameters() {
    	if (displayerParameters != null) {
	    	acop.setForeground(displayerParameters.getColor());
	    	acop.setDrawStyle(displayerParameters.getDrawStyle());
	    	acop.setFFT(displayerParameters.getFFT());
	    	acop.setDisplayMode(displayerParameters.getMode());
	    	acop.setDrawWidth(displayerParameters.getWidth());
	    	if (displayerParameters.isTrend()) {
	    		acop.setGraphStyle(AcopGraphStyleEnum.TimeLin.ordinal());
	    		double[] data = new double[displayerParameters.getTrendLength()];
	    		if (yData != null) {
		    		for (int i = 0; i < yData.length && i < data.length; i++) {
		    			data[i] = yData[i];
		    		}
	    		}
	    		yData = data;
	    		maxNumber = data.length;
	    	}
    	} else if (color != null) {
    		acop.setForeground(color);
    	}
    }

	public AcopGraphParameters getDisplayerParameters() {
		return displayerParameters;
	}

	public void setDisplayerParameters(AcopGraphParameters displayerParameters) {
		this.displayerParameters = displayerParameters;
		this.color = displayerParameters.getColor() != null ? displayerParameters.getColor() : this.color;
		setTrendChart(displayerParameters.isTrend());
		if (isTrendChart()) {
			setArraySize(displayerParameters.getTrendLength());
		} else {
			setArraySize(displayerParameters.getConnectionParameters().getPropertySize());
		}
	}

	public boolean isTrendChart() {
		return isTrendChart;
	}

	public void setTrendChart(boolean isTrendChart) {
		this.isTrendChart = isTrendChart;
	}

}
