package de.desy.acop.displayers.tools;

import java.util.Map;

import com.cosylab.gui.adapters.Converter;
import com.cosylab.gui.displayers.DisplayerParameters;
import com.cosylab.gui.displayers.DoubleDisplayer;

import de.desy.acop.transport.ConnectionParameters;

public class AcopDisplayerParameters extends DisplayerParameters {

	private static final long serialVersionUID = 2758958154017521990L;


	public AcopDisplayerParameters() {
		this(null, 0, Double.NaN, Double.NaN, null, null);
	}
	
	public AcopDisplayerParameters(ConnectionParameters parameters) {
		this(parameters, 0, Double.NaN, Double.NaN, null, null);
	}
	
	public AcopDisplayerParameters(ConnectionParameters parameters, int index) {
		this(parameters, index, Double.NaN, Double.NaN, null, null);
	}
	
	public AcopDisplayerParameters(ConnectionParameters parameters, int index, double min, double max, String units, String format) {
		this(parameters.getRemoteName(), 6, parameters, index, min, max, units, format);
	}
	
	protected AcopDisplayerParameters(String name, int size, ConnectionParameters parameters, int index, double min, double max, String units, String format) {
		super(parameters.getRemoteName(), size, min, max, units, format);
		
		values.put(AcopDisplayer.CONNECTION_PARAMETERS_PROPERTY,parameters);
		values.put(AcopDisplayer.ARRAY_INDEX_PROPERTY,index);
	}

	protected AcopDisplayerParameters(String name, int size, AcopDisplayer adisp, DoubleDisplayer ddisp) {
		super(name, size, ddisp);
		
		values.put(AcopDisplayer.CONNECTION_PARAMETERS_PROPERTY,adisp.getConnectionParameters());
		values.put(AcopDisplayer.ARRAY_INDEX_PROPERTY,adisp.getArrayIndex());
	}
	
	public AcopDisplayerParameters(AcopDisplayer adisp, DoubleDisplayer ddisp) {
		this(adisp.getConnectionParameters() != null ? adisp.getConnectionParameters().getRemoteName() : null, 6, adisp, ddisp);
	}

	public AcopDisplayerParameters(ConnectionParameters parameters, int index, Converter converter) {
		this(parameters, index, Double.NaN, Double.NaN, null, null, converter);
	}
	
	public AcopDisplayerParameters(ConnectionParameters parameters, int index, double min, double max, String units, String format, Converter converter) {
		this(parameters.getRemoteName(), 6, parameters, index, min, max, units, format, converter);
	}
	
	protected AcopDisplayerParameters(String name, int size, ConnectionParameters parameters, int index, double min, double max, String units, String format, Converter converter) {
		this(name, size, parameters, index, min, max, units, format);
		values.put(AcopDisplayer.CONVERTER_PROPERTY, converter);
	}
	
	public Converter getConverter() {
		return (Converter) values.get(AcopDisplayer.CONVERTER_PROPERTY);
	}
	
	
	
	/**
	 * Creates new instance of AcopDisplayerParameters.
	 * @param name
	 * @param names
	 * @param vaues
	 */
	protected AcopDisplayerParameters(String name, Map<String,Object> values) {
		super(name, values);
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#hashCode()
	 */
	@Override
	public int hashCode() {
		return toString().hashCode();
	}

	/* (non-Javadoc)
	 * @see java.lang.Comparable#compareTo(java.lang.Object)
	 */
	public int compareTo(Object o) {
		if (!(o instanceof AcopDisplayerParameters)) return -1;
	      return this.toString().compareTo(o.toString());
	}

	/**
	 * @return the connectionParameters
	 */
	public ConnectionParameters getConnectionParameters() {
		return (ConnectionParameters)values.get(AcopDisplayer.CONNECTION_PARAMETERS_PROPERTY);
	}

	/**
	 * @return the arrayIndex
	 */
	public int getArrayIndex() {
		return getInt(AcopDisplayer.ARRAY_INDEX_PROPERTY);
	}


	public AcopDisplayerParameters deriveWith(int arrayIndex) {
		AcopDisplayerParameters a=new AcopDisplayerParameters(name,values);
		values.put(AcopDisplayer.ARRAY_INDEX_PROPERTY,arrayIndex);
		return a;
	}

}
