package de.desy.acop.displayers.tools;

import java.awt.Component;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyVetoException;
import java.io.IOException;

import javax.swing.JComponent;

import com.cosylab.gui.displayers.DisplayerParameters;
import com.cosylab.gui.displayers.DoubleDisplayer;
import com.cosylab.gui.util.DefaultDisplayerParametersTransferHandler;
import com.cosylab.gui.util.DisplayerParametersFlavor;
import com.cosylab.introspection.BeanIntrospector;

import de.desy.acop.displayers.selector.ChannelSelector;
import de.desy.acop.displayers.selector.SelectorUtilities;
import de.desy.acop.transport.ConnectionParameters;

public class AcopDisplayerTransferHandler extends DefaultDisplayerParametersTransferHandler {

	private static final long serialVersionUID = -8218250806158424887L;

	public AcopDisplayerTransferHandler(AcopDisplayer comp) {
		super(comp, true, true, new DataFlavor[]{AcopDisplayerParametersFlavor.FLAVOR,
				DisplayerParametersFlavor.FLAVOR,ConnectionParametersFlavor.FLAVOR, 
				DataFlavor.stringFlavor}, true);
	}
	
	@Override
	protected DisplayerParameters exportDisplayerParameters() {
		AcopDisplayer disp = (AcopDisplayer)displayer; 
		ConnectionParameters parameters = disp.getConnectionParameters();
		int index = disp.getArrayIndex();
		if (disp instanceof DoubleDisplayer) {
			return new AcopDisplayerParameters(parameters, index, 
					((DoubleDisplayer)disp).getMinimum(),((DoubleDisplayer)disp).getMaximum(),
					((DoubleDisplayer)disp).getUnits(),((DoubleDisplayer)disp).getFormat(),disp.getConverter());
		}
		else return new AcopDisplayerParameters(parameters, index, disp.getConverter());
	}
	
	/* (non-Javadoc)
	 * @see com.cosylab.gui.components.util.CosyTransferHandler#getTransferData(java.awt.datatransfer.DataFlavor)
	 */
	@Override
	public Object getTransferData(DataFlavor flavor)
			throws UnsupportedFlavorException, IOException {
		if (flavor instanceof AcopDisplayerParametersFlavor) {
			return new AcopDisplayerParameters[]{
					(AcopDisplayerParameters) exportDisplayerParameters()};
		} else if (flavor instanceof ConnectionParametersFlavor) {
			return new ConnectionParameters[]{
					((AcopDisplayerParameters) exportDisplayerParameters()).getConnectionParameters()};
		} else if (flavor.isFlavorTextType()) {
			return ((AcopDisplayerParameters) exportDisplayerParameters()).getConnectionParameters().getRemoteName();
		} else {
			return super.getTransferData(flavor);
		}
		
//		throw new UnsupportedFlavorException(flavor);
	}
	
	@Override
	public boolean importData(JComponent comp, Transferable t) {
		try {
			if (t.isDataFlavorSupported(AcopDisplayerParametersFlavor.FLAVOR)) {
				Object[] drop = (Object[]) t.getTransferData(AcopDisplayerParametersFlavor.FLAVOR);
				DisplayerParameters[] dp = null;
				dp = castParameters(drop);
				if (dp!=null) {
					return importDisplayerParameters(dp);
				}
			} else if (t.isDataFlavorSupported(ConnectionParametersFlavor.FLAVOR)) {
				Object drop = (Object[]) t.getTransferData(ConnectionParametersFlavor.FLAVOR);
				
				if (drop != null) {
					ConnectionParameters[] cp = null;
					if (drop instanceof ConnectionParameters) {
						cp = new ConnectionParameters[]{(ConnectionParameters)drop};
					} else if (drop instanceof ConnectionParameters[]) {
						cp = (ConnectionParameters[])drop;
					}
					
					AcopDisplayerParameters[] param = new AcopDisplayerParameters[cp.length];
					for (int i = 0; i < param.length; i++) {
						param[i] = new AcopDisplayerParameters(cp[i]);
					}
					
					return importDisplayerParameters(param);
				}
			}
			return super.importData(comp, t);
		} catch (Exception e) {
			e.printStackTrace();
		}
		return false;
		
	}
	
	@Override
	protected boolean importString(String string) throws PropertyVetoException {
		if (string.indexOf('\n') > 0) {
			return false;
		}
		try {
			ConnectionParameters cp = new ConnectionParameters(string);
			return importDisplayerParameters(new AcopDisplayerParameters[]{new AcopDisplayerParameters(cp)});
		} catch (IllegalArgumentException e) {
			return false;
		}
		
	}
	
	@Override
	protected DisplayerParameters[] castParameters(Object... parameters) {
		if (parameters instanceof AcopDisplayerParameters[]) {
			return (AcopDisplayerParameters[])parameters.getClass().cast(parameters);
		} else if (parameters.length == 1) {
			if (parameters[0] instanceof AcopDisplayerParameters[]) {
				return (AcopDisplayerParameters[]) parameters[0];
			} else if (parameters[0] instanceof AcopDisplayerParameters) {
				return new AcopDisplayerParameters[] {(AcopDisplayerParameters)parameters[0]};
			}
		}
		return super.castParameters(parameters);
	}

	protected int selectChannelIndex(ConnectionParameters cp) {
		int length = SelectorUtilities.getSequenceLength(cp);
		if (length==1) {
			return 1;
		}
		return ChannelSelector.selectChannelDialog((Component) displayer, length, ((AcopDisplayer)displayer).getArrayIndex());
	}
	
	@Override
	public boolean setParametersToDisplayer(Object disp, DisplayerParameters parameters) throws PropertyVetoException {
		if (!(parameters instanceof AcopDisplayerParameters)) 
			super.setParametersToDisplayer(disp, parameters);
				
		String[] n= parameters.getNames();
		Object[] o= parameters.getValues();
		
		ConnectionParameters cp = null;
		int index = -1;
		for (int i = 0; i < n.length ; i++) {
			if (AcopDisplayer.CONNECTION_PARAMETERS_PROPERTY.equals(n[i])) {
				cp = (ConnectionParameters) o[i];
				if (SelectorUtilities.isChannel(cp)) {
					index = 0;
				} else {
					index = selectChannelIndex(cp);
				}
			}
		}
		try {
			BeanIntrospector.setPropertyValue(disp,AcopDisplayer.CONNECTION_PARAMETERS_PROPERTY,cp);
			BeanIntrospector.setPropertyValue(disp, "arrayIndex", index);
		} catch (Exception e) {
			throw new PropertyVetoException("Property '" +AcopDisplayer.CONNECTION_PARAMETERS_PROPERTY +"' not set: "+e.getMessage(),new PropertyChangeEvent(disp,AcopDisplayer.CONNECTION_PARAMETERS_PROPERTY,null,cp));
		}
		
		for (int i = 0; i < n.length; i++) {
			if (!AcopDisplayer.CONNECTION_PARAMETERS_PROPERTY.equals(n[i]) && o[i]!=null && (!(o[i] instanceof Double) || !((Double)o[i]).isNaN())) {
				try {
					BeanIntrospector.setPropertyValue(disp,n[i],o[i]);
				} catch(NoSuchMethodException e) {
					//do nothing
				} catch (Exception e) {
					throw new PropertyVetoException("Property '"+n[i]+"' not set: "+e.getMessage(),new PropertyChangeEvent(disp,n[i],null,o[i]));
				}
			}
		}
		return true;
	}
	
	public static void test(Object... object) {
		System.out.println(object.length);
	}
}
