package de.desy.acop.displayers.tools;

import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.io.IOException;

import javax.swing.JComponent;

import com.cosylab.gui.displayers.DisplayerParameters;
import com.cosylab.gui.util.DisplayerParametersFlavor;

import de.desy.acop.displayers.AcopTable;
import de.desy.acop.displayers.selector.AcopTableParametersDialog;
import de.desy.acop.displayers.table.AcopTableParameters;
import de.desy.acop.displayers.table.AcopTableParametersFlavor;
import de.desy.acop.transport.ConnectionParameters;

public class AcopTableTransferHandler extends AcopMultipleDisplayerParametersTransferHandler{
	
	private static final long serialVersionUID = 4277524441131938702L;
	private AcopTableParametersDialog tableDialog;

	public AcopTableTransferHandler(AcopTable comp, boolean reciveEnabled, boolean exportEnabled, DataFlavor[] f, boolean resetFlavors) {
		super(comp, reciveEnabled, exportEnabled, f, resetFlavors);
	}
	
	/* (non-Javadoc)
	 * @see com.cosylab.gui.components.util.CosyTransferHandler#getTransferData(java.awt.datatransfer.DataFlavor)
	 */
	@Override
	public Object getTransferData(DataFlavor flavor)
			throws UnsupportedFlavorException, IOException {
		if (flavor.equals(AcopTableParametersFlavor.FLAVOR)) {
			return exportDisplayerParameters();
		} else if (flavor instanceof DisplayerParametersFlavor) {
			return exportDisplayerParameters();
		} else if (flavor instanceof ConnectionParametersFlavor) {
			return ((AcopTableParameters)exportDisplayerParameters()).getConnectionParameters();
		} else if (flavor.isFlavorTextType()){
			return ((AcopTableParameters)exportDisplayerParameters()).getConnectionParameters().toString();
		} else {
			return super.getTransferData(flavor);
		}
	}
	
	@Override
	protected DisplayerParameters exportDisplayerParameters() {
		return ((AcopTable)displayer).getSelectedParameters();
	}
	
	/*
	 * (non-Javadoc)
	 * @see de.desy.acop.displayers.tools.AcopMultipleDisplayerParametersTransferHandler#convertParameters(de.desy.acop.displayers.tools.AcopDisplayerParameters)
	 */
	protected AcopTableParameters convertParameters(AcopDisplayerParameters adp) {
		if (tableDialog == null) {
			tableDialog = new AcopTableParametersDialog(displayer);
		}
		return tableDialog.showDialog(displayer,adp);	
	}
	
	/*
	 * (non-Javadoc)
	 * @see de.desy.acop.displayers.tools.AcopMultipleDisplayerParametersTransferHandler#convertParameters(de.desy.acop.transport.ConnectionParameters)
	 */
	protected AcopTableParameters convertParameters(ConnectionParameters adp) {
		if (tableDialog == null) {
			tableDialog = new AcopTableParametersDialog(displayer);
		}
		return tableDialog.showDialog(displayer,adp);	
	}
	
	/*
	 * (non-Javadoc)
	 * @see com.cosylab.gui.util.DisplayerParametersTransferHandler#castParameters(java.lang.Object[])
	 */
	@Override
	protected DisplayerParameters[] castParameters(Object... parameters) {
		if (parameters instanceof AcopTableParameters[]) {
			return (AcopTableParameters[])parameters.getClass().cast(parameters);
		} else if (parameters instanceof AcopDisplayerParameters[]) {
			return (AcopDisplayerParameters[])parameters.getClass().cast(parameters);
		} else if (parameters.length == 1) {
			if (parameters[0] instanceof AcopTableParameters) {
				return new AcopTableParameters[] {(AcopTableParameters)parameters[0]};
			} else if (parameters[0] instanceof AcopDisplayerParameters) {
				return new AcopDisplayerParameters[] {(AcopDisplayerParameters)parameters[0]};
			}
		} 
		return super.castParameters(parameters);
	}
	
	/* (non-Javadoc)
	 * @see com.cosylab.gui.util.DisplayerParametersTransferHandler#importData(javax.swing.JComponent, java.awt.datatransfer.Transferable)
	 */
	@Override
	public boolean importData(JComponent comp, Transferable t) {
		try {
			if (t.isDataFlavorSupported(AcopTableParametersFlavor.FLAVOR)) {
				Object drop = t.getTransferData(AcopTableParametersFlavor.FLAVOR);
				
				if (!(drop instanceof AcopTableParameters[])) {
					return false;
				}
				
				AcopTableParameters[] p = (AcopTableParameters[])drop;
				for (int i = 0; i < p.length; i++) {
					setParametersToDisplayer(displayer,p[i]);
				}
				return true;
			} 			
		} catch (Exception e) {
			e.printStackTrace();
		}
		
		if (super.importData(comp,t)) {
			return true;
		}		

		return false;
	}
	
}
