package de.desy.acop.displayers.tools;

import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.beans.PropertyVetoException;
import java.io.IOException;

import javax.swing.JComponent;

import com.cosylab.gui.displayers.DisplayerParameters;

import de.desy.acop.displayers.selector.AcopGraphParametersDialog;
import de.desy.acop.transport.ConnectionParameters;

public abstract class GraphParametersTransferHandler extends AbstractMDPTransferHandler{
	
	private static final long serialVersionUID = 4277524441131938702L;
	private AcopGraphParametersDialog graphDialog;

	public GraphParametersTransferHandler(JComponent comp, boolean reciveEnabled, boolean exportEnabled, DataFlavor[] f, boolean resetFlavors) {
		super(comp, reciveEnabled, exportEnabled, f, resetFlavors);
	}
	
	@Override
	protected boolean importString(String string) throws PropertyVetoException {
		ConnectionParameters cp = new ConnectionParameters(string);
		return importDisplayerParameters(new AcopGraphParameters[]{new AcopGraphParameters(cp)});
	}
	
	/* (non-Javadoc)
	 * @see com.cosylab.gui.components.util.CosyTransferHandler#getTransferData(java.awt.datatransfer.DataFlavor)
	 */
	@Override
	public Object getTransferData(DataFlavor flavor)
			throws UnsupportedFlavorException, IOException {
		if (flavor instanceof AcopGraphParametersFlavor){
			DisplayerParameters[] p = exportMultipleDisplayerParameters();
			return p;
		} else {
			return super.getTransferData(flavor);
		}
	}
	
	/*
	 * (non-Javadoc)
	 * @see de.desy.acop.displayers.tools.AcopMultipleDisplayerParametersTransferHandler#convertParameters(de.desy.acop.displayers.tools.AcopDisplayerParameters)
	 */
	@Override
	protected AcopDisplayerParameters convertParameters(AcopDisplayerParameters adp) {
		if (graphDialog == null) {
			graphDialog = new AcopGraphParametersDialog(displayer);
		}
		AcopGraphParameters p = graphDialog.showDialog(displayer,adp);
		
		return p;
	}
	
	/*
	 * (non-Javadoc)
	 * @see de.desy.acop.displayers.tools.AcopMultipleDisplayerParametersTransferHandler#convertParameters(de.desy.acop.transport.ConnectionParameters)
	 */
	@Override
	protected AcopDisplayerParameters convertParameters(ConnectionParameters adp) {
		if (graphDialog == null) {
			graphDialog = new AcopGraphParametersDialog(displayer);
		}
		return graphDialog.showDialog(displayer,adp);	
	}
	
	/**
	 * Override to provide conversion from AcopGraphParameters to higher level 
	 * parameters.
	 * @param agp
	 * @return
	 */
	protected AcopDisplayerParameters convertParameters(AcopGraphParameters agp) {
		return agp;
	}
	
	/*
	 * (non-Javadoc)
	 * @see com.cosylab.gui.util.DisplayerParametersTransferHandler#castParameters(java.lang.Object[])
	 */
	@Override
	protected DisplayerParameters[] castParameters(Object... parameters) {
		if (parameters instanceof AcopGraphParameters[]) {
			return (AcopGraphParameters[])parameters.getClass().cast(parameters);
		} else if (parameters.length == 1 && parameters[0] instanceof AcopGraphParameters) {
			return new AcopGraphParameters[] {(AcopGraphParameters)parameters[0]};
		} 
		return super.castParameters(parameters);
	}
		
	/* (non-Javadoc)
	 * @see com.cosylab.gui.util.DisplayerParametersTransferHandler#importData(javax.swing.JComponent, java.awt.datatransfer.Transferable)
	 */
	@Override
	public boolean importData(JComponent comp, Transferable t) {
		try {
			if (t.isDataFlavorSupported(AcopGraphParametersFlavor.FLAVOR)) {
				Object drop = t.getTransferData(AcopGraphParametersFlavor.FLAVOR);
				
				if (!(drop instanceof AcopGraphParameters[])) {
					return false;
				}
				
				AcopGraphParameters[] p = (AcopGraphParameters[])drop;
				for (int i = 0; i < p.length; i++) {
					AcopDisplayerParameters a = convertParameters(p[i]);
					if (a!=null) {
						importDisplayerParameters(new AcopDisplayerParameters[]{a});
					}
				}
				return true;
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
		
		if (super.importData(comp,t)) {
			return true;
		}		
		return false;
	}
	
	protected abstract AcopDisplayerParameters[] exportMultipleDisplayerParameters();
	
}
