package de.desy.acop.displayers.tools;

import java.beans.PropertyVetoException;

import com.cosylab.gui.displayers.DataSource;
import com.cosylab.gui.displayers.MultipleDisplayer;
import com.cosylab.util.CommonException;

import de.desy.acop.transport.ConnectionParameters;

/**
 * <code>AcopMultipleDisplayer</code> should be implemented by those displayers that
 * connects to multiple data sources. The class should accept multiple AcopGraphParameters and
 * create DataSources for those parameters and connect the DataSources with GUI. DataSources should
 * be obtained via AdapterFactory and released in the manner prescribed by AdapterFactory. 
 * 
 * @author Jaka Bobnar, Cosylab
 *
 */
public interface MultipleAcopDisplayer<T extends AcopDisplayerParameters> extends MultipleDisplayer {
	
	public static final String DISPLAYER_PARAMETERS_PROPERTY = "displayerParameters";
	public ConnectionParameters[] getConnectionParameters();
	public void addConnectionParameters(ConnectionParameters parameters) throws CommonException, PropertyVetoException;
	public void setConnectionParameters(ConnectionParameters[] parameters) throws CommonException, PropertyVetoException;
	public void addDisplayerParameters(T parameters) throws CommonException, PropertyVetoException;
	public T[] getDisplayerParameters();
	public void setDisplayerParameters(T[] parameters) throws CommonException, PropertyVetoException;
	/* This method is required for backward compatibility. */
	public DataSource removeConnectionParameters(ConnectionParameters parameters);
	public DataSource removeDisplayerParameters(T parameters);
}


/* __oOo__ */