/**
 * 
 */
package de.desy.acop.displayers.tools;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JSpinner;
import javax.swing.SpinnerNumberModel;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import com.cosylab.gui.components.util.RunnerHelper;

/**
 * @author Tilen Kusterle, Cosylab
 *
 */
public class RecentTimespanSelector extends JPanel {
	
	private static final long serialVersionUID = 1L;
	
	public static final String TIME_SPAN = "timeSpan";
	
	private static final int SECS = 60;
	private static final int MINS = 60;
	private static final int DAY = 24;
	
	private int min = 1;
	private int hour = 1;
	private int day = 1;
	
	private SpinnerNumberModel minModel;
	private SpinnerNumberModel hourModel;
	private SpinnerNumberModel dayModel;
	
	private JSpinner minSpin;
	private JSpinner hourSpin;
	private JSpinner daySpin;
	
	private ButtonGroup group;
	
	private JRadioButton minRButton;
	private JRadioButton hourRButton;
	private JRadioButton dayRButton;
	
	private JRadioButton wasSelected;
	
	private JButton applyButton;
	private JButton cancelButton;
	
	private double timeSpan;
	
	public RecentTimespanSelector() {
		initialize();
	}
	
	private void initialize() {
		minRButton = new JRadioButton("Minutes");
		minRButton.setSelected(true);
		wasSelected = minRButton;
		minRButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if (minRButton.isSelected()) {
					performSelectedAction(minRButton);
				}
			}});
		hourRButton = new JRadioButton("Hours");
		hourRButton.setSelected(false);
		hourRButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if (hourRButton.isSelected()) {
					performSelectedAction(hourRButton);
				}
			}});
		dayRButton = new JRadioButton("Days");
		dayRButton.setSelected(false);
		dayRButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if (dayRButton.isSelected()) {
					performSelectedAction(dayRButton);
				}
			}});
		
		group = new ButtonGroup();
		group.add(minRButton);
		group.add(hourRButton);
		group.add(dayRButton);
		
		minModel = new SpinnerNumberModel(1,1,59,1);
		hourModel = new SpinnerNumberModel(1,1,23,1);
		dayModel = new SpinnerNumberModel(1,1,99,1);
		
		minSpin = new JSpinner(minModel);
		minSpin.setToolTipText("Set Recent Past in Minutes");
		hourSpin = new JSpinner(hourModel);
		hourSpin.setToolTipText("Set Recent Past in Hours");
		hourSpin.setEnabled(false);
		hourSpin.setValue(0);
		daySpin = new JSpinner(dayModel);
		daySpin.setToolTipText("Set Recent Past in Days");
		daySpin.setEnabled(false);
		daySpin.setValue(0);
		
		minSpin.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent evt) {
				if(min != minModel.getNumber().intValue()) {
					setButtonsEnabled(true);
				}
				else {
					setButtonsEnabled(false);
				}
	        }
		});
		
		hourSpin.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent evt) {
				if(hour != hourModel.getNumber().intValue()) {
					setButtonsEnabled(true);
				}
				else {
					setButtonsEnabled(false);
				}
	        }
		});
		
		daySpin.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent evt) {
				if(day != dayModel.getNumber().intValue()) {
					setButtonsEnabled(true);
				}
				else {
					setButtonsEnabled(false);
				}
	        }
		});
		
		setLayout(new GridBagLayout());
		JPanel spinPanel = new JPanel(new GridBagLayout());
		spinPanel.add(minRButton,new GridBagConstraints(0, 0, 1, 1, 0, 0, GridBagConstraints.WEST, 0, new Insets(4,4,4,4), 0, 0));
		spinPanel.add(hourRButton,new GridBagConstraints(0, 1, 1, 1, 0, 0, GridBagConstraints.WEST, 0, new Insets(4,4,4,4), 0, 0));
		spinPanel.add(dayRButton,new GridBagConstraints(0, 2, 1, 1, 0, 0, GridBagConstraints.WEST, 0, new Insets(4,4,4,4), 0, 0));
		spinPanel.add(minSpin,new GridBagConstraints(1, 0, 1, 1, 0, 0, GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(4,4,4,4), 0, 0));
		spinPanel.add(hourSpin,new GridBagConstraints(1, 1, 1, 1, 0, 0, GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(4,4,4,4), 0, 0));
		spinPanel.add(daySpin,new GridBagConstraints(1, 2, 1, 1, 0, 0, GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(4,4,0,4), 0, 0));
		
		add(spinPanel, new GridBagConstraints(0,0,1,1,1,1,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(0,0,0,0),0,0));
		
		JPanel buttonsPanRecent = new JPanel();
		buttonsPanRecent.setLayout(new GridLayout(1,2, 20, 4));
		buttonsPanRecent.add(getApplyButton());
		buttonsPanRecent.add(getCancelButton());
		
		add(buttonsPanRecent,new GridBagConstraints(0, 3, 2, 1, 0, 0, GridBagConstraints.SOUTH, GridBagConstraints.NONE, new Insets(4,4,11,4), 0, 0));
		
	}
	
	private JButton getApplyButton() {
		if (applyButton == null) {
			applyButton = new JButton("Apply");
			applyButton.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent event) {
					double oldValue = getTimeSpan();
					setTimeSpan(calculateRecentPast());
					setButtonsEnabled(false);
					if (minRButton.isSelected()) wasSelected = minRButton;
					else if (hourRButton.isSelected()) wasSelected = hourRButton;
					else wasSelected = dayRButton;
					firePropertyChange(TIME_SPAN, oldValue, getTimeSpan());
				}	
			});
			applyButton.setToolTipText("Apply Recent Past Selection");
		}	
		return applyButton;
	}
	
	private JButton getCancelButton() {
		if (cancelButton == null) {
			cancelButton = new JButton("Cancel");
			cancelButton.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent event) {
					wasSelected.setSelected(true);
					performSelectedAction(wasSelected);
				}	
			});
			cancelButton.setToolTipText("Cancel Recent Past Selection");
		}	
		return cancelButton;
	}
	
	public void setButtonsEnabled(boolean enable) {
	    getApplyButton().setEnabled(enable);
	    getCancelButton().setEnabled(enable);
	}
	
	private double calculateRecentPast() {
		
		if (minRButton.isSelected()) {
			min = minModel.getNumber().intValue();
			return min*SECS;
		}
		else if (hourRButton.isSelected()) {
			hour = hourModel.getNumber().intValue();
			return hour*MINS*SECS;
		}
		else {
			day = dayModel.getNumber().intValue();
			return day*DAY*MINS*SECS;
		}
	}
	
	/**
	 * Returns the selected time span.
	 * @return the timeSpan
	 */
	public double getTimeSpan() {
		return timeSpan;
	}
	
	/**
	 * Sets selected time span.
	 * @param timeSpan the timeSpan to set
	 */
	public void setTimeSpan(double timeSpan) {
		this.timeSpan = timeSpan;
	}
	
	private void performSelectedAction(JRadioButton button) {
		if (button.equals(minRButton)) {
			hourSpin.setEnabled(false);
			hourSpin.setValue(0);
			daySpin.setEnabled(false);
			daySpin.setValue(0);
			minSpin.setEnabled(true);
			minSpin.setValue(min);
			
		}
		else if (button.equals(hourRButton)) {
			minSpin.setEnabled(false);
			minSpin.setValue(0);
			daySpin.setEnabled(false);
			daySpin.setValue(0);
			hourSpin.setEnabled(true);
			hourSpin.setValue(hour);
		}
		else {
			minSpin.setEnabled(false);
			minSpin.setValue(0);
			hourSpin.setEnabled(false);
			hourSpin.setValue(0);
			daySpin.setEnabled(true);
			daySpin.setValue(day);
		}
		if (calculateRecentPast() == getTimeSpan()) {
			setButtonsEnabled(false);
		}
		else setButtonsEnabled(true);
	}

	/* (non-Javadoc)
	 * @see javax.swing.JComponent#setEnabled(boolean)
	 */
	@Override
	public void setEnabled(boolean enabled) {
		super.setEnabled(enabled);
		minRButton.setEnabled(enabled);
		hourRButton.setEnabled(enabled);
		dayRButton.setEnabled(enabled);
		if (enabled) {
			if (minRButton.isSelected()) {
				minSpin.setEnabled(enabled);
			}
			else if (hourRButton.isSelected()) {
				hourSpin.setEnabled(enabled);
			}
			else {
				daySpin.setEnabled(enabled);
			}
			if (calculateRecentPast() != getTimeSpan()) setButtonsEnabled(enabled);
		}
		else {
			minSpin.setEnabled(enabled);
			hourSpin.setEnabled(enabled);
			daySpin.setEnabled(enabled);
			setButtonsEnabled(enabled);
		}
		
	}
	
	public static void main(String[] args) {
		RecentTimespanSelector tss = new RecentTimespanSelector();
		tss.addPropertyChangeListener(TIME_SPAN, new PropertyChangeListener() {

			public void propertyChange(PropertyChangeEvent evt) {
				System.out.println(evt.getNewValue());
				
			}});
		RunnerHelper.runComponent(tss, 300, 400);
	}

}
