package de.desy.acop.transport.adapters;

import java.beans.PropertyVetoException;
import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.Map;

import com.cosylab.gui.adapters.DataSourceJoint;
import com.cosylab.gui.displayers.DataConsumer;
import com.cosylab.gui.displayers.DataSource;
import com.cosylab.gui.displayers.DataState;
import com.cosylab.gui.displayers.DoubleSeqConsumer;
import com.cosylab.util.CommonException;

/**
 * 
 * @author Jaka Bobnar, Cosylab
 *
 */
public class AcopDataSourceJoint extends AcopTransportDataSource implements DataSourceJoint<AcopTransportDataSource>{
	
	private static final long serialVersionUID = 1099113487965270874L;

	protected class DataSourceWrapper {
		
		protected AcopTransportDataSource dataSource;
		protected String identifier;
		protected double[] lastValue = null;
		public DataSourceWrapper(AcopTransportDataSource source, String identifier) throws PropertyVetoException {
			this.dataSource = source;
			this.identifier = identifier;
			source.addConsumer(new DoubleSeqConsumer() {

				public void updateValue(long timestamp, double[] value) throws CommonException {
					DataSourceWrapper.this.lastValue = new double[value.length];
					System.arraycopy(value, 0, lastValue, 0, value.length);	
					updateConsumers(timestamp);
					
				}

				@SuppressWarnings("unchecked")
				public DataConsumer getDataConsumer(Class type) {
					if (type.isAssignableFrom(DoubleSeqConsumer.class)) {
						return this;
					} 
					return null;
					
				}

				public DataConsumer getDefaultDataConsumer() {
					// TODO Auto-generated method stub
					return this;
				}

				public String getName() {
					// TODO Auto-generated method stub
					return null;
				}

				public String[] getSupportedCharacteristics() {
					// TODO Auto-generated method stub
					return null;
				}

				@SuppressWarnings("unchecked")
				public Class<DataConsumer>[] getSupportedConsumerTypes() {
					return new Class[]{DoubleSeqConsumer.class};
				}

				public void setCharacteristics(Map characteristics) {
					// TODO Auto-generated method stub
					
				}

				public void updateDataState(DataState state) {
					// TODO Auto-generated method stub
					
				}
				
			});
		}
		
	}
	
	protected ArrayList<DataSourceWrapper> dataSources;
	
	public AcopDataSourceJoint() {
		super();
		dataSources = new ArrayList<DataSourceWrapper>();
	}
	
	/*
	 * (non-Javadoc)
	 * @see com.cosylab.gui.adapters.DataSourceJoint#getDataSources()
	 */
	public Map<AcopTransportDataSource, String> getDataSources() {
		LinkedHashMap<AcopTransportDataSource, String> map = new LinkedHashMap<AcopTransportDataSource, String>();
		for (DataSourceWrapper w : dataSources) {
			map.put(w.dataSource,w.identifier);
		}
		
		return map;
	}

	/*
	 * (non-Javadoc)
	 * @see com.cosylab.gui.adapters.DataSourceJoint#removeDataSource(com.cosylab.gui.displayers.DataSource)
	 */
	public String removeDataSource(AcopTransportDataSource dataSource) {
		for (DataSourceWrapper w: dataSources) {
			if (w.dataSource.equals(dataSource)) {
				return w.identifier;
			}
		}
		return null;
	}
	
	/*
	 * (non-Javadoc)
	 * @see com.cosylab.gui.adapters.DataSourceJoint#addDataSource(com.cosylab.gui.displayers.DataSource, java.lang.String)
	 */
	public void addDataSource(AcopTransportDataSource dataSource, String operation) throws PropertyVetoException {
		addDataSource(dataSource, operation, dataSources.size());		
	}
	
	private boolean isDataSourceAccepted(AcopTransportDataSource dataSource) {
		Class[] supported = dataSource.getAcceptableConsumerTypes();
		Class[] existing = getAcceptableConsumerTypes();
		for (Class ex : existing) {
			boolean exist = false;
			for (Class sup : supported) {
				if (ex.equals(sup)) {
					exist = true;
					break;
				}
			}
			if (!exist) return false;
		}
		
		return true;
	}

	/*
	 * (non-Javadoc)
	 * @see com.cosylab.gui.adapters.DataSourceJoint#addDataSource(com.cosylab.gui.displayers.DataSource, java.lang.String, int)
	 */
	public void addDataSource(AcopTransportDataSource dataSource, String operation, int index) throws PropertyVetoException {
		if (index > dataSources.size()) 
			throw new IllegalArgumentException("Invalid index. It should be less or equal " +
					"to " + dataSources.size() + ".");
		if (!isDataSourceAccepted(dataSource))
			throw new IllegalArgumentException("Given DataSource does not support all " +
					"characteristics supported by this class.");
		
		if (index == 0 || operation == null || (operation.equals(FIRST_IN_A_ROW) && index != 0)) {
			operation = FIRST_IN_A_ROW;
			if (dataSources.size() > 0)
				dataSources.remove(0);
			dataSources.add(0, new DataSourceWrapper(dataSource, operation));
			return;
		}
		
		DataSourceWrapper wrapper = new DataSourceWrapper(dataSource, operation);
		dataSources.add(index, wrapper);
	}
	
	private void updateConsumers(long timestamp) {
		if (dataSources.get(0).lastValue == null) return;
		double[] d = new double[dataSources.get(0).lastValue.length];
		System.arraycopy(dataSources.get(0).lastValue, 0, d, 0, d.length);

		for (int i = 1; i < dataSources.size(); i++) {
			if (dataSources.get(i).lastValue == null) return;
			operate(d, dataSources.get(i).lastValue, dataSources.get(i).identifier);
		}
		
		DoubleSeqConsumer[] dsc= (DoubleSeqConsumer[])dcon.toArray();
    	for (int i = 0; i < dsc.length; i++) {
			try {
				//System.out.println(dsc[i]);
				dsc[i].updateValue(timestamp,d);
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
	}
	
	private void operate(double[] higherChain, double[] values, String operation) {
		if (higherChain.length != values.length) 
			throw new IllegalArgumentException("Array length does not match");
		if (operation.equals(ADD)) {
			for (int i = 0; i < higherChain.length; i++) {
				higherChain[i] += values[i];
			}
		} else if (operation.equals(SUBSTRACT)) {
			for (int i = 0; i < higherChain.length; i++) {
				higherChain[i] -= values[i];
			}
		} else if (operation.equals(MULTIPLY)) {
			for (int i = 0; i < higherChain.length; i++) {
				higherChain[i] *= values[i];
			}
		} else if (operation.equals(DIVIDE)) {
			for (int i = 0; i < higherChain.length; i++) {
				if (values[i] == 0)
					higherChain[i] = higherChain[i] > 0 ? Double.POSITIVE_INFINITY : Double.NEGATIVE_INFINITY;
				else if (Double.isInfinite(values[i]))
					higherChain[i] = 0;
				else if (Double.isNaN(values[i]))
					higherChain[i] = Double.NaN;
				else
					higherChain[i] /= values[i];				
			}
		} 
	}
	
	public void addConsumer(DataConsumer consumer) throws PropertyVetoException {
		super.addConsumer(consumer);
		//do not allow setting
		if (consumer instanceof DataSource) {
			((DataSource)consumer).removeConsumer(getReverseConsumer());
		}
		
	}	
}
