package de.desy.acop.transport.adapters;

import java.beans.Beans;
import java.beans.PropertyVetoException;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Map;

import com.cosylab.gui.displayers.CommonDisplayer;
import com.cosylab.gui.displayers.DataConsumer;
import com.cosylab.gui.displayers.DataSource;
import com.cosylab.gui.displayers.DataSourceSupport;
import com.cosylab.gui.displayers.DataState;
import com.cosylab.gui.displayers.Displayer;
import com.cosylab.gui.displayers.DoubleSeqConsumer;
import com.cosylab.gui.displayers.StringConsumer;
import com.cosylab.gui.displayers.StringSeqConsumer;
import com.cosylab.util.ObjectList;

import de.desy.acop.transport.AcopTransport;
import de.desy.acop.transport.AcopTransportEvent;
import de.desy.acop.transport.AcopTransportListener;
import de.desy.acop.transport.ConnectionFailed;
import de.desy.acop.transport.ConnectionParameters;
import de.desy.tine.definitions.TArrayType;
import de.desy.tine.definitions.TErrorList;
import de.desy.tine.definitions.TFormat;
import de.desy.tine.queryUtils.TQuery;
import de.desy.tine.types.USTRING;


/**
 * @author Igor Kriznar (igor.kriznarATcosylab.com)
 *
 */
public class AcopTransportDataSource extends DataSourceSupport implements AcopTransportListener {

	private static final long serialVersionUID = -6371576395329401933L;
	int useCount = 0;
	protected ObjectList dcon= new ObjectList(DoubleSeqConsumer.class);
	protected ObjectList scon = new ObjectList(StringConsumer.class);
	protected ObjectList sseqcon = new ObjectList(StringSeqConsumer.class);
	private AcopTransport transport;
	private int id=-1;
	protected double[] data;
	protected String[] objectData;
	private Object inData;
	private long timestamp;
	private Map<String,Object> characteristics;
	private ConnectionParameters param;
	private ReverseConsumer reverseConsumer;
	private TFormat dataFormat;
	private int errorId=-1;
	private DataState dataState = new DataState(DataState.UNDEFINED);

	/**
	 * Creates new instance of AcopTransportDataSource.
	 */
	public AcopTransportDataSource() {
		super(new Class[]{DoubleSeqConsumer.class});
		transport= new AcopTransport();
        transport.addAcopTransportListener(this);
	}
	
	/* (non-Javadoc)
	 * @see com.cosylab.gui.displayers.DataSourceSupport#addConsumer(com.cosylab.gui.displayers.DataConsumer)
	 */
	@Override
	public void addConsumer(DataConsumer consumer) throws PropertyVetoException {
		DataConsumer dc = consumer.getDefaultDataConsumer();
		
		if (dc instanceof StringConsumer) {
			scon.add(dc);
			super.addConsumer(consumer);
			
		} else if (dc instanceof StringSeqConsumer) {
			sseqcon.add(dc);
			super.addConsumer(consumer);
			
		} else {
			dc = consumer.getDataConsumer(DoubleSeqConsumer.class);
			
			if (dc==null) {
				return;
			}
			dcon.add(dc);
			super.addConsumer(consumer);
			
		}
		
		
		try {
			Map<String, Object> ch = null;
			
			if (!Beans.isDesignTime()) {
				ch= getCharacteristics();
			} else if (param != null){
				ch = new HashMap<String, Object>();
				ch.put(CommonDisplayer.C_DISPLAY_NAME,param.getRemoteName());	
			}
			if (ch!=null) {
				consumer.setCharacteristics(ch);
			}
		} catch (ConnectionFailed e) {
			e.printStackTrace();
		}
		
		if (consumer instanceof DataSource) {
			if (dataFormat.getValue() != TFormat.CF_USTRING) {
				((DataSource)consumer).addConsumer(getReverseConsumer());
			} else {
				Map<String, Object> ch = new HashMap<String, Object>();
				ch.put(Displayer.C_EDITABLE, false);
				consumer.setCharacteristics(ch);
			}
		}		
		
		consumer.updateDataState(dataState);
	}	
	
	public Map<String,Object> getCharacteristics() throws ConnectionFailed {
		if (characteristics == null && param!=null) {
			characteristics = DataSourceUtilities.getCharacteristics(param);		
		}
		return characteristics;
	}

	/**
	 * Returns the transport.
	 * @return Returns the transport.
	 */
	public AcopTransport getTransport() {
		return transport;
	}
	
	public ConnectionParameters getConnectionParameters() {
		return param;
	}

	public void connect(ConnectionParameters conn) throws ConnectionFailed {
		param=conn;
				
		transport.setConnectionParameters(conn);
		int size = conn.getPropertySize();
		if ((Integer)getCharacteristics().get(CommonDisplayer.C_SEQUENCE_LENGTH) > 1 &&
				!((TArrayType)getCharacteristics().get("arrayType")).isChannel()) {
			size = (Integer)getCharacteristics().get(CommonDisplayer.C_SEQUENCE_LENGTH);
		}
		if (size == ConnectionParameters.ACTUAL_PROPERTY_SIZE ||
				size == ConnectionParameters.AUTO_PROPERTY_SIZE) {
			if (!param.getDeviceName().contains("*")) {
				size= (Integer)getCharacteristics().get(CommonDisplayer.C_SEQUENCE_LENGTH);
			} else {
				String[] names = TQuery.getDeviceNames(conn.getDeviceContext(), conn.getDeviceGroup());
				size = names.length;
			}
		} 
		if (param.getDeviceProperty().contains("*")) {
			dataFormat = TFormat.valueOf(TFormat.CF_USTRING);
		} else {
			dataFormat = (TFormat)getCharacteristics().get("dataFormat");
		}
		
		inData= DataSourceUtilities.getDataObject(dataFormat,size);
					
		if (inData instanceof double[]) {
			data= (double[])inData;
		} else if (inData instanceof USTRING[]) {
			objectData= new String[size];
		} else {
			data= new double[size];
		}
		
		if (!Beans.isDesignTime()) {
			id= transport.attachLink(inData);
		}
		updateCharacteristics();
	}
	
	public void disconnect() {
		if (!Beans.isDesignTime()) {
			transport.closeLink(id);
			clear();
			if (getReverseConsumer() != null)
				getReverseConsumer().release();
	    	reverseConsumer=null;
		}
		clear();
	}
	
    public void dataEventReceived(AcopTransportEvent e)
    {
      int id = e.getLinkId();
      if (id != this.id) 
    	  return;
      int error = e.getLinkError();
      timestamp = DataSourceUtilities.toUTC(transport.getDataTimeStamp());

      if (error!=errorId) {
    	  errorId=error;
    	 
    	  if (error==0) {
    		  dataState= new DataState(DataState.NORMAL,timestamp,TErrorList.toString(error));
    	  } else {
    		  dataState= new DataState(DataState.ERROR,timestamp,TErrorList.toString(error));
    	  }
          updateDataState(dataState);
          
      }
      
      updateConsumers();   
    }
    
    private void updateConsumers() {
    	//System.out.println(">> "+Arrays.toString(data));
    	if (dataFormat.getValue() == TFormat.CF_USTRING) {
    		objectData = DataSourceUtilities.extractData(dataFormat,objectData.length,inData,objectData);
    		StringConsumer[] sc= (StringConsumer[])scon.toArray();
        	for (int i = 0; i < sc.length; i++) {
    			try {
    				sc[i].updateValue(timestamp,Arrays.toString(objectData));
    			} catch (Exception e) {
    				e.printStackTrace();
    			}
    		}
        	
        	StringSeqConsumer[] ssc= (StringSeqConsumer[])sseqcon.toArray();
        	for (int i = 0; i < ssc.length; i++) {
    			try {
    				ssc[i].updateValue(timestamp,objectData);
    			} catch (Exception e) {
    				e.printStackTrace();
    			}
    		}
    	} else {
    		data= DataSourceUtilities.extractData(dataFormat,data.length,inData,data);
    		DoubleSeqConsumer[] dsc= (DoubleSeqConsumer[])dcon.toArray();
        	for (int i = 0; i < dsc.length; i++) {
    			try {
    				dsc[i].updateValue(timestamp,data);
    			} catch (Exception e) {
    				e.printStackTrace();
    			}
    		}
        	StringConsumer[] sc= (StringConsumer[])scon.toArray();
        	for (int i = 0; i < sc.length; i++) {
    			try {
    				sc[i].updateValue(timestamp,Arrays.toString(data));
    			} catch (Exception e) {
    				e.printStackTrace();
    			}
    		}
        	
        	StringSeqConsumer[] ssc= (StringSeqConsumer[])sseqcon.toArray();
        	for (int i = 0; i < ssc.length; i++) {
    			try {
    				ssc[i].updateValue(timestamp,DataSourceUtilities.toStringData(data));
    			} catch (Exception e) {
    				e.printStackTrace();
    			}
    		}
    	}
    	
    }

    private void updateCharacteristics() {
		try {
			Map ch= getCharacteristics();
			DataConsumer[] dsc= (DataConsumer[])dcon.toArray();
			for (int i = 0; i < dsc.length; i++) {
				try {
					dsc[i].setCharacteristics(ch);
				} catch (Exception e) {
					e.printStackTrace();
				}
			}
		} catch (ConnectionFailed e) {
			e.printStackTrace();
		}
    }
    
    protected ReverseConsumer getReverseConsumer() {
    	if (reverseConsumer == null) {
    		if (data != null)
    			reverseConsumer = new ReverseConsumer(param,transport,data.length);
		}

		return reverseConsumer;
    }
    
    /* (non-Javadoc)
     * @see com.cosylab.gui.displayers.DataSourceSupport#clear()
     */
    @Override
    public void clear() {
    	DataConsumer[] cc= getConsumers();
    	for (DataConsumer c:cc) {
    		if (c instanceof DataSource) {
    			((DataSource)c).removeConsumer(getReverseConsumer());
    		}
    	}
    	super.clear();
    }

	
	/**
	 * Returns the format.
	 * @return Returns the format.
	 */
	public TFormat getDataFormat() {
		return dataFormat;
	}

	
	/**
	 * Sets new format.
	 * @param format The format to set.
	 */
	public void setDataFormat(TFormat format) {
		this.dataFormat = format;
	}
	
	public void updateDataState(DataState state) {
		DataConsumer[] delegates = getConsumers();
	
		for (int i = 0; i < delegates.length; i++) {
			try {
				delegates[i].updateDataState(state);
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
	}
	
	public void removeConsumer(DataConsumer consumer) {
		super.removeConsumer(consumer);
		dcon.remove(consumer);
		scon.remove(consumer);
		sseqcon.remove(consumer);
		if (consumer instanceof DataSource && dataFormat.getValue() != TFormat.CF_USTRING) {
			((DataSource)consumer).removeConsumer(getReverseConsumer());
		} 
		
	}
	
	public int getDataLength() {
		return data.length;
	}

}
