/*
 * Copyright (c) 2006 by Cosylab d.o.o.
 *
 * The full license specifying the redistribution, modification, usage and other
 * rights and obligations is included with the distribution of this project in
 * the file license.html. If the license is not included you may find a copy at
 * http://www.cosylab.com/legal/abeans_license.htm or may write to Cosylab, d.o.o.
 *
 * THIS SOFTWARE IS PROVIDED AS-IS WITHOUT WARRANTY OF ANY KIND, NOT EVEN THE
 * IMPLIED WARRANTY OF MERCHANTABILITY. THE AUTHOR OF THIS SOFTWARE, ASSUMES
 * _NO_ RESPONSIBILITY FOR ANY CONSEQUENCE RESULTING FROM THE USE, MODIFICATION,
 * OR REDISTRIBUTION OF THIS SOFTWARE.
 */

package de.desy.acop.transport.adapters;


import java.util.Map;

import de.desy.acop.transport.ConnectionFailed;
import de.desy.acop.transport.ConnectionParameters;


/**
 * 
 * @author Jaka Bobnar, Cosylab
 *
 */
public interface AdapterFactory
{
	/**
	 * Returns data source for parameters and increases use count for that data
	 * source for 1. If adapter does not already exists, it is created.
	 *
	 * @param param parameters for the data source
	 *
	 * @return data source 
	 */
	public AcopTransportDataSource createDataSource(ConnectionParameters param) throws ConnectionFailed;

	/**
	 * Returns adapter assotiated with the parameters, does not increase use
	 * count. If no data source is associated, then create is called.
	 *
	 * @param param parameters for the data source
	 *
	 * @return 
	 */
	public AcopTransportDataSource getDataSource(ConnectionParameters param) throws ConnectionFailed;

	/**
	 * Decreases use count on data source. If use count reaches 0, then destroys
	 * data source and releases all handles to the data source.
	 *
	 * @param ds data source to be destroyed
	 */
	public void releaseDataSource(AcopTransportDataSource ds);

	/**
	 * Decreases use count on data source associated with parameters. 
	 * If use count reaches 0, then destroys data source and releases all handles 
	 * to the data source.
	 * 
	 * @param param 
	 * @return released DataSource 
	 */
	public AcopTransportDataSource releaseDataSource(ConnectionParameters param);
	
	/**
	 * Destroys all data source and clears all cache information.
	 */
	public void clear();
	
	/**
	 * Method returns the characteristics for specified parameters. Characteristics should
	 * be read from cache (from DataSource) if DataSource associated with these parameters 
	 * exists. If not, characteristics should be obtained by other means. 
	 * @param param
	 * @return
	 * @throws ConnectionFailed
	 * 
	 */
	public Map<String, Object> getCharacteristics(ConnectionParameters param) throws ConnectionFailed;
}

/* __oOo__ */
