package de.desy.acop.transport.adapters;

import java.util.Arrays;
import java.util.HashMap;
import java.util.Map;

import com.cosylab.gui.displayers.CommonDisplayer;
import com.cosylab.gui.displayers.Displayer;

import de.desy.acop.transport.AccessMode;
import de.desy.acop.transport.ConnectionFailed;
import de.desy.acop.transport.ConnectionParameters;
import de.desy.tine.definitions.TAccess;
import de.desy.tine.definitions.TArrayType;
import de.desy.tine.definitions.TFormat;
import de.desy.tine.queryUtils.TQuery;
import de.desy.tine.queryUtils.XPropertyQuery;
import de.desy.tine.types.USTRING;

/**
 * 
 * @author Jaka Bobnar, Cosylab
 *
 */
public class DataSourceUtilities {
	
	public static Map<String, Object> getCharacteristics(ConnectionParameters param) throws ConnectionFailed {
		XPropertyQuery[] info=null;
		try {
	        info = TQuery.getDevicePropertyInformationX(param.getDeviceContext(),
	        		param.getDeviceGroup(), param.getDeviceName(), param.getDeviceProperty());
	        //is property a StockProperty
	        if (info==null) {
	        	info = TQuery.getStockPropertyInformationX(param.getDeviceContext(),
	            		param.getDeviceGroup(), param.getDeviceName(), param.getDeviceProperty());
	        } 
		} catch (RuntimeException e) {
			throw new ConnectionFailed(param,e);
		}
        
        if (info==null) {
        	throw new ConnectionFailed(param);
        }
        Map<String, Object> characteristics = new HashMap<String, Object>();
        characteristics.put(CommonDisplayer.C_DESCRIPTION, info[0].prpDescription);
        characteristics.put(CommonDisplayer.C_DISPLAY_NAME, param.getRemoteName());
        characteristics.put(CommonDisplayer.C_MAXIMUM, info[0].prpMaxValue);
        characteristics.put(CommonDisplayer.C_MINIMUM, info[0].prpMinValue);
        characteristics.put("dataFormat", TFormat.valueOf(info[0].prpFormat));
        characteristics.put(CommonDisplayer.C_FORMAT, getFormat(info[0].prpFormat));
        characteristics.put(CommonDisplayer.C_UNITS, info[0].prpUnits);
        characteristics.put(CommonDisplayer.C_SEQUENCE_LENGTH,info[0].prpSize);
        characteristics.put("redirection",info[0].prpRedirection);
        characteristics.put("tag",info[0].prpTag);
        characteristics.put("rows",(int)info[0].numRows);
        characteristics.put("rowSize",(int)info[0].rowSize);
        characteristics.put("access",TAccess.valueOf(info[0].prpAccess));
        characteristics.put(Displayer.C_EDITABLE,TAccess.valueOf(info[0].prpAccess).isWrite());
        TArrayType arrayType = TArrayType.valueOf(info[0].prpArrayType);
        
        if (arrayType.isChannel()) {
       		String[] names = TQuery.getDeviceNames(param.getDeviceContext(), param.getDeviceGroup(), param.getDeviceProperty());
       		if (names != null) {
	       		String[] newNames = new String[names.length];
	       		int j = -1;
	       		int startPoint = -1;
	       		for (int i = 0; i < 2*names.length; i++) {
	       			if (i%names.length == startPoint) break;
	       			if (names[i%names.length].equals(param.getDeviceName())) {
	       				j = 0;
	       				startPoint = i;
	       			}
	       			if (j >= 0) {
	       				newNames[j] = names[i%names.length];
	       				j++;
	       			}
	       		}
	       		characteristics.put("xLabels",newNames);
       		}
       	}
        //DOOCS sequence and names
        if (param.getDeviceName().equals("*")) {
        	String[] names = (String[])characteristics.get("xLabels"); 
        	if (names == null) {
        		names = TQuery.getDeviceNames(param.getDeviceContext(), param.getDeviceGroup(), param.getDeviceProperty());
        		characteristics.put("xLabels", names);
        		characteristics.put(CommonDisplayer.C_SEQUENCE_LENGTH, names.length);
        	}
        } 
        
       	//first fallback 
       	//TODO this does not work OK - if type is a spectrum the returned names are empty strings
//       	if (characteristics.get("xLabels") == null) {
//       		AcopTransport atr = new AcopTransport();
//       		atr.setConnectionParameters(param);
//       		USTRING[] data = new USTRING[(Integer)characteristics.get(CommonDisplayer.C_SEQUENCE_LENGTH)];
//       		characteristics.put("xLabels", atr.getXAxisChannelNames(data));
//       	}
       	//second fallback
       	if (characteristics.get("xLabels") == null) {
       		String[] data = new String[(Integer)characteristics.get(CommonDisplayer.C_SEQUENCE_LENGTH)];
			for (int i = 0; i < data.length; i++) {
				data[i] = String.valueOf(i);
			}
			characteristics.put("xLabels", data);
       	}
       	characteristics.put("arrayType",arrayType);
       	
        return characteristics;
	}
	
	public static String getDescription(ConnectionParameters param) throws ConnectionFailed {
		XPropertyQuery[] info = TQuery.getDevicePropertyInformationX(param.getDeviceContext(),
        		param.getDeviceGroup(), param.getDeviceName(), param.getDeviceProperty());
		//is property a StockProperty
        if (info==null) {
        	info = TQuery.getStockPropertyInformationX(param.getDeviceContext(),
            		param.getDeviceGroup(), param.getDeviceName(), param.getDeviceProperty());
        }
        if (info==null) {
        	throw new ConnectionFailed(param);
        }
        return info[0].prpDescription;
	}
	
    static long toUTC(double stamp) {
    	return (long)(stamp*1E3);
    }
    
    static Object getDataObject(TFormat format, int size) {
    	switch (format.getValue()) {
    		case TFormat.CF_DOUBLE :
    			return new double[size];
    		case TFormat.CF_SHORT :
    			return new short[size];
    		case TFormat.CF_BYTE :
    			return new byte[size];
    		case TFormat.CF_LONG:
    			return new long[size];
    		case TFormat.CF_FLOAT :
    			return new float[size];
    		case TFormat.CF_BIT :
    			return new boolean[size];
    		case TFormat.CF_SPECTRUM :
    			return new double[size];
    		case TFormat.CF_USTRING :
    			return new USTRING[size];
    		default:
    			return new double[size];
    	}
    }
    
    static double[] extractData(TFormat format, int size, Object inData, double[] data) {
    	switch (format.getValue()) {
    		case TFormat.CF_DOUBLE:
    			return (double[])inData;
    		case TFormat.CF_SHORT: {
    			short[] d= (short[])inData;
    			for (int i = 0; i < d.length; i++) {
    				data[i]=d[i];
				}
    			return data;
    		}
    		case TFormat.CF_BYTE: {
    			byte[] d= (byte[])inData;
    			for (int i = 0; i < d.length; i++) {
    				data[i]=d[i];
				}
    			return data;
    		}
    		case TFormat.CF_LONG: {
    			long[] d= (long[])inData;
    			for (int i = 0; i < d.length; i++) {
    				data[i]=d[i];
				}
    			return data;
    		}
    		case TFormat.CF_FLOAT: {
    			float[] d= (float[])inData;
    			for (int i = 0; i < d.length; i++) {
    				data[i]=d[i];
				}
    			return data;
    		}
    		case TFormat.CF_USTRING: {
    			USTRING[] d = (USTRING[])inData;
    			for (int i = 0; i < d.length; i++) {
    				data[i] = d[i].f1val;
    			}
    			return data;
    		}
    		default:
    			return (double[])inData;
    			
    	}
    }
    
    static String[] extractData(TFormat format, int size, Object inData, String[] data) {
    	switch (format.getValue()) {
	    	case TFormat.CF_USTRING: {
				USTRING[] d = (USTRING[])inData;
				for (int i = 0; i < d.length; i++) {
					data[i] = d[i].f1val + "," + d[i].f2val + "," + d[i].ival + "," + d[i].str.trim() + "," + d[i].tm;
				}
				return data;
			}
	    	default : {
	    		Object[] d = (Object[])inData;
	    		for (int i = 0; i < d.length; i++) {
					data[i] = String.valueOf(d[i]);
				}
				return data;
	    	}
	    		
    	}
    }
    
    static String[] toStringData(double[] data) {
    	String[] s = new String[data.length];
    	for (int i = 0; i < data.length; i++) {
    		s[i] = String.valueOf(data[i]);
    	}
    	return s;
    }
    
    private static String getFormat(short format) {
    	switch(format) {
	    	case TFormat.CF_DOUBLE : return "%3.2f";
	    	case TFormat.CF_FLOAT : return "%3.2f";
	    	case TFormat.CF_LONG : return "%d";
	    	case TFormat.CF_SHORT : return "%d";
	    	default : return null;
    	}    	
    }
    
    public static void main(String[] args) throws ConnectionFailed {
		ConnectionParameters cp = new ConnectionParameters("TINE","TTF2","BLM1","*","CH00*", AccessMode.POLL, 500);
		System.out.println(Arrays.toString(TQuery.getDeviceNames("TTF2", "BLM1","CH00*")));
		
		//    	ConnectionParameters cp = new ConnectionParameters("TINE","DORIS","DORISDATA","V2 Rlf","DoArcTemp", AccessMode.POLL, 1000);
//		AcopTransport tp = new AcopTransport();
		Map<String, Object> map = DataSourceUtilities.getCharacteristics(cp);
		System.out.println(Arrays.toString((String[])map.get("xLabels")));
//		tp.setConnectionParameters(cp);
//		final USTRING[] data = new USTRING[40];
//		tp.addAcopTransportListener(new AcopTransportListener(){
//			public void dataEventReceived(AcopTransportEvent event) {
////				System.out.println(event.getLinkError());
//				for (USTRING u : data) {
//					System.out.println(u.str.trim() + " " + u.f1val + " " + u.f2val);
//					
//				}
//			}
//		});
//		
//		tp.attachLink(data);
		
		
		
	}
	
}


/* __oOo__ */

