package de.desy.acop.transport.adapters;

/*
 * Copyright (c) 2004 by Cosylab d.o.o.
 *
 * The full license specifying the redistribution, modification, usage and other
 * rights and obligations is included with the distribution of this project in
 * the file license.html. If the license is not included you may find a copy at
 * http://www.cosylab.com/legal/abeans_license.htm or may write to Cosylab, d.o.o.
 *
 * THIS SOFTWARE IS PROVIDED AS-IS WITHOUT WARRANTY OF ANY KIND, NOT EVEN THE
 * IMPLIED WARRANTY OF MERCHANTABILITY. THE AUTHOR OF THIS SOFTWARE, ASSUMES
 * _NO_ RESPONSIBILITY FOR ANY CONSEQUENCE RESULTING FROM THE USE, MODIFICATION,
 * OR REDISTRIBUTION OF THIS SOFTWARE.
 */

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import de.desy.acop.transport.ConnectionFailed;
import de.desy.acop.transport.ConnectionParameters;


/**
 * 
 * @author Jaka Bobnar, Cosylab
 *
 */
public class DefaultAdapterFactory implements AdapterFactory
{
	private Map<ConnectionParameters, AcopTransportDataSource> adapters = new HashMap<ConnectionParameters, AcopTransportDataSource>();
	private boolean useCache = true;

	protected DefaultAdapterFactory()
	{
		super();
	}

	/* (non-Javadoc)
	 * @see com.cosylab.adapters.dal.AdapterFactory#createDataSource(abeans.datatypes.DynamicValueProperty)
	 */
	public synchronized AcopTransportDataSource createDataSource(
			ConnectionParameters param) throws ConnectionFailed
	{
		if (useCache) {
			AcopTransportDataSource ad = adapters.get(param);

			if (ad == null) {
				ad = createAdapter(param);
				adapters.put(param, ad);
			}

			ad.useCount++;

			return ad;
		}

		return createAdapter(param);
	}

	/* (non-Javadoc)
	 * @see com.cosylab.adapters.dal.AdapterFactory#getDataSource(abeans.datatypes.DynamicValueProperty)
	 */
	public synchronized AcopTransportDataSource getDataSource(
			ConnectionParameters param) throws ConnectionFailed
	{
		if (useCache) {
			AcopTransportDataSource ad = adapters.get(param);

			if (ad == null) {
				ad = createDataSource(param);
			}

			return ad;
		}

		return createAdapter(param);
	}

	/* (non-Javadoc)
	 * @see com.cosylab.adapters.dal.AdapterFactory#releaseDataSource(com.cosylab.adapters.dal.DynamicValuePropertyAdapter)
	 */
	public synchronized void releaseDataSource(AcopTransportDataSource ds)
	{
		ds.useCount--;

		if (ds.useCount <= 0 || !useCache) {
			adapters.remove(ds.getConnectionParameters());
			ds.disconnect();
		}
	}
	
	/*
	 * (non-Javadoc)
	 * @see de.desy.acop.transport.adapters.AdapterFactory#releaseDataSource(de.desy.acop.transport.ConnectionParameters)
	 */
	public synchronized AcopTransportDataSource releaseDataSource(ConnectionParameters param) {
		AcopTransportDataSource ds = adapters.get(param);
		
		ds.useCount--;

		if (ds.useCount <= 0 || !useCache) {
			adapters.remove(param);
			ds.disconnect();
		}
		
		return ds;
	    
    }

	/* (non-Javadoc)
	 * @see com.cosylab.adapters.dal.AdapterFactory#clear()
	 */
	public synchronized void clear()
	{
		Iterator<AcopTransportDataSource> it = adapters.values().iterator();

		while (it.hasNext()) {
			AcopTransportDataSource element = it.next();
			element.disconnect();
		}

		adapters.clear();
	}

	protected AcopTransportDataSource createAdapter(ConnectionParameters param)
		throws ConnectionFailed
	{
		if (param == null) {
			throw new NullPointerException("param");
		}

		AcopTransportDataSource retVal = new AcopTransportDataSource();
		retVal.connect(param);
		
		return retVal;
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @return DOCUMENT ME!
	 */
	public boolean isUseCache()
	{
		return useCache;
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @param useCache DOCUMENT ME!
	 */
	public void setUseCache(boolean useCache)
	{
		this.useCache = useCache;
	}

	/*
	 * (non-Javadoc)
	 * @see de.desy.acop.transport.adapters.AdapterFactory#getCharacteristics(de.desy.acop.transport.ConnectionParameters)
	 */
	public Map<String, Object> getCharacteristics(ConnectionParameters param) throws ConnectionFailed {
		if (param == null) return new HashMap<String, Object>(0);
		AcopTransportDataSource ds = adapters.get(param);
		if (ds != null) {
			return ds.getCharacteristics();
		}
		return DataSourceUtilities.getCharacteristics(param);
	}

	
}

/* __oOo__ */
