package de.desy.acop.transport.adapters;

/*
 * Copyright (c) 2006 by Cosylab d.o.o.
 *
 * The full license specifying the redistribution, modification, usage and other
 * rights and obligations is included with the distribution of this project in
 * the file license.html. If the license is not included you may find a copy at
 * http://www.cosylab.com/legal/abeans_license.htm or may write to Cosylab, d.o.o.
 *
 * THIS SOFTWARE IS PROVIDED AS-IS WITHOUT WARRANTY OF ANY KIND, NOT EVEN THE
 * IMPLIED WARRANTY OF MERCHANTABILITY. THE AUTHOR OF THIS SOFTWARE, ASSUMES
 * _NO_ RESPONSIBILITY FOR ANY CONSEQUENCE RESULTING FROM THE USE, MODIFICATION,
 * OR REDISTRIBUTION OF THIS SOFTWARE.
 */

import java.util.Map;

import com.cosylab.gui.displayers.DataConsumer;
import com.cosylab.gui.displayers.DataState;
import com.cosylab.gui.displayers.NonblockingNumberConsumer;
import com.cosylab.gui.displayers.SynchronousNumberConsumer;
import com.cosylab.util.CommonException;

import de.desy.acop.transport.AccessMode;
import de.desy.acop.transport.AcopTransport;
import de.desy.acop.transport.ConnectionParameters;


/**
 * 
 * @author Jaka Bobnar, Cosylab
 *
 */
public class ReverseConsumer implements SynchronousNumberConsumer, NonblockingNumberConsumer
	
{
	public static final Class[] SUPPORTED_CONSUMER_TYPES = {
		SynchronousNumberConsumer.class, NonblockingNumberConsumer.class};
	
	private AcopTransport transport;
	private ConnectionParameters parameters;
	private int timeout = 30000;
	private double[] data;
	
	public ReverseConsumer(ConnectionParameters param, AcopTransport tran, int size)
	{
		transport= tran;
        parameters = param.deriveWith(AccessMode.WRITE,timeout);
        transport.setConnectionParameters(parameters);
        data= new double[size];
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.displayers.DataConsumer#getDataConsumer(java.lang.Class)
	 */
	@SuppressWarnings("unchecked")
    public DataConsumer getDataConsumer(Class type)
	{
		if (type.isAssignableFrom(this.getClass())) {
			return this;
		}

		return null;
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.displayers.DataConsumer#getDefaultDataConsumer()
	 */
	public DataConsumer getDefaultDataConsumer()
	{
		return this;
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.displayers.DataConsumer#updateDataState(com.cosylab.gui.displayers.DataState)
	 */
	public void updateDataState(DataState state)
	{
		// TODO Auto-generated method stub
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.displayers.DataConsumer#setCharacteristics(java.util.Map)
	 */
	public void setCharacteristics(Map attributes)
	{
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.displayers.DataConsumer#getName()
	 */
	public String getName()
	{
		return "ObjectAdapter";
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.displayers.DataConsumer#getSupportedCharacteristics()
	 */
	public String[] getSupportedCharacteristics()
	{
		return null;
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.displayers.DataConsumer#getSupportedConsumerTypes()
	 */
	public Class[] getSupportedConsumerTypes()
	{
		return SUPPORTED_CONSUMER_TYPES;
	}

	/*
	 * (non-Javadoc)
	 * @see com.cosylab.gui.displayers.SynchronousNumberConsumer#updateSynchronously(long, java.lang.Number)
	 */
	public void updateSynchronously(long timestamp, Number value) throws CommonException {
		data[0] = value.doubleValue();
	    transport.execute(null,0,data,1);
	    
    }

	/*
	 * (non-Javadoc)
	 * @see com.cosylab.gui.displayers.NonblockingNumberConsumer#updateNonblocking(java.lang.Number)
	 */
	public void updateNonblocking(Number value) {
//		FIXME this is an synchronous call
		data[0] = value.doubleValue();
	    transport.execute(null,0,data,1);
//		transport.attachLink(null, 0, data, 1);
	    
    }
	
	
	/**
	 * 
	 */
	public void release() {
		transport=null;
		parameters=null;
		data=null;
	}
}
/* __oOo__ */