package de.desy.tine.addrUtils;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.net.InetAddress;
import java.net.UnknownHostException;
import de.desy.tine.endianUtils.Swap;

public final class DevAddr
{
  public String FecName;  // = new String(new StringBuffer(16));
  public String SubSystem;
  public byte[] Port = new byte[2];
  public String TagName; // = new String(new StringBuffer(16));
  public String EqName; // = new String(new StringBuffer(7));
  public String Context; // = new String(new StringBuffer(16));
  public short alarmSystem;
  public String Info; // = new String(new StringBuffer(14));
  public byte[] Network = new byte[4];
  public byte[] Node = new byte[6];
  public byte[] ImmediateAddress = new byte[6];
  public String IP; // = new String(new StringBuffer(16));
  public byte PortOffset;
  public byte[] IPh_addr = new byte[4];
  public short  Prot;
  public static int sizeInBytes = (21+16+7+4+14) + (21+16+21+2);
  ByteArrayOutputStream dBuffer = new ByteArrayOutputStream(sizeInBytes);
  private byte[] hByteArray;
  DevAddr()
  {
  }
  public DevAddr(FECAddr fecAddr)
  {
    FecName = fecAddr.Name;
    IP = fecAddr.IP;
    System.arraycopy(fecAddr.Port,0,Port,0,2);
    PortOffset = fecAddr.PortOffset;
    if (fecAddr.IPh_addr[0] == 0 && fecAddr.IPh_addr[1] == 0 && 
        fecAddr.IPh_addr[2] == 0 && fecAddr.IPh_addr[3] == 0)
    {
      try
      {
        InetAddress ia = InetAddress.getByName(IP.trim());
        fecAddr.IPh_addr = ia.getAddress();
      }
      catch (UnknownHostException e)
      {
        // TODO Auto-generated catch block
        e.printStackTrace();
      }
      
    }
    System.arraycopy(fecAddr.IPh_addr,0,IPh_addr,0,4);
  }
  public short getPortOffset()
  {
    short poff;
    short hb = Port[1];
    if (hb < 0) hb += 256;
    short lb = Port[0];
    if (lb < 0) lb += 256;
    poff = (short)(hb * 256 + lb);
    if (poff == 0) poff = PortOffset;
    if (poff < 0) poff += 256;
    return poff;
  }
  public byte[] toByteArray()
  {
    try
    {
      int i, len;
      dBuffer.reset();
      DataOutputStream ds = new DataOutputStream(dBuffer);
      // Export Name information
      byte[] bstr = new byte[16];
      if (FecName != null)
      {
        if ((len=FecName.length()) > 16) len = 16;
        System.arraycopy(FecName.getBytes(),0,bstr,0,len);
      } 
      else len = 0;
      for (i=len; i<16; i++) bstr[i] = 0;
      ds.write(bstr,0,16);
      if (SubSystem != null)
      {
        if ((len=SubSystem.length()) > 16) len = 16;
        System.arraycopy(SubSystem.getBytes(),0,bstr,0,len);        
      }
      for (i=len; i<16; i++) bstr[i] = 0;
      ds.write(bstr,0,5);  // short subsystem tag
      if (TagName != null)
      {
        if ((len=TagName.length()) > 16) len = 16;
        System.arraycopy(TagName.getBytes(),0,bstr,0,len);
      } 
      else len = 0;
      for (i=len; i<16; i++) bstr[i] = 0;
      ds.write(bstr,0,16);      
      if (EqName != null)
      {
        if ((len=EqName.length()) > 6) len = 6;
        System.arraycopy(EqName.getBytes(),0,bstr,0,len);
      } 
      else len = 0;
      for (i=len; i<7; i++) bstr[i] = 0;
      ds.write(bstr,0,7);
      if (Context != null)
      {
        if ((len=Context.length()) > 16) len = 16;
        System.arraycopy(Context.getBytes(),0,bstr,0,len);
      } 
      else len = 0;
      for (i=len; i<16; i++) bstr[i] = 0;
      ds.write(bstr,0,16);
      ds.writeShort(Swap.Short(alarmSystem));
      // FEC Name information
      if (FecName != null)
      {
        if ((len=FecName.length()) > 16) len = 16;
        System.arraycopy(FecName.getBytes(),0,bstr,0,len);
      } 
      else len = 0;
      for (i=len; i<16; i++) bstr[i] = 0;
      ds.write(bstr,0,16);
      ds.write(bstr,0,3);  // reserved
      ds.write(Port,0,2);
      ds.write(Network,0,4);
      ds.write(Node,0,6);
      ds.write(ImmediateAddress,0,6);
      if (IP != null)
      {
        System.arraycopy(IP.getBytes(),0,bstr,0,len=IP.length());
      } 
      else len = 0;
      for (i=len; i<16; i++) bstr[i] = 0;
      ds.write(bstr,0,16);
      ds.writeByte(PortOffset);
      ds.write(IPh_addr,0,4);
      ds.writeShort(Swap.Short(Prot));
      ds.close();	
    }
    catch (IOException e)
    {
      e.printStackTrace();
      System.out.println("could not prepare byte stream");
      return null;
    }	 
    hByteArray = dBuffer.toByteArray();
    return hByteArray;
  }  
  public void toStruct()
  {
    try
    {
      ByteArrayInputStream dis = new ByteArrayInputStream(hByteArray);
      DataInputStream ds = new DataInputStream(dis);
      // Export resolution information:
      byte[] bstr = new byte[16];
      ds.read(bstr,0,16);
      // fill in FecName with RPCFec structure below 
      bstr = new byte[5];
      ds.read(bstr,0,5);
      SubSystem = new String(bstr);
      bstr = new byte[16];
      ds.read(bstr,0,16);
      TagName = new String(bstr);
      bstr = new byte[7];
      ds.read(bstr,0,7);
      EqName = new String(bstr);
      bstr = new byte[16];
      ds.read(bstr,0,16);
      Context = new String(bstr);
      alarmSystem = Swap.Short(ds.readShort());
      // FEC address information:
      bstr = new byte[16];
      ds.read(bstr,0,16);
      FecName = new String(bstr);
      bstr = new byte[3];
      ds.read(bstr,0,3);
      // ignore reserved field for now
      ds.read(Port,0,2);
      ds.read(Network,0,4);
      ds.read(Node,0,6);
      ds.read(ImmediateAddress,0,6);
      bstr = new byte[16];
      ds.read(bstr,0,16);
      IP = new String(bstr);
      PortOffset = ds.readByte();
      ds.read(IPh_addr,0,4);
      Prot = Swap.Short(ds.readShort());
      ds.close(); dis.close();
    }
    catch (IOException e)
    {
      e.printStackTrace();
      System.out.println("could not prepare byte stream");
    }	 
  }  
  public String toString()
  {
    String msg = new String("Exp name : " + TagName);
    msg = msg.concat("\nEqm name : " + EqName);
    msg = msg.concat("\nContext  : " + Context);
    msg = msg.concat("\nAlm sys  : " + alarmSystem);
    msg = msg.concat("\nFEC name : " + FecName);
    msg = msg.concat("\nFEC IP   : " + IP);
    msg = msg.concat("\nFEC Port : " + getPortOffset());
    return msg;  
  }
}