package de.desy.tine.addrUtils;

import de.desy.tine.client.TLink;
import de.desy.tine.dataUtils.TDataType;
import de.desy.tine.definitions.TAccess;
import de.desy.tine.definitions.TErrorList;
import de.desy.tine.queryUtils.TQuery;
import de.desy.tine.types.NAME16;

public class ENSTools
{
  public static int sendExportInformationToENS(FECAddr fec,FECInfo info, DevAddr da)
  {
    if (fec == null || info == null || da == null) return TErrorList.argument_list_error;
    if (TModAddress.currentConfiguredNameServerTag == null)
    {
      TQuery.getModuleAddressInfo("ENS","DEFAULT");
    }
    fec.Port[0] = (byte) (fec.PortOffset & 0xff);
    fec.Port[1] = (byte) ((fec.PortOffset >> 8) & 0xff);
    // prepare it and send it out ...
    // devAddr-- + FecAddr + FecInfo
    byte[] expAddr = new byte[DevAddr.sizeInBytes + FECInfo.getSizeInBytes()];
    System.arraycopy(da.toByteArray(), 0, expAddr, 0, DevAddr.sizeInBytes);
    System.arraycopy(info.toByteArray(), 0, expAddr, DevAddr.sizeInBytes, FECInfo.getSizeInBytes());
    TDataType din = new TDataType(expAddr, "");
    TDataType dout = new TDataType(expAddr, "");
    // this should always point to the primary ENS
    if (TModAddress.currentConfiguredNameServerTag != null)
    {
      TLink expLnk = new TLink(TModAddress.currentConfiguredNameServerTag,"EXPORT",dout,din,TAccess.CA_WRITE);
      return expLnk.execute(500, true);
    }
    else
    {
      return TErrorList.not_running;
    }
  }
  public static int sendExportInformationToENS(DevAddr da, FECInfo info)
  {
    if (info == null || da == null) return TErrorList.argument_list_error;
    if (TModAddress.currentConfiguredNameServerTag == null)
    {
      TQuery.getModuleAddressInfo("ENS","DEFAULT");
    }
    if (da.PortOffset > 0 && da.Port[0] == 0 && da.Port[1] == 0)
    { // consistency check ...  
      da.Port[0] = (byte) (da.PortOffset & 0xff);
      da.Port[1] = (byte) ((da.PortOffset >> 8) & 0xff);
    }
    // prepare it and send it out ...
    // devAddr-- + FecAddr + FecInfo
    byte[] expAddr = new byte[DevAddr.sizeInBytes + FECInfo.getSizeInBytes()];
    System.arraycopy(da.toByteArray(), 0, expAddr, 0, DevAddr.sizeInBytes);
    System.arraycopy(info.toByteArray(), 0, expAddr, DevAddr.sizeInBytes, FECInfo.getSizeInBytes());
    TDataType din = new TDataType(expAddr, "");
    TDataType dout = new TDataType(expAddr, "");
    // this should always point to the primary ENS
    if (TModAddress.currentConfiguredNameServerTag != null)
    {
      TLink expLnk = new TLink(TModAddress.currentConfiguredNameServerTag,"EXPORT",dout,din,TAccess.CA_WRITE);
      return expLnk.execute(500, true);
    }
    else
    {
      return TErrorList.not_running;
    }
  }
  public static int removeFECFromENS(String fecName)
  {
    if (fecName == null) return TErrorList.argument_list_error;
    if (TModAddress.currentConfiguredNameServerTag == null)
    {
      TQuery.getModuleAddressInfo("ENS","DEFAULT");
    }
    NAME16[] fn = new NAME16[1];
    fn[0] = new NAME16(fecName);
    TDataType din = new TDataType(fn);
    if (TModAddress.currentConfiguredNameServerTag != null)
    {
      String dn = TModAddress.currentConfiguredNameServerTag + "/FEC";
      TLink rmvLnk = new TLink(dn,"REMOVE",null,din,TAccess.CA_WRITE);
      return rmvLnk.execute(500, true);
    }
    else
    {
      return TErrorList.not_running;
    }
  }
  public static int removeDeviceServerFromENS(String context, String deviceServer)
  {
    if (context == null || deviceServer == null) return TErrorList.argument_list_error;
    if (TModAddress.currentConfiguredNameServerTag == null)
    {
      TQuery.getModuleAddressInfo("ENS","DEFAULT");
    }
    NAME16[] sn = new NAME16[1];
    sn[0] = new NAME16(deviceServer);
    TDataType din = new TDataType(sn);
    if (TModAddress.currentConfiguredNameServerTag != null)
    {
      String dn = TModAddress.currentConfiguredNameServerTag + "/" + context;
      TLink rmvLnk = new TLink(dn,"REMOVE",null,din,TAccess.CA_WRITE);
      return rmvLnk.execute(500, true);
    }
    else
    {
      return TErrorList.not_running;
    }
  }
}
