package de.desy.tine.client;

import java.io.*;
import java.net.*;
import java.util.Date;
import java.util.HashSet;
import java.util.Hashtable;
import java.util.LinkedList;
import de.desy.tine.addrUtils.*;
import de.desy.tine.dataUtils.*;
import de.desy.tine.definitions.*;
import de.desy.tine.endianUtils.Swap;
import de.desy.tine.headers.*;
import de.desy.tine.io.*;
import de.desy.tine.startup.*;
import de.desy.tine.structUtils.*;

public class TLinkFactory
{
  private static TLinkFactory instance = new TLinkFactory();
  boolean active;
  public boolean terminate;
  Date date = new Date();
  int time;
  private static String tineUserName = System.getProperty("user.name");
  public String getUserName() { return tineUserName; }
  public void setUserName(String userName) { tineUserName = new String(userName); }
  private TPacket atp; // general socket for asynchronous communication
  private TPacket stp; // socket for synchronous communication
  private TPacket qtp; // socket for synchronous queries
  private TPacket nmtp; // socket for async net service requests
  private TPacket gtp; // globals socket
  public TPacket amtp; // asynchronous multicast socket
  TPublishedData phs;
  TGlobalsData ghs;
  public static int debugLevel = 0;
  private int totalLinkTimeouts = 0;
  private int totalConnectionArrivals = 0;
  private boolean useConnectedSockets = false;
  public boolean isUseConnectedSockets()
  {
    return useConnectedSockets;
  }
  public void setUseConnectedSockets(boolean useConnectedSockets)
  {
    this.useConnectedSockets = useConnectedSockets;
  }
  public int getTotalLinkTimeouts() { return totalLinkTimeouts; }
  public int getTotalConnectionArrivals() { return totalConnectionArrivals; }
  static private int numberTLinksInTable = 1; // assume the ENS is there ...
  public int getNumberTLinksInTable() { return numberTLinksInTable;}
  // is this significantly quicker to have a static link table rather than a linked list ?
  // probably not => TODO: convert this to a linked list
  static private int maximumNumberTLinks = 1024;
  static private int HEARTBEAT = 60000; // inactive seconds allowed for REFRESH links
  static public int getMaximumNumberOfLinks()
  {
    return maximumNumberTLinks;
  }
  static public int setMaximumNumberOfLinks(int numberOfLinks)
  {
    if (numberOfLinks > 10) maximumNumberTLinks = numberOfLinks;
    return maximumNumberTLinks;
  }
  private TLinkHook tLinkHook = null;
  public void setTLinkHook(TLinkHook hook) { tLinkHook = hook; }
  public TLinkHook getTLinkHook() { return tLinkHook; }
  //private int nlinks = 0;
  private static final TLink tNullLink = new TLink();
  private static TLink[] linkTable = new TLink[maximumNumberTLinks];
  public TLink[] getLinkTable() { return linkTable; }
  public int getLinkTableId(TLink lnk)
  {
    for (int i=0; i<numberTLinksInTable; i++) 
    {      
      if (linkTable[i] == lnk) return i;
    }
    return -1;
  }
  public static final int adjustLinkTableRemove = 0;
  public static final int adjustLinkTableAdd = 1;
  public static final int adjustLinkTableReplace = 2;
  public static synchronized int adjustLinkTable(TLink lnk,int direction)
  {
    int i;
    if (lnk == null) return -TErrorList.argument_list_error;
    switch (direction)
    {
      case adjustLinkTableRemove:
        if (lnk == null || (i = lnk.linkId) < 0) return -TErrorList.argument_list_error;
        if (i == 0) return 0; // never remove the ENS link
        if (linkTable[i] == null) return 0;
        if (debugLevel > 0) System.out.println("removing link " + linkTable[i]);
        linkTable[i].terminate = false;
        if (lnk.tb == null)
        { // no tcp link buckets ...
          linkTable[i] = null;
          while (numberTLinksInTable > 1 && linkTable[numberTLinksInTable-1] == null)
          {
            if (debugLevel > 0) System.out.println("decrement number of entries in link table");
            numberTLinksInTable--;
          }
        }
        if (lnk.twcl != null && lnk.twcl.parent != null)
        { // if part of a wildcard link, signal the watchdog to get rid of the parent
          lnk.twcl.parent.terminate = true;
        }  
        break;
      case adjustLinkTableAdd:
        String srv = lnk.getDeviceServer();
        String ctx = lnk.getContext();
        if (srv != null &&
            (ctx == null || ctx.length() == 0 || ctx.compareTo("DEFAULT") == 0) &&
            srv.startsWith("ENS")) 
        {
          if (numberTLinksInTable == 0) 
          {
            if (debugLevel > 0) System.out.println("add ENS entry to link table");
            numberTLinksInTable++;
          }
          return 0;
        }
        int freeslot = 0;
        for (i = 1; i < numberTLinksInTable && (linkTable[i] !=  lnk || linkTable[i] == tNullLink); i++)
        {
          if (freeslot == 0 && linkTable[i] == null) freeslot = i; 
        }
        if (i < numberTLinksInTable) 
        {
          linkTable[i].terminate = false;
          return i; // already in table
        }
        if (freeslot > 0)
        {
          linkTable[freeslot] = lnk;
          lnk.linkId = freeslot;
          return freeslot;
        }
        if (numberTLinksInTable == maximumNumberTLinks) return -TErrorList.resources_exhausted;
        if (debugLevel > 0) System.out.println("increment number of entries in link table");
        lnk.linkId = i;
        linkTable[i] = lnk;
        numberTLinksInTable++;
        return i;
      case adjustLinkTableReplace:
        if (lnk == null || (i = lnk.linkId) < 0) return -TErrorList.argument_list_error;
        if (lnk == null) return -TErrorList.argument_list_error;
        if (i == 0) return 0; // never replace the ENS link
        if (debugLevel > 0) System.out.println("replace link " + linkTable[i]);
        linkTable[i] = lnk;
        break;
      default:
        break;
    }
    return 0;
  }
  public TLink[] getActiveLinks()
  {
    int n = 0;
    for (int i=1; i<numberTLinksInTable; i++) 
      if (linkTable[i] != null && linkTable[i].active) n++;
    if (n == 0) return null;
    TLink[] tbl = new TLink[n];
    n = 0;
    for (int i=1; i<numberTLinksInTable; i++) 
    {      
      if (linkTable[i] != null && linkTable[i].active) 
        tbl[n++] = linkTable[i];
      if (n >= tbl.length) break;
    }
    return tbl;
  }
  //private TCallback tcb;
  //private TLinkCallback tlcb;
  boolean isInsideCallback = false;
  private LinkedList wcList = null;
  // java 1.5 or higher !!!! ->
  private static Hashtable<Object,TLinkGroup> grpList = null;
  public TLinkGroup getGroup(TLinkCallback cb)
  {
    TLinkGroup grp = null;
    if (grpList == null) grpList = new Hashtable<Object,TLinkGroup>();
    if (grpList.containsKey(cb)) return (TLinkGroup)grpList.get(cb);
    if (debugLevel > 0) System.out.println("adding new group to group table");
    grp = new TLinkGroup();
    grpList.put(cb,grp);
    return grp;
  }
  public TLinkGroup getGroup(TCallback cb)
  {
    TLinkGroup grp = null;
    if (grpList == null) grpList = new Hashtable<Object,TLinkGroup>();
    if (grpList.containsKey(cb)) return (TLinkGroup)grpList.get(cb);
    grp = new TLinkGroup();
    grpList.put(cb,grp);
    return grp;
  }
  public TWildcardLink getWildcardLink()
  {
    TWildcardLink wc = null;
    if (wcList != null)
    {
      for (int i=0; i<wcList.size(); i++)
      {
        wc = (TWildcardLink)wcList.get(i);
        if (wc.parent == null) return wc;
      }
    }
    else
    {
      wcList = new LinkedList();
    }
    wc = new TWildcardLink();
    wcList.add(wc);
    return wc;
  }
  public int getWildcardLinkId(TWildcardLink wc)
  {
    if (wc == null ||  wcList == null) return -1;
    for (int i=0; i<wcList.size(); i++)
    {
      if ((TWildcardLink)wcList.get(i) == wc) return i;
    }
    return -1;
  }
  public void rmvWildcardLink(TWildcardLink wc)
  {
    if (wc == null ||  wcList == null) return;
    for (int i=0; i<wcList.size(); i++)
    {
      if ((TWildcardLink)wcList.get(i) == wc)
      {
        //wc.fvalues = null;
        //wc.ivalues = null;
        wc.links = null;
        wc.list = null;
        wc.status = null;
        wc.parent = null;
      }
    }    
  }
  // static TErrorList tel;
  private TFactoryThread atfThrd;
  public TFactoryThread getAsynchronousLinkThread() { return atfThrd; }
  private TFactoryThread stfThrd;
  public TFactoryThread getSynchronousLinkThread() { return stfThrd; }
  private TFactoryThread qtfThrd;
  public TFactoryThread getQueryLinkThread() { return qtfThrd; }
  public TFactoryGlobalsThread gtfThrd;
  public TFactoryThread mtfThrd;
  public TFactoryThread getMulticastLinkThread() { return mtfThrd; }
  public TFactoryThread ntfThrd;
  public TFactoryThread getNetcastLinkThread() { return ntfThrd; }
  final TFactoryWatchdogThread tfwdThrd = new TFactoryWatchdogThread();
  static final int maximumNumberTBuckets = 10;
  static final int maximumNumberGlobals = 25;
  private TLinkBucket[] bucketTable = new TLinkBucket[maximumNumberTBuckets];
  private TInitializer initializer = TInitializerFactory.getInstance().getInitializer();
  public TInitializer getInitializer()
  {
    return initializer;
  }
  public class TLinkBucket extends TBucket
  {
    public TLinkBucket(int index)  // index in link table 
    {
      if (index < 0 || index >= TLinkFactory.getMaximumNumberOfLinks()) return;
      if (linkTable[index].getBucket() != null) return; // already have a bucket 
      TBucketThread tfBckThrd = getTBucketThread(index);
      if (tfBckThrd != null)  // a thread is already attached to this link
      {
        TLinkBucket tb = tfBckThrd.getBucket();
        setSocket(tb.getSocket());
        setOutputStream(tb.getOutputStream());
        setInputStream(tb.getInputStream());
        return;
      }
      // this is a new one ...
      initBucket(linkTable[index].modAddress.srvAddress.srvHost,
          getInitializer().getTCPPort() + linkTable[index].modAddress.srvAddress.srvPortOffset,
          new TBucketThread(this));
      putBucketThread(this);
      activate();
    }
  }
  public class TBucketThread extends Thread
  {
    TLinkBucket tb;
    boolean isWaiting = false;
    public TLinkBucket getBucket() { return tb; }
    public TBucketThread(TLinkBucket tBucket)
    {
      tb = tBucket;
      tb.active = true;
      String s = null;
      Socket sck = tb.getSocket();
      if (sck != null) s = new String("tcp port " + sck.getLocalPort());
      else s = new String("tcp unbound socket ?");
      this.setName("Link Factory " + s);
    }
    public synchronized void run()
    {
      byte[] wa = new byte[1472];
      byte[] msgsizb = new byte[2];
      int nread, nleft, n, waptr;
      if (debugLevel > 0) System.out.println("Factory Bucket Thread started ...");
      try
      {
        InputStream is = tb.getInputStream();
        while (tb.active)
        {
          if (terminate) break;
          if (debugLevel > 0) System.out.println("Waiting for steam data ...");
          isWaiting = true;
          if ((nread = is.read(wa, 0, 1472)) == -1)
          {
            tb.isDeactivating = true;
            break;
          }
          totalConnectionArrivals++;
          nleft = nread;
          waptr = 0;
          while (nleft > 0)
          {
            if (tb.getBucketPointer() == 0)
            {
              System.arraycopy(wa, waptr, msgsizb, 0, 2);
              tb.setBucketSize(Swap.Short(new DataInputStream(new ByteArrayInputStream(msgsizb)).readShort()));
            }
            int ptr = tb.getBucketPointer();
            int siz = tb.getBucketSize();
            byte[] buf = tb.getBucketBuffer();
            n = siz - ptr;
            if (nleft >= n) // at least enough
            {
              System.arraycopy(wa, waptr, buf, ptr, n);
              InterpretIncomingData(buf, siz, tb.getBucketEndpoint(), false);
              waptr += n; // move the read buffer pointer along
              nleft -= n; // decrement what's remaining to be read
              ptr = 0; // reset the bucket pointer
            }
            else
            {
              System.arraycopy(wa, waptr, buf, ptr, nleft);
              ptr += nleft;
              nleft = 0;
            }
            tb.setBucketPointer(ptr);
            // NotifyDeferredCallbacks();
          }
        }
        tb.getSocket().close();
      }
      catch (Exception e)
      {
        if (debugLevel > 1) e.printStackTrace();
      }
      finally
      {
        removeBucketThread(tb);
      }
    }
  }
  public TBucketThread makeBucketThread(TLinkBucket tb)
  {
    return new TBucketThread(tb);
  }
  public TBucketThread getTBucketThread(int index)
  {
    for (int i = 0; i < maximumNumberTBuckets; i++)
    {
      if (bucketTable[i] == null) continue;
      if (bucketTable[i].getBucketPort() != initializer.getTCPPort()
          + linkTable[index].modAddress.srvAddress.srvPortOffset) continue;
      if (bucketTable[i].getBucketEndpoint() != linkTable[index].modAddress.srvAddress.srvHost) continue;
      return (TBucketThread)bucketTable[i].getBucketThread();
    }
    return null;
  }
  public int putBucketThread(TLinkBucket tb)
  {
    for (int i = 0; i < maximumNumberTBuckets; i++)
    {
      if (bucketTable[i] == null)
      {
        bucketTable[i] = tb;
        return 0;
      }
    }
    return -1;
  }
  private int removeBucketThread(TLinkBucket tb)
  {
    if (tb == null) return -1;
    for (int i = 0; i < numberTLinksInTable; i++)
    {
      if (linkTable[i] == null) continue;
      if (tb.getBucketPort() != initializer.getTCPPort() + linkTable[i].modAddress.srvAddress.srvPortOffset) continue;
      if (tb.getBucketEndpoint() != linkTable[i].modAddress.srvAddress.srvHost) continue;
      linkTable[i].active = false;
      linkTable[i].tb = null;
      linkTable[i].terminate = true;
    }
    for (int i = 0; i < maximumNumberTBuckets; i++)
    {
      if (bucketTable[i] == null) continue;
      if (bucketTable[i] != tb) continue;
      bucketTable[i].setBucketPort(0);
    }
    try
    {
      Socket sck = tb.getSocket();
      sck.shutdownInput();
      sck.shutdownOutput();
      sck.close();
    }
    catch (IOException e)
    {
      if (debugLevel > 1) System.out.println(" IOException : " + e);
    }
    finally
    {
      tb.isDeactivating = false;
    }
    return 0;
  }
  public class TFactoryWatchdogThread extends Thread
  {
    public synchronized void run()
    {
      this.setName("Link Factory watchdog");
      TLink lnk;
      long ltime, deltatime, gracePeriod, tcpIdlePeriod;
      ltime = System.currentTimeMillis();
      time = (int) (ltime / 1000);
      while (active)
      {
        if (terminate) break;
        try
        {
          sleep(100);
          ltime = System.currentTimeMillis();
          time = (int) (ltime / 1000);
          int i;
          for (i = 0; i < numberTLinksInTable; i++)
          {
            if (linkTable[i] != null && linkTable[i] != tNullLink)
            {
              lnk = linkTable[i];
              if (lnk.linkId != i)
              {
                System.out.println("link table id does not match entry index !");
              }
              if (lnk.delayEstablishLink)
              {
                lnk.delayEstablishLink = false;
                establishLink(lnk);
                continue;
              }
              if (lnk.terminate == true && lnk.linkId > 0) 
              { // marked for termination and removal
                if (lnk.sub.starttime < time) 
                { // only remove this if the starttime is old
                  if (debugLevel > 0)
                  {
                    System.out.println("remove terminated link " + lnk.linkId + " /" + lnk.cntName + "/"
                        + lnk.expName + "/" + lnk.devName + " " + lnk.devProperty + " "
                        + TDataTime.toString(ltime));
                  }
                  synchronized (lnk)
                  { // wake up the calling thread
                    lnk.notifyAll();
                  } 
                  removeTLink(lnk);
                }
              }
              if (!lnk.active) continue;
              if (lnk.linkStatus == TErrorList.server_redirection)
              {
                short mod = lnk.sub.mode;
                lnk.con = new TContract(lnk);
                lnk.sub = new TSubscription(lnk.con, lnk); // going out ...
                lnk.sub.mode = mod;
                establishLink(lnk);
                lnk.linkStatus = 0;
                continue;
              }
              // the link is still active ...
              short baseMode = (short) (lnk.sub.mode & 0x00ff);
              tcpIdlePeriod = 3000;
              deltatime = ltime - lnk.sub.linkLastTime;
              gracePeriod = lnk.devTimeout < 1000 ? 100 : 1000;
              if (baseMode == TMode.CM_REFRESH) gracePeriod += HEARTBEAT; 
              if (deltatime > (long) (lnk.devTimeout + gracePeriod) && baseMode > TMode.CM_CANCEL
                  && baseMode < TMode.CM_REGISTER)
              {
                if (lnk.linkStatus == -1) 
                {
                  lnk.linkStatus = TErrorList.link_not_open;
                  continue;
                }
                totalLinkTimeouts++;
                lnk.linkTimeouts++;
                if (debugLevel > 0)
                {
                  System.out.println("link " + lnk.linkId + " link timeout " + lnk.devTimeout
                      + " exceeded : " + ltime + " vs " + lnk.sub.linkLastTime + " timeout counter : "
                      + lnk.linkTimeouts + " " + TDataTime.toString(ltime));
                }
                lnk.pending = false;
                lnk.dOutput.timestamp = time;
                lnk.linkStatus = TErrorList.link_not_open;
                phs.linkCounter = 0;
                lnk.needsNotification = true;
                if (lnk.isGrouped())
                { 
                  if (lnk.getGroup().getNumberPending() > 0) continue;
                  lnk.getGroup().reset();
                }
                if (baseMode > TMode.CM_SINGLE) synchronized (lnk)
                {
                  if (lnk.terminate == false)
                  {
                    if (lnk.tcb != null)
                    {
                      lnk.isInsideCallback = true;
                      lnk.tcb.callback(lnk.callbackId, lnk.linkStatus);
                      lnk.isInsideCallback = false;
                    }
                    else if (lnk.tlcb != null)
                    {
                      lnk.isInsideCallback = true;
                      lnk.tlcb.callback(lnk);
                      lnk.isInsideCallback = false;
                    }
                    lnk.needsNotification = false;
                  }
                  lnk.notifyAll(); // wake up the calling thread
                  lnk.sub.linkLastTime = ltime;
                }
                if (baseMode > TMode.CM_SINGLE)
                {
                  establishLink(lnk);
                }
                else if (baseMode == TMode.CM_SINGLE)
                {
                  if (lnk.retryOnTimeoutError && lnk.linkTimeouts < 3)
                  {
                    lnk.sub.mode |= TMode.CM_RETRY;
                    lnk.sub.linkLastTime = ltime;
                    establishLink(lnk);
                  }
                  else
                  {
                    synchronized (lnk)
                    {
                      if (lnk.terminate == false)
                      {
                        if (lnk.tcb != null)
                        {
                          lnk.isInsideCallback = true;
                          lnk.tcb.callback(lnk.callbackId, lnk.linkStatus);
                          lnk.isInsideCallback = false;
                        }
                        else if (lnk.tlcb != null)
                        {
                          lnk.isInsideCallback = true;
                          lnk.tlcb.callback(lnk);
                          lnk.isInsideCallback = false;
                        }
                      }
                      if (debugLevel > 0)
                        System.out.println(" cancel single link " + lnk.linkId + " from watchdog" + " "
                            + TDataTime.toString(ltime));
                      if ((lnk.sub.mode & TMode.CM_CONNECT) == TMode.CM_CONNECT)
                      {
                        tcpIdlePeriod = 0;
                        lnk.sub.mode = (short) (TMode.CM_CANCEL | TMode.CM_CONNECT);
                      }
                      else
                      {
                        lnk.sub.mode = TMode.CM_CANCEL; // turn off the link
                      }
                      //cancel(lnk);
                      lnk.active = false; lnk.pending = false;
                      lnk.linkTimeouts = 0;
                      lnk.notifyAll(); // wake up the calling thread
                    }
                  }
                }
              }
              if (deltatime > (long) (lnk.devTimeout + tcpIdlePeriod)
                  && lnk.sub.mode == (TMode.CM_CANCEL | TMode.CM_CONNECT))
              {
                removeBucketThread((TLinkBucket)lnk.tb);
              }
              if (lnk.sub.mode == TMode.CM_CANCEL && !lnk.active && lnk.linkStatus != -1) 
              {
                if (debugLevel > 0) System.out.println("watchdog ignoring cancelled link " + lnk.linkId); 
                //lnk.terminate = true;
              }
              if (debugLevel > 2 || (debugLevel > 1 && lnk.linkStatus != 0))
              {
                System.out.println("link : " + "/" + lnk.cntName + "/" + lnk.expName + "/" + lnk.devName
                    + " " + lnk.devProperty + " : linkStatus " + lnk.linkStatus + " Link active : "
                    + lnk.active + " terminate : " + lnk.terminate + " " + TDataTime.toString(ltime));
              }
            }
          }
        }
        catch (InterruptedException e)
        {
          if (debugLevel > 0)
          {
            System.out.println(" Watchdog Thread : " + active + " terminate : " + terminate);
          }
        }
        catch (Exception e)
        {
          if (debugLevel > 1) e.printStackTrace();
        }
      }
    }
  }
  private int NotifyDeferredCallbacks()
  {
    TLink eqm;
    for (int i = 0; i < numberTLinksInTable; i++)
    {
      if (linkTable[i] != null && linkTable[i] != tNullLink)
      {
        eqm = linkTable[i];
        if (eqm.needsNotification)
        {
          synchronized (eqm)
          {
            try 
            {
              if (eqm.terminate == false)
              {
                if (eqm.tcb != null)
                {
                  eqm.isInsideCallback = true;
                  eqm.tcb.callback(eqm.callbackId, eqm.linkStatus);
                  eqm.isInsideCallback = false;
                }
                else if (eqm.tlcb != null)
                {
                  eqm.isInsideCallback = true;
                  eqm.tlcb.callback(eqm);
                  eqm.isInsideCallback = false;
                }
              }
              eqm.needsNotification = false;
            }
            catch (Exception e)
            {
              if (debugLevel > 1) e.printStackTrace();
            }
          }
          eqm.notifyAll(); // wake up the calling thread
        }
      }
    }
    return 0;
  }
  public int fillinIncomingData(TLink lnk)
  {
    return fillinIncomingData(lnk.dOutput);
  }
  public int fillinIncomingData(TDataType dtype)
  {
    Object hDataObject = dtype.getDataObject();
    switch (dtype.getFormat())
    {
      case TFormat.CF_STRUCT:
      case TFormat.CF_BYTE:
        byte[] hBytes = (byte[])hDataObject; 
        if (hBytes != null) dtype.getData((byte[])hBytes);
        Object hStructObject = dtype.getStructObject(); 
        if (hStructObject instanceof TTaggedStructure) 
        {
          ((TTaggedStructure)hStructObject).toStruct(hBytes);
        }
        else if (hStructObject instanceof TTaggedStructure[])
        {
          TTaggedStructure[] tts = (TTaggedStructure[])hStructObject;
          int dsiz = tts[0].getSizeInBytes();
          byte[] hb = new byte[dsiz];
          for (int i = 0; i < tts.length; i++)
          {
            System.arraycopy(hBytes, i * dsiz, hb, 0, dsiz);
            tts[i].toStruct(hb);
          }
        }
        break;
      default:
        if (hDataObject != null) dtype.getData(hDataObject);
        break;
    }
    return 0;
  }
  public int fillinIncomingDataWithErrValue(TLink lnk)
  {
    if (lnk.linkStatus == 0) return 0;
    if (lnk.useErrObject)
    {
      lnk.dOutput.dataCopy(lnk.dError);
      return 0;
    }
    if (lnk.useErrValue)
    {
      lnk.dOutput.dataFill(lnk.linkErrValue);
      if (lnk.linkErrString != null) lnk.dOutput.dataFill(lnk.linkErrString);
    }
    return 0;
  }
  private int InterpretIncomingData(byte[] data, int length, InetAddress addr,boolean deferCallbacks)
  {
    TLink lnk;
    boolean hasData;
    synchronized (phs)
    { // this will run through everything that has come in and fill in the blanks
      // also checking return code vs server_redirection, etc. ...
      phs.toStruct(data, length);
    }
    if (debugLevel > 0) System.out.println("recv " + length + " bytes");
    for (int i = 0; i < numberTLinksInTable; i++)
    { // TODO: loop over everything ?! -> should be easier to just look at what came in ...
      if (linkTable[i] != null && linkTable[i] != tNullLink)
      {
        lnk = linkTable[i];
        if (debugLevel > 1)
        {
          System.out.print("link " + lnk.linkId + "(" + "/" + lnk.cntName + "/" + lnk.expName + "/"
              + lnk.devName + "[" + lnk.devProperty + "] : ");
          System.out.print((lnk.active ? "active" : "inactive") + (lnk.linkStale ? " stale" : " not stale"));
          System.out.println(" -> " + lnk.dOutput.blksin + " blks in from " + lnk.dOutput.numblks + " <"
              + lnk.linkStatus + ">");
        }
        if (!lnk.active) continue;
        if (!lnk.linkStale) continue;
        lnk.linkStale = false;
        if (lnk.linkStatus == TErrorList.server_redirection) continue;
        lnk.linkPeer = addr;
        // use the phs.subId to get the incoming link id !
        if (lnk.dOutput.blksin == lnk.dOutput.numblks)
        {
          long ltime = System.currentTimeMillis();
          lnk.sub.linkLastTime = ltime;
          if (lnk.adjustDefaultValues) // adjust contract upon first completion
          {
            int fmtsize = TFormat.formatSizeOf(lnk.dOutput.dFormat);
            int dretsize = lnk.dOutput.bytesin;
            if (lnk.linkStatus != 0) dretsize -= TDataType.RPCERR_SIZE;
            if (dretsize > 0 && fmtsize > 0) lnk.dOutput.dArrayLength = dretsize / fmtsize;
            lnk.adjustDefaultValues = false;
          }
          lnk.pending = false;
          if (lnk.linkStatus == TErrorList.get_subscription_id) continue;
          if (lnk.linkStatus == 0 || (lnk.linkStatus & TErrorList.CE_SENDDATA) == TErrorList.CE_SENDDATA)
            hasData = true;
          else
            hasData = false;
          if (hasData)
          {
            if (debugLevel > 0)
              System.out.println("call getData for link " + lnk.linkId + " format "
                  + TFormat.toString(lnk.dOutput.getFormat()) + " "
                  + TDataTime.toString(System.currentTimeMillis()));
            // do something with it (events ?) ...
            fillinIncomingData(lnk);
          }
          else
          {
            fillinIncomingDataWithErrValue(lnk);
          }
          if (lnk.isGrouped())
          { 
            if (lnk.getGroup().getNumberPending() > 0) continue;
            if (debugLevel > 0)
              System.out.println("all members of group have updated");
            lnk.getGroup().reset();
          }
          if (lnk.linkStatus >= 0)
          {
            lnk.pending = true;
            if (lnk.dOutput.timestamp == 0)
            {
              lnk.dOutput.timestamp = (int) (ltime / 1000);
              lnk.dOutput.timestampMSEC = (int) (ltime % 1000);
            }
            lnk.linkTimeouts = 0; // reset timeout counter
            // if caller "cancel()s" then terminate is set to true (should be
            // polite and tell the server).
            // if the link automatically terminates (due to SINGLE -> CANCEL,
            // etc.
            // then the TMode is set to CM_CANCEL (don't need to tell the
            // server)
            if (lnk.sub.mode == TMode.CM_CANCEL) lnk.active = false;
            // set notification flag:
            if (!lnk.isWithinTolerance()) 
            {
              lnk.needsNotification = true;
              if (!deferCallbacks)
              {
                synchronized (lnk)
                {
                  try
                  {
                    if (lnk.active == true)
                    {
                      if (lnk.tcb != null)
                      {
                        isInsideCallback = true;
                        if (!lnk.isCancelledWithDependencies())
                        {
                          lnk.isInsideCallback = true;
                          lnk.tcb.callback(lnk.callbackId, lnk.linkStatus);
                          lnk.isInsideCallback = false;
                        }
                        if (lnk.hasDependencies())
                        { // chained dependencies ?
                          TLink xlnk;
                          TDataType xlnkData;
                          LinkedList xlst = lnk.getDependencies();
                          for (int k=0; k < xlst.size(); k++)
                          {
                            xlnk = (TLink)xlst.get(k);
                            if ((xlnkData=xlnk.getOutputDataObject()) != null)
                            {
                              xlnkData.dataCopy(lnk.dOutput);
                              xlnkData.setDataTimeStamp(lnk.dOutput.getDataTimeStamp());
                            }                          
                            xlnk.linkStatus = lnk.linkStatus;
                            xlnk.isInsideCallback = true;
                            if (xlnk.tcb != null)
                              xlnk.tcb.callback(xlnk.callbackId,xlnk.linkStatus);
                            else if (xlnk.tlcb != null)
                              xlnk.tlcb.callback(xlnk);
                            xlnk.isInsideCallback = false;
                          }
                        }
                        isInsideCallback = false;
                      }
                      else if (lnk.tlcb != null)
                      { 
                        isInsideCallback = true;
                        if (!lnk.isCancelledWithDependencies())
                        {
                          lnk.isInsideCallback = true;
                          lnk.tlcb.callback(lnk);
                          lnk.isInsideCallback = false;
                        }
                        if (lnk.hasDependencies())
                        { // chained dependencies ?
                          TLink xlnk;
                          TDataType xlnkData;
                          LinkedList xlst = lnk.getDependencies();
                          for (int k=0; k < xlst.size(); k++)
                          {
                            xlnk = (TLink)xlst.get(k);
                            if ((xlnkData=xlnk.getOutputDataObject()) != null)
                            {
                              xlnkData.dataCopy(lnk.dOutput);
                              xlnkData.setDataTimeStamp(lnk.dOutput.getDataTimeStamp());
                            }                          
                            xlnk.isInsideCallback = true;
                            xlnk.linkStatus = lnk.linkStatus;
                            if (xlnk.tcb != null)
                              xlnk.tcb.callback(xlnk.callbackId,xlnk.linkStatus);
                            else if (xlnk.tlcb != null)
                              xlnk.tlcb.callback(xlnk);
                            xlnk.isInsideCallback = false;
                          }
                        }
                        isInsideCallback = false;
                      }
                      lnk.needsNotification = false;
                    }
                    else
                    {
                      lnk.sub.mode = TMode.CM_CANCEL;
                    }
                    // wake up the calling thread
                    lnk.notifyAll(); 
                  }
                  catch (Exception e)
                  {
                    if (debugLevel > 0)
                    {
                      System.out.println("Exeception in link callback");
                      e.printStackTrace();
                    }
                  }
                }
              }
            }
            if ((lnk.devAccess & TAccess.CA_SYNC) != 0 && TMode.getBaseMode(lnk.sub.mode) == TMode.CM_SINGLE)
            {
              if (debugLevel > 0) System.out.println("Thread " + this + " cancel single link " + lnk.linkId);
              if ((lnk.sub.mode & TMode.CM_CONNECT) == TMode.CM_CONNECT)
              {
                lnk.sub.mode = (short) (TMode.CM_CANCEL | TMode.CM_CONNECT);
              }
              else
              {
                lnk.sub.mode = TMode.CM_CANCEL; // turn off the link
                lnk.active = false;
                // don't cause link removal here !
                if (lnk.removeOnClose) 
                {
                  if (debugLevel > 0) System.out.println("mark link " + lnk.linkId + " for termination");
                  lnk.terminate = true; // let the watchdog thread flush the table
                }
              }
            }
            if (lnk.sub.mode == TMode.CM_SINGLE) 
            { // async single link (not connected)
              if (debugLevel > 0) System.out.println("de-activate link " + lnk.linkId);
              lnk.active = false;
            }
          }
          // need to re-establish the link ?
          // reset counter if < 10 !!!!
          if (debugLevel > 1)
            System.out.println("Thread " + this + " Link counter: " + lnk.linkCounter + " at " + ltime);
          if (TMode.getBaseMode(lnk.sub.mode) > TMode.CM_SINGLE && lnk.linkCounter < 10)
          { // a static link !
            if (debugLevel > 0) System.out.println("Thread " + this + " renewing link ...");
            establishLink(lnk);
          }
        }
      }
    }
    return 0;
  }
  private int InterpretIncomingGlobalsData(byte[] data, int length, boolean deferCallbacks)
  {
    TLink glb;
    synchronized (ghs)
    {
      ghs.toStruct(data, length);
    }
    if (debugLevel > 0) System.out.println("glb recv " + length + " bytes");
    for (int i = 0; i < numberTLinksInTable; i++)
    {
      if (linkTable[i] != null && linkTable[i] != tNullLink)
      {
        glb = linkTable[i];
        if (glb.sub.mode == TMode.CM_GLOBAL)
        {
          glb.sub.linkLastTime = System.currentTimeMillis();
          glb.pending = false;
          if (debugLevel > 0) System.out.println("call getData for format " + glb.dOutput.getFormat());
          if (glb.linkStatus >= 0 && glb.pending == false)
          {
            glb.pending = true;
            glb.dOutput.timestamp = time;
            glb.linkTimeouts = 0; // reset timeout counter
            // if caller "cancel()s" then terminate is set to true (should be
            // polite and tell the server).
            // if the link automatically terminates (due to SINGLE -> CANCEL,
            // etc.
            // then the TMode is set to CM_CANCEL (don't need to tell the
            // server)
            // set notification flag:
            glb.needsNotification = true;
            synchronized (glb)
            {
              try
              {
                if (glb.terminate == false)
                {
                  if (glb.tcb != null)
                  {
                    isInsideCallback = true;
                    glb.isInsideCallback = true;
                    glb.tcb.callback(glb.callbackId, glb.linkStatus);
                    glb.isInsideCallback = false;
                    isInsideCallback = false;
                  }
                  else if (glb.tlcb != null)
                  {
                    isInsideCallback = true;
                    glb.isInsideCallback = true;
                    glb.tlcb.callback(glb);
                    glb.isInsideCallback = false;
                    isInsideCallback = false;
                  }
                  glb.needsNotification = false;
                }
                glb.notifyAll(); // wake up the calling thread
              }
              catch (Exception e)
              {
                System.out.println("Execption in global link callback");
                e.printStackTrace();
              }
            }
          }
        }
      }
    }
    return 0;
  }
  public class TFactoryThread extends Thread
  {
    TPacket tp;
    boolean isWaiting = false;
    TFactoryThread(TPacket tPacket)
    {
      tp = tPacket;
      String s = null;
      if (tp.s != null) s = new String("datagram port " + tp.s.getLocalPort());
      else if (tp.m != null) s = new String("multicast port " + tp.s.getLocalPort());
      else s = new String("unbound socket ?");
      this.setName("Link Factory " + s);
    }
    public synchronized void run()
    {
      if (debugLevel > 0) System.out.println("Factory Thread " + getName() + " started ...");
      while (active)
      {
        if (terminate) break;
        if (debugLevel > 0) System.out.println("Waiting for data ...");
        try
        {
          isWaiting = true;
          tp.dpIn.setLength(1472); // reset the receive buffer length
          if (tp.s != null)
          {
            tp.s.receive(tp.dpIn);
          }
          else if (tp.m != null)
          {
            tp.m.receive(tp.dpIn);
          }
          totalConnectionArrivals++;
          isWaiting = false;
          // got it, now interpret it ...
          if (debugLevel > 0) System.out.println("examine " + tp.dpIn.getLength() + " bytes");
          InterpretIncomingData(tp.dpIn.getData(), tp.dpIn.getLength(), tp.dpIn.getAddress(),false);
        }
        catch (Exception e)
        {
          if (debugLevel > 1) e.printStackTrace();
        }
      }
    }
  }
  public class TFactoryGlobalsThread extends Thread
  {
    TPacket tp;
    boolean isWaiting = false;
    TFactoryGlobalsThread(TPacket tPacket)
    {
      tp = tPacket;
      String s = null;
      if (tp.s != null) s = new String("globals port " + tp.s.getPort());
      else s = new String("globals unbound socket ?");
      this.setName("Link Factory " + s);
    }
    public synchronized void run()
    {
      if (debugLevel > 0) System.out.println("Globals Thread started ...");
      while (active)
      {
        if (terminate) break;
        if (debugLevel > 0) System.out.println("Waiting for globals data ...");
        try
        {
          isWaiting = true;
          tp.dpIn.setLength(1472); // reset the receive buffer length
          tp.m.receive(tp.dpIn);
          isWaiting = false;
          // got it, now interpret it ...
          InterpretIncomingGlobalsData(tp.dpIn.getData(), tp.dpIn.getLength(), false);
        }
        catch (Exception e)
        {
          if (debugLevel > 1) e.printStackTrace();
        }
      }
    }
  }
  /**
   * TLinkFactory constructor comment.
   */
  private TLinkFactory()
  {
    initFactory();
  }
  private static boolean factoryHasInitialzed = false;
  private void initFactory()
  {
    if (factoryHasInitialzed) return;
    if (debugLevel > 0) System.out.println("TLink factory initializing ...");
    atp = new TPacket();
    stp = new TPacket();
    qtp = new TPacket();
    nmtp = new TPacket();
    phs = new TPublishedData(this);
    ghs = new TGlobalsData(this);
    stfThrd = new TFactoryThread(stp);
    qtfThrd = new TFactoryThread(qtp);
    atfThrd = new TFactoryThread(atp);
    gtfThrd = null;
    active = true;
    stfThrd.start();
    qtfThrd.start();
    atfThrd.start();
    tfwdThrd.start();
    String dbglvlStr;
    if ((dbglvlStr = System.getProperty("debug.level")) != null)
    {
      debugLevel = Integer.parseInt(dbglvlStr);
    }
    String env = System.getProperty("tine.transport");
    if (env == null) env = System.getenv("TINE_TRANSPORT");
    if (env != null)
    {
      if (env.substring(0,3).compareToIgnoreCase("TCP") == 0)
      {
        useConnectedSockets = true;
      }
    }
    factoryHasInitialzed = true;
  }
  /**
   * cancel() terminates an active link
   */
  public synchronized int cancel(TLink lnk)
  {
    if (lnk == null || lnk.sub == null) return 0; // never allocated
    TLink eqm = lnk;
    if (lnk.isBound())
    { // this one is just bound to another link (don't cancel for real)
      eqm = lnk.getBoundLink();
      eqm.rmvDependency(lnk);
      if (!eqm.hasDependencies())
      { // that was the last dependency, was the link cancelled with dependencies ?
        if (!eqm.isCancelledWithDependencies()) return 0; // still active
      }
      else
      {
        return 0;
      }
    }
    if (eqm.hasDependencies())
    { // primary link cancelled, but there are still some hangers on ...
      eqm.cancelledWithDependencies = true;
      return 0;
    }
    if (eqm.isWildcardLink && eqm.twcl != null)
    { // closing a wildcard link
      TWildcardLink wc = eqm.twcl;
      eqm.twcl = null;
      wc.list = null;
      if (wc.length > 0)
      { // not a single server link
        for (int i=0; i<wc.length; i++)
        { // this will re-entry this routine -> null out the twcl reference first!
          wc.links[i].close();
        }
        wc.length = 0;
        eqm.active = false;
        eqm.terminate = true;
        eqm.pending = false;
        return 0;
      }
    }
    if (TMode.getBaseMode(eqm.sub.mode) > TMode.CM_SINGLE)
    {
      if (debugLevel > 0)
        System.out.println("Thread " + this + " cancel link " + eqm.linkId + " mode "
            + TMode.toString(eqm.sub.mode));
      if ((eqm.sub.mode & TMode.CM_CONNECT) == TMode.CM_CONNECT)
      {
        eqm.sub.mode = TMode.CM_CANCEL | TMode.CM_CONNECT;
      }
      else
      {
        eqm.sub.mode = TMode.CM_CANCEL; // turn off the link
      }
      establishLink(eqm);
    }
    eqm.active = false;
    if (eqm.linkId == 0) eqm.removeOnClose = false; // just in case
    else eqm.terminate = true;
    eqm.pending = false;
    return 0;
  }
  TLink getExistingLink(String devname, String devproperty, TDataType dout, TDataType din, short devaccess)
  {
    int i;
    for (i = 1; i < numberTLinksInTable; i++)
    {
      if (linkTable[i] == null) continue;
      if (linkTable[i] == tNullLink) continue;
      if (!linkTable[i].active) continue;
      if (linkTable[i].getFullDeviceName().compareTo(devname) != 0) continue;
      if (linkTable[i].getProperty().compareTo(devproperty) != 0) continue;
      if (linkTable[i].devAccess != devaccess) continue;
      if (!linkTable[i].getOutputDataObject().equals(false,dout)) continue;
      if (!linkTable[i].getInputDataObject().equals(true,din)) continue;
      return linkTable[i];
    }
    return null;
  }
  /**
   * Put a new TLink in the link table.  
   * Calling this method directly bypasses various system
   * checks for cdi hooks, duplicate links, etc.  
   * A direct call is generally used internally for address queries from the ENS. 
   *
   * @return de.desy.tine.client.TLink
   */
  public TLink createTLink(String devname, String devproperty, TDataType dout, TDataType din, short devaccess)
  {
    int i;
    if (devname.startsWith("ENS"))
    { // given without context => internal queries
      i = 0;
      adjustLinkTable(new TLink(this, 0, devname, devproperty, dout, din, devaccess),adjustLinkTableAdd);
    }
    else
    { // add a place holder in the link table
      //if ((i=adjustLinkTable(tNullLink,adjustLinkTableAdd)) < 0) return null;
      if ((i=adjustLinkTable(new TLink(this, 0, devname, devproperty, dout, din, devaccess),adjustLinkTableAdd)) < 0)
         return null;
    }
    // re-assign the place-holder ...
    //adjustLinkTable(new TLink(this, i, devname, devproperty, dout, din, devaccess),adjustLinkTableReplace);
    if (debugLevel > 0) System.out.println("creating TLink " + linkTable[i] + " (" + i + ")");
    if (debugLevel > 0) linkTable[i].setDebugLevel(debugLevel);
    if (i == 0) linkTable[0].removeOnClose = false; // here it is again ! 
    return linkTable[i];
  }
  private boolean isQueryLink(TLink eqm)
  {
    if (eqm == null) return false;
    if (eqm.expName.startsWith("ENS") || eqm.expName.startsWith("GENS"))
      return true;
    if (eqm.devProperty.indexOf("PROPS") >= 0 || 
        eqm.devProperty.indexOf("PROPERTIES") >= 0)
      return true;
    if (eqm.devProperty.indexOf("DEVICES") >= 0)
      return true;
    return false;
  }
  synchronized int establishLink(TLink lnk)
  {
    if (lnk == null || lnk == tNullLink) 
    {
      lnk.linkStatus = TErrorList.non_existent;
      return TErrorList.non_existent;
    }
    if (lnk.delayEstablishLink) return 0;
    boolean isSynchronous = ((lnk.devAccess & TAccess.CA_SYNC) != 0);
    boolean isQuery = isQueryLink(lnk);
    boolean isServiceRequest = ((lnk.sub.contract.EqpName.compareTo(TModAddress.SRVEQM_NAME)) == 0);
    if (lnk.linkId < 0)
    {
      lnk.linkStatus = TErrorList.out_of_client_memory;
      return TErrorList.out_of_client_memory;
    }
    if (lnk.active && lnk.linkStatus == -1) return 0; // this link is already being established
    lnk.linkStatus = -1; // pending
    // TODO:  eqm.active is set all over the place. Is it necessary here?
    //lnk.active = true;
    if ((lnk.sub.mode & TMode.CM_GLOBAL) == TMode.CM_GLOBAL)
    {
      if (gtfThrd == null)
      {
        gtp = new TPacket(initializer.getGCastPort());
        gtfThrd = new TFactoryGlobalsThread(gtp);
        gtfThrd.start();
      }
      if (lnk.dOutput == null) return -1; 
      lnk.dOutput.blksin = 0;
      lnk.dOutput.timestamp = time;
      lnk.active = true;
      return 0;
    }
    if ((lnk.sub.mode & TMode.CM_MCAST) == TMode.CM_MCAST)
    {
      if (mtfThrd == null)
      {
        amtp = new TPacket(initializer.getMCastPort());
        mtfThrd = new TFactoryThread(amtp);
        mtfThrd.start();
      }
    }
    if (useConnectedSockets) lnk.sub.mode |= TMode.CM_CONNECT;
//    if (isServiceRequest)
//    {
//      if (ntfThrd == null)
//      {
//        nmtp = new TPacket(initializer.getNetCastPort());
//        ntfThrd = new TFactoryThread(nmtp);
//        ntfThrd.start();
//      }
//    }
    byte[] hdrbytes, subbytes;
    for (int n = 0; n < lnk.dInput.numblks; n++)
    {
      subbytes = lnk.sub.toByteArray();
      lnk.hdr.setMsgSizeInBytes((short)(TContract.hdrSizeInBytes + TSubscription.hdrSizeInBytes + lnk.sub.nextDataSizeInBytes));
      hdrbytes = lnk.hdr.toByteArray();
      if (debugLevel > 0)
        System.out.println("trying to establish link " + lnk.linkId + "(" + (n + 1) + " of "
            + lnk.dInput.numblks + " " + lnk.hdr.getMsgSizeInBytes()+ " bytes)" + " : " + lnk.expName + " "
            + lnk.devName + " " + lnk.devProperty + " " + lnk.devTimeout + " msec");
      try
      {
        TSrvAddress srv = lnk.modAddress.srvAddress;
        ByteArrayOutputStream bs = new ByteArrayOutputStream(1472);
        DataOutputStream ds = new DataOutputStream(bs);
        ds.write(hdrbytes);
        ds.write(subbytes);
        if (srv == null)
        {
          if (debugLevel > 0) System.out.println("establish link : can't resolve server address");
          lnk.active = false;
          return TErrorList.non_existent_fec;
        }
        if (lnk.sub.contract.dBuffer == null)
        {
          if (debugLevel > 0) System.out.println("establish link : no address data returned");
          lnk.active = false;
          return TErrorList.non_existent_elem;
        }
        if (lnk.sub.contract.EqpName.length() == 0 || lnk.sub.msgSizeInBytes == 0)
        {
          if (debugLevel > 0) System.out.println("establish link : header corrupt");
          lnk.active = false;
          return TErrorList.code_failure;
        }
        if (debugLevel > 0)
          System.out.println("send to : " + lnk.modAddress.srvAddress.srvHost.getHostAddress() + ":"
              + lnk.modAddress.srvAddress.srvPortOffset);
        // ds.write(eqm.sub.contract.dBuffer.toByteArray());
        ds.write(lnk.sub.contract.toByteArray());
        // package the input data payload ...
        if (lnk.dInput.getArrayLength() > 0) ds.write(lnk.dInput.getDataBuffer(n));
        // send it out ...
        if (isInsideCallback && TMode.getBaseMode(lnk.sub.mode) == TMode.CM_SINGLE)
          lnk.sub.mode &= ~(TMode.CM_CONNECT);
        if ((lnk.sub.mode & TMode.CM_CONNECT) == TMode.CM_CONNECT)
        {
          if (lnk.tb == null) lnk.tb = new TLinkBucket(lnk.linkId);
          if (lnk.tb.isDeactivating) return TErrorList.tcp_connect_error;
          if (lnk.tb.getSocket() != null)
          {
            OutputStream os = lnk.tb.getOutputStream();
            os.write(bs.toByteArray());
            os.flush();
          }
        }
        else if (isSynchronous && isQuery)
        {
          qtp.dpOut = new DatagramPacket(bs.toByteArray(), bs.size(), srv.srvHost, initializer.getSrvPort()
              + srv.srvPortOffset);
          qtp.s.send(qtp.dpOut);          
        }
        else if (isSynchronous)
        {
          if (isServiceRequest)
          {
            stp.dpOut = new DatagramPacket(bs.toByteArray(), bs.size(), srv.srvHost, initializer.getNetCastPort());            
          }
          else
          {
            stp.dpOut = new DatagramPacket(bs.toByteArray(), bs.size(), srv.srvHost, initializer.getSrvPort()
                + srv.srvPortOffset);
          }
          stp.s.send(stp.dpOut);
        }
        else
        {
          atp.dpOut = new DatagramPacket(bs.toByteArray(), bs.size(), srv.srvHost, initializer.getSrvPort()
              + srv.srvPortOffset);
          atp.s.send(atp.dpOut);
        }
        lnk.dOutput.blksin = 0;
        lnk.dOutput.timestamp = time;
      }
      catch (IOException e)
      {
        e.printStackTrace();
        if (debugLevel > 0) System.out.println("establish link : IO Exception");
        //if (eqm.tb != null) removeBucketThread(eqm.tb);
        //eqm.active = false;
        return TErrorList.net_read_error;
      }
    }
    if (debugLevel > 0) System.out.println("establish link : success");
    lnk.active = true;
    return 0;
  }
  public TLink findTLink(int linkid, int linkstarttime)
  {
    if (linkstarttime == TSrvAddress.BCAST_ID) /* BCAST or MCAST contract */
    {
      for (int i = 0; i < numberTLinksInTable; i++) /* cross reference */
      {
        if (linkTable[i] == null || linkTable[i] == tNullLink) continue;
        if (linkTable[i].srvId != linkid) continue;
        // what to do here?????
        // ia = linkTable[i].tf.amtp.m.getInetAddress();
        // if (!linkTable[i].modAddress.srvAddress.srvHost.equals(ia)) continue;
        return linkTable[i];
      }
      return null;
    }
    if (linkstarttime == TSrvAddress.ENS_ID)
    {
      if (linkTable[0] != null && linkTable[0].sub.id != linkid) return null;
      linkTable[0].removeOnClose = false; // how many times do I have to set this ?
      return linkTable[0];
    }
    if (debugLevel > 0)
    {
      System.out.println("incoming link id " + linkid + " start time " + linkstarttime);
      System.out.println("link table : " + numberTLinksInTable + " registered links");
    }
    if (linkid < 0 || linkid >= numberTLinksInTable) return null;
    if (linkTable[linkid] == null || linkTable[linkid].sub == null) return null;
    if (debugLevel > 0) System.out.println("link table id " + linkid + 
        " : start time " + linkTable[linkid].sub.starttime);
    if (linkTable[linkid].sub.starttime != linkstarttime) return null;
    if (debugLevel > 0) System.out.println("returning " + linkTable[linkid]);
    return linkTable[linkid];
  }
  public TLink findTLink(String keyword)
  {
    keyword = keyword.trim();
    if (debugLevel > 0)
      System.out.println("looking for link " + keyword + " amoung " + numberTLinksInTable
          + " registered links");
    for (int i = 0; i < numberTLinksInTable; i++)
    {
      if (linkTable[i] == null || linkTable[i] == tNullLink) continue;
      if (linkTable[i].sub.mode != TMode.CM_GLOBAL)
      {
        if (debugLevel > 0) System.out.println(linkTable[i].devProperty + " is not a global");
        continue;
      }
      if (debugLevel > 0) System.out.println("Compare " + keyword + " against " + linkTable[i].devProperty);
      if (keyword.compareTo(linkTable[i].devProperty) == 0) // found it
      {
        if (debugLevel > 0) System.out.println("returning " + linkTable[i]);
        return linkTable[i];
      }
    }
    return null;
  }
  /**
   * Insert the method's description here. Creation date: (8/24/01 3:51:37 PM)
   * 
   * @return de.desy.tine.client.TLinkFactory
   */
  public static TLinkFactory getInstance()
  {
    if (instance == null) instance = new TLinkFactory();
    return instance;
  }
  /**
   * Put a new TLink in the link table Creation date: (8/17/01 5:59:27 PM)
   * 
   * @return de.desy.tine.client.TLink
   */
  public void linkReady(TLink link)
  {
    linkTable[link.linkId] = link;
    if (debugLevel > 0)
      System.out.println("creating TLink " + linkTable[link.linkId] + " (" + link.linkId + ")");
  }
  /**
   * Put a new TLink in the link table Creation date: (8/17/01 5:59:27 PM)
   * 
   * @return de.desy.tine.client.TLink
   */
  public int registerLink(TLink link)
  {
    int i;
    if (link.expName.startsWith("ENS") && 
        (link.cntName.startsWith("DEFAULT") || link.cntName.length() == 0))
    {
      i = adjustLinkTable(link,adjustLinkTableAdd);
    }
    else
    {
      i = adjustLinkTable(tNullLink,adjustLinkTableAdd);
    }
    return i;
  }
  /**
   * \brief Remove a TLink from the link table Removes a TLink from the link
   * table by setting the entry explicitly to null. If the link is a TCP/IP link
   * then the termination flag is reset to false in order to give the
   * terminateBucket thread a chance to clean up the connection.
   */
  private void removeTLink(TLink lnk)
  {
    adjustLinkTable(lnk, adjustLinkTableRemove);
  }
  /**
   * Insert the method's description here. Creation date: (8/23/01 10:51:32 AM)
   */
  public void shutdown()
  {
    active = false;
  }
  public int setDebugLevel(int level)
  {
    debugLevel = level;
    return debugLevel;
  }
  public int getDebugLevel()
  {
    return debugLevel;
  }
  public static int setOutputDebugLevel(int level)
  {
    if (instance == null) return 0;
    instance.debugLevel = level;
    return instance.debugLevel;
  }
  public int getOutputDebugLevel()
  {
    if (instance == null) return 0;
    return instance.debugLevel;
  }
}
