package de.desy.tine.definitions;

/**
 * @brief tine data format types
 */
public final class TFormat
{
  public static final short CF_DOUBLE = 0;
  public static final short CF_SHORT = 1;
  public static final short CF_BYTE = 2;
  public static final short CF_LONG = 3;
  public static final short CF_TEXT = 4;
  public static final short CF_FLOAT = 5;
  public static final short CF_STRUCT = 7;
  public static final short CF_NAME8 = 8;
  public static final short CF_NAME16 = 9;
  public static final short CF_NAME16FI = 10;
  public static final short CF_NAMEFI = 10;
  public static final short CF_BIT = 11;
  public static final short CF_HITTS = 12;
  public static final short CF_NAME32 = 13;
  public static final short CF_FLTINT = 14;
  public static final short CF_LNGINT = 15;
  public static final short CF_INTINT = 15;
  public static final short CF_DBLINT = 16;
  public static final short CF_SHTINT = 17;
  public static final short CF_INTFLTINT = 18;
  public static final short CF_TDS = 18;
  public static final short CF_NAME48 = 19;
  public static final short CF_NAME8I = 20;
  public static final short CF_NAME16I = 21;
  public static final short CF_NAME32I = 22;
  public static final short CF_NAME48I = 23;
  public static final short CF_USTRING = 24;
  public static final short CF_FLTFLTINT = 25;
  public static final short CF_XYS = 25;
  public static final short CF_FLTINTINT = 26;
  public static final short CF_FIS = 26;
  public static final short CF_FILTER = 27;
  public static final short CF_INTFLTFLTFLT = 27;
  public static final short CF_IFFF = 27;
  public static final short CF_SPECTRUM = 28;
  public static final short CF_ADDRESS = 29;
  public static final short CF_IIII = 29;
  public static final short CF_INTINTINTINT = 29;
  public static final short CF_WINDOW = 30;
  public static final short CF_TTII = 30;
  public static final short CF_POINT = 31;
  public static final short CF_FF = 31;
  public static final short CF_XY = 31;
  public static final short CF_FLTFLT = 31;
  public static final short CF_TDSI = 32;
  public static final short CF_INTFLTSHTSHT = 32;
  public static final short CF_BOOLEAN = 33;
  public static final short CF_IIFF = 34;
  public static final short CF_INTINTFLTFLT = 34;
  public static final short CF_FWINDOW = 34;
  public static final short CF_FIFI = 35;
  public static final short CF_FLTINTFLTINT = 35;
  public static final short CF_NAME64 = 36;
  public static final short CF_NAME64I = 37;
  public static final short CF_INTDBL = 38;
  public static final short CF_FLTDBL = 39;
  public static final short CF_DBLDBL = 40;
  public static final short CF_NAMEDBLDBL = 41;
  public static final short CF_NAME16DBLDBL = 41;
  public static final short CF_NAME16DD = 41;
  public static final short CF_NAMEII = 42;
  public static final short CF_NAME16II = 42;
  public static final short CF_III = 43;
  public static final short CF_INTINTINT = 43;
  public static final short CF_NAME32DBLDBL = 44;
  public static final short CF_NAME64DBLDBL = 45;
  public static final short CF_UNAME = 46;
  public static final short CF_IFFFNAME = 46;
  public static final short CF_XML = 47;
  public static final short CF_DEFAULT = 254;
  public static final short CF_NULL = 255;
  public static final String[] availableFormats = {"NULL", "BYTE", "BOOLEAN","SHORT", "LONG", "FLOAT", "DOUBLE",
      "STRUCT", "TEXT", "XML", "NAME8", "NAME16", "NAME32", "NAME48", "NAME64", "INTINT", "FLTINT", "DBLDBL",
      "NAME8I", "NAME16I", "NAME32I", "NAME48I", "NAME64I", "INTFLTINT", "INTINTINT", "USTRING", "UNAME",
      "SPECTRUM", "WINDOW", "FWINDOW", "ADDRESS", "NAME16FI", "NAME16II", "NAME16DBLDBL", "NAME32DBLDBL",
      "NAME64DBLDBL", "FILTER", "FLTFLTINT"};
  public static boolean isSimpleFormat(short dataFormat)
  {
    if (dataFormat <= CF_NAME16) return true;
    if (dataFormat == CF_NAME32) return true;
    if (dataFormat == CF_NAME48) return true;
    if (dataFormat == CF_NAME64) return true;
    return false;
  }
  public static boolean formatsCanMap(short dataFormat1, short dataFormat2,boolean input)
  {
    if (dataFormat1 == dataFormat2) return true;
    switch (dataFormat1)
    {
      case CF_LONG:
      case CF_SHORT:
      case CF_FLOAT:
      case CF_DOUBLE:
        if (dataFormat2 == CF_FLOAT || dataFormat2 == CF_DOUBLE || dataFormat2 == CF_SHORT
            || dataFormat2 == CF_LONG) return true;
        break;
      case CF_DBLDBL:
      case CF_LNGINT:
      case CF_FLTINT:
        if (input) break; // compound input formats must match
        if (dataFormat2 == CF_LNGINT || dataFormat2 == CF_FLTINT || dataFormat2 == CF_DBLDBL) return true;
        if (dataFormat2 == CF_FLOAT || dataFormat2 == CF_DOUBLE || dataFormat2 == CF_LONG) return true;
        break;
      case CF_TEXT:
      case CF_NAME64:
      case CF_NAME48:
      case CF_NAME32:
      case CF_NAME16:
      case CF_NAME8:
        if (dataFormat2 == CF_NAME8 || dataFormat2 == CF_NAME16 || dataFormat2 == CF_NAME32
            || dataFormat2 == CF_NAME48 || dataFormat2 == CF_NAME64 || dataFormat2 == CF_TEXT) return true;
        break;
      case CF_NAME64I:
      case CF_NAME48I:
      case CF_NAME32I:
      case CF_NAME16I:
        if (input) break; // compound input formats must match
        if (dataFormat2 == CF_NAME16I || dataFormat2 == CF_NAME32I || dataFormat2 == CF_NAME48I
            || dataFormat2 == CF_NAME64I) return true;
        break;
      case CF_NAME16DBLDBL:
      case CF_NAME32DBLDBL:
      case CF_NAME64DBLDBL:
        if (input) break; // compound input formats must match
        if (dataFormat2 == CF_NAME16DBLDBL || dataFormat2 == CF_NAME32DBLDBL
            || dataFormat2 == CF_NAME64DBLDBL) return true;
    }
    return false;
  }
  /**
   * @brief The size in bytes of the specified data format
   * @param dataFormat
   *          is the tine data type for which the size in bytes is desired
   * @return the size in bytes of the specified data format
   */
  public static int formatSizeOf(short dataFormat)
  {
    short df = dataFormat >= 0 ? dataFormat : (short) (256 + dataFormat);
    if (df >= 512) df -= 512;
    switch (df)
    {
      case TFormat.CF_NULL:
        return 0;
      case TFormat.CF_BIT:
      case TFormat.CF_STRUCT:
      case TFormat.CF_TEXT:
      case TFormat.CF_XML:
      case TFormat.CF_BYTE:
        return 1;
      case TFormat.CF_SHORT:
        return 2;
      case TFormat.CF_BOOLEAN:
      case TFormat.CF_LONG:
      case TFormat.CF_FLOAT:
        return 4;
      case TFormat.CF_DOUBLE:
        return 8;
      case TFormat.CF_NAME8:
        return 8;
      case TFormat.CF_NAME16:
        return 16;
      case TFormat.CF_NAME32:
        return 32;
      case TFormat.CF_NAME48:
        return 48;
      case TFormat.CF_NAME64:
        return 64;
      case TFormat.CF_FLTINT:
      case TFormat.CF_INTINT:
      case TFormat.CF_FLTFLT:
        return 8;
      case TFormat.CF_DBLINT: //TODO: get rid of this format
        return 12;
      case TFormat.CF_FLTFLTINT:
      case TFormat.CF_FLTINTINT:
      case TFormat.CF_INTINTINT:
        return 12;
      case TFormat.CF_FIFI:
      case TFormat.CF_ADDRESS:
      case TFormat.CF_IFFF:
      case TFormat.CF_FWINDOW:
        return 16;
      case TFormat.CF_NAME16II:
        return 24;
      case TFormat.CF_NAME16FI:
        return 24;
      case TFormat.CF_NAME16I:
        return 20;
      case TFormat.CF_NAME32I:
        return 36;
      case TFormat.CF_NAME48I:
        return 52;
      case TFormat.CF_NAME64I:
        return 68;
      case TFormat.CF_NAME16DBLDBL:
        return 32;
      case TFormat.CF_NAME32DBLDBL:
        return 48;
      case TFormat.CF_NAME64DBLDBL:
        return 80;
      case TFormat.CF_INTDBL://TODO: get rid of this format
        return 12;
      case TFormat.CF_FLTDBL://TODO: get rid of this format
        return 12;
      case TFormat.CF_DBLDBL:
        return 16;
      case TFormat.CF_SPECTRUM:
        return 4;
      case TFormat.CF_TDS:
        return 12;
      case TFormat.CF_TDSI:
        return 16;
      case TFormat.CF_USTRING:
        return 96;
      case TFormat.CF_UNAME:
        return 32;
      default:
        return -1;
    }
  }
  /**
   * @brief The string representation of the specified data format
   * @param dataFormat
   *          is the tine data type for which the string representation is
   *          desired
   * @return The string representation of the specified data format
   */
  public static String toString(short dataFormat)
  {
    short df = dataFormat >= 0 ? dataFormat : (short) (256 + dataFormat);
    if (df >= 512) df -= 512;
    switch (df)
    {
      case TFormat.CF_NULL:
        return "null";
      case TFormat.CF_BIT:
        return "bit";
      case TFormat.CF_STRUCT:
        return "struct";
      case TFormat.CF_TEXT:
        return "char";
      case TFormat.CF_BYTE:
        return "byte";
      case TFormat.CF_SHORT:
        return "short";
      case TFormat.CF_LONG:
        return "int";
      case TFormat.CF_FLOAT:
        return "float";
      case TFormat.CF_DOUBLE:
        return "double";
      case TFormat.CF_NAME8:
        return "NAME8";
      case TFormat.CF_NAME16:
        return "NAME16";
      case TFormat.CF_NAME32:
        return "NAME32";
      case TFormat.CF_NAME48:
        return "NAME48";
      case TFormat.CF_NAME64:
        return "NAME64";
      case TFormat.CF_FLTINT:
        return "FLTINT";
      case TFormat.CF_LNGINT:
        return "LNGINT";
      case TFormat.CF_DBLINT:
        return "DBLINT";
      case TFormat.CF_FLTFLT:
        return "FLTFLT";
      case TFormat.CF_FIFI:
        return "FIFI";
      case TFormat.CF_FLTFLTINT:
        return "FLTFLTINT";
      case TFormat.CF_FLTINTINT:
        return "FLTINTINT";
      case TFormat.CF_ADDRESS:
        return "ADDRESS";
      case TFormat.CF_BOOLEAN:
        return "boolean";
      case TFormat.CF_FWINDOW:
        return "WINDOW";
      case TFormat.CF_NAME16FI:
        return "NAMEFI";
      case TFormat.CF_NAME16II:
        return "NAMEII";
      case TFormat.CF_NAME16I:
        return "NAME16I";
      case TFormat.CF_NAME32I:
        return "NAME32I";
      case TFormat.CF_NAME48I:
        return "NAME48I";
      case TFormat.CF_NAME64I:
        return "NAME64I";
      case TFormat.CF_INTDBL:
        return "INTDBL";
      case TFormat.CF_FLTDBL:
        return "FLTDBL";
      case TFormat.CF_DBLDBL:
        return "DBLDBL";
      case TFormat.CF_SPECTRUM:
        return "SPECTRUM";
      case TFormat.CF_TDS:
        return "TDS";
      case TFormat.CF_TDSI:
        return "TDSI";
      case TFormat.CF_USTRING:
        return "USTRING";
      case TFormat.CF_UNAME:
        return "UNAME";
      case TFormat.CF_IFFF:
        return "IFFF";
      case TFormat.CF_III:
        return "INTINTINT";
      case TFormat.CF_NAME16DBLDBL:
        return "NAME16DBLDBL";
      case TFormat.CF_NAME32DBLDBL:
        return "NAME32DBLDBL";
      case TFormat.CF_NAME64DBLDBL:
        return "NAME64DBLDBL";
      case TFormat.CF_XML:
        return "xml";
      default:
        return "unknown";
    }
  }
  /**
   * @brief the format code of the specified String
   * @param dataFormatString
   *          is the string representation for which the tine data type is
   *          desired
   * @return the format code of the specified String
   */
  public static short getFormatCode(String dataFormatString)
  {
    String dfs = dataFormatString.toUpperCase();
    if (dfs.compareTo("NULL") == 0) return TFormat.CF_NULL;
    if (dfs.startsWith("BIT")) return TFormat.CF_BIT;
    if (dfs.startsWith("STRUCT")) return TFormat.CF_STRUCT;
    if (dfs.startsWith("TEXT")) return TFormat.CF_TEXT;
    if (dfs.startsWith("BYTE")) return TFormat.CF_BYTE;
    if (dfs.startsWith("INT8")) return TFormat.CF_BYTE;
    if (dfs.startsWith("SHORT")) return TFormat.CF_SHORT;
    if (dfs.startsWith("INT16")) return TFormat.CF_SHORT;
    if (dfs.startsWith("LONG")) return TFormat.CF_LONG;
    if (dfs.startsWith("INT32")) return TFormat.CF_LONG;
    if (dfs.startsWith("UINT32")) return TFormat.CF_LONG;
    if (dfs.startsWith("FLOAT")) return TFormat.CF_FLOAT;
    if (dfs.startsWith("DOUBLE")) return TFormat.CF_DOUBLE;
    if (dfs.startsWith("NAME16FI")) return TFormat.CF_NAME16FI;
    if (dfs.startsWith("STRING16FI")) return TFormat.CF_NAME16FI;
    if (dfs.startsWith("CHAR16FI")) return TFormat.CF_NAME16FI;
    if (dfs.startsWith("NAMEFI")) return TFormat.CF_NAME16FI;
    if (dfs.startsWith("NAME16II")) return TFormat.CF_NAME16II;
    if (dfs.startsWith("NAMEII")) return TFormat.CF_NAME16II;
    if (dfs.startsWith("STRING16II")) return TFormat.CF_NAME16II;
    if (dfs.startsWith("CHAR16II")) return TFormat.CF_NAME16II;
    if (dfs.startsWith("NAME16I")) return TFormat.CF_NAME16I;
    if (dfs.startsWith("STRING16I")) return TFormat.CF_NAME16I;
    if (dfs.startsWith("CHAR16I")) return TFormat.CF_NAME16I;
    if (dfs.startsWith("NAME32I")) return TFormat.CF_NAME32I;
    if (dfs.startsWith("STRING32I")) return TFormat.CF_NAME32I;
    if (dfs.startsWith("CHAR32I")) return TFormat.CF_NAME32I;
    if (dfs.startsWith("NAME48I")) return TFormat.CF_NAME48I;
    if (dfs.startsWith("STRING48I")) return TFormat.CF_NAME48I;
    if (dfs.startsWith("CHAR48I")) return TFormat.CF_NAME48I;
    if (dfs.startsWith("NAME64I")) return TFormat.CF_NAME64I;
    if (dfs.startsWith("STRING64I")) return TFormat.CF_NAME64I;
    if (dfs.startsWith("CHAR64I")) return TFormat.CF_NAME64I;
    if (dfs.startsWith("NAMEDBLDBL")) return TFormat.CF_NAME16DBLDBL;
    if (dfs.startsWith("NAME16DBLDBL")) return TFormat.CF_NAME16DBLDBL;
    if (dfs.startsWith("STRING16DBLDBL")) return TFormat.CF_NAME16DBLDBL;
    if (dfs.startsWith("CHAR16DBLDBL")) return TFormat.CF_NAME16DBLDBL;
    if (dfs.startsWith("NAME32DBLDBL")) return TFormat.CF_NAME32DBLDBL;
    if (dfs.startsWith("STRING32DBLDBL")) return TFormat.CF_NAME32DBLDBL;
    if (dfs.startsWith("CHAR32DBLDBL")) return TFormat.CF_NAME32DBLDBL;
    if (dfs.startsWith("NAME64DBLDBL")) return TFormat.CF_NAME64DBLDBL;
    if (dfs.startsWith("STRING64DBLDBL")) return TFormat.CF_NAME64DBLDBL;
    if (dfs.startsWith("CHAR64DBLDBL")) return TFormat.CF_NAME64DBLDBL;
    if (dfs.startsWith("NAME8")) return TFormat.CF_NAME8;
    if (dfs.startsWith("STRING8")) return TFormat.CF_NAME8;
    if (dfs.startsWith("CHAR8")) return TFormat.CF_NAME8;
    if (dfs.startsWith("NAME16")) return TFormat.CF_NAME16;
    if (dfs.startsWith("STRING16")) return TFormat.CF_NAME16;
    if (dfs.startsWith("CHAR16")) return TFormat.CF_NAME16;
    if (dfs.startsWith("NAME32")) return TFormat.CF_NAME32;
    if (dfs.startsWith("STRING32")) return TFormat.CF_NAME32;
    if (dfs.startsWith("CHAR32")) return TFormat.CF_NAME32;
    if (dfs.startsWith("NAME48")) return TFormat.CF_NAME48;
    if (dfs.startsWith("STRING48")) return TFormat.CF_NAME48;
    if (dfs.startsWith("CHAR48")) return TFormat.CF_NAME48;
    if (dfs.startsWith("NAME64")) return TFormat.CF_NAME64;
    if (dfs.startsWith("STRING64")) return TFormat.CF_NAME64;
    if (dfs.startsWith("CHAR64")) return TFormat.CF_NAME64;
    if (dfs.startsWith("FLTFLTINT")) return TFormat.CF_FLTFLTINT;
    if (dfs.startsWith("FLTFLT")) return TFormat.CF_FLTFLT;
    if (dfs.startsWith("FIFI")) return TFormat.CF_FIFI;
    if (dfs.startsWith("FLTINTFLTINT")) return TFormat.CF_FIFI;
    if (dfs.startsWith("FFI")) return TFormat.CF_FLTFLTINT;
    if (dfs.startsWith("FLTINTINT")) return TFormat.CF_FLTINTINT;
    if (dfs.startsWith("FII")) return TFormat.CF_FLTINTINT;
    if (dfs.startsWith("ADDRESS")) return TFormat.CF_ADDRESS;
    if (dfs.startsWith("IIII")) return TFormat.CF_ADDRESS;
    if (dfs.startsWith("III")) return TFormat.CF_III;
    if (dfs.startsWith("INTINTINT")) return TFormat.CF_III;
    if (dfs.startsWith("BOOL")) return TFormat.CF_BOOLEAN;
    if (dfs.startsWith("WINDOW")) return TFormat.CF_WINDOW;
    if (dfs.startsWith("FWINDOW")) return TFormat.CF_FWINDOW;
    if (dfs.startsWith("SPECTRUM")) return TFormat.CF_SPECTRUM;
    if (dfs.startsWith("TDSI")) return TFormat.CF_TDSI;
    if (dfs.startsWith("TDS")) return TFormat.CF_TDS;
    if (dfs.startsWith("UNAME")) return TFormat.CF_UNAME;
    if (dfs.startsWith("IFFFNAME")) return TFormat.CF_UNAME;
    if (dfs.startsWith("INTFLTFLTFLTNAME")) return TFormat.CF_UNAME;
    if (dfs.startsWith("INTFLTINTINT")) return TFormat.CF_TDSI;
    if (dfs.startsWith("INTFLTINT")) return TFormat.CF_TDS;
    if (dfs.startsWith("INTFLTFLTFLT")) return TFormat.CF_FILTER;
    if (dfs.startsWith("FILTER")) return TFormat.CF_FILTER;
    if (dfs.startsWith("USTRING")) return TFormat.CF_USTRING;
    if (dfs.startsWith("INTDBL")) return TFormat.CF_INTDBL;
    if (dfs.startsWith("FLTDBL")) return TFormat.CF_FLTDBL;
    if (dfs.startsWith("DBLDBL")) return TFormat.CF_DBLDBL;
    if (dfs.startsWith("FLTINT")) return TFormat.CF_FLTINT;
    if (dfs.startsWith("LNGINT")) return TFormat.CF_LNGINT;
    if (dfs.startsWith("INTINT")) return TFormat.CF_LNGINT;
    if (dfs.startsWith("DBLINT")) return TFormat.CF_DBLINT;
    if (dfs.startsWith("XML")) return TFormat.CF_XML;
    if (dfs.startsWith("CHAR")) return TFormat.CF_TEXT;
    if (dfs.startsWith("STRING")) return TFormat.CF_TEXT;
    if (dfs.startsWith("INT")) return TFormat.CF_LONG;
    if (dfs.startsWith("BOOLEAN")) return TFormat.CF_BOOLEAN;
    return TFormat.CF_NULL;
  }
  public static TFormat valueOf(short val)
  {
    return new TFormat(val);
  }
  private short value;
  /**
   * Creates new instance of TFormat.
   */
  private TFormat(short val)
  {
    value = val;
  }
  /**
   * Returns short value for format enum.
   * 
   * @return short value for format enum
   */
  public short getValue()
  {
    return value;
  }
  /*
   * (non-Javadoc)
   * 
   * @see java.lang.Object#toString()
   */
  @Override
  public String toString()
  {
    return toString(value);
  }
}