//: TLink.java
package de.desy.tine.headers;
import java.io.ByteArrayOutputStream;
import java.io.ByteArrayInputStream;
import java.io.DataOutputStream;
import java.io.DataInputStream;
import java.io.IOException;
import de.desy.tine.addrUtils.TModAddress;
import de.desy.tine.client.TLink;
import de.desy.tine.dataUtils.TDataType;
import de.desy.tine.definitions.TAccess;
import de.desy.tine.definitions.TFormat;
import de.desy.tine.endianUtils.Swap;
import de.desy.tine.types.SPECTRUM;
public final class TContract
{
  public String EqpProperty;   // 32 char
  public String EqpDeviceName; // 16 char
  public String EqpName;       // 8 char
  public int EqpSizeIn;
  public int EqpSizeOut;
  public byte hEqpName;        // as yet un-used
  public byte EqpAccess;
  public byte EqpFormatIn;
  public byte EqpFormatOut;
  public String strTagIn;      // 8 char
  public String strTagOut;     // 8 char
  // non 'network' fields :
  static public final int hdrSizeInBytes = (32+16+8+2*4+4*1+2*8);
  int totalSizeInBytes;
  public ByteArrayOutputStream dBuffer;
  public boolean equals(TContract contract)
  {
    if (contract.EqpProperty.compareTo(EqpProperty) != 0) return false;
    if (contract.EqpDeviceName.compareTo(EqpDeviceName) != 0) return false;
    if (contract.EqpName.compareTo(EqpName) != 0) return false;
    if (contract.EqpSizeIn != EqpSizeIn) return false;
    if (contract.EqpSizeOut != EqpSizeOut) return false;
    if (TAccess.toBase(contract.EqpAccess) != TAccess.toBase(EqpAccess)) return false;
    if (contract.EqpFormatIn != EqpFormatIn) return false;
    if (contract.EqpFormatOut != EqpFormatOut) return false;
    if (contract.strTagIn.compareTo(strTagIn) != 0) return false;
    if (contract.strTagOut.compareTo(strTagOut) != 0) return false;
    return true;
  }
  // local constructor for local histories, alarm watches, etc....
  public TContract(String property,String device,String eqm,TDataType dout,TDataType din,short access) 
  {
    EqpProperty = property;
    EqpDeviceName = device;
    EqpName = eqm;
    if (din != null)
    {
      EqpSizeIn = din.dArrayLength;
      EqpFormatIn = (byte)din.dFormat;
      strTagIn = din.getTag();
    }
    else
    {
      EqpSizeIn = 0;
      EqpFormatIn = (byte)TFormat.CF_NULL;
      strTagIn = "";
    }
    if (dout != null)
    {
      EqpSizeOut = dout.dArrayLength;
      EqpFormatOut = (byte)dout.dFormat;
      strTagOut = dout.getTag();
    }
    else
    {
      EqpSizeOut = 0;
      EqpFormatOut = (byte)TFormat.CF_NULL;
      strTagOut = "";
    }
    EqpAccess = (byte)access;
  }
  public TContract(TLink eqm)  // going out (client constructor)
  {
    int i;
    // get 6-character module name
    TModAddress mod;
    mod = new TModAddress(eqm.tf,eqm.expName,eqm.cntName);
    if (mod.srvAddress == null) return;
    if (eqm.modAddress != null) 
    {
      eqm.modAddress.srvAddress = mod.srvAddress;
      eqm.modAddress.modName = mod.modName;
      // fill in the local fields ...
      EqpName = mod.modName;
      EqpProperty = eqm.devProperty;
      EqpDeviceName = eqm.devName;
      EqpSizeOut = eqm.dOutput.getArrayLength();
      EqpSizeIn = eqm.dInput.getArrayLength();
      EqpFormatOut = (byte)eqm.dOutput.getFormat();
      EqpFormatIn = (byte)eqm.dInput.getFormat();
      EqpAccess = (byte)eqm.devAccess;
      strTagOut = eqm.dOutput.getTag();
      strTagIn = eqm.dInput.getTag();
    }
    try
    {
      dBuffer = new ByteArrayOutputStream(hdrSizeInBytes);
      DataOutputStream ds = new DataOutputStream(dBuffer);
      byte[] bstr = new byte[32];
      byte[] tmp  = eqm.devProperty.getBytes();
      byte[] tmp2  = eqm.devName.getBytes();
      if (tmp2.length > 16 && tmp.length < 16)  // the Channel Access Long device names trick ...
      {
        for (i=0; i<16; i++) bstr[i] =  i < tmp.length ? tmp[i] : 0; bstr[16] = 0;
        for (i=0; i<15; i++) bstr[i+17] = i < tmp2.length - 15 ? tmp2[i+15] : 0;
        byte[] amp = "&".getBytes();
        tmp2[15] = amp[0];
      }
      else
      {
        for (i=0; i<32; i++) bstr[i] =  i < tmp.length ? tmp[i] : 0;
      }
      ds.write(bstr,0,32);
      for (i=0; i<16; i++) bstr[i] = i < tmp2.length ? tmp2[i] : 0;
      ds.write(bstr,0,16);
      tmp = mod.modName.getBytes();
      for (i=0; i<8; i++) bstr[i] = i < tmp.length ? tmp[i] : 0;
      ds.write(bstr,0,8);
      int inlen = eqm.dInput.getArrayLength();
      int outlen = eqm.dOutput.getArrayLength();
      if (eqm.dOutput.getFormat() == TFormat.CF_STRUCT) outlen = eqm.dOutput.getSizeInBytes();
      if (eqm.dInput.getFormat() == TFormat.CF_STRUCT) inlen = eqm.dInput.getSizeInBytes();
      ds.writeInt(Swap.Long(inlen));
      ds.writeInt(Swap.Long(outlen));
      ds.writeByte(0);
      ds.writeByte(eqm.devAccess);
      ds.writeByte((byte)eqm.dInput.getFormat());
      ds.writeByte((byte)eqm.dOutput.getFormat());
      tmp = eqm.dInput.getTag().getBytes();
      for (i=0; i<8; i++) bstr[i] = i < tmp.length ? tmp[i] : 0;
      ds.write(bstr,0,8);
      tmp = eqm.dOutput.getTag().getBytes();
      for (i=0; i<8; i++) bstr[i] = i < tmp.length ? tmp[i] : 0;
      ds.write(bstr,0,8);
      totalSizeInBytes = hdrSizeInBytes + eqm.dInput.getSizeInBytes();
      ds.close();
    }
    catch (IOException e)
    {
      e.printStackTrace();
    }	 
  }
  private void prepareInComing(DataInputStream ds)
  {                        
    byte[] d = new byte[32];
    try
    {
      ds.read(d,0,32);
      EqpProperty = new String(d,0,32).trim();
      ds.read(d,0,16);
      EqpDeviceName = new String(d,0,16).trim();
      ds.read(d,0,8);
      EqpName = new String(d,0,8).trim();
      EqpSizeIn = Swap.Long(ds.readInt());
      EqpSizeOut = Swap.Long(ds.readInt());
      hEqpName = ds.readByte();
      EqpAccess = ds.readByte();
      EqpFormatIn = ds.readByte();
      EqpFormatOut = ds.readByte();
      ds.read(d,0,8);
      strTagIn = new String(d,0,8).trim();
      ds.read(d,0,8);
      strTagOut = new String(d,0,8).trim();
    } 
    catch (IOException e)
    {
      e.printStackTrace();
    }  
  }
  public TContract(byte[] data, int off, int len)
  {
     try
    {
      ByteArrayInputStream is = new ByteArrayInputStream(data,off,len); 
      DataInputStream ds = new DataInputStream(is);
      prepareInComing(ds);
      ds.close();
      is.close();
    } 
    catch (IOException e)
    {
      e.printStackTrace();
    }      
  }
  public TContract(DataInputStream ds) // coming in (server constructor)
  {
    prepareInComing(ds);
  }
  public TContract(ByteArrayInputStream inBuffer) // coming in (server constructor)
  {
    DataInputStream ds = new DataInputStream(inBuffer);
    prepareInComing(ds);
    try
    {
      ds.close();
    } 
    catch (IOException e)
    {
      e.printStackTrace();
    }  
  }
  public int getOutputDataSize()
  {
    int fmtsiz = 0;
    if ((fmtsiz=TFormat.formatSizeOf(EqpFormatOut)) > 0)
    {
      int datasize = fmtsiz * EqpSizeOut;
      if (EqpFormatOut == TFormat.CF_SPECTRUM) datasize += SPECTRUM.hdrSizeInBytes;
      return datasize;
    }
    return 0;
  }
  public String toString()
  {
    String msg = new String("target : <"+EqpName+">/"+EqpDeviceName+"["+EqpProperty+"]");
    msg = msg.concat("" + TAccess.toString(EqpAccess) + " access");
    msg = msg.concat("\ninput  : " + EqpSizeIn + " " + TFormat.toString(EqpFormatIn) + " value(s)");
    if (strTagIn.length() > 0) msg = msg.concat(" tag = " + strTagIn);
    msg = msg.concat("\noutput : " + EqpSizeOut + " " + TFormat.toString(EqpFormatOut) + " value(s)");
    if (strTagOut.length() > 0) msg = msg.concat(" tag = " + strTagOut);
    return msg;
  }
  public byte[] toByteArray() { return dBuffer != null ? dBuffer.toByteArray() : null; }
}