package de.desy.tine.headers;
import java.io.ByteArrayInputStream;
import java.io.DataInputStream;
import java.io.IOException;
import de.desy.tine.client.TLinkFactory;
import de.desy.tine.client.TLink;
import de.desy.tine.dataUtils.TDataType;
import de.desy.tine.definitions.TFormat;
import de.desy.tine.endianUtils.Swap;
import de.desy.tine.types.*;

public class TGlobalsData 
{
  short msgSizeInBytes;
  short mtu;
  int   debugLevel = 0;
  byte[] globalsNameAsByteArray = new byte[16];  // 16 chars
  short  dArrayLength;
  short  dFormat; 
  int   dTimestamp;
  static final int glbhdrSizeInBytes = (16 + 2*2 + 1*4);
  static ByteArrayInputStream dBuffer;
  byte[] usernameAsByteArray = new byte[16];
  private short  protocol;
  private short  numInPacket;
  static final short  hdrSizeInBytes = 2 + 16 + 2 + 2; // network header size
  private short totalSizeInBytes;
  TLinkFactory tf;
 /**
   * Attach a GlobalsData object to the factory
   * Creation date: (8/21/01 6:33:36 PM)
   */
  public TGlobalsData(TLinkFactory tfactory) 
  {
    tf = tfactory;
  }
  public void toStruct(byte[] data, int length)
  {
    TLink glb;
    int bytesread = 0;
    try
    {
      dBuffer = new ByteArrayInputStream(data);
      DataInputStream ds = new DataInputStream(dBuffer);
      // overall message length:
      totalSizeInBytes = Swap.Short(ds.readShort());
      if (totalSizeInBytes < hdrSizeInBytes + glbhdrSizeInBytes)
      {
        String msg = "recv: " + totalSizeInBytes + " bytes : hdr " + hdrSizeInBytes + glbhdrSizeInBytes;
        System.out.println(msg);
        return;
      }
      ds.read(usernameAsByteArray,0,16);
      protocol = Swap.Short(ds.readShort());
      numInPacket = Swap.Short(ds.readShort());
      bytesread = 22;
      if (protocol != 3) return; // is not a globals packet!
      //System.out.println("recv: " + numInPacket + " globals");
      for (int i=0; i<numInPacket && bytesread < totalSizeInBytes; i++)
      {
        ds.read(globalsNameAsByteArray,0,16);        
        dArrayLength = Swap.Short(ds.readShort());
        dFormat = Swap.Short(ds.readShort());
        dFormat -= (short)512;
        dTimestamp = Swap.Long(ds.readInt());

        glb = tf.findTLink(new String(globalsNameAsByteArray));
        int dataSizeInBytes = dArrayLength * TFormat.formatSizeOf(dFormat);
        byte[] d = new byte[dataSizeInBytes];
        ds.read(d,0,d.length);
        bytesread += dataSizeInBytes;
        // push this data set into the corresponding TDataType object
        if (glb != null)
        {
          glb.linkStatus = 0;
          if (dFormat == glb.dOutput.getFormat())
          {
            glb.dOutput.update(d,1,0);
            tf.fillinIncomingData(glb);
          }
          else
          {
            TDataType dt = new TDataType(dArrayLength,dFormat);
            dt.update(d,1,0);
            Object srcData = dt.getDataObject();
            dt.getData(srcData);
            Object dstData = glb.dOutput.getDataObject();
            switch (glb.dOutput.getFormat())
            {
              case TFormat.CF_FLOAT:
                 switch (dFormat)
                 {
                    case TFormat.CF_SHORT:
                      for (int k=0; k<dArrayLength; k++)
                        ((float[])dstData)[k] = (float)((short[])srcData)[k];
                      break;
                    case TFormat.CF_LONG:
                      for (int k=0; k<dArrayLength; k++)
                        ((float[])dstData)[k] = (float)((int[])srcData)[k];
                      break;
                    case TFormat.CF_DOUBLE:
                      for (int k=0; k<dArrayLength; k++)
                        ((float[])dstData)[k] = (float)((double[])srcData)[k];
                      break;
                    default:
                      break;
                 }
                 break;
              case TFormat.CF_DOUBLE:
                 switch (dFormat)
                 {
                    case TFormat.CF_SHORT:
                      for (int k=0; k<dArrayLength; k++)
                        ((double[])dstData)[k] = (double)((short[])srcData)[k];
                      break;
                    case TFormat.CF_LONG:
                      for (int k=0; k<dArrayLength; k++)
                        ((double[])dstData)[k] = (double)((int[])srcData)[k];
                      break;
                    case TFormat.CF_FLOAT:
                      for (int k=0; k<dArrayLength; k++)
                        ((double[])dstData)[k] = (double)((float[])srcData)[k];
                      break;
                    default:
                      break;
                 }
                 break;
              case TFormat.CF_LONG:
                 switch (dFormat)
                 {
                    case TFormat.CF_SHORT:
                      for (int k=0; k<dArrayLength; k++)
                        ((int[])dstData)[k] = (int)((short[])srcData)[k];
                      break;
                    case TFormat.CF_DOUBLE:
                      for (int k=0; k<dArrayLength; k++)
                        ((int[])dstData)[k] = (int)((double[])srcData)[k];
                      break;
                    case TFormat.CF_FLOAT:
                      for (int k=0; k<dArrayLength; k++)
                        ((int[])dstData)[k] = (int)((float[])srcData)[k];
                      break;
                    default:
                      break;
                 }
                 break;
              case TFormat.CF_SHORT:
                 switch (dFormat)
                 {
                    case TFormat.CF_LONG:
                      for (int k=0; k<dArrayLength; k++)
                        ((short[])dstData)[k] = (short)((int[])srcData)[k];
                      break;
                    case TFormat.CF_DOUBLE:
                      for (int k=0; k<dArrayLength; k++)
                        ((short[])dstData)[k] = (short)((double[])srcData)[k];
                      break;
                    case TFormat.CF_FLOAT:
                      for (int k=0; k<dArrayLength; k++)
                        ((short[])dstData)[k] = (short)((float[])srcData)[k];
                      break;
                    default:
                      break;
                 }
                 break;
              case TFormat.CF_NAME32:
                 switch (dFormat)
                 {
                    case TFormat.CF_NAME16:
                      for (int k=0; k<dArrayLength; k++)
                      {
                         System.arraycopy(((NAME16[])srcData)[k],0,((NAME32[])dstData)[k],0,16);
                      }
                      break;
                    default:
                      break;
                 }
                 break;
              case TFormat.CF_NAME16:
                 switch (dFormat)
                 {
                    case TFormat.CF_NAME32:
                      for (int k=0; k<dArrayLength; k++)
                      {
                         System.arraycopy(((NAME32[])srcData)[k],0,((NAME16[])dstData)[k],0,16);
                      }
                      break;
                    default:
                      break;
                 }
                 break;
              default:
                 break;
            }
            glb.dOutput.dCompletionLength = dArrayLength;
          }
        }
      }
      ds.close();
      dBuffer.close();
    }
    catch (IOException e)
    {
      e.printStackTrace();
    }
  }
}
