//: TLink.java
package de.desy.tine.headers;
import java.io.ByteArrayOutputStream;
import java.io.DataOutputStream;
import java.io.ByteArrayInputStream;
import java.io.DataInputStream;
import java.io.IOException;
import de.desy.tine.client.TLinkFactory;
import de.desy.tine.client.TLink;
import de.desy.tine.dataUtils.TDataTime;
import de.desy.tine.dataUtils.TDataType;
import de.desy.tine.definitions.TErrorList;
import de.desy.tine.definitions.TFormat;
import de.desy.tine.endianUtils.Swap;
import de.desy.tine.server.connections.TClientEntry;
import de.desy.tine.server.connections.TContractTable;
// Producer Header structure (published data)
public class TPublishedData
{
  public short msgSizeInBytes;
  public short subId;
  public short linkCompCode;
  public short numblks;
  public short blknum;
  public short mtu;
  public int   dTimestampMSEC;  // short
  public byte  dFormat;
  public byte linkCounter;
  public int   dTimestamp;      
  public int   linkStarttime;
  // non-network fields below ...
  static public final int hdrSizeInBytes = (7*2 + 2*1 + 2*4);
  public ByteArrayOutputStream dBuffer;
  private byte[] bBuffer = new byte[1500];
  // prepare out-going data (server side) ...
  private void prepare()
  {
    try
    {
      dBuffer.reset();
      DataOutputStream ds = new DataOutputStream(dBuffer);
      ds.writeShort(Swap.Short(msgSizeInBytes));
      ds.writeShort(Swap.Short(subId));
      ds.writeShort(Swap.Short(linkCompCode));
      ds.writeShort(Swap.Short(numblks));
      ds.writeShort(Swap.Short(blknum));
      ds.writeShort(Swap.Short(mtu));
      ds.writeShort(Swap.Short(dTimestampMSEC));
      ds.writeByte(dFormat);
      ds.writeByte(linkCounter);
      ds.writeInt(Swap.Long(dTimestamp));
      ds.writeInt(Swap.Long(linkStarttime));
      ds.close();
    }
    catch (IOException e)
    {
      e.printStackTrace();
    }  
  }
  // prepare in-coming data (client side) ...
  public void toStruct(byte[] data, int length)
  {
    TLink eqm;
    int bytesread = 0, pktsread = 0;
    int datasize, fmtsize, doutsize, gapsize, gapptr;
    short len;
    byte[] d;
    ByteArrayInputStream dinBuffer;
    try
    {
      dinBuffer = new ByteArrayInputStream(data);
      DataInputStream ds = new DataInputStream(dinBuffer);
      // overall message length:
      len = Swap.Short(ds.readShort());
      if (tf.debugLevel > 0) System.out.println("recv : " + length + " bytes : hdr " + len + " bytes");
      if (len < length)
      {
        if (tf.debugLevel > 0) System.out.println("recv error : abort ! ");
        return;
      }
      bytesread = 2;
      while (bytesread < length)
      {
          msgSizeInBytes = Swap.Short(ds.readShort());
          subId = Swap.Short(ds.readShort());
          linkCompCode = Swap.Short(ds.readShort());
          numblks = Swap.Short(ds.readShort());
          blknum = Swap.Short(ds.readShort());
          mtu = Swap.Short(ds.readShort());
          dTimestampMSEC = Swap.Short(ds.readShort());
          dFormat = ds.readByte();
          linkCounter = ds.readByte();
          dTimestamp = Swap.Long(ds.readInt());
          linkStarttime = Swap.Long(ds.readInt());
          
          if (tf.debugLevel > 0)
            System.out.println("data : " + "bytes " + msgSizeInBytes + ", blknum " + blknum + " from " + numblks + " <" + linkCompCode + ">");
          bytesread += msgSizeInBytes;
          if ((eqm=tf.findTLink((int)subId,linkStarttime)) == null)
          {
        	// assume the worst and bail out ...
            if (tf.debugLevel > 0) System.out.println("id " + subId + " not found!");
            break;
          }
          eqm.dOutput.numblks = numblks;
          eqm.linkStatus = linkCompCode;
          eqm.linkCounter = linkCounter;
          eqm.linkStale = true;
          datasize = eqm.dOutput.dArrayLength * TFormat.formatSizeOf(dFormat);// size of incoming data in bytes
          doutsize = msgSizeInBytes - hdrSizeInBytes; // size of data + msg in bytes
          gapsize = 0; gapptr = 0;
          pktsread++;
          
          switch (linkCompCode)
          {
            case TErrorList.server_redirection:
              d = new byte[TDataType.RPCERR_SIZE];
              ds.read(d,0,16); // redirected device server name
              eqm.expName = (new String(d)).trim();
              ds.read(d,0,16); // redirected device property name
              if (d[0] != 0)
              { // trap an empty string with the first char
                String prp = (new String(d)).trim();               
                if (prp.length() > 0) eqm.devProperty = prp;
              }
              break;
            case TErrorList.get_subscription_id:
              eqm.srvId = Swap.Short(ds.readShort());
              break;
            default: 
              if (doutsize < TDataType.RPCERR_SIZE && eqm.dOutput.blksin == 0)
              {
                eqm.dOutput.bytesin = 0;
                continue;
              }
    
              if ((linkCompCode & TErrorList.CE_SENDDATA) != TErrorList.CE_SENDDATA) datasize = 0;
              
              gapsize = 0;
              if (eqm.dOutput.blknum == eqm.dOutput.numblks || datasize == 0)
              {
                gapsize = TDataType.RPCERR_SIZE;
                if (doutsize < TDataType.RPCERR_SIZE)
                {
                  if (eqm.dOutput.numblks == 1) continue;
                  gapsize = doutsize;
                }
                gapptr = TDataType.RPCERR_SIZE - gapsize;
                doutsize -= gapsize;
                eqm.dOutput.bytesin += gapsize;
              }
              if ((linkCompCode & TErrorList.CE_SENDDATA) != TErrorList.CE_SENDDATA)
              {
                eqm.dOutput.blknum = blknum;
                eqm.dOutput.blksin = 1;
                eqm.dOutput.numblks = 1;
                break;
              }
            case 0:
              if (eqm.dOutput.dFormat == TFormat.CF_DEFAULT)
              {
                eqm.dOutput.dFormat = dFormat;
                if ((fmtsize = TFormat.formatSizeOf(eqm.dOutput.dFormat)) < 1) continue;
                eqm.dOutput.dArrayLength = (doutsize/fmtsize) * eqm.dOutput.numblks;
                // will be adjusted properly in InterpretIncomingData() from the factory
                eqm.adjustDefaultValues = true;
              }
              // push this data set into the corresponding TDataType object
              // can now delete the 3 lines below ...
              //d = new byte[msgSizeInBytes-hdrSizeInBytes-gapsize];
              //ds.read(d,0,d.length);
              //eqm.dOutput.update(d,blknum,linkCounter);
              int sizeInBytes = msgSizeInBytes-hdrSizeInBytes-gapsize;
              d = bBuffer;
              ds.read(d,0,sizeInBytes);
              eqm.dOutput.update(d,0,sizeInBytes,blknum,linkCounter);
              break;
          }  
          if (gapsize > 0)
          {
            d = new byte[gapsize];
            ds.read(d,0,d.length);
            System.arraycopy(d,0,eqm.linkStatusStringBuffer,gapptr,gapsize);
          }      
          eqm.dOutput.bytesin += doutsize;
          if (eqm.dOutput.blksin == eqm.dOutput.numblks)
          {
            eqm.dOutput.timestamp = dTimestamp;
            eqm.dOutput.timestampMSEC = dTimestampMSEC;
          }
      }
      if (tf.debugLevel > 0) System.out.println(pktsread + " incoming contracts read");
      ds.close();
      dinBuffer.close();
    }
    catch (IOException e)
    {
      e.printStackTrace();
    }	 
  }                  
  TLinkFactory tf;
 /**
 * Attach the object to a client link factory ...
 * Creation date: (8/21/01 6:33:36 PM)
 */
  public TPublishedData(TLinkFactory tfactory) 
  {
    tf = tfactory;
  }
  public TPublishedData(int msgsize,TClientEntry cln,TContractTable con)
  {
    msgSizeInBytes = (short)msgsize;
    if (cln != null && cln.sts != null)
    {
      subId = cln.sts.id;
      linkCompCode = cln.sts.statusCode;
      numblks = cln.sts.numblks;
      blknum = cln.sts.blknum;
      mtu = cln.sts.mtu;
      linkCounter = (byte)cln.sts.counter;
      linkStarttime = (int)(cln.sts.starttime/1000);
    }
    if (con != null)
    {
      dTimestampMSEC = (short)(con.dtimestamp%1000);
      dTimestamp = (int)(con.dtimestamp/1000);
      if (con.contract != null) dFormat = con.contract.EqpFormatOut;
    }
    dBuffer = new ByteArrayOutputStream(hdrSizeInBytes);
    prepare();
  }
}