//: TLink.java
package de.desy.tine.headers;
import java.io.ByteArrayOutputStream;
import java.io.ByteArrayInputStream;
import java.io.DataOutputStream;
import java.io.DataInputStream;
import java.io.IOException;
import java.util.Date;
import de.desy.tine.addrUtils.TSrvAddress;
import de.desy.tine.client.TLink;
import de.desy.tine.definitions.TFormat;
import de.desy.tine.definitions.TMode;
import de.desy.tine.endianUtils.Swap;
import de.desy.tine.types.SPECTRUM;
public final class TSubscription
{
  public short msgSizeInBytes;
  public short mtu;
  public byte  numblks;  // careful here!  byte is signed in java !!!!!
  public byte  blknum;
  public short id;
  public short mode;
  public short blkid;
  public int   pollingRate;
  public int   starttime;
  public TContract contract;
  // non 'network' fields ...
  public long linkStartTime;
  public long linkLastTime;
  public int dataSizeInBytes = 0;
  public int nextDataSizeInBytes = 0;
  static public final int hdrSizeInBytes = (6*2 + 2*4);
  static private short ensLinkId = 0;
  TLink link;
  ByteArrayOutputStream dBuffer;
  public byte[] toByteArray()
  {
    short m = (short)(mode & 0x0fff);
    //ByteArrayOutputStream db = new ByteArrayOutputStream(msgSizeInBytes);
    ByteArrayOutputStream db = new ByteArrayOutputStream(hdrSizeInBytes + TContract.hdrSizeInBytes);
    try
    {
      msgSizeInBytes = (short)(hdrSizeInBytes + TContract.hdrSizeInBytes);
      short payload = 0;
      if (link.dInput.blknum == link.dInput.numblks) // finished
      {
        payload = (short)(dataSizeInBytes % mtu);
        link.dInput.blkid = (link.dInput.blkid++) % 1000;
      }
      else                   // move along ...
      { 
        payload = mtu;
        link.dInput.blknum++;
      }
      msgSizeInBytes += payload;
      DataOutputStream ds = new DataOutputStream(db);
      ds.writeShort(Swap.Short(msgSizeInBytes));
      ds.writeShort(Swap.Short(mtu));
      ds.writeByte(numblks);
      ds.writeByte(blknum);
      ds.writeShort(Swap.Short(id));
      ds.writeShort(Swap.Short(m));
      ds.writeShort(Swap.Short(blkid));
      ds.writeInt(Swap.Long(pollingRate));
      ds.writeInt(Swap.Long(starttime));
      ds.close();
      //nextDataSizeInBytes = dataSizeInBytes - mtu > mtu ? mtu : dataSizeInBytes % mtu; 
      nextDataSizeInBytes = payload; 
      blknum = (byte)link.dInput.blknum;
      return db.toByteArray(); 
    }
    catch (IOException e)
    {
      e.printStackTrace();
    }
    return null;  
  }
  Date  date = new Date();
  public TSubscription(TContract c, TLink eqm) // out-going ...
  {
    contract = c;
    link = eqm;
    msgSizeInBytes = (short)(hdrSizeInBytes + TContract.hdrSizeInBytes);
    mtu = (short)eqm.dInput.mtu;
    numblks = (byte)(eqm.dInput.numblks);
    blknum = (byte)(eqm.dInput.blknum);
    id = (short)eqm.linkId;
    mode = TMode.CM_REGISTER;
    dataSizeInBytes = eqm.dInput.getSizeInBytes();
    nextDataSizeInBytes = blknum == numblks ? dataSizeInBytes : mtu;
    blkid = (short)(eqm.dInput.blkid);
    pollingRate = eqm.devTimeout;
    linkStartTime = date.getTime();
    linkLastTime = linkStartTime;
    starttime = (int)(linkStartTime/1000);
    if (id == 0 && eqm.expName.startsWith("ENS"))
    {
      starttime = TSrvAddress.ENS_ID;
      ensLinkId = (short)((ensLinkId+1)%10000);
      id = ensLinkId;
    }
    dBuffer = new ByteArrayOutputStream(msgSizeInBytes);
  }
  public TSubscription(byte[] data, int off, int len) // in-coming
  {  
    try
    {
      ByteArrayInputStream is = new ByteArrayInputStream(data,off,len); 
      DataInputStream ds = new DataInputStream(is);
      msgSizeInBytes = Swap.Short(ds.readShort());
      mtu = Swap.Short(ds.readShort());
      numblks = ds.readByte();
      blknum = ds.readByte();
      id = Swap.Short(ds.readShort());
      mode = Swap.Short(ds.readShort());
      blkid = Swap.Short(ds.readShort());
      pollingRate = Swap.Long(ds.readInt());
      starttime = Swap.Long(ds.readInt());
      //contract = new TContract(ds);
      contract = new TContract(data,off+hdrSizeInBytes,TContract.hdrSizeInBytes);
      ds.close();
      is.close();
    } 
    catch (IOException e)
    {
      e.printStackTrace();
    }      
  }
  public int getInputDataSize()
  {
    int fmtsiz = 0;
    if ((fmtsiz=TFormat.formatSizeOf(contract.EqpFormatIn)) > 0)
    {
      int datasize = fmtsiz * contract.EqpSizeIn;
      if (datasize > mtu) 
      {
        datasize = blknum < numblks ? mtu : datasize % mtu;
      }
      return datasize;
    }
    return 0;
  }
  public int getOutputDataSize()
  {
    int fmtsiz = 0;
    if ((fmtsiz=TFormat.formatSizeOf(contract.EqpFormatOut)) > 0)
    {
      int datasize = fmtsiz * contract.EqpSizeOut;
      if (contract.EqpFormatOut == TFormat.CF_SPECTRUM) datasize += SPECTRUM.hdrSizeInBytes;
      return datasize;
    }
    return 0;
  }
  public String toString()
  {
    String msg = new String("msg size = " + msgSizeInBytes);
    msg = msg.concat("\nmtu : " + mtu);
    msg = msg.concat("\nnumblks : " + numblks);
    msg = msg.concat("\nblknum : " + blknum);
    msg = msg.concat("\nid : " + id);
    msg = msg.concat("\nmode : " + TMode.toString(mode));
    msg = msg.concat("\nblkid : " + blkid);
    msg = msg.concat("\npollingRate : " + pollingRate + " ms");
    msg = msg.concat("\nstarttime : " + starttime);
    msg = msg.concat("\n" + contract.toString());
    return msg;
  }
}