package de.desy.tine.histUtils;

import java.util.Vector;
import de.desy.tine.client.TLink;
import de.desy.tine.dataUtils.TDataType;
import de.desy.tine.definitions.TAccess;
import de.desy.tine.definitions.TErrorList;
import de.desy.tine.queryUtils.TQuery;
import de.desy.tine.types.*;

// TODO: finish this ...
public final class THistory
{
  private static final int MAX_HSTCFG_SIZE = 100;
  private static HstCfg[] cfgs = new HstCfg[MAX_HSTCFG_SIZE];
  private static McaCfg[] mcacfgs = new McaCfg[MAX_HSTCFG_SIZE];
  private static boolean isInitialized = false;
  private static void initCfgs()
  {
    if (isInitialized) return;
    for (int i=0; i<MAX_HSTCFG_SIZE; i++)
    {
      cfgs[i] = new HstCfg();
      mcacfgs[i] = new McaCfg();
    }
    isInitialized = true;
  }
  private static boolean isCentralArchiveServer(String server)
  {
    return server.endsWith("HISTORY");
  }
  private static synchronized int getArchivedNumPointsInInterval(String context,String server, String property, String device, double start, double stop)
  {
    int cc = 0;
    int[] startstop = new int[2];
    int[] npts = new int[1];
    TDataType dout, din;
    
    if (server == null && context == null) return TErrorList.argument_list_error;
    if (context == null) context = "DEFAULT";
    if (server == null) server = "HISTORY"; // assume the standard central archive server for the given context
    startstop[0] = (int)start;
    startstop[1] = (int)stop;
    din = new TDataType(startstop);
    dout = new TDataType(npts);
    
    String dname = new String("/" + context + "/" + server + "/" + device);
    String pname = new String(property);
    if (!isCentralArchiveServer(server))
    { // not a call to a central archive server
      if (!property.endsWith("HIST") &&
          !property.endsWith("HST") &&
          !property.endsWith("ARCH") &&
          !property.endsWith("ARC") &&
          !property.endsWith("AR"))
        pname = pname + ".HIST"; // meta extension not given, try this ...
    }
    TLink tl = new TLink(dname,pname,dout,din,TAccess.CA_READ);
    cc = tl.execute(2000, true);
    tl.close();
    return cc > 0 ? -cc : npts[0];
  }  
  private static synchronized int getArchivedDataArray(String context,String server, String property, String device, int index, double start, double stop,TCompoundDataObject[] data,int timeout)
  {
    int cc = 0;
    int[] startstop = new int[3];
    TDataType dout, din;
    if (server == null && context == null) return TErrorList.argument_list_error;
    if (data == null) return TErrorList.argument_list_error;
    if (context == null) context = "DEFAULT";
    if (server == null) server = "HISTORY"; // assume the standard central archive server for the given context
    if (timeout < 100) timeout = 100;
    startstop[0] = (int)start;
    startstop[1] = (int)stop;
    startstop[2] = index;
    din = new TDataType(startstop);
    dout = new TDataType(data);
    
    String dname = new String("/" + context + "/" + server + "/" + device);
    String pname = new String(property);
    if (!isCentralArchiveServer(server))
    { // not a call to a central archive server
      if (!property.endsWith("HIST") &&
          !property.endsWith("HST") &&
          !property.endsWith("ARCH") &&
          !property.endsWith("ARC") &&
          !property.endsWith("AR"))
        pname = pname + ".HIST"; // meta extension not given, try this ...
    }
    TLink tl = new TLink(dname,pname,dout,din,TAccess.CA_READ);
    cc = tl.execute(timeout, true);
    tl.close();
    return cc;
  }  
  private static int getArchivedDataArray(String context,String server, String property, String device, int index, double start, double stop,TCompoundDataObject[] data)
  {
    return getArchivedDataArray(context,server,property,device,index,start,stop,data,2000);
  }
  private static synchronized int getArchivedDataSnapshot(String context,String server, String property, String device, double[] targetTime,float[] data)
  {
    int cc = 0;
    int[] startstop = new int[2];
    TDataType dout, din;

    if (server == null && context == null) return TErrorList.argument_list_error;
    if (targetTime == null || data == null) return TErrorList.argument_list_error;
    if (context == null) context = "DEFAULT";
    if (server == null) server = "HISTORY"; // assume the standard central archive server for the given context
    startstop[0] = (int)targetTime[0];
    startstop[1] = (int)targetTime[0];
    din = new TDataType(startstop);
    dout = new TDataType(data);
    
    String dname = new String("/" + context + "/" + server + "/" + device);
    String pname = new String(property);
    if (!server.endsWith("HISTORY"))
    { // not a call to a central archive server
      if (!property.endsWith("HIST") &&
          !property.endsWith("HST") &&
          !property.endsWith("ARCH") &&
          !property.endsWith("ARC") &&
          !property.endsWith("AR"))
        pname = pname + ".HIST"; // meta extension not given, try this ...
    }
    TLink tl = new TLink(dname,pname,dout,din,TAccess.CA_READ);
    cc = tl.execute(2000, true);
    targetTime[0] = tl.getLastDataTimeStamp();
    tl.close();
    return cc;
  }  
  public static int getArchivedData(String context,String server, String property, String device, double start, double stop,FLTINT[] data)
  {
    return getArchivedDataArray(context,server,property,device,0,start,stop,data);
  }
  public static int getArchivedData(String context,String server, String property, String device, int index,double start, double stop,FLTINT[] data)
  {
    return getArchivedDataArray(context,server,property,device,index,start,stop,data);
  }
  public static int getArchivedData(String context,String server, String property, String device, int index,double start, double stop,FLTINT[] data,int timeout)
  {
    return getArchivedDataArray(context,server,property,device,index,start,stop,data,timeout);
  }
  public static int getArchivedData(String context,String server, String property, String device, double start, double stop,DBLDBL[] data)
  {
    return getArchivedDataArray(context,server,property,device,0,start,stop,data);
  }
  public static int getArchivedData(String context,String server, String property, String device, int index,double start, double stop,DBLDBL[] data)
  {
    return getArchivedDataArray(context,server,property,device,index,start,stop,data);
  }
  public static int getArchivedData(String context,String server, String property, String device, int index,double start, double stop,DBLDBL[] data,int timeout)
  {
    return getArchivedDataArray(context,server,property,device,index,start,stop,data,timeout);
  }
  public static int getArchivedData(String context,String server, String property, String device, double start, double stop,NAME16I[] data)
  {
    return getArchivedDataArray(context,server,property,device,0,start,stop,data);
  }
  public static int getArchivedData(String context,String server, String property, String device, double start, double stop,NAME16I[] data,int timeout)
  {
    return getArchivedDataArray(context,server,property,device,0,start,stop,data,timeout);
  }
  public static int getArchivedData(String context,String server, String property, String device, double start, double stop,NAME32I[] data)
  {
    return getArchivedDataArray(context,server,property,device,0,start,stop,data);
  }
  public static int getArchivedData(String context,String server, String property, String device, double start, double stop,NAME32I[] data,int timeout)
  {
    return getArchivedDataArray(context,server,property,device,0,start,stop,data,timeout);
  }
  public static int getArchivedData(String context,String server, String property, String device, double start, double stop,NAME64I[] data)
  {
    return getArchivedDataArray(context,server,property,device,0,start,stop,data);
  }
  public static int getArchivedData(String context,String server, String property, String device, double start, double stop,NAME64I[] data,int timeout)
  {
    return getArchivedDataArray(context,server,property,device,0,start,stop,data,timeout);
  }
  public static int getNumPointsInInterval(String context,String server, String property, String device, double start, double stop)
  {
    return getArchivedNumPointsInInterval(context, server, property, device, start, stop);
  }
  public static int getArchivedSnapshot(String context,String server, String property, String device, double[] targetTime,float[] data)
  {
    return getArchivedDataSnapshot(context,server,property,device,targetTime,data);
  }
  public static String[] getArchivedProperties(String context)
  {
    return getArchivedProperties(context,null,null);
  }
  public static String[] getArchivedProperties(String context,String server)
  {
    return getArchivedProperties(context,server,null);
  }
  public static String[] getArchivedProperties(String context,String server,String subsystem)
  {
    if (server == null && context == null) return null;
    if (context == null) context = "DEFAULT";
    if (server == null) server = "HISTORY";
    boolean isCentral = isCentralArchiveServer(server);
    TDataType dtsubs;
    if (subsystem != null && subsystem.length() > 0 && 
        !subsystem.startsWith("ALL") && !subsystem.startsWith("All"))
    {
      dtsubs = new TDataType(subsystem);
    }
    else
    { 
      dtsubs = new TDataType();
    }
    NAME32[] nlist = null;
    int[] nprps = new int[1];
    String ntagsString, tagsString;
    TDataType dtnprps = new TDataType(nprps);
    TDataType dtprps;
    tagsString = isCentral ? "TAGS" : "HISTORIES";
    ntagsString = "N" + tagsString;
    
    String dname = "/" + context + "/" + server;
    TLink tl = new TLink(dname,ntagsString,dtnprps,dtsubs,TAccess.CA_READ);
    int cc = tl.execute(800, true);
    tl.close();
    if (cc != 0)
    { // TODO: throw an exception here
      return null;
    }
    if (nprps[0] > 0)
    {
      nlist = new NAME32[nprps[0]];
      dtprps = new TDataType(nlist);
      tl = new TLink(dname,tagsString,dtprps,dtsubs,TAccess.CA_READ);
      cc = tl.execute(800, true);
      tl.close();
      if (cc != 0)
      { // TODO: throw an exception here
        return null;
      }
    }
    String nam;
    Vector<String> v = new Vector<String>(nlist.length);
    int n = 0;
    for (int i=0; i<nlist.length; i++)
    {
      nam = nlist[i].getName();
      if (nam.endsWith(".NAM")) continue;
      v.add(nam);
      n++;
    }
    return v.toArray(new String[n]);
  }
  public static String[] getArchivedDevices(String context,String server, String property)
  {
    return TQuery.getDeviceNames(context, server, property);
  }
  public static String[] getArchivedSubsystems(String context)
  {
    return getArchivedSubsystems(context,null);
  }
  public static String[] getArchivedSubsystems(String context,String server)
  {
    if (server == null && context == null) return null;
    if (context == null) context = "DEFAULT";
    if (server == null) server = "HISTORY"; // assume the standard central archive server for the given context
    if (!isCentralArchiveServer(server)) return null;
    int cc;
    NAME32[] nlist = null;
    int[] nsubs = new int[1];
    String ntagsString, tagsString;
    TDataType dtnsubs = new TDataType(nsubs);
    TDataType dtsubs;
    tagsString = "SUBSYSTEMS";
    ntagsString = "N" + tagsString;
    
    String dname = "/" + context + "/" + server;
    TLink tl = new TLink(dname,ntagsString,dtnsubs,null,TAccess.CA_READ);
    cc = tl.execute(800, true);
    tl.close();
    if (cc != 0)
    { // TODO: throw an exception here
      return null;
    }
    if (nsubs[0] > 0)
    {
      nlist = new NAME32[nsubs[0]];
      dtsubs = new TDataType(nlist);
      tl = new TLink(dname,tagsString,dtsubs,null,TAccess.CA_READ);
      cc = tl.execute(800, true);
      tl.close();
      if (cc != 0)
      { // TODO: throw an exception here
        return null;
      }
    }
    String nam;
    Vector<String> v = new Vector<String>(nlist.length);
    int n = 0;
    for (int i=0; i<nlist.length; i++)
    {
      nam = nlist[i].getName();
      if (nam.length() == 0) continue;
      v.add(nam);
      n++;
    }
    return v.toArray(new String[n]);
  }
  public static String[] getArchiveServerContexts()
  {
    return TQuery.getContexts("HISTORY");
  }
  public static String[] getViewerMcaConfigurationNames(String context)
  {
    if (context == null) return null;
    return TQuery.getDeviceNames(context, "HISTORY", "CONFIGURATION.MCA");
  }
  public static String[] getViewerConfigurationNames(String context)
  {
    if (context == null) return null;
    return TQuery.getDeviceNames(context, "HISTORY", "CONFIGURATION");
  }
  public static String[] getViewerConfigurationNames(String context,String group)
  {
    if (context == null) return null;
    String prp = "CONFIGURATION";
    if (group != null) prp += "." + group;
    return TQuery.getDeviceNames(context, "HISTORY", prp);
  }
  public static String[] getViewerConfigurationGroups(String context)
  {
    if (context == null) return null;
    NAME64[] prps = new NAME64[100];
    TDataType dtprps = new TDataType(prps);
    String dname = "/" + context + "/HISTORY";
    String dprop = "CONFIGURATION.*";
    TLink tl = new TLink(dname,dprop,dtprps,null,TAccess.CA_READ);
    int cc = tl.execute(800, true);
    tl.close();
    if (cc != 0)
    { // TODO: throw an exception here
      return null;
    }
    int len = dtprps.getCompletionLength();
    String nam;
    Vector<String> v = new Vector<String>(len);
    int n = 0;
    for (int i=0; i<len; i++)
    {
      nam = prps[i].getName();
      if (nam.compareToIgnoreCase("CONFIGURATION") == 0) continue;
      if (nam.length() == 0) continue;
      if (nam.endsWith(".NAM")) continue;
      if (nam.endsWith(".MCA")) continue;
      v.add(nam.substring(14));
      n++;
    }
    return n > 0 ? v.toArray(new String[n]) : null;
    
  }
  public static HstCfg[] getSelectableProptiesConfiguration(String context,String group)
  {
    if (context == null || group == null) return null;
    return getViewerConfiguration(context,"properties",group);
  }
  public static HstCfg[] getViewerConfiguration(String context,String name)
  {
    return getViewerConfiguration(context,name,null);
  }
  public static synchronized HstCfg[] getViewerConfiguration(String context,String name,String group)
  {
    if (context == null || name == null) return null;
    if (!isInitialized) initCfgs();
    TDataType dtcfg = new TDataType(cfgs);
    String dname = "/" + context + "/HISTORY/" + name;
    String dprop = "CONFIGURATION";
    if (group != null && group.length() > 0) dprop += "." + group;
    TLink tl = new TLink(dname,dprop,dtcfg,null,TAccess.CA_READ);
    int cc = tl.execute(800, true);
    tl.close();
    if (cc != 0)
    { // TODO: throw an exception here
      System.out.println("get Configuration : " + TErrorList.getErrorString(cc));
      return null;
    }
    int num = dtcfg.getCompletionLength();
    if (num == 0) return null;
    HstCfg[] cfg = new HstCfg[num];
    for (int i=0; i<num; i++) 
    {
      cfg[i] = new HstCfg(cfgs[i]);
    }
    return cfg;
  }
  public static synchronized McaCfg[] getMcaViewerConfiguration(String context,String name)
  {
    if (context == null || name == null) return null; 
    if (!isInitialized) initCfgs();
    TDataType dtcfg = new TDataType(mcacfgs);
    String dname = "/" + context + "/HISTORY/" + name;
    String dprop = "CONFIGURATION.MCA";
    TLink tl = new TLink(dname,dprop,dtcfg,null,TAccess.CA_READ);
    int cc = tl.execute(800, true);
    tl.close();
    if (cc != 0)
    { // TODO: throw an exception here
      System.out.println("get Configuration : " + TErrorList.getErrorString(cc));
      return null;
    }
    int num = dtcfg.getCompletionLength();
    if (num == 0) return null;
    McaCfg[] cfg = new McaCfg[num];
    for (int i=0; i<num; i++) 
    {
      cfg[i] = new McaCfg(mcacfgs[i]);
    }
    return cfg;
  }
}
