package de.desy.tine.queryUtils;

import java.io.IOException;
import de.desy.tine.addrUtils.FECInfo;
import de.desy.tine.addrUtils.TModAddress;
import de.desy.tine.client.TLink;
import de.desy.tine.client.TLinkFactory;
import de.desy.tine.dataUtils.TDataType;
import de.desy.tine.definitions.TAccess;
import de.desy.tine.definitions.TErrorList;
import de.desy.tine.definitions.TFormat;
import de.desy.tine.server.connections.TClient;
import de.desy.tine.server.connections.TClientStruct;
import de.desy.tine.server.connections.TContractStruct;
import de.desy.tine.structUtils.TStructRegistry;
import de.desy.tine.structUtils.TStructIo;
import de.desy.tine.types.*;


/**
 * @brief tine Query routines
 * 
 * This is a static class which can be used to make system queries without
 * the need for instantiating a TQuery object.
 */
public final class TQuery
{
  public static final int nicePropertyQuerySize = 10;
  public static final int niceContextsQuerySize = 100;
  private static int maxQueryBufferSize = 65000;
  private static byte[] hByteBlobX = new byte[nicePropertyQuerySize * XPropertyQuery.sizeInBytes];
  private static byte[] hByteBlob = new byte[nicePropertyQuerySize * PropertyQueryLegacy.sizeInBytes];
  private static  byte[] hLegacyByteBlob;
  private static String lastQueriedContext = null;
  private static String lastQueriedServer = null;
  private static String lastQueriedDevice = null;
  private static TModAddress modAddr = null;
  public static boolean properties_have_query_function = false;
  public static boolean devices_have_query_function = false;
  public static boolean isStockProperty(String property)
  {
    if (property.lastIndexOf("PROPS") >= 0) return true;
    if (property.lastIndexOf("PROPERTIES") >= 0) return true;
    if (property.lastIndexOf("DEVICES") >= 0) return true;
    return false;
  }
  public static String getModuleAddressInfo()
  { // not thread-safe
    if (modAddr == null) modAddr = new TModAddress();
    return modAddr.toString(lastQueriedServer,lastQueriedContext);
  }
  public static String getModuleAddressInfo(String server,String context)
  {
    modAddr = new TModAddress(TLinkFactory.getInstance(),server,context);
    return modAddr.toString(server,context);
  }
  public static TModAddress getModuleAddress(String server,String context)
  {
    return new TModAddress(TLinkFactory.getInstance(),server,context);
  }
  private static String[] getTagList(String context,String tagtype)
  {
    return getTagList(context,tagtype,null);
  }
  private static String[] getTagList(String context,String tagtype,String subsys)
  {
    return getTagList(context,tagtype,subsys,TLink.defaultTimeout);
  }
  private static String[] getTagList(String context,String tagtype,String subsys,int timeout)
  {
    int cc,n,i;
    TDataType dout;
    TDataType din;
    TLink tl;
    StringBuffer host = new StringBuffer(32);
    StringBuffer query = new StringBuffer(32);
    short[] numout = new short[1];
    TDataType numoutData = new TDataType(numout);
    NAME16[] taglist16;
    NAME32[] taglist32;
    String[] strlist;

    if (subsys == null || subsys.compareTo("ALL") == 0)
    {
      din = new TDataType();
    }
    else
    {
      din = new TDataType(subsys);
    }
    
    host.delete(0,31); query.delete(0,31);
    if (context != null && context.length() != 0)
    {
      host.insert(0,"ENS/" + context);
    }
    else 
    {
      host.insert(0,"ENS");
    }
    query.insert(0,"N" + tagtype);
    try 
    {
      tl = new TLink(host.toString(),query.toString(),numoutData,din,TAccess.CA_READ);
      cc = tl.execute(timeout,true);
      modAddr = tl.modAddress;
      tl.cancel();
    }
    catch (Exception e)
    {
      cc = TErrorList.non_existent_elem;
    }
    if (cc != 0) return null;
//  /* Get Tags from name server (synchronous call) */
    query.delete(0,31);
    query.insert(0,tagtype);
    n = numout[0];

    taglist32 = new NAME32[n];
    for (i=0; i<n; i++) taglist32[i] = new NAME32();
    if (taglist32.length == 0) return null;
    if (numout[0] > taglist32.length) numout[0] = (short)taglist32.length;  
    dout = new TDataType(taglist32);
    tl = new TLink(host.toString(),query.toString(),dout,din,TAccess.CA_READ);
    cc = tl.execute(TLink.defaultTimeout,true);
    tl.cancel();
    if (cc == 0)
    { // this worked, so we're finished 
      strlist = new String[n];
      for (i=0; i<n; i++) strlist[i] = taglist32[i].name;      
      return strlist;
    }
    if (cc == TErrorList.illegal_format)
    { // old ENS ? -> try with reduced lengths ...
      taglist16 = new NAME16[n];
      for (i=0; i<n; i++) taglist16[i] = new NAME16();
      if (taglist16.length == 0) return null;
      if (numout[0] > taglist16.length) numout[0] = (short)taglist16.length;  
      dout = new TDataType(taglist16);
      tl = new TLink(host.toString(),query.toString(),dout,din,TAccess.CA_READ);
      cc = tl.execute(TLink.defaultTimeout,true);
      tl.cancel();
      if (cc != 0) return null;
      strlist = new String[n];
      for (i=0; i<n; i++) strlist[i] = taglist16[i].name;
      return strlist;      
    }
    return null;    
  }
  private static int getNumberOf(String property,String target)
  {
    return getNumberOf(property,target,TLink.defaultTimeout);
  }
  private static int getNumberOf(String property,String target,int timeout)
  {
    int cc = TErrorList.non_existent_elem;
    short[] n = new short[1];
    TDataType dout;
    TLink tl;
    String propertyq;
    
    if (property == null || target == null) return -TErrorList.argument_list_error;
   
    dout = new TDataType(n);
    propertyq = new String("N" + property); 
    try 
    {
      tl = new TLink(target,propertyq,dout,null,TAccess.CA_READ);
      cc = tl.execute(timeout,true);
      modAddr = tl.modAddress;
      tl.cancel();
    }
    catch (Exception e)
    {
      //
    }
    return cc > 0 ? -cc : (int)n[0];    
  }
  /**
   * @brief Sets the maximum size for use in tine queries.
   *
   * @param size is the size in bytes of the buffer used to hold the query
   * information (65000 bytes is the default)
   *  
   * @return 0 if successful 
   */
  public static short setMaximumQueryBufferSize(int size)
  {
    if (size < 4000 || size > 256000) return TErrorList.out_of_range;
    maxQueryBufferSize = size;
    return 0;
  }
  /**
   * @brief Returns a list of Contexts (obtained from the name services)
   * 
   * @return A string array containing a list of available contexts as 
   * obtained from the Equipment Name Server (ENS)
   */
  public static String[] getContexts()
  {
    return getTagList(null,"CONTEXTS");
  }
  public static String[] getContexts(String serverName)
  {
    int cc,n,i;
    TDataType dout;
    TDataType din;
    TLink tl;
    NAME32[] taglist32;
    String[] strlist;

    if (serverName == null || serverName.compareToIgnoreCase("ALL") == 0)
    {
      din = new TDataType();
    }
    else
    {
      din = new TDataType(serverName);
    }
    
    taglist32 = new NAME32[niceContextsQuerySize];
    for (i=0; i<niceContextsQuerySize; i++) taglist32[i] = new NAME32();
    if (taglist32.length == 0) return null;
    dout = new TDataType(taglist32);
    tl = new TLink("/SERVICE/ENS","CONTEXTS",dout,din,TAccess.CA_READ);
    cc = tl.execute(TLink.defaultTimeout,true);
    tl.cancel();
    n = dout.getCompletionLength();
    if (cc == 0)
    { // this worked, so we're finished 
      strlist = new String[n];
      for (i=0; i<n; i++) strlist[i] = taglist32[i].name;      
      return strlist;
    }
    return null;    
  }
  /**
   * @brief Returns a list of device servers associated with the specified
   * context (obtained from the name services)
   * 
   * @param context Is the context for which the device server name list is
   * desired
   * 
   * @return A string array containing a list of device servers associated
   * with the given context as obtained from the Equipment Name Server (ENS)
   */
  public static String[] getDeviceServers(String context)
  {
    return getTagList(context,"TAGS");
  }
  /**
   * @brief Returns a list of device servers associated with the specified
   * context and subsystem (obtained from the name services)
   * 
   * @param context Is the context for which the device server name list is
   * desired
   * @param subsys Is the subsystem for which the device server name list is
   * desired
   * 
   * @return A string array containing a list of device servers associated
   * with the given context and subsystem as obtained from the Equipment Name Server (ENS)
   */
  public static String[] getDeviceServers(String context,String subsys)
  {
    return getTagList(context,"TAGS",subsys);
  }
  /**
   * @brief Returns a list of device servers associated with the specified
   * context, subsystem, and importance level (obtained from the name services)
   * 
   * @param context Is the context for which the device server name list is
   * desired
   * @param subsys Is the subsystem for which the device server name list is
   * desired
   * @param imporance Is the importance level for which the device server name list is
   * desired (one of "ALL", "IMPORTANT", "ESSENTIAL", or "CRITICAL")
   * 
   * @return A ServerQuery array containing a list of device servers associated
   * with the given context, subsystem, and importance as obtained from the 
   * Equipment Name Server (ENS)
   */
  public static ServerQuery[] getDeviceServersEx(String context,String subsys,String importance)
  {
    return getDeviceServersEx(context,subsys,importance,TLink.defaultTimeout);
  }
  public static ServerQuery[] getDeviceServersEx(String context,String subsys,String importance,int timeout)
  {
    return getXTagList(context,"TAGS",subsys,importance,timeout);
  }
  /**
   * @brief Returns a list of registered subsystems 
   * (obtained from the name services)
   * 
   * @return A string array containing a list of subsystems.
   * The subsystem is itself not part of the de.desy.tine.client name space,
   * but can be used to refine queries to the Equipment Name
   * Servers (ENS).
   */
  public static String[] getDeviceSubsystems()
  {
    return getTagList(null,"SUBSYSTEMS");
  }
  /**
   * @brief Returns a list of registered subsystems in a given context 
   * (obtained from the name services)
   * 
   * @param context Is the context for which the subsystem list is desired
   * 
   * @return A string array containing a list of available subsystems in the given context.
   * The subsystem is itself not part of the de.desy.tine.client name space,
   * but can be used to refine queries to the Equipment Name
   * Servers (ENS).
   */
  public static String[] getDeviceSubsystems(String context)
  {
    return getTagList(context,"SUBSYSTEMS");
  }
  /**
   * @brief Returns a list of front end computers (FECs) associated with the specified
   * context (obtained from the name services)
   * 
   * @param context Is the context for which the FEC name list is
   * desired
   * 
   * @return A string array containing a list of front end computers associated
   * with the given context as obtained from the Equipment Name Server (ENS)
   */
  public static String[] getServers(String context)
  {
    return getTagList(context,"FECS");
  }
  /**
   * @brief Returns a list of front end computers (FECs) associated with the specified
   * context and sub system (obtained from the name services)
   * 
   * @param context Is the context for which the FEC name list is
   * desired
   * @param subsys Is the subsystem for which the device server name list is
   * desired
   * 
   * @return A string array containing a list of front end computers associated
   * with the given context and subsystem as obtained from the Equipment Name Server (ENS)
   */
  public static String[] getServers(String context,String subsys)
  {
    return getTagList(context,"FECS",subsys);
  }
  /**
   * @brief Returns a list of front end computers (FECs) associated with the specified
   * context, subsystem, and importance level (obtained from the name services)
   * 
   * @param context Is the context for which the device server name list is
   * desired
   * @param subsys Is the subsystem for which the device server name list is
   * desired
   * @param imporance Is the importance level for which the device server name list is
   * desired (one of "ALL", "IMPORTANT", "ESSENTIAL", or "CRITICAL")
   * 
   * @return A ServerQuery array containing a list of device servers associated
   * with the given context, subsystem, and importance as obtained from the 
   * Equipment Name Server (ENS)
   */
  public static ServerQuery[] getServersEx(String context,String subsys,String importance)
  {
    return getServersEx(context,subsys,importance,TLink.defaultTimeout);
  }
  public static ServerQuery[] getServersEx(String context,String subsys,String importance,int timeout)
  {
    return getXTagList(context,"FECS",subsys,importance,timeout);
  }
  public static TClientStruct[] getServerClients(String fullServerName)
  { 
    if (fullServerName == null) return null;
    TClientStruct[] tcs = new TClientStruct[100];
    for (int i=0; i<100; i++) tcs[i] = new TClientStruct();
    TDataType tcsd = new TDataType(tcs);
    TLink tl = new TLink(fullServerName,"CLIENTS",tcsd,null,TAccess.CA_READ);
    if (tl.execute(400,true) == 0) 
    {
      int len;
      for (len=0; len<100 && tcs[len].getProtocol()[0] != (short)0; len++);
      if (len < 100)
      {
        TClientStruct[] tcss = new TClientStruct[len];
        System.arraycopy(tcs,0,tcss,0,len);
        return tcss;
      }
      return tcs;
    }
    System.out.println("error : " + tl.linkStatus);
    return null;
  }
  public static TContractStruct[] getServerContracts(String fullServerName)
  { 
    if (fullServerName == null) return null;
    TContractStruct[] tcs = new TContractStruct[100];
    for (int i=0; i<100; i++) tcs[i] = new TContractStruct();
    TDataType tcsd = new TDataType(tcs);
    TLink tl = new TLink(fullServerName,"CONTRACTS",tcsd,null,TAccess.CA_READ);
    if (tl.execute(400,true) == 0) 
    {
      int len;
      for (len=0; len<100 && tcs[len].getEqpProperty()[0] != (char)0; len++);
      if (len < 100)
      {
        TContractStruct[] tcss = new TContractStruct[len];
        System.arraycopy(tcs,0,tcss,0,len);
        return tcss;
      }
      return tcs;
    }
    System.out.println("error : " + tl.linkStatus);
    return null;
  }
  public static FECInfo getServerInformation(String frontEndComputer)
  {
    String os, desc, loc, hdw, sub, resp;
    FECInfo finf = null;
    char[] fi = new char[188];
    TDataType fid = new TDataType(fi);
    TLink tl = new TLink("ENS/FEC.EXT",frontEndComputer,fid,null,TAccess.CA_READ);
    if (tl.execute(400,true) == 0)
    {
      os = new String(fi,0,12);
      desc = new String(fi,12,64); 
      loc = new String(fi,76,32);
      hdw = new String(fi,108,32);
      sub = new String(fi,140,16);
      resp = new String(fi,156,32);
      // parse away legacy description info
      if (desc.startsWith("["))
      {
        int p = desc.indexOf(']');
        if (p != -1)
        {
          desc = desc.substring(p+1);
        }
      }
    }
    else
    {
      os = "unknown"; desc = "unknown"; loc = "unknown";
      hdw = "unknown"; sub = "unknown"; resp = "unknown";
    }
    finf = new FECInfo(os,desc,loc,hdw,sub,resp);
    return finf;
  }
  /**
   * @brief Returns a list of properties associated with the specified
   * context and device server and device (obtained from the device server)
   * 
   * @param context Is the context of the device server for which the property list is
   * desired.
   * @param server Is the device server for which the property list is
   * desired.
   * @param device Is the device name (module name) for which the property list
   * is desired. 
   * 
   * @return A string array containing a list of properties
   * as obtained from the given device server
   */
  public static String[] getDeviceProperties(String context,String server,String device)
  {
    return getDeviceProperties(context,server,device,TLink.defaultTimeout);
  }
  public static String[] getDeviceProperties(String context,String server,String device,int timeout)
  {
    return getDeviceProperties(context,server,device,null,timeout);
  }
  /**
   * @brief Returns a list of properties associated with the specified
   * context and device server, device, and property (sub)string 
   * (obtained from the device server)
   * 
   * @param context Is the context of the device server for which the property list is
   * desired.
   * @param server Is the device server for which the property list is
   * desired.
   * @param device Is the device name (module name) for which the property list
   * is desired.
   * @param property Can be a queriable string with wildcard 
   * parameter '*'.  e.g. property = 'P*' will return all properties whose
   * first letter begins with 'P'.
   * 
   * @return A string array containing a list of properties
   * as obtained from the given device server
   */
  public static String[] getDeviceProperties(String context,String server,String device,String property)
  {
    return getDeviceProperties(context,server,device,property,TLink.defaultTimeout);
  }
  public static String[] getDeviceProperties(String context,String server,String device,String property,int timeout)
  {
    String[] prps = getDeviceProperties64(context,server,device,property,timeout);
    if (prps != null) return prps;
    return getDeviceProperties16(context,server,device,property,timeout);
  }
  private static String[] getDeviceProperties16(String context,String server,String device,String property,int timeout)
  {
    String tgt = getNamesQueryTarget(context,server,device);
    if (tgt == null) return null;
    TLink tl;
    TDataType dout;
    TDataType din;
    int cc = 0,n,i,j;
    NAME32[] properties;
    String[] strprops;

    n = getNumberOf("PROPS",tgt);
    if (n <= 0) return null;
    if (n*32 > maxQueryBufferSize) n = maxQueryBufferSize/32;
    properties = new NAME32[n];
    for (i=0; i<n; i++) properties[i] = new NAME32();
    if (properties.length <= 0) return null;
    dout = new TDataType(properties);
    if (property != null) din = new TDataType(property); else din = null;
    properties_have_query_function = false;
    try 
    {
      tl = new TLink(tgt,"PROPS",dout,din,TAccess.CA_READ);
      cc = tl.execute(TLink.defaultTimeout,true);
      tl.cancel();
      if ((cc & TErrorList.CE_SENDDATA) == TErrorList.CE_SENDDATA)
      {
        if ((cc & TErrorList.has_query_function) == TErrorList.has_query_function)
        {
          properties_have_query_function = true;
        }
        cc = 0;
      }
    }
    catch (Exception e)
    {
      cc = TErrorList.non_existent_elem;
    }
    if (cc == 0)
    {
      for (i=0,n=0; i<properties.length; i++) if (properties[i].name.length() > 0) n++;
      strprops = new String[n];
      for (i=0,j=0; j<n && i<properties.length; i++) 
      {
        if (properties[i].name.length() == 0) continue;
        strprops[j++] = properties[i].name;
      }
      return strprops;
    }
    else if (cc != TErrorList.link_not_open)
    {
      if (lastQueriedContext == context && lastQueriedServer == server &&
          lastQueriedDevice == device && lastQueriedServer != null)
      {
        cc = 0;
      }
      else
      {
        hLegacyByteBlob = new byte[n * PropertyQuery.sizeInBytes];
        dout = new TDataType(hLegacyByteBlob,"");
        tl = new TLink(tgt,"PROPS",dout,din,TAccess.CA_READ);
        cc = tl.execute(TLink.defaultTimeout,false);
        tl.cancel();
      }
      if (cc == 0)
      {
        lastQueriedContext = context;
        lastQueriedServer = server;
        lastQueriedDevice = device;
        strprops = new String[n];
        PropertyQuery[] pq = new PropertyQuery[n];
        byte b[];
        if (hLegacyByteBlob == null) return null;
        for (i=0; i<n; i++)
        {
          pq[i] = new PropertyQuery();
          b = pq[i].toByteArray();
          System.arraycopy(hLegacyByteBlob,i*PropertyQuery.sizeInBytes,b,0,PropertyQuery.sizeInBytes);
          pq[i].toStruct();
          strprops[i] = pq[i].name;
        }        
        return strprops;
      }
    }
    throw new RuntimeException("Could not acquire property information for " + server + " (" + TErrorList.getErrorString(cc) + ")");
  }
  private static String[] getDeviceProperties64(String context,String server,String device,String property,int timeout)
  {
    String tgt = getNamesQueryTarget(context,server,device);
    if (tgt == null) return null;
    TLink tl;
    TDataType dout;
    TDataType din;
    int cc = 0,n,i,j;
    NAME64[] properties;
    String[] strprops;

    n = getNumberOf("PROPS",tgt);
    if (n <= 0) return null;
    if (n*32 > maxQueryBufferSize) n = maxQueryBufferSize/32;
    properties = new NAME64[n];
    for (i=0; i<n; i++) properties[i] = new NAME64();
    if (properties.length <= 0) return null;
    dout = new TDataType(properties);
    if (property != null) din = new TDataType(property); else din = null;
    properties_have_query_function = false;
    try 
    {
      tl = new TLink(tgt,"PROPS",dout,din,TAccess.CA_READ);
      cc = tl.execute(TLink.defaultTimeout,true);
      tl.cancel();
      if ((cc & TErrorList.CE_SENDDATA) == TErrorList.CE_SENDDATA)
      {
        if ((cc & TErrorList.has_query_function) == TErrorList.has_query_function)
        {
          properties_have_query_function = true;
        }
        cc = 0;
      }
    }
    catch (Exception e)
    {
      cc = TErrorList.non_existent_elem;
    }
    if (cc == TErrorList.illegal_format) return null; // legacy server
    if (cc == 0)
    {
      if (properties[0].name.length() == 0) return null; // release 3.3 java problem
      for (i=0,n=0; i<properties.length; i++) if (properties[i].name.length() > 0) n++;
      strprops = new String[n];
      for (i=0,j=0; j<n && i<properties.length; i++) 
      {
        if (properties[i].name.length() == 0) continue;
        strprops[j++] = properties[i].name;
      }
      return strprops;
    }
    throw new RuntimeException("Could not acquire property information for " + server + " (" + TErrorList.getErrorString(cc) + ")");
  }
  private static String getTineStockString(String context,String server,String property,String text)
  {
    TLink tl;
    TDataType dout;
    String tgt;
    int cc = 0;
    StringBuffer ver = new StringBuffer(32);
  
    tgt = new String("/" + context + "/" + server + "/#0");
    dout = new TDataType(ver);
    try
    {
      tl = new TLink(tgt,property,dout,null,TAccess.CA_READ);
      cc = tl.execute(TLink.defaultTimeout,true);
      tl.cancel();
    }
    catch (Exception e)
    {
      cc = TErrorList.non_existent_elem;
    }
    if (cc == 0)
    {
      return text + " : " + ver.toString().trim();
    }
    return text + " : unavailable";
  }
  public static String getTineVersion(String context,String server)
  {
    return getTineStockString(context,server,"SRVVERSION","tine version");
  }
  public static String getAppVersion(String context,String server)
  {
    return getTineStockString(context,server,"APPVERSION","Appl version");
  }
  public static String getAppDate(String context,String server)
  {
    return getTineStockString(context,server,"APPDATE","Appl date");
  }
  /**
   * @brief Returns a list of stock properties associated with the specified
   * context and device server (obtained from the device server).
   * 
   * @param context Is the context of the device server for which the property list is
   * desired.
   * @param server Is the device server for which the property list is
   * desired.
   * 
   * @return A string array containing a list of stock properties
   * as obtained from the given device server
   */
  public static String[] getStockProperties(String context,String server)
  {
    return getStockProperties(context,server,"#0",null);
  }
  /**
   * @brief Returns a list of stock properties associated with the specified
   * context and device server, device, and property (sub)string 
   * (obtained from the device server).  Typically, the device parameter does not 
   * play a role in this query.
   * 
   * @param context Is the context of the device server for which the property list is
   * desired.
   * @param server Is the device server for which the property list is
   * desired.
   * @param device Is the device name (module name) for which the property list
   * is desired.
   * @param property Can be a queriable string with wildcard 
   * parameter '*'.  e.g. property = 'P*' will return all stock properties whose
   * first letter begins with 'P'.
   * 
   * @return A string array containing a list of stock properties
   * as obtained from the given device server
   */
  public static String[] getStockProperties(String context,String server,String device,String property)
  {
    TLink tl;
    TDataType dout;
    TDataType din;
    String tgt;
    int cc = 0,n,i,j;
    NAME32[] properties;
    String[] strprops;

    tgt = new String("/" + context + "/" + server + "/" + device);
    n = getNumberOf("STOCKPROPS",tgt);
    if (n <= 0) return null;
    if (n*32 > maxQueryBufferSize) n = maxQueryBufferSize/32;
    properties = new NAME32[n];
    for (i=0; i<n; i++) properties[i] = new NAME32();
    if (properties.length <= 0)
    {
      throw new RuntimeException("properties argument has zero length!");    
    }
    dout = new TDataType(properties);
    if (property != null) din = new TDataType(property); else din = null;
    properties_have_query_function = false;
    tl = new TLink(tgt,"STOCKPROPS",dout,din,TAccess.CA_READ);
    cc = tl.execute(TLink.defaultTimeout,true);
    if ((cc & TErrorList.CE_SENDDATA) == TErrorList.CE_SENDDATA)
    {
      if ((cc & TErrorList.has_query_function) == TErrorList.has_query_function)
      {
        properties_have_query_function = true;
      }
      cc = 0;
    }
    tl.cancel();
    if (cc == 0)
    {
      for (i=0,n=0; i<properties.length; i++) if (properties[i].name.length() > 0) n++;
      strprops = new String[n];
      for (i=0,j=0; j<n && i<properties.length; i++) 
      {
        if (properties[i].name.length() == 0) continue;
        strprops[j++] = properties[i].name;
      }
      return strprops;
    }
    else if (cc != TErrorList.link_not_open)
    {
      if (lastQueriedContext == context && lastQueriedServer == server &&
          lastQueriedDevice == device && lastQueriedServer != null)
      {
        cc = 0;
      }
      else
      {
        hLegacyByteBlob = new byte[n * PropertyQuery.sizeInBytes];
        dout = new TDataType(hLegacyByteBlob,"");
        tl = new TLink(tgt,"STOCKPROPS",dout,din,TAccess.CA_READ);
        cc = tl.execute(TLink.defaultTimeout,false);
        tl.cancel();
      }
      if (cc == 0)
      {
        lastQueriedContext = context;
        lastQueriedServer = server;
        lastQueriedDevice = device;
        strprops = new String[n];
        PropertyQuery[] pq = new PropertyQuery[n];
        byte b[];
        if (hLegacyByteBlob == null) return null;
        for (i=0; i<n; i++)
        {
          pq[i] = new PropertyQuery();
          b = pq[i].toByteArray();
          System.arraycopy(hLegacyByteBlob,i*PropertyQuery.sizeInBytes,b,0,PropertyQuery.sizeInBytes);
          pq[i].toStruct();
          strprops[i] = pq[i].name;
        }        
        return strprops;
      }
    }
    throw new RuntimeException("Could not acquire property information for " + server + " (" + TErrorList.getErrorString(cc) + ")");
  }
  /**
   * @brief Returns a list of device names associated with the specified
   * context and device server 
   * (obtained from the device server)
   *
   * As the call is directed to the device server, it assumes the return 
   * list applies to all properties
   *  
   * @param context Is the context of the device server for which the property list is
   * desired.
   * @param server Is the device server for which the property list is
   * desired.
   * 
   * @return A string array containing a list of devices
   * as obtained from the given device server
   */
  public static String[] getDeviceNames(String context,String server)
  {
    return getDeviceNames(context,server,null);
  }
  /**
   * @brief Returns a list of device names associated with the specified
   * context, device server and property 
   * (obtained from the device server)
   *
   * @param context Is the context of the device server for which the device list is
   * desired.
   * @param server Is the device server for which the device list is
   * desired.
   * @param property Is the property for which the device list is
   * desired.
   * 
   * @return A string array containing a list of devices
   * as obtained from the given device server
   */
  public static String[] getDeviceNames(String context,String server,String property)
  {
    return getDeviceNames(context,server,property,TLink.defaultTimeout);
  }
  public static String[] getDeviceNames(String context,String server,String property,int timeout)
  {
    String[] devs = getDeviceNames64(context,server,property,timeout);
    if (devs != null) return devs;
    return getDeviceNames16(context,server,property,timeout);
  }
  private static String[] getDeviceNames16(String context,String server,String property,int timeout)
  {
    DeviceNamesQueryParams dnqp = getDeviceNamesQueryProperty(context,server,property,timeout);
    if (dnqp == null) return null;
    int n = dnqp.size;

    TLink tl;
    TDataType dout;
    int cc = 0,i,j;
    String tmp;
    NAME16[] devices;
    String[] strdevs;

    try
    {
      if (n*16 > maxQueryBufferSize) n = maxQueryBufferSize/16;
      devices = new NAME16[n];
      for (i=0; i<n; i++) devices[i] = new NAME16();
      if (devices.length == 0) return null;
      if (n == 1)
      { // server claimed not to have device names ?
        devices[0].name = "#0"; 
      }
      dout = new TDataType(devices);
      tl = new TLink(dnqp.target,dnqp.property,dout,null,TAccess.CA_READ);
      cc = tl.execute(timeout,true);
      tl.cancel();
      
      if ((cc & TErrorList.CE_SENDDATA) == TErrorList.CE_SENDDATA)
      {
        if ((cc & TErrorList.has_query_function) == TErrorList.has_query_function)
        {
          devices_have_query_function = true;
        }
        cc = 0;
      }
      if (cc != 0) return null;
      // check for long names (the Channel Access kluge)
      for (i=0,n=0; i<devices.length; i++) 
        if (devices[i].name.length() > 0 && !devices[i].name.endsWith("&")) n++;
      strdevs = new String[n];
      for (i=0,j=0; i<devices.length && j<n; i++) 
      {
        tmp = devices[i].name;
        if (tmp.length() == 0) continue;
        if (tmp.endsWith("&"))
        {
          i++;
          tmp = tmp.substring(0, tmp.length() - 1) + devices[i].name;
        }
        strdevs[j++] = tmp;
      }
      return strdevs;
    }
    catch (Exception e)
    {
      return null;
    }
  }
  private static String getNamesQueryTarget(String context,String server,String device)
  {
    String tgt, dev;
    if (server == null || server.length() == 0)
    {
      throw new RuntimeException("empty server name entry !");
    }
    dev = (device == null) ? "#0" : device;
    if (context == null || context.length() == 0)
      tgt = new String("/DEFAULT/" + server + "/" + dev);
    else
      tgt = new String("/" + context + "/" + server + "/" + dev);
    return tgt;
  }
  private class DeviceNamesQueryParams
  {
    String target;
    String property;
    int size;
  }
  private static TQuery tQueryInstance = new TQuery();
  private static TQuery getInstance()
  {
    if (tQueryInstance == null) tQueryInstance = new TQuery();
    return tQueryInstance;
  }
  private static DeviceNamesQueryParams getDeviceNamesQueryProperty(String context,String server,String property,int timeout)
  {
    TQuery tq = getInstance();
    String tgt = getNamesQueryTarget(context,server,null);
    DeviceNamesQueryParams dnqp = tq.new DeviceNamesQueryParams();
    int n = 0;
    String prp=null;
    PropertyQueryEx[] pqx;
    try
    {
      n = getNumberOf("DEVICES",tgt,timeout);
      if (property != null && property.length() > 0)
      {
        devices_have_query_function = false;
        if ((pqx=getDevicePropertyInformation(context,server,"#0",property,timeout)) != null)
        { // this should always work
          if (pqx[0].prpSize > n) n = pqx[0].prpSize;    
          prp = property + ".NAM";
        }
      }
      if (prp == null)
      { // fallback :
        devices_have_query_function = false;
        if (n < 0) return null;
        n++;
        prp = "DEVICES"; 
      }
      dnqp.target = tgt;
      dnqp.property = prp;
      dnqp.size = n;
      return dnqp;
    }
    catch (Exception e)
    {
      return null;
    }
  }
  private static String[] getDeviceNames64(String context,String server,String property,int timeout)
  {
    DeviceNamesQueryParams dnqp = getDeviceNamesQueryProperty(context,server,property,timeout);
    if (dnqp == null) return null;
    
    TLink tl;
    TDataType dout;
    int cc = 0,i;
    NAME64[] devices;
    String[] strdevs;
    int n = dnqp.size;

    try
    {
      if (n*64 > maxQueryBufferSize) n = maxQueryBufferSize/64;
      devices = new NAME64[n];
      for (i=0; i<n; i++) devices[i] = new NAME64();
      if (devices.length == 0) return null;
      dout = new TDataType(devices);
      tl = new TLink(dnqp.target,dnqp.property,dout,null,TAccess.CA_READ);
      cc = tl.execute(timeout,true);
      tl.cancel();
        
      if ((cc & TErrorList.CE_SENDDATA) == TErrorList.CE_SENDDATA)
      {
        if ((cc & TErrorList.has_query_function) == TErrorList.has_query_function)
        {
          devices_have_query_function = true;
        }
        cc = 0;
      }
      if (cc != 0) return null;
      // check for empty names (java pre-release 4.0 specialty)
      if (devices[0].name.length() == 0) return null;
      for (i=0,n=0; i<devices.length; i++) 
      { // recount the returned list 
        if (devices[i].name.length() > 0) n++;
      }
      strdevs = new String[n];
      for (i=0; i<n; i++) 
      {
        if (devices[i].name.length() == 0) continue;
        strdevs[i] = devices[i].name;
      }
      return strdevs;
    }
    catch (Exception e)
    {
      return null;
    }
  }
  
  /**
   * @brief Returns a list of extended property query information objects associated
   * with the specified context, device server, device. and target property 
   * (obtained from the device server).
   *
   * Usually a call to getDeviceProperties() returns a property list
   * A secondary call to getDevicePropertyInformation() returns a list of 
   * all property information pertaining to the specified property including
   * all property overloads (maximum 10).  If 'property' is null then the
   * call returns information for the first 10 items.
   *  
   * @param context Is the context of the device server for which the property
   * information is desired.
   * @param server Is the device server for which the property information is
   * desired.
   * @param device Is the device name (module name) for which the property 
   * information is desired. 
   * @param property Is the property for which the property information is
   * desired.
   * 
   * @return An array of PropertyQueryEx (extended Property Query) objects,
   * as obtained from the given device server
   */
  public static PropertyQueryEx[] getDevicePropertyInformation(String context,String server,String device,String property)
  {
    return getDevicePropertyInformation(context,server,device,property,TLink.defaultTimeout);
  }
  public static synchronized PropertyQueryEx[] getDevicePropertyInformation(String context,String server,String device,String property,int timeout)
  {
    XPropertyQuery[] xpq = getDevicePropertyInformationX(context, server, device, property, timeout);
    if (xpq == null) return null;
    PropertyQueryEx[] pqx = new PropertyQueryEx[xpq.length];
    for (int i=0; i<pqx.length; i++) pqx[i] = new PropertyQueryEx(xpq[i]);
    return pqx;
  }
  public static synchronized int AcquireAndRegisterStructInfo(String context,String server,String tag)
  {
    if (TStructRegistry.contains(tag)) return 0; // already registered
    int cc = 0;
    INTINT[] sf = new INTINT[64];
    TDataType din = new TDataType(tag);
    TDataType dout = new TDataType(sf);
    String tgt;
    if (context == null || context.length() == 0)
      tgt = new String("/DEFAULT/" + server + "/#0");
    else
      tgt = new String("/" + context + "/" + server + "/#0");
    TLink tl = new TLink(tgt,"STRUCTFORMAT",dout,din,TAccess.CA_READ);
    cc = tl.execute(500,true);
    if ((cc & TErrorList.CE_SENDDATA) == TErrorList.CE_SENDDATA) cc = 0;
    tl.cancel();
    if (cc == 0)
    {
      for (int i=0; i<64; i++)
      {
        if ((sf[i].i2val%512) == TFormat.CF_NULL) break;
        TStructRegistry.fill(tag,(short)sf[i].i2val,sf[i].i1val);
      }
      TStructRegistry.fill(tag,(short)TFormat.CF_NULL,100);
    }
    return cc;
  }
  /**
   * @brief Returns a list of extended property query information objects associated
   * with the specified context, device server, device. and target property 
   * (obtained from the device server).
   *
   * Usually a call to getDeviceProperties() returns a property list
   * A secondary call to getDevicePropertyInformationX() returns a list of 
   * all property information pertaining to the specified property including
   * all property overloads (maximum 10).  If 'property' is null then the
   * call returns information for the first 10 items.  This is an extended
   * method call which returns information as to the array 'type' if the 
   * property returns an array, as well as the horizontal axis engineering
   * units and range if the property returns a spectrum and the row size
   * and number or rows if the property returns a double array (matrix).
   *  
   * @param context Is the context of the device server for which the property
   * information is desired.
   * @param server Is the device server for which the property information is
   * desired.
   * @param device Is the device name (module name) for which the property 
   * information is desired. 
   * @param property Is the property for which the property information is
   * desired.
   * 
   * @return An array of XPropertyQuery (extended Property Query) objects,
   * as obtained from the given device server
   */
  public static XPropertyQuery[] getDevicePropertyInformationX(String context,String server,String device,String property)
  {
    return getDevicePropertyInformationX(context,server,device,property,TLink.defaultTimeout);
  }
  public static synchronized XPropertyQuery[] getDevicePropertyInformationX(String context,String server,String device,String property,int timeout)
  {
    TLink tl;
    TDataType dout, din;
    int cc = 0,i;
    String tgt, dev;
    XPropertyQuery[] xpq = null;
    XPropertyQuery xpq1 = new XPropertyQuery();

    if (server == null || server.length() == 0)
    {
      throw new RuntimeException("empty server name entry !");
    }
    if (device == null || device.length() == 0)
      dev = new String("#0");
    else
      dev = device;
    if (context == null || context.length() == 0)
      tgt = new String("/DEFAULT/" + server + "/" + dev);
    else
      tgt = new String("/" + context + "/" + server + "/" + dev);
    dout = new TDataType(hByteBlobX,"XPQS");
    din = new TDataType(property);
    try 
    {
      tl = new TLink(tgt,"PROPS",dout,din,TAccess.CA_READ);
      cc = tl.execute(timeout,true);
      modAddr = tl.modAddress;
      if ((cc & TErrorList.CE_SENDDATA) == TErrorList.CE_SENDDATA) cc = 0;
      tl.cancel();
    }
    catch (Exception e)
    {
      cc = TErrorList.non_existent_elem;
    }
    if (cc == 0) // it's at lease a 3.31 server
    {      
      try {
        TStructIo.bytesToStruct(xpq1, hByteBlobX,0,XPropertyQuery.sizeInBytes);
      } catch (IOException e1) {
        // XXX Auto-generated catch block
        e1.printStackTrace();
      }
      
      if (xpq1.prpNumOverloads < 1) xpq1.prpNumOverloads = 1; // can't be less than 1 !
      int nxpq = xpq1.prpNumOverloads;
      if (nxpq > nicePropertyQuerySize) nxpq = nicePropertyQuerySize;
      xpq = new XPropertyQuery[nxpq];
      xpq[0] = xpq1;
      for (i=1; i<nxpq; i++)
      {
        xpq[i] = new XPropertyQuery();
        try {
          TStructIo.bytesToStruct(xpq[i],hByteBlobX,i*XPropertyQuery.sizeInBytes,XPropertyQuery.sizeInBytes);
        } catch (Exception e) {
          // XXX Maybe do something
        }
      }
      return xpq;
    }
    if (cc == TErrorList.link_not_open)
    {
      throw new RuntimeException("Could not acquire property information for " + server + " (" + TErrorList.getErrorString(cc) + ")");
    }
    if (cc == TErrorList.non_existent) return null;
    PropertyQueryLegacy[] pqx = null;
    PropertyQueryLegacy pqx1 = new PropertyQueryLegacy();
    dout = new TDataType(hByteBlob,"PQSX");
    tl = new TLink(tgt,"PROPS",dout,din,TAccess.CA_READ);
    cc = tl.execute(timeout,true);
    tl.cancel();
    if (cc == 0) // it's at lease a 3.20 server
    {
      byte b[] = pqx1.toByteArray();
      System.arraycopy(hByteBlob,0,b,0,PropertyQueryLegacy.sizeInBytes);
      pqx1.toStruct();
      if (pqx1.prpNumOverloads < 1) pqx1.prpNumOverloads = 1; // can't be less than 1 !
      int npqx = pqx1.prpNumOverloads;
      if (npqx > nicePropertyQuerySize) npqx = nicePropertyQuerySize;
      pqx = new PropertyQueryLegacy[npqx];
      xpq = new XPropertyQuery[npqx];
      pqx[0] = pqx1;
      for (i=1; i<npqx; i++)
      {
        pqx[i] = new PropertyQueryLegacy();
        b = pqx[i].toByteArray();
        System.arraycopy(hByteBlob,i*PropertyQueryLegacy.sizeInBytes,b,0,PropertyQueryLegacy.sizeInBytes);
        pqx[i].toStruct();
      }
      for (i=0; i<npqx; i++)
      {
        xpq[i] = new XPropertyQuery();
        xpq[i].prpFormat = pqx[i].prpFormat;
        xpq[i].prpFormatIn = pqx[i].prpFormatIn;
        xpq[i].prpSize = pqx[i].prpSize;
        xpq[i].prpSizeIn = pqx[i].prpSizeIn;
        xpq[i].prpAccess = pqx[i].prpAccess;
        xpq[i].prpDescription = pqx[i].prpDescription;
        xpq[i].prpName = pqx[i].prpName;
        xpq[i].prpNumOverloads = pqx[i].prpNumOverloads;
        xpq[i].prpTag = pqx[i].prpTag;
        xpq[i].prpTagIn = pqx[i].prpTagIn;
        xpq[i].prpRedirection = pqx[i].prpRedirection;
        xpq[i].prpHistoryDepthLong = pqx[i].prpHistoryDepthLong;
        xpq[i].prpHistoryDepthShort = pqx[i].prpHistoryDepthShort;
        xpq[i].prpUnits = pqx[i].prpUnits;
        xpq[i].prpMaxValue = pqx[i].prpMaxValue;
        xpq[i].prpMinValue = pqx[i].prpMinValue;
        xpq[i].prpGraphType = pqx[i].prpGraphType;
      }
      return xpq;
    }
    if (cc == TErrorList.link_not_open)
    {
      throw new RuntimeException("Could not acquire property information for " + server + " (" + TErrorList.getErrorString(cc) + ")");
    }
    // legacy server
    hLegacyByteBlob = null;
    String[] prps = getDeviceProperties(context,server,device);
    if (prps == null || prps.length == 0)
    {
      throw new RuntimeException("empty properties list !");
    }
    PropertyQuery[] pq = new PropertyQuery[prps.length];
    xpq = new XPropertyQuery[1];
    byte b[];
    if (hLegacyByteBlob == null) return null;
    for (i=0; i<prps.length; i++)
    {
      pq[i] = new PropertyQuery();
      b = pq[i].toByteArray();
      System.arraycopy(hLegacyByteBlob,i*PropertyQuery.sizeInBytes,b,0,PropertyQuery.sizeInBytes);
      pq[i].toStruct();
      if (pq[i].name.compareTo(property) != 0) continue;
      xpq[0] = new XPropertyQuery();
      xpq[0].prpFormat = pq[i].prpFormat;
      xpq[0].prpSize = pq[i].prpSize;
      xpq[0].prpAccess = pq[i].prpAccess;
      xpq[0].prpDescription = pq[i].prpDesc;
      xpq[0].prpName = pq[i].name;
      xpq[0].prpNumOverloads = 1;
      if ((pq[0].prpAccess & TAccess.CA_WRITE) == TAccess.CA_WRITE)
      {
         xpq[0].prpFormatIn = pq[i].prpFormat;
         xpq[0].prpSizeIn = pq[i].prpSize;
      }
      else     
      {
         xpq[0].prpFormatIn = (byte)TFormat.CF_NULL;
         xpq[0].prpSizeIn = 0;
      }
      USTRING[] egu = new USTRING[1];
      egu[0] = new USTRING();
      dout = new TDataType(egu);
      din = new TDataType();
      String prpegu = new String(property + ".EGU");
      tl = new TLink(tgt,prpegu,dout,din,TAccess.CA_READ);
      cc = tl.execute(timeout,false);
      tl.cancel();
      if (cc == 0)
      {
        xpq[0].prpMinValue = egu[0].f1val;
        xpq[0].prpMaxValue = egu[0].f2val;
        xpq[0].prpUnits = egu[0].str;
        xpq[0].prpGraphType = (byte)egu[0].ival;  
      }
      break;
    }        
    return xpq;
  }
  /**
   * @brief Returns a list of extended property query information objects associated
   * with the specified context, device server, device. and target property 
   * (obtained from the device server).  Typically, the device name is ignored
   * in this query.
   *
   * Usually a call to getStockProperties() returns a property list
   * A secondary call to getStockPropertyInformationX() returns a list of 
   * all property information pertaining to the specified property including
   * all property overloads (maximum 10).  If 'property' is null then the
   * call returns information for the first 10 items.  This is an extended
   * method call which returns information as to the array 'type' if the 
   * property returns an array, as well as the horizontal axis engineering
   * units and range if the property returns a spectrum and the row size
   * and number or rows if the property returns a double array (matrix).
   *  
   * @param context Is the context of the device server for which the property
   * information is desired.
   * @param server Is the device server for which the property information is
   * desired.
   * @param device Is the device name (module name) for which the property 
   * information is desired. 
   * @param property Is the property for which the property information is
   * desired.
   * 
   * @return An array of XPropertyQuery (extended Property Query) objects,
   * as obtained from the given device server
   */
  public static synchronized XPropertyQuery[] getStockPropertyInformationX(String context,String server,String device,String property)
  {
    TLink tl;
    TDataType dout, din;
    int cc = 0,i;
    String tgt;
    XPropertyQuery[] xpq = null;
    XPropertyQuery xpq1 = new XPropertyQuery();

    if (context == null || context.length() == 0)
      tgt = new String("/DEFAULT/" + server + "/" + device);
    else
      tgt = new String("/" + context + "/" + server + "/" + device);
    dout = new TDataType(hByteBlobX,"XPQS");
    din = new TDataType(property);
    tl = new TLink(tgt,"STOCKPROPS",dout,din,TAccess.CA_READ);
    cc = tl.execute(TLink.defaultTimeout,true);
    modAddr = tl.modAddress;
    if ((cc & TErrorList.CE_SENDDATA) == TErrorList.CE_SENDDATA) cc = 0;
    tl.cancel();
    if (cc == 0) // it's at lease a 3.31 server
    {
      
      xpq1.readBytes(hByteBlobX,0,XPropertyQuery.sizeInBytes);
      if (xpq1.prpNumOverloads < 1) xpq1.prpNumOverloads = 1; // can't be less than 1 !
      int nxpq = xpq1.prpNumOverloads;
      if (nxpq > nicePropertyQuerySize) nxpq = nicePropertyQuerySize;
      xpq = new XPropertyQuery[nxpq];
      xpq[0] = xpq1;
      for (i=1; i<nxpq; i++)
      {
        xpq[i] = new XPropertyQuery();
        xpq[i].readBytes(hByteBlobX,i*XPropertyQuery.sizeInBytes,XPropertyQuery.sizeInBytes);
      }
      return xpq;
    }
    
    if (cc == TErrorList.link_not_open)
    {
      throw new RuntimeException("Could not acquire property information for " + server + " (" + TErrorList.getErrorString(cc) + ")");
    }
    PropertyQueryLegacy[] pqx = null;
    PropertyQueryLegacy pqx1 = new PropertyQueryLegacy();
    dout = new TDataType(hByteBlob,"");
    tl = new TLink(tgt,"STOCKPROPS",dout,din,TAccess.CA_READ);
    cc = tl.execute(TLink.defaultTimeout,true);
    tl.cancel();
    if (cc == 0) // it's at lease a 3.20 server
    {
      byte b[] = pqx1.toByteArray();
      System.arraycopy(hByteBlob,0,b,0,PropertyQueryLegacy.sizeInBytes);
      pqx1.toStruct();
      if (pqx1.prpNumOverloads < 1) pqx1.prpNumOverloads = 1; // can't be less than 1 !
      int npqx = pqx1.prpNumOverloads;
      if (npqx > nicePropertyQuerySize) npqx = nicePropertyQuerySize;
      pqx = new PropertyQueryLegacy[npqx];
      xpq = new XPropertyQuery[npqx];
      pqx[0] = pqx1;
      for (i=1; i<npqx; i++)
      {
        pqx[i] = new PropertyQueryLegacy();
        b = pqx[i].toByteArray();
        System.arraycopy(hByteBlob,i*PropertyQueryLegacy.sizeInBytes,b,0,PropertyQueryLegacy.sizeInBytes);
        pqx[i].toStruct();
      }
      for (i=0; i<npqx; i++)
      {
        xpq[i] = new XPropertyQuery();
        xpq[i].prpFormat = pqx[i].prpFormat;
        xpq[i].prpFormatIn = pqx[i].prpFormatIn;
        xpq[i].prpSize = pqx[i].prpSize;
        xpq[i].prpSizeIn = pqx[i].prpSizeIn;
        xpq[i].prpAccess = pqx[i].prpAccess;
        xpq[i].prpDescription = pqx[i].prpDescription;
        xpq[i].prpName = pqx[i].prpName;
        xpq[i].prpNumOverloads = pqx[i].prpNumOverloads;
        xpq[i].prpTag = pqx[i].prpTag;
        xpq[i].prpTagIn = pqx[i].prpTagIn;
        xpq[i].prpRedirection = pqx[i].prpRedirection;
        xpq[i].prpHistoryDepthLong = pqx[i].prpHistoryDepthLong;
        xpq[i].prpHistoryDepthShort = pqx[i].prpHistoryDepthShort;
        xpq[i].prpUnits = pqx[i].prpUnits;
        xpq[i].prpMaxValue = pqx[i].prpMaxValue;
        xpq[i].prpMinValue = pqx[i].prpMinValue;
        xpq[i].prpGraphType = pqx[i].prpGraphType;
      }
      return xpq;
    }
    if (cc == TErrorList.link_not_open)
    {
      throw new RuntimeException("Could not acquire property information for " + server + " (" + TErrorList.getErrorString(cc) + ")");
    }
    // legacy server
    String[] prps = getDeviceProperties(context,server,device);
    if (prps == null) return null;
    PropertyQuery[] pq = new PropertyQuery[prps.length];
    xpq = new XPropertyQuery[1];
    byte b[];
    if (hLegacyByteBlob == null) return null;
    for (i=0; i<prps.length; i++)
    {
      pq[i] = new PropertyQuery();
      b = pq[i].toByteArray();
      System.arraycopy(hLegacyByteBlob,i*PropertyQuery.sizeInBytes,b,0,PropertyQuery.sizeInBytes);
      pq[i].toStruct();
      if (pq[i].name.compareTo(property) != 0) continue;
      xpq[0] = new XPropertyQuery();
      xpq[0].prpFormat = pq[i].prpFormat;
      xpq[0].prpSize = pq[i].prpSize;
      xpq[0].prpAccess = pq[i].prpAccess;
      xpq[0].prpDescription = pq[i].prpDesc;
      xpq[0].prpName = pq[i].name;
      xpq[0].prpNumOverloads = 1;
      if ((pq[0].prpAccess & TAccess.CA_WRITE) == TAccess.CA_WRITE)
      {
         xpq[0].prpFormatIn = pq[i].prpFormat;
         xpq[0].prpSizeIn = pq[i].prpSize;
      }
      else     
      {
         xpq[0].prpFormatIn = (byte)TFormat.CF_NULL;
         xpq[0].prpSizeIn = 0;
      }
      USTRING[] egu = new USTRING[1];
      egu[0] = new USTRING();
      dout = new TDataType(egu);
      din = new TDataType();
      String prpegu = new String(property + ".EGU");
      tl = new TLink(tgt,prpegu,dout,din,TAccess.CA_READ);
      cc = tl.execute(TLink.defaultTimeout,false);
      tl.cancel();
      if (cc == 0)
      {
        xpq[0].prpMinValue = egu[0].f1val;
        xpq[0].prpMaxValue = egu[0].f2val;
        xpq[0].prpUnits = egu[0].str;
        xpq[0].prpGraphType = (byte)egu[0].ival;  
      }
      break;
    }        
    return xpq;
  }
  private static ServerQuery[] getXTagList(String context,String tagtype,String subsys,String importance)
  {
    return getXTagList(context,tagtype,subsys,importance,TLink.defaultTimeout);
  }
  private static ServerQuery[] getXTagList(String context,String tagtype,String subsys,String importance,int timeout)
  {
    int cc,n,i;
    TDataType dout;
    TDataType din;
    TLink tl;
    StringBuffer host = new StringBuffer(32);
    StringBuffer query = new StringBuffer(32);
    short[] numout = new short[1];
    TDataType numoutData = new TDataType(numout);
    NAME16[] inplist = new NAME16[2];
    USTRING[] taglist;
    ServerQuery[] strlist;

    if (subsys == null || subsys.length() == 0)
    {
      inplist[0] = new NAME16("ALL");
    }
    else
    {
      inplist[0] = new NAME16(subsys);
    }
    if (importance == null || importance.length() == 0)
    {
      inplist[1] = new NAME16("ALL");    
    }
    else
    {
      inplist[1] = new NAME16(importance);
    }
    din = new TDataType(inplist);
    
    host.delete(0,31); query.delete(0,31);
    if (context != null && context.length() != 0)
    {
      host.insert(0,"ENS/" + context);
    }
    else 
    {
      host.insert(0,"ENS");
    }
    query.insert(0,"N" + tagtype);
    try 
    {
      tl = new TLink(host.toString(),query.toString(),numoutData,din,TAccess.CA_READ);
      cc = tl.execute(timeout,true);
      modAddr = tl.modAddress;
      tl.cancel();
    }
    catch (Exception e)
    {
      cc = TErrorList.non_existent_elem;
    }
    if (cc != 0) return null;
    // Get Tags from name server (synchronous call)
    query.delete(0,31);
    query.insert(0,tagtype);
    n = numout[0];
    taglist = new USTRING[n];
    for (i=0; i<n; i++) taglist[i] = new USTRING();
    if (taglist.length == 0) return null;
    if (numout[0] > taglist.length) numout[0] = (short)taglist.length;  
    dout = new TDataType(taglist);
    tl = new TLink(host.toString(),query.toString(),dout,din,TAccess.CA_READ);
    cc = tl.execute(TLink.defaultTimeout,true);
    tl.cancel();
    if (cc != 0) return null;
    strlist = new ServerQuery[n];
    for (i=0; i<n; i++) 
    {
      String s = taglist[i].getString();
      strlist[i] = new ServerQuery();
      strlist[i].setName(s.substring(0,16).trim()); 
      strlist[i].setOs(s.substring(16,24).trim()); 
      strlist[i].setSubsystem(s.substring(24,32).trim()); 
      strlist[i].setXref(s.substring(32,48).trim());
      strlist[i].setContext(context);
    }
    return strlist;
  }
}

