/*
 * Created on Nov 9, 2004
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package de.desy.tine.server.connections;

import java.io.DataInput;
import java.io.DataOutput;
import java.io.IOException;
import java.util.LinkedList;
import de.desy.tine.dataUtils.TDataType;
import de.desy.tine.definitions.TErrorList;
import de.desy.tine.definitions.TFormat;
import de.desy.tine.headers.TContract;
import de.desy.tine.server.equipment.TEquipmentModule;
import de.desy.tine.stringUtils.StringToBytes;
import de.desy.tine.structUtils.TStructBase;
import de.desy.tine.structUtils.TStructDescription;
import de.desy.tine.structUtils.TStructRegistry;

/**
 * \internal
 * 
 * @author duval
 */
public class TContractTable extends TStructBase
{
  // below should be a linked list (ArrayList ?)
  // TClientEntry[] clt;
  public LinkedList clt = new LinkedList();/* assciated client list */
  public int nclients; /* number of clients for this contract */
  public int contractID; /* local contract ID */
  public TContract contract; /* see above */
  public int pollingRate; /* in mseconds, smallest number wins ... */
  public long lasttime; /* most recent time the data polled */
  public long lastmarked; /* last time the data were marked STALE */
  public long starttime; /* start time of the current call */
  public long dtimestamp; /* data timestamp */
  // is there a thread-safe problem with the data out buffer ?
  public TDataType dout; /* prepared output DTYPE */
  public TDataType drb; // readback output DTYPE, i.e. the result of the call
  public TDataType din; /* prepared input DTYPE */
  public int dataSizeOut; /* output data size < contract.dataSize */
  public int pending; /* pending flag for DeliverData() vs ProduceData() */
  public boolean expired; /* expired flag for contract removal */
  public int compStatus; /* completion status */
  public int returnCode; /* contract completion return code */
  public int delays; /* delayed scheduling due to pending */
  public int misses; /* missed scheduling (i.e. > 2x polling rate) */
  public String compString; /* completion string (redirection ?) */
  public TEquipmentModule eqm; /* associated export element */
  private static final int ARRAY_SIZE_HINT = 30;
  private static TStructDescription myStructDsc; // = new TStructDescription("CONQS");  //("XCTR");
  static
  {
    createQueryStruct();
  }
  public void setStale(byte staleFlag)
  {
    TClientEntry tce = null;
    for (int i = 0; i < clt.size(); i++)
    {
      tce = (TClientEntry) clt.get(i);
      tce.sts.Stale = staleFlag;
    }
  }
  public int renegotiateContract(short statusCode)
  {
    TClientEntry tce = null;
    int datasize = 0;
    // get datasize from drb (read-back data set)
    datasize = drb.getDataSize();
    if (statusCode != 0) datasize += TErrorList.ERROR_SIZE;
    for (int i = 0; i < clt.size(); i++)
    {
      tce = (TClientEntry) clt.get(i);
      tce.sts.BytesRemaining = datasize;
      tce.sts.BytesTotal = datasize;
      tce.sts.nextDataSize = tce.sts.BytesRemaining;
      if (tce.sts.BytesRemaining > tce.sts.mtu) tce.sts.nextDataSize = tce.sts.mtu;
      tce.sts.numblks = (short) (datasize / tce.sts.mtu + (datasize % tce.sts.mtu > 0 ? 1 : 0));
      if (datasize == 0) tce.sts.numblks = 1;
    }
    return 0;
  }
  /**
   * Creates a tagged structure description for TContractTable;
   */
  private static void createQueryStruct()
  {
    if (TStructRegistry.contains("CONQS")) return;
    myStructDsc = new TStructDescription("CONQS"); //("XCLNS");
    myStructDsc.beginDefinition();
    myStructDsc.addField(TFormat.CF_TEXT, 32);
    myStructDsc.addField(TFormat.CF_TEXT, 16);
    myStructDsc.addField(2, TFormat.CF_LONG, 1);
    myStructDsc.addField(4, TFormat.CF_BYTE, 1);
    myStructDsc.addField(2, TFormat.CF_TEXT, 8);
    myStructDsc.addField(2, TFormat.CF_LONG, 1);
    myStructDsc.setArraySize(ARRAY_SIZE_HINT);
    myStructDsc.endDefinition();
    TStructRegistry.add(myStructDsc);
  }
  /**
   * Returns tagged structure description.
   * 
   * @return
   */
  public static TStructDescription getStructDescription()
  {
    return myStructDsc;
  }
  /*
   * (non-Javadoc)
   * 
   * @see de.desy.tine.client.TStructIfc#readData(java.io.DataInput)
   */
  public void readData(DataInput din) throws IOException
  {
    // TODO Not implemented yet.
  }
  /*
   * (non-Javadoc)
   * 
   * @see de.desy.tine.client.TStructIfc#writeData(java.io.DataOutput)
   */
  public void writeData(DataOutput dout) throws IOException
  {
    dout.write(StringToBytes.stringToFixedBytes(contract.EqpProperty, 32));
    dout.write(StringToBytes.stringToFixedBytes(contract.EqpDeviceName, 16));
    dout.write(StringToBytes.stringToFixedBytes(contract.EqpName,8));
    dout.writeInt(contract.EqpSizeIn);
    // System.out.println(contract.EqpSizeIn + " --- " + contract.EqpSizeOut);
    dout.writeInt(contract.EqpSizeOut);
    dout.writeByte(contract.hEqpName);
    dout.writeByte(contract.EqpAccess);
    dout.writeByte(contract.EqpFormatIn);
    dout.writeByte(contract.EqpFormatOut);
    dout.write(StringToBytes.stringToFixedBytes(contract.strTagIn, 8));
    dout.write(StringToBytes.stringToFixedBytes(contract.strTagOut, 8));
    dout.writeInt(this.pollingRate);
    dout.writeInt(this.clt.size());
  }
}
