/*
 * Created on Aug 24, 2005
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package de.desy.tine.server.devices;

import java.util.LinkedList;
import de.desy.tine.dataUtils.TDataType;
import de.desy.tine.server.alarms.TAlarm;
import de.desy.tine.server.alarms.TAlarmDescriptor;
import de.desy.tine.server.equipment.TEquipmentModule;
import de.desy.tine.server.logger.TFecLog;
/**
 * \brief Instances of devices served by a de.desy.tine.client device server should inherit from TDevice
 *
 * @author duval
 *
 */
public class TDevice
{
  public int devNumber;
  public String devName;
  private LinkedList devAlarmList = new LinkedList();
  private int devAlarmMrt = 0; // Most Recent Timestamp
  private int devAlarmMrct = 0; // Most Recent Complete Timestamp
  private short devAlarmHsv = 0;
  private int nDevAlarmsAtMrt;
  private int nDevAlarmsAtMrct;
  private int nDevAlarmsAtHsv;
  private TEquipmentModule eqm = null;
  private String redirection = null;
  private LinkedList devPrpList = null;
  public LinkedList getPropertyList() { return devPrpList; }
  public void setPropertyList(LinkedList prpList) { devPrpList = prpList; }
  public int getHighestAlarmSeverity() { return devAlarmHsv; }
  public int getNumberOfAlarmsAtMostRecentTimestamp() { return nDevAlarmsAtMrct; }
  public int getNumberOfAlarmsAtHighestSeverity() { return nDevAlarmsAtHsv; }
  public TAlarm[] getAlarmList()
  {
    return (TAlarm[])devAlarmList.toArray(new TAlarm[0]);
  }
  private TAlarm getAlarmFromList(int code,byte[] data,short flags)
  { // called only by setAlarm() ...
    TAlarm alm = null;
    int listsiz = devAlarmList.size();
    
    for (int i=0; i<listsiz; i++)
    {
      if ((alm=(TAlarm)devAlarmList.get(i)).isTerminated()) continue;
      if (alm.getCode() == code) 
      {
        // remove NEW bit !
        byte dsc = alm.getDescriptor();
        byte[] adata = alm.getData();
        int dlen = alm.getAlmDef() == null ? 0 : alm.getAlmDef().getAlmDataArraySize();
        if (dlen > 0 && data != null && adata != null)
        { // check for alarm data changed
          for (int n=0; n<dlen; n++) 
            if (data[n] != adata[n])
            {
              dsc &= ~(byte)(TAlarmDescriptor.NEW);
              dsc |= (byte)TAlarmDescriptor.DATACHANGE;
              TFecLog.log(alm.getTag() + " : alarm data changed");
            }
        }
        if (alm.getClearCount() > 1)
        { // check for oscillating alarm ...
          dsc &= ~(byte)(TAlarmDescriptor.NEW);
          dsc |= (byte)TAlarmDescriptor.OSCILLATION;
          TFecLog.log(alm.getTag() + " : mark alarm as oscillating");
        }
        alm.resetClearCount();
        alm.setDescriptor(dsc); 
        return alm;
      }
    }
    // not found, so this is a new one ...
    // TODO: worry about 'max_alarms_exceeded', i.e. alarm storms and the length of this list!
    flags |= TAlarmDescriptor.NEW;
    if ((flags & TAlarmDescriptor.INSTANT) != 0) flags |= TAlarmDescriptor.TERMINATE;
    alm = new TAlarm(code,(byte)flags,data,this);
    if (eqm.casReadRequired) alm.blockRemoval();
    devAlarmList.add(alm);
    TFecLog.log(alm.getTag() + "(" + devName + ") : add alarm to list");
    return alm;
  }
  public int getNumberOfAlarms() { return devAlarmList.size(); }
  public int getMostRecentTimestamp() { return devAlarmMrct; }
  private void refreshMrt(int ts)
  {
    if (ts > devAlarmMrt) 
    {
      devAlarmMrt = ts;
      nDevAlarmsAtMrt = 1;
    }
    else if (ts == devAlarmMrt)
    {
      nDevAlarmsAtMrt++;
    }  
  }
  private void refreshHsv(short sv)
  {
    if (sv > devAlarmHsv)
    {
      devAlarmHsv = sv;
      nDevAlarmsAtHsv = 1;
    }
    else if (sv == devAlarmHsv)
    {
      nDevAlarmsAtHsv++;
    }
  }
  public void processAlarms()
  {
    int thisTimeStamp = (int)(System.currentTimeMillis()/1000);
    TAlarm alm = null;
    nDevAlarmsAtHsv = 0; devAlarmHsv = 0;
    nDevAlarmsAtMrt = 0; devAlarmMrt = 0; 
    nDevAlarmsAtMrct = 0; devAlarmMrct = 0;
    for (int i=0; i<devAlarmList.size();)
    {
      alm = (TAlarm)devAlarmList.get(i);
      // check alarm removal
      if (alm.isTerminated() &&  
          thisTimeStamp > alm.getTimeStamp() + alm.getTerminationWindow())
      {
        if (alm.isRemovalBlocked())
        {
          TFecLog.log(alm.getTag() + " : missing CAS read -> removal blocked");
        }
        else
        {
          TFecLog.log(alm.getTag() + " : remove alarm");
          devAlarmList.remove(i);
          continue;
        }
      }
      // apply termination flag if necessary ...
      if (!alm.isTerminated() &&
          alm.getClearCount() > 3 && 
          thisTimeStamp > alm.getTimeStamp() + alm.getTerminationWindow())
      {
        byte dsc = alm.getDescriptor();
        dsc &= ~(byte)TAlarmDescriptor.NEW;
        dsc |= (byte)TAlarmDescriptor.TERMINATE;
        alm.setDescriptor(dsc);
        alm.setTimeStamp(thisTimeStamp);
        refreshMrt(thisTimeStamp);
        alm.setTerminated(true);
        TFecLog.log(alm.getTag() + " : mark alarm as terminated");
      }
      // apply heartbeat if necessary
      if (thisTimeStamp > alm.getTimeStamp() + alm.getAlmHeartbeatWindow())
      {
        byte dsc = alm.getDescriptor();
        dsc &= ~(byte)TAlarmDescriptor.NEW;
        dsc |= (byte)TAlarmDescriptor.HEARTBEAT;
        alm.setDescriptor(dsc);
        alm.setTimeStamp(thisTimeStamp);
        refreshMrt(thisTimeStamp);
      }
      refreshMrt(alm.getTimeStamp());
      refreshHsv(alm.getSeverity());
      i++;
    }
    // adjust most recent completed timestamp (if necessary)
    if (thisTimeStamp > devAlarmMrt && devAlarmMrt > devAlarmMrct)
    {
      nDevAlarmsAtMrct = nDevAlarmsAtMrt;
      devAlarmMrct = devAlarmMrt;          
    }
  }
  /**
   * \brief sets an alarm with the given code
   * 
   * @param code The alarm code to apply to the alarm
   * @return
   */
  public TAlarm setAlarm(int code)
  {
    return setAlarm(code,null,TAlarmDescriptor.NEW);    
  }
  /**
   * \brief sets an alarm with the given data set
   * 
   * @param code The alarm code to apply to the alarm
   * @param data The alarm data associated with the alarm 
   * @return
   */
  public TAlarm setAlarm(int code,byte[] data)
  {
    return setAlarm(code,data,TAlarmDescriptor.NEW);
  }
  /**
   * \brief sets an alarm with the given data set
   * 
   * @param code The alarm code to apply to the alarm
   * @param data The alarm data associated with the alarm 
   * @return
   */
  public TAlarm setAlarm(int code,TDataType data)
  {
    return setAlarm(code,data.getDataBuffer(),TAlarmDescriptor.NEW);
  }
  /**
   * \brief sets an alarm with the given data set
   * 
   * @param code The alarm code to apply to the alarm
   * @param data The alarm data associated with the alarm 
   * @return
   */
  public TAlarm setAlarm(int code,float data)
  {
    float[] fdata = new float[1];
    fdata[0] = data;
    TDataType td = new TDataType(fdata);
    return setAlarm(code,td.getDataBuffer(),TAlarmDescriptor.NEW);
  }
  /**
   * \brief sets an alarm with the given data set
   * 
   * @param code The alarm code to apply to the alarm
   * @param data The alarm data associated with the alarm 
   * @return
   */
  public TAlarm setAlarm(int code,int data)
  {
    int[] idata = new int[1];
    idata[0] = data;
    TDataType td = new TDataType(idata);
    return setAlarm(code,td.getDataBuffer(),TAlarmDescriptor.NEW);
  }
  /**
   * \brief sets an alarm with the given data set
   * 
   * @param code The alarm code to apply to the alarm
   * @param data The alarm data associated with the alarm 
   * @return
   */
  public TAlarm setAlarm(int code,short[] data)
  {
    TDataType td = new TDataType(data);
    return setAlarm(code,td.getDataBuffer(),TAlarmDescriptor.NEW);
  }
  /**
   * \brief sets an alarm with the given data set and given flags
   * 
   * On rare occasions you might want to apply the alarm descriptor flags
   * yourself in a call to setAlarms().  
   * 
   * @param code The alarm code to apply to the alarm
   * @param data The alarm data associated with the alarm 
   * @param flags The alarm descriptor to apply to the alarm
   * @return
   */
  public TAlarm setAlarm(int code,byte[] data,short flags)
  {
    TAlarm alm = getAlarmFromList(code,data,flags);
    if (alm.isTerminated()) TFecLog.log(alm.getTag() + " : set alarm as terminated");
    return alm;
  }
  /**
   * \brief Clears all alarms in the local alarm server's list
   *
   */
  public void clearAlarms()
  {
    int asiz = devAlarmList.size();
    for (int i=0; i<asiz; i++) ((TAlarm)devAlarmList.get(i)).clear();
  }
  /**
   * \brief clears the alarm with the given code in the local alarm server's list
   * 
   * The alarm's clear counter will be incremented.  A call to clearAlarm() will
   * not cause the alarm to be removed from the list until the clear counter is
   * greater than 8.
   * 
   * @param code Is the alarm code specifying which alarm to clear 
   */
  public void clearAlarm(int code)
  {
    int asiz = devAlarmList.size();
    TAlarm alm = null;
    for (int i=0; i<asiz; i++) 
    {
      alm = (TAlarm)devAlarmList.get(i);
      if ((alm.getDescriptor() & TAlarmDescriptor.TERMINATE) != 0) continue;
      if (alm.getCode() != code) continue;
      alm.clear();
    }
  }
  /**
   * \brief removes the alarm with the given code from the local alarm server's list
   * 
   * @param code Is the alarm code specifying which alarm to remove from the 
   * local alarm server's list
   */
  public void removeAlarm(int code)
  {
    int asiz = devAlarmList.size();
    TAlarm alm = null;
    for (int i=0; i<asiz; i++) 
    {
      alm = (TAlarm)devAlarmList.get(i);
      if ((alm.getDescriptor() & TAlarmDescriptor.TERMINATE) != 0) continue;
      if (alm.getCode() != code) continue;
      alm.setDescriptor((byte)TAlarmDescriptor.TERMINATE);
      alm.clear();
    }
  }
  /**
   * \brief Simple constructor which assigns a device name and number
   * 
   * Use this constructor if you do not make use of a '<local-names>-devices.csv'
   * but instead either read your own database to determine the device name
   * name list.  You can assign a number and name to the device in question. 
   * 
   * @param name The device name to apply to this device
   * @param number The device number to apply to this device
   */
  public TDevice(String name, int number)
  {
    devName = name;
    devNumber = number;
  }
  /**
   * \brief Simple constructor which assigns a device number
   * 
   * Use this constructor if you make use of a '<local-names>-devices.csv' 
   * configuration file to register devices or if you intend on using the
   * setName() method to attach a name to the device.
   *  
   * @param number The device number associated with this device
   */
  public TDevice(int number)
  {
    devName = "#" + number;
    devNumber = number;
  }
  /**
   * \brief Complex constructor which assigns name, number, and equipment module
   * 
   * You should not normally need to make use of this constructor, as the
   * identity of the equipment module is usually determined elsewhere during
   * initialization.
   * 
   * @param name The device name associated with this device
   * @param number The device number associated with this device
   * @param equipmentModule The equipment module to associate with this device
   */
  public TDevice(String name, int number, TEquipmentModule equipmentModule)
  {
    devName = name;
    devNumber = number;
    eqm = equipmentModule;
  }
  public TEquipmentModule getEqm()
  {
    return eqm;
  }
  /**
   * \brief returns the registered device name
   * 
   * @return The registered device name
   */
  public String getName() { return devName; }
  /**
   * \brief sets the device name for this device
   * 
   * @param name The device name to associate with this device
   */
  public void setName(String name) 
  {
    devName = name;
  }
  public void setEqm(TEquipmentModule eqm)
  {
    this.eqm = eqm;
  }
  public String getRedirection()
  {
    return redirection;
  }
  /**
   * \brief sets a redirection string to apply to this device
   * 
   * If the device has a redirection string associated with it, then
   * all remote requests for properties for this device will be
   * redirected to the remote device server given in the redirection string.
   * 
   * @param redirection The redirection string of the general form 
   * '/<context>/<server>/<device name>[property]'.  In this case, ONLY
   * the '<server>' element is relevant.
   */
  public void setRedirection(String redirection)
  {
    this.redirection = redirection;
  }
}
