/*
 * Created on Nov 5, 2004
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 * 
 * CHANGES
 * 2005-02-09 JW * main() removed; see de.desy.tine.client.test.sine
 * 2005-02-10 JW * Constructors made public
 *               * registerProperty(), registerDeviceName() made public
 * 				 * getExportPro[pertyNames(), getExportDeviceNames() added (wrappers)
 *               * Made (almost...) independent from TEquipmentModuleFactory-Singleton
 * 				 * SetBackgroundTask removed; belongs to EqmFactory.
 * 				 * CreateEquipmentModule removed / replaced by initialize().
 * 2004-02-14 JW * old stock properties replaced by TPropertyList
 *               * Stock Equipment function removed, replaced by property handlers
 * 2004-02-15 JW * registerStockProperties changed; register only those properites which 
 * 					are actually supported.
 * 13.7.05 * PD added getExportInformationFromFile()
 * 
 */
package de.desy.tine.server.equipment;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.StringTokenizer;
import de.desy.tine.csvUtils.csv;
import de.desy.tine.csvUtils.csvColumn;
import de.desy.tine.csvUtils.csvHandler;
import de.desy.tine.csvUtils.csvRowHandler;
import de.desy.tine.dataUtils.TDataType;
import de.desy.tine.definitions.TAccess;
import de.desy.tine.definitions.TArrayType;
import de.desy.tine.definitions.TErrorList;
import de.desy.tine.definitions.TFormat;
import de.desy.tine.endianUtils.Swap;
import de.desy.tine.queryUtils.PropertyQuery;
import de.desy.tine.queryUtils.XPropertyQuery;
import de.desy.tine.server.alarms.TAlarm;
import de.desy.tine.server.alarms.TAlarmDefinition;
import de.desy.tine.server.alarms.TAlarmMessage;
import de.desy.tine.server.alarms.TAlarmWatchEntry;
import de.desy.tine.server.connections.TClient;
import de.desy.tine.server.connections.TClientEntry;
import de.desy.tine.server.connections.TContractTable;
import de.desy.tine.server.devices.TDevice;
import de.desy.tine.server.devices.TDeviceList;
import de.desy.tine.server.histories.THistoryRecord;
import de.desy.tine.server.histories.THistorySpecification;
import de.desy.tine.server.logger.TFecLog;
import de.desy.tine.server.properties.TExportProperty;
import de.desy.tine.server.properties.TMetaProperties;
import de.desy.tine.server.properties.TPropertyDescription;
import de.desy.tine.server.properties.TPropertyHandler;
import de.desy.tine.server.properties.TPropertyList;
import de.desy.tine.server.properties.TStockProperties;
import de.desy.tine.startup.TInitializer;
import de.desy.tine.startup.TInitializerFactory;
import de.desy.tine.stringUtils.StringToName;
import de.desy.tine.stringUtils.WildcardMatch;
import de.desy.tine.types.*;

/**
 * \brief A server handles all data requests through its equipment module.  
 * 
 *  An exported device server refers to one and only one equipment module. The
 *  equipment module will have a property list and a device list associated with it.
 *  
 * @author duval 
 */
public class TEquipmentModule
{
  private String context; // the server context (necessary here?)
  private String subsystem;
  private String exportName = null; // the device server name
  private String moduleName = null; // the 'local' name
  private String groupName = null; // the device group (if any)
  private int groupIndex = 0;  // the device group index (if any)
  protected TEquipmentModuleFactory gEqmFactory = null;
  public TEquipmentModuleFactory getEquipmentModuleFactory() 
  {
    if (gEqmFactory == null) gEqmFactory = TEquipmentModuleFactory.getInstance();
    return gEqmFactory; 
  }
  protected LinkedList gRegisteredUsersList = new LinkedList();
  protected LinkedList gRegisteredNetsList = new LinkedList();
  protected LinkedList gGCastNetsList = new LinkedList();
  protected HashMap<Integer,TAlarmDefinition> alarmDefinitionList = new HashMap<Integer,TAlarmDefinition>();
  public HashMap getAlarmDefinitionList() { return alarmDefinitionList; }
  public int addAlarmDefinition(int code,TAlarmDefinition adef)
  {
    if (code < 0 || adef == null) return TErrorList.argument_list_error;
    alarmDefinitionList.put(new Integer(code),adef);
    return 0;
  }
  public int addAlarmDefinition(TAlarmDefinition adef)
  {
    if (adef == null) return TErrorList.argument_list_error;
    return addAlarmDefinition(adef.getAlmCode(),adef);
  }
  public TAlarmDefinition getAlarmDefinition(int code)
  {
    return alarmDefinitionList.get(new Integer(code));
  }
  TPropertyList stockList; // = new TPropertyList(); // a list or a query fcn
  TPropertyList propertyList = new TPropertyList(); // a list or a query fcn
  TDeviceList deviceList = new TDeviceList();
  public TDeviceList getDeviceList() { return deviceList; }
  public int getDeviceNumber(String devName)
  {
    try
    {
      if (devName.startsWith("#")) return Integer.parseInt(devName.substring(1));
      return deviceList.getDeviceNumber(devName);
    }
    catch (Exception e)
    {
      return -1;
    }
  }
  public int getDeviceNumber(String devName,String devProperty)
  {
    try
    {
      if (devName == null) return -TErrorList.argument_list_error;
      if (devName.startsWith("#")) return Integer.parseInt(devName.substring(1));
      if (deviceList.isPropertyOriented() && devProperty != null)
      {
        TExportProperty prp = propertyList.getFirstProperty(devProperty);
        if (prp != null)
        {
          ArrayList lst = prp.getDeviceList();
          if (lst != null) return lst.indexOf(devName);
        }
      }
      return deviceList.getDeviceNumber(devName);
    }
    catch (Exception e)
    {
      return -1;
    }
  }
  protected String[] gPropertyNameList = null;
  protected LinkedList gAlarmWatchList = new LinkedList();
  protected LinkedList gLclHstList = new LinkedList();
  private boolean useHighResolutionTimestamps = false;
  private boolean useMonthlyHistoryFiles = false;
  public void setUseMSecHistoryTimestamps(boolean useMsecHistoryTimestamps)
  {
    useHighResolutionTimestamps = useMsecHistoryTimestamps;
  }
  public boolean isUseMSecHistoryTimestamps() { return useHighResolutionTimestamps; }
  public void setUseMonthlyHistoryFiles(boolean useMonthlyFiles)
  {
    useMonthlyHistoryFiles = useMonthlyFiles;
  }
  public boolean isUseMonthlyHistoryFiles() { return useMonthlyHistoryFiles; }
  THistoryRecord getLocalHistoryRecord(String Property,String Device)
  {
    int listsize = gLclHstList.size();
    THistoryRecord lhr = null;
    for (int i=0; i<listsize; i++)
    {
      lhr = (THistoryRecord)gLclHstList.get(i);
      if (lhr.getPrp().compareTo(Property) != 0) continue;
      // TODO: worry about matching device name ? (vs multi-channel array)
      return lhr;
    }
    return null;
  }
  public int addLocalHistoryRecord(int recordIndex,String device,String property,int dataSize,int dataFormat)
  {
    return addLocalHistoryRecord(recordIndex,device,property,dataSize,dataFormat,TArrayType.AT_UNKNOWN);
  }
  public int addLocalHistoryRecord(int recordIndex,String device,String property,int dataSize,int dataFormat,int arrayType)
  {
    THistorySpecification spc = 
      new THistorySpecification(1000,1000,600,1,18000,10,0,"");
    return addLocalHistoryRecord(recordIndex,device,property,dataSize,dataFormat,arrayType,spc);
  }
  public int addLocalHistoryRecord(int recordIndex,String device,String property,int dataSize,int dataFormat,int arrayType,THistorySpecification histSpec)
  {
    THistoryRecord hst = new THistoryRecord(this,device,property,dataSize,dataFormat,arrayType,recordIndex,histSpec);
    if (useHighResolutionTimestamps) hst.setUseMsecHistoryTimeStamps(true);
    if (useMonthlyHistoryFiles) hst.setUseMonthlyHistoryFiles(true);
    gLclHstList.add(hst);
    return 0;    
  }
  public boolean isRegistered; // registered with name server
  public boolean hasCasAttached = false;
  public boolean isCasConfigured = false;
  public boolean casReadRequired = false;
  public boolean grpRegistered = false;
  //TAlarmTable alarms; // active alarms + alarm definitions
  int numClients;
  int numContracts;
  boolean isCasConfigured()
  {
    if (isCasConfigured || 
        (isCasConfigured = gEqmFactory.gCasConfigured)) return true;
    return false;
  }
  boolean isCasInClientList()
  {
    if (!isCasConfigured()) return false; // don't know who he is !
    String casName = gEqmFactory.getCasName();
    TClientEntry[] tce = getClientList();
    for (int i=0; i<tce.length; i++) 
      if (tce[i].cln.userName.compareTo(casName) == 0)
      {
        casReadRequired = true;
        hasCasAttached = true;
        return true;
      }
    return false;
  }
  class AccessLock
  {
    static final int LOCK_UNLOCKED = 0;
    static final int LOCK_PREEMPTIVE = 1;
    static final int LOCK_PERSISTENT = 2;
    static final int LOCK_ABORT = 3;
    private TClient cln;
    private int lockType;
    private int lockDuration;
    private long lockStart;
    int getLockDuration() { return lockDuration; }
    int getLockType() { return lockType; }
    long getLockStart() { return lockStart; }
    void checkAccessLocks()
    {
      long t = System.currentTimeMillis();
      if (t > lockStart + lockDuration)
      {
        lockStart = 0;
        lockType = LOCK_UNLOCKED;
      }
    }
    boolean clientCanWrite(TClient tc)
    {
      if (lockType == LOCK_UNLOCKED) return true;
      if (tc.IPaddress == cln.IPaddress &&
          tc.userName.compareTo(cln.userName) == 0 &&
          tc.port == cln.port)
        return true;
      return false;
    }
    int setAccessLock(TClient tc,int type,int duration)
    {
      boolean passed = false;
      if (type < LOCK_UNLOCKED || type > LOCK_ABORT) return TErrorList.out_of_range;
      if (clientCanWrite(tc)) passed = true;
      else if (lockType != LOCK_PERSISTENT) passed = true;
      else if (type == LOCK_ABORT) passed = true;
      if (!passed) return TErrorList.access_denied;
      if (type == LOCK_ABORT || type == LOCK_UNLOCKED)
      {
        lockStart = 0;
        lockType = LOCK_UNLOCKED;
      }
      else
      {
        cln = tc;
        lockType = type;
        lockDuration = duration;
        lockStart = System.currentTimeMillis();        
      }
      return 0;
    }
    String[] getAccessLockAsString()
    {
      String[] s = new String[2];
      if (lockType == LOCK_UNLOCKED) 
      {
        s[0] = s[1] = "";
      }
      else
      {
        s[0] = cln.userName;
        s[1] = cln.IPaddress.getHostAddress();
      }
      return s;
    }
  }
  public AccessLock accessLock = new AccessLock();
  // REMOVED TEquipmentFunction stkMod; // stock meta property eqp. fcn.
  //TEquipmentFunction stkFcn; // stock property eqp. fcn.
  TEquipmentFunction eqpFcn = null;
  //TEquipmentBackgroundTask eqpBkg;
  TEquipmentInitRoutine eqpIni;
  TEquipmentExitRoutine eqpExi;
  public NAME16[] getRegisteredUsers() { return StringToName.stringSetToName16(gRegisteredUsersList); }
  public int addRegisteredUser(String newUser)
  {
    gRegisteredUsersList.add(newUser);    
    return 0;
  }
  public int removeRegisteredUser(String newUser)
  {
    gRegisteredUsersList.remove(newUser);    
    return 0;
  }
  public int addRegisteredNet(String newNet)
  {
    gRegisteredNetsList.add(newNet);    
    return 0;
  }
  public NAME16[] getRegisteredNets() { return StringToName.stringSetToName16(gRegisteredNetsList); }
  public int addGCastNet(String newNet)
  {
    gGCastNetsList.add(newNet);    
    return 0;
  }
  public NAME16[] getGCastNets() { return StringToName.stringSetToName16(gGCastNetsList); }
  public void setCompletionString(String completionString)
  {
    TContractTable tct = this.getTEqmFactory().getCurrentContractEntry();
    if (tct != null) tct.compString = completionString;
  }
  public int setLocalName(String lclName)
  {
    moduleName = lclName;
    getEquipmentModuleFactory().registrationPending = true;
    return 0;
  }
  public String getLocalName() { return moduleName; }
  public String getModuleName() { return moduleName; }
  public int setExportName(String expName)
  {
    if (exportName == null) 
    {
      exportName = new String(expName);
      if (exportName.length() > 16) exportName = exportName.substring(0,17);
    }
    return 0;
  }
  public String getExportName()
  {
    return exportName;
  }
  public void setGroupName(String grpName)
  {
    if (groupName == null) 
    {
      groupName = new String(grpName);
      if (groupName.length() > 16) groupName = groupName.substring(0,17);
      grpRegistered = false;
      TEquipmentModuleFactory.getInstance().registrationPending = true;
    }
  }
  public String getGroupName() { return groupName; }
  public void setGroupIndex(int grpIndex)
  {
    groupIndex =  grpIndex;
  }
  public int getGroupIndex() { return groupIndex; }
  public int setContext(String expContext)
  {
    if (context == null && expContext != null) 
    {
      context = expContext;
      if (context.length() > 16) context = context.substring(0,17);
    }
    return 0;
  }
  public String getContext()
  {
    return context;
  }
  public TEquipmentModuleFactory getTEqmFactory() { return gEqmFactory; }
  public TClient getCaller()
  {
    TContractTable tct = gEqmFactory.getCurrentContractEntry();
    return tct != null ? ((TClientEntry)tct.clt.getFirst()).cln : null;
  }   
  public TClientEntry[] getClientList()
  {
    TContractTable tct = gEqmFactory.getCurrentContractEntry();
    return tct != null ? ((TClientEntry[])tct.clt.toArray(new TClientEntry[0])) : null;
  }   
  /**
   * Initializer used by contructors.
   * 
   * @param name Module name.
   * @param eqpFunction The TEquipmentFunction to attach to the equipment module.
   */
  private void initialize(String name, TEquipmentFunction eqpFunction)
  {
    gEqmFactory = TEquipmentModuleFactory.getInstance();
    if (name != null)
    {
      moduleName = name.length() > 6 ? name.substring(0, 6) : name;
    }
    else
    {
      moduleName = "";
    }
    if (eqpFunction != null)
    {
      this.eqpFcn = eqpFunction;
      this.eqpFcn.setEquipmentModule(this);
    }
    registerStockProperties();
    getRegisteredUsersFromFile();
    getRegisteredNetsFromFile();
    getRegisteredAlarmDefinitionsFromFile();
    gEqmFactory.addEquipmentModule(this);
  }
  private long lastHistoryCycleTime = 0;
  private int historyCycleInterval = 1000; // milliseconds
  public void setHistoryCycleInterval(int cycleInterval) { historyCycleInterval = cycleInterval; }
  protected void historyCycle()
  {
    if (gEqmFactory.gEqpFcnBusySemaphore) return;
    long thisCycleTime = System.currentTimeMillis();
    THistoryRecord hst = null;
    THistorySpecification spc = null;
    TDataType dout = null;
    TDataType din = new TDataType();
    TAccess access = new TAccess(TAccess.CA_HIST + TAccess.CA_READ);
    int cc = 0;
    
    if (thisCycleTime-lastHistoryCycleTime < historyCycleInterval) return;
    lastHistoryCycleTime = thisCycleTime;
    int nHistoryRecords = gLclHstList.size();
    for (int i=0; i<nHistoryRecords; i++)
    {
      hst = (THistoryRecord)gLclHstList.get(i);
      spc = hst.getHspec();
      if (spc.getDepthShort() <= 0) continue;
      hst.setArchiveFileName(thisCycleTime); // will also remove outdated files if necessary
      if (thisCycleTime-hst.getTimestamp() < spc.getPollngRate()) continue;
      dout = new TDataType(hst.getArraySize(),hst.getFmt());      
      gEqmFactory.gEqpFcnBusySemaphore = true;
      cc = callProperty(hst.getPrp(),hst.getDev(),dout,din,access);
      gEqmFactory.gEqpFcnBusySemaphore = false;
      if (cc != 0)
      {
        continue;
      }
      hst.writeRecordToSTS(dout.getDataObject());
      if (needToArchiveRecord(hst)) hst.writeRecordToLTS(dout.getDataObject());
    }
    return;
  }
  private boolean needToArchiveRecord(THistoryRecord hst)
  {
    long ts = System.currentTimeMillis();
    double t;
    int p,ptr0,ptr1;
    double d0, d1;
    String s0, s1;
    boolean doarchive = false;
    THistorySpecification spc;

    if (hst == null) return false;
    if((spc=hst.getHspec()) == null) return false;
    if (spc.getArchiveRate() < 0 ||
        spc.getHeartbeat() < 0   ||
        spc.getDepthLong() < 0) return false;/* no long term storage */
    if (hst.getRngBufferSize() <= 0) return false;
    ptr0 = hst.getRingBufferPointer();
    ptr1 = (ptr0 + hst.getRngBufferSize() - 1) % hst.getRngBufferSize(); 

    if (hst.getTimestamp() <= hst.getLastArchiveTime()) return false;/* data timestamp hasn't changed */
    if (ts < hst.getLastArchiveTime() + spc.getArchiveRate()) return false;
    if (ts >= hst.getLastArchiveTime() + spc.getHeartbeat()) return true; /* archive no matter what */

    short fmt = hst.getFmt();
    int siz = hst.getArraySize();
    float aTol = spc.getAbsoluteTolerance();
    float pTol = spc.getPercentTolerance();
    switch (fmt)
    {
      case TFormat.CF_DOUBLE:
      case TFormat.CF_FLOAT:
      case TFormat.CF_SHORT:
      case TFormat.CF_LONG:
        for (p=0; !doarchive && p<siz; p++)
        {
          d0 = hst.getDoubleValueAtPosition(ptr0,p);
          d1 = hst.getDoubleValueAtPosition(ptr1,p);
          t = aTol + pTol * Math.abs(d1);
          if (Math.abs(d0 - d1) > t) doarchive = true;
        }
        break;
      case TFormat.CF_NAME16:
      case TFormat.CF_NAME32:
      case TFormat.CF_NAME64:
        for (p=0; !doarchive && p<siz; p++)
        {
          s0 = hst.getNAME64ValueAtPosition(ptr0,p).toString();
          s1 = hst.getNAME64ValueAtPosition(ptr1,p).toString();
          if (s0.compareTo(s1) == 0) continue;
          doarchive = true;
          break;
        }
        break;
      default:
        break;
    }
    return doarchive;    
  }
  protected void checkAlarmsInWatchTable()
  {
    if (gEqmFactory.gEqpFcnBusySemaphore) return;
    TDataType dout = null;
    TDataType din = new TDataType();
    TAccess access = new TAccess(TAccess.CA_ALARM + TAccess.CA_READ);
    TAlarmWatchEntry awe = null;
    TAlarmDefinition adef = null;
    TAlarm alm = null;
    TDevice dev = null;
    int cc = 0, cnt = 0, siz = 0, sev = 0, devNr, code = 0, asys = 0;
    float fvals[] = null;
    int ivals[] = null;
    byte[] abytes = null;
    int awlength = gAlarmWatchList.size();
    for (int i=0; i<awlength; i++)
    {
      awe = (TAlarmWatchEntry)gAlarmWatchList.get(i);
      if (awe.checkNormal())
      {
        ivals = new int[awe.getSiz()];
        dout = new TDataType(ivals);
        gEqmFactory.gEqpFcnBusySemaphore = true;
        cc = callProperty(awe.getPrp(),awe.getDev(),dout,din,access);
        gEqmFactory.gEqpFcnBusySemaphore = false;        
      }
      else
      {
        fvals = new float[awe.getSiz()];
        dout = new TDataType(fvals);
        gEqmFactory.gEqpFcnBusySemaphore = true;
        cc = callProperty(awe.getPrp(),awe.getDev(),dout,din,access);
        gEqmFactory.gEqpFcnBusySemaphore = false;
      }
      if (cc != 0) continue;
      devNr = deviceList.getDeviceNumber(awe.getDev());
      cnt = 0; siz = awe.getSiz();
      for (int j=0; j<siz; j++)
      {
        if (awe.getAtyp() == TArrayType.AT_CHANNEL) devNr = j;
        if (awe.getAtyp() != TArrayType.AT_WAVEFORM && siz == deviceList.getNumberOfDevices()) 
          devNr = j; /* assume array of devices */
        dev = deviceList.getDevice(devNr);
        if (ivals != null)
        {
          if (awe.getNormal() != (awe.getMask() & ivals[j]))
          {
            code = TErrorList.invalid_data;
            sev = awe.getSev();
            asys = awe.getAsys();
          }
          abytes = Swap.LongToBytes(ivals[j]);
        }
        else if (fvals != null)
        {
          if (fvals[j] > awe.getHi()) 
          {
            code = TErrorList.value_too_high;
            sev = awe.getSev();
            asys = awe.getAsys();
          }
          else if (fvals[j] > awe.getHiwarn()) 
          {
            code = TErrorList.warn_too_high;          
            sev = awe.getSev()-2;
            asys = awe.getAsys();
          }
          else if (fvals[j] < awe.getLo()) 
          {
            code = TErrorList.value_too_low;          
            sev = awe.getSev();
            asys = awe.getAsys();
          }
          else if (fvals[j] < awe.getLowarn()) 
          {
            code = TErrorList.warn_too_low;          
            sev = awe.getSev()-2;
            asys = awe.getAsys();
          }
          abytes = Swap.Float(fvals[j]);
        }
        else
        {
          if (awe.getCnt() > 0) /* if there is or was an alarm somewhere, run through RemoveAlarm() for the others */
          {
            dev.removeAlarm(TErrorList.invalid_data);
            dev.removeAlarm(TErrorList.value_too_high);
            dev.removeAlarm(TErrorList.value_too_low);
            dev.removeAlarm(TErrorList.warn_too_high);
            dev.removeAlarm(TErrorList.warn_too_low);
          }
          continue; 
        }
        cnt++;
        awe.setCnt(cnt);
        if ((dev = deviceList.getDevice(j)) != null)
        {
          alm = dev.setAlarm(code,abytes);
          if (alm != null)
          {
            adef = alm.getAlmDef();
            adef.setAlmSeverity((short)sev);
            adef.setAlmSystem(asys);
          }
        }
      }
      /* only clear the watch table counter when a complete pass has no alarms */
      if (cnt == 0) awe.setCnt(0);
    }
  }
  protected void update()
  {
    accessLock.checkAccessLocks();
    checkAlarmsInWatchTable();
    deviceList.refreshAlarms();
    historyCycle();
  }
  /**
   * Registers the equipment module's stock properties
   */
  private int prpQueryCall(String devName, TDataType dout, TDataType din, TAccess devAccess)
  { // use in calls for "PROPS" or "PROPERTIES"
    int cc = 0;
    TDevice dev = getDeviceList().getDevice(devName);
    LinkedList lst = null;
    boolean checkWildCard = (devName.compareTo("*") != 0 && devName.indexOf("*") != -1);
    
    if (dev != null && (lst=dev.getPropertyList()) != null)
    {
      String[] slst = new String[lst.size()];
      lst.toArray(slst);
      if (checkWildCard) slst = StringToName.matchStringsInArray(slst,devName);
      if (dout.getFormat() == TFormat.CF_USTRING)
      {
        USTRING[] ustr = new USTRING[slst.length];
        char[] cha;
        StringBuffer sb = new StringBuffer(80);
        TExportProperty prp;
        for (int i = 0; i<slst.length; i++)
        {
          ustr[i] = new USTRING();
          if ((prp = propertyList.getFirstProperty(slst[i])) == null) continue;
          cha = prp.getName().toCharArray();
          sb.insert(0,cha);
          for (int c=cha.length; c<32; c++) sb.insert(c,(char)0);
          cha = prp.getDescription().getText().toCharArray();
          sb.insert(32,cha);
          ustr[i].setValues(prp.getOutputSize(),0,0,prp.getOutputFormat(),new String(sb));
        }
        cc = dout.putData(ustr);
      }
      else
      {
        cc = dout.putData(StringToName.stringArrayToName32(slst));
      }
    }
    else
    {
      String[] slst = new String[propertyList.countUniqueProperties()];
      propertyList.getPropertyNames().toArray(slst);
      if (checkWildCard) slst = StringToName.matchStringsInArray(slst,devName);
      if (dout.getFormat() == TFormat.CF_USTRING)
      {
        USTRING[] ustr = new USTRING[slst.length];
        char[] cha;
        StringBuffer sb = new StringBuffer(80);
        TExportProperty prp;
        for (int i = 0; i<slst.length; i++)
        {
          ustr[i] = new USTRING();
          if ((prp = propertyList.getFirstProperty(slst[i])) == null) continue;
          cha = prp.getName().toCharArray();
          sb.insert(0,cha);
          for (int c=cha.length; c<32; c++) sb.insert(c,(char)0);
          cha = prp.getDescription().getText().toCharArray();
          sb.insert(32,cha);
          ustr[i].setValues(prp.getOutputSize(),0,0,prp.getOutputFormat(),new String(sb));
        }
        cc = dout.putData(ustr);
      }
      else
      {
        cc = dout.putData(StringToName.stringArrayToName32(slst));
      }
    }
    if (cc == 0)
    {
      if (propertyList.isDeviceOriented()) 
        cc = TErrorList.has_query_function | TErrorList.CE_SENDDATA;
    }
    return cc;      
  }
  void registerStockProperties()
  {
    final TEquipmentModule thisEqm = this;
    if (gEqmFactory == null) gEqmFactory = TEquipmentModuleFactory.getInstance();
    //stockList = gEqmFactory.getStockList();
    //if (stockList == null) stockList = new TPropertyList();
    stockList = new TPropertyList();
    // TODO Separate handlers for overloaded properties
    TPropertyHandler propertiesStructHandler = new TPropertyHandler()
    {
      public int call(String devName, TDataType dout, TDataType din, TAccess devAccess)
      {
        if (dout != null)
        {
          switch (dout.dFormat)
          {
            //case TFormat.CF_NAME32:
            //  return (short)
            // dout.putData(propertyList.getExportPropertyNames());
            case TFormat.CF_STRUCT:
              if (dout.getTag().compareTo("XPQS") == 0 || (dout.getArrayLength() % XPropertyQuery.sizeInBytes) == 0)
              {
                if (din != null)
                { // looking for a specific property ...
                  LinkedList propInstances;
                  String propertyName = din.toString();
                  propertyName = propertyName.replace('\n', (char) 0).trim();
                  propInstances = propertyList.getEqualProperties(propertyName);
                  if ((propInstances == null) || (propInstances.size() == 0))
                      return TErrorList.not_exported;
                  // fill in the targetted property info
                  THistoryRecord lhr = null;
                  XPropertyQuery[] xpq = new XPropertyQuery[propInstances.size()];
                  byte[] tba = new byte[xpq.length * XPropertyQuery.sizeInBytes];
                  Iterator it = propInstances.iterator();
                  for (int i = 0; (it.hasNext()) && (i < xpq.length); i++)
                  {
                    xpq[i] = new XPropertyQuery((TExportProperty) it.next(), propInstances.size() - 1);
                    if ((lhr=getLocalHistoryRecord(xpq[i].prpName,"#0")) != null)
                    {
                      xpq[i].prpHistoryDepthLong = (short)lhr.getHspec().getDepthLong();
                      xpq[i].prpHistoryDepthShort = (short)lhr.getHspec().getDepthShort();
                    }
                    System.arraycopy(xpq[i].toByteArray(), 0, tba, i * XPropertyQuery.sizeInBytes,
                        XPropertyQuery.sizeInBytes);
                  }
                  //return (short)dout.putData(xpq[0].toByteArray(),"XPQS");
                  return dout.putData(tba, "XPQS");
                }
                else
                { // want the whole list ...
                  String[] slst = new String[propertyList.countUniqueProperties()];
                  propertyList.getPropertyNames().toArray(slst);
                  XPropertyQuery[] xpq = new XPropertyQuery[slst.length];
                  if (slst.length == 0) break;
                  byte[] tba = new byte[slst.length * XPropertyQuery.sizeInBytes];
                  TExportProperty prp;
                  for (int i=0; i<slst.length; i++)
                  {
                    prp = propertyList.getFirstProperty(slst[i]);
                    xpq[i] = new XPropertyQuery(prp,1);
                    System.arraycopy(xpq[i].toByteArray(), 0, tba, i * XPropertyQuery.sizeInBytes,
                        XPropertyQuery.sizeInBytes);
                  }
                  return dout.putData(tba, "XPQS");
                }
              }
              else if (dout.getTag().compareTo("PQS") == 0 || (dout.getArrayLength() % PropertyQuery.sizeInBytes) == 0)
              { // legacy request ...
                if (din != null)
                { // looking for a specific property ...
                  LinkedList propInstances;
                  String propertyName = din.toString();
                  propertyName = propertyName.replace('\n', (char) 0).trim();
                  propInstances = propertyList.getEqualProperties(propertyName);
                  if ((propInstances == null) || (propInstances.size() == 0))
                      return TErrorList.not_exported;
                  // fill in the targetted property info
                  PropertyQuery[] pqs = new PropertyQuery[1];
                  TExportProperty prp = propertyList.getFirstProperty(propertyName);
                  byte[] tba = new byte[PropertyQuery.sizeInBytes];
                  pqs[0] = new PropertyQuery();
                  pqs[0].name = propertyName;
                  pqs[0].prpDesc = prp.getDescription().getText();
                  pqs[0].prpAccess = (byte)prp.getAccessMode();
                  pqs[0].prpFormat = (byte)(prp.getOutputFormat()%512);
                  pqs[0].prpSize = (short)prp.getOutputSize();
                  System.arraycopy(pqs[0].toByteArray(), 0, tba, 0, PropertyQuery.sizeInBytes);
                  return dout.putData(tba, "PQS");
                }
                else
                { // want the whole list ...
                  String[] slst = new String[propertyList.countUniqueProperties()];
                  propertyList.getPropertyNames().toArray(slst);
                  if (slst.length == 0) break;
                  PropertyQuery[] pqs = new PropertyQuery[slst.length];
                  byte[] tba = new byte[slst.length * PropertyQuery.sizeInBytes];
                  TExportProperty prp;
                  for (int i=0; i<slst.length; i++)
                  {
                    pqs[i] = new PropertyQuery();
                    pqs[i].name = slst[i];
                    prp = propertyList.getFirstProperty(slst[i]);
                    pqs[i].prpDesc = prp.getDescription().getText();
                    pqs[i].prpAccess = (byte)prp.getAccessMode();
                    pqs[i].prpFormat = (byte)(prp.getOutputFormat()%512);
                    pqs[i].prpSize = (short)prp.getOutputSize();
                    System.arraycopy(pqs[i].toByteArray(), 0, tba, i * PropertyQuery.sizeInBytes,
                        PropertyQuery.sizeInBytes);
                  }
                  return dout.putData(tba, "PQS");                
                }
              }
          }
        }
        /**/
        return TErrorList.illegal_format;
      }
    };
    
    TPropertyHandler alarmsStructHandler = new TPropertyHandler()
    {
      public int call(String devName, TDataType dout, TDataType din, TAccess devAccess)
      {
        if (dout == null) return TErrorList.dimension_error;
        if (dout.dFormat != TFormat.CF_STRUCT) return TErrorList.illegal_format;
        int[] amrb = new int[4];
        if (din != null)
        {
          switch (din.dFormat)
          {
            case TFormat.CF_STRUCT:
              if (din.dArrayLength != 16) return TErrorList.illegal_format;
              byte[] bdata = din.getDataBuffer();
              byte[] ival = new byte[4];
              for (int i=0; i<4; i++)
              {
                System.arraycopy(bdata,i*4,ival,0,4);
                amrb[i] = Swap.Long(ival);
              }
              break;
            case TFormat.CF_LONG:
              din.getData(amrb);
              break;
            default:
              return TErrorList.illegal_format;
          }
        }
        else
        {
          amrb[0] = 0; // start time
          amrb[1] = (int)(System.currentTimeMillis()/1000); // stop time
          amrb[2] = 0; // minimum severity
          amrb[3] = 0; // as yet unused ...
        }
        int dstart, dstop;
        int astart = amrb[0];
        int astop = amrb[1];
        int asev = amrb[2];
        if (devName == null || devName.length() == 0 || devName.startsWith("#"))
        {
          // currently any of these is a signal that the caller want's ALL alarms
          dstart = 0; dstop = deviceList.getNumberOfDevices();
        }
        else
        {
          dstart = deviceList.getDeviceNumber(devName);
          dstop = dstart + 1;
        }
        String dev = null;
        TAlarm[] alms = null;
        TAlarmMessage[] ams = null;
        int na = 0;
        int ts = 0;
        int bufstart = 0;
        int buflength = deviceList.getNumberOfAlarms();
        byte[] tba = new byte[buflength * TAlarmMessage.sizeInBytes];
        TDevice[] dlst = deviceList.getDeviceList();
        for (int i=dstart; i<dstop; i++)
        {
          alms = dlst[i].getAlarmList();
          if ((alms == null) || (alms.length == 0)) continue;
          ams = new TAlarmMessage[alms.length];
          for (int n=0; n<alms.length && n<(buflength-bufstart); n++)
          {
            if ((ts=alms[n].getTimeStamp()) < astart || ts > astop) continue;
            if (alms[n].getSeverity() < asev) continue;
            ams[na] = new TAlarmMessage(thisEqm,dev,alms[n]);
            System.arraycopy(ams[na].toByteArray(), 0, tba, (na + bufstart)*TAlarmMessage.sizeInBytes, TAlarmMessage.sizeInBytes);
            // CAS is configured and has now read the alarm => can remove it !
            if (casReadRequired && isCasInClientList()) alms[na].allowRemoval();
            na++;
          }
          bufstart += na; 
        }
        return dout.putData(tba, "AMS");
      }
    };
    TPropertyHandler alarmDefsStructHandler = new TPropertyHandler()
    {
      public int call(String devName, TDataType dout, TDataType din, TAccess devAccess)
      {
        if (dout == null) return TErrorList.dimension_error;
        if (dout.dFormat != TFormat.CF_STRUCT) return TErrorList.illegal_format;
        int almdefsize = alarmDefinitionList.size();
        byte[] tba = new byte[almdefsize * TAlarmDefinition.sizeInBytes];
        TAlarmDefinition[] adef = (TAlarmDefinition[])alarmDefinitionList.values().toArray(new TAlarmDefinition[0]);

        for (int i=0; i<almdefsize; i++)
        {
          System.arraycopy(adef[i].toByteArray(), 0, tba, i*TAlarmDefinition.sizeInBytes, TAlarmDefinition.sizeInBytes);
        }
        return dout.putData(tba, "ADS");
      }
    };
    stockList.addProperty(TStockProperties.NALARMS, new TPropertyHandler()
    {
      public int call(String devName, TDataType dout, TDataType din, TAccess devAccess)
      {
        if (isCasConfigured()  && !hasCasAttached) isCasInClientList(); 
        int[] nalms = new int[5];
        nalms[0] = deviceList.getNumberOfAlarms();
        nalms[1] = deviceList.getMostRecentAlarmTimestamp(nalms,3);
        nalms[2] = deviceList.getHighestAlarmSeverity(nalms,4);
        return dout.putData(nalms);
      }
    });
    stockList.addProperty(TStockProperties.NALARMDEFS, new TPropertyHandler()
    {
      public int call(String devName, TDataType dout, TDataType din, TAccess devAccess)
      {
        return dout.putData((short)alarmDefinitionList.size());
      }
    });
    stockList.addProperty(TStockProperties.ALARMS, alarmsStructHandler);
    stockList.addProperty(TStockProperties.ALARMSEXT, alarmsStructHandler);
    stockList.addProperty(TStockProperties.ALARMDEFS, alarmDefsStructHandler);
    stockList.addProperty(TStockProperties.NHISTORIES, new TPropertyHandler()
    {
      public int call(String devName, TDataType dout, TDataType din, TAccess devAccess)
      {
        return dout.putData((short)gLclHstList.size());
      }
    });
    stockList.addProperty(TStockProperties.HISTORIES, new TPropertyHandler()
    {
      public int call(String devName, TDataType dout, TDataType din, TAccess devAccess)
      {
        THistoryRecord[] hsts =  (THistoryRecord[])gLclHstList.toArray(new THistoryRecord[0]);
        NAME32[] nams = new NAME32[hsts.length];
        for (int i=0; i<nams.length; i++) nams[i] = new NAME32(hsts[i].getPrp());
        return dout.putData(nams);
      }
    });
    stockList.addProperty(TStockProperties.PROPERTIES, new TPropertyHandler()
    {
      public int call(String devName, TDataType dout, TDataType din, TAccess devAccess)
      {
        return prpQueryCall(devName,dout,din,devAccess);
      }
    });
    stockList.addProperty(TStockProperties.PROPERTIES_USTRING, new TPropertyHandler()
        {
          public int call(String devName, TDataType dout, TDataType din, TAccess devAccess)
          {
            return prpQueryCall(devName,dout,din,devAccess);
          }
        });
    stockList.addProperty(TStockProperties.PROPERTIES_STRUCT, propertiesStructHandler);
    stockList.addProperty(TStockProperties.PROPS, new TPropertyHandler()
    {
      public int call(String devName, TDataType dout, TDataType din, TAccess devAccess)
      {
        return prpQueryCall(devName,dout,din,devAccess);
      }
    });
    stockList.addProperty(TStockProperties.PROPS_USTRING, new TPropertyHandler()
        {
          public int call(String devName, TDataType dout, TDataType din, TAccess devAccess)
          {
            return prpQueryCall(devName,dout,din,devAccess);
          }
        });
    stockList.addProperty(TStockProperties.PROPS_STRUCT, propertiesStructHandler);
    stockList.addProperty(TStockProperties.NPROPERTIES, new TPropertyHandler()
    {
      public int call(String devName, TDataType dout, TDataType din, TAccess devAccess)
      {
        return dout.putData(propertyList.countUniqueProperties());
      }
    });
    // Same as NPROPERTIES
    stockList.addProperty(TStockProperties.NPROPS, new TPropertyHandler()
    {
      public int call(String devName, TDataType dout, TDataType din, TAccess devAccess)
      {
        return dout.putData(propertyList.countUniqueProperties());
      }
    });
    stockList.addProperty(TStockProperties.NDEVICES, new TPropertyHandler()
    {
      public int call(String devName, TDataType dout, TDataType din, TAccess devAccess)
      {
        //return dout.putData(deviceList.size());
        return dout.putData(deviceList.getNumberOfDevices());
      }
    });
    stockList.addProperty(TStockProperties.DEVICES, new TPropertyHandler()
    {
      public int call(String devName, TDataType dout, TDataType din, TAccess devAccess)
      {
        String[] namlst = deviceList.getDeviceNameList();
        if (devName.compareTo("*") != 0 && devName.indexOf("*") != -1)
        { // query for a subset of device names ...
          namlst = StringToName.matchStringsInArray(namlst,devName);
        }
        //return dout.putData(StringToName.stringSetToName16(deviceList));
        int cc = dout.putData(StringToName.stringArrayToName16(namlst));
        if (cc == 0)
        {
          if (getDeviceList().isPropertyOriented()) 
            cc = TErrorList.has_query_function | TErrorList.CE_SENDDATA;
        }
        return cc;
      }
    });
    // TODO Belongs to server
    stockList.addProperty(TStockProperties.SRVOS, new TPropertyHandler()
    {
      public int call(String devName, TDataType dout, TDataType din, TAccess devAccess)
      {
        return dout.putData(TStockProperties.getStockSrvOS());
      }
    });
    // TODO belongs to server
    stockList.addProperty(TStockProperties.SRVVERSION, new TPropertyHandler()
    {
      public int call(String devName, TDataType dout, TDataType din, TAccess devAccess)
      {
        return dout.putData(TStockProperties.getStockSrvVersion());
      }
    });
    stockList.addProperty(TStockProperties.USERS, new TPropertyHandler()
    {
      public int call(String devName, TDataType dout, TDataType din, TAccess devAccess)
      {
        return dout.putData(getRegisteredUsers());
      }
    });
    stockList.addProperty(TStockProperties.NUSERS, new TPropertyHandler()
    {
      public int call(String devName, TDataType dout, TDataType din, TAccess devAccess)
      {
        return dout.putData((short)gRegisteredUsersList.size());
      }
    });
    stockList.addProperty(TStockProperties.ADDUSER, new TPropertyHandler()
    {
      public int call(String devName, TDataType dout, TDataType din, TAccess devAccess)
      {
        int cc;
        if (din != null && devAccess.isWrite())
        {
          NAME16[] usr = new NAME16[1];
          if ((cc=din.getData(usr)) != 0) return cc;
          return addRegisteredUser(usr.toString());
        }
        return TErrorList.illegal_read_write;
      }
    });
    stockList.addProperty(TStockProperties.DELUSER, new TPropertyHandler()
    {
      public int call(String devName, TDataType dout, TDataType din, TAccess devAccess)
      {
        int cc;
        if (din != null && devAccess.isWrite())
        {
          NAME16[] usr = new NAME16[1];
          if ((cc=din.getData(usr)) != 0) return cc;
          return removeRegisteredUser(usr.toString());
        }
        return TErrorList.illegal_read_write;
      }
    });
    stockList.addProperty(TStockProperties.IPNETS, new TPropertyHandler()
    {
      public int call(String devName, TDataType dout, TDataType din, TAccess devAccess)
      {
        return dout.putData(getRegisteredNets());
      }
    });
    stockList.addProperty(TStockProperties.NIPNETS, new TPropertyHandler()
    {
      public int call(String devName, TDataType dout, TDataType din, TAccess devAccess)
      {
        return dout.putData((short)gRegisteredNetsList.size());
      }
    });
    stockList.addProperty(TStockProperties.ACCESSLOCK, new TPropertyHandler()
    {
      public int call(String devName, TDataType dout, TDataType din, TAccess devAccess)
      {
        int cc = 0;
        if (din != null && devAccess.isWrite())
        {
          short[] svals = new short[2];
          if ((cc=din.getData(svals)) != 0) return cc;
          if ((cc=accessLock.setAccessLock(getCaller(),svals[0],svals[1])) != 0) return cc;
        }
        if (dout != null)
        {
          String[] s = accessLock.getAccessLockAsString();
          NAME16[] n16 = new NAME16[2];
          n16[0] = new NAME16(s[0]);
          n16[1] = new NAME16(s[1]);
          cc = dout.putData(n16);
        }
        return cc;
      }
    });
    stockList.addProperty(TStockProperties.SRVSELFTEST, new TPropertyHandler()
    {
      protected int call(String devName, TDataType dout, TDataType din, TAccess devAccess)
      {
        if (dout.getFormat() != TFormat.CF_TEXT) return TErrorList.illegal_format;
        String fileName = gEqmFactory.getInitializer().getFecHome() + "/" + getLocalName() + "-selftest.csv";
        if (!TFecLog.fileExists(fileName)) return TErrorList.no_such_file;
        int nlines = dout.getArrayLength()/80;
        return (short) dout.putData(TFecLog.getLines(fileName,nlines));
      }
    });

    /* TEMPLATE */
    /*
     * stockList.setPropertyHandler("?", new TPropertyHandler() { public short
     * call(String devName, TDataType dout, TDataType din, short devAccess) {
     * return (short) 0; } });
     */
  }
  public TEquipmentModule(TEquipmentFunction eqpFunction)
  {
    String name = eqpFunction.getClass().getName();
    StringTokenizer t = new StringTokenizer(name, ".$");
    String lastToken = name;
    while (t.hasMoreTokens()) lastToken = t.nextToken();
    initialize(lastToken, eqpFunction);
  }
  public TEquipmentModule(String localName, TEquipmentFunction eqpFunction)
  {
    initialize(localName, eqpFunction);
  }
  public TEquipmentModule(String localName)
  {
    initialize(localName, null);
  }
  /**
   * \brief Principal constructor
   * 
   * Use this constructor in most cases.  This will attempt to determine
   * a suitable local name for the equipment module.
   *
   */
  public TEquipmentModule()
  {
    String name = this.getClass().getName();
    StringTokenizer t = new StringTokenizer(name, ".$");
    String localName = name;
    while (t.hasMoreTokens()) localName = t.nextToken();
    initialize(localName, null);
  }
  /**
   * Assigns an equipment function.
   * 
   * @param eqpFunction
   */
  public void setEquipmentFunction(TEquipmentFunction eqpFunction)
  {
    this.eqpFcn = eqpFunction;
    this.eqpFcn.setEquipmentModule(this);
  }
  /**
   * Registers a property without a handler. This property has to be handled by
   * the equipment function.
   * 
   * @param prpName property name
   * @param dout Provides output format specification
   * @param din Provides input format specification
   * @param acc Access information
   * @param prpDesc Property description
   * @param prpId Roperty ID
   * @return
   */
  public void registerProperty(String prpName, TDataType dout, TDataType din, short acc,
      TPropertyDescription prpDesc, int prpId)
  {
    if (prpDesc == null) prpDesc = new TPropertyDescription(null);
    if (prpDesc.getArrayRowLength() == 0) prpDesc.setArrayRowLength(dout.dArrayLength);
    if (dout.dArrayLength == 1) prpDesc.setArrayType(TArrayType.AT_SCALAR);
    TExportProperty prp = new TExportProperty(prpId, prpName, acc, din, dout);
    prp.setDescription(prpDesc);
    propertyList.addProperty(this, prp, null);
    if (prpName.endsWith(".NAM")) deviceList.setPropertyOriented(true);
  }
  /**
   * Registers a property and assigns a handler.
   * 
   * @param prp
   * @param handler
   */
  public void registerProperty(TExportProperty prp, TPropertyHandler handler)
  {
    propertyList.addProperty(this, prp, handler);
    String aprpnam = gEqmFactory.getAliasFromName(prp.getName());
    if (aprpnam != null)
    { // register the alias as well !
      TExportProperty aprp = new TExportProperty(prp);
      aprp.setName(aprpnam);
      propertyList.addProperty(this,aprp, handler);
    }
    if (prp.getName().endsWith(".NAM")) deviceList.setPropertyOriented(true);
 }
  public int attachPropertyHandler(String prp,TPropertyHandler handler)
  {
    String aprp = gEqmFactory.getAliasFromName(prp);
    if (aprp != null) propertyList.setHandler(aprp,handler);
    return propertyList.setHandler(prp,handler);
  }
  /**
   * Adds a collection of device names to the module's device list.
   */
  // TODO Test if sequence remains unchanged
  public void registerDevices(Collection deviceCollection)
  {
//    deviceList.addAll(deviceNames);
    int ndevs = deviceCollection.size();
    String[] devs = (String[])deviceCollection.toArray(new String[0]);
    for (int i=0; i<ndevs; i++) 
    {  
      deviceList.addDevice(devs[i],this);
    }
  }
  public void registerDevices(String[] deviceNames)
  {
    if (deviceNames == null) return;
    for (int i=0; i<deviceNames.length; i++) 
    {
      deviceList.addDevice(deviceNames[i],i,this);
    }
  }
  public void registerDevices(TDevice[] devices)
  {
    if (devices == null) return;
    for (int i=0; i<devices.length; i++) 
    {
      devices[i].setEqm(this);
      deviceList.addDevice(devices[i]);
    }
  }
  public void registerDevices(int numberOfDevices)
  {
    if (numberOfDevices <= 0) return;
    TDevice[] devices = new TDevice[numberOfDevices];
    for (int i=0; i<numberOfDevices; i++) 
    {
      devices[i] = new TDevice(i);
      devices[i].setEqm(this);
      deviceList.addDevice(devices[i]);
    }
  }
  /**
   * Adds a device name to the module's device list.
   * 
   * @param devName
   * @return
   */
  public int registerDeviceName(String devName)
  {
    //deviceList.add(devName);
    deviceList.addDevice(devName,this);
    String adev = gEqmFactory.getAliasFromName(devName);
    if (adev != null)
    { // register the alias as well !
      //deviceList.add(adev);
      deviceList.addDevice(adev,this);
    }
    return deviceList.getDeviceNumber(devName);
  }
  public int registerDeviceName(String devName,int devNumber)
  {
    //deviceList.add(devName);
    deviceList.addDevice(devName,devNumber,this);
    String adev = gEqmFactory.getAliasFromName(devName);
    if (adev != null)
    { // register the alias as well !
      //deviceList.add(adev);
      deviceList.addDevice(adev,devNumber,this);
    }
    return devNumber;
  }
  /**
   * Returns all property names in a linked list.
   * 
   * @return
   */
  public LinkedList getPropertyNames()
  {
    LinkedList rv = new LinkedList();
    rv.addAll(propertyList.getPropertyNames());
    return rv;
  }
  private void dumpList(String text,LinkedList lst)
  {
    System.out.println(text);
    for (Iterator it = lst.iterator(); it.hasNext();)
    {
      System.out.println((String) it.next());
    }
  }
  public void dumpProperties()
  {
    dumpList("Registered Properties :",getPropertyNames());
  }
  public void dumpDevices()
  {
    dumpList("Registered Devices :",getDeviceNames());    
  }
  /**
   * Returns all device names in a linked list.
   * 
   * @return
   */
  public LinkedList getDeviceNames()
  {
    return (LinkedList) deviceList.getDeviceLinkedList();
  }
  /**
   * Returns true if this equipment module has registered a stock property of a
   * given name.
   * 
   * @param propertyName
   * @return
   */
  public boolean hasStockProperty(String propertyName)
  {
    return stockList.hasProperty(propertyName);
  }
  /**
   * Calls a stock property handler
   * 
   * @param propertyName
   * @param devName
   * @param dout
   * @param din
   * @param devAccess
   * @return
   */
  public short callStockProperty(String propertyName, String devName, TDataType dout,
      TDataType din, TAccess devAccess)
  {
    return stockList.callPropertyHandler(propertyName, devName, dout, din, devAccess);
  }
  /**
   * Finds a property and calls the property handler. If no property handler is
   * installed, the equipment function is called instead.
   * 
   * @param propertyName
   * @param devName
   * @param dout
   * @param din
   * @param devAccess
   * @return Redurn code of property handler/equipment function.
   *         TErrorList.non_existent_elem if no property handler and no
   *         equipment function could be found.
   */
  public short callProperty(String propertyName, String devName, TDataType dout, TDataType din,
      TAccess devAccess)
  {
    if (propertyList.hasProperty(propertyName))
    {
      return propertyList.callPropertyHandler(propertyName, devName, dout, din, devAccess);
    }
    else if (eqpFcn != null)
    {
      return eqpFcn.call(propertyName, devName, dout, din, devAccess.toShort());
    }
    else
    {
      return TErrorList.non_existent_elem;
    }
  }
  public short callMetaProperty(String metaPropertyName, String devName, TDataType dout, TDataType din,
      TAccess devAccess)
  {
    if (!gEqmFactory.metaProps.isMetaProperty(metaPropertyName)) return TErrorList.illegal_property;
    String propertyName = gEqmFactory.metaProps.getTargetProperty(metaPropertyName);
    if (!propertyList.hasProperty(propertyName)) return TErrorList.illegal_property;
    TExportProperty prp = propertyList.getFirstProperty(propertyName);
    switch (gEqmFactory.metaProps.getIndex(metaPropertyName))
    {
      case TMetaProperties.DSC_ID:
        return (short) dout.putData(prp.getDescription().getText());        
      case TMetaProperties.EGU_ID: // to satisfy DOOCS calls ...
        switch (dout.dFormat)
        {
          case TFormat.CF_TEXT:
          case TFormat.CF_NAME16:
          case TFormat.CF_NAME32:
          case TFormat.CF_NAME64:
            return (short) dout.putData(prp.getDescription().getYRange().getUnits());
          case TFormat.CF_FLOAT:
          case TFormat.CF_DOUBLE:
          case TFormat.CF_LONG:
          case TFormat.CF_SHORT:
            float[] yrng = new float[2];
            yrng[0] = prp.getDescription().getYRange().getMinValue();
            yrng[1] = prp.getDescription().getYRange().getMaxValue();
            return (short) dout.putData(yrng);
          case TFormat.CF_USTRING:
            USTRING[] u = new USTRING[1];
            u[0] = new USTRING(prp.getDescription().getYRange().getMinValue(),
                               prp.getDescription().getYRange().getMaxValue(),
                               prp.getDescription().getYRange().getUnits());
            return (short) dout.putData(u);
          default:
            return TErrorList.illegal_format;
        }
      case TMetaProperties.HST_ID:
        THistoryRecord hst = null;
        int listsize = gLclHstList.size();
        int index = 0, sample = 0;
        long start = 0;
        long stop = System.currentTimeMillis()-1000;
        float lwr = 0, upr = 0;
        Object data = null;
        int dataSize = 0, inputDataSize = 0;
        
        if (TMetaProperties.isCentralArchive(metaPropertyName))
        {
          // TODO: return getRemoteHistoryServer();
        }
        boolean isSnapshot = TMetaProperties.isHistorySnapshot(metaPropertyName);
        dataSize = dout.getArrayLength();
        start = stop - dataSize * 1000; // millisecond timestamps here!
        // check the time range and/or filtering criteria the caller has given us ...
        if (din != null && (inputDataSize=din.getArrayLength()) > 0)
        {
          switch (din.getFormat())
          {
            case TFormat.CF_LONG:
              int[] ival = (int[])din.getDataObject();
              switch (inputDataSize)
              {
                case 4: sample = ival[3];
                case 3: index = ival[2];
                case 2: if (ival[1] < (int)(stop/1000)) stop = ((long)ival[1])*1000;
                case 1: start = ((long)ival[0])*1000;
                default: break;
              }
              if (sample < 0) sample = 0;
              break;
            case TFormat.CF_DOUBLE:
              double[] dval = (double[])din.getDataObject();
              switch (inputDataSize)
              {
                case 4: sample = (int)dval[3];
                case 3: index = (int)dval[2];
                case 2: if (dval[1] < stop/1000) stop = (long)(dval[1]*1000);
                case 1: start = (long)(dval[0]*1000);
                default: break;
              }
              if (sample < 0) sample = 0;
              break;
            case TFormat.CF_FWINDOW:
              FWINDOW[] fw = (FWINDOW[])din.getDataObject();
              start = ((FWINDOW[])fw)[0].i1val * 1000;
              stop = ((FWINDOW[])fw)[0].i2val * 1000;
              lwr = ((FWINDOW[])fw)[0].f1val;
              upr = ((FWINDOW[])fw)[0].f2val;
              if (lwr < upr) sample = -1;
              break;
            default:
              return TErrorList.illegal_format;
          }
        }
        if (isSnapshot)
        {
          if (start < 1) return TErrorList.argument_list_error;
          if (dout.getFormat() <= TFormat.CF_FLOAT) return TErrorList.illegal_format;
          stop = start;
          sample = 0;
          index = 0;
          lwr = upr = (float)0.0;          
        }
        // what the caller wants returned ....
        boolean literalDevice = false;
        for (int i=0; i<listsize; i++)
        { // go thru the list to see if the requested device was stored literally
          hst = (THistoryRecord)gLclHstList.get(i);
          if (hst.getPrp().compareTo(prp.getName()) != 0) continue;
          if (hst.getDev().compareTo(devName) != 0) continue;
          literalDevice = true;
          break;
        }
        for (int i=0; i<listsize; i++)
        {
          hst = (THistoryRecord)gLclHstList.get(i);
          if (hst.getPrp().compareTo(prp.getName()) == 0)
          {
            if (hst.getDev().compareTo(devName) != 0)
            { // should we look for a match in the device name ?
              if (index == 0 && !literalDevice)
                index = getDeviceList().getDeviceNumber(devName);
              else
                continue;
            }
            // snapshots always have index = 0
            if (start == stop && dataSize > 1) index = 0; 
            switch (dout.dFormat)
            {
              case TFormat.CF_BYTE:
                data = new byte[dataSize]; break;
              case TFormat.CF_SHORT:
                data = new short[dataSize]; break;
              case TFormat.CF_LONG:
                data = new int[dataSize]; break;
              case TFormat.CF_LNGINT:
                data = new INTINT[dataSize]; break;
              case TFormat.CF_FLOAT:
                data = new float[dataSize]; break;
              case TFormat.CF_FLTINT:
                data = new FLTINT[dataSize]; break;
              case TFormat.CF_INTFLTINT:
                data = new INTFLTINT[dataSize]; break;
              case TFormat.CF_DOUBLE:
                data = new double[dataSize]; break;
              case TFormat.CF_DBLDBL:
                data = new DBLDBL[dataSize]; break;
              case TFormat.CF_NAME16:  
                data = new NAME16[dataSize]; break;
              case TFormat.CF_NAME16I:  
                data = new NAME16I[dataSize]; break;
              case TFormat.CF_NAME32:  
                data = new NAME32[dataSize]; break;
              case TFormat.CF_NAME32I:  
                data = new NAME32I[dataSize]; break;
              case TFormat.CF_NAME64:  
                data = new NAME64[dataSize]; break;
              case TFormat.CF_NAME64I:  
                data = new NAME64I[dataSize]; break;
            }
            int nr = hst.getDataFromLTS(index,start,stop,lwr,upr,data,dataSize,0,sample);
            if (nr < 0) return (short)-nr;
            if (nr > 0) 
            {
              if (isSnapshot || nr >= dataSize - 1) 
              { // found the snapshot in or filled the buffer from LTS 
                dout.putData(data);
                dout.setDataTimeStamp(hst.getLastReadTimestamp());
                return 0;
              }
              start = hst.getLastReadTimestamp() + 1;
            }
            nr = hst.getDataFromSTS(index,start,stop,lwr,upr,data,dataSize,nr,sample);
            if (nr < 0) return (short)-nr;
            dout.putData(data);
            if (dataSize > 1 && nr < dataSize) dout.dArrayLength = nr;
            dout.setDataTimeStamp(hst.getLastReadTimestamp());
            return 0;
          }
        }
        return TErrorList.not_implemented;
      case TMetaProperties.NAM_ID:
        // if .NAM properties are registered they are handled with the normal property handler
        String[] slst = deviceList.getDeviceNameList();
        ArrayList lst = prp.getDeviceList();
        if (lst != null && lst.size() > 0) 
        {
          slst = new String[lst.size()];
          lst.toArray(slst);
        }
        return (short)dout.putData(StringToName.stringArrayToName16(slst));
      default:  
        break;
    }
    return TErrorList.illegal_property;
  }
  // csv file EXPORTS handlers:
  class expRowHndlr implements csvRowHandler
  { // the Row Handler will be called when all columns have been read in
    private TExportProperty expp;
    private TPropertyDescription prpDesc;
    private String targetLocalName = null;
    private String localName = null;
    private String expName = null;
    public void setLocalName(String value) { localName = value; }
    public void setExpName(String value) { expName = value; }
    public TExportProperty getExportProperty() { return expp; } 
    public TPropertyDescription getPropertyDescription() { return prpDesc; } 
    expRowHndlr(String targetName) 
    { 
      expp = new TExportProperty();
      prpDesc = new TPropertyDescription();
      if (targetLocalName == null) targetLocalName = targetName;
    }
    public int process(int index)
    {
      if (localName == null || localName.compareTo(moduleName) != 0) return 0;
      if (targetLocalName != null && targetLocalName.compareTo(moduleName) != 0) return 0;
      // register this one
      if (expName != null) setExportName(expName);
      expp.setDescription(prpDesc);
      registerProperty(expp,null);
      // allocate a new one
      expp = new TExportProperty();
      prpDesc = new TPropertyDescription();
      return 0;
    }
  }
  class expXnHndlr implements csvHandler
  {
    private expRowHndlr rHndlr;
    expXnHndlr(expRowHndlr rowHndlr)
    {
      rHndlr = rowHndlr; 
    }
    public int process(String strValue,int index)
    {
      if (strValue == null || strValue.length() == 0) return 0;
      String nm = (strValue.length() > 16) ? strValue.substring(0,16) : strValue;
      rHndlr.setExpName(nm);
      return 0;
    }
  }
  class expGrpHndlr implements csvHandler
  {
    public int process(String strValue,int index)
    {
      if (strValue == null || strValue.length() == 0) return 0;
      setGroupName(strValue);
      return 0;
    }
  }
  class expGrpIdxHndlr implements csvHandler
  {
    public int process(String strValue,int index)
    {
      if (strValue == null || strValue.length() == 0) return 0;
      setGroupIndex(Integer.valueOf(strValue).intValue());
      return 0;
    }
  }
  class expLnHndlr implements csvHandler
  {
    private expRowHndlr rHndlr;
    private String targetLocalName = null;
    expLnHndlr(expRowHndlr rowHndlr, String targetName)
    {
      if (targetLocalName == null) targetLocalName = targetName;
      rHndlr = rowHndlr; 
      //localName = targetName;
    }
    public int process(String strValue,int index)
    {
      rHndlr.setLocalName(strValue);
      if (strValue == null || strValue.length() == 0) return 0;
      if (targetLocalName != null && targetLocalName.compareTo(strValue) != 0) return 0;
      String nm = (strValue.length() > 6) ? strValue.substring(0,6) : strValue;
      setLocalName(nm);
      return 0;
    }
  }
  class expPrpHndlr implements csvHandler
  {
    private expRowHndlr rHndlr;
    expPrpHndlr(expRowHndlr rowHndlr) { rHndlr = rowHndlr; }
    public int process(String strValue,int index)
    {
      rHndlr.getExportProperty().setName(strValue);
      return 0;
    }
  }
  class expPrpDescHndlr implements csvHandler
  {
    private expRowHndlr rHndlr;
    expPrpDescHndlr(expRowHndlr rowHndlr) { rHndlr = rowHndlr; }
    public int process(String strValue,int index)
    {
      if (strValue == null || strValue.length() == 0) return 0;
      // parse the csv file text description according to convention
      rHndlr.getPropertyDescription().parseComplexString(strValue);
      return 0;
    }
  }
  class expPrpSizeHndlr implements csvHandler
  {
    private expRowHndlr rHndlr;
    expPrpSizeHndlr(expRowHndlr rowHndlr) { rHndlr = rowHndlr; }
    public int process(String strValue,int index)
    {
      if (strValue == null || strValue.length() == 0) return 0;
      rHndlr.getExportProperty().setOutputSize(new Integer(strValue).intValue());
      return 0;
    }
  }
  class expPrpFmtHndlr implements csvHandler
  {
    private expRowHndlr rHndlr;
    expPrpFmtHndlr(expRowHndlr rowHndlr) { rHndlr = rowHndlr; }
    public int process(String strValue,int index)
    {
      // parse the format string according to convention
      int stop = strValue.indexOf(".");
      String fmtString = stop != -1 ? strValue.substring(0,stop) : strValue;
      short fmt = TFormat.getFormatCode(fmtString);
      rHndlr.getExportProperty().setOutputFormat(fmt);
      if (stop != -1)
      {
        String fss = strValue.substring(stop+1);
        if (fmt == TFormat.CF_STRUCT)
        {
          rHndlr.getExportProperty().setOutputTag(fss);
        }
        else
        {
          rHndlr.getPropertyDescription().setArrayType(TArrayType.getType(fss));
        }
      }
      return 0;
    }
  }
  class expPrpInSizeHndlr implements csvHandler
  {
    private expRowHndlr rHndlr;
    expPrpInSizeHndlr(expRowHndlr rowHndlr) { rHndlr = rowHndlr; }
    public int process(String strValue,int index)
    {
      if (strValue == null || strValue.length() == 0) return 0;
      rHndlr.getExportProperty().setInputSize(new Integer(strValue).intValue());
      return 0;
    }
  }
  class expPrpInFmtHndlr implements csvHandler
  {
    private expRowHndlr rHndlr;
    expPrpInFmtHndlr(expRowHndlr rowHndlr) { rHndlr = rowHndlr; }
    public int process(String strValue,int index)
    {
      int stop = strValue.indexOf(".");
      String fmtString = stop != -1 ? strValue.substring(0,stop) : strValue;
      short fmt = TFormat.getFormatCode(fmtString);
      rHndlr.getExportProperty().setInputFormat(fmt);
      if (fmt == TFormat.CF_STRUCT)
      {
        rHndlr.getExportProperty().setInputTag(strValue.substring(stop+1));
      }
      return 0;
    }
  }
  class expPrpIdHndlr implements csvHandler
  {
    private expRowHndlr rHndlr;
    expPrpIdHndlr(expRowHndlr rowHndlr) { rHndlr = rowHndlr; }
    public int process(String strValue,int index)
    {
      int idx = index;
      try
      {
        if (strValue != null && strValue.length() > 0) 
          idx = new Integer(strValue).intValue();
      }
      catch (NumberFormatException e)
      {
        idx = index;
      }
      rHndlr.getExportProperty().setId(idx);
      return 0;
    }
  }
  class expPrpAccessHndlr implements csvHandler
  {
    private expRowHndlr rHndlr;
    expPrpAccessHndlr(expRowHndlr rowHndlr) { rHndlr = rowHndlr; }
    public int process(String strValue,int index)
    {
      if (strValue == null || strValue.length() == 0) return 0;
      // parse access mode according to convention
      String accessString = strValue.trim().toUpperCase();
      short ta = 0;
      if (accessString.indexOf("WRITE") >= 0) ta |=  TAccess.CA_WRITE;
      if (accessString.indexOf("READ") >= 0) ta |=  TAccess.CA_READ;
      rHndlr.getExportProperty().setAccessMode(ta);
      return 0;
    }
  }
  class expPrpRdrHndlr implements csvHandler
  {
    private expRowHndlr rHndlr;
    expPrpRdrHndlr(expRowHndlr rowHndlr) { rHndlr = rowHndlr; }
    public int process(String strValue,int index)
    {
      if (strValue == null || strValue.length() == 0) return 0;
      // TODO: parse the csv file redirection string according to convention
      rHndlr.getPropertyDescription().setRedirection(strValue);
      return 0;
    }
  }
  // TODO: call this from some strategic location 
  // (currently tested inside of SineEquipmentModule) 
  public int getExportInformationFromFile()
  {
    return getExportInformationFromFile(moduleName);
  }
  public int getExportInformationFromFile(String eqmName)
  {
    TInitializer initializer = TInitializerFactory.getInstance().getInitializer();
    // register the csv database structure: 
    csvColumn[] expCols = new csvColumn[13];
    expRowHndlr expRows = new expRowHndlr(getLocalName());
    expCols[0] = new csvColumn("EXPORT_NAME","",new expXnHndlr(expRows));
    expCols[1] = new csvColumn("LOCAL_NAME","",new expLnHndlr(expRows,getLocalName()));
    expCols[2] = new csvColumn("PROPERTY","",new expPrpHndlr(expRows));
    expCols[3] = new csvColumn("DESCRIPTION","",new expPrpDescHndlr(expRows));
    expCols[4] = new csvColumn("PROPERTY_SIZE","",new expPrpSizeHndlr(expRows));
    expCols[5] = new csvColumn("FORMAT","",new expPrpFmtHndlr(expRows));
    expCols[6] = new csvColumn("PROPERTY_INSIZE","",new expPrpInSizeHndlr(expRows));
    expCols[7] = new csvColumn("INFORMAT","",new expPrpInFmtHndlr(expRows));
    expCols[8] = new csvColumn("PROPERTY_ID","",new expPrpIdHndlr(expRows));
    expCols[9] = new csvColumn("ACCESS","READ",new expPrpAccessHndlr(expRows));
    expCols[10] = new csvColumn("REDIRECTION","",new expPrpRdrHndlr(expRows));
    expCols[11] = new csvColumn("GROUP","",new expGrpHndlr());
    expCols[12] = new csvColumn("GROUP_INDEX","",new expGrpIdxHndlr());
    // open it
    csv expFile = new csv(initializer.getExportsResource(eqmName));
    // read it
    int rc = expFile.readFile(expCols,expRows);
    // close it
    expFile.close();  
    if (rc != TErrorList.no_such_file)
      TFecLog.log("get registered exports and properties from exports.csv : " + TErrorList.errorString[rc]);
    return 0;
  }
  // csv file USERS handlers
  class usrNameHndlr implements csvHandler
  {
    public int process(String strValue,int index)
    {
      if (strValue == null || strValue.length() == 0) return 0;
      addRegisteredUser(strValue.toUpperCase());
      return 0;
    }
  }
  public int getRegisteredUsersFromFile()
  {
    TInitializer initializer = TInitializerFactory.getInstance().getInitializer();
    // register the csv database structure: 
    csvColumn[] usrCols = new csvColumn[1];
    //usrRowHndlr usrRows = new usrRowHndlr(getLocalName());
    usrCols[0] = new csvColumn("USER_NAME","",new usrNameHndlr());
    // open it
    csv usrFile = new csv(initializer.getUsersResource());
    // read it
    int rc = usrFile.readFile(usrCols);
    // close it
    usrFile.close();
    if (rc != TErrorList.no_such_file)
      TFecLog.log("get registered users from users.csv : " + TErrorList.errorString[rc]);
    return 0;
  }
  // csv file IPNETS handlers
  class subnetNameHndlr implements csvHandler
  {
    public int process(String strValue,int index)
    {
      if (strValue == null || strValue.length() == 0) return 0;
      addRegisteredNet(strValue);
      return 0;
    }
  }
  public int getRegisteredNetsFromFile()
  {
    TInitializer initializer = TInitializerFactory.getInstance().getInitializer();
    // register the csv database structure: 
    csvColumn[] netsCols = new csvColumn[1];
    //usrRowHndlr usrRows = new usrRowHndlr(getLocalName());
    netsCols[0] = new csvColumn("SUBNET","",new subnetNameHndlr());
    // open it
    csv netsFile = new csv(initializer.getNetsResource());
    // read it
    int rc = netsFile.readFile(netsCols);
    // close it
    netsFile.close();  
    if (rc != TErrorList.no_such_file)
      TFecLog.log("get registered ip nets from ipnets.csv : " + TErrorList.errorString[rc]);
    return 0;
  }
  // csv file <EQM>-alarms handlers:
  class almDefRowHndlr implements csvRowHandler
  { // the Row Handler will be called when all columns have been read in
    private TAlarmDefinition adef;
    private String tag; // 16 char
    private int code;   // 4 bytes
    private int mask;   // 4 bytes
    private short severity; // 2 bytes;
    private short dataFormat; // 1 byte
    private int dataSize; // 1 byte
    private long timeResolution; // 4 bytes
    private short hrtChannel; // 2 bytes
    private short hrtModule;  // 2 bytes
    private String text;      // 40 chars
    private String deviceText; // 38 chars
    private int system;   // 2 bytes;
    private String dataText; // 40 chars
    private String url; // 40 chars
    public TAlarmDefinition getAlarmDefinition() { return adef; } 
    almDefRowHndlr() 
    {
      tag = "";
      text = "";
      deviceText = "";
      dataText = "";
      url = "";
    }
    public int process(int index)
    {
      // make a new one
      adef = new TAlarmDefinition(tag,text,deviceText,code,severity,system,mask,dataFormat,
          dataSize,dataText,url,timeResolution,hrtChannel,hrtModule);
      // add this to the list
      addAlarmDefinition(code,adef);
      return 0;
    }
  }
  class almDefTagHndlr implements csvHandler
  {
    private almDefRowHndlr rHndlr;
    almDefTagHndlr(almDefRowHndlr rowHndlr) { rHndlr = rowHndlr; }
    public int process(String strValue,int index)
    {
      rHndlr.tag = strValue;
      return 0;
    }
  }
  class almDefCodeHndlr implements csvHandler
  {
    private almDefRowHndlr rHndlr;
    almDefCodeHndlr(almDefRowHndlr rowHndlr) { rHndlr = rowHndlr; }
    public int process(String strValue,int index)
    {
      try
      {
        rHndlr.code = Integer.parseInt(strValue);
      }
      catch (NumberFormatException e)
      {
        rHndlr.code = 0;
      }
      return 0;
    }
  }
  class almDefMaskHndlr implements csvHandler
  {
    private almDefRowHndlr rHndlr;
    almDefMaskHndlr(almDefRowHndlr rowHndlr) { rHndlr = rowHndlr; }
    public int process(String strValue,int index)
    {
      try
      {
        rHndlr.mask = Integer.parseInt(strValue);
      }
      catch (NumberFormatException e)
      {
        rHndlr.mask = 0;
      }
      return 0;
    }
  }
  class almDefSeverityHndlr implements csvHandler
  {
    private almDefRowHndlr rHndlr;
    almDefSeverityHndlr(almDefRowHndlr rowHndlr) { rHndlr = rowHndlr; }
    public int process(String strValue,int index)
    {
      try
      {
        rHndlr.severity = Short.parseShort(strValue);
      }
      catch (NumberFormatException e)
      {
        rHndlr.severity = 0;
      }
      return 0;
    }
  }
  class almDefDataFmtHndlr implements csvHandler
  {
    private almDefRowHndlr rHndlr;
    almDefDataFmtHndlr(almDefRowHndlr rowHndlr) { rHndlr = rowHndlr; }
    public int process(String strValue,int index)
    {
      rHndlr.dataFormat = TFormat.getFormatCode(strValue);
      return 0;
    }
  }
  class almDefDataSizHndlr implements csvHandler
  {
    private almDefRowHndlr rHndlr;
    almDefDataSizHndlr(almDefRowHndlr rowHndlr) { rHndlr = rowHndlr; }
    public int process(String strValue,int index)
    {
      try
      {
        rHndlr.dataSize = Integer.parseInt(strValue);
      }
      catch (NumberFormatException e)
      {
        rHndlr.dataSize = 0;
      }
      return 0;
    }
  }
  class almDefTimeResHndlr implements csvHandler
  {
    private almDefRowHndlr rHndlr;
    almDefTimeResHndlr(almDefRowHndlr rowHndlr) { rHndlr = rowHndlr; }
    public int process(String strValue,int index)
    {
      try
      {
        rHndlr.timeResolution = Long.parseLong(strValue);
      }
      catch (NumberFormatException e)
      {
        rHndlr.timeResolution = 0;
      }
      return 0;
    }
  }
  class almDefHrtChanHndlr implements csvHandler
  {
    private almDefRowHndlr rHndlr;
    almDefHrtChanHndlr(almDefRowHndlr rowHndlr) { rHndlr = rowHndlr; }
    public int process(String strValue,int index)
    {
      try
      {
        rHndlr.hrtChannel = Short.parseShort(strValue);
      }
      catch (NumberFormatException e)
      {
        rHndlr.hrtChannel = 0;
      }
      return 0;
    }
  }
  class almDefHrtModHndlr implements csvHandler
  {
    private almDefRowHndlr rHndlr;
    almDefHrtModHndlr(almDefRowHndlr rowHndlr) { rHndlr = rowHndlr; }
    public int process(String strValue,int index)
    {
      try
      {
        rHndlr.hrtModule = Short.parseShort(strValue);
      }
      catch (NumberFormatException e)
      {
        rHndlr.hrtModule = 0;
      }
      return 0;
    }
  }
  class almDefTextHndlr implements csvHandler
  {
    private almDefRowHndlr rHndlr;
    almDefTextHndlr(almDefRowHndlr rowHndlr) { rHndlr = rowHndlr; }
    public int process(String strValue,int index)
    {
      rHndlr.text = strValue;
      return 0;
    }
  }
  class almDefDevTextHndlr implements csvHandler
  {
    private almDefRowHndlr rHndlr;
    almDefDevTextHndlr(almDefRowHndlr rowHndlr) { rHndlr = rowHndlr; }
    public int process(String strValue,int index)
    {
      rHndlr.deviceText = strValue;
      return 0;
    }
  }
  class almDefDataTextHndlr implements csvHandler
  {
    private almDefRowHndlr rHndlr;
    almDefDataTextHndlr(almDefRowHndlr rowHndlr) { rHndlr = rowHndlr; }
    public int process(String strValue,int index)
    {
      rHndlr.dataText = strValue;
      return 0;
    }
  }
  class almDefUrlHndlr implements csvHandler
  {
    private almDefRowHndlr rHndlr;
    almDefUrlHndlr(almDefRowHndlr rowHndlr) { rHndlr = rowHndlr; }
    public int process(String strValue,int index)
    {
      rHndlr.url = strValue;
      return 0;
    }
  }
  class almDefSystemHndlr implements csvHandler
  {
    private almDefRowHndlr rHndlr;
    almDefSystemHndlr(almDefRowHndlr rowHndlr) { rHndlr = rowHndlr; }
    public int process(String strValue,int index)
    {
      try
      {
        rHndlr.system = Integer.parseInt(strValue);
      }
      catch (NumberFormatException e)
      {
        rHndlr.system = 0;
      }
      return 0;
    }
  }
  public int getRegisteredAlarmDefinitionsFromFile()
  {
    TInitializer initializer = TInitializerFactory.getInstance().getInitializer();
    // register the csv database structure: 
    csvColumn[] almCols = new csvColumn[14];
    almDefRowHndlr almRows = new almDefRowHndlr();
    almCols[0] = new csvColumn("ALARM_TAG","",new almDefTagHndlr(almRows));
    almCols[1] = new csvColumn("ALARM_CODE","",new almDefCodeHndlr(almRows));
    almCols[2] = new csvColumn("ALARM_MASK","",new almDefMaskHndlr(almRows));
    almCols[3] = new csvColumn("SEVERITY","",new almDefSeverityHndlr(almRows));
    almCols[4] = new csvColumn("DATA_FORMAT","",new almDefDataFmtHndlr(almRows));
    almCols[5] = new csvColumn("DATA_ARRAYSIZE","",new almDefDataSizHndlr(almRows));
    almCols[6] = new csvColumn("TIME_RESOLUTION","",new almDefTimeResHndlr(almRows));
    almCols[7] = new csvColumn("HITS_CHANNEL","",new almDefHrtChanHndlr(almRows));
    almCols[8] = new csvColumn("HITS_MODULE","",new almDefHrtModHndlr(almRows));
    almCols[9] = new csvColumn("ALARM_TEXT","READ",new almDefTextHndlr(almRows));
    almCols[10] = new csvColumn("DEVICE_TEXT","",new almDefDevTextHndlr(almRows));
    almCols[11] = new csvColumn("DATA_TEXT","",new almDefDataTextHndlr(almRows));
    almCols[12] = new csvColumn("URL","",new almDefUrlHndlr(almRows));
    almCols[13] = new csvColumn("ALARM_SYSTEM","",new almDefSystemHndlr(almRows));
    // open it
    csv almDefsFile = new csv(initializer.getAlmDefinitionResource(getLocalName()));
    // read it
    int rc = almDefsFile.readFile(almCols,almRows);
    // close it
    almDefsFile.close();  
    if (rc != TErrorList.no_such_file)
      TFecLog.log("get alarm definitions from " + getLocalName() + "-alarms.csv : " + TErrorList.errorString[rc]);
    return 0;
  }
  // csv file <EQM>-devices.csv and <property>-names.csv handlers:
  class devNameRowHndlr implements csvRowHandler
  { // the Row Handler will be called when all columns have been read in
    private String deviceName;
    private int deviceNumber;
    private boolean isPrpOriented = false;
    private String deviceRedirection;
    private TExportProperty prp = null;
    private String prpListFile = null;
    devNameRowHndlr() 
    {
      deviceNumber = 0;
      deviceName = "#0";
    }
    devNameRowHndlr(TExportProperty property) 
    {
      deviceNumber = 0;
      deviceName = "#0";
      deviceRedirection = null;
      prp = property;
      isPrpOriented = prp == null ? false : true;
    }
    public int process(int index)
    {
      TDeviceList dlst = getDeviceList();
      if (dlst == null) return TErrorList.device_not_connected;
      int ndevs = dlst.getNumberOfDevices();
      if (ndevs == 0) return TErrorList.device_not_connected;
      if (deviceNumber < 0 && deviceNumber >= ndevs) return TErrorList.illegal_equipment_number;
      if (!isPrpOriented)
      {
        TDevice tdv = dlst.getDevice(deviceNumber);
        if (tdv == null) return TErrorList.device_not_connected;
        String oldName = tdv.getName();
        if (deviceName.compareTo(oldName) != 0)
        {
          dlst.renameDevice(oldName,deviceName);
        }
        if (deviceRedirection != null && deviceRedirection.length() > 0)
        {
          tdv.setRedirection(deviceRedirection);
          deviceRedirection = null; // reset this !
        }
        if (prpListFile != null && prpListFile.length() > 0)
        {
          getRegisteredPropertyListFromFile(tdv,prpListFile);
          propertyList.setDeviceOriented(true);
          prpListFile = null; // reset this !
        }
      }
      else
      { 
        if (prp != null) 
        {
          int prplen = prp.getOutputSize();
          ArrayList lst = prp.getDeviceList();
          if (lst == null) 
          {
            lst = new ArrayList(prplen);
            prp.setDeviceList(lst);  
          }
          if (deviceNumber < prplen) lst.add(deviceNumber,deviceName);
          dlst.setPropertyOriented(true);
        }
      }
      return 0;
    }
  }
  class devNameHndlr implements csvHandler
  {
    private devNameRowHndlr rHndlr;
    devNameHndlr(devNameRowHndlr rowHndlr) { rHndlr = rowHndlr; }
    public int process(String strValue,int index)
    {
      if (strValue == null || strValue.length() == 0) return 0;
      String nm = (strValue.length() > 16) ? strValue.substring(0,16) : strValue;
      rHndlr.deviceName = nm;
      return 0;
    }
  }
  class devNumHndlr implements csvHandler
  {
    private devNameRowHndlr rHndlr;
    devNumHndlr(devNameRowHndlr rowHndlr) { rHndlr = rowHndlr; }
    public int process(String strValue,int index)
    {
      if (strValue == null) return 0;
      try
      {
        int devnr = Integer.parseInt(strValue);
        if (devnr < 0) return TErrorList.illegal_equipment_number;
        rHndlr.deviceNumber = devnr;
      }
      catch (NumberFormatException e)
      {
        return 0;
      }
      
      return 0;
    }
  }
  class devRdrHndlr implements csvHandler
  {
    private devNameRowHndlr rHndlr;
    devRdrHndlr(devNameRowHndlr rowHndlr) { rHndlr = rowHndlr; }
    public int process(String strValue,int index)
    {
      if (strValue == null || strValue.length() == 0) return 0;
      rHndlr.deviceRedirection = strValue;
      return 0;
    }
  }
  class devPrpHndlr implements csvHandler
  {
    private devNameRowHndlr rHndlr;
    devPrpHndlr(devNameRowHndlr rowHndlr) { rHndlr = rowHndlr; }
    public int process(String strValue,int index)
    {
      if (strValue == null || strValue.length() == 0) return 0;
      String nm = (strValue.length() > 32) ? strValue.substring(0,32) : strValue;
      rHndlr.prpListFile = nm;
      return 0;
    }
  }
  public int getRegisteredDevicesFromFile()
  {
    TInitializer initializer = TInitializerFactory.getInstance().getInitializer();
    csv devNamesFile = new csv(initializer.getDevicesResource(getLocalName()));
    int rc = devNamesFile == null ? 
        TErrorList.file_error : 
        getRegisteredDevicesFromFile(devNamesFile,new devNameRowHndlr());
    if (rc != TErrorList.no_such_file)
      TFecLog.log("get device names from " + getLocalName() + "-devices.csv : " + TErrorList.errorString[rc]);
    return rc;
  }
  public int getRegisteredNamesFromPropertyFile(String propertyName)
  {
    TInitializer initializer = TInitializerFactory.getInstance().getInitializer();
    TExportProperty prp = propertyList.getFirstProperty(propertyName);
    String devFileName = propertyName + "-names.csv";
    csv devNamesFile = new csv(initializer.getFecHome() + "/" + getLocalName() + "/" + devFileName);
    int rc = devNamesFile == null ? 
        TErrorList.file_error : 
        getRegisteredDevicesFromFile(devNamesFile,new devNameRowHndlr(prp));
    if (rc == TErrorList.no_such_file)
    {
      devNamesFile = new csv(initializer.getFecHome() + "/" + devFileName);
      rc = devNamesFile == null ? 
          TErrorList.file_error : 
          getRegisteredDevicesFromFile(devNamesFile,new devNameRowHndlr(prp));
    }
    if (rc != TErrorList.no_such_file)
      TFecLog.log("get device names from " + devFileName + " : " + TErrorList.errorString[rc]);
    return rc;
  }
  public int getRegisteredDevicesFromFile(csv csvFile,devNameRowHndlr devRows)
  {
    // register the csv database structure: 
    devNumHndlr numhdlr = new devNumHndlr(devRows);
    devNameHndlr namhdlr = new devNameHndlr(devRows);
    devRdrHndlr rdrhdlr = new devRdrHndlr(devRows);
    devPrpHndlr prphdlr = new devPrpHndlr(devRows);
    csvColumn[] devCols = new csvColumn[6];
    devCols[0] = new csvColumn("DEVICE_NUMBER","",numhdlr);
    devCols[1] = new csvColumn("DEVICE_NAME","",namhdlr);
    devCols[2] = new csvColumn("MODULE_NUMBER","",numhdlr); // alias for DEVICE_NAME
    devCols[3] = new csvColumn("MODULE_NAME","",namhdlr);
    devCols[4] = new csvColumn("REDIRECTION","",rdrhdlr);
    devCols[5] = new csvColumn("PROPERTY_LIST","",prphdlr);
    // read the file
    int rc = csvFile.readFile(devCols,devRows);
    // close it
    csvFile.close();  
    return rc;
  }
  class devPrpLstHndlr implements csvHandler
  {
    protected LinkedList prpLst = null;
    public int process(String strValue,int index)
    {
      if (strValue == null || strValue.length() == 0) return 0;
      if (prpLst == null) prpLst = new LinkedList();
      prpLst.add(strValue);
      return 0;
    }
  }
  public int getRegisteredPropertyListFromFile(TDevice device, String filename)
  {
    TInitializer initializer = TInitializerFactory.getInstance().getInitializer();
    csv csvFile = new csv(initializer.getFecHome() + "/" + filename);
    devPrpLstHndlr namhdlr = new devPrpLstHndlr();
    csvColumn[] devCols = new csvColumn[1];
    devCols[0] = new csvColumn("PROPERTY_NAME","",namhdlr);
    // read the file
    int rc = csvFile.readFile(devCols,null);
    // close it
    csvFile.close();
    if (rc == 0) device.setPropertyList(namhdlr.prpLst);
    return rc;
  }
  public boolean isDeviceSetLocal(String devName, String propertyName)
  {
    TExportProperty prp = propertyList.getFirstProperty(propertyName);
    WildcardMatch wc = new WildcardMatch(devName);    
    String tgt;
    TDevice[] dlst = getDeviceList().getDeviceList();
    for (int i=0; i<dlst.length; i++)
    {
      if ((tgt=dlst[i].getRedirection()) != null) 
      {
        if (wc.matches(tgt)) return false;
      }
    }    
    if ((tgt=prp.getDescription().getRedirection()) != null)
    {
      if (wc.matches(tgt)) return false;      
    }
    return true; /* appears to be local so-far */    
  }
  public String getSubsystem()
  {
    return subsystem;
  }
  public void setSubsystem(String subsystem)
  {
    this.subsystem = subsystem;
  }
  /**
   * \brief An equipment module can schedule a property or set of properties for immediate 
   * execution and delivery  
   * 
   *  \param devPropertyList A single string argument containing one or more
   *  properties to be scheduled for immediate execution.  This should be a
   *  single string.  If more than one property is to be scheduled, then the
   *  property list string should be a set of properties separated by commas or
   *  blanks
   *  
   *  
   * @author duval 
   */
  public int scheduleProperty(String devPropertyList)
  {
    if (devPropertyList == null) return TErrorList.argument_list_error;
    String[] prps  = null;
    String tok = " "; // try a space first
    if (!devPropertyList.contains(tok)) tok = ";";
    if (!devPropertyList.contains(tok)) tok = ",";
    if ((prps = devPropertyList.split(tok)) == null) return TErrorList.code_failure;
    for (int i=0; i<prps.length; i++)
    {
      if (propertyList.hasProperty(prps[i])) gEqmFactory.systemScheduleProperty(this,prps[i].trim());
    }
    return 0;
  }
}