/*
 * Created on Aug 31, 2005
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package de.desy.tine.server.histories;
import java.util.Calendar;
import java.io.*;
import de.desy.tine.dataUtils.TDataTime;
import de.desy.tine.dataUtils.TDataType;
import de.desy.tine.definitions.TErrorList;
import de.desy.tine.definitions.TFormat;
import de.desy.tine.server.equipment.TEquipmentModule;
import de.desy.tine.server.logger.TFecLog;
import de.desy.tine.startup.TInitializerFactory;
import de.desy.tine.types.*;

public class THistoryRecord
{
  private TEquipmentModule eqm;
  private String prp;   // 32 char
  private String dev; // 16 char
  private int siz;
  private byte fmt;
  private int  aType;
  private THistorySpecification hspec;
  private int  rngPtr;
  private int  rcdIdx;       // local index identifier
  private int  rngBufferSize;
  private long lastArchiveTime;
  private String ArchiveFileName; // current working file name
  private int lastArchivedDayOfTheMonth = -1;
  private boolean useMonthlyHistoryFiles = false;
  private boolean useHighResolutionTimeStamp = false;
  private Calendar calendar;
  private long[] timestamp;         // timestamp array
  private long lastReadTimestamp = 0;
  private boolean isScheduled = false;
  public void setScheduled(boolean value) { isScheduled = value; }
  byte[] bRngBuffer;       // history data ?
  short[] sRngBuffer;
  int[] lRngBuffer;
  float[] fRngBuffer;
  double[] dRngBuffer;
  NAME16[] n16RngBuffer;
  NAME32[] n32RngBuffer;
  NAME64[] n64RngBuffer;
  public static boolean isAllowableFormat(String fmt)
  {
    return isAllowableFormat(TFormat.getFormatCode(fmt));
  }
  public static boolean isAllowableFormat(short fmt)
  {
    switch (fmt)
    {
      case TFormat.CF_BYTE:
      case TFormat.CF_DOUBLE:
      case TFormat.CF_SHORT:
      case TFormat.CF_LONG:
      case TFormat.CF_FLOAT:
      case TFormat.CF_NAME16:
      case TFormat.CF_NAME32:
      case TFormat.CF_NAME64:
        return true;
    }
    return false;
  }
  public THistoryRecord(TEquipmentModule equipmentModule,String devName,String devProperty,int size,
      int format,int arrayType,int recordIndex,THistorySpecification hstSpecification)
  {
    rngPtr = -1; // ring buffer as yet empty !
    eqm = equipmentModule;
    prp = devProperty;
    dev = devName;
    siz = size;
    fmt = (byte)format;
    aType = arrayType;
    rcdIdx = recordIndex;
    hspec = hstSpecification;
    TFecLog.log("keep history of " + devName + " " + devProperty);
    switch (fmt)
    {
      case TFormat.CF_BYTE:
        rngBufferSize = TFormat.formatSizeOf(fmt) * siz * hspec.getDepthShort();
        bRngBuffer = new byte[rngBufferSize];
        break;
      case TFormat.CF_SHORT:
        rngBufferSize = siz * hspec.getDepthShort();
        sRngBuffer = new short[rngBufferSize];
        break;
      case TFormat.CF_LONG:
        rngBufferSize = siz * hspec.getDepthShort();
        lRngBuffer = new int[rngBufferSize];
        break;
      case TFormat.CF_FLOAT:
        rngBufferSize = siz * hspec.getDepthShort();
        fRngBuffer = new float[rngBufferSize];
        break;
      case TFormat.CF_DOUBLE:
        rngBufferSize = siz * hspec.getDepthShort();
        dRngBuffer = new double[rngBufferSize];
        break;
      case TFormat.CF_NAME16:
        rngBufferSize = siz * hspec.getDepthShort();
        n16RngBuffer = new NAME16[rngBufferSize];
        break;
      case TFormat.CF_NAME32:
        rngBufferSize = siz * hspec.getDepthShort();
        n32RngBuffer = new NAME32[rngBufferSize];
        break;
      case TFormat.CF_NAME64:
        rngBufferSize = siz * hspec.getDepthShort();
        n64RngBuffer = new NAME64[rngBufferSize];
        break;
      default:
        TFecLog.log("history record of format type " + TFormat.toString(fmt) + " not allowed!");
        return;
    }
    timestamp = new long[rngBufferSize];
  }
  public int writeRecordToLTS(Object values)
  { // write to Long-Term-Storage (i.e. disk)
    long ts = getTimestamp();
    int tslen = useHighResolutionTimeStamp ? 8 : 4;
    short fmt = getFmt();
    String dirName = TInitializerFactory.getInstance().getInitializer().getHistoryHome();
    String fileName;
    try 
    {
      File d = new File(dirName);
      if (!d.exists()) d.mkdirs();
      int nsiz = TFormat.formatSizeOf(fmt);
      ByteArrayOutputStream baos = new ByteArrayOutputStream(tslen+siz*nsiz);
      DataOutputStream os = new DataOutputStream(baos);
      if (useHighResolutionTimeStamp)
        os.writeDouble((double)ts/1000.0);
      else
        os.writeInt((int)(ts/1000));
      switch (fmt)
      {
        case TFormat.CF_BYTE:
          for (int i=0; i<siz; i++) os.writeByte(((byte[])values)[i]);
          break;
        case TFormat.CF_SHORT:
          for (int i=0; i<siz; i++) os.writeShort(((short[])values)[i]);
          break;
        case TFormat.CF_LONG:
          for (int i=0; i<siz; i++) os.writeInt(((int[])values)[i]);
          break;
        case TFormat.CF_FLOAT:
          for (int i=0; i<siz; i++) os.writeFloat(((float[])values)[i]);
          break;
        case TFormat.CF_DOUBLE:
          for (int i=0; i<siz; i++) os.writeDouble(((double[])values)[i]);
          break;
        case TFormat.CF_NAME16:
        case TFormat.CF_NAME32:
        case TFormat.CF_NAME64:
          for (int i=0; i<siz; i++) os.write(((NAME[])values)[i].toByteArray(),i*nsiz,nsiz);
          break;
      }
      fileName = dirName + "/" + ArchiveFileName;
      RandomAccessFile raf = new RandomAccessFile(fileName,"rw");
      raf.seek(raf.length()); // SEEK_END
      raf.write(baos.toByteArray());
      raf.close();
      lastArchiveTime = ts;
    }
    catch (Exception e)
    {
      TFecLog.log("HST: cannot write record " + getDev() + " " + getPrp());
    }
    return 0;
  }
  public long getLastReadTimestamp() { return lastReadTimestamp; }
  public int getNumDaysInMonth(int year,int month)
  {
    boolean isLeapYear = false;
    if ((year % 4) == 0) isLeapYear = true;
    switch (month)
    {
      case 0: case 2: case 4: case 6: case 7: case 9: case 11: return 31;
      case 1: return isLeapYear ? 29 : 28;
      default: return 30;
    }
  }
  public int getNumDaysInTimeSpan(long t1,long t2)
  {
    if (calendar == null) calendar = Calendar.getInstance();    
    int n = 0, nx, y, d, ms, me, i;

    calendar.setTimeInMillis(t1);
    int y1 = calendar.get(Calendar.YEAR);
    int m1 = calendar.get(Calendar.MONTH);
    int d1 = calendar.get(Calendar.DAY_OF_MONTH);
    
    calendar.setTimeInMillis(t2);
    int y2 = calendar.get(Calendar.YEAR);
    int m2 = calendar.get(Calendar.MONTH);
    int d2 = calendar.get(Calendar.DAY_OF_MONTH);

    d = getNumDaysInMonth(y2,m2);
    nx = d1 + (d-d2-1);
    for (y=y1; y<y2+1; y++)
    {
      ms = y == y1 ? m1 : 0;
      me = y == y2 ? m2 : 11;
      for (i=ms; i<me+1; i++) 
      {
        n += getNumDaysInMonth(y,i);
      }
    }
    return n - nx;
  }
  public int getDataFromLTS(int index,long starttime,long endtime,
      float lwrLimit,float uprLimit,Object dataTarget,int arraysize,int arrayOffset,int sample)
  {
    int ps;
    int recsread=0,recsfound=0,rnumber=0,asize,daystotal,daysinmonth;
    boolean finalPass=false;
    long t = System.currentTimeMillis(),itime = 0;
    THistorySpecification spc = getHspec();
    
    if (index < 0 || index > siz) return -TErrorList.out_of_range;
    if (arraysize < 1) return -TErrorList.buffer_too_small;
    if (spc.getDepthShort() <= 0) return -TErrorList.server_redirection;   
    if (spc.getDepthLong() <= 0) return 0;
    if (starttime >= t - spc.getDepthShort()) return 0;

    short targetFmt = TDataType.getObjectFormat(dataTarget);
    if (targetFmt == TFormat.CF_NULL) return -TErrorList.illegal_format;
    
    if (calendar == null) calendar = Calendar.getInstance();    
    calendar.setTimeInMillis(endtime); // end time
    int ye = calendar.get(Calendar.YEAR);
    int de = calendar.get(Calendar.DAY_OF_MONTH);
    int me = calendar.get(Calendar.MONTH);
    calendar.setTimeInMillis(starttime); // start time
    int ys = calendar.get(Calendar.YEAR); 
    int ds = calendar.get(Calendar.DAY_OF_MONTH);
    int ms = calendar.get(Calendar.MONTH);
    int yi, di, mi; // interim time
    
    if (me == ms && de == ds)
    {
      asize = arraysize;
      finalPass = true;
    }
    else if (arraysize == 1)   
    { // just a number query
      asize = arraysize;
    }
    else                       
    { // spans multiple files
      asize = 0;
    }

    daystotal = getNumDaysInTimeSpan(starttime,endtime);

    // if (arraysize > 1) recsread = 1; < a hold-over from the old days ?
    
    while (recsread < arraysize)
    {
      if (asize < arraysize || arraysize == 1)
      {
        yi = ys; mi = ms; di = ds;
        if (yi == ye && mi == me && 
            (di == de || useMonthlyHistoryFiles))
        {
          asize = arraysize;
          finalPass = true;
        }
        else  
        { // move along
          di = (di+1)%(getNumDaysInMonth(yi,mi)+1);
          if (di == 0)
          {
            di++;
            mi = (mi+1)%12;
            if (mi == 0) yi++;
          }
          // start at midnight 
          calendar.set(yi,mi,di,0,0,0);
          itime = calendar.getTimeInMillis();
          daysinmonth = 1;
          if ((daysinmonth =(int)((itime-starttime)/(3600*24))) < 1) daysinmonth = 1;
          if (arraysize > 1) 
          {
            if (targetFmt <= TFormat.CF_FLOAT) 
            {
              asize = arraysize;
            }
            else 
            {
              int add = (arraysize * daysinmonth)/daystotal;
              if (add < 1) add = 1;
              asize += add;
            }
            if (asize >= arraysize) finalPass = true; // just in case
          }
        }
      }
      if (asize > arraysize) asize = arraysize;
      ps = loadLTSData(index,starttime,endtime,lwrLimit,uprLimit,dataTarget,asize,recsread,sample);
      if (ps < 0) return ps;
      if (asize == 1) recsfound += (int)ps; else recsread += (int)ps;
      if (finalPass) break;
      starttime = itime;
      calendar.setTimeInMillis(starttime);
      ys = calendar.get(Calendar.YEAR);
      ds = calendar.get(Calendar.DAY_OF_MONTH);
      ms = calendar.get(Calendar.MONTH);
    }
    rnumber = recsfound > 0 ? recsfound : recsread;
    if (arraysize == 1) recsread = recsfound;

    if (arraysize < 2) assignNumberOfPoints(dataTarget,rnumber);

    return rnumber;
  }
  private String makeFileNameFromTime(long time)
  {
    String prefix = useHighResolutionTimeStamp ? "ha" : "ar";
    if (calendar == null) calendar = Calendar.getInstance();
    calendar.setTimeInMillis(time);
    int dd = calendar.get(Calendar.DAY_OF_MONTH);
    int yy, mm;
    lastArchivedDayOfTheMonth = dd;
    yy = calendar.get(Calendar.YEAR) % 100;
    mm = calendar.get(Calendar.MONTH) + 1; // January = 0
    if (useMonthlyHistoryFiles) dd = 1; 
    return prefix +  twoDigitString(yy) + twoDigitString(mm) + 
                      twoDigitString(dd) + "." + Integer.toHexString(rcdIdx);
  }
  private int locateRecordFromLTS(String fileName,double dt)
  {
    double dt0,dt1,dt2;
    int nrecs,p0 = 0,p1 = 0,p2 = 0;
    double f;
    int it=0;
    int rlen = getRecordLength();

    try
    {
      String fn = TInitializerFactory.getInstance().getInitializer().getHistoryHome() + "/" + fileName;
      RandomAccessFile raf = new RandomAccessFile(fn,"r");
      nrecs = (int)raf.length()/rlen;
      p2 = (nrecs-1)*rlen;
      raf.seek(p0); 
      dt0 = useHighResolutionTimeStamp ? raf.readDouble() : (double)raf.readInt();
      if (dt < dt0) p2 = 0;  // stay out of the while loop
      raf.seek(p2);
      dt2 = useHighResolutionTimeStamp ? raf.readDouble() : (double)raf.readInt();
      while (p2 > p0 + rlen)
      {
        if (dt > (dt1=dt2)) { p1 = p2; break; }
        while (dt1 > dt0)   
        { // seek <--
          f = ((dt-dt0)/(dt1-dt0))*((double)(p2-p0)/rlen);
          if ((p1=p0+rlen*(int)f) >= p2) break;
          raf.seek(p1);
          dt1 = useHighResolutionTimeStamp ? raf.readDouble() : (double)raf.readInt();
          if (dt1 < dt) break;
          p2 = p1; dt2 = dt1;
          it++;
        }
        dt0 = dt1; p0 = p1;
        while (dt2 > dt1) 
        {  // seek -->
          f = ((dt2-dt)/(dt2-dt1))*((double)(p2-p0)/rlen);
          if ((p1=p2-rlen*(int)f) <= p0) break;
          raf.seek(p1);
          dt1 = useHighResolutionTimeStamp ? raf.readDouble() : (double)raf.readInt();
          if (dt1 > dt) break;
          p0 = p1; dt0 = dt1;
          it++;
        }
        p2 = p1; dt2 = dt1;
      }
      raf.close();
    }
    catch (Exception e)
    {
      System.out.println(e.toString());
    }
    return p1;
  }
  private int loadLTSData(int index,long timestampStart,long timestampEnd,
      float lwrLimit, float uprLimit,Object dataTarget,int arraysize,int arrayoffset,int sample)
  {
    int depth = hspec.getDepthLong();
    if (arraysize < 1) return 0;
    if (depth < 0) return 0;
    long tnow = System.currentTimeMillis();
    if ((tnow - timestampStart) < hspec.getDepthShort()) return 0;
    if (index < 0 || index >= getArraySize()) return -TErrorList.invalid_index;
    double dstarttime = (double)(timestampStart/1000.0);
    double dendtime = (double)(timestampEnd/1000.0);
    double dtnow = (double)(tnow)/1000.0;
    double datatime = 0.0;
    int nrecs,raster = 1;
    int timebin = 1,pstart,pstop,number = 0;
    boolean isSnapshot = false;

    String fileName = makeFileNameFromTime(timestampStart); 
    if (dendtime < dstarttime) dendtime = dtnow;
    if ((pstart=locateRecordFromLTS(fileName,dstarttime)) < 0) return number;
    if ((pstop=locateRecordFromLTS(fileName,dendtime)) < 0) return number;
    int rlen = getRecordLength();
    nrecs = (int)((pstop-pstart)/rlen);
    if (nrecs == 0 || arraysize == 1) 
    {
      lastReadTimestamp = timestampEnd;
      return nrecs > 0 ? nrecs : 0;
    }
    arraysize -= arrayoffset;  // subtract off the offset
    if (arraysize < nrecs) raster += nrecs/(arraysize+1);
    if (sample > 0) raster = sample;

    short targetFmt = TDataType.getObjectFormat(dataTarget);
    if (targetFmt == TFormat.CF_NULL) return -TErrorList.illegal_format;
    isSnapshot = (dendtime-dstarttime) < 0.01 && targetFmt <= TFormat.CF_FLOAT;
    if (isSnapshot)
    {
      index = 0;
      datatime = dstarttime - timebin;
    }
    try 
    {
      int cnt = 0;
      double dts = 0.0;
      short sourceFmt = getFmt(); 
      double[] dv = null;
      NAME[] nv = null;
      int nsiz = TFormat.formatSizeOf(sourceFmt);
      
      String fn = TInitializerFactory.getInstance().getInitializer().getHistoryHome() + "/" + fileName;
      RandomAccessFile raf = new RandomAccessFile(fn,"r");
      long flen = raf.length();
      raf.seek(pstart);
      
      while (raf.getFilePointer() < flen) // read through file
      {
        dts = useHighResolutionTimeStamp ? raf.readDouble() : (double)raf.readInt();
        cnt++;
        if (isSnapshot && dts > dendtime) dendtime = dts;
        if (dts > dendtime) return number;
        if (dts < datatime + timebin)
        {
          raf.seek(raf.getFilePointer()+rlen);
          continue;
        }
        datatime = dts;
        // read the data
        switch (sourceFmt)
        {
          case TFormat.CF_DOUBLE:
            if (dv == null) dv = new double[siz];
            for (int i=0; i<siz; i++) dv[i] = raf.readDouble();
            break;
          case TFormat.CF_FLOAT:
            if (dv == null) dv = new double[siz];
            for (int i=0; i<siz; i++) dv[i] = raf.readFloat();
            break;
          case TFormat.CF_LONG:
            if (dv == null) dv = new double[siz];
            for (int i=0; i<siz; i++) dv[i] = raf.readInt();
            break;
          case TFormat.CF_SHORT:
            if (dv == null) dv = new double[siz];
            for (int i=0; i<siz; i++) dv[i] = raf.readShort();
            break;
          case TFormat.CF_BYTE:
            if (dv == null) dv = new double[siz];
            for (int i=0; i<siz; i++) dv[i] = raf.readByte();
            break;
          case TFormat.CF_NAME16:
          case TFormat.CF_NAME32:
          case TFormat.CF_NAME64:
            if (nv == null) nv = new NAME[siz];
            byte[] b = new byte[nsiz];
            for (int i=0; i<siz; i++) 
            {
              nv[i] = new NAME(nsiz);
              raf.read(b,0,nsiz);
              nv[i].setName(b);
            }
            break;
        }
        if (isSnapshot)
        {
          if (targetFmt == TFormat.CF_FLOAT)
          { // return the trace snapshot
            if (dv == null) return -TErrorList.illegal_format;
            number = arraysize < siz ? arraysize : siz;
            for (int i=0; i<arraysize; i++) ((float[])dataTarget)[i] = (float)dv[i];
            break;          
          }
          else if (targetFmt == TFormat.CF_LONG)
          { // return the trace snapshot
            if (dv == null) return -TErrorList.illegal_format;
            number = arraysize < siz ? arraysize : siz;
            for (int i=0; i<arraysize; i++) ((int[])dataTarget)[i] = (int)dv[i];
            break;          
          }
          else if (targetFmt == TFormat.CF_DOUBLE)
          { // return the trace snapshot
            if (dv == null) return -TErrorList.illegal_format;
            number = arraysize < siz ? arraysize : siz;
            for (int i=0; i<arraysize; i++) ((double[])dataTarget)[i] = dv[i];
            break;          
          }
        }
        switch (targetFmt)
        {
          case TFormat.CF_FLOAT:
            if (dv == null) return -TErrorList.illegal_format;
            ((float[])dataTarget)[number] = (float)dv[index];
            if (sample < 0 && !isInRange((float)dv[index],lwrLimit,uprLimit)) continue;
            break;
          case TFormat.CF_FLTINT:
            if (dv == null) return -TErrorList.illegal_format;
            if (((FLTINT[])dataTarget)[number] == null) ((FLTINT[])dataTarget)[number] = new FLTINT();
            ((FLTINT[])dataTarget)[number].ival = (int)dts;
            ((FLTINT[])dataTarget)[number].fval = (float)dv[index];
            if (sample < 0 && !isInRange((float)dv[index],lwrLimit,uprLimit)) continue;
            break;
          case TFormat.CF_INTFLTINT:            
            if (dv == null) return -TErrorList.illegal_format;
            if (((INTFLTINT[])dataTarget)[number] == null) ((INTFLTINT[])dataTarget)[number] = new INTFLTINT();
            ((INTFLTINT[])dataTarget)[number].i1val = (int)dts;
            ((INTFLTINT[])dataTarget)[number].fval = (float)dv[index];
            if (sample < 0 && !isInRange((float)dv[index],lwrLimit,uprLimit)) continue;
            break;
          case TFormat.CF_LONG:
            if (dv == null) return -TErrorList.illegal_format;
            ((int[])dataTarget)[number] = (int)dv[index];
            if (sample < 0 && !isInRange((float)dv[index],lwrLimit,uprLimit)) continue;
            break;
          case TFormat.CF_LNGINT:            
            if (dv == null) return -TErrorList.illegal_format;
            if (((INTINT[])dataTarget)[number] == null) ((INTINT[])dataTarget)[number] = new INTINT();
            ((INTINT[])dataTarget)[number].i2val = (int)dts;
            ((INTINT[])dataTarget)[number].i1val = (int)dv[index];
            if (sample < 0 && !isInRange((float)dv[index],lwrLimit,uprLimit)) continue;
            break;
          case TFormat.CF_DOUBLE:
            if (dv == null) return -TErrorList.illegal_format;
            ((double[])dataTarget)[number] = (double)dv[index];
            if (sample < 0 && !isInRange((float)dv[index],lwrLimit,uprLimit)) continue;
            break;
          case TFormat.CF_DBLDBL:            
            if (dv == null) return -TErrorList.illegal_format;
            if (((DBLDBL[])dataTarget)[number] == null) ((DBLDBL[])dataTarget)[number] = new DBLDBL();
            ((DBLDBL[])dataTarget)[number].d2val = dts;
            ((DBLDBL[])dataTarget)[number].d1val = dv[index];
            if (sample < 0 && !isInRange((float)dv[index],lwrLimit,uprLimit)) continue;
            break;
          case TFormat.CF_NAME16I:
            if (nv == null) return -TErrorList.illegal_format;
            if (((NAME16I[])dataTarget)[number] == null) ((NAME16I[])dataTarget)[number] = new NAME16I();
            ((NAME16I[])dataTarget)[number].ival = (int)dts;
            ((NAME16I[])dataTarget)[number].name = nv[index].toString();
            break;
          case TFormat.CF_NAME32I:
            if (nv == null) return -TErrorList.illegal_format;
            if (((NAME32I[])dataTarget)[number] == null) ((NAME32I[])dataTarget)[number] = new NAME32I();
            ((NAME32I[])dataTarget)[number].ival = (int)dts;
            ((NAME32I[])dataTarget)[number].name = nv[index].toString();
            break;
          case TFormat.CF_NAME64I:
            if (nv == null) return -TErrorList.illegal_format;
            if (((NAME64I[])dataTarget)[number] == null) ((NAME64I[])dataTarget)[number] = new NAME64I();
            ((NAME64I[])dataTarget)[number].ival = (int)dts;
            ((NAME64I[])dataTarget)[number].name = nv[index].toString();
            break;            
        }
        number++;
        if (number >= arraysize) break;
        if (raster > 1) // move along through file
        {
          raf.seek(raf.getFilePointer()+rlen*raster-1);
        }        
      }
      raf.close();
      if (datatime > 0.0) lastReadTimestamp = (long)(datatime*1000);
      return number;
    }
    catch (Exception e)
    {
      TFecLog.log("Error reading history record : " + e.toString());
    }    
    return -TErrorList.file_error; 
  }  
  public int getDataFromSTS(int index,long starttime,long endtime,
         float lwrLimit,float uprLimit,Object targetData,int arraysize,int arrayOffset,int sample)
  {
    int depth = hspec.getDepthShort();
    int raster=1,cnt;
    int n,number,rnumber=0,num;
    int prate = hspec.getPollngRate();
    boolean isSnapshot;
    long thistimestamp;
    float fval;
    int ival;
    double dval;
    NAME nval;
    
    lastReadTimestamp = 0;
    if (depth == 0) return -TErrorList.un_allocated;
    if (index >= siz) return -TErrorList.out_of_range;
    if (arraysize < 1) return -TErrorList.buffer_too_small;    
    if (starttime < endtime - depth*prate) starttime = endtime - depth*prate;
    if ((raster=(int)((endtime-starttime)/(prate*arraysize))) < 1) raster = 1;
    if (arraysize == 1) raster = 1;
    isSnapshot = (starttime == endtime) ? true : false;
    n = rngPtr;
    /* move raster past the current position to start at the beginning */
    n = (n + raster + depth) % depth;
    if (arraysize == endtime-starttime) /* caller wants the array filled */
    {
      if (arraysize <= (num=depth))
      {
        n = (n + (num-arraysize) + depth) % depth;
        starttime = timestamp[n];
        endtime = System.currentTimeMillis();
      }
    }
    lastReadTimestamp = starttime;
    for (number=arrayOffset,cnt=0; number < arraysize && number <= depth && cnt < depth; cnt += raster)
    {
      thistimestamp = timestamp[n];
      n = (n + raster + depth) % depth;
      if (thistimestamp < lastReadTimestamp || thistimestamp > endtime) continue;
      lastReadTimestamp = thistimestamp;
      rnumber++;
      if (arraysize < 2) continue;
      if (targetData instanceof float[])
      {
        if (isSnapshot)
        {
          rnumber = number = arraysize < depth ? arraysize : depth;
          System.arraycopy(getFloatArrayAtPosition(n),0,(float[])targetData,0,number);
          return rnumber; // was a goto => more to it than this !
        }
        ((float[])targetData)[number] = getFloatValueAtPosition(n,index);
      }
      else if (targetData instanceof FLTINT[])
      {
        fval = getFloatValueAtPosition(n,index);
        if (((FLTINT[])targetData)[number] == null) ((FLTINT[])targetData)[number] = new FLTINT();
        ((FLTINT[])targetData)[number].ival = (int)(thistimestamp/1000);
        ((FLTINT[])targetData)[number].fval = fval;
        if (sample < 0 && !isInRange(fval,lwrLimit,uprLimit)) continue;
      }
      else if (targetData instanceof INTFLTINT[])
      {
        fval = getFloatValueAtPosition(n,index);
        if (((INTFLTINT[])targetData)[number] == null) ((INTFLTINT[])targetData)[number] = new INTFLTINT();
        ((INTFLTINT[])targetData)[number].i1val = (int)(thistimestamp/1000);
        ((INTFLTINT[])targetData)[number].fval  = fval;
        ((INTFLTINT[])targetData)[number].i2val = 0;
        if (sample < 0 && !isInRange(fval,lwrLimit,uprLimit)) continue;
      }
      else if (targetData instanceof FLTINTINT[])
      {
        fval = getFloatValueAtPosition(n,index);
        if (((FLTINTINT[])targetData)[number] == null) ((FLTINTINT[])targetData)[number] = new FLTINTINT();
        ((FLTINTINT[])targetData)[number].i1val = (int)(thistimestamp/1000);
        ((FLTINTINT[])targetData)[number].i2val = (int)(thistimestamp%1000);
        ((FLTINTINT[])targetData)[number].f1val  = fval;
        if (sample < 0 && !isInRange(fval,lwrLimit,uprLimit)) continue;
      }
      else if (targetData instanceof int[])
      {
        if (isSnapshot)
        {
          rnumber = number = arraysize < depth ? arraysize : depth;
          System.arraycopy(getLongArrayAtPosition(n),0,(long[])targetData,0,number);
          return rnumber; // was a goto => more to it than this !
        }
        ((int[])targetData)[number] = getLongValueAtPosition(n,index);
      }
      else if (targetData instanceof INTINT[])
      {
        ival = getLongValueAtPosition(n,index);
        if (((INTINT[])targetData)[number] == null) ((INTINT[])targetData)[number] = new INTINT();
        ((INTINT[])targetData)[number].i2val = (int)(thistimestamp/1000);
        ((INTINT[])targetData)[number].i1val = ival;
        if (sample < 0 && !isInRange((float)ival,lwrLimit,uprLimit)) continue;
      }
      else if (targetData instanceof double[])
      {
        if (isSnapshot)
        {
          rnumber = number = arraysize < depth ? arraysize : depth;
          System.arraycopy(getDoubleArrayAtPosition(n),0,(double[])targetData,0,number);
          return rnumber; // was a goto => more to it than this !
        }
        ((double[])targetData)[number] = getDoubleValueAtPosition(n,index);
      }
      else if (targetData instanceof DBLDBL[])
      {
        dval = getDoubleValueAtPosition(n,index);
        if (((DBLDBL[])targetData)[number] == null) ((DBLDBL[])targetData)[number] = new DBLDBL();
        ((DBLDBL[])targetData)[number].d2val = TDataTime.getDataTimeStamp(thistimestamp);
        ((DBLDBL[])targetData)[number].d1val = dval;
        if (sample < 0 && !isInRange((float)dval,lwrLimit,uprLimit)) continue;
      }
      else if (targetData instanceof NAME16I[])
      {
        nval = getNAME16ValueAtPosition(n,index);
        if (((NAME16I[])targetData)[number] == null) ((NAME16I[])targetData)[number] = new NAME16I();
        ((NAME16I[])targetData)[number].ival = (int)(thistimestamp/1000);
        ((NAME16I[])targetData)[number].name = nval.toString();
      }
      else if (targetData instanceof NAME32I[])
      {
        nval = getNAME32ValueAtPosition(n,index);
        if (((NAME32I[])targetData)[number] == null) ((NAME32I[])targetData)[number] = new NAME32I();
        ((NAME32I[])targetData)[number].ival = (int)(thistimestamp/1000);
        ((NAME32I[])targetData)[number].name = nval.toString();
      }
      else if (targetData instanceof NAME64I[])
      {
        nval = getNAME64ValueAtPosition(n,index);
        if (((NAME64I[])targetData)[number] == null) ((NAME64I[])targetData)[number] = new NAME64I();
        ((NAME64I[])targetData)[number].ival = (int)(thistimestamp/1000);
        ((NAME64I[])targetData)[number].name = nval.toString();
      }
      number++;    
    }    
    if (arraysize < 2) assignNumberOfPoints(targetData,rnumber);
    return number;
  }
  private void assignNumberOfPoints(Object data,int numberOfPoints)
  {
    if (data instanceof float[])
    {
      ((float[])data)[0] = (float)numberOfPoints;
    }
    else if (data instanceof FLTINT[])
    {
      if (((FLTINT[])data)[0] == null) ((FLTINT[])data)[0] = new FLTINT();
      ((FLTINT[])data)[0].ival = numberOfPoints;
      ((FLTINT[])data)[0].fval = (float)numberOfPoints;
    }
    else if (data instanceof INTFLTINT[])
    {
      if (((INTFLTINT[])data)[0] == null) ((INTFLTINT[])data)[0] = new INTFLTINT();
      ((INTFLTINT[])data)[0].i1val = numberOfPoints;
      ((INTFLTINT[])data)[0].fval  = (float)numberOfPoints;
      ((INTFLTINT[])data)[0].i2val = numberOfPoints;
    }
    else if (data instanceof FLTINTINT[])
    {
      if (((FLTINTINT[])data)[0] == null) ((FLTINTINT[])data)[0] = new FLTINTINT();
      ((FLTINTINT[])data)[0].i1val = numberOfPoints;
      ((FLTINTINT[])data)[0].i2val = numberOfPoints;
      ((FLTINTINT[])data)[0].f1val  = (float)numberOfPoints;
    }
    else if (data instanceof int[])
    {
      ((int[])data)[0] = numberOfPoints;
    }
    else if (data instanceof INTINT[])
    {
      if (((INTINT[])data)[0] == null) ((INTINT[])data)[0] = new INTINT();
      ((INTINT[])data)[0].i2val = numberOfPoints;
      ((INTINT[])data)[0].i1val = numberOfPoints;
    }
    else if (data instanceof double[])
    {
      ((double[])data)[0] = (double)numberOfPoints;
    }
    else if (data instanceof DBLDBL[])
    {
      if (((DBLDBL[])data)[0] == null) ((DBLDBL[])data)[0] = new DBLDBL();
      ((DBLDBL[])data)[0].d2val = (double)numberOfPoints;
      ((DBLDBL[])data)[0].d1val = (double)numberOfPoints;
    }
    else if (data instanceof NAME16I[])
    {
      if (((NAME16I[])data)[0] == null) ((NAME16I[])data)[0] = new NAME16I();
      ((NAME16I[])data)[0].ival = numberOfPoints;
      ((NAME16I[])data)[0].name = "";
    }
    else if (data instanceof NAME32I[])
    {
      if (((NAME32I[])data)[0] == null) ((NAME32I[])data)[0] = new NAME32I();
      ((NAME32I[])data)[0].ival = numberOfPoints;
      ((NAME32I[])data)[0].name = "";
    }
    else if (data instanceof NAME64I[])
    {
      if (((NAME64I[])data)[0] == null) ((NAME64I[])data)[0] = new NAME64I();
      ((NAME64I[])data)[0].ival = numberOfPoints;
      ((NAME64I[])data)[0].name = "";
    }    
  }
  private boolean isInRange(float value,float lower, float upper)
  {
    if (value < lower) return false;
    if (value > upper) return false;
    return true;
  }
  public float getFloatValueAtPosition(int ptr,int index)
  {
    if (index < 0 || index >= siz) return -1;
    if (ptr < 0 || ptr >= hspec.getDepthShort()) return -1;
    if (fRngBuffer != null) return fRngBuffer[ptr*siz+index];
    if (bRngBuffer != null) return (float)bRngBuffer[ptr*siz+index];
    if (sRngBuffer != null) return (float)sRngBuffer[ptr*siz+index];
    if (lRngBuffer != null) return (float)lRngBuffer[ptr*siz+index];
    if (dRngBuffer != null) return (float)dRngBuffer[ptr*siz+index];
    return -1;    
  }  
  public double getDoubleValueAtPosition(int ptr,int index)
  {
    if (index < 0 || index >= siz) return -1;
    if (ptr < 0 || ptr >= hspec.getDepthShort()) return -1;
    if (dRngBuffer != null) return dRngBuffer[ptr*siz+index];
    if (bRngBuffer != null) return (double)bRngBuffer[ptr*siz+index];
    if (sRngBuffer != null) return (double)sRngBuffer[ptr*siz+index];
    if (lRngBuffer != null) return (double)lRngBuffer[ptr*siz+index];
    if (fRngBuffer != null) return (double)fRngBuffer[ptr*siz+index];
    return -1;    
  }
  public int getLongValueAtPosition(int ptr,int index)
  {
    if (index < 0 || index >= siz) return -1;
    if (ptr < 0 || ptr >= hspec.getDepthShort()) return -1;
    if (lRngBuffer != null) return lRngBuffer[ptr*siz+index];
    if (bRngBuffer != null) return (int)bRngBuffer[ptr*siz+index];
    if (sRngBuffer != null) return (int)sRngBuffer[ptr*siz+index];
    if (dRngBuffer != null) return (int)dRngBuffer[ptr*siz+index];
    if (fRngBuffer != null) return (int)fRngBuffer[ptr*siz+index];
    return -1;    
  }
  public short getShortValueAtPosition(int ptr,int index)  
  {
    if (index < 0 || index >= siz) return -1;
    if (ptr < 0 || ptr >= hspec.getDepthShort()) return -1;
    if (sRngBuffer != null) return sRngBuffer[ptr*siz+index];
    if (bRngBuffer != null) return (short)bRngBuffer[ptr*siz+index];
    if (lRngBuffer != null) return (short)lRngBuffer[ptr*siz+index];
    if (dRngBuffer != null) return (short)dRngBuffer[ptr*siz+index];
    if (fRngBuffer != null) return (short)fRngBuffer[ptr*siz+index];
    return -1;    
  }
  public NAME16 getNAME16ValueAtPosition(int ptr,int index)
  {
    if (index < 0 || index >= siz) return null;
    if (ptr < 0 || ptr >= hspec.getDepthShort()) return null;
    if (n16RngBuffer != null) return n16RngBuffer[ptr*siz+index];
    if (n32RngBuffer != null) return new NAME16(n32RngBuffer[ptr*siz+index].toString());
    if (n64RngBuffer != null) return new NAME16(n64RngBuffer[ptr*siz+index].toString());
    return null;    
  }
  public NAME32 getNAME32ValueAtPosition(int ptr,int index)
  {
    if (index < 0 || index >= siz) return null;
    if (ptr < 0 || ptr >= hspec.getDepthShort()) return null;
    if (n32RngBuffer != null) return n32RngBuffer[ptr*siz+index];
    if (n16RngBuffer != null) return new NAME32(n16RngBuffer[ptr*siz+index].toString());
    if (n64RngBuffer != null) return new NAME32(n64RngBuffer[ptr*siz+index].toString());
    return null;    
  }
  public NAME64 getNAME64ValueAtPosition(int ptr,int index)
  {
    if (index < 0 || index >= siz) return null;
    if (ptr < 0 || ptr >= hspec.getDepthShort()) return null;
    if (n64RngBuffer != null) return n64RngBuffer[ptr*siz+index];
    if (n16RngBuffer != null) return new NAME64(n16RngBuffer[ptr*siz+index].toString());
    if (n32RngBuffer != null) return new NAME64(n32RngBuffer[ptr*siz+index].toString());
    return null;    
  }
  public float[] getFloatArrayAtPosition(int ptr)
  {
    float[] values = new float[siz];
    
    if (fRngBuffer != null)
    {
      System.arraycopy(fRngBuffer,ptr*siz,values,0,siz);
    }
    else if (bRngBuffer != null)
    {
      for (int i=0; i<siz; i++) values[i] = (float)bRngBuffer[i];
    }
    else if (sRngBuffer != null)
    {
      for (int i=0; i<siz; i++) values[i] = (float)sRngBuffer[i];
    }
    else if (lRngBuffer != null)
    {
      for (int i=0; i<siz; i++) values[i] = (float)lRngBuffer[i];
    }
    else if (dRngBuffer != null)
    {
      for (int i=0; i<siz; i++) values[i] = (float)dRngBuffer[i];
    }
    return values;
  }
  public double[] getDoubleArrayAtPosition(int ptr)
  {
    double[] values = new double[siz];
    
    if (dRngBuffer != null)
    {
      System.arraycopy(dRngBuffer,ptr*siz,values,0,siz);
    }
    else if (bRngBuffer != null)
    {
      for (int i=0; i<siz; i++) values[i] = (double)bRngBuffer[i];
    }
    else if (sRngBuffer != null)
    {
      for (int i=0; i<siz; i++) values[i] = (double)sRngBuffer[i];
    }
    else if (lRngBuffer != null)
    {
      for (int i=0; i<siz; i++) values[i] = (double)lRngBuffer[i];
    }
    else if (fRngBuffer != null)
    {
      for (int i=0; i<siz; i++) values[i] = (double)fRngBuffer[i];
    }
    return values;
  }
  public int[] getLongArrayAtPosition(int ptr)
  {
    int[] values = new int[siz];
    
    if (lRngBuffer != null)
    {
      System.arraycopy(lRngBuffer,ptr*siz,values,0,siz);
    }
    else if (bRngBuffer != null)
    {
      for (int i=0; i<siz; i++) values[i] = (int)bRngBuffer[i];
    }
    else if (sRngBuffer != null)
    {
      for (int i=0; i<siz; i++) values[i] = (int)sRngBuffer[i];
    }
    else if (dRngBuffer != null)
    {
      for (int i=0; i<siz; i++) values[i] = (int)dRngBuffer[i];
    }
    else if (fRngBuffer != null)
    {
      for (int i=0; i<siz; i++) values[i] = (int)fRngBuffer[i];
    }
    return values;
  }
  public short[] getShortArrayAtPosition(int ptr)
  {
    short[] values = new short[siz];
    
    if (sRngBuffer != null)
    {
      System.arraycopy(sRngBuffer,ptr*siz,values,0,siz);
    }
    else if (bRngBuffer != null)
    {
      for (int i=0; i<siz; i++) values[i] = (short)bRngBuffer[i];
    }
    else if (lRngBuffer != null)
    {
      for (int i=0; i<siz; i++) values[i] = (short)lRngBuffer[i];
    }
    else if (dRngBuffer != null)
    {
      for (int i=0; i<siz; i++) values[i] = (short)dRngBuffer[i];
    }
    else if (fRngBuffer != null)
    {
      for (int i=0; i<siz; i++) values[i] = (short)fRngBuffer[i];
    }
    return values;
  }
  public NAME16[] getNAME16ArrayAtPosition(int ptr)
  {
    NAME16[] values = new NAME16[siz];
    
    if (n16RngBuffer != null)
    {
      for (int i=0; i<siz; i++) values[i] = new NAME16(n16RngBuffer[i].toString());
    }
    else if (n32RngBuffer != null)
    {
      for (int i=0; i<siz; i++) values[i] = new NAME16(n32RngBuffer[i].toString());
    }
    else if (n64RngBuffer != null)
    {
      for (int i=0; i<siz; i++) values[i] = new NAME16(n64RngBuffer[i].toString());
    }
    return values;
  }
  public NAME32[] getNAME32ArrayAtPosition(int ptr)
  {
    NAME32[] values = new NAME32[siz];
    
    if (n16RngBuffer != null)
    {
      for (int i=0; i<siz; i++) values[i] = new NAME32(n16RngBuffer[i].toString());
    }
    else if (n32RngBuffer != null)
    {
      for (int i=0; i<siz; i++) values[i] = new NAME32(n32RngBuffer[i].toString());
    }
    else if (n64RngBuffer != null)
    {
      for (int i=0; i<siz; i++) values[i] = new NAME32(n64RngBuffer[i].toString());
    }
    return values;
  }
  public NAME64[] getNAME64ArrayAtPosition(int ptr)
  {
    NAME64[] values = new NAME64[siz];
    
    if (n16RngBuffer != null)
    {
      for (int i=0; i<siz; i++) values[i] = new NAME64(n16RngBuffer[i].toString());
    }
    else if (n32RngBuffer != null)
    {
      for (int i=0; i<siz; i++) values[i] = new NAME64(n32RngBuffer[i].toString());
    }
    else if (n64RngBuffer != null)
    {
      for (int i=0; i<siz; i++) values[i] = new NAME64(n64RngBuffer[i].toString());
    }
    return values;
  }
  public int writeRecordToSTS(Object values)
  { // write to Short-Term-Storage (i.e. ram)
    rngPtr = (rngPtr + 1) % hspec.getDepthShort();
    if (values instanceof byte[]) 
    {
      if (bRngBuffer == null) return TErrorList.illegal_format;
      int len = ((byte[])values).length;
      if (len > siz) len = siz;
      System.arraycopy(values,0,bRngBuffer,rngPtr*siz,len);            
    }
    else if (values instanceof short[]) 
    {
      if (sRngBuffer == null) return TErrorList.illegal_format;
      int len = ((short[])values).length;
      if (len > siz) len = siz;
      System.arraycopy(values,0,sRngBuffer,rngPtr*siz,len);            
    }
    else if (values instanceof int[]) 
    {
      if (lRngBuffer == null) return TErrorList.illegal_format;
      int len = ((int[])values).length;
      if (len > siz) len = siz;
      System.arraycopy(values,0,lRngBuffer,rngPtr*siz,len);            
    }
    else if (values instanceof float[]) 
    {
      if (fRngBuffer == null) return TErrorList.illegal_format;
      int len = ((float[])values).length;
      if (len > siz) len = siz;
      System.arraycopy(values,0,fRngBuffer,rngPtr*siz,len);      
    }
    else if (values instanceof double[]) 
    {
      if (dRngBuffer == null) return TErrorList.illegal_format;
      int len = ((double[])values).length;
      if (len > siz) len = siz;
      System.arraycopy(values,0,dRngBuffer,rngPtr*siz,len);            
    }
    else if (values instanceof NAME16[]) 
    {
      if (n16RngBuffer == null) return TErrorList.illegal_format;
      int len = ((NAME16[])values).length;
      if (len > siz) len = siz;
      System.arraycopy(values,0,n16RngBuffer,rngPtr*siz,len);            
    }
    else if (values instanceof NAME32[]) 
    {
      if (n32RngBuffer == null) return TErrorList.illegal_format;
      int len = ((NAME32[])values).length;
      if (len > siz) len = siz;
      System.arraycopy(values,0,n32RngBuffer,rngPtr*siz,len);            
    }
    else if (values instanceof NAME64[]) 
    {
      if (n64RngBuffer == null) return TErrorList.illegal_format;
      int len = ((NAME64[])values).length;
      if (len > siz) len = siz;
      System.arraycopy(values,0,n64RngBuffer,rngPtr*siz,len);            
    }
    else
    {
      return TErrorList.illegal_format;
    }
    timestamp[rngPtr] = System.currentTimeMillis();
    return 0;
  }
  private String twoDigitString(int n)
  {
    return n < 10 ? "0" + n : "" + n;
  }
  public void setArchiveFileName(long cycleTime)
  {
    if (calendar == null) calendar = Calendar.getInstance();
    calendar.setTimeInMillis(cycleTime);
    int dd = calendar.get(Calendar.DAY_OF_MONTH);
    if (dd == lastArchivedDayOfTheMonth) return;
    int yy, mm;
    lastArchivedDayOfTheMonth = dd;
    yy = calendar.get(Calendar.YEAR) % 100;
    mm = calendar.get(Calendar.MONTH) + 1; // January = 0
    if (useMonthlyHistoryFiles) dd = 1; 
    String prefix = useHighResolutionTimeStamp ? "ha" : "ar";
    ArchiveFileName = prefix +  twoDigitString(yy) + twoDigitString(mm) + 
                      twoDigitString(dd) + "." + Integer.toHexString(rcdIdx);
    removeOutdatedLTSFiles(cycleTime);
  }
  public void removeOutdatedLTSFiles(long cycleTime)
  {
    String fileName = null;
    int yy,mm,mm0;
    calendar.setTimeInMillis(cycleTime);
    yy = calendar.get(Calendar.YEAR) % 100;
    mm0 = calendar.get(Calendar.MONTH) + 1; // January = 0
    mm = ((mm0+12) - ((hspec.getDepthLong()+1)%12))%12;
    yy -= hspec.getDepthLong()/12;
    if (mm > mm0) yy--;
    String prefix = useHighResolutionTimeStamp ? "ha" : "ar";
    fileName = prefix +  twoDigitString(yy) + twoDigitString(mm) + 
                  "*." + Integer.toHexString(rcdIdx);
    try
    {
      File f = new File(fileName);
      f.delete();
    }
    catch (Exception e)
    {
      TFecLog.log("unable to delete file " + fileName);
    }
  }
  public String getArchiveFileName()
  {
    return ArchiveFileName;
  }
  public int getArrayType()
  {
    return aType;
  }
  public int getRingBufferPointer()
  {
    return rngPtr;
  }
  public String getDev()
  {
    return dev;
  }
  public TEquipmentModule getEqm()
  {
    return eqm;
  }
  public byte getFmt()
  {
    return fmt;
  }
  public THistorySpecification getHspec()
  {
    return hspec;
  }
  public long getLastArchiveTime()
  {
    return lastArchiveTime;
  }
  public String getPrp()
  {
    return prp;
  }
  public int getRcdIdx()
  {
    return rcdIdx;
  }
  public int getRngBufferSize()
  {
    return rngBufferSize;
  }
  public int getArraySize()
  {
    return siz;
  }
  public int getRecordLength()
  {
    int tslen = useHighResolutionTimeStamp ? 8 : 4;
    return siz * TFormat.formatSizeOf(fmt) + tslen;
  }
  public long getTimestamp()
  {
    return rngPtr < 0 ? 0 : timestamp[rngPtr];
  }
  public boolean isUseMonthlyHistoryFiles()
  {
    return useMonthlyHistoryFiles;
  }
  public void setUseMonthlyHistoryFiles(boolean useMonthlyHistoryFiles)
  {
    this.useMonthlyHistoryFiles = useMonthlyHistoryFiles;
  }
  public boolean isUseMsecHistoryTimeStamps()
  {
    return useHighResolutionTimeStamp;
  }
  public void setUseMsecHistoryTimeStamps(boolean useMsecHistoryTimeStamps)
  {
    useHighResolutionTimeStamp = useMsecHistoryTimeStamps;
  }
}
