package debugTest;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.io.IOException;
import java.io.Serializable;

import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.TransferHandler;

import de.desy.acop.transport.AccessMode;

public class DnDPanel extends JPanel {

	public class AcopMetaInfo implements Serializable {
		public String device;
		public String property;
		public AccessMode accessMode;
		public int accessRate;
		
		public AcopMetaInfo() {
		}
		
		public AcopMetaInfo(String d, String p, AccessMode am, int ar) {
			device=d;
			property=p;
			accessMode=am;
			accessRate=ar;
		}
		
		/* (non-Javadoc)
		 * @see java.lang.Object#toString()
		 */
		@Override
		public String toString() {
			return "AcopMetaInfo={d='"+device+"',p='"+property+"',am='"+accessMode+"',ar='"+accessRate+"'}";
		}
	}
	
	DataFlavor flavor= new DataFlavor(AcopMetaInfo.class,"Acop Meta Info Flavor");
	
	class AcopInfoLabel extends JLabel {
		AcopMetaInfo info;
		
		public AcopInfoLabel() {
			setTransferHandler(new TransferHandler() {
			
				@Override
				public boolean importData(JComponent comp, Transferable t) {
					try {
					if (t.isDataFlavorSupported(flavor)) {
						Object o=t.getTransferData(flavor);
						setInfo((AcopMetaInfo)o);
						return true;
					}
					if (t.isDataFlavorSupported(DataFlavor.stringFlavor)) {
						setInfo(new AcopMetaInfo(
								getInfo().device,
								t.getTransferData(DataFlavor.stringFlavor).toString(),
								getInfo().accessMode,
								getInfo().accessRate));
						return true;
					}
					} catch (Exception e) {
						e.printStackTrace();
					}
					return false;
				}
			
				@Override
				protected Transferable createTransferable(JComponent c) {
					return new Transferable() {
					
						public Object getTransferData(DataFlavor flavor)
								throws UnsupportedFlavorException, IOException {
							if (flavor.isFlavorSerializedObjectType() && flavor.getRepresentationClass().equals(AcopMetaInfo.class)) {
								return getInfo();
							}
							if (flavor.isFlavorTextType()) {
								return getInfo().toString();
							}
							throw new UnsupportedFlavorException(flavor);
						}
					
						public boolean isDataFlavorSupported(DataFlavor flavor) {
							return ((flavor.isFlavorSerializedObjectType() && flavor.getRepresentationClass().equals(AcopMetaInfo.class)) || flavor.isFlavorTextType());
						}
					
						public DataFlavor[] getTransferDataFlavors() {
							return new DataFlavor[]{flavor,DataFlavor.stringFlavor};
						}
					};
				}
			
				@Override
				public boolean canImport(JComponent comp, DataFlavor[] transferFlavors) {
					for (int i = 0; i < transferFlavors.length; i++) {
						if ((flavor.isFlavorSerializedObjectType() && flavor.getRepresentationClass().equals(AcopMetaInfo.class)) || transferFlavors[i].isFlavorTextType()) {
							return true;
						}
					}
					return false;
				}
				@Override
				public int getSourceActions(JComponent c) {
					return TransferHandler.COPY;
				}
			
			});
			addMouseListener(new MouseAdapter() {
			
				@Override
				public void mousePressed(MouseEvent e) {
					getTransferHandler().exportAsDrag(AcopInfoLabel.this,e,TransferHandler.COPY);
				}
			
			});
		}

		public AcopMetaInfo getInfo() {
			return info;
		}

		public void setInfo(AcopMetaInfo info) {
			this.info = info;
			setText(info.toString());
		}
		
	}
	

	private JTextField field;
	private AcopInfoLabel label;
	
	
	
	public DnDPanel() {
		super();
		initialize();
	}
	
	

	private void initialize() {
		
		setLayout(new GridBagLayout());
		
		field= new JTextField();
		field.setText("Some text.");
		field.setDragEnabled(true);
		
		label = new AcopInfoLabel();
		label.setInfo(new AcopMetaInfo("DEV01","ORB.X",AccessMode.READ,1000));

		add(field, new GridBagConstraints(0,0,1,1,1.0,1.0,GridBagConstraints.CENTER,GridBagConstraints.HORIZONTAL,new Insets(4,4,4,4),0,0));
		add(label, new GridBagConstraints(0,1,1,1,1.0,1.0,GridBagConstraints.CENTER,GridBagConstraints.HORIZONTAL,new Insets(4,4,4,4),0,0));
	}



	/**
	 * @param args
	 */
	public static void main(String[] args) {
		try {
			
			JFrame f= new JFrame();
			f.setContentPane(new DnDPanel());
			f.setSize(500,100);
			f.setVisible(true);
			
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

}
